/* definitions for the APES job dispatcher and brain */

/* initial version: 10 july 2001, h brandenburg */

/* last updated: Wed Sep 12 12:01:27 PDT 2001, h brandenburg (v1.4) */
/* prev updated: Tue Aug 14 15:55:44 PDT 2001, h brandenburg */
/* prev updated: Wed Aug  8 11:56:28 PDT 2001, Jing Li */

#define APESVERSN   1.5

#define TRUE 1
#define FALSE 0

/* job dispatcher types */
#define HEAVY_DUTY 1
#define REGULAR 0

/* Definition:              Meaning:                            Set by:           */
#define UNINIT_LIST 0   /*  shm manifest is uninitialized       Job Dispatcher    */
#define READY_LIST 1    /*  shm manifest is ready to go         Brain             */
#define FILL_LIST 2     /*  shm manifest needs to be refilled   Job Dispatcher    */
#define EMPTY_LIST 3    /*  no jobs left to service             Brain             */

#define MAX_LINE_LENGTH 1024
#define MAX_KEY_LENGTH 64
#define MAX_VALUE_LENGTH 32
#define MAX_COMMENT_LENGTH 128
#define MAX_FNAME_LENGTH 161
#define MAX_SUBTYPE_LENGTH 5
#define MAX_MONIKER_LENGTH 64
#define MAX_STATUS_BITS 32
#define MAX_NUM_JOBS 16000

/* brain thread error codes */
#define ERR_ENVIRONMENT_TBL     -2
#define ERR_ENVIRONMENT_DATA    -3
#define ERR_ENVIRONMENT_TOOLS   -4

#define ERR_STATICTABLE_PIPE    -5
#define ERR_STATICTABLE_PROC    -6
#define ERR_STATICTABLE_YMASK   -7
#define ERR_STATICTABLE_CONFIG  -8
#define ERR_STATICTABLE_STAT    -9
#define ERR_STATICTABLE_MONIKER -12

#define ERR_PROCLIST      -10
#define ERR_TOOLSCALL     -11

/* include files for shared memory */
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

/* include files for pthreads and their semaphores */
#include <pthread.h>
#include <semaphore.h> 
#include <synch.h>


/* The jobdescriptor_t describes a single job, to be scheduled and dispatched by
   the executive. Information contained in the structure is unique to that single 
   job; general information about the job is provided by the metadata_t, defined
   below.

   Note that the entire list of jobs, the shared memory manifest or "shortlist",
   will be an array of jobdescripter_t.
*/

typedef struct {
  int job_id;
  int pipe_num;
  int complete;
} jobdescriptor_t;


/* The metadata_t describes all jobs in the shared memory manifest at any given
   time.
*/

typedef struct {
  int jobs_in_array;
  int num_jobs;
  short  ensemble_data;   /* boolean */
  int pipe_type;
  int file_id;
  int last_index;
  int pipeline_id;
  int cal_id;

  char pline_subtype[MAX_SUBTYPE_LENGTH];

  char dce_proclist_moniker[MAX_VALUE_LENGTH];

  sema_t  joblist_semaphore;
  sema_t  lastindex_semaphore;

  short joblist_status;

  short masks_updated;  /* boolean */
  short heavy_duty;
} metadata_t;


/* The APESbrain_args_t is for packing information for passing to the brain   
   thread. These are pointers to the shared memory used by both threads 
*/

typedef struct {
  jobdescriptor_t *joblist0_ptr;
  jobdescriptor_t *joblist1_ptr;
  metadata_t *metadata0_ptr;
  metadata_t *metadata1_ptr;

  /* type 0 or 1. 0 is REGULAR processing, 1 is HEAVY_DUTY */
  int dispatcher_type;

  int *status;
} APESbrain_args_t;


/* The config_opts_t structure holds the contents of the apes_config.tbl,
   which is part of the JRDB. These parameters allow us to fully modify
   the runtime behavior of the APES system - Job Dispatcher, brain, and
   their interpretation of the JRDB contents.
*/

typedef struct {
  int time_offset;

  int JRDB_updater_sleep;
  int JRDB_viewer_sleep;

  int job_dispatcher_APES_brain_sleep;
  int job_dispatcher_sleep;

  int short_list_max;

  int number_jobbers_per_pipeline_machine;
  int number_of_pipeline_machines;
  
  int total_number_jobbers;
  
  int xmask_proc_window;
} config_opts_t;

unsigned int timeoffset;
