#include <stdio.h>
#include "fitsio.h"

#define TRUE 1
#define FALSE 0

#define MAX_FNAME_LENGTH 1025
#define MAX_HISTORY_LENGTH 81

#define VERSION 2.0

/* dmask bit */
#define FFCORR_NOAPPLY 0x02

enum bunit {dn, dnpersec, electrons, electronspersec};
enum delaytype {immediate, frame};

typedef struct {
  int  verbose;
  int  debug;
  
  enum modes {value, image} mymode;
  int  errorcode;
  int  warningcode;
  char iam[MAX_FNAME_LENGTH];

  FILE* logstream;
} state_t;

typedef struct {
  char namelist[FLEN_FILENAME];
  char input[FLEN_FILENAME];
  char insigma[FLEN_FILENAME];
  char output[FLEN_FILENAME];
  char outcorrection[FLEN_FILENAME];
  char outsigma[FLEN_FILENAME];
  char pmask[FLEN_FILENAME];
  char dmask[FLEN_FILENAME];
  char previousdce[FLEN_FILENAME];
  char frame0dce[FLEN_FILENAME];
  char frame1dce[FLEN_FILENAME];
  enum delaytype type;
  char correction[FLEN_FILENAME];
  char correctionsigma[FLEN_FILENAME];

  int pmaskfatal;
  int dmaskfatal;
  float delay;
  float immeddelay;

  char logfile[FLEN_FILENAME];
  char ancillary[FLEN_FILENAME];

  int  uncertainty; 
} param_t;

typedef struct {
  fitsfile *fp;
  
  char  filename[FLEN_FILENAME];
  enum  bunit units;
  float integration;
  float gain[4];
  int   naxes;
  long  naxis[4];
  int   pixels;

  float* data;
} img_t;
 

typedef struct {
  fitsfile *fp;
  char filename[FLEN_FILENAME];
  int naxes;
  int naxis[4];
  int pixels;
  
  int* data;
} maskimg_t;


typedef struct {
  int atimeendn;
  int atimeendnminus1;
  int awaitpern;
  int afowlnumn;
  int channel;
} expdata_t;

void print_help(state_t* state);
void error_exit(state_t* mystate);
void standard_exit(state_t* mystate);

int parse_namelist(int argc, char **argv, state_t* state, param_t* param);
int parse_args(int argc, char **argv, state_t* state, param_t* param);
int check_args(state_t* state, param_t* param);

void free_img(img_t* img);
void free_mask(maskimg_t* img);
void init_img(img_t* img);
void init_mask(maskimg_t* img);
int  init(state_t* state, param_t* param);


int  read_img(state_t *mystate, img_t *img, char *name);
int  read_header_data(state_t* state, img_t* img, char* previous, expdata_t* data);
void init_empty_img(state_t* state, img_t* img, img_t* new);
void init_empty_mask(state_t* state, img_t* img, maskimg_t* mask);
void sigma_unit_match(state_t* state, img_t* data, img_t* sigma);
void sigma_dim_match(state_t* state, param_t* param, img_t* data, img_t* sigma);
void check_images(state_t* state, param_t* param, img_t* data, img_t* sigma, 
                  maskimg_t* pmask, maskimg_t* dmask);
int  load_data(state_t* state, param_t* param, 
	       img_t* img, img_t* sigma,
	       maskimg_t* pmask, maskimg_t* dmask);

int  write_img(state_t *mystate, img_t *img, char *name, param_t* param);
int  pack_output_unc(state_t *mystate, img_t *img, char *name);
int  write_mask(state_t *mystate, maskimg_t *img, char *name);
int  write_data(state_t* state, param_t* param, img_t* data, img_t* sigma, 
		img_t* corr, img_t* corrsigma, maskimg_t* dmask);
void write_module_history(fitsfile* fp, int* status);

double compute_delay(expdata_t data);
float  linear_interp(float point1, float point2, float span);
int    interpolate_correction(state_t* state, param_t* param, double delay, 
			      img_t* corr, img_t* corrsigma);
int     compute(state_t* state, param_t* param, img_t* img, img_t* sigma,
		img_t* corr, img_t* corrsigma,
		maskimg_t* pmask, maskimg_t* dmask);

