#ifndef __ge_mask_h
#define __ge_mask_h

/*
   unified mask definition file for MIPS Ge bmask, dmask, pmask and cmask files

   This file replaces mips_ge_bmask.h and mips_ge_dmask.h


   naming conventions:
   gep prefix       -    pmask file definition 
   ged prefix       -    dmask file definition
   geb prefix       -    bmask file definition
   gec prefix       -    cmask file definition

   Very often bits have identical semantics in different files because bits
   get copied from one file to another. It is highly desirable to have spelling
   identical (as far as it can be) in this case. Thus the prefix is invented.



*/

/* contained in pmask.fits file 
   - status bits for each pixel contained in a image plane 
   - maintained by IST, delivered through caltrans */

#define gep_NOISY           0x0004 
#define gep_BADHALF         0x0100 
#define gep_BADPIX          0x4000 

/* contained in dmask.fits file 
   - ramp status bits for each pixel contained in a fitscube (third dimension is sample number in ramp)
   - produced by sloper pipeline */
#define ged_DIDRHDET        0x0004      
#define ged_SATURATEHI      0x0008   
#define ged_SATURATELO      0x0010
#define ged_SEGMENT         0x0100      
#define ged_RADHIT          0x0200
#define ged_BADSAMP         0x0800      
#define ged_RHBADSAMP       0x2000
#define ged_SAMPSMISSING    0x4000


/* contained in a bmask.fits file
   - status bits for each pixel contained in an image plane
   - produced by sloper pipeline, updated by caler pipeline */ 
#define geb_STIMDCE         0x0001
/* unused                   0x0002 */
#define geb_NOISY           0x0004
#define geb_SATURATED       0x0008
#define geb_USER4           0x0010      
#define geb_USER5           0x0020
#define geb_STIMEXTRAP      0x0040
/* unused                   0x0080 */
#define geb_SAMPSMISSING    0x0100
#define geb_RADHIT          0x0200
#define geb_UNCERTSTATUS    0x0400      
#define geb_BADPIX          0x0800
#define geb_CALERFAIL       0x1000 
#define geb_SLOPERFAIL      0x2000
#define geb_MISSLAYER       0x4000

/* contained in a cmask.fits file
   - status bits for each pixel contained in an image plane
   - produced by a calibration pipeline, supplied by caltrans to pipelines needing the cal file */
#define gec_NOISY           0x0004      
#define gec_DARKFEW         0x0010      
#define gec_ILCORRFEW       0x0020      
#define gec_BADHALF         0x0100      
#define gec_BADCAL          0x2000      
#define gec_BADPIX          0x4000      


#endif
