#ifndef __NANVALUE
#define __NANVALUE

/* a way to define a NAN for all machines with IEEE floating point 
   works on both big-endian and little-endian variants because the 
   byte order of float is the same as the byte order of long
*/

#ifdef	__cplusplus
extern "C" {
#endif

#undef NANVALUE
static union {unsigned long i; float f;} static_nan = {0x7fc00000};
#define NANVALUE (static_nan.f)

#undef INFVALUE
static union {unsigned long i; float f;} static_inf = {0x7f800000};
#define INFVALUE (static_inf.f)

#ifdef __cplusplus
}
#endif

/* Needed for OS X. To quote Bill Northcott:

isnan() is a C99 extension to standard C.
Standard C++ is based on an older standard of C.
Hence isnan() is not part of standard C++ and may or may not work.

The C++ equivalent of math.h is cmath. cmath on MacOS X and probably other platforms undefines isnan() and other C99 macros. Hence if your program includes cmath or one of the many headers such iostream that themselves include cmath then isnan() will not be available.

The code works on Linux because isnan() is a function not a macro and 
the macro expansion takes place in the preprocessor before the compiler sees it.

The solution for any math library for C++ (such as libRmath) is to program your own isnan() like function that access the underlying system functions.
*/

#ifdef _DEFINE_ISNAN
#undef  isnan
#define isnan(X) (X!=X)
#endif

#endif
