#ifndef __sortutil_
#define __sortutil_

#ifdef	__cplusplus
extern "C" {
#endif


/* key compare routines for stdlib.h utility qsort */

int qsort_comp_chars (const void *p1, const void *p2);
int qsort_comp_shorts (const void *p1, const void *p2);
int qsort_comp_ints (const void *p1, const void *p2);
int qsort_comp_longs (const void *p1, const void *p2);
int qsort_comp_longlongs (const void *p1, const void *p2);
int qsort_comp_floats (const void *p1, const void *p2);
int qsort_comp_doubles (const void *p1, const void *p2);


/* create an index array for an array of GENERIC_TYPEs 
     inputs nitem - number of GENERIC_TYPEs 
            v     - zero origin array of GENERIC_TYPEs
            value_ascend- negative to sort values in descending order
            index_ascend- negative to sort indices in descending order
     output 
            ndx: index array 
               if i<=j then v[ndx[i]] <= v[ndx[j]]
	    *count_num_p : set to number of items that were not NAN

     return value:
            0    everything OK
            1    exactly one item to index
           16    zero items to index
           64    fatal - cannot allocate temp array
     
     Notes.
       1. all elements of v are indexed, presented in a well defined order
       2. NaNs are indexed at the end of the ndx array
          all NaNs are presented in the same order as in the input array
       3. this is an order preserving operation, for non-nan v[i] and v[j] 
          if v[i] == v[j] and i < j then 
             the ndx array presents element i before element j
 */

/* floating point index routines */
int float_index(int nitem, float v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int double_index(int nitem, double v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;

/* signed integer index routines */
int char_index(int nitem, char v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int int_index(int nitem, int v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int long_index(int nitem, long v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int longlong_index(int nitem, long long v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int short_index(int nitem, short v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;

/* unsigned integer index routines */
int unsignedchar_index(int nitem,unsigned char v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int unsignedint_index(int nitem,unsigned int v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int unsignedlong_index(int nitem,unsigned long v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int unsignedlonglong_index(int nitem,unsigned long long v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;
int unsignedshort_index(int nitem,unsigned short v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;

/* character string index routine - note well - a NAN is a null pointer */
int string_index(int nitem, char* v[], int ndx[], int*count_num_p, int value_ascend, int index_ascend) ;

#ifdef __cplusplus
}
#endif


#endif
