/* routines to give trig functions with aruguments in degrees
   (replacement for sunmath.h include file in some downlink applications) */

#ifndef __trig_degrees
#define __trig_degrees

#include <math.h>

#ifdef DO_SUNMATH
#include <sunmath.h>
#else

/* taken from Appendix A of Knuth - Semi-Numerical Algorithms */
#define RADIANS_PER_DEGREE 0.017453292519943295769236907684

/* input argument is degrees, output is dimensionless */
#define sind(X) sin(RADIANS_PER_DEGREE*(X))
#define cosd(X) cos(RADIANS_PER_DEGREE*(X))
#define tand(X) tan(RADIANS_PER_DEGREE*(X))

/* input arguments are dimensionless, output is degrees */
#define asind(X) (asin(X)/RADIANS_PER_DEGREE)
#define acosd(X) (acos(X)/RADIANS_PER_DEGREE)
#define atan2d(X,Y) (atan2(X,Y)/RADIANS_PER_DEGREE)

#endif

#endif


