/*********************************************** 
dlutl_detcomp.c

Purpose

Compute the determinant of the product computed in radhit_init_transp.

Overview
Apply pivoting. Based on code by A.J. Penny, 2-16-83.

Definitions of Variables

Input:
I_MatLength  = Length of One Side of Matrix.
DP_MatInput = Input Matrix.

Output:
D_Det_Out = Determinant of DP_MatInput.

Internal:


***********************************************/

#include <stdio.h>
#include "dlutl.h"



dlutl_detcomp(		int	I_MatLength,
			double	*DP_MatInput,
			double	*DP_Det_Out,
			int	*IP_Status)
{
int	k, l, m;
double	save, D_Det_Out;
double	*DP_MatCopy;

if(*IP_Status) return;

/* Allocate space for copy of matrix */
DP_MatCopy = (double *) calloc(I_MatLength*I_MatLength, sizeof(double));

/* Set status if malloc failed */
if(DP_MatCopy==NULL) *IP_Status = DETCOMP|MALLOC_FAILED;

/* Don't continue if memory not allocated */
if((RIGHT_MASK&(*IP_Status))==MALLOC_FAILED) return;

memcpy(DP_MatCopy, DP_MatInput, I_MatLength*I_MatLength*sizeof(double));

D_Det_Out = 1;
 
for(k=0;k<I_MatLength;k++)	{

/*  Interchange columns if diagonal element is zero */
	if( DP_MatCopy[k*I_MatLength+k] == 0 )	{
	for(l=k;l < I_MatLength && DP_MatCopy[l*I_MatLength+k] == 0;l++);
	if(l == I_MatLength) {D_Det_Out = 0; break;}
          
	for(m=k;m<I_MatLength;m++)	{
		save = DP_MatCopy[l*I_MatLength+m];
		DP_MatCopy[l*I_MatLength+m] = DP_MatCopy[k*I_MatLength+m];
		DP_MatCopy[k*I_MatLength+m]= save;
	}
	D_Det_Out = -D_Det_Out;
	}

/*   Subtract row k from lower rows to get diagonal matrix */
	D_Det_Out = D_Det_Out*DP_MatCopy[k*I_MatLength+k];
	if(k < I_MatLength - 1)		
	for(m=k+1;m<I_MatLength;m++)	
	for(l=k+1;l<I_MatLength;l++)	
	DP_MatCopy[l*I_MatLength+m]-= DP_MatCopy[k*I_MatLength+m]*
	DP_MatCopy[l*I_MatLength+k]/DP_MatCopy[k*I_MatLength+k];

}

/* Deallocate Memory */
if(DP_MatCopy!=NULL) free(DP_MatCopy);

*DP_Det_Out = D_Det_Out;

}





