/*********************************************** 
dlutl_invertmat.c

Purpose

Invert a matrix.

Overview
Direct computation through minor determinants.

Definitions of Variables

Input:
I_MatLength  = Length of One Side of Matrix.
DP_MatInput = Input Matrix.

Output:
DP_MatInv = Inverse of DP_MatInput.


Internal:
I_Sign_Minor = Sign of the Minor.
D_Det_Mat = Determinant of Input Matrix.
DP_MatMinor = Minor.
D_Det_Minor = Determinant of Minor.


***********************************************/

#include <stdio.h>
#include "dlutl.h"


dlutl_invertmat(	int	I_MatLength,
			double	*DP_MatInput,
			double	*DP_MatInv,
			int	*IP_Status)
{
int	k, l, m, n;
int	I_Sign_Minor, I_Sign_Minor_Row, I_Sign_Minor_Col;
double	D_Det_Mat, D_Det_Minor;
double	*DP_MatMinor;

if(*IP_Status) return;

/* Allocate space for submatrices */
DP_MatMinor = (double *) calloc((I_MatLength-1)*(I_MatLength-1), sizeof(double));

/* Set status if malloc failed */
if(DP_MatMinor==NULL)*IP_Status = MAT_INVERT|MALLOC_FAILED;

/* Don't continue if memory not allocated */
if((RIGHT_MASK&(*IP_Status))==MALLOC_FAILED) return;
dlutl_detcomp(		I_MatLength,
			DP_MatInput,
			&D_Det_Mat,
			IP_Status);
			
I_Sign_Minor_Row  = -1;
for(k=0;k<I_MatLength;k++)	{
I_Sign_Minor_Row *= -1;
I_Sign_Minor_Col  = -1;
for(l=0;l<I_MatLength;l++)	{
I_Sign_Minor_Col *= -1;
I_Sign_Minor = I_Sign_Minor_Row*I_Sign_Minor_Col;

/* Find Minor */
for(m=0;m<I_MatLength;m++)
for(n=0;n<I_MatLength;n++)	{
if(m<l && n<k)
DP_MatMinor[n*(I_MatLength-1)+m] 	= DP_MatInput[n*I_MatLength+m];
if(m<l && n>k)
DP_MatMinor[(n-1)*(I_MatLength-1)+m] 	= DP_MatInput[n*I_MatLength+m];
if(m>l && n<k)
DP_MatMinor[n*(I_MatLength-1)+(m-1)] 	= DP_MatInput[n*I_MatLength+m];
if(m>l && n>k)
DP_MatMinor[(n-1)*(I_MatLength-1)+(m-1)]= DP_MatInput[n*I_MatLength+m];
}


/* Take Determinant of Minor */
dlutl_detcomp(		I_MatLength-1,
			DP_MatMinor,
			&D_Det_Minor,
			IP_Status);

/* Check for singularity */
if(D_Det_Mat==0) { *IP_Status = MAT_INVERT|MAT_SINGULAR; break;}
		
/* Apply formula for inverse of matrix */	
DP_MatInv[l*I_MatLength+k] = I_Sign_Minor*D_Det_Minor/D_Det_Mat;

}
}

free(DP_MatMinor);

}





