/*********************************************** 
dlutl_pseudocomp.c

Purpose

Compute the pseudoinverse of DP_GMatrix.


Overview
The pseudoinverse of G is (G^T G)^-1 G^T, where G^T is the
transpose of G.

Definitions of Variables

Input:
I_Num_Rows  = Number of Rows of Matrix.
I_Num_Columns = Number of Columns of Matrix.
DP_GMatrix  =  Matrix to find pseudoinverse of.
DP_GTransProd = Product of transpose of G Matrix and itself.


Output:
DP_GMat_Pseudo = Pseudoinverse of DP_GMatrix.

Internal:
DP_MatInv = Inverse of DP_GTransProd.

***********************************************/

#include <stdio.h>
#include "dlutl.h"


dlutl_pseudocomp(	int	I_Num_Rows,
			int	I_Num_Columns,
			double	*DP_GMatrix,
			double	*DP_GTransProd,
			double	*DP_GMat_Pseudo,
			int	*IP_Status)
{
int	k, l;
double	*DP_MatInv;
double	*DP_RHMult;
double	*DP_GTrans;

if(*IP_Status) return;

/* Allocate space for inverse of matrix */
DP_MatInv = (double *) calloc(I_Num_Columns*I_Num_Columns, sizeof(double));

/* Set status if malloc failed */
if(DP_MatInv==NULL) *IP_Status = PSEUDOCOMP|MALLOC_FAILED;

/* Don't continue if memory not allocated */
if((RIGHT_MASK&(*IP_Status))==MALLOC_FAILED) return;

/* Invert the product of the transpose of G and itself */
dlutl_invertmat(	I_Num_Columns,
			DP_GTransProd,
			DP_MatInv,
			IP_Status);

/* Don't continue if matrix inversion failed */
if(MAT_INVERT&(*IP_Status)) return;

/* Allocate space for  the transpose of G */
DP_GTrans = (double *) calloc(I_Num_Columns*I_Num_Rows, sizeof(double));

/* Set status if malloc failed */
if(DP_GTrans==NULL)*IP_Status = PSEUDOCOMP|MALLOC_FAILED;

/* Don't continue if memory not allocated */
if((RIGHT_MASK&(*IP_Status))==MALLOC_FAILED) return;

/* Compute the transpose of G */
dlutl_transp(		DP_GMatrix,
			I_Num_Rows,
			I_Num_Columns,
			DP_GTrans,
			IP_Status);

/* Multiply DP_MatInv on the right by the transpose of G */
dlutl_matmult(		DP_MatInv,
			I_Num_Columns,
			I_Num_Columns,
			DP_GTrans,
			I_Num_Columns,
			I_Num_Rows,
			DP_GMat_Pseudo,
			IP_Status);

free(DP_MatInv);
free(DP_GTrans);

}





