c=======================================================================
c
c    SIRTF Baseline calibration program; cloned from lincal, this
c    routine fits a full quadratic to a ramp in order to estimate
c    the baseline at zero integration time, then it subtracts that
c    baseline off of all planes.
c
c     $Id: basecal.f,v 1.13 2005/02/17 22:59:14 jwf Exp $
c
c  Vsn 1.0  A20412: Initial version
c  Vsn 1.1  A20808: Removed hooks for alpha method; made not weighting
c                   with input uncertainties the default
c  Vsn 1.2  A20822: Added Qmin to NAMELIST, defaulted to 0.1
c  Vsn 1.3  A20918: changed COMMENT to standardized HISTORY line for vsn ID
c      1.4  A40318: completed linux compatibility by changing makefile to
c                   link new aloha (from aloha.F) from include library
c      1.5  A41222: fixed NSkip zero-crossing error (zero crossing should
c                   be at plane zero, not plane NSkip); added radhit
c                   processing (know what the tag in the dmask is, don't
c                   use any part of a ramp above one); added linear model
c                   for case in which only two planes survive for fitting;
c                   removed NLModel array, which wasn't being used
c      1.51 A50112: added MinMax parameter
c      1.52 A50125: added "-r" processing and baseline uncertainty output
c                   image file
c      1.53 A50128: added DBI/DBJ debug processing; fixed bug in radhit-
c                   impacted ramp's uncertainty computation
c      1.54 A50202: made malloc of Base unconditional and added use of it
c                   for A/D-channel averaging of baseline values to be
c                   applied to dmask-bit-10 pixels iff A2Davg = T
c      1.55 A50208: added umask option; changed A2Davg processing from
c                   bit-10 trigger to Base(n) = NaN trigger; changed DMB2pt
c                   default to 10
c
      Integer*4     IArgC, LNBlnk, NPix, FileID1, FileID2, DMBRad, 
     +              DMB2pt,
     +              MAlloc, NRows, NCols, NRNL, NCNL, Access, Ninfl8,
     +              I, ImOpen, IStat, ImWkeyT, ImWPixR, ImClose, N,
     +              ImRPixR, J, ImWKeyI, NPlanes, NFail(6), Kode, NClip,
     +              NArgs, NArg, NPCd, NPlns, ImRPixS, NPixSat, Ncomp,
     +              ImRKeyC, ImWKeyC, L1, L2, FileID0, ImWPixS, NA2D(4),
     +              NA2DC(4)
      Integer*2     DMask(*), PMask(*), umask(*)
      Logical*1     Used(*), NeedHelp, GotIF, GotOF, UseDM, UsePM,
     +              GotBN, UseUnc, UseUnc2, GotNL, GotUM
      Character*150 InFNam, OutFNam, Out2FNam, NLNam, BasNam, UMNam,
     +              DMaskNam, PMaskNam, UncNam, Unc2Nam, UncBasNam
      Character*50  BUNIT, BUNIT2
      Character*25  Line
      Character*11  Vsn
      Character*8   CDate, CTime
      Character*2   Flag
      Real*4        Cube(*), Absc(*), Sigmas(*), T(*),
     +              Unc(*), Base(*), UncBase(*), AvgA2DBase(4),
     +              SigA2DBase(4), A2DBase, A2DSig
      Real*8        SumBase(4), SumSqBase(4)
      Pointer       (pC, Cube), (pA, Absc), (pB, Base),
     +              (pSg, Sigmas), (pU, Used),  (pT, T),
     +              (pDM, DMask), (pPM, PMask), (pUn, Unc),
     +              (pUB, UncBase), (pUM, umask)
c
      Data Vsn/'1.55 A50208'/, NeedHelp/.False./, SumBase/4*0.0d0/,
     +     GotIF/.False./, GotOF/.False./, UseDM/.False./, NClip/0/,
     +     UsePM/.False./, GotNL/.False./, NPCd/0/, NA2D/4*0/,
     +     NPixSat/0/, UseUnc2/.False./, SumSqBase/4*0.0d0/,
     +     UseUnc/.False./, Ncomp/0/, GotBN/.False./, Kode/0/,
     +     GotUM/.False./, NA2DC/4*0/
C
      Common / VDT / CDate, CTime, Vsn
c
c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      Integer*4 NSigMax, NSkip, DBI, DBJ
      Integer*2 MaskD, MaskP
      Real*4    ZSig, MinDN, SigFill, VarFill, Qmin,
     +          MaxDN, SigMax, Infl8, MinMax, ClipVal
      Logical   DoSN, DoWgt, RSS, A2Davg, IzNaN, ClipA2D
c
      Data ZSig/1.5/, MaxDN/9.9e9/, SigMax/250.0/, RSS/.False./,
     +     MinDN/-9.9e9/, NSkip/0/, DoWgt/.False./, MinMax/9.9e9/,
     +     MaskD/30728/, MaskP/32640/, SigFill/1.0/, Qmin/0.1/,
     +     DoSN/.False./, Infl8/100.0/, DMBRad/9/, DMB2pt/10/,
     +     DBI/-9/, DBJ/-9/, A2Davg/.True./, ClipA2D/.True./,
     +     ClipVal/1.0/
c
      Common / Contrl / ZSig, MaxDN, SigMax, NSigMax, NSkip, DBI, DBJ,
     +                  MinDN, MinMax, MaskD, MaskP, DoWgt, RSS
c
      NameList / BasCIn / A2Davg, ClipA2D, ClipVal, DBI, DBJ,
     +                    DMB2pt, DMBRad, DoSN, DoWgt, Infl8,
     +                    MaskD, MaskP, MaxDN, MinDN, MinMax, NSkip,
     +                    Qmin, RSS, SigFill, SigMax, ZSig
c
c=======================================================================
c
      NArgs = IArgC()
1     If ((NArgs .lt. 4) .or. (NArgs .gt. 20) .or.
     +    (mod(NArgs,2) .ne. 0) .or. NeedHelp) then
        print *,'SIRTF Baseline Calibration program vsn ',Vsn
        print *
        print *,'usage:  basecal <flags> <specifications>'
        print *
        print *,'where <flags> may be: ',
     +          '-b -i -d -m -n -o -p -r -u -v -x -z'
        print *,'any one of which, if specified, must be followed ',
     +          'by a blank'
        print *,'and a corresponding specification, as follows.'
        print *
        print *,'    -b basfilnam (name of output baseline image; ',
     +                            'optional)'
        print *,'    -i infilnam  (name of input data cube; required)'
        print *,'    -d dmasknam  (name of input dmask ',
     +                            'file; optional)'
        print *,'    -m maxpixval (maximum pixel value to use; ',
     +                            'optional)'
        print *,'    -n nlnam     (name of NAMELIST file; optional)'
        print *,'    -o outfilnam (name of output data cube; optional)'
        print *,'    -p pmasknam  (name of pmask file; optional)'
        print *,'    -r yes|no    ("yes" --> RSS baseline & input ',
     +                            'sigmas; optional)'
        print *,'    -u uncrtnam1 (name of input uncertainty data ',
     +                            'cube; optional)'
        print *,'    -v uncrtnam2 (name of output uncertainty data ',
     +                            'cube; optional)'
        print *,'    -x nskip     (number of samples to ',
     +                            'skip; optional)'
        print *,'    -z umasknam  (name of input umask ',
     +                            'file; optional)'
        print *
        print *,'Either the -b or -o option must be specified. The ',
     +          'baseline'
        print *,'uncertainties will NOT be RSS''d with input ',
     +          'uncertainties'
        print *,'unless "-r yes" is specified.'
        call exit(64)
      end if
c
      write (6, 6000)
c
      call Aloha(1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
2     NArg = 0
4     NArg = NArg + 1
      call GetArg(NArg,Flag)
      NArg = NArg + 1
c
      If (Flag .eq. '-b') then
        call GetArg(NArg,BasNam)
        If (Index(BasNam,'.fits') .eq. 0)
     +  BasNam = BasNam(1:LNBlnk(BasNam))//'.fits'
        GotBN = .True.
c
      else if (Flag .eq. '-i') then
        call GetArg(NArg,InFNam)
        If (Index(InFNam,'.fits') .eq. 0)
     +  InFNam = InFNam(1:LNBlnk(InFNam))//'.fits'
        if (Access(InFNam(1:LNBlnk(InFNam)),' ') .ne. 0) then
          print *,'File not found: ',InFNam(1:LNBlnk(InFNam))
          call exit(64)
        end if
        GotIF = .True.
c
      else If (Flag .eq. '-d') then
        call GetArg(NArg,DMaskNam)
        If (Index(DMaskNam,'.fits') .eq. 0)
     +  DMaskNam = DMaskNam(1:LNBlnk(DMaskNam))//'.fits'
        if (Access(DMaskNam(1:LNBlnk(DMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',DMaskNam(1:LNBlnk(DMaskNam))
          call exit(64)
        end if
        UseDM = .True.
c
      else If (Flag .eq. '-n') then
        If (GotNL) Go to 8
        call GetArg(NArg,NLNam)
        if (Access(NLNam(1:LNBlnk(NLNam)),' ') .ne. 0) then
          print *,'File not found: ',NLNam(1:LNBlnk(NLNam))
          call exit(64)
        end if
        Open (10, File = NLNam)
        Read (10, BasCIn, End = 3003, Err= 3002)
        Close (10)
        GotNL = .True.                 ! reprocess command-line after
        Go to 2                        !  reading in NAMELIST
c
      else If (Flag .eq. '-o') then
        call GetArg(NArg,OutFNam)
        if (Index(OutFNam,'.fits') .eq. 0)
     +    OutFNam = OutFNam(1:LNBlnk(OutFNam))//'.fits'
        GotOF = .True.
c
      else If (Flag .eq. '-p') then
        call GetArg(NArg,PMaskNam)
        if (Index(PMaskNam,'.fits') .eq. 0)
     +    PMaskNam = PMaskNam(1:LNBlnk(PMaskNam))//'.fits'
        if (Access(PMaskNam(1:LNBlnk(PMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',PMaskNam(1:LNBlnk(PMaskNam))
          call exit(64)
        end if
        UsePM = .True.
c
      else if (Flag .eq. '-m') then
        Call GetArg(NArg, Line)
        Read (Line,*,Err=5) MaxDN
        print *,'Using maximum pixel value = ',MaxDN
        Go to 8
5       print *,'WARNING: ignored illegal -m specification: ',Line
        Kode = 32
c
      else if (Flag .eq. '-x') then
        Call GetArg(NArg, Line)
        Read (Line,*,Err=7) I
        If (I .lt. 0) go to 7
        NSkip = I
        print *,'Using NSkip = ',NSkip
        Go to 8
7       print *,'WARNING: ignored illegal -x specification: ',Line
        Kode = 32
c
      else If (Flag .eq. '-u') then
        call GetArg(NArg,UncNam)
        If (Index(UncNam,'.fits') .eq. 0)
     +  UncNam = UncNam(1:LNBlnk(UncNam))//'.fits'
        if (Access(UncNam(1:LNBlnk(UncNam)),' ') .ne. 0) then
          print *,'File not found: ',UncNam(1:LNBlnk(UncNam))
          call exit(64)
        end if
        UseUnc = .True.
c
      else If (Flag .eq. '-v') then
        call GetArg(NArg,Unc2Nam)
        If (Index(Unc2Nam,'.fits') .eq. 0)
     +  Unc2Nam = Unc2Nam(1:LNBlnk(Unc2Nam))//'.fits'
        UseUnc2 = .True.
c
      else if (Flag .eq. '-r') then
        Call GetArg(NArg, Line)
        if (Line .eq. 'yes') RSS = .True.
c
      else if (Flag .eq. '-z') then
        call GetArg(NArg,UMNam)
        If (Index(UMNam,'.fits') .eq. 0)
     +  UMNam = UMNam(1:LNBlnk(UMNam))//'.fits'
        GotUM = .True.
c
      Else
        print *,'Unrecognized flag: ',Flag
        NeedHelp = .True.
        Print *
        Go to 1
      End If
c
8     If (NArg .lt. NArgs) Go to 4
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (.not.GotIF) then
        print *,'No input image list specified'
        Print *
        NeedHelp = .True.
        Go to 1
      End If
c
      If (.not. (GotOF .or. GotBN)) then
        print *,'No output file name specified'
        Print *
        NeedHelp = .True.
        Go to 1
      End If
c
      Write (6, BasCIn)
c
      If (GotUM .and. .not.A2Davg) then
        print *,
     +   'WARNING: umask specified, but A/D-channel averaging is'
        print *,
     +   '         disabled; the umask specification will be ignored.'
        GotUM = .False.
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      Call GetNAX(InFNam,NCols,NRows,NPlanes,1,FileID0)
      NPix = NCols*NRows
      If (NPlanes .le. NSkip) then
        print *,'ERROR: NPlanes must be greater than NSkip'
        print *,'       NPlanes = ',NPlanes,'; NSkip = ',NSkip
        call exit(64)
      End If
c
      pC = MAlloc(4*NPix*NPlanes)      ! Allocate mem for input
      If (pC .eq. 0) then
        print *,'ERROR: unable to allocate memory for input data cube'
        call exit(64)
      End If
c
      IStat = ImRPixR(FileID0,Cube,NPlanes*NPix)
      If (IStat .ne. NPlanes*NPix) then
        print *,'ERROR: could not read input data cube'
        print *,'       File name: ',InFNam(1:LNBlnk(InFNam))
        print *,'       ImRPixR returned error code ',IStat
        call exit(64)
      end if
c
      IStat = ImRKeyC(FileID0,BUNIT,'BUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading BUNIT in input data cube'
        print *,'      File name: ',InFNam(1:LNBlnk(InFNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      pSg = MAlloc(8*NPix)             ! Allocate mem for sigmas &
      If (pSg .eq. 0) then             !  no. pts used for fits
        print *,'ERROR: unable to allocate memory for sigma image'
        call exit(64)
      End If
c
      pA = MAlloc(4*NPlanes)           ! Allocate mem for abscissae
      If (pA .eq. 0) then
        print *,'ERROR: unable to allocate memory for abscissae'
        call exit(64)
      End If
c
      pT = MAlloc(4*NPlanes)           ! Allocate mem for T abscissae
      If (pT .eq. 0) then
        print *,'ERROR: unable to allocate memory for T abscissae'
        call exit(64)
      End If
c
      pU = MAlloc(NPlanes)             ! Allocate mem for abscissae
      If (pU .eq. 0) then              !  usage array
        print *,'ERROR: unable to allocate memory for absc usage'
        call exit(64)
      End If
c
      pPM = MAlloc(2*NPix)            ! Allocate mem for PMask
      If (pPM .eq. 0) then
        print *,'ERROR: unable to allocate memory for pmask image'
        call exit(64)
      End If
c
      pDM = MAlloc(2*NPlanes*NPix)    ! Allocate mem for DMask
      If (pDM .eq. 0) then
        print *,'ERROR: unable to allocate memory for dmask image'
        call exit(64)
      End If
c
      pB = MAlloc(4*NPix)             ! Allocate mem for Base
      If (pB .eq. 0) then
        print *,'ERROR: Unable to allocate memory for baseline image'
        call exit(64)
      End If
c
      If (GotBN) then
        pUB = MAlloc(4*NPix)          ! Allocate mem for UncBase
        If (pUB .eq. 0) then
          print *,
     +    'ERROR: Unable to allocate memory for baseline uncertainties'
          call exit(64)
        End If
      Else
        pUB = MAlloc(1)
      End If
c
      If (GotUM) then
        pUM = MAlloc(2*NPix)          ! Allocate mem for umask
        If (pUM .eq. 0) then
          print *,
     +    'ERROR: Unable to allocate memory for umask'
          call exit(64)
        End If
        Call GetNAX(UMNam,NCNL,NRNL,NPlns,1,FileID1)
        If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols) .or.
     +      (NPlns .ne. 1)) then
          print *,'ERROR: wrong dimensions in umask'
          print *,'       File name: ',UMNam(1:LNBlnk(UMNam))
          print *,'       Expected ',NCols,' by ',NRows,' by 1'
          print *,'       Got ',NCNL,' by ',NRNL,' by ',NPlns
          call exit(64)
        End If
        IStat = ImRPixS(FileID1,umask,NPix)
        If (IStat .ne. NPix) then
          print *,'ERROR: could not read input umask'
          print *,'       File name: ',UMNam(1:LNBlnk(UMNam))
          print *,'       ImRPixS returned error code ',IStat
          call exit(64)
        end if
        IStat = ImClose(FileID1)
      Else
        pUM = MAlloc(1)
      End If
c
      If (UseUnc .or. UseUnc2) then
        pUn = MAlloc(4*NPix*NPlanes)  ! Allocate mem for uncertainties
      Else
        pUn = Malloc(1)               ! Legitimize array address
      End If
      If (pUn .eq. 0) then
        print *,
     +    'ERROR: unable to allocate memory for uncertainty image'
        call exit(64)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
58    If (.not.UsePM) then
        Do 60 I = 1, NPix
          PMask(I) = 0
60      Continue
        Go to 65
      End If
c
      Call GetNAX(PMaskNam,NCNL,NRNL,NPlns,1,FileID1)
      If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols) .or.
     +    (NPlns .ne. 1)) then
        print *,'ERROR: wrong dimensions in pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by 1'
        print *,'       Got ',NCNL,' by ',NRNL,' by ',NPlns
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID1,PMask,NPix)
      If (IStat .ne. NPix) then
        print *,'ERROR: could not read input pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        call exit(64)
      end if
c
      IStat = ImClose(FileID1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
65    If (.not.UseDM) then
        Do 68 I = 1, NPlanes*NPix
          DMask(I) = 0
68      Continue
        Go to 72
      End If
c
      Call GetNAX(DMaskNam,NCNL,NRNL,NPlns,1,FileID1)
      If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols) .or.
     +    (NPlanes .ne. NPlns)) then
        print *,'ERROR: wrong dimensions in dmask image'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,
     +                  ' by ',NPlanes
        print *,'       Got ',NCNL,' by ',NRNL,' by ',NPlns
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID1,DMask,NPlanes*NPix)
      If (IStat .ne. NPlanes*NPix) then
        print *,'ERROR: could not read input dmask'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        call exit(64)
      end if
c
      IStat = ImClose(FileID1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
72    If (.not.UseUnc) Go to 80
      Call GetNAX(UncNam,NCNL,NRNL,NPlns,1,FileID1)
      If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols) .or.
     +    (NPlanes .ne. NPlns)) then
        print *,'ERROR: wrong dimensions in uncertainty image'
        print *,'       File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'       Expected ',NCols,' by ',NRows,
     +                  ' by ',NPlanes
        print *,'       Got ',NCNL,' by ',NRNL,' by ',NPlns
        call exit(64)
      End If
c
      IStat = ImRKeyC(FileID1,BUNIT2,'BUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading BUNIT in uncertainty image'
        print *,'      File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
      If (BUNIT2 .ne. BUNIT) then
        print *,'ERROR in BUNIT in uncertainty image'
        print *,'      File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'      unc. BUNIT: ',BUNIT2(1:LNBlnk(BUNIT2))
        print *,'      img. BUNIT: ',BUNIT(1:LNBlnk(BUNIT))
        print *,'unc. BUNIT must match img. BUNIT'
        call exit(64)
      end if
c
      IStat = ImRPixR(FileID1,Unc,NPlanes*NPix)
      If (IStat .ne. NPlanes*NPix) then
        print *,'ERROR: could not read input uncertainty image'
        print *,'       File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'       ImRPixR returned error code ',IStat
        call exit(64)
      end if
c
      IStat = ImClose(FileID1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
80    VarFill = SigFill*SigFill
      If (VarFill .eq. 0.0) then
        print *,'WARNING: Illegal value of SigFill specified: ',SigFill
        print *,'         Set to 1.0'
        VarFill = 1.0
        Kode = 32
      end if
c
      Call GenMod(Cube,Sigmas,PMask,DMask,Base,UncBase,
     +            Absc,T,NCols,NRows,NPlanes,Used,NFail,
     +            Unc,UseUnc,UseUnc2,GotBN,VarFill,Qmin,Ncomp,
     +            Ninfl8,Infl8,DMBRad,DMB2pt)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (.not.A2Davg) go to 95
      Do 82 I = 1, NPix
        If (IzNaN(Base(I))) go to 82
        If (GotUM) then
          If (umask(I) .lt. 1) go to 82
        end if
        J = mod(I-1,4) + 1             ! A/D channel number
        NA2D(J) = NA2D(J) + 1
        SumBase(J) = SumBase(J) + Base(I)
        SumSqBase(J) = SumSqBase(J) + Base(I)**2
82    Continue
c
      Do 84 J = 1, 4
        If (NA2D(J) .gt. 0) then
          AvgA2DBase(J) = SumBase(J)/Float(NA2D(J))
        else
          print *,
     +    'WARNING: no points to average baselines in A/D channel ',J
        end if
        If (NA2D(J) .gt. 1)
     +      SigA2DBase(J) = sqrt(abs(SumSqBase(J)/Float(NA2D(J))
     +                    - AvgA2DBase(J)**2))
84    Continue
c
      Do 86 I = 1, NPix
        If (.not.IzNaN(Base(I))) go to 86
        J = mod(I-1,4) + 1             ! A/D channel number
        NA2DC(J) = NA2DC(J) + 1
        If ((Cube(NSkip*NPix+I) .gt. AvgA2DBase(J)) .or.
     +      (.not.ClipA2D)) then
          A2DBase = AvgA2DBase(J)
          A2DSig  =  SigA2DBase(J)
        Else
          A2DBase = Cube(NSkip*NPix+I) - ClipVal
          A2DSig  = sqrt(SigA2DBase(J)**2 + A2DBase**2)
          NClip = NClip + 1
        End If
        Do 85 N = NSkip, NPlanes-1
          Base(I) = A2DBase
          If (GotBN) UncBase(I) = A2DSig
          Cube(N*NPix+I) = Cube(N*NPix+I) - A2DBase
85      Continue
86    Continue
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
95    NFail(4) = NFail(4)/(NPlanes - NSkip)
      J = NFail(2) + NFail(3) + NFail(4)
      print *,'No. of pixels for which model could not be computed:',J
      If (J .gt. 0) then
        print *,NFail(2),': insufficient number of usable points'
        print *,NFail(3),': linear system was singular'
        print *,NFail(4),': masked in the PMask'
      End If
      If (NFail(1) .gt. 0) print *, NFail(1),
     +   ' pixel samples were less than MinDN or greater than MaxDN'
      If (NFail(5) .gt. 0) Print *,NFail(5),
     +   ' pixel samples were masked in the DMask'
      If (NSigMax .gt. 0) Print *,
     +   'No. of pixels with fitting sigma > maximum: ',NSigMax
      If (NFail(6) .gt. 0) Print *,
     +   'No. of pixels with 2-point extrapolations:  ',NFail(6)
      If (A2Davg) then
        Print *,'No. of A/D-avg baseline corrections/chan:   ',NA2DC
        Print *,'No. of points averaged per A/D channel:     ',NA2D
        Print *,'Average baseline in each A/D channel:       ',
     +           AvgA2DBase
        Print *,'Sigma about baseline in each A/D channel:   ',
     +           SigA2DBase
        If (ClipA2D) print *,
     +      'No. of A/D-avg baseline corrections clipped:',NClip
      End If
      If (DoWgt) then
        Print *,'No. of goodness-of-fit compensations:       ',Ncomp
        Print *,'No. of goodness-of-fit inflations:          ',Ninfl8
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      L1 = LNBlnk(InFNam) - 63
      If (L1 .lt. 1) then
        L1 = 1
      Else
        InFNam(L1:L1+2) = '...'
      End If
c
      If (UseUnc) then
        L2 = LNBlnk(UncNam) - 63
        If (L2 .lt. 1) then
          L2 = 1
        Else
          UncNam(L2:L2+2) = '...'
        End If
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (GotOF) then
c                                        ! DCE output
        FileID2 = ImOpen(OutFNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'ERROR: unable to open output file ',
     +             OutFNam(1:LNBlnk(OutFNam))
          call exit(64)
        End If
c
        Call CpyHdr(FileID0,FileID2)
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'A')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +            //InFNam(L1:LNBlnk(InFNam)),'A')
        If (UseUnc) then
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +              //UncNam(L2:LNBlnk(UncNam)),'A')
        End If
        If (A2Davg) then
          IStat = ImWKeyT(FileID2,'COMMENT    A/D-averaged baselines '
     +          // 'were used for dmask-bit-10 pixels','A')
        End If
c
        IStat = ImWPixR(FileID2,Cube,NPix*NPlanes)
        If (IStat .ne. NPix*NPlanes) then
          print *,'ERROR: unable to generate output file ',
     +             OutFNam(1:LNBlnk(OutFNam))
          print *,'Error code: ',IStat
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (UseUnc2) then
c                                        ! DCE output
        FileID2 = ImOpen(Unc2Nam, "W")
        If (FileID2 .lt. 1) Then
          print *,'ERROR: unable to open output uncertainty file ',
     +             Unc2Nam(1:LNBlnk(Unc2Nam))
          call exit(64)
        End If
c
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,3,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyI(FileID2,NPlanes,'NAXIS3')
        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
        IStat = ImWKeyT(FileID2,'COMMENT      This is a DCE ' //
     +                 'uncertainty image','A')
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'A')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +            //InFNam(L1:LNBlnk(InFNam)),'A')
        If (UseUnc) then
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +              //UncNam(L2:LNBlnk(UncNam)),'A')
        End If
c
        IStat = ImWPixR(FileID2,Unc,NPix*NPlanes)
        If (IStat .ne. NPix*NPlanes) then
          print *,'ERROR: unable to generate output uncertainty file ',
     +             Unc2Nam(1:LNBlnk(Unc2Nam))
          print *,'Error code: ',IStat
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (GotBN) then
c                                        ! Baseline output
        FileID2 = ImOpen(BasNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'ERROR: unable to open output file ',
     +             BasNam(1:LNBlnk(BasNam))
          call exit(64)
        End If
c
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,2,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
        IStat = ImWKeyT(FileID2,'COMMENT      This is a ' //
     +                 'baseline image','A')
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'A')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +            //InFNam(L1:LNBlnk(InFNam)),'A')
        If (UseUnc) then
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +              //UncNam(L2:LNBlnk(UncNam)),'A')
        End If
c
        IStat = ImWPixR(FileID2,Base,NPix)
        If (IStat .ne. NPix) then
          print *,'ERROR: unable to generate output file ',
     +             BasNam(1:LNBlnk(BasNam))
          print *,'Error code: ',IStat
          call exit(64)
        End If
        IStat = ImClose(FileID2)
c                                        ! Baseline uncertainty output
        I = Index(BasNam,'.fits') - 1
        UncBasNam = BasNam(1:I) // '_unc.fits'
        if (UseUnc2) then
          if (Unc2Nam .eq. UncBasNam) then
            I = Index(UncBasNam,'.fits') - 1
            UncBasNam = UncBasNam(1:I) // '_2.fits'
          end if
        end if
        FileID2 = ImOpen(UncBasNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'ERROR: unable to open output file ',
     +             UncBasNam(1:LNBlnk(UncBasNam))
          call exit(64)
        End If
c
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,2,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
        IStat = ImWKeyT(FileID2,'COMMENT      This is a ' //
     +                 'baseline uncertainty image','A')
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'A')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +            //InFNam(L1:LNBlnk(InFNam)),'A')
        If (UseUnc) then
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +              //UncNam(L2:LNBlnk(UncNam)),'A')
        End If
c
        IStat = ImWPixR(FileID2,UncBase,NPix)
        If (IStat .ne. NPix) then
          print *,'ERROR: unable to generate output file ',
     +             UncBasNam(1:LNBlnk(UncBasNam))
          print *,'Error code: ',IStat
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (DoSN) then
c                                        !  Auxiliary output
        If (GotOF) then
          I = Index(OutFNam,'.fits') - 1
          Out2FNam = OutFNam(1:I) // '_sn.fits'
        Else
          I = Index(BasNam,'.fits') - 1
          Out2FNam = BasNam(1:I) // '_sn.fits'
        End If
        FileID2 = ImOpen(Out2FNam, "W")  ! Output sigmas & # pts,
        If (FileID2 .lt. 1) Then         !  for quadratic fits
          print *,'ERROR: unable to open output file ',
     +             Out2FNam(1:LNBlnk(Out2FNam))
          Kode = 32
          Go to 100
        End If
c
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,3,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyI(FileID2,2,'NAXIS3')
        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
        IStat = ImWKeyT(FileID2,'COMMENT      This is a ' //
     +                 'sigmas/Ndata image','A')
        IStat = ImWKeyT(FileID2,'COMMENT      Plane 1 = ' //
     +                 'sigmas about quadratic fit','A')
        IStat = ImWKeyT(FileID2,'COMMENT      Plane 2 = ' //
     +                 'no. points used in quadratic fit','A')
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'A')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +            //InFNam(L1:LNBlnk(InFNam)),'A')
        If (UseUnc) then
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +              //UncNam(L2:LNBlnk(UncNam)),'A')
        End If
c
        IStat = ImWPixR(FileID2,Sigmas,2*NPix)
        If (IStat .ne. 2*NPix) then
          print *,'ERROR: unable to generate output file ',
     +             Out2FNam(1:LNBlnk(Out2FNam))
          print *,'Error code: ',IStat
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
100   If (UseDM) then
        FileID2 = ImOpen (DMaskNam, "U")
        If (FileID2 .lt. 1) Then
          print *,'Unable to update dmask file ',DMaskNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyT(FileID2,'HISTORY basecal           v.  '
     +          //Vsn//',  updated  '//CDate//' at '//CTime,'A')
        IStat = ImWPixS(FileID2,DMask,NCols*NRows*NPlanes)
        If (IStat .ne. NCols*NRows*NPlanes) then
          print *,'Unable to write updated dmask image ',DMaskNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      write (6, 6000)
c
      call Aloha(2)
c
      call exit(Kode)
      stop
c
c=======================================================================
c
3002  print *,'ERROR reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      call exit(64)
3003  print *,'EOF reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      call exit(64)
      stop
c
c-----------------------------------------------------------------------
c
6000  Format('basecal'$)
c
      End
c
c=======================================================================
c
      Function IzNaN(R4)
c
      Logical   IzNaN
      Real*4    R4, R4a
      Integer*4 I4, MaskExp, MaskMant
      Equivalence (I4, R4a)
c
      Data MaskExp/2139095040/, MaskMant/8388607/
c
c-----------------------------------------------------------------------
c
c   01111111100000000000000000000000 = 2139095040 = 7F80000 hex
c   00000000011111111111111111111111 =    8388607 =  7FFFFF hex
c
c-----------------------------------------------------------------------
c
      R4a = R4
      IzNaN = ((And(I4,MaskExp)  .eq. MaskExp) .and.
     +         (And(I4,MaskMant) .ne. 0))
      Return
      End
