/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@ PROGRAM:  coadd2d.c
@@
@@ PURPOSE:  Coadd a sequence of FITS files, 
@@              using any appropriate mask files, calculating uncertainties
@@
@@ ROUTINES: command line main routine
@@
@@ PROGRAMMERS:
@@	CW  - Clare Waterson
@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

#include <stdio.h>
#include <string.h>  /* For strtok() */
#include <math.h>
#include "coadd2d.h"

#define DBG 0
#define TESTING_ASYMMETRIC_TRIMMING 0
#define DEBUG_ASYMMETRIC_TRIMMING   0

/***************************************************************************/
int main( int argc, char *argv[]) 
{ 
cmd_struct	cs;           /* Info from the command line is stored here   */
int             i,j,n_good;
int             x,y,xy;   
int             bmask_or_all;  /* logical OR of BMASK of all planes for a pixel */
int             status;                                  
char		firstfile[MAX_FILENAME_LEN];  /* A sample file to find dims   */
char		data_lfile[MAX_FILENAME_LEN], /* Name of the file containing  */
                sdev_lfile[MAX_FILENAME_LEN], /*   the list of filenames      */
                bmask_lfile[MAX_FILENAME_LEN], 
                pmask_lfile[MAX_FILENAME_LEN];
char            fovlist_str[MAX_FILENAME_LEN];                
char            bitlist_str[MAX_FILENAME_LEN];                
files_struct	*data_files, *sdev_files,     /* Arrays to hold the lists     */
                *bmask_files, *pmask_files;   /*   of filenames               */
Flag		sdev_in_nl, bmask_in_nl, pmask_in_nl; /* is data in the nl?   */
Flag		use_sdev;                        /* is this requested? */
Flag            use_fmask /*,use_cr_thresh*/;    /* are these requested? */
Flag            use_bmask, use_pmask;            /* are these requested? */
Flag            use_weights;                     /* should weighting be done? */
float           max_weight;
image_struct	*data_planes,  *sdev_planes;  /* Big 3d arrays of data        */
simage_struct   *bmask_planes, *pmask_planes; /* Big 3d arrays of short ints  */
short int       *fmask_plane;                 /* One 2d array  of short ints  */
int		ith_xdim, ith_ydim;       
data_struct	*data_column;        /* dimensions of the ith image plane         */
                                     /* a one-pixel-wide column of data           */
                                     /* selected from a big 3d data array.        */
                                     /* Holds GOOD DATA and the GOOD DATA weights */
float           *dce_weight;      /* Array containing weights for each */
                                  /*   input DCE data image            */
int             *dce_num_good;    /* Number of good datapoints in each DCE data image */ 
int             req_fov_num;      /* Require that all input data files have the same FOVID */
int             req_fov_dummy;      /* not require FOVID for unc files */

/* int          trimfactflag;    To use with trimmedstatistics()            */
/*                               "Set to 0 for first call, then leave alone */
/* double       facttotptr;      To use with trimmedstatistics(): ignore    */
/* double       datascale;       To use with trimmedstatistics(): ignore    */

float		*means, *sdevs;      /* 2D image results */
float		wtd_mean, wtd_sdev;  /* temp results */
short int	*amask;
float           flt_NaN;
Flag		good_data;

double sum_sdev_sqd;


/*===========================================================================================
**   
**     S P E C I A L   A S Y M M E T R I C   T R I M M I N G   T E S T I N G   C O D E
*/
#if TESTING_ASYMMETRIC_TRIMMINMG /*useful*/
data_struct  a[11];

   cs.cut_frac     =  0.5;
   cs.cut_multiple =  5.0;
   cs.debug        =  1;
   cs.verbose      =  1;
   printf("SET: cs.cut_frac     =%6.1f\n", cs.cut_frac     );
   printf("     cs.cut_multiple =%6.1f\n", cs.cut_multiple );
   printf("\n" );
   
   n_good          =  5;
   
   for ( i=0; i<n_good; i++ ) {
      a[i].plane = i+1;
   }
   
   a[ 0].value = 94;
   a[ 1].value = 92;
   a[ 2].value = 73;
   a[ 3].value = 98;
   a[ 4].value =222;
      
   printf("VALUES: ");
   for ( i=0; i<n_good; i++ ) printf("  a[%02d]=%5.1f",i,a[i].value);
   printf("\nPLANES: ");
   for ( i=0; i<n_good; i++ ) printf("  a[%02d]=%5d",i,a[i].plane);
   printf("\n");

   asymmetric_trimming( a, &n_good, cs );
   printf("ASYMMETRIC TRIMMING COMPLETE\n");

   printf("VALUES: ");
   for ( i=0; i<n_good; i++ ) printf("  a[%02d]=%5.1f",i,a[i].value);
   printf("\nPLANES: ");
   for ( i=0; i<n_good; i++ ) printf("  a[%02d]=%5d",i,a[i].plane);
   printf("\n");
   
   exit(1);
#endif  
   
/*==================================================================================*/
   
   
   sscanf("NaN", "%f",  &flt_NaN);  /* THIS IS THE GOOD FLOAT NaN TO USE!!! */

   /********************************************************************
   ** Acknowledge program initialization 
   ** (as requested by /ssc/pipe/dlmtg/sw_dev_guidelines #2)
   */
   if (argc > 1) ack_program_init( argv[0] );
   
   /********************************************************************
   ** Process command line args  
   ** (as requested by /ssc/pipe/dlmtg/sw_dev_guidelines #1)
   */
   cs = process_cmd_args( argc, argv );
   if (cs.debug) {
      printf("BEFORE DEFAULTS HAVE BEEN SET:\n");
      print_args( cs );
   }
    
   /********************************************************************
   ** Set some parameters to defaults, if necessary  
   */
   if (!cs.wts_out_file_f) { 
      strcpy( cs.wts_out_file,    DEFAULT_WTS_OUT_FILE );
      cs.wts_out_file_f         = TRUE;
   }
/* if (!cs.trim_f1_f) {
**    cs.trim_f1                = DEFAULT_TRIM_F1;
**    cs.trim_f1_f              = TRUE;
** }
** if (!cs.trim_f2_f) {      
**    cs.trim_f2                = DEFAULT_TRIM_F2;
**    cs.trim_f2_f              = TRUE;
** }
*/ if (!cs.cut_frac_f) {
      cs.cut_frac               = DEFAULT_CUT_FRAC;
      cs.cut_frac_f             = TRUE;
   }
   if (!cs.cut_multiple_f) {
      cs.cut_multiple           = DEFAULT_CUT_MULTIPLE;
      cs.cut_multiple_f         = TRUE;
   }
   if (!cs.amask_min_good_f) {
      cs.amask_min_good         = DEFAULT_AMASK_MIN_GOOD;
      cs.amask_min_good_f       = TRUE;
   }
   if (!cs.lists_in_nlfile_f) {
      cs.lists_in_nlfile        = FALSE;
      cs.lists_in_nlfile_f      = TRUE;
   }
   if (!cs.header_prop_f) {
      cs.header_prop            = FALSE;
      cs.header_prop_f          = TRUE;
   }
   
   /* Set use_fmask if have fmask file specified. */   /* IS MORE LOGIC NEEDED HERE? */
   use_fmask = cs.fmask_in_file_f; 
   
   /* Set use_weights to follow -wt, otherwise set it to 1 if there is an fmask specified. */
   if (cs.use_weights_f) use_weights = cs.use_weights;
   else                  use_weights = use_fmask;
   cs.use_weights   = use_weights;
   cs.use_weights_f = TRUE;
   
   if (use_weights && !use_fmask) {
      printf("\aERROR:  Cannot set -wt to 1 without specifying an FMASK file with -fm\n");
      exit( TERMINATE_ERROR );
   }
   
/* if ( (cs.trim_f1<0) ||  (cs.trim_f1>1) ) {
**    printf("\aERROR:  Input parameter f1 must be in the range (0-1), but f1=%g\n",cs.trim_f1);
**    exit( TERMINATE_ERROR );
** }
** if ( (cs.trim_f2<0) ||  (cs.trim_f2>1) ) {
**    printf("\aERROR:  Input parameter f2 must be in the range (0-1), but f2=%g\n",cs.trim_f1);
**    exit( TERMINATE_ERROR );
** }
**   
** use_cr_thresh = cs.cr_thresh_f;     /@ For simplicity of notation @/
*/
   
   /********************************************************************
   ** Read in the input data file list names
   */
   if (cs.lists_in_nlfile) {
      strcpy( data_lfile,  cs.namelist );    
      strcpy( sdev_lfile,  cs.namelist );    
      strcpy( bmask_lfile, cs.namelist );    
      strcpy( pmask_lfile, cs.namelist );    
      sdev_in_nl  = TRUE;
      bmask_in_nl = TRUE;
      pmask_in_nl = TRUE;
   }
   else {
      strcpy( data_lfile,  cs.data_in_lfile );    
      strcpy( sdev_lfile,  cs.sdev_in_lfile );    
      strcpy( bmask_lfile, cs.bmask_in_lfile );    
      strcpy( pmask_lfile, cs.pmask_in_lfile );    
      sdev_in_nl  = FALSE;
      bmask_in_nl = FALSE;
      pmask_in_nl = FALSE;
   }
   
   /********************************************************************
   ** Done initializing. 
   */
   if (cs.verbose) {
      printf("AFTER DEFAULTS HAVE BEEN SET:\n");
      print_args( cs );
   }

   /********************************************************************
   ** Some failure conditions -- no input data or no output data spec'd
   */
   if ( (!cs.data_in_lfile_f) && (!cs.lists_in_nlfile) )
   /*then*/ error_and_exit(argv[0],"no input FITS data file list specified (-di)");
   if (!cs.data_out_file_f) 
   /*then*/ error_and_exit(argv[0],"no output FITS data file specified (-do)");
     
   /********************************************************************
   ** Determine cs.num_files, grab sample file
   */
   cs.num_files = count_filenames( data_lfile,  cs.lists_in_nlfile, 
                                   DATA_MARKER, END_MARKER, firstfile   );

   if (cs.debug) printf( "Counted %0d files; First filename = \"%s\"\n", 
                                  cs.num_files,                firstfile  );
   if (cs.num_files==0) {
      printf("\aERROR:  No data filenames found in : \"%s\"", 
              cs.lists_in_nlfile ? cs.namelist : cs.data_in_lfile );
      printf("Failure after routine count_filenames(), in coadd2d.c\n");
      exit( TERMINATE_ERROR );
   }
   
   /* TEMPORARY CODE used for CREATING TEST DATA ONLY 
   ** insert_int_keyword( cs.fmask_in_file, "FOVID",     26 );
   ** insert_int_keyword( cs.fmask_in_file, "channelID",  0 );
   **
   ** insert_int_keyword( cs.fmask_in_file, "FOVID",     42 );
   ** insert_int_keyword( cs.fmask_in_file, "USEFOVID",  26 );
   **
   ** insert_int_keyword( cs.fmask_in_file, "USEFOVID",   0 ); 
   */ 
   
   /********************************************************************
   ** Determine the X and Y dimensions of the data (cs.xdim and cs.ydim)
   ** using the first data file as the example that all must follow
   */
   read_fits_dims_2d( firstfile, &(cs.xdim), &(cs.ydim), cs.debug );

   /********************************************************************
   ** Create lists of all the filenames -- 
   */
   data_files     = (files_struct *) malloc( (unsigned) ( cs.num_files * sizeof(files_struct) ) );
   if (data_files == NULL) malloc_failed( "data_files" );
   read_filenames(    data_lfile,   cs.lists_in_nlfile, data_files,
                      cs.num_files, DATA_MARKER,      END_MARKER      );
   if (cs.lists_in_nlfile) find_markers( cs.namelist, &sdev_in_nl, &bmask_in_nl, &pmask_in_nl );
   use_sdev  = (   cs.sdev_in_lfile_f  || sdev_in_nl  );
   use_bmask = ( ( cs.bmask_in_lfile_f || bmask_in_nl ) && cs.bfatal_f ); /* have list of b/pmask files */
   use_pmask = ( ( cs.pmask_in_lfile_f || pmask_in_nl ) && cs.pfatal_f ); /*   AND the b/pfatal bits    */
   if (cs.debug) printf("use_bmask=%1d  use_pmask=%1d\n", use_bmask, use_pmask );
   if (use_sdev)  { 
      sdev_files  = (files_struct *) malloc( (unsigned) ( cs.num_files * sizeof(files_struct) ) );
      if (sdev_files == NULL) malloc_failed( "sdev_files" );
      read_filenames( sdev_lfile,   cs.lists_in_nlfile, sdev_files,  
                      cs.num_files, SDEV_MARKER,      END_MARKER      );
   }                                           
   if (use_bmask)  { 
      bmask_files = (files_struct *) malloc( (unsigned) ( cs.num_files * sizeof(files_struct) ) );
      if (bmask_files == NULL) malloc_failed( "bmask_files" );
      read_filenames( bmask_lfile,  cs.lists_in_nlfile, bmask_files, 
                      cs.num_files, BMASK_MARKER,     END_MARKER      );
   }
   if (use_pmask)  { 
      pmask_files = (files_struct *) malloc( (unsigned) ( cs.num_files * sizeof(files_struct) ) );
      if (pmask_files == NULL) malloc_failed( "pmask_files" );
      read_filenames( pmask_lfile,  cs.lists_in_nlfile, pmask_files, 
                      cs.num_files, PMASK_MARKER,     END_MARKER      );
   }
   
   if (cs.verbose) {
      printf("\n%18s %18s %18s %18s\n", "DATA ", "SDEV ", "BMASK ", "PMASK ");
      for (i=0; i<cs.num_files; i++) {
         printf("%18s %18s %18s %18s\n", data_files[i].v,
                          ( use_sdev  ?  sdev_files[i].v : "--- " ),
                          ( use_bmask ? bmask_files[i].v : "--- " ),
                          ( use_pmask ? pmask_files[i].v : "--- " )  );
      }
      printf("\n");
   }


   /********************************************************************
   ** Read the data from all the files in lists, put it in a big 3D array
   **    and ensure that all planes have identical dimensions of data
   */
   data_planes = (image_struct *) malloc( (unsigned) ( cs.num_files * sizeof(image_struct) ) );
   if (data_planes == NULL) malloc_failed( "data_planes" );
   if (use_sdev) {
      sdev_planes = (image_struct *) malloc( (unsigned) ( cs.num_files * sizeof(image_struct) ) );
      if (sdev_planes == NULL) malloc_failed( "sdev_planes" );
   }
   if (use_bmask) {
      bmask_planes = (simage_struct *) malloc( (unsigned) ( cs.num_files * sizeof(simage_struct) ) );
      if (bmask_planes == NULL) malloc_failed( "bmask_planes" );
   }
   if (use_pmask) {
      pmask_planes = (simage_struct *) malloc( (unsigned) ( cs.num_files * sizeof(simage_struct) ) );
      if (pmask_planes == NULL) malloc_failed( "pmask_planes" );
   }
   if (cs.debug) printf("Allocated data arrays.\n");
   
   if (use_sdev)  { 
      printf("\n");
      printf("NOTE: The sdev files are not used when calculating flux.\n");
      printf("      They are only only used to calculate unc (unweighted, untrimmed).\n");
   }
   
   req_fov_num = -1;  /* This parameter will hold the FOVID from the first data file.  All subsequent */
                      /* FITS data files must then contain the identical FOVID in their headers.      */
                      /* Initialize it to -1 to indicate that it hasn't been set yet.  The first call */
                      /* to read_fits() will set it.                                                  */
   
   for (i=0; i<cs.num_files; i++ ) {
      data_planes[i].p = read_fits( data_files[i].v, &ith_xdim, &ith_ydim, &req_fov_num, cs.debug );
      if ( (ith_xdim!=cs.xdim) || (ith_ydim!=cs.ydim) ) {
         printf("\aERROR in input data planes, in program %s\n", cs.progname );
         printf("\aERROR:  Data plane number %0d is %0dx%0d but should be %0dx%0d\n",
                i, ith_xdim, ith_ydim, cs.xdim, cs.ydim );
         exit(TERMINATE_ERROR);
      }
      if (use_sdev)  {
         req_fov_dummy = -9; /* do not try to read FOVID in file */
         sdev_planes[i].p = read_fits( sdev_files[i].v, &ith_xdim, &ith_ydim, &req_fov_dummy, cs.debug );
         if ( (ith_xdim!=cs.xdim) || (ith_ydim!=cs.ydim) ) {
            printf("\aERROR in input sdev planes, in program %s\n", cs.progname );
            printf("\aERROR:  Sdev plane number %0d is %0dx%0d but should be %0dx%0d\n",
                   i, ith_xdim, ith_ydim, cs.xdim, cs.ydim );
            exit(TERMINATE_ERROR);
         }
      }
      if (use_bmask)  {
         bmask_planes[i].p = read_fits_s( bmask_files[i].v, &ith_xdim, &ith_ydim, cs.debug );
         if ( (ith_xdim!=cs.xdim) || (ith_ydim!=cs.ydim) ) {
            printf("\aERROR in input BMASK planes, in program %s\n", cs.progname );
            printf("\aERROR:  BMASK plane number %0d is %0dx%0d but should be %0dx%0d\n",
                   i, ith_xdim, ith_ydim, cs.xdim, cs.ydim );
            exit(TERMINATE_ERROR);
         }
      }
      if (use_pmask)  {
         pmask_planes[i].p = read_fits_s( pmask_files[i].v, &ith_xdim, &ith_ydim, cs.debug );
         if ( (ith_xdim!=cs.xdim) || (ith_ydim!=cs.ydim) ) {
            printf("\aERROR in input PMASK planes, in program %s\n", cs.progname );
            printf("\aERROR:  PMASK plane number %0d is %0dx%0d but should be %0dx%0d\n",
                   i, ith_xdim, ith_ydim, cs.xdim, cs.ydim );
            exit(TERMINATE_ERROR);
         }
      }
   }
   
   
   /*****************************************************************************************
   ** Set cs.fov_for_wts
   **
   ** I use cs.fov_for_weights to store the fov number "to define good pixels for weighting".
   */
   cs.fov_for_wts = -1;  /* Default is to not do weighting */
   if (use_weights) {
      if (req_fov_num > 0)
         cs.fov_for_wts = req_fov_num;  /* cs.fov_num is FOVID from first data file */
      else { 
         printf("ERROR: FOVID wasn't found and stored in req_fov_num -- this shouldn't happen!\n" );
         exit(TERMINATE_ERROR);
      }
   }
   else if (cs.debug) printf("cs.fov_for_wts NOT SET since it's not needed.\n");
   
   if (cs.verbose && (cs.fov_for_wts>0) && use_weights ) {
      printf("The selected FOV NUMBER = %5d will be used for DCE weighting\n",cs.fov_for_wts);
   }
   if (cs.debug) {
      printf("(DBG) The selected FOV NUMBER = %5d will be used for DCE weighting\n",cs.fov_for_wts);
      if (cs.fov_for_wts<=0) printf("  (-1 ==> no weighting)\n");
      else                   printf("\n");
   }

   /********************************************************************
   ** Read the FOVLIST and BITLIST from the Fmask FITS file
   ** Read the DATA from the Fmask FITS file 
   ** Also set cs.bit_for_wts from cs.fov_for_wts
   **
   ** FROM THE SIS defining the FMASK lists:
   **
   **    The F-MASK identifies pixels where spectra in each field of view are expected
   **    to fall.  The pixels are marked bit-wise so that all FOV's in a module can be
   **    identified in a single mask.  The header contains two keywords that explain the
   **    mask:
   **
   **             FOVLIST -- the list of FOVs that are identified in the mask
   **             BITLIST -- the bits that are set for pixels in each FOV.
   **
   **    For example, in SL
   **
   **             FOVLIST = '[26,27,28,29,32,33,34]'
   **             BITLIST = '[1,2,4,8,16,32,64]'
   **
   **    so any pixel in the mask with the 1 it set is in FOV 26, and any pixel with the
   **    8 bit set is in FOV 29.  In practice, FOV 28 and 29 overlap entirely so many
   **    pixels have both 4 and 8 set, that is a value of 12.
   */
   if (use_fmask) {   /* If we have cs.fmask_in_file_f */
   
      /* Read and digest the FOVLIST and BITLIST from the FMASK file */
      read_fmask_lists( &cs );         

      /* Set cs.bit_for_wts (leave as -1 if not found) */
      cs.bit_for_wts = -1;
      if ( (cs.num_fovs>0) && (cs.fov_for_wts>0) ) {
         for( i=0; i<cs.num_fovs; i++ ) {
            if (cs.fov_for_wts==cs.fovlist[i]) {
               cs.bit_for_wts = cs.bitlist[i];
               break;
            }
         }
         if (cs.bit_for_wts==-1) {
            printf("ERROR:  FOVID=%1d not found in FOVLIST from FMASK.\n", cs.fov_for_wts);
            exit(TERMINATE_ERROR);
         }
         if (cs.verbose) {
            printf("Selecting FOV=%1d (BIT=%1d) for weighting\n",cs.fov_for_wts, cs.bit_for_wts);
         }
      }
      if (cs.debug) printf("cs.num_fovs = %1d  cs.fov_for_wts = %1d  cs.bit_for_wts = %1d\n",
                            cs.num_fovs,       cs.fov_for_wts,       cs.bit_for_wts           );
            
      /* Read the DATA from the Fmask FITS file */
      fmask_plane = (short int *) malloc( (unsigned) (cs.xdim*cs.ydim*sizeof(short int)) );
      if (fmask_plane == NULL) malloc_failed( "fmask_plane" );
      fmask_plane = read_fits_s( cs.fmask_in_file, &ith_xdim, &ith_ydim, cs.debug );
      if ( (ith_xdim!=cs.xdim) || (ith_ydim!=cs.ydim) ) {
         printf("\aERROR in input FMASK, in program %s\n", cs.progname );
         printf("\aERROR:  FMASK dims are %0dx%0d but should be %0dx%0d\n",
                ith_xdim, ith_ydim, cs.xdim, cs.ydim );
         exit(TERMINATE_ERROR);
      }

   }
   
   
   /******************************************************************************************
   ** Create several storage arrays
   */
   data_column    = (data_struct *)  malloc( (unsigned)(cs.num_files*sizeof(data_struct)) );
   if (data_column    == NULL) malloc_failed( "data_column" );
   dce_weight     = (float *)  malloc( (unsigned)(cs.num_files*sizeof(float)) );
   if (dce_weight     == NULL) malloc_failed( "dce_weight" );
   dce_num_good   = (int *)    malloc( (unsigned)(cs.num_files*sizeof(int)) );
   if (dce_num_good   == NULL) malloc_failed( "dce_num_good" );
      
   /******************************************************************************************
   ** Set pixels to NaN where bmask and pmask mark bad pixels
   **
   ** Examine all images (data_planes[].p[]), set all "BAD DATA" to NaN and calculate the 
   ** dce_num_good (number of "GOOD DATA") and "preliminary" dce_weight for each image. 
   ** (the preliminary dce_weight is for diagnostic purposes only);
   */
   for (i=0; i<cs.num_files; i++ ) { 
   
      /* Initialize sums */
      /* dce_weight[i] = 0; */
      dce_num_good[i] = 0;  
      /* if (DBG) printf("===== i=%1d =====\n",i); */
      
      for    ( y=0; y<(cs.ydim); y++ )  {
         for ( x=0; x<(cs.xdim); x++ )  {
            xy  = ( y * cs.xdim ) + x;

            /* Is the pixel already a NaN? */
            if ( iznand( (double) data_planes[i].p[xy] ) ) continue;

            /* Are any of the bmask or pmask bits the same as the appropriate fatal bits? */
            /* Note that: ( bmask_pixel & bfatal )==0 IF no corresponding bits are both 1s  */
            /* That is:   ( bmask_pixel & bfatal )==0 IF the pixel should not be masked out */
            /* That is:   ( bmask_pixel & bfatal )==0 means "GOOD DATA"                     */
            if (use_bmask) {
               if ( bmask_planes[i].p[xy] & cs.bfatal ) {  /* bmask identifies "BAD DATA" */
                  /* if (DBG) printf("bmask[%1d,%1d]=%2d, cs.bfatal=%2d\n", x,y,bmask_planes[i].p[xy],cs.bfatal); */
                  data_planes[i].p[xy] = flt_NaN;
                  continue;
               }
            }
            if (use_pmask) {
               if ( pmask_planes[i].p[xy] & cs.pfatal ) {  /* pmask identifies "BAD DATA" */
                  /* if (DBG) printf("pmask[%1d,%1d]=%2d, cs.pfatal=%2d\n", x,y,pmask_planes[i].p[xy],cs.pfatal); */
                  data_planes[i].p[xy] = flt_NaN;
                  continue;
               }
            }
             
/*          /@ Does the pixel exceed the Cosmic Ray Threshold? @/
**          if (use_cr_thresh) {
**             if ( data_planes[i].p[xy] > cs.cr_thresh ) {  /@ pixel may contain a cosmic ray @/
**                /@ if (DBG) printf(" data[%1d,%1d]=%2g> cs.cr_thresh=%g\n", x,y,data_planes[i].p[xy],cs.cr_thresh); @/
**                data_planes[i].p[xy] = flt_NaN;
**                continue;
**             }
**          }
*/                 
            /* Is the pixel part of the specified FOV? */
            if (use_weights) {
               /* USED TO BE: if ( fmask_plane[xy] != cs.fov_for_wts )  /@ fmask identifies WRONG FOV @/  */
               if ( !(fmask_plane[xy] & cs.bit_for_wts) ) {  /* fmask identifies WRONG FOV */
                  /* DON'T NEED TO MARK IT AS BAD DATA, but is should not be used in calculating counting good points  */
                  /* if (DBG) printf("fmask[%1d,%1d]=%2d, cs.fov_num=%2d\n", x,y,fmask_plane[xy],cs.fov_num); */
                  /* PREVIOUS VERSION:  data_planes[i].p[xy] = flt_NaN; */ 
                  continue;
               }
            }
                  
            /* MUST Have "GOOD DATA" to get to these lines of code */
            dce_num_good[i]++;
            /* if (use_weights) dce_weight[i] += data_planes[i].p[xy];
            ** else             dce_weight[i]  = 1;
            */
               
            
         } /* for x */
      }    /* for y */

      if (cs.verbose) {
         if (i==0) { 
            printf("\nAfter b/pmask, BEFORE trimming"  );
            if (use_weights) printf(", in FOV %2d ONLY", cs.fov_for_wts );
            printf(":\n");
         }
         printf("   In DCE %3d, Number of good pixels is %5d => Number of pixels masked ", i, dce_num_good[i] );
         if (use_weights) printf("or in wrong FOV ");
         printf("is %5d \n", cs.xdim*cs.ydim-dce_num_good[i] );      
      }

   } /* for i */
   /* if (cs.debug && use_weights) printf("Preliminary DCE weights calculated.\n"); */
   

   /******************************************************************************************
   ** Extract single-pixel-wide "columns" from the data, and perform
   ** asymmetric trimming on them (current code is VERSION 3).
   **
   ** Be sure to *DELETE* (mark and skip over) "BAD DATA", which includes 
   ** any NaNs, any bmasked or pmasked data.  Remaining data is "GOOD DATA (for now)"
   */
             /* VERSION 1:
             ** Used to use Russ' routine: trimmedstatistics().
             ** trimfactflag = 0;  FOR RUSS' ROUTINE
             **
             ** Also used to *DELETE* any pixels over cr_thresh,
             */
             /* VERSION 2:
             ** Used to use my own trimmed_weighted_average() routine
             */

   for    ( y=0; y<(cs.ydim); y++ )  {
      for ( x=0; x<(cs.xdim); x++ )  {
         xy  = ( y * cs.xdim ) + x;
         
         /* Create the "data column" at pixel position (x,y), skipping all "BAD DATA" */
         /* (column is ~perpendicular to the image planes)                            */
         n_good = 0;       /* n_good counts only "GOOD DATA"  */
         for (i=0; i<cs.num_files; i++ ) {  
         
            /* Is the pixel a NaN?  (all "BAD DATA" are already marked with NaNs) */
            if ( iznand( (double) data_planes[i].p[xy] ) ) continue;

            data_column[n_good].value = data_planes[i].p[xy];
            data_column[n_good].plane = i;
            n_good++;
            
         }
         
         /******************************************************************/
         /*** Execute my newer specialer routine, asymmetric_trimming()  ***/
         /*** (see appendix at bottom of this code for previous versions ***/
         /******************************************************************/
         #if DEBUG_ASYMMETRIC_TRIMMING
            printf("#### BEFORE asymmetric_trimming(), n_good = %d\n",n_good);
         #endif
         
         if (cs.cut_frac>0) {
         
            #if DEBUG_ASYMMETRIC_TRIMMING
               if (cs.debug) printf("#### Doing asymmetric trimming\n");
            #endif
            
            asymmetric_trimming( data_column, &n_good, cs );  /* Trims data_column and updates n_good */
            
         }
         
         #if DEBUG_ASYMMETRIC_TRIMMING
            else if (cs.debug) printf("Skipping asymmetric trimming since cut_frac = %g <= 0 .\n",cs.cut_frac);
         #endif
         
         #if DEBUG_ASYMMETRIC_TRIMMING
            printf("#### AFTER  asymmetric_trimming(), n_good = %d\n",n_good);
         #endif
         
         /* Put those trimed data values back into the data_planes array (necessary to calculate weights) */
         for (i=0; i<cs.num_files; i++ ) data_planes[i].p[xy] = flt_NaN;
         for (i=0; i<n_good; i++ ) data_planes[ data_column[i].plane ].p[xy] = data_column[i].value;
         
      }
   }   
   
   /******************************************************************************************
   ** Calculate the weights, image by image, then normalize them.
   */
   if (use_weights) {
   
      max_weight = -1e-42;
      
      for (i=0; i<cs.num_files; i++ ) { 

         dce_weight[i]   = 0;  /* Initialize sums */
         dce_num_good[i] = 0;  
         /* if (DBG) printf("===== i=%1d =====\n",i); */

         for    ( y=0; y<(cs.ydim); y++ )  {
            for ( x=0; x<(cs.xdim); x++ )  {
               xy  = ( y * cs.xdim ) + x;

               /* Is the pixel already a NaN? */
               if ( iznand( (double) data_planes[i].p[xy] ) ) continue;
               
               /* Is the pixel part of the specified FOV? */
                  /* if (use_fmask)  (not necessary, since use_weights ==> use_fmask) */
               /* USED TO BE: if ( fmask_plane[xy] != cs.fov_for_wts ) {  /@ fmask identifies WRONG FOV @/  */
               if ( !(fmask_plane[xy] & cs.bit_for_wts) ) {  /* fmask identifies WRONG FOV */
                  /* DON'T NEED TO MARK IT AS BAD DATA, but is should not be used in calculating the weight  */
                  /* if (DBG) printf("fmask[%1d,%1d]=%2d, cs.fov_num=%2d\n", x,y,fmask_plane[xy],cs.fov_num); */
                  /* PREVIOUS VERSION:  data_planes[i].p[xy] = flt_NaN; */ 
                  continue;
               }
                  
               dce_num_good[i]++;
               dce_weight[i] += data_planes[i].p[xy];

            } /* for x */
         }    /* for y */
         
         max_weight =  MAX(dce_weight[i],max_weight);
         
      }       /* for i */

      /* Normalize the weights to 1, if the weights are nonzero and positive */
      if      (max_weight==0) printf("\n\aCAUTION:  WEIRD DATA RESULT:  ALL WEIGHTS ARE ZERO!\n\n");
      else if (max_weight<0)  printf("\n\aCAUTION:  WEIRD DATA RESULT:  MAX WEIGHT IS NEGATIVE!\n\n");
      else {
         for (i=0; i<cs.num_files; i++ ) dce_weight[i] /= max_weight; /* Normalize the weights to 1 */
      }
               
   }
   else  {  /* (!use_weights) */
      for (i=0; i<cs.num_files; i++ ) dce_weight[i] = 1;
   }
   
   if (cs.verbose) {
      if (cs.cut_frac > 0) {
         printf("AFTER asymmetric trimming"  );
         if (use_weights) printf(", AFTER calculating weights, in FOV %2d ONLY", cs.fov_for_wts );
         else             printf(", AFTER setting all weights to ONE" );
         printf(":\n");
         for (i=0; i<cs.num_files; i++ ) {
            printf("   In DCE %3d, good pixels = %5d => pixels masked, rejected", i, dce_num_good[i] );
            if (use_weights) printf(", or in wrong FOV");
            printf(" = %5d, ", cs.xdim*cs.ydim-dce_num_good[i] );      
            printf("WEIGHT = %7.5f\n", dce_weight[i] );
         }         
         printf("\n");
      }   
      else {
         printf("NO asymmetric trimming was performed, as requested by cut_frac (-cf) set to ZERO.\n\n");                  
         printf("AFTER calculating weights:\n"  );
         for (i=0; i<cs.num_files; i++ ) {
            if (i==0) { 
            }
            printf("   In DCE %3d, good pixels = %5d => pixels masked, rejected", i, dce_num_good[i] );
            printf(" = %5d, ", cs.xdim*cs.ydim-dce_num_good[i] );      
            printf("WEIGHT = %7.5f\n", dce_weight[i] );
         }         
         printf("\n");
      }
   }
   

   /********************************************************************
   ** Create several storage arrays
   */
   means = (float *)     malloc( (unsigned) ( (cs.xdim)*(cs.ydim) * sizeof(float) ) );
   if (means  == NULL)   malloc_failed( "means" );
   sdevs = (float *)     malloc( (unsigned) ( (cs.xdim)*(cs.ydim) * sizeof(float) ) );
   if (sdevs  == NULL)   malloc_failed( "sdevs" );
   amask = (short int *) malloc( (unsigned) ( (cs.xdim)*(cs.ydim) * sizeof(short int) ) );
   if (amask  == NULL)   malloc_failed( "amask" );
   

   /******************************************************************************************
   ** Extract single-pixel-wide "columns" from the data, WITH associated weights, and
   ** calculate weighted mean and sdev for them.
   **
   ** As usual, Be sure to *DELETE* (mark and skip over) "BAD DATA", which includes 
   ** any NaNs, any bmasked or pmasked data, and newly trimmed data.  
   */

   for    ( y=0; y<(cs.ydim); y++ )  {
      for ( x=0; x<(cs.xdim); x++ )  {
         sum_sdev_sqd = 0;
         xy  = ( y * cs.xdim ) + x;
         
         /* Create the "data column" at pixel position (x,y), skipping all "BAD DATA" */
         /* (column is ~perpendicular to the image planes)                            */
         bmask_or_all = 0;       /* logical OR of BMASKs in al planes */
         n_good = 0;       /* n_good counts only "GOOD DATA"  */
         for (i=0; i<cs.num_files; i++ ) {  
         
            /* Is the pixel a NaN?  (all "BAD DATA" and Trimmed Data are already marked with NaNs) */
            if ( iznand( (double) data_planes[i].p[xy] ) ) continue;

            data_column[n_good].value  = data_planes[i].p[xy];
/*          data_column[n_good].plane  = i;  /*                 /* not necessary */
            data_column[n_good].weight = dce_weight[i];
            sum_sdev_sqd += (sdev_planes[i].p[xy] * sdev_planes[i].p[xy]);
            bmask_or_all |= bmask_planes[i].p[xy];
            n_good++;
            
         }

         
         /*******************************************************************/
         /*** Calculate the weighted stats, now that all info is prepared ***/
         /*******************************************************************/
         if (n_good>0) {        
            calc_weighted_stats( data_column, 0,  n_good, &wtd_mean, &wtd_sdev, cs.debug );
            means[xy] = wtd_mean;
            sdevs[xy] = sqrt(sum_sdev_sqd) / n_good;
            amask[xy] = ( (n_good<cs.amask_min_good) ? AM_STATS_POOR : 0 ); 
            amask[xy] |= (bmask_or_all & cs.bpass); /* passthru selected bits bmask->amask */
         }
         else /* (n_good==0) */ {
            means[xy] = flt_NaN;
            sdevs[xy] = flt_NaN;
            amask[xy] = ( AM_STATS_POOR | AM_STATS_BAD );            
         }
         /* if (DBG) printf("    n_good=%0d x=%0d y=%0d   mean=%7.2f  sdev=%7.2f  amask=%03d\n", 
         **                      n_good,    x,    y,      means[xy],  sdevs[xy],  amask[xy]    );
         */
         
      }
   }   

   
   /********************************************************************
   ** Output data
   */
   if (!cs.header_prop) {
      write_flt_fits( cs.data_out_file, means, cs );
      if (cs.verbose) printf("Output    data file written: %s \n", cs.data_out_file );
   }
   else { 
      write_flt_fits_with_hdrs( data_files[0].v, cs.data_out_file, means, cs );
      if (cs.verbose) printf("Output    data file written: %s ", cs.data_out_file );
      if (cs.verbose) printf("(with headers copied from %s)\n", firstfile );
   }
   if (cs.sdev_out_file_f) {
      write_flt_fits( cs.sdev_out_file, sdevs, cs );
      if (cs.verbose) printf("Output    sdev file written: %s \n", cs.sdev_out_file );
   }
   if (cs.amask_out_file_f) {
      write_amask( amask, cs );
      if (cs.verbose) printf("Output   amask file written: %s \n", cs.amask_out_file );
   }
   write_weights_list( dce_weight, data_files, cs );
   if (cs.verbose)    printf("Output weights list written: %s \n", cs.wts_out_file );
   
   /* VERIFY OUTPUT DATA */
   /* if (DBG) print_fits_data( cs.data_out_file );
   ** if (DBG) if (cs.sdev_out_file_f)  print_fits_data(   cs.sdev_out_file );
   ** if (DBG) if (cs.amask_out_file_f) print_fits_data_s( cs.amask_out_file );
   */
   
   /* COMPARE FITS RESULTS FOR TESTING */
   /* if (DBG) compare_fits_data("test_out_100first.fits","test_out_100second.fits"); */
   
   /********************************************************************
   ** Clean up and Acknowledge program completion 
   ** (as requested by /ssc/pipe/dlmtg/sw_dev_guidelines #5)
   */
   free( data_column );
   free( data_planes );
   free( dce_weight );
   if (use_sdev)  free( sdev_planes );
   if (use_bmask) free( bmask_planes );  
   if (use_pmask) free( pmask_planes );  
   free( means );    
   free( sdevs );    
   free( amask );  
   free( data_files );
   if (use_sdev)  free( sdev_files );
   if (use_bmask) free( bmask_files );
   if (use_pmask) free( pmask_files );
   ack_program_complete( cs.progname );
   if (cs.verbose) fprintf(stderr, "\n   <<<<< END OF %s PROGRAM >>>>> \n\n", argv[0] );
   return( TERMINATE_NORMAL );

}


/******************************************************************************
*******************************************************************************
**  ORIGINAL EMAIL FROM FAN THAT I USED TO DESIGN THIS CODE

    Date: Thu, 5 Sep 2002 18:44:24 -0700 (PDT)
    From: Fan Fang <fan@ipac.caltech.edu>
    To: Clare Waterson <clare@columbus.ipac.caltech.edu>
    Subject: 2d coadd

    Hi Clare,

    Here is what I think the module should do:

    1.  It shall be able to read lists of FITS files.
    Lists may include data list, uncertainty list, and
    bmask file list.  It can be assumed that corresponding
    files are in sequence in multiple lists.

    2.  It can assume that data in the same list have the
    same dimension.  The code can otherwise check the
    consistency, or leave it open to accept a good data
    index on which the product dimension and header depend.
    Since the module is used to coadd 2-d frames I don't
    really care one way or the other.  You design it.

    3.  It shall support a namelist function.

    4.  It shall support fatal templates for mask files.
    Relevant mask files for this module are the bmask and
    possibly pmask.

    5.  It shall check mask pixel values against fatal
    templates and qualify/disqualify a pixel accordingly.
    It shall exclude NaNs in the calculation, output NaNs
    if pixel is NaN in all files or computation cannot
    proceed.

    6.  It shall support standard statistics on a pixel
    to generate the output value.  This may include mean,
    median, stddev, etc.  The dnlstats library and the
    trimmedstatistics() subroutine is recommended.

    7.  An output uncertainty file is to be generated.
    Currently the uncertainty may just include the one
    from statistics for a pixel.  Reading input uncertainties
    therefore becomes a place holder.

    8.  You may define an output mask file to record pixels
    that cannot be computed or has questionable computation.
    A sis is to be defined for it.  It may not be the existing
    b, d, c, and other masks (an amask?).

    That's all I can think of.  The sample code is darkest
    located in /ssc/pipe/fan/irs/darkest/tst for the current
    version.  It may have more functions than you need to
    implement.  Let me know when you encounter further problems.

    -Fan
**
*/


/******************************************************************************
*******************************************************************************
**  NOTES ON USING trimmedstatistics()
**  From email and discussions with Russ Laher
**

trimmedstatistics(double y[], long nstart, long n, double f1, 
                  double f2, int *trimfactflag, double *facttotptr,
                  double *mean, double *stddev, double *datascale) {

    Inputs:
    y[]           Input array
    nstart        Starting sample in the input array (counting from one)
    n             Number of input value
    f1            Central fraction for trimmed average
    f2            Central fraction for trimmed sigma
    trimfactflag  Initialization flag - set to zero before calling
                  trimmedstatistics() each time!  (An undesirable
                  artifact of Stu's library implementation?)

    Outputs:
    mean          Trimmed mean based on f1 (f1=0 for median,
                     f1=1 for no trimming)
    stddev        Trimmed standard deviation based on f2
    datascale     Data-scale
    
MY NOTES:

1)  Ignore facttotptr -- leave it alone

2)  Set trimfactflag to 0 once (before first call to trimmedstatistics),
since it needs to do a tricky calculation based on f2.  Then leave it
alone on all subsequent calls that contain the same f2.  It calculates
facttotptr, so also leave that alone both before and after it's set.

3)  nstart tells what the start sample in the array is, counting from 1.  So,
set nstart=1 to calculate statistics on an entire array

4)  n is the number of values in the array.  

5)  f1 is the central fraction of the data that is *kept*.  I.e. if f1
is 0.9 then the central 90% of the data will be kept and 5% will be\
trimmed off each end.

6)  This routine DOES NOT HANDLE NaNs, so they must be filtered out from
the data before the routine is called.

7)  The array y[] is one-dimensional.

8)  I still don't know what "data-scale" is, but I'm told that I can 
safely ignore it.

9)  Mehrdad says a central fraction of 90% makes a good default.

*******************************************************************************
*******************************************************************************
*/


#if 0
/*@@@@@@@@@@@@@@@@@@ MY ORIGINAL TEST CODE for trimmedstatistics() @@@@@@@@@@@@@@*/
double f1,f2;
int    numdata;
double data[1000];

   printf("@@@ TESTING @@@\n");
   
   
   trimfactflag = 0;
   numdata = 9;
   for (i=0;i<9;i++) {
      data[i] = i+1;
      printf("   data[%0d] = %6.2f",i,data[i]);
      if (i%3==2) printf("\n");
   }
   printf("\n");

   printf("trimfactflag outside loop.\n");
   for (j=0;j<11;j++) {
      f1 = (double)j/10.; 
      f2 = 0.5;
      trimmedstatistics( data, 1, 9, f1, f2, 
                         &trimfactflag, &facttotptr, &mean, &sdev, &datascale);
      printf("f1=%4.1f  f2=%4.1f  mean=%6.1f  sdev=%6.1f  facttotptr=%g\n",f1,f2,mean,sdev,facttotptr ); 
   }
   printf("\n");
   
   numdata = 9;
   for (i=0;i<9;i++) {
      if (i<5) data[i] = 5;
      else     data[i] = i+1;
      printf("   data[%0d] = %6.2f",i,data[i]);
      if (i%3==2) printf("\n");
   }
   printf("\n");
   
   printf("trimfactflag outside loop.\n");
   trimfactflag = 0;
   for (j=0;j<11;j++) {
      f1 = (double)j/10.; 
      f2 = 0.5;
      trimmedstatistics( data, 1, 9, f1, f2, 
                         &trimfactflag, &facttotptr, &mean, &sdev, &datascale);
      printf("f1=%4.1f  f2=%4.1f  mean=%6.1f  sdev=%6.1f  facttotptr=%g\n",f1,f2,mean,sdev,facttotptr ); 
   }
   printf("\n");
   
   printf("trimfactflag inside loop.\n");
   for (j=0;j<11;j++) {
      f1 = (double)j/10.; 
      f2 = 0.5;
      trimfactflag = 0;
      trimmedstatistics( data, 1, 9, f1, f2, 
                         &trimfactflag, &facttotptr, &mean, &sdev, &datascale);
      printf("f1=%4.1f  f2=%4.1f  mean=%6.1f  sdev=%6.1f  facttotptr=%g\n",f1,f2,mean,sdev,facttotptr ); 
   }
   printf("\n");
   
#endif

#if 0
/*#################### Here are some previous versions of my trimming algorithms ################*/

         /***************************************************************/
         /*** Execute my special routine, trimmed_weighted_average()  ***/
         /***************************************************************/         
         /* printf("===> f1=%g\n", cs.trim_f1); */
         
         /* TEMPORARY CODE FOR TEMPORARY OUTPUT */
         /*trimmed_weighted_average( data_column, n_good, 0.9, 0.9,
         **     /* Return values: */  &mean, &sdev, cs.debug                      );
         */
         
         /*trimmed_weighted_average( data_column, n_good, cs.trim_f1, cs.trim_f2, 
         **    /* Return values: */  &mean, &sdev, cs.debug                      );
         */  
	                
         /* SOME DEBUGGING OUTPUT */
         /* if (n_good>0) {
         **    printf("==> x=%03d y=%03d | ", x, y );
         **    for( j=0; j<n_good; j++ ) printf("d[%1d]=%7.4f ",j,data_column[j].data);
         **    printf(" | ");
         **    for( j=0; j<n_good; j++ ) printf("w[%1d]=%7.4f ",j,data_column[j].weight);
         **    printf(" | mean = %g \n",mean);
         ** }
         */

         
         /***************************************************************/
         /*** FOR NOTES ON trimmedstatistics(), SEE MY EXTRA COMMENTS ***/
         /*** SECTION INCLUDED AT THE BOTTOM OF THIS CODE MODULE      ***/
         /***************************************************************/
         /* 
         ** trimmedstatistics( data_column, 1, n_good, cs.trim_f1, cs.trim_f2, 
         **                    &trimfactflag, &facttotptr, &mean, &sdev, &datascale);
         */                   
                            
         if (n_good>0) {        
            means[xy] = mean;
            sdevs[xy] = sdev;
            amask[xy] = ( (n_good<cs.amask_min_good) ? AM_STATS_POOR : 0 ); 
         }
         else /* (n_good==0) */ {
            means[xy] = flt_NaN;
            sdevs[xy] = flt_NaN;
            amask[xy] = ( AM_STATS_POOR | AM_STATS_BAD );            
         }
         /* if (DBG) printf("    n_good=%0d x=%0d y=%0d   mean=%7.2f  sdev=%7.2f  amask=%03d\n", 
         **                      n_good,    x,    y,      means[xy],  sdevs[xy],  amask[xy]    );
         */

#endif
