/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@
@@ PROGRAM:  coadd2d.h 
@@
@@ PURPOSE:  Shared header file for coadd2d calculations
@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

#ifndef _COADD2D_H_
#define _COADD2D_H_


/*================================= CODE VERSION =================================*/
/*#define COADD2D_VERSION  6.30    Initial version for S6.3                       */
/*#define COADD2D_VERSION  6.31    Fixed minor bugs: bmask, help, headers         */
/*#define COADD2D_VERSION  6.32    Added -h1 Header Propagation, add version      */
                                /* number to Usage statement                      */
/*#define COADD2D_VERSION  7.50 /* Updated per "IRS COADD2D Module Requirements"  */
                                /* by Harry Teplitz, 2/10/2003                    */
/*#define COADD2D_VERSION  7.51 /* Bugfixes requested by Harry Teplitz, 5/15/2003 */
/*#define COADD2D_VERSION  7.52 /* Some more Bugfixes  (interrim version)         */
/*#define COADD2D_VERSION  7.53 /* Still more Bugfixes (interrim version)         */
/*#define COADD2D_VERSION  7.54 /* Upgrade to "Asymmetric Trimming"               */
/*#define COADD2D_VERSION  7.55 /* An interrim version for testing                */
/*#define COADD2D_VERSION  7.56 /* An interrim version for more testing           */
/*#define COADD2D_VERSION  8.00 /* Tested version, for S8.0, incorporating CR4439 */
/*#define COADD2D_VERSION  8.01 /* Fixed typo in usage statement                  */
/*#define COADD2D_VERSION  8.02 /* Added FOVLIST and BITLIST support              */
/*#define COADD2D_VERSION  8.03 /* Added option BPASS (BMASK passthru bits)       */
/* #define   COADD2D_VERSION  8.04 /* Used input unc's and fixed BMASK passthru bits */
#define   COADD2D_VERSION  8.05 /* change outunc to = sqrt(SUM(inunc**2))/N */


/*============================= CONSTANT DEFINITIONS =============================*/
/*   XD => Cross-dispersion direction   */
/*   DS => Dispersion direction         */

#ifndef FALSE
#define FALSE		   0
#define TRUE            (!FALSE)
#endif


/*============================ MAX/MIN MACRO DEFINITIONS =========================*/
#ifndef MAX
#define MAX(u,v)    ( ((u)>(v)) ? (u) : (v) );
#define MIN(u,v)    ( ((u)<(v)) ? (u) : (v) );
#endif

/* Termination codes as specified in:
**     SIRTF Downlink Software Development Guidelines  20010515
**     (file: /ssc/pipe/dlmtg/sw_dev_guidelines)
*/
#define TERMINATE_NORMAL   0
#define TERMINATE_WARNING 32
#define TERMINATE_ERROR	  64

/* Some max string lengths */
#define MAX_FILENAME_LEN 256 /* max # of characters allowed in a filename    */
#define NL_MAXLINE       256 /* max width of data line in the namelist file  */
#define MAX_NUM_C2D_HDRS  30 /* Maximum number of headers *added* by coadd2d */

/* Default values for some parameters */
#define DEFAULT_WTS_OUT_FILE    "2dcoadd_wt.txt"  /* As required */
#define DEFAULT_CUT_FRAC        0.2  /* Default: trim at most 20% of data (leave >= 80%) */
#define DEFAULT_CUT_MULTIPLE    5.0  /* As recommended in darkcal SDS */
#define DEFAULT_AMASK_MIN_GOOD  3  /* Minimum number of data values for      */
                                   /*   "good" statistics -- less than this  */
                                   /*   number will be flagged in the AMASK  */
                                   /*   using the AM_STATS_POOR bit          */
                                   
/* Markers for data in namelist file */
#define DATA_MARKER   "&COADD_DATA"
#define SDEV_MARKER   "&COADD_SDEV"
#define BMASK_MARKER  "&COADD_BMASK"
#define PMASK_MARKER  "&COADD_PMASK"
#define END_MARKER    "&END"

/* FMASK bit list definitions */
#define MAX_NUM_FOVS      20
#define FOV_SEPCHARS      " [],"
#define FOVLIST_KEYWORD   "FOVLIST"  
#define BITLIST_KEYWORD   "BITLIST"

/*=========================== MASK BITS DEFINITIONS ==========================*/

/* AMASK BITS -- In OCTAL for something not entirely unlike clarity */
#define AM_0                    01  /* Bit  0 =     1 */  /* NOT USED */
#define AM_1                    02  /* Bit  1 =     2 */  /* NOT USED */
#define AM_2                    04  /* Bit  2 =     4 */  /* NOT USED */
#define AM_3                   010  /* Bit  3 =     8 */  /* NOT USED */
#define AM_4                   020  /* Bit  4 =    16 */  /* NOT USED */
#define AM_5                   040  /* Bit  5 =    32 */  /* NOT USED */
#define AM_6                  0100  /* Bit  6 =    64 */  /* NOT USED */
#define AM_7                  0200  /* Bit  7 =   128 */  /* NOT USED */
#define AM_8                  0400  /* Bit  8 =   256 */  /* NOT USED */
#define AM_9                 01000  /* Bit  9 =   512 */  /* NOT USED */
#define AM_10                02000  /* Bit 10 =  1024 */  /* NOT USED */
#define AM_11                04000  /* Bit 11 =  2048 */  /* NOT USED */
#define AM_STATS_POOR       010000  /* Bit 12 =  4096 */  /* SEE SIS  */
#define AM_STATS_BAD        020000  /* Bit 13 =  8192 */  /* SEE SIS  */
#define AM_14               040000  /* Bit 14 = 16384 */  /* NOT USED */
#define AM_UNUSED_SIGN_BIT 0100000  /* Bit 15 = 32768 */  /* NOT USED */

/*========================== NAMELIST DEFINITIONS ===========================*/

/*========== INPUT & OUPTUT FILES ==========*/
#define   SW_NAMELIST        "n"

#define   SW_LISTS_IN_NLFILE "ls"
#define   NL_LISTS_IN_NLFILE "LISTS_IN_NLFILE"

#define   SW_DATA_IN_LFILE   "di"
#define   NL_DATA_IN_LFILE   "DATA_IN_LFILE"
#define   SW_DATA_OUT_FILE   "do"
#define   NL_DATA_OUT_FILE   "DATA_OUT_FILE"
#define   SW_SDEV_IN_LFILE   "si"
#define   NL_SDEV_IN_LFILE   "SDEV_IN_LFILE"
#define   SW_SDEV_OUT_FILE   "so"
#define   NL_SDEV_OUT_FILE   "SDEV_OUT_FILE"
#define   SW_FMASK_IN_FILE   "fm"
#define   NL_FMASK_IN_FILE   "FMASK_IN_FILE"
#define   SW_BMASK_IN_LFILE  "bm"
#define   NL_BMASK_IN_LFILE  "BMASK_IN_LFILE"
#define   SW_PMASK_IN_LFILE  "pm"
#define   NL_PMASK_IN_LFILE  "PMASK_IN_LFILE"
#define   SW_AMASK_OUT_FILE  "am"
#define   NL_AMASK_OUT_FILE  "AMASK_OUT_FILE"
#define   SW_WTS_OUT_FILE    "wo"
#define   NL_WTS_OUT_FILE    "WTS_OUT_FILE"

/*========== GENERAL PARAMETERS ==========*/
/*#define SW_FOV_NUM         "fv"         */
/*#define NL_FOV_NUM         "FOV_NUM"    */
#define   SW_BPASS           "bp"
#define   NL_BPASS           "BPASS"
#define   SW_BFATAL          "bf"
#define   NL_BFATAL          "BFATAL"
#define   SW_PFATAL          "pf"
#define   NL_PFATAL          "PFATAL"
/*#define SW_TRIM_F1         "f1"         */
/*#define NL_TRIM_F1         "TRIM_F1"    */
/*#define SW_TRIM_F2         "f2"         */
/*#define NL_TRIM_F2         "TRIM_F2"    */
#define   SW_AMASK_MIN_GOOD  "ag"
#define   NL_AMASK_MIN_GOOD  "AMASK_MIN_GOOD"
#define   SW_HEADER_PROP     "h1"
#define   NL_HEADER_PROP     "FILE1_HEADER_PROP"
#define   SW_USE_WEIGHTS     "wt"
#define   NL_USE_WEIGHTS     "USE_WEIGHTS"
/*define  SW_CR_THRESH       "cr"                 */
/*define  NL_CR_THRESH       "COSMIC_RAY_THRESH"  */
#define   SW_CUT_FRAC        "cf"
#define   NL_CUT_FRAC        "CUT_FRAC"
#define   SW_CUT_MULTIPLE    "cm"
#define   NL_CUT_MULTIPLE    "CUT_MULTIPLE"

/*========== HELP AND DEBUGGING ==========*/
#define SW_VERBOSE           "V"
#define NL_VERBOSE           "VERBOSE"
#define SW_DEBUG             "D"
#define NL_DEBUG             "DEBUG"

/*============================ MACRO DEFINITIONS ============================*/

/* My C book says that toint(c) is not in ISO C, so here's a quick little macro */
#define TOINT(c)	(  ( ((c)>='0') && ((c)<='9') ) ? ((c)-'0') : -1 )



/*============================ TYPE DEFINITIONS =============================*/

typedef unsigned char Flag;

/* Structure to hold stuff read from the command line, and derived values 
**
** The _f parameters are flags that show whether a parameter has been set yet or not.
*/
typedef struct {

   /*========== GENERAL STUFF ==========*/
   
   Flag      debug;
   Flag      verbose;
   char	     progname[MAX_FILENAME_LEN];        /* holds argv[0] */
   char	     namelist[MAX_FILENAME_LEN];
   Flag	     namelist_f;
   int       fovlist[MAX_NUM_FOVS];
   int       bitlist[MAX_NUM_FOVS];
   char      num_fovs;
                 
   /*========== INPUT & OUTPUT FILENAMES ==========*/
   
   /* lfile => file containing a list of filenames */
   Flag      lists_in_nlfile;                  /* TRUE if lists are IN the NL */
   Flag      lists_in_nlfile_f;
   char	     data_in_lfile[MAX_FILENAME_LEN];  /* List of 2d FITS data files */
   Flag	     data_in_lfile_f;                 
   char	     data_out_file[MAX_FILENAME_LEN];  /* Coadded 2d FITS data result */
   Flag	     data_out_file_f;                 
   char	     sdev_in_lfile[MAX_FILENAME_LEN];  /* List of 2d FITS sdev files */
   Flag	     sdev_in_lfile_f;                 
   char	     sdev_out_file[MAX_FILENAME_LEN];  /* SDEV of 2d FITS data result */
   Flag	     sdev_out_file_f;                 
   char	     fmask_in_file[MAX_FILENAME_LEN];  /* Input Fmask FITS file */
   Flag	     fmask_in_file_f;                  
   char	     bmask_in_lfile[MAX_FILENAME_LEN]; /* List of BMASKs for data_in */
   Flag	     bmask_in_lfile_f;                  
   char	     pmask_in_lfile[MAX_FILENAME_LEN]; /* List of PMASKs for data_in */  
   Flag	     pmask_in_lfile_f;                  
   char	     amask_out_file[MAX_FILENAME_LEN]; /* Output Amask FITS file */
   Flag	     amask_out_file_f;                  
   char	     wts_out_file[MAX_FILENAME_LEN];   /* Output weights ASCII file */
   Flag	     wts_out_file_f;
   
   /*========== GENERAL COMMAND-LINE PARAMETERS ==========*/   
   
   Flag      header_prop;      /* Should headers from FILE1 be propatated? */
   Flag	     header_prop_f;
   short int bpass;            /* BMASK passthru bits (int) */
   Flag	     bpass_f;
   short int bfatal;           /* BMASK fatal template (int) */
   Flag	     bfatal_f;
   short int pfatal;           /* PMASK fatal template (int) */
   Flag	     pfatal_f;
/* float     trim_f1;       */ /* Parameter f1 for trimmedstatistics() */
/* Flag      trim_f1_f;     */
/* float     trim_f2;       */ /* Parameter f2 for trimmedstatistics() */
/* Flag      trim_f2_f;     */
/* float     cr_thresh;     */ /* Cosmic Ray Threshold */
/* Flag      cr_thresh_f;   */
   Flag      use_weights;      /* Should WEIGHTED average be calculated? */
   Flag	     use_weights_f;
   float     cut_frac;          /* Cut out at most cut_frac of the data */
   Flag      cut_frac_f;
   float     cut_multiple;      /* Cut out all data cut_multiple*MedAbsDev above the MedAbsDev */
   Flag      cut_multiple_f;
   int	     amask_min_good;   /* Minimum number of data points for "good" data */
   Flag	     amask_min_good_f;             
   
   /*========== INPUT & OUPTUT DATA ===========*/
   short int fov_num;          /* The FOV number to be selected for, from FMASK */
   Flag      fov_num_f;       
   short int usefovid;         /* The USEFOVID parameter, from FMASK */
   Flag      usefovid_f;       
   short int channelID;        /* The instrument number (Channel ID) from FMASK */
   Flag      channelID_f;       
   short int fov_for_wts;      /* If weighting is to be done, this is set to desired FOV */
   short int bit_for_wts;      /* This is the corresponding bit for cs.fov_for_wts */
   int	     num_files;	       /* Number of input FITS data files */
   int	     xdim;             /* Dimensions of the data  */
   int       ydim;
   short int *amask;
   
} cmd_struct;

typedef struct {
   char v[MAX_FILENAME_LEN];   /* an individual filename */
} files_struct;

typedef struct {
   float *p;      /* will point to a 2D image (1D array of data in C) */
} image_struct;   /* its dimensions will be xdim by ydim */

typedef struct {
   short int *p;  /* will point to a 2D array (1D array of data in C) */
} simage_struct;  /* its dimensions will be xdim by ydim */

typedef struct {
   float value;   /* Pixel value */
   int   plane;   /* What image plane this pixel value comes from (i) */
   float weight;  /* Weight for this pixel */
} data_struct;




/*================================ PROTOTYPES ================================*/

/***** IN coadd2d_fits.c *****/
void malloc_failed( char *array_name );
void read_masks( cmd_struct *cs );
void create_amask( cmd_struct *cs );
void process_external_maskbits( cmd_struct cs );
void read_fits_dims_2d( char *fits_filename, int *xdim, int *ydim, Flag dbg );
void read_fmask_params( char *fits_filename, cmd_struct *cs );
float *read_fits( char *fits_filename, int *xdim, int *ydim, int *req_fov_num, Flag dbg );
short int *read_fits_s( char *fits_filename, int *xdim, int *ydim, Flag dbg );
void read_fmask_lists( cmd_struct *cp );
void write_amask( short int *amask, cmd_struct cs );
void write_weights_list( float *dce_weight, 
                         files_struct *data_files, cmd_struct cs );
void write_flt_fits( char *fname, float *image, cmd_struct cs );
void write_flt_fits_with_hdrs( char *infname, char *outfname, 
                               float *image,  cmd_struct cs    );
void test_fits( char *fits_filename );
void print_fits_data( char *fitsfile );
void insert_int_keyword( char *fname, char *keyname, int keyval );

/***** IN coadd2d_chores.c *****/
void error_and_exit( char *pname, char *str );
void ack_program_init( char *progname );
void ack_program_complete( char *progname );
void print_args( cmd_struct cs );
cmd_struct process_cmd_args( int argc, char **argv );
int count_filenames( char *filelist, Flag lists_in_nlfile, 
                     char *start_str, char *end_str, char *firstfile );
void read_filenames( char *filelist, Flag lists_in_nlfile, 
                     files_struct *filenames, int num_files, 
                     char *start_str, char *end_str            );
void find_markers( char *nl, Flag *sdev_in_nl, 
                   Flag *bmask_in_nl, Flag *pmask_in_nl );
/* STATISTICAL ROUTINES */
void trimmed_weighted_average( data_struct *data, int data_count,
                               float f1,    float f2, 
         /* Return values: */  float *mean, float *sdev, 
                               int   dbg                              );
void asymmetric_trimming( data_struct *pixel, int *data_count, cmd_struct cs );

#endif
