/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@ coadd_chores.c  --  Do the dumb things I've gotta do (touch the puppet head!)
@@
@@ MODIFICATION HISTORY:  (set FLATFIELD_VERSION in flatfield.h)
@@	Ver.	Prg.  What
@@ 	6.30	CW    Initial Version
@@ 	6.40	Bob   Added BPASS option (BMASK passthru bits)
@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

#include <stdio.h>
#include <math.h>
#include <strings.h>  /* for strcpy() */
#include <ctype.h>    /* for isdigit(), toupper(), islower() */
#include <stdlib.h>   /* for drand48(), qsort() */
#include "coadd2d.h"

#define DEBUG_CS_STUFF                 0
#define DEBUG_ASYMMETRIC_TRIMMING      0
#define DEBUG_QUICKSELECT              0
#define DEBUG_TRIMMED_WEIGHTED_AVERAGE 0
#define DEBUG_CALC_WEIGHTED_STATS      0

/*******************************************************************************
**  Convert a string to UPPERCASE 
*/
void str2uc(char *stuff)
{
int i;

   if (!stuff) return;
   for (i=0; i<(strlen(stuff)); i++) {
   
      /* Checking islower() first is s'posta be safer */
      if (islower(stuff[i])) stuff[i] = toupper(stuff[i]);  
   
   }
}


/*******************************************************************************
**  PRINT ERROR MESSAGE and EXIT
*/
void error_and_exit( char *pname, char *str )
{
   printf("\aERROR in program %s:  %s.\n\n", pname, str );
   exit( TERMINATE_ERROR );
}

/*******************************************************************************
**  PRINT ARGUMENT # ERROR MESSAGE and EXIT
*/
void print_err_cmd( int i, char *str, cmd_struct cs )
{
   printf("\aERROR in processing command line at argument #%1d: \"%s\"\n",i,str);
   exit( TERMINATE_ERROR );
}

/*******************************************************************************
**  PRINT NAMELIST PARAMETER ERROR MESSAGE and EXIT
*/
void print_err_nl( char *param, char *str )
{
   printf("\aERROR in processing namelist file: PARAM=<%s> VAL=<%s>\n",param,str);
   exit( TERMINATE_ERROR );
}



/*******************************************************************************
** Acknowledge program initialization 
** (as requested by /ssc/pipe/dlmtg/sw_dev_guidelines #2)
*/
void ack_program_init( char *progname )
{
   printf("\n==============================================");
   printf("==============================================\n");
   printf("===>     ");
   printf( "COADD2D MODULE \"%s\"  VERSION = %g BEGUN.\n", 
            progname, COADD2D_VERSION );
   printf("===>     ");
   printf( "Processed at: " );
   fflush( stdout );
   system( "date" );
   printf("==============================================");
   printf("==============================================\n");
   printf( "\n" );
}


/*******************************************************************************
** Acknowledge program completion 
** (as requested by /ssc/pipe/dlmtg/sw_dev_guidelines #5)
*/
void ack_program_complete( char *progname )
{
   printf("\n==============================================");
   printf("==============================================\n");
   printf("===>     ");
   printf( "COADD2D MODULE \"%s\"  VERSION = %g COMPLETED.\n", 
            progname, COADD2D_VERSION );
   printf("===>     ");
   printf( "Processing COMPLETED at: " );
   fflush( stdout );
   system( "date" );
   printf("==============================================");
   printf("==============================================\n");
   printf( "\n" );
}


/*******************************************************************************
**  PRINT OUT ARGUMENTS STORED IN A cmd_struct (for debugging purposes )
*/
void print_args( cmd_struct cs )
{
   printf("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
   printf("        LIST OF COMMAND-LINE ARGUMENTS AND DEFAULTS:\n\n");
   printf("\n");
   printf("========== GENERAL STUFF =================\n");
   printf("\n");    
   if ( cs.debug )             printf("  (-%s)   DEBUG   mode      ON\n",    SW_DEBUG   );
   else                        printf("  (-%s)   DEBUG   mode      OFF\n",   SW_DEBUG   );
   if ( cs.verbose )           printf("  (-%s)   VERBOSE mode      ON\n",    SW_VERBOSE );
   else                        printf("  (-%s)   VERBOSE mode      OFF\n",   SW_VERBOSE );
   /* NO if */                 printf("         progname        = \"%s\"\n", cs.progname);
   if ( cs.namelist_f )        printf("         namelist        = \"%s\"\n", cs.namelist);
   else                        printf("         namelist          NOT DEFINED. \n");
   printf("\n");
   printf("========== INPUT & OUPTUT FILES ==========\n");
   printf("\n");    
   if ( cs.lists_in_nlfile_f ) printf("  (-%s)  lists_in_nlfile = %s\n",            SW_LISTS_IN_NLFILE,
                                                (!cs.lists_in_nlfile) ? "FALSE" : "TRUE"                                 );
   else                        printf("  (-%s)  lists_in_nlfile   NOT DEFINED. \n", SW_LISTS_IN_NLFILE                   );
   if ( cs.data_in_lfile_f )   printf("  (-%s)  data_in_lfile   = \"%s\"\n",        SW_DATA_IN_LFILE,  cs.data_in_lfile  );
   else                        printf("  (-%s)  data_in_lfile     NOT DEFINED. \n", SW_DATA_IN_LFILE                     );
   if ( cs.data_out_file_f )   printf("  (-%s)  data_out_file   = \"%s\"\n",        SW_DATA_OUT_FILE,  cs.data_out_file  );
   else                        printf("  (-%s)  data_out_file     NOT DEFINED. \n", SW_DATA_OUT_FILE                     );
   if ( cs.sdev_in_lfile_f )   printf("  (-%s)  sdev_in_lfile   = \"%s\"\n",        SW_SDEV_IN_LFILE,  cs.sdev_in_lfile  );
   else                        printf("  (-%s)  sdev_in_lfile     NOT DEFINED. \n", SW_SDEV_IN_LFILE                     );
   if ( cs.sdev_out_file_f )   printf("  (-%s)  sdev_out_file   = \"%s\"\n",        SW_SDEV_OUT_FILE,  cs.sdev_out_file  );
   else                        printf("  (-%s)  sdev_out_file     NOT DEFINED. \n", SW_SDEV_OUT_FILE                     );
   if ( cs.fmask_in_file_f )   printf("  (-%s)  fmask_in_file   = \"%s\"\n",        SW_FMASK_IN_FILE,  cs.fmask_in_file  );
   else                        printf("  (-%s)  fmask_in_file     NOT DEFINED. \n", SW_FMASK_IN_FILE                     );
   if ( cs.bmask_in_lfile_f )  printf("  (-%s)  bmask_in_lfile  = \"%s\"\n",        SW_BMASK_IN_LFILE, cs.bmask_in_lfile );
   else                        printf("  (-%s)  bmask_in_lfile    NOT DEFINED. \n", SW_BMASK_IN_LFILE                    );
   if ( cs.pmask_in_lfile_f )  printf("  (-%s)  pmask_in_lfile  = \"%s\"\n",        SW_PMASK_IN_LFILE, cs.pmask_in_lfile );
   else                        printf("  (-%s)  pmask_in_lfile    NOT DEFINED. \n", SW_PMASK_IN_LFILE                    );
   if ( cs.amask_out_file_f )  printf("  (-%s)  amask_out_file  = \"%s\"\n",        SW_AMASK_OUT_FILE, cs.amask_out_file );
   else                        printf("  (-%s)  amask_out_file    NOT DEFINED. \n", SW_AMASK_OUT_FILE                    );
   if ( cs.wts_out_file_f )    printf("  (-%s)  wts_out_file    = \"%s\"\n",        SW_WTS_OUT_FILE,   cs.wts_out_file   );
   else                        printf("  (-%s)  wts_out_file      NOT DEFINED. \n", SW_WTS_OUT_FILE                      );
   printf("\n"); 
   printf("========== GENERAL SWITCHES ==============\n");
   printf("\n");       
/* if ( cs.fov_num_f )         printf("  (-%s)  fov_num         = %1d\n",           SW_FOV_NUM,        cs.fov_num        );
** else                        printf("  (-%s)  fov_num           NOT DEFINED. \n", SW_FOV_NUM                           );
*/
   if ( cs.bpass_f  )          printf("  (-%s)  bpass           = %1d\n",           SW_BPASS,          cs.bpass          );
   else                        printf("  (-%s)  bpass             NOT DEFINED. \n", SW_BPASS                             );
   if ( cs.bfatal_f )          printf("  (-%s)  bfatal          = %1d\n",           SW_BFATAL,         cs.bfatal         );
   else                        printf("  (-%s)  bfatal            NOT DEFINED. \n", SW_BFATAL                            );
   if ( cs.pfatal_f )          printf("  (-%s)  pfatal          = %1d\n",           SW_PFATAL,         cs.pfatal         );
   else                        printf("  (-%s)  pfatal            NOT DEFINED. \n", SW_PFATAL                            );
/* if ( cs.trim_f1_f )         printf("  (-%s)  trim_f1         = %g\n",            SW_TRIM_F1,        cs.trim_f1        );
** else                        printf("  (-%s)  trim_f1           NOT DEFINED. \n", SW_TRIM_F1                           );
** if ( cs.trim_f2_f )         printf("  (-%s)  trim_f2         = %g\n",            SW_TRIM_F2,        cs.trim_f2        );
** else                        printf("  (-%s)  trim_f2           NOT DEFINED. \n", SW_TRIM_F2                           );
** if ( cs.cr_thresh_f )       printf("  (-%s)  cr_thresh       = %g\n",            SW_CR_THRESH,      cs.cr_thresh      );
** else                        printf("  (-%s)  cr_thresh         NOT DEFINED. \n", SW_CR_THRESH                         );
*/ if ( cs.use_weights_f )     printf("  (-%s)  use_weights     = %s\n",            SW_USE_WEIGHTS,
                                                cs.use_weights  ? "YES" : "NO"                                           );
   else                        printf("  (-%s)  use_weights       NOT DEFINED. \n", SW_USE_WEIGHTS                       );
   if ( cs.cut_frac_f )        printf("  (-%s)  cut_frac        = %g\n",            SW_CUT_FRAC,       cs.cut_frac       );
   else                        printf("  (-%s)  cut_frac          NOT DEFINED. \n", SW_CUT_FRAC                          );
   if ( cs.cut_multiple_f )    printf("  (-%s)  cut_multiple    = %g\n",            SW_CUT_MULTIPLE,   cs.cut_multiple   );
   else                        printf("  (-%s)  cut_multiple      NOT DEFINED. \n", SW_CUT_MULTIPLE                      );
   if ( cs.amask_min_good_f )  printf("  (-%s)  amask_min_good  = %1d\n",           SW_AMASK_MIN_GOOD, cs.amask_min_good );
   else                        printf("  (-%s)  amask_min_good    NOT DEFINED. \n", SW_AMASK_MIN_GOOD                    );
/*   if ( cs.header_prop_f )     printf("  (-%s)  header_prop     = %s\n",            SW_HEADER_PROP,
                                                cs.header_prop  ? "YES" : "NO"                                           );                   
   else                        printf("  (-%s)  header_prop       NOT DEFINED. \n", SW_HEADER_PROP                       );                                     );
   printf("\n"); */
   printf("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n");
}



/*******************************************************************************
**  PRINT USAGE -- what should be output when the program 
**                 is called without any arguments
*/
void print_usage( cmd_struct cs )
{
   printf("\n");
   printf( "COADD2D MODULE \"%s\"  VERSION = %g\n", 
            cs.progname, COADD2D_VERSION );
   printf("\n");
   printf("USAGE:  %s -di DATA_FILES.list -do RESULT.fits \n",cs.progname );
   printf("                                                                   \n");
   printf("========== INPUT FILES ==========                                  \n");
   printf("                                                                   \n");
   printf("  -di DATA_FILES.list   Req* List of 2D FITS data files            \n");
   printf("  -si SDEV_FILES.list   Opt  List of 2d FITS sdev files            \n");
   printf("                             (these files are unused at present)   \n");
   printf("  -fm FMASK.fits        Opt  FMASK (the FOV Mask) FITS file        \n");
   printf("  -bm BMASK_FILES.list  Opt  List of BMASKs for DATA_FILES         \n");
   printf("  -pm PMASK_FILES.list  Opt  List of PMASKs for DATA_FILES         \n");
   printf("                                                                   \n");
   printf("========== OUTPUT FILES ==========                                 \n");
   printf("                                                                   \n");
   printf("  -do RESULT.fits       Req  Coadded 2d FITS data result           \n");
   printf("  -so SDEV.fits         Opt  SDEV of 2d FITS data result           \n");
   printf("  -am AMASK.fits        Opt  Resulting AMASK                       \n");
   printf("  -wo WEIGHTS.txt       Opt  ASCII file of weights & (di) filenames\n");
   printf("                             DEFAULT: \"%s\" \n", DEFAULT_WTS_OUT_FILE);
   printf("                                                                   \n");
   printf("========== GENERAL SWITCHES ==========                             \n");
   printf("                                                                   \n");
   printf("  -n  NAMELIST          Opt  A Namelist File                       \n");
   printf("  -ls <0 or 1>          Opt  Input files lists are in namelist     \n");
   printf("                             (not in separate files)               \n");
   printf("  -wt <0 or 1>          Opt  Calculate weighted statistics?        \n");
   printf("                             Default = 0 if no FMASK file, else 1  \n");
   printf("  -h1                   Opt  Propagate *all* the headers from      \n");
   printf("                             the first file in DATA_FILES (-di)    \n");
   printf("                                                                   \n");
   printf("========== PARAMETERS ==========                                   \n");
   printf("                                                                   \n");
/* printf("  -fv FOV_NUM           Opt  FOV number of interest (Int)          \n");
** printf("                             (Use with option -fm)                 \n");
*/ printf("  -bf BMASK_FATAL_BITS  Opt  BMASK fatal bits template (Int)       \n");
   printf("                       (Req  if -bm is specified)                  \n");
   printf("  -pf BMASK_FATAL_BITS  Opt  PMASK fatal bits template (Int)       \n");
   printf("                       (Req  if -bm is specified)                  \n");
/* printf("  -f1 TRIM_F1(CENTRAL)  Opt  f1 central trim fraction FOR DATA     \n");
** printf("                             DEFAULT=0.9 => trim to keep 90%% of   \n");
** printf("                             the original data [*]                 \n");
** printf("  -f2 TRIM_F2(CENTRAL)  Opt  f2 central trim fraction FOR SDEV     \n");
** printf("                             DEFAULT=0.9 => calculate sdev using   \n");
** printf("                             90%% of the original data [*]         \n");
*/ printf("  -ag AMASK_MIN_GOOD    Opt  AMASK marks pixels with fewer data    \n");
   printf("                             points than this value.  DEFAULT=%1d  \n", DEFAULT_AMASK_MIN_GOOD );
/* printf("  -cr COSMIC_RAY_THRESH Opt  Ignore pixels above this value (Float)\n");
*/ printf("                                                                   \n");
   printf("  -cf CUT_FRAC          Opt  Cut at most cut_frac of the data (Flt)\n");
   printf("                             DEFAULT=%3.1f  (0 => DO NO TRIMMING)  \n", DEFAULT_CUT_FRAC ); 
   printf("  -cm CUT_MULTIPLE      Opt  Cut above Multiple of MedAbsDev (Flt) \n");
   printf("                             DEFAULT=%3.1f                         \n", DEFAULT_CUT_MULTIPLE ); 
   printf("                                                                   \n");
   printf("========== HELP AND DEBUGGING ==========                           \n");
   printf("                                                                   \n");
   printf("  -V                    Opt  Verbose mode: more functional info    \n");
   printf("  -D                    Opt  Debugging mode: adds debugging data   \n");
   printf("  -HELP                 Opt  Print out an example NAMELIST file    \n");
   printf("                                                                   \n");
   printf("========== FOOTNOTES ==========                                    \n");
   printf(".list => Text file containing list of FITS files                   \n");
   printf(".tbl  => IPAC Table Format                                         \n");
   printf(".fits => 2D FITS file                                              \n");
   printf("Req*  => Required, unless -ls is 1                                 \n");
   printf("         Must have either -ls or [-di -si -bm -pm]                 \n");
/* printf("[*] NOTE: f1 & f2 are defined *similarly* to Russ Laher's f1 & f2, \n");
** printf("    HOWEVER, they are NOT *identical*.                             \n");
*/ printf("                                                                   \n");
}



/*******************************************************************************
**  PRINT OUT A SAMPLE NAMELIST AS A HELP FILE
*/
void detailed_help( void )
{
   printf("\n");
   printf("This is a Sample NAMELIST file for the coadd2d program.         \n");
   printf("                                                                \n");
   printf("      Command-line form of the stuff below is:                  \n");
   printf("                                                                \n");
   printf("      coadd2d -di t1_data_in.list   -do t1_data_out.fits        \n");
   printf("              -so t1_sdev_out.fits  -am t1_amask_out.fits       \n");
   printf("              -bm t1_bmask_in.list  -pm t1_pmask_in.list        \n");
   printf("              -fm b0_fmask.fits     -wo t1_weights.list         \n");
   printf("              -bf 128  -pf 64                                   \n");
   printf("              -ag 5  -h1 1  -wt 1  -cb 2.5  -ct 3.5             \n");
   printf("              -V  -D                                            \n");
   printf("                                                                \n");
   printf("&COADD2D - - - - - - - - - - - - - - - - - - - - - - - - - - -  \n");
   printf("                                                                \n");
   printf("   #================= INPUT & OUTPUT FILES ===============      \n");
   printf("    C2D_ls = 1                          LISTS_IN_NLFILE         \n");
   printf("    C2D_di = t1_data_in.list            DATA_IN_LFILE           \n");
   printf("    C2D_do = t1_data_out.fits           DATA_OUT_FILE           \n");
   printf("   #C2D_si =                            SDEV_IN_LFILE           \n");
   printf("    C2D_so = t1_sdev_out.fits           SDEV_OUT_FILE           \n");
   printf("    C2D_fm = b0_fmask.fits              FMASK_IN_FILE           \n");
   printf("    C2D_bm = t1_bmask_in.list           BMASK_IN_LFILE          \n");
   printf("    C2D_pm = t1_pmask_in.list           PMASK_IN_LFILE          \n");
   printf("    C2D_am = t1_amask_out.fits          AMASK_OUT_FILE          \n");
   printf("    C2D_wo = t1_weights.list            WTS_OUT_FILE            \n");
   printf("                                                                \n");
   printf("   #================ GENERAL PARAMETERS ==================      \n");
   printf("    C2D_bp =    0                       BPASS                   \n");
   printf("    C2D_bf =  128                       BFATAL                  \n");
   printf("    C2D_pf =   64                       PFATAL                  \n");
/* printf("    C2D_fv =   27                       FOV_NUM                 \n");
** printf("    C2D_f1 =   .5                       TRIM_F1                 \n");
** printf("    C2D_f2 =   .5                       TRIM_F2                 \n");
** printf("    C2D_cr =  863.39                    COSMIC_RAY_THRESH       \n");
*/ printf("    C2D_ag =    5                       AMASK_MIN_GOOD          \n");
   printf("    C2D_h1 =    1                       FILE1_HEADER_PROP       \n");
   printf("    C2D_wt =    1                       USE_WEIGHTS             \n");
   printf("    C2D_cf =    0.25                    CUT_FRAC                \n");
   printf("    C2D_cm =    6.0                     CUT_MULTIPLE            \n");
   printf("                                                                \n");
   printf("   #================= HELP AND DEBUGGING =================      \n");
   printf("    C2D_V  =    1                       VERBOSE                 \n");
   printf("    C2D_D  =    1                       DEBUG                   \n");
   printf("                                                                \n");
   printf("&END                                                            \n");
   printf("                                                                \n");
   printf("&COADD_DATA  - - - - - - - - - - - - - - - - - - - - - - - - -  \n");
   printf("  d1.fits                                                       \n");
   printf("  d2.fits                                                       \n");
   printf("  d3.fits                                                       \n");
   printf("&END                                                            \n");
   printf("                                                                \n");
   printf("&COADD_SDEV  - - - - - - - - - - - - - - - - - - - - - - - - -  \n");
   printf("  s1.fits                                                       \n");
   printf("  s2.fits                                                       \n");
   printf("  s3.fits                                                       \n");
   printf("&END                                                            \n");
   printf("                                                                \n");
   printf("&COADD_BMASK  - - - - - - - - - - - - - - - - - - - - - - - - - \n");
   printf("  b1.fits                                                       \n");
   printf("  b2.fits                                                       \n");
   printf("  b3.fits                                                       \n");
   printf("&END                                                            \n");
   printf("                                                                \n");
   printf("&COADD_PMASK  - - - - - - - - - - - - - - - - - - - - - - - - - \n");
   printf("  p1.fits                                                       \n");
   printf("  p2.fits                                                       \n");
   printf("  p3.fits                                                       \n");
   printf("&END                                                            \n");
   printf("\n");
}



/*******************************************************************************
**  INITIALIZE ALL THE _f FLAGS IN A CS TO FALSE
**  i.e. CLEAR THE FLAGS THAT INDICATE WHETHER EACH PARAMETER HAS BEEN SET YET
**  and set num_fovs to -1 to indicate no FOVLIST found yet
*/
void clear_cs_flags( cmd_struct *cp )
{
   /*========== GENERAL STUFF ==========*/
   cp->debug             = FALSE;
   cp->verbose           = FALSE;
   /*.progname           = set elsewhere */
   cp->namelist_f        = FALSE;
   cp->num_fovs          = -1;

   /*========== INPUT & OUTPUT FILES ==========*/          
   cp->lists_in_nlfile_f = FALSE;                   
   cp->data_in_lfile_f   = FALSE;                   
   cp->data_out_file_f   = FALSE;
   cp->sdev_in_lfile_f   = FALSE;
   cp->sdev_out_file_f   = FALSE;
   cp->fmask_in_file_f   = FALSE;
   cp->bmask_in_lfile_f  = FALSE;
   cp->pmask_in_lfile_f  = FALSE;
   cp->amask_out_file_f  = FALSE;
   cp->wts_out_file_f    = FALSE;


   /*========== GENERAL COMMAND-LINE PARAMETERS ==========*/      
   cp->bpass_f           = FALSE;
   cp->bfatal_f          = FALSE;
   cp->pfatal_f          = FALSE;
/* cp->fov_num_f         = FALSE;
** cp->trim_f1_f         = FALSE;
** cp->trim_f2_f         = FALSE;
** cp->cr_thresh_f       = FALSE;
*/ cp->amask_min_good_f  = FALSE;
   cp->header_prop_f     = FALSE;
   cp->use_weights_f     = FALSE;
   cp->cut_frac_f        = FALSE;
   cp->cut_multiple_f    = FALSE;
  
   /*========== INPUT & OUPTUT DATA ===========*/
   cp->fov_num_f         = FALSE;
   cp->usefovid_f        = FALSE;
   cp->channelID_f       = FALSE;
}



/***********************************************************************************************
**  PROCESS THE TWO (LEADING) VALUES IN ONE LINE FROM THE NAMELIST FILE
*/
void namelist_line( cmd_struct *cp, char *rawtoken, char *value )
{
char		*sw;
char	        token[NL_MAXLINE];
int		i;
int		nums;
int		flagval;
char		noswitch[] = "noswitch";

                            
   strcpy( token, rawtoken );  /* WAS: token = rawtoken */
   if (strlen(token)>3)  sw = rawtoken + 4;    /* EXAMPLE: in the line "C2D_px" the character "p" is at token[4] */
   else                  sw = noswitch;
   str2uc( token );  /* token is case-insensitive, sw is case-sensitive */

   if (0) printf("######  rawtoken=<%s>  token=<%s>  ",  rawtoken, token ); 
   if (0) printf("strlen(token) = %1d\n", strlen(token) ); 
      
   if (0) printf("######  rawtoken=<%s>  token=<%s>  sw=<%s>  value=<%s>\n",
                          rawtoken,      token,      sw,      value );
   
   /*========== GENERAL STUFF ==========*/
   if      ( !strcmp( SW_VERBOSE,         sw ) ||
             !strcmp( NL_VERBOSE,         token ) ) { nums = sscanf(value,"%d",&flagval);
                                                      cp->verbose = (Flag)flagval;
                                                      if (nums!=1) print_err_nl( token, value ); }
   else if ( !strcmp( SW_DEBUG,           sw ) ||
             !strcmp( NL_DEBUG,           token ) ) { nums = sscanf(value,"%d",&flagval);
                                                      cp->debug = (Flag)flagval;
                                                      if (nums!=1) print_err_nl( token, value ); }
                                                  
   /*========== INPUT & OUPTUT FILES ==========*/
   else if ( !strcmp( SW_LISTS_IN_NLFILE, sw ) ||
             !strcmp( NL_LISTS_IN_NLFILE, token ) ) { nums = sscanf(value,"%d",&flagval);
                                                      cp->lists_in_nlfile   = (Flag)flagval;
                                                      cp->lists_in_nlfile_f = TRUE;
                                                      if (nums!=1) print_err_nl( token, value ); }
                                                  
   else if ( !strcmp( SW_DATA_IN_LFILE,   sw ) ||
             !strcmp( NL_DATA_IN_LFILE,   token ) ) { strcpy( cp->data_in_lfile,  value ); cp->data_in_lfile_f  = TRUE; }
   else if ( !strcmp( SW_DATA_OUT_FILE,   sw ) ||
             !strcmp( NL_DATA_OUT_FILE,   token ) ) { strcpy( cp->data_out_file,  value ); cp->data_out_file_f  = TRUE; }
   else if ( !strcmp( SW_SDEV_IN_LFILE,   sw ) ||
             !strcmp( NL_SDEV_IN_LFILE,   token ) ) { strcpy( cp->sdev_in_lfile,  value ); cp->sdev_in_lfile_f  = TRUE; }
   else if ( !strcmp( SW_SDEV_OUT_FILE,   sw ) ||
             !strcmp( NL_SDEV_OUT_FILE,   token ) ) { strcpy( cp->sdev_out_file,  value ); cp->sdev_out_file_f  = TRUE; }
   else if ( !strcmp( SW_FMASK_IN_FILE,   sw ) ||
             !strcmp( NL_FMASK_IN_FILE,   token ) ) { strcpy( cp->fmask_in_file,  value ); cp->fmask_in_file_f  = TRUE; }
   else if ( !strcmp( SW_BMASK_IN_LFILE,  sw ) ||
             !strcmp( NL_BMASK_IN_LFILE,  token ) ) { strcpy( cp->bmask_in_lfile, value ); cp->bmask_in_lfile_f = TRUE; }
   else if ( !strcmp( SW_PMASK_IN_LFILE,  sw ) ||
             !strcmp( NL_PMASK_IN_LFILE,  token ) ) { strcpy( cp->pmask_in_lfile, value ); cp->pmask_in_lfile_f = TRUE; }
   else if ( !strcmp( SW_AMASK_OUT_FILE,  sw ) ||
             !strcmp( NL_AMASK_OUT_FILE,  token ) ) { strcpy( cp->amask_out_file, value ); cp->amask_out_file_f = TRUE; }
   else if ( !strcmp( SW_WTS_OUT_FILE,    sw ) ||
             !strcmp( NL_WTS_OUT_FILE,    token ) ) { strcpy( cp->wts_out_file,   value ); cp->wts_out_file_f   = TRUE; }

   /*========== GENERAL SWITCHES ==========*/
   else if ( !strcmp( SW_BPASS,           sw ) ||
             !strcmp( NL_BPASS,           token ) ) { nums = sscanf(value,"%hd",&(cp->bpass));  /* "hd" means short int */
                                                     if (nums!=1) print_err_nl( token, value );
                                                      cp->bpass_f          = TRUE;                }

   else if ( !strcmp( SW_BFATAL,          sw ) ||
             !strcmp( NL_BFATAL,          token ) ) { nums = sscanf(value,"%hd",&(cp->bfatal));  /*RECALL: "hd" means short int */
                                                     if (nums!=1) print_err_nl( token, value );
                                                      cp->bfatal_f         = TRUE;                }
   else if ( !strcmp( SW_PFATAL,          sw ) ||
             !strcmp( NL_PFATAL,          token ) ) { nums = sscanf(value,"%hd",&(cp->pfatal));
                                                      if (nums!=1) print_err_nl( token, value );
                                                      cp->pfatal_f         = TRUE;                }                                                                                    
/* else if ( !strcmp( SW_FOV_NUM,         sw ) ||
**           !strcmp( NL_FOV_NUM,         token ) ) { nums = sscanf(value,"%hd",&(cp->fov_num));    
**                                                   if (nums!=1) print_err_nl( token, value );
**                                                    cp->fov_num_f        = TRUE;                }
** else if ( !strcmp( SW_TRIM_F1,         sw ) ||
**           !strcmp( NL_TRIM_F1,         token ) ) { nums = sscanf(value,"%f", &(cp->trim_f1));
**                                                   if (nums!=1) print_err_nl( token, value );
**                                                   cp->trim_f1_f         = TRUE;                }
** else if ( !strcmp( SW_TRIM_F2,         sw ) ||
**           !strcmp( NL_TRIM_F2,         token ) ) { nums = sscanf(value,"%f", &(cp->trim_f2));
**                                                   if (nums!=1) print_err_nl( token, value );
**                                                   cp->trim_f2_f         = TRUE;                }
** else if ( !strcmp( SW_CR_THRESH,       sw ) ||
**           !strcmp( NL_CR_THRESH,       token ) ) { nums = sscanf(value,"%f", &(cp->cr_thresh));
**                                                   if (nums!=1) print_err_nl( token, value );
**                                                   cp->cr_thresh_f       = TRUE;                }
*/ else if ( !strcmp( SW_AMASK_MIN_GOOD,  sw ) ||
             !strcmp( NL_AMASK_MIN_GOOD,  token ) ) { nums = sscanf(value,"%d", &(cp->amask_min_good));
                                                      if (nums!=1) print_err_nl( token, value );
                                                      cp->amask_min_good_f = TRUE;                }
   else if ( !strcmp( SW_HEADER_PROP,     sw ) ||
             !strcmp( NL_HEADER_PROP,     token ) ) { nums = sscanf(value,"%d", &flagval);
                                                      cp->header_prop = (Flag)flagval;
                                                      if (nums!=1) print_err_nl( token, value ); 
                                                      cp->header_prop_f    = TRUE;                }
                                                      
   else if ( !strcmp( SW_USE_WEIGHTS,     sw ) ||
             !strcmp( NL_USE_WEIGHTS,     token ) ) { nums = sscanf(value,"%d",&flagval);
                                                      cp->use_weights   = (Flag)flagval;
                                                      cp->use_weights_f = TRUE;
                                                      if (nums!=1) print_err_nl( token, value );  }
             
   else if ( !strcmp( SW_CUT_FRAC,        sw ) ||
             !strcmp( NL_CUT_FRAC,        token ) ) { nums = sscanf(value,"%f", &(cp->cut_frac));
                                                     if (nums!=1) print_err_nl( token, value );
                                                     cp->cut_frac_f         = TRUE;               }
   else if ( !strcmp( SW_CUT_MULTIPLE,    sw ) ||
             !strcmp( NL_CUT_MULTIPLE,    token ) ) { nums = sscanf(value,"%f", &(cp->cut_multiple));
                                                     if (nums!=1) print_err_nl( token, value );
                                                     cp->cut_multiple_f         = TRUE;           }
                                                                                    
  
   else printf("\nWARNING: Unknown switch \"%s\" IGNORED in namelist file.\n\n", token) /* DEFAULT CASE */;

}


/*******************************************************************************
**  DETERMINE IF THERE IS A NAMELIST, AND IF SO, PROCESS IT 
**
**  ASSUME: The cs structure has already been initialized:  
**               all the *_f flags are set to FALSE.
**  WARNING: MUST PUT SPACES AROUND ALL "=" SIGNS
*/
void process_namelist( cmd_struct *cp, int argc, char **argv )
{
int	i;
char	nl_fname[MAX_FILENAME_LEN];
char	nl_dataline[NL_MAXLINE];
char	token[NL_MAXLINE], value[NL_MAXLINE];
FILE	*nl;
Flag	found;
int	rv;

   found = FALSE;      /* found marks whether "-n" has been found in the args */
   for( i=1; i<argc-1; i++ ) {        /* used argc-1 since a filename must follow the "-n" */
      if ( !strcmp( "-n", argv[i]) ) { 
         i++; 
         strcpy( nl_fname, argv[i] ); 
         found = TRUE; 
         break; 
      }
   }

   if (!found) { /* No need to do anything here if there's no "-n" NAMELIST */
      #if DEBUG_CS_STUFF 
         printf("NOT FOUND: -n  ==> NO NAMELIST PROCESSING. \n");
      #endif
      return;  
   }
   #if DEBUG_CS_STUFF 
      printf("FOUND: -n \n");
   #endif

   #if DEBUG_CS_STUFF 
      printf("PROCESS NAMELIST FILE: \"%s\"\n",nl_fname);
   #endif
   nl = fopen( nl_fname, "r" );
   if (!nl) {
      printf("\aERROR opening NAMELIST file: \"%s\"",nl_fname);
      printf(" in routine process_namelist()\n");
      exit( TERMINATE_ERROR );
   }
   strcpy( cp->namelist, nl_fname );
   cp->namelist_f = TRUE;
   

   /* GET FIRST "&COADD2D" TOKEN */
   #if DEBUG_CS_STUFF 
      printf("Read in the NAMELIST data...\n");
   #endif
   while ( fgets(nl_dataline, NL_MAXLINE, nl) ) {
      rv = sscanf( nl_dataline, "%s", token ); /* Read 1st token on the line */
      if (rv>0) {
         str2uc( token );
         found = !strcmp("&COADD2D",token);
         if (found) break;
      }
   }
   if (!found) {
      printf("CAUTION: MISTAKE IN NAMELIST, %s -- \"&COADD2D\" not found.\n", 
                                          nl_fname );
      printf("         IGNORING NAMELIST. \n\n");
      return;
   }
   #if DEBUG_CS_STUFF 
      printf("FOUND: \"&COADD2D\"\n");
   #endif 
     
   /* GET DATA UNTIL "&END" TOKEN is reached */
   if (cp->debug) printf("Read in the NAMELIST data...\n");
   while ( fgets(nl_dataline, NL_MAXLINE, nl) ) {
      rv = sscanf( nl_dataline, "%s", token ); /* Read 1st token on the line */
      if (rv<=0) continue;  /* Skip a blank line */
      str2uc( token );
      found = !strcmp("&END",token);
      if (0) printf("rv=%1d, TOKEN:<%s>  LINE:<%s>",rv,token,nl_dataline);
      if (found) break;
      if (token[0]=='#') continue;  /* Marks a comment */
      
      /* Replace <=> <,> <'> <"> chars with space chars */
      for( i=1; i<strlen(nl_dataline); i++ ) {  
         if ( (nl_dataline[i]=='=')  || (nl_dataline[i]==',') || 
              (nl_dataline[i]=='\'') || (nl_dataline[i]=='\"')   ) 
            /*then*/ nl_dataline[i] = ' '; 
      } 
      
      rv = sscanf( nl_dataline, "%s %s", token, value );
      if (rv!=2) {
         printf("CAUTION: Malformed line in NAMELIST IGNORED: \"%s\"\n",
                  nl_dataline);
         continue;
      }
      
      if (0) printf("**********  token=<%s>  value=<%s> \n", token, value);  
      namelist_line( cp, token, value );
      
   }
   if (!found) {
      printf("CAUTION: MISTAKE IN NAMELIST, %s -- \"&END\" not found.\n", 
                                          nl_fname );
      printf("         IGNORING LACK OF \"&END\". \n\n");
      return;
   }
   #if DEBUG_CS_STUFF 
      printf("FOUND: \"&END\" ==> END OF NAMELIST PROCESSING\n\n");
      printf("\nAfter parsing the namelist, the values are:\n");
      print_args( *cp );
   #endif
   
   fclose(nl);

}


/*******************************************************************************
**  PROCESS COMMAND LINE SWITCHES AND ARGUMENTS
*/
cmd_struct process_cmd_args( int argc, char **argv )
{
int		i;
int		nums;
double		temp_c;
cmd_struct	cs;
char		*sw;
int             flagval;

   clear_cs_flags( &cs );   
   strcpy( cs.progname, argv[0] );
   
   if (argc == 1) {             /* NO ARGUMENTS => PRINT USAGE STATEMENT */
      print_usage( cs );
      exit( TERMINATE_NORMAL );
   }
   
   process_namelist( &cs, argc, argv );

   for( i=1; i<argc; i++ ) {
      if (argv[i][0] != '-') {
         printf("\a\n*** NOTE:  Unexpected parameter #%1d: \"%s\" IGNORED.\n\n",
                                                      i,     argv[i]);              

      } else {
      
         sw = &(argv[i][1]); /* Point sw to the first letter in the switch */
             
         /*========== GENERAL STUFF ==========*/
         if      (!strcmp( SW_VERBOSE,         sw )) { cs.verbose = TRUE; }
         else if (!strcmp( SW_DEBUG,           sw )) { cs.debug   = TRUE; }
         else if (!strcmp( "HELP",             sw )) { detailed_help(); exit( TERMINATE_NORMAL ); }
         else if (!strcmp( "Help",             sw )) { detailed_help(); exit( TERMINATE_NORMAL ); }
         else if (!strcmp( "help",             sw )) { detailed_help(); exit( TERMINATE_NORMAL ); }
         else if (!strcmp( SW_NAMELIST,        sw )) { i++; /* skip this parameter & its value -- already processed */ }

         /*========== INPUT & OUPTUT FILES ==========*/
         else if (!strcmp( SW_LISTS_IN_NLFILE, sw )) { cs.lists_in_nlfile = TRUE; cs.lists_in_nlfile_f = TRUE; }
         
         else if (!strcmp( SW_DATA_IN_LFILE,   sw )) { i++; strcpy( cs.data_in_lfile,   argv[i] ); cs.data_in_lfile_f  = TRUE; }
         else if (!strcmp( SW_DATA_OUT_FILE,   sw )) { i++; strcpy( cs.data_out_file,   argv[i] ); cs.data_out_file_f  = TRUE; }
         else if (!strcmp( SW_SDEV_IN_LFILE,   sw )) { i++; strcpy( cs.sdev_in_lfile,   argv[i] ); cs.sdev_in_lfile_f  = TRUE; }
         else if (!strcmp( SW_SDEV_OUT_FILE,   sw )) { i++; strcpy( cs.sdev_out_file,   argv[i] ); cs.sdev_out_file_f  = TRUE; }
         else if (!strcmp( SW_FMASK_IN_FILE,   sw )) { i++; strcpy( cs.fmask_in_file,   argv[i] ); cs.fmask_in_file_f  = TRUE; }
         else if (!strcmp( SW_BMASK_IN_LFILE,  sw )) { i++; strcpy( cs.bmask_in_lfile,  argv[i] ); cs.bmask_in_lfile_f = TRUE; }
         else if (!strcmp( SW_PMASK_IN_LFILE,  sw )) { i++; strcpy( cs.pmask_in_lfile,  argv[i] ); cs.pmask_in_lfile_f = TRUE; }
         else if (!strcmp( SW_AMASK_OUT_FILE,  sw )) { i++; strcpy( cs.amask_out_file,  argv[i] ); cs.amask_out_file_f = TRUE; }
         else if (!strcmp( SW_WTS_OUT_FILE,    sw )) { i++; strcpy( cs.wts_out_file,    argv[i] ); cs.wts_out_file_f   = TRUE; }

         /*========== GENERAL SWITCHES ==========*/
         else if (!strcmp( SW_BPASS,          sw )) { i++;  nums = sscanf(argv[i],"%hd",&(cs.bpass));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.bfatal_f         = TRUE;                             }
         else if (!strcmp( SW_BFATAL,         sw )) { i++;  nums = sscanf(argv[i],"%hd",&(cs.bfatal));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.bfatal_f         = TRUE;                             }
         else if (!strcmp( SW_PFATAL,         sw )) { i++;  nums = sscanf(argv[i],"%hd",&(cs.pfatal));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.pfatal_f         = TRUE;                             }
/*       else if (!strcmp( SW_FOV_NUM,        sw )) { i++;  nums = sscanf(argv[i],"%hd",&(cs.fov_num));
**                                                    if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
**                                                    cs.fov_num_f        = TRUE;                             }
**       else if (!strcmp( SW_TRIM_F1,        sw )) { i++;  nums = sscanf(argv[i],"%f", &(cs.trim_f1));
**                                                    if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
**                                                    cs.trim_f1_f        = TRUE;                             }
**       else if (!strcmp( SW_TRIM_F2,        sw )) { i++;  nums = sscanf(argv[i],"%f", &(cs.trim_f2));
**                                                    if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
**                                                    cs.trim_f2_f        = TRUE;                             }
**       else if (!strcmp( SW_CR_THRESH,      sw )) { i++;  nums = sscanf(argv[i],"%f", &(cs.cr_thresh));
**                                                    if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
**                                                    cs.trim_f1_f        = TRUE;                             }
*/       else if (!strcmp( SW_AMASK_MIN_GOOD, sw )) { i++;  nums = sscanf(argv[i],"%d", &(cs.amask_min_good));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.amask_min_good_f = TRUE;                             }
         else if (!strcmp( SW_HEADER_PROP,    sw )) { cs.header_prop      = TRUE; 
                                                      cs.header_prop_f    = TRUE;                             }
         else if (!strcmp( SW_USE_WEIGHTS,    sw )) { i++;  nums = sscanf(argv[i],"%d", &flagval);
                                                      cs.use_weights      = (Flag)flagval;
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.use_weights_f    = TRUE;                             }
         else if (!strcmp( SW_CUT_FRAC,       sw )) { i++;  nums = sscanf(argv[i],"%f", &(cs.cut_frac));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.cut_frac_f        = TRUE;                            }
         else if (!strcmp( SW_CUT_MULTIPLE,   sw )) { i++;  nums = sscanf(argv[i],"%f", &(cs.cut_multiple));
                                                      if (nums!=1) print_err_cmd(i-1,argv[i-1],cs);
                                                      cs.cut_multiple_f    = TRUE;                            }
 
         else printf("\nWARNING: Unknown switch %1d: \"%s\" IGNORED.\n\n", i, argv[i]) /* DEFAULT CASE */;           

      } /* end else */

   }
   return( cs );
      
}


/*******************************************************************************
**  COUNT FILENAMES IN A FILE, POSSIBLY BETWEEN DELIMITERS
**  Return the first filename to be examined as a sample
*/
int count_filenames( char *filelist, Flag lists_in_nlfile, char *start_str, char *end_str, char *firstfile)
{
int     num_filenames;
FILE	*fptr;
char	dataline[NL_MAXLINE], token[NL_MAXLINE], filename[NL_MAXLINE];
int     rv; /* returned value */
Flag    found;

   num_filenames = 0;
   
   /* OPEN THE FILELIST FILE */
   #if DEBUG_CS_STUFF 
      printf("count_filenames() -- PROCESS FILELIST: \"%s\"\n", filelist );
   #endif
   fptr = fopen( filelist, "r" );
   if (!fptr) {
      printf("\aERROR opening FILELIST file: \"%s\"", filelist);
      printf(" in routine count_filenames()\n");
      exit( TERMINATE_ERROR );
   }
      
   if (lists_in_nlfile)  {   /* SKIP TO AFTER THE START MARKER IF NECESSARY */ 

      /* GET FIRST start_str TOKEN ("&COADD_DATA" for the datafile list)*/
      #if DEBUG_CS_STUFF 
         printf("Reading data filenames from NAMELIST...\n");
      #endif
      while ( fgets(dataline, NL_MAXLINE, fptr) ) {
         rv = sscanf( dataline, "%s", token ); /* Read 1st token on the line */
         if (rv!=1) continue;
         str2uc( token );
         found = !strcmp( start_str, token );
         if (found) break;
      }
      if (!found) {
         printf("CAUTION: MISKAKE IN NAMELIST, %s -- \"%s\" not found.\n", 
                                             filelist, start_str );
         printf("         IGNORING NAMELIST. \n\n");
         return;
      }
      #if DEBUG_CS_STUFF 
         printf("FOUND: \"%s\"\n", start_str );
      #endif
      
   }

   /* GET DATA UNTIL end_str TOKEN is reached, or EOF ("&END" for the datafile list) */
   while ( fgets( dataline, NL_MAXLINE, fptr ) ) {
      rv = sscanf( dataline, "%s", token ); /* Read 1st token on the line */
      if (rv<=0) continue;  /* Skip a blank line */
      str2uc( token );
      found = !strcmp( end_str, token );
      #if DEBUG_CS_STUFF 
         printf("rv=%1d, TOKEN:<%s>  LINE:<%s>", rv, token, dataline );
      #endif
      if (found) break;
      if (token[0]=='#') continue;  /* Marks a comment */

      rv = sscanf( dataline, "%s", filename );
      if (rv!=1) {
         printf("CAUTION: Malformed line in NAMELIST IGNORED: \"%s\"\n",
                  dataline);
         continue;
      }
      else {  /* Good filename found */
         if (0) printf( "**********  filename = \"%s\"\n", filename );  
         num_filenames++;
         if (num_filenames==1) strcpy( firstfile, filename );
      }
   }
   if (!found) {
      if (lists_in_nlfile) {    
          printf("CAUTION: MISTAKE IN NAMELIST, %s -- \"%s\" not found.\n", 
                                             filelist, end_str );
          printf("         IGNORING LACK OF \"%s\". \n\n", end_str);
          return;
      }
      /* else separate list, no beginning or end markers -- so no problem */
   }
   #if DEBUG_CS_STUFF 
      else printf("FOUND: \"&END\" ==> END OF NAMELIST PROCESSING\n\n");
   #endif

   fclose( fptr );
   return( num_filenames );
   
}

/*******************************************************************************
**  READ FILENAMES IN A FILE, POSSIBLY BETWEEN DELIMITERS
*/
void read_filenames( char *filelist, Flag lists_in_nlfile, 
                     files_struct *filenames, int num_files, 
                     char *start_str, char *end_str            )
{
int     i;
FILE	*fptr;
char	dataline[NL_MAXLINE], token[NL_MAXLINE], filename[NL_MAXLINE];
int     rv; /* returned value */
Flag    found;

   /* OPEN THE FILELIST FILE */
   #if DEBUG_CS_STUFF 
      printf("read_filenames() -- PROCESS FILELIST: \"%s\"\n", filelist );
   #endif
   fptr = fopen( filelist, "r" );
   if (!fptr) {
      printf("\aERROR opening FILELIST file: \"%s\"", filelist);
      printf(" in routine read_filenames()\n");
      exit( TERMINATE_ERROR );
   }
      
   if (lists_in_nlfile)  {   /* SKIP TO AFTER THE START MARKER IF NECESSARY */ 

      /* GET FIRST start_str TOKEN ("&COADD_DATA" for the datafile list)*/
      #if DEBUG_CS_STUFF 
         printf("Reading data filenames from NAMELIST...\n");
      #endif
      while ( fgets(dataline, NL_MAXLINE, fptr) ) {
         rv = sscanf( dataline, "%s", token ); /* Read 1st token on the line */
         if (rv!=1) continue;
         str2uc( token );
         found = !strcmp( start_str, token );
         if (found) break;
      }
      if (!found) {
         printf("CAUTION: MISTAKE IN NAMELIST, %s -- \"%s\" not found.\n", 
                                             filelist, start_str );
         printf("         IGNORING NAMELIST. \n\n");
         return;
      }
      #if DEBUG_CS_STUFF 
         printf("FOUND: \"%s\"\n", start_str );
      #endif
      
   }

   /* GET DATA UNTIL end_str TOKEN is reached, or EOF ("&END" for the datafile list) */
   i = 0;
   while ( fgets( dataline, NL_MAXLINE, fptr ) ) {
      rv = sscanf( dataline, "%s", token ); /* Read 1st token on the line */
      if (rv<=0) continue;  /* Skip a blank line */
      str2uc( token );
      found = !strcmp( end_str, token );
      #if DEBUG_CS_STUFF 
         printf("i=%01d, rv=%1d, TOKEN:%s  LINE:%s", i, rv, token, dataline );
      #endif
      if (found) break;
      if (token[0]=='#') continue;  /* Marks a comment */

      rv = sscanf( dataline, "%s", filename );
      #if DEBUG_CS_STUFF 
         printf("i=%01d, rv=%1d, FNAME:%s  LINE:%s", i, rv, filename, dataline );
      #endif
      if (rv!=1) {
         printf("CAUTION: Malformed line in NAMELIST IGNORED: \"%s\"\n",
                  dataline);
         continue;
      }
      else {  /* Good filename found */
         #if DEBUG_CS_STUFF 
            printf( "**********  filename = \"%s\"\n", filename );  
         #endif
         strcpy( filenames[i].v, filename );
         i++;
      }
   }
   
   if (!found) {
      if (lists_in_nlfile) {    
          printf("CAUTION: MISTAKE IN NAMELIST, %s -- \"%s\" not found.\n", 
                                             filelist, end_str );
          printf("         IGNORING LACK OF \"%s\". \n\n", end_str);
          return;
      }
      /* else separate list, no beginning or end markers -- so no problem */
   }
   #if DEBUG_CS_STUFF 
      else printf("FOUND: \"&END\" ==> END OF NAMELIST PROCESSING\n\n");
   #endif
   
   if (i!=num_files) printf("\aWARNING:  FILELIST %s has %0d filenames, but should have %0d\n",
                                         filelist,        i,                         num_files );
   fclose( fptr );

}

/*******************************************************************************
**  FIND MARKERS IN NAMELIST FILE
*/
void find_markers( char *nl, Flag *sdev_in_nl, Flag *bmask_in_nl, Flag *pmask_in_nl )
{
int     i;
FILE	*fptr;
char	dataline[NL_MAXLINE], token[NL_MAXLINE], filename[NL_MAXLINE];
int     rv; /* returned value */
Flag    found;

   /* OPEN THE NAMELIST FILE */
   #if DEBUG_CS_STUFF 
      printf("find_markers() -- PROCESS NAMELIST: \"%s\"\n", nl );
   #endif
   fptr = fopen( nl, "r" );
   if (!fptr) {
      printf("\aERROR opening NAMELIST file: \"%s\"", nl);
      printf(" in routine find_markers()\n");
      exit( TERMINATE_ERROR );
   }
      
   /* FIND THE MARKERS */
   #if DEBUG_CS_STUFF 
      printf("Reading data filenames from NAMELIST...\n");
   #endif
   *sdev_in_nl=0; *bmask_in_nl=0; *pmask_in_nl=0;
   while ( fgets(dataline, NL_MAXLINE, fptr) ) {
      rv = sscanf( dataline, "%s", token ); /* Read 1st token on the line */
      if (rv!=1) continue;
      str2uc( token );
      *sdev_in_nl  |= !strcmp( SDEV_MARKER,  token );
      *bmask_in_nl |= !strcmp( BMASK_MARKER, token );
      *pmask_in_nl |= !strcmp( PMASK_MARKER, token );
   }
   #if DEBUG_CS_STUFF 
      printf("SDEV_MARKER  %s FOUND.\n", sdev_in_nl  ? "   " : "NOT" );
      printf("BMASK_MARKER %s FOUND.\n", bmask_in_nl ? "   " : "NOT" );
      printf("PMASK_MARKER %s FOUND.\n", pmask_in_nl ? "   " : "NOT" );
   #endif
   
   fclose( fptr );

}

/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@                                   @@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@   WEIGHTED TRIMMED AVERAGE CODE   @@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@                                   @@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

/*******************************************************************************
**  Calculate calc_weighted_stats()
**
**  From: FUNDAMENTAL FORMULAS OF PHYSICS, edited by Donald H. Menzel (1960) p. 113
**  And:  STATISTICAL TREATMENT OF EXPERIMENTAL DATA, by Hugh D. Young (1962) p. 17
**
**  Let  mu_wtd        be the   weighted average
**  and  sigma_wtd     be the   weighted standard deviation
**  and  regular_mean  be the unweighted average
**  and  sigma_0       be the unweighted standard deviation
**  Then:
**              n                                                  [   n         ]
**            SIGMA w_i * x_i                                  SQRT[ SIGMA w_i^2 ]
**             i=1                                                 [  i=1        ]
**  mu_wtd = -----------------   AND   sigma_wtd = sigma_0 * -----------------------
**                 n                                                   n
**               SIGMA w_i                                           SIGMA w_i
**                i=1                                                 i=1
**
**
**                      1    n               
** with  sigma_0^2  =  --- SIGMA (x_i^2)  -  regular_mean^2
**                      N   i=1     
**    
**     
**                        1    n
** and  regular_mean  =  --- SIGMA x_i
**                        N   i=1
**
*/
void calc_weighted_stats( data_struct *data, 
                          int data_start,  int data_count,
                          float *wtd_mean, float *wtd_sdev,    int dbg )
{                          
int   i,j;
double w_sum, w2_sum, wx_sum, x2_sum;
double x_sum, regular_mean; /* for diagnostic output */
float sigma_0;

   /* Initialize all the sums */
   w_sum  = 0; w2_sum = 0; wx_sum = 0;
   x_sum  = 0; x2_sum = 0;

   /* Have i and j count in synchrony */   
   for ( i=data_start,j=0;  j<data_count; i++,j++ ) {
      /* if (1) printf("Summing data[%1d]=%10.3f with weight=%13.3f\n",i,data[i],weights[i]); */
      w_sum  += (double) data[i].weight;
      x_sum  += (double) data[i].value;
      wx_sum += (double) data[i].weight * (double) data[i].value;
      w2_sum += (double) data[i].weight * (double) data[i].weight;
      x2_sum += (double) data[i].value  * (double) data[i].value;
   }
   
   regular_mean =  (float)( x_sum / data_count ); 
   *wtd_mean    =  (float)( wx_sum / w_sum );
   sigma_0      =  (float)sqrt( (x2_sum/data_count) - (regular_mean*regular_mean) );
   *wtd_sdev    =  sigma_0 * (float)( sqrt( w2_sum ) / w_sum );
   
   #if DEBUG_CALC_WEIGHTED_STATS
      /* for diagnostic output */
      if (1) printf("STATS:  mean = %g  wtd_mean = %g  sdev = %g  wtd_sdev = %g\n",
                     regular_mean,     *wtd_mean,      sigma_0,  *wtd_sdev         );
   #endif
}      
   

/*******************************************************************************
** FLOAT Compare routine for use with qsort() -- compares floating point values
*/
int flt_compare( float *a, float *b )
{
   if (*a <  *b)     return -1;
   if (*a >  *b)     return  1;
   /* (*a == *b) */  return  0;
}


/*******************************************************************************
** wt_data_struct Compare routine for use with qsort() -- compares ".data" values
*/
/* int data_struct_compare( const void *a, const void *b )
** {
**    if ( (*a).value <  (*b).value )     return -1;
**    if ( (*a).value >  (*b).value )     return  1;
**    /@ ( (*a).value == (*b).value ) @/  return  0;
** }
*/

/*******************************************************************************
** wt_data_struct Compare routine for use with qsort() -- compares ".data" values
*/
int data_struct_compare( const void *a, const void *b )
{
   if ( ((data_struct *)a)->value <  ((data_struct *)b)->value )     return -1;
   if ( ((data_struct *)a)->value >  ((data_struct *)b)->value )     return  1;
   /* ( ((data_struct *)a)->value == ((data_struct *)b)->value ) */  return  0;
}


/*******************************************************************************
**  Calculate trimmed_weighted_average()
**
**  NOTE:  The parameters f1 and f2, while defined similarly to those of
**         Russ Laher's trimmedstatistics(), they are defined very simply
**         here, and are NOT IDENTICAL to Russ' parameters.
**
**  f1 is the CENTRAL FRACTION of data used to calculate the average.
**  f2 is the CENTRAL FRACTION of data used to calculate the sdev.
**
**  SET:  f1=0 for median (~trim all), f1=1 for mean (no trimming)
**
** How I define f1:
**
**                   N = 4                                N = 5
**         +-----+-----+-----+-----+        +-----+-----+-----+-----+-----+
** Index i |  0  |  1  |  2  |  3  |        |  0  |  1  |  2  |  3  |  4  |
**         +-----+-----+-----+-----+        +-----+-----+-----+-----+-----+
**         ^     ^  LO ^  HI ^     ^        ^     ^  LO ^     ^  HI ^     ^
**         |     |  ^  |  ^  |     |        |     |  ^  |     |  ^  |     |
**         0%   25% | 50% | 75%   100%      0%   20% | 40%   60% | 80%   100%
**               |  |     |  |                       |  |     |  |
**               |   \___/   |                       |   \___/   |
**               |   f1=25%  |                       |   f1=20%  |
**                \         /                         \         /
**                 \_______/                           \_______/
**                   f1=50%                              f1=40%
**
**
**  N = 4, STEP = 1/N = 25%                 N = 5, STEP = 1/N = 20 % 
**
**         /GOAL\                                  /GOAL\
**  f1(%)  LO  HI   L1(%) L2  LF  f1 range  f1(%)  LO  HI   L1(%) L2  LF  f1 range
**  ======================================  ======================================
**  100     0   3    0     0   0  (50-100]  100     0   4    0     0   0  (60-100]
**   75     0   3   12.5   0   0             80     0   4   10     0   0
**  --------------------------------------  --------------------------------------
**   50     1   2   25     1   1   [0-50]    60     1   3   20     1   1  (20-60]
**   25     1   2   37.5   1   1             40     1   3   30     1   1
**    0     1   2   50     2   1            -----------------------------
**                                           20     2   2   40     2   2   [0-20]
**                                            0     2   2   50     2   2
**                                          -----------------------------
** 
**        L1 = (1-f1)/2                                          == lo_1 
**        L2 = INT( L1/STEP ) = INT( L1*N ) = INT[ N*(1-f1)/2 ]  == lo_2
**        LF = MAX{ L2, INT[ (N-1)/2 ] }                         == lo_f
**     so LF == LO as desired.        
**        HF == N - LF - 1                                       == hi_f
**        N                                                      == data_count
*/
void trimmed_weighted_average( data_struct *data, int data_count,
                               float f1,    float f2, 
         /* Return values: */  float *mean, float *sdev, 
                               int   dbg                                    )
{
int    i;
int    lo_2, lo_f, hi_f, max_i, new_n;
float  flt_NaN;
float  dummy_mean;
  
   sscanf("NaN", "%f",  &flt_NaN);  /* THIS IS THE GOOD FLOAT NaN TO USE!!! */

   if (data_count<1) {
      *mean = flt_NaN;
      *sdev = flt_NaN;
      return;
   }
   else if (data_count==1) {
      *mean = data[0].value;
      *sdev = 0.0;
   }   

   #if DEBUG_TRIMMED_WEIGHTED_AVERAGE
      printf("Unsorted data:\n");
      for (i=0; i<data_count; i++) {
         printf("D%1d=%6.2f  ",i,data[i]);
      }  
      printf("\n");
   #endif

   qsort( (void *)data, data_count, sizeof(data_struct), data_struct_compare );
   
   #if DEBUG_TRIMMED_WEIGHTED_AVERAGE
      printf("Sorted data:\n");
      for (i=0; i<data_count; i++) {
         printf("D%1d=%6.2f  ",i,data[i]);
      }
      printf("\n");
   #endif
   
   /* TO CALCULATE THE STARTING AND ENDING INDICES INTO THE ARRAY USING F1 */
   lo_2  = (int)( (float)data_count * (1-f1) / 2.0 );
   max_i = (int)( (data_count-1)/2 );
   lo_f  = ( (lo_2 < max_i) ? lo_2 : max_i );   /* lo_f = MIN(lo_2,max_i) */
   hi_f  = data_count - lo_f - 1;
   new_n = hi_f - lo_f + 1;     /* number of data points in trimmed array */
   
   #if DEBUG_TRIMMED_WEIGHTED_AVERAGE
      printf("F1:  lo_2=%1d   max_i=%1d   lo_f=%1d   hi_f=%1d   new_n=%1d\n",
                   lo_2,      max_i,      lo_f,      hi_f,      new_n         );
      printf("Trimmed data using f1=%g (new_n=%1d points):\n", f1, new_n);
      for (i=lo_f; i<=hi_f; i++) {
         printf("D%1d=%6.2f  ",i,data[i]);
      }
      printf("\n");
   #endif
   
   calc_weighted_stats( data, lo_f, new_n, mean,  sdev, dbg );   
   
   /* TO CALCULATE THE STARTING AND ENDING INDICES INTO THE ARRAY USING F2 */
   lo_2  = (int)( (float)data_count * (1-f2) / 2.0 );
   max_i = (int)( (data_count-1)/2 );
   lo_f  = ( (lo_2 < max_i) ? lo_2 : max_i );   /* lo_f = MIN(lo_2,max_i) */
   hi_f  = data_count - lo_f - 1;
   new_n = hi_f - lo_f + 1;     /* number of data points in trimmed array */
   #if DEBUG_TRIMMED_WEIGHTED_AVERAGE
      if (0) printf("F2:  lo_2=%1d   max_i=%1d   lo_f=%1d   hi_f=%1d   new_n=%1d\n",
                          lo_2,      max_i,      lo_f,      hi_f,      new_n         );
       printf("Trimmed data using f2=%g (new_n=%1d points):\n", f2, new_n);
      for (i=lo_f; i<=hi_f; i++) {
         printf("D%1d=%6.2f  ",i,data[i]);
      }
      printf("\n");
   #endif
   
   /* Use dummy_mean this time, preserving the previous mean  */
   /* while calculating the desired new sdev value            */
   calc_weighted_stats( data, lo_f, new_n, &dummy_mean,  sdev, dbg );
   
}


/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@                                  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@     ASYMMETRIC TRIMMING CODE     @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@                                  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

/*===========================================================================
** Find the MEDIAN of an array in O(n) time on *average*
** 
**   FROM http://www.ics.uci.edu/~eppstein/161/960125.html 
**   PSEUDOCODE TO SELECT THE kTH SMALLEST ELEMENT OF ARRAY L:
** 
**            quickselect(L,k)
**            {
**               pick x in L
**               partition L into L1<x, L2=x, L3>x
**               if (k <= length(L1))
**                   return quickselect(L1,k)
**               else if (k > length(L1)+length(L2))
**                   return quickselect(L3,k-length(L1)-length(L2))
**               else return x
**            }
**
**
**   RANDOM FUNCTION drand48() and srand48() NOTES:
**
**      Function drand48() returns non-negative double-precision
**      floating-point values uniformly distributed over the
**      interval [0.0, 1.0).
**
**      Function srand48() is an initialization entry  point,  which
**      should be invoked before drand48() is called.
**      (Although it is not recommended practice, constant default
**      initializer values will be supplied automatically  if drand48()
**      is called without a prior call to an initialization entry point.)
**===========================================================================
*/
data_struct quickselect(data_struct *L, int len, int k, cmd_struct cs )
{
int          i,len1,len2,len3;
data_struct  x;
data_struct  *L1, *L2, *L3;  /* Pointers to arrays */

   /********************************************************************
   ** Create a new arrays to use for storage of medians 
   ** (this *could* be optimized to be more storage-efficient, but I
   ** didn't bother)
   */
   /* if (cs.debug) printf("Allocate memory for local median values storage\n"); */
   L1 = (data_struct *) malloc( (unsigned) ( len * sizeof(data_struct) ) );
   L2 = (data_struct *) malloc( (unsigned) ( len * sizeof(data_struct) ) );
   L3 = (data_struct *) malloc( (unsigned) ( len * sizeof(data_struct) ) );
   if ( (L1 == NULL) || (L2 == NULL) || (L3 == NULL) ) {
      printf("ERROR: %s: Can't allocate required memory for medians in quickselect()\n",cs.progname);
      exit( TERMINATE_ERROR );
   }
   
   /******************************************************************
   ** PSEUDOCODE: pick x in L 
   */
   i = (int)( len * drand48() );  /* A random integer in [0..len-1] */
   x.value = L[i].value;  x.plane = L[i].plane;
   /* if (cs.debug) printf("Chose L[%1d] = x = %g\n", i, x.value); */

   /******************************************************************
   ** PSEUDOCODE: partition L into L1<x, L2=x, L3>x 
   */
   len1=0; len2=0; len3=0;
   for( i=0; i<len; i++ ) {
      if      ( L[i].value <  x.value )  { L1[len1].value = L[i].value;  L1[len1].plane = L[i].plane; len1++; }
      else if ( L[i].value >  x.value )  { L3[len3].value = L[i].value;  L3[len3].plane = L[i].plane; len3++; }
      else   /* L[i].value == x.value */ { L2[len2].value = L[i].value;  L2[len2].plane = L[i].plane; len2++; }
   }
   
   /******************************************************************
   ** PSEUDOCODE: 
   ** if (k <= length(L1))
   **    return quickselect(L1,k)
   ** else if (k > length(L1)+length(L2))
   **    return quickselect(L3,k-length(L1)-length(L2))
   ** else return x
   */

   #if DEBUG_QUICKSELECT
      if      (k <= len1      ) printf("RECURSE: L1, <len1=%02d> len2=%02d  len3=%02d  k=%02d  x=%g\n",len1,len2,len3,k,x);
      else if (k >  len1+len2 ) printf("RECURSE: L3,  len1=%02d  len2=%02d <len3=%02d> k=%02d  x=%g\n",len1,len2,len3,k-len1-len2,x);
      else                      printf("RETURN:  x=%g\n",x);
   #endif
   
   if      (k <= len1      ) { free(L2); free(L3);
                               return quickselect(L1, len1, k, cs );
                             }
   else if (k >  len1+len2 ) { free(L1); free(L2);
                               return quickselect(L3, len3, k-len1-len2, cs );
                             }
   else                      { free(L1); free(L2); free(L3);
                               return x;
                             }

}



/*===========================================================================
** Find the MEDIAN of a 1D array of data values (in a data_struct)
**
** requires routine quickselect()
*/
data_struct find_median( data_struct *data, int ndata, cmd_struct cs )
{
data_struct m_lo, m_hi, median;

   /***********************************************************************
   ** Initialize the random number generator used in quickselect()
   */
   srand(999);
   
   /***********************************************************************
   ** Find the median of these data
   */
   if (ndata%2==1) {  /* ndata is ODD */
      median = quickselect( data, ndata, (ndata+1)/2, cs );
   } else {           /* ndata is EVEN */ 
      m_lo = quickselect( data, ndata,  ndata/2,     cs );
      m_hi = quickselect( data, ndata, (ndata/2)+1 , cs );
      median.value = ( m_lo.value + m_hi.value ) / 2;
      /* There is no way to set the median plane pointer to the correct place, so just arbitrarily pick the lowest plane: */
      median.plane =   m_lo.plane;
   }

   /***********************************************************************
   ** Return median
   */
   return median;


}


/***********************************************************************************************
** Asymmetric Trimming Algorithm
**
** Adapted from the algorithm specification in the document:
** DARKCAL SDS, by John Fowler, 14 June 2002, Version 1.12
**
**
** Instructions from the SDS (minorly reworded):
**
** For simplicity, let p_k represent the value for a given pixel in subframe k, where 
** there are N_k accepted frames in the sequence.  The asymmetrical trimming fraction 
** is F_2a, so that N_a = N_k( 1 - F_2a ) values may be rejected.  The algorithm is as 
** follows:
**
** (A)  Find the median of the current set of pixel velues {p_k}; denote this as p_med.
**
** (B)  Determine which of the two extreme values of {p_k} is more distant from p_med;
**      denote this as p_extreme and its absolute distance from p_med as D_extreme.
**
** (C)  Excluding p_extreme and p_med, compute the absolute distances of all the other p_k
**      values from p_med, and find the median value of these distances, denoted as D_med.
**
** (D)  If D_extreme < D_fac * D_med, where D_fac is a controllable parameter (default 5.0),
**      then the current set of p_k values is acceptable, and this processing terminates;
**      otherwise reject p_extreme from the set of p_k values, and if fewer than N_a passes
**      through this logic have been completed, go back to step (A) and continue, otherwise
**      terminate this processing.
**
** Rearranging and optimizing:
**
** (B')  Compute the absolute distances D_abs_k of all the p_k values from p_med.  Denote the 
**       largest absolute distance D_extreme (the corresponding point as p_extreme can be
**       inferred from D_extreme).
**
** (C')  "Excluding p_extreme and p_med, and find the median value of these (absolute) 
**       distances, denoted as D_med."  BUT since p_extreme has the largest D_abs_k and 
**       p_med has D_abs_k=0 one can compute the median WITHOUT excluding these two points.
**       So, merely find the median of the D_abs_k, denoted as D_med.
**
** In my code:
**
**         k         -> i (ith data array image)
**         p_k       -> pixel[k]
**         N_k       -> num_data
**         F_2a      -> keep_frac   (== fraction of data values to KEEP)
**         1 - F_2a  -> cs.cut_frac (== fraction of data values to CUT)
**         N_a       -> max_num_rejected
**         D_abs_k   -> abs_delta
**         p_med     -> pixel_median
**         p_extreme -> pixel_extreme     NOTE: not used
**         D_extreme -> delta_extreme     NOTE: all deltas are absolute
**         D_med     -> median_of_deltas  
**         D_fac     -> cs.cut_multiple
**
** This code operates on a single "data column" (the pixel array), and assumes that all 
** NaNs have already been removed from it.  This pixel array will be updated by deleting
** ("trimming") extreme pixel values as descibed by the above algorithm (removing the
** value and compressing the array).  The number of remaining pixels will be stored back
** into num_data.
**
**
** USES cs elements:  cs.cut_frac
**                    cs.cut_multiple
*/
void asymmetric_trimming( data_struct *pixel, int *data_count, cmd_struct cs )
{
int          i;
int          num_data;          /* Update num_data as points are rejected. */
int          num_rejected, max_num_rejected;
data_struct  *abs_delta;
data_struct  pixel_median, delta_extreme, median_of_deltas;
int          delta_extreme_index;  /* Index where delta_extreme is found, in the already trimmed array */


   if ( *data_count <= 3 ) return;  /* Don't bother trimming for 3 or less pixels */

   num_data = *data_count;                  /* max_num_rejected will stay fixed, while num_data may vary */
   abs_delta = (data_struct *) malloc( (unsigned) ( num_data * sizeof(data_struct) ) );
   num_rejected = 0;                         
   max_num_rejected = ( (float) num_data * cs.cut_frac );
   #if DEBUG_ASYMMETRIC_TRIMMING
      if (cs.debug) printf( "######## num_data = %1d, cs.cut_frac = %g  ==>  max_num_rejected = %1d\n", 
                             num_data,       cs.cut_frac,           max_num_rejected           );
   #endif
   
   for ( num_rejected = 0; num_rejected<max_num_rejected; num_rejected++ ) {

      /*========== STEP (A)  ==========*/

      pixel_median = find_median( pixel, num_data, cs );
      #if DEBUG_ASYMMETRIC_TRIMMING
         if (cs.debug) printf( "###########################################################################\n" ); 
         if (cs.debug) printf( "######## STEP (A), iteration %2d, pixel_median = %5.1f, num_data=%1d\n", 
                                                   num_rejected,  pixel_median.value,   num_data        ); 
      #endif
 
      /*========== STEP (B') ==========*/

      delta_extreme.value = -1;
      delta_extreme.plane = -1;
      delta_extreme_index = -1;
      for ( i=0; i<num_data; i++ ) {   
         abs_delta[i].value = fabs( pixel[i].value - pixel_median.value );
         abs_delta[i].plane = pixel[i].plane;
         if ( abs_delta[i].value > delta_extreme.value ) {  
            delta_extreme.value = abs_delta[i].value; 
            delta_extreme.plane = abs_delta[i].plane;
            delta_extreme_index = i;
         }
      #if DEBUG_ASYMMETRIC_TRIMMING
         if (cs.debug) printf( "######## p[%02d]=%5.1f  abs_delta = %5.1f     delta_extreme.value = %5.1f delta_extreme_index = %2d\n", 
                                         i, pixel[i].value, abs_delta[i].value, delta_extreme.value, delta_extreme_index            );
      #endif
      }

      /*========== STEP (C') ==========*/

      median_of_deltas = find_median( abs_delta, num_data, cs );
      #if DEBUG_ASYMMETRIC_TRIMMING
         if (cs.debug) printf( "######## median_of_deltas = %g\n", median_of_deltas.value ); 
      #endif

      /*========== STEP (D)  ==========*/
      
      #if DEBUG_ASYMMETRIC_TRIMMING
         if (cs.debug) if ( delta_extreme.value < cs.cut_multiple * median_of_deltas.value ) 
                            printf( "######## cs.cut_multiple=%5.1f exceeded:   =====BREAK=====\n", cs.cut_multiple);  
         #endif
          
      if ( delta_extreme.value < cs.cut_multiple * median_of_deltas.value ) break;

      /* Delete pixel_extreme from the array */
      if (delta_extreme_index <= num_data - 2) {  /* If delta_extreme_index == num_data-1, then don't need to move any values */
         #if DEBUG_ASYMMETRIC_TRIMMING
            if (cs.debug) printf( "######## Pixel p[%02d]=%5.1f deleted.  abs_delta=%5.1f.\n", 
                                  delta_extreme_index, pixel[delta_extreme_index].value, abs_delta[delta_extreme_index].value ); 
         #endif
         for ( i=delta_extreme_index; i<num_data-1; i++ ) {
            pixel[i].value = pixel[i+1].value;
            pixel[i].plane = pixel[i+1].plane;
         }
      }
      #if DEBUG_ASYMMETRIC_TRIMMING
         else if (cs.debug) printf( "######## NO Pixel deleted.\n");
      #endif
      num_data--;
            
   } 
   
   *data_count = num_data;   /* Return updated num_data */
   #if DEBUG_ASYMMETRIC_TRIMMING
      if (cs.debug) printf( "###########################################################################\n" ); 
      if (cs.debug) printf( "######## Final num_data = %1d\n", num_data );
   #endif

}









