/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@@
@@ PROGRAM:  fits_stuff.c
@@
@@ PURPOSE:  Routines to perform various operations on FITS files
@@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <sys/param.h>
#include "fitsio.h"
#include "coadd2d.h"

/********************************************************************
** Read in the input mask FITS files -- bmask, pmask
*/
void malloc_failed( char *array_name )
{
   printf("\aERROR: Can't allocate required memory for %s\n", 
           array_name);
   exit( TERMINATE_ERROR );
}


/********************************************************************
** Create the amask array; initialize it to 0;
*/
void create_amask( cmd_struct *cs )
{
int i;
   
   printf("\aPROGRAMMING ERROR CALLING create_amask() -- doesn't exist!\n");

#if(0) 
   cs->amask = (short int *) malloc( (unsigned) ( (cs->xdim)*(cs->ydim) * sizeof(short int)  ) );
   if (cs->amask == NULL) {
      printf("\aERROR: create_amask: Can't allocate required memory for short_image\n");
      exit( TERMINATE_ERROR );
   }
   if (cs->debug) printf("Allocated amask image array.\n");
 
   for (i=0; i<(cs->xdim*cs->ydim); i++) cs->amask[i]=0;    /* Initialization */
   
   if (cs->debug) printf("Initialized amask image array.\n");
#endif

}

   
/********************************************************************
** Process maskbits that come from other masks ("external")
*/
void process_external_maskbits( cmd_struct cs )
{
int i;

/*
   if (!cs.cmask_f) {
      if (cs.verbose) printf("No external maskbits processed, since no PMASK was specified\n");
      return;
   }

   for (i=0; i<(cs.xdim*cs.ydim); i++) {
      if ( cs.pm[i] & PM_VAR_DARK_CUR    ) cs.cm[i] |= CM_PIXEL_OTHER_BAD;
      if ( cs.pm[i] & PM_TOO_VARIABLE    ) cs.cm[i] |= CM_PIXEL_OTHER_BAD;
      if ( cs.pm[i] & PM_FAST_SATURATION ) cs.cm[i] |= CM_PIXEL_SATURATED;
      if ( cs.pm[i] & PM_PIXEL_HOT       ) cs.cm[i] |= CM_PIXEL_HOT;
      if ( cs.pm[i] & PM_PIXEL_DEAD      ) cs.cm[i] |= CM_PIXEL_DEAD;
   }
*/

}


/****************************************************************************
** Obtain the dimensions of a 2D FITS file
**    Also adapted from the cfitsio imarith.c routine cited below
*/
void read_fits_dims_2d( char *fits_filename, int *xdim, int *ydim, Flag dbg )
{
fitsfile *infptr;   /* FITS file pointer */
int	 status = 0, anaxis, ii;
long	 npixels = 1;
long	 anaxes[] = {1,1,1,1,1,1,1,1,1,1};
double	 apix[10000];

   if (dbg) printf("===> read_fits_dims_2d() from \"%s\" using cfitsio:\n",fits_filename);
     
   fits_open_file(&infptr, fits_filename, READONLY, &status);
   if (status) {
      printf( "\aERROR opening FITS file %s in read_fits_dims_2d()\n", fits_filename );
      printf( "returned status=%d\n", status );
      exit( TERMINATE_ERROR );
   }   
   
   fits_get_img_dim(infptr, &anaxis, &status);
   fits_get_img_size(infptr, 10, anaxes, &status);

   /* Multiply up the dimensions to get the numbers of pixels */
   for (ii = 0; ii < 10; ii++) npixels *= anaxes[ii];
   *xdim = anaxes[0]; 
   *ydim = anaxes[1]; 
   
   if (dbg) {
      printf("     xdim =%5d\n", *xdim);
      printf("     ydim =%5d\n", *ydim);
   }
    
   if ( npixels != (*xdim)*(*ydim) ) {
      printf("\aERROR: more than two dimensions in file %s [read_fits_dims_2d()]\n", fits_filename);
      printf("       xdim=%1d, ydim=%1d, anaxes[2]=%1d, npixels=%1d\n",
                     xdim,     ydim,     anaxes[2],     npixels         );
      exit( TERMINATE_ERROR );
   }


}


/****************************************************************************
** Read two parameters from the FMASK FITS file:  USEFOVID, FOVID
** (may upgrade later to include channelID, if desired)
**
** NO LONGER NECESSARY since FOVLIST and BITLIST were created.
*/
void read_fmask_params( char *fits_filename, cmd_struct *cp )
{
fitsfile *infptr;   /* FITS file pointer */
int	 status = 0;

   if (cp->debug) printf("===> read_fmask_params() from \"%s\" using cfitsio:\n",fits_filename);
     
   fits_open_file( &infptr, fits_filename, READONLY, &status );
   if (status) {
      printf( "\aERROR opening FMASK FITS file %s in read_fmask_params()\n", fits_filename );
      printf( "returned status=%d\n", status );
      exit( TERMINATE_ERROR );
   }   
   
   fits_read_key( infptr, TSHORT, "USEFOVID",  &(cp->usefovid),  NULL, &status );
   if (status) {
      printf( "\aERROR: Required keyword USEFOVID not found in FITS file %s in read_fmask_params()\n", fits_filename );
      printf( "returned status=%d\n", status );
      exit( TERMINATE_ERROR );
   }   
   
/* fits_read_key( infptr, TSHORT, "FOVID",     &(cp->fov_num),   NULL, &status );
** if (status) {
**    printf( "\aERROR: Required keyword FOVID not found in FITS file %s in read_fmask_params()\n", fits_filename );
**    printf( "returned status=%d\n", status );
**    exit( TERMINATE_ERROR );
** }
*/

/* fits_read_key( infptr, TSHORT, "channelID", &(cp->channelID), NULL, &status );
** if (status) {
**    printf( "\aERROR: Required keyword channelID not found in FITS file %s in read_fmask_params()\n", fits_filename );
**    printf( "returned status=%d\n", status );
**    exit( TERMINATE_ERROR );
** }   
*/  
   cp->usefovid_f  = TRUE;
/* cp->fov_num_f   = TRUE;
** cp->channelID_f = TRUE;
*/   
   if (cp->verbose) { 
      printf("In fmask file %s, read keyword:  ", fits_filename );
      printf("   USEFOVID = %2d\n", cp->usefovid  );
/*    printf("      FOVID = %2d\n", cp->fov_num   );
**    printf("  channelID = %2d\n", cp.channelID );
*/
   }
   if (cp->debug) {
      printf(" USEFOVID_f = %2d\n", cp->usefovid_f  );
/*    printf("    FOVID_f = %2d\n", cp->fov_num_f   );
*/ }

}


/****************************************************************************
** Also adapted from the cfitsio imarith.c routine cited below
** ALLOCATES an array, *image.
** CHANGED TO PRODUCE A float image!
**
** Before first call, set req_fov_num to -1.  After the first call,
** this parameter will hold the FOVID from the first data file.  All subsequent
** FITS data files must then contain the identical FOVID in their headers.
*/
float * read_fits( char *fits_filename, int *xdim, int *ydim, int *req_fov_num, Flag dbg )
{
fitsfile     *infptr;  /* FITS file pointers */
int	     status = 0, anaxis, ii, bitpix;
long	     npixels = 1, firstpix= 1, ntodo;
long	     anaxes[] = {1,1,1,1,1,1,1,1,1,1};
float        *float_image;
float 	     sum, data_min, data_max;
int 	     x, y, xy, NaN_count;
int          fovid_in_file;       /* The FOVID read from the input fits_filename */
const float  flt_NaN = (float)atof("NaN");  /* Easiest way I know to get a NaN -- Thanx Diane! */

   if (dbg) printf("In read_fits() with fits_filename=\"%s\"\n",fits_filename);

   fits_open_file(&infptr, fits_filename, READONLY, &status);
   if (status) {
      printf("\aERROR: Attempt to read file %s FAILED in read_fits().\n", fits_filename); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
   
   /* Ensure that the FOVID in the input file matches req_fov_id, if req_fov_id>0 */
   if (*req_fov_num != -9)  { /* (no FOVID in unc files) */
     fits_read_key( infptr, TINT, "FOVID", &fovid_in_file, NULL, &status );
     if (*req_fov_num < 0) *req_fov_num = fovid_in_file;
     if (status) {
        printf( "\aERROR: Required keyword FOVID not found in FITS file %s [in read_fits()]\n", fits_filename );
        printf( "returned status=%d\n", status );
        exit( TERMINATE_ERROR );
     }   
     if (fovid_in_file != *req_fov_num) {
        printf( "\aERROR: FOVID=%1d in first data file, but FOVID=%1d in FITS file %s [in read_fits()]\n", 
                          *req_fov_num, fovid_in_file, fits_filename );
        printf("       All input data files MUST have identical FOVID numbers\n");
        printf( "returned status=%d\n", status );
        exit( TERMINATE_ERROR );      
     }         
   }

   fits_get_img_type(infptr, &bitpix, &status);
   if (status) {
      printf("\aERROR: Attempt to read bitpix from file %s FAILED in read_fits().\n", fits_filename); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
   
   /* Allow bitpix to be FLOAT_IMG (correct) or DOUBLE_IMG (OK) */
   if ( (bitpix!=FLOAT_IMG) && (bitpix!=DOUBLE_IMG) ) {
      printf("\aERROR: In read_fits() the file %s has incorrect BITPIX:\n",fits_filename);
      printf("\aERROR: BITPIX is %1d ",bitpix);
      printf("but it should be %1d (FLOAT_IMG) ", FLOAT_IMG);
      printf("[%1d (DOUBLE_IMG) is also permitted]\n", DOUBLE_IMG);
      exit( TERMINATE_ERROR );
   }
   if (bitpix!=FLOAT_IMG) {
      printf("\nCAUTION: in file %s BITPIX is %0d, not %0d (FLOAT_IMG) -- will continue anyway.\n\n",
                             fits_filename, bitpix, FLOAT_IMG          );
   }
   else if (dbg) printf("BITPIX is %1d -- GOOD.\n", bitpix);
   
   fits_get_img_dim(infptr, &anaxis, &status);
   fits_get_img_size(infptr, 10, anaxes, &status);
   if (status) {
      printf("\aERROR: Attempt to read size and dims from file %s FAILED in read_fits().\n", fits_filename); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   

   /* Multiply up the dimensions to get the numbers of pixels */
   for (ii = 0; ii < 10; ii++) npixels *= anaxes[ii];
   *xdim = anaxes[0];
   *ydim = anaxes[1];
   if (dbg) printf("Dimensions read as xdim=%1d ydim=%1d in read_fits()\n",*xdim,*ydim);

   if ( npixels != (*xdim)*(*ydim) ) {
      printf("\aERROR: dimensions don't match: file %1d, but arrray %1d in read_fits()\n",npixels,(*xdim)*(*ydim));
      exit( TERMINATE_ERROR );
   }

   float_image = (float  *) malloc( (unsigned) ( (*xdim)*(*ydim) * sizeof(float)  ) );
   if (float_image == NULL) {
      printf("\aERROR: Can't allocate required memory for float_image in read_fits()\n");
      exit( TERMINATE_ERROR );
   }
   if (dbg) printf("Allocated image array.\n");
   
   /* read and write as FLOAT */
   if (!status) {
      fits_read_img(infptr, TFLOAT, firstpix, npixels,
                    NULL, float_image, NULL, &status);
   }
   if (dbg) printf("Reading... COMPLETED.\n");
   
   fits_close_file(infptr, &status);
 
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
   
   /* Check the values from the FITS file */
   if (dbg) {
      printf("In read_fits(), file \"%s\" has:\n", fits_filename );
      data_max = data_min = float_image[0];
      sum = 0;
      NaN_count = 0;
      /* Calculate the average */
      NaN_count = 0;
      for (y=0; y<(*ydim); y++) {
         for (x=0; x<(*xdim); x++) {
            xy = y*(*xdim)+x;
            if ( iznand( (double) float_image[xy] ) ) {
               NaN_count++;
            } else {
               data_min = MIN( data_min, float_image[xy] );
               data_max = MAX( data_max, float_image[xy] );
               sum += float_image[xy];
            }
        }
      }
      printf("    NaN Count     is %1d\n", NaN_count  );
      printf("    Minimum value is %g\n",  data_min  );
      printf("    Maximum value is %g\n",  data_max  );
      printf("    Average value is %g\n",  sum/( ((*xdim)*(*ydim)) - NaN_count)  );
   }
   
   return( float_image  );

}



/****************************************************************************
** Also adapted from the cfitsio imarith.c routine cited below
** ALLOCATES an array of SHORT INT, *short_image.
*/
short int * read_fits_s( char *fits_filename, int *xdim, int *ydim, Flag dbg )
{
fitsfile   *infptr;  /* FITS file pointers */
int	   status = 0, anaxis, i, ii, bitpix;
long	   npixels = 1, firstpix= 1, ntodo;
long	   anaxes[] = {1,1,1,1,1,1,1,1,1,1};
short int  *short_image;
int        fovid_in_file;       /* The FOVID read from the input fits_filename */
int 	   x, y, xy, pt;

   if (dbg) printf("In read_fits_s() with fits_filename=\"%s\"\n",fits_filename);

   fits_open_file(&infptr, fits_filename, READONLY, &status);
   if (status) {
      printf("\aERROR: Attempt to read file %s FAILED in read_fits_s().\n", fits_filename); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   

   fits_get_img_type(infptr, &bitpix, &status);
   /* bitpix MUST BE be SHORT_IMG */
   if (bitpix!=SHORT_IMG) {
      printf("\aERROR: In read_fits_s() the file %s has incorrect BITPIX:\n",fits_filename);
      printf("\aERROR: BITPIX should be %1d but it is %1d\n\n",SHORT_IMG,bitpix);
      exit( TERMINATE_ERROR );
   }
   else if (dbg) printf("BITPIX is %1d -- GOOD.\n", bitpix);
   
   fits_get_img_dim(infptr, &anaxis, &status);
   fits_get_img_size(infptr, 10, anaxes, &status);
   if (status) {
      printf("\aERROR: Attempt to read size and dims from file %s FAILED in read_fits_s().\n", fits_filename); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   

   /* Multiply up the dimensions to get the numbers of pixels */
   for (ii = 0; ii < 10; ii++) npixels *= anaxes[ii];
   *xdim = anaxes[0];
   *ydim = anaxes[1];
   if (dbg) printf("Dimensions read as xdim=%1d ydim=%1d\n",*xdim,*ydim);

   if ( npixels != (*xdim)*(*ydim) ) {
      printf("\aERROR: dimensions don't match: file %1d, but arrray %1d in read_fits_s()\n",npixels,(*xdim)*(*ydim));
      exit( TERMINATE_ERROR );
   }

   short_image = (short int *) malloc( (unsigned) ( (*xdim)*(*ydim) * sizeof(short int)  ) );
   if (short_image == NULL) {
      printf("\aERROR: Can't allocate required memory for short_image in read_fits_s()\n");
      exit( TERMINATE_ERROR );
   }
   if (dbg) printf("Allocated image array.\n");
   
   /* read as SHORT INT */
   if (!status) {
      fits_read_img(infptr, TSHORT, firstpix, npixels, NULL, short_image, NULL, &status);
      if (status) {
         printf("\aERROR: Attempt to read short image from file %s FAILED in read_fits_s().\n", fits_filename); 
         fits_report_error(stderr, status);
         exit( TERMINATE_ERROR );
      }   
   }
   if (dbg) printf("Reading... COMPLETED.\n");
      
   if (dbg) {
      printf("Read in file %s, with sample values as follows:\n",fits_filename);
      if ( (*xdim)*(*ydim) <= 35 ) {
         for(y=0;y<(*ydim);y++) {
            for(x=0;x<(*xdim);x++) {
               xy = ( y * (*xdim) ) + x;
               printf( "s[%1d,%1d]=%5d  ", x,y,short_image[xy] ); 
            }
            printf("\n");
         }
      }
      else {
         for ( i=0,ii=10; i<20; ii+=(*xdim*5),i++ ) {
            printf("s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d  s[%05d]=%3d\n",
                    ii, short_image[ii],      ii+5, short_image[ii+5], 
                    ii+10,short_image[ii+10], ii+15,short_image[ii+15], 
                    ii+20,short_image[ii+20], ii+25,short_image[ii+25],
                    ii+30,short_image[ii+30], ii+35,short_image[ii+35],
                    ii+40,short_image[ii+40], ii+45,short_image[ii+45] );
         }
      }
   }
   
   fits_close_file(infptr, &status);
 
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
   
   return( short_image );

}


/***********************************************************************************
** Reads and digests the FMASK keywords, FOVLIST and BITLIST, sets cs.num_fovs
**
** FROM THE SIS defining the FMASK lists:
**
**    The F-MASK identifies pixels where spectra in each field of view are expected
**    to fall.  The pixels are marked bit-wise so that all FOV's in a module can be
**    identified in a single mask.  The header contains two keywords that explain the
**    mask:
**
**             FOVLIST -- the list of FOVs that are identified in the mask
**             BITLIST -- the bits that are set for pixels in each FOV.
**
**    For example, in SL
**
**             FOVLIST = '[26,27,28,29,32,33,34]'
**             BITLIST = '[1,2,4,8,16,32,64]'
**
**    so any pixel in the mask with the 1 it set is in FOV 26, and any pixel with the
**    8 bit set is in FOV 29.  In practice, FOV 28 and 29 overlap entirely so many
**    pixels have both 4 and 8 set, that is a value of 12.
*/
void read_fmask_lists( cmd_struct *cp )
{
fitsfile   *fptr;  /* FITS file pointers */
int	   status = 0;
int        nums;
 /*  These are standard FITS sizes - TYB 6/1/09 */
char       fovlist_str[71];
char       bitlist_str[71];
char       mycomment[73];
//char       *fovlist_str = "[26,27,28,29,32,34,33]";
//char       *bitlist_str = "[ 1,2, 4 ,8, 16,64,32]  ";
char       *fov_num_str;
char       *bit_num_str;
int        i_fovlist, i_bitlist, i;

   /* Open the FMASK FITS file */
   if (!cp->fmask_in_file_f) {
      printf("ERROR:  Weird programming error in read_fmask_lists() -- FMASK file not defined.\n");
      exit( TERMINATE_ERROR );
   }

   fits_open_file( &fptr, cp->fmask_in_file, READONLY, &status );
   if (status) {
      printf("\aERROR: Attempt to read file %s FAILED in read_fits_s().\n", cp->fmask_in_file ); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
      
   /* Read the FOVLIST and BITLIST character strings from the FITS file */

   /*  This would be the standard way to call this - TYB 6/1/09 */

   fits_read_key( fptr, TSTRING, FOVLIST_KEYWORD, &fovlist_str, mycomment, &status);

   //   fits_read_key( fptr, TSTRING, FOVLIST_KEYWORD, fovlist_str, NULL, &status);

   if (status) {
      printf("\aERROR: unable to find required keyword %s in FITS file %s\n", FOVLIST_KEYWORD, cp->fmask_in_file); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
   fits_read_key( fptr, TSTRING, BITLIST_KEYWORD, bitlist_str, NULL, &status);
   if (status) {
      printf("\aERROR: unable to find required keyword %s in FITS file %s\n", BITLIST_KEYWORD, cp->fmask_in_file); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
   

   /* Quick check on fovlist and bitlist strings */
   if (strlen(fovlist_str)<2) {
      printf("Insufficient info in FOVLIST in FMASK file\n");      
      exit(TERMINATE_ERROR);
   }
   if (strlen(bitlist_str)<2) {
      printf("Insufficient info in BITLIST in FMASK file\n");      
      exit(TERMINATE_ERROR);
   }
   
   /* Convert FOVLIST string to integer array */
   i_fovlist=0;
   fov_num_str = strtok( fovlist_str, FOV_SEPCHARS );
   while (fov_num_str!=NULL) {
      nums = sscanf( fov_num_str, "%d", &(cp->fovlist[i_fovlist]) );
      if (nums!=1) {
         printf("ERROR converting FOVLIST string, \"%s\"\n",fovlist_str);      
         exit(TERMINATE_ERROR);
      }
      nums = sscanf( fov_num_str, "%d", &(cp->fovlist[i_fovlist]) );
      
      /* Get next tokens, if any */
      fov_num_str = strtok( NULL, FOV_SEPCHARS );
      i_fovlist++;
      if (i_fovlist>MAX_NUM_FOVS) {
         printf("ERROR: Maximum number of FOVs, %d, exceeded in FOVLIST\n", MAX_NUM_FOVS);
         exit(TERMINATE_ERROR);
      }
   }
   
   /* Convert BITLIST string to integer array */
   i_bitlist=0;
   bit_num_str = strtok( bitlist_str, FOV_SEPCHARS );
   while (bit_num_str!=NULL) {
      nums = sscanf( bit_num_str, "%d", &(cp->bitlist[i_bitlist]) );
      if (nums!=1) {
         printf("ERROR converting BITLIST string, \"%s\"\n",bitlist_str);      
         exit(TERMINATE_ERROR);
      }
      nums = sscanf( bit_num_str, "%d", &(cp->bitlist[i_bitlist]) );
      
      /* Get next tokens, if any */
      bit_num_str = strtok( NULL, FOV_SEPCHARS );
      i_bitlist++;
      if (i_bitlist>MAX_NUM_FOVS) {
         printf("ERROR: Maximum number of FOVs, %d, exceeded in BITLIST\n", MAX_NUM_FOVS);
         exit(TERMINATE_ERROR);
      }
   }

   /* Set cp->num_fovs */   
   if (i_fovlist!=i_bitlist) {
      printf("ERROR: Number of list elements in FOVLIST (%1d) and BITLIST (%1d) do not match\n",
              i_fovlist, i_bitlist                                                             );
      exit(TERMINATE_ERROR);
   }
   cp->num_fovs = i_fovlist;
   
   if (cp->debug) { 
      printf("Completed reading in %1d elements in FOVLIST and BITLIST as follows: \n",cp->num_fovs);  
      for (i=0; i<cp->num_fovs; i++ ) {
         printf("   For i=%1d, FOVLIST = %2d  BITLIST = %2d\n",i, cp->fovlist[i], cp->bitlist[i] );
      }
   }

   fits_close_file(fptr, &status);
 
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
}
   
/*==============================================================================
** Write FITS KEYWORDS to the FITS Header
**    -- Write special Flatfield Module header values and comments
*/
void write_fits_headers( fitsfile *f, cmd_struct cs )
{
int		status=0;
char		line[100];
time_t		t;		/* Time holder */

   if (cs.debug) {
      printf("REMINDER: Maximum number of headers that may be ");
      printf("*added* by coadd2d is: %d\n",MAX_NUM_C2D_HDRS);
   }

   /*                     "         1         2         3         4         5         6         7  " */
   /*                     "123456789012345678901234567890123456789012345678901234567890123456789012" */

   fits_write_comment( f, "[COADD2D]===============================================================", &status);

   sprintf(line,          "[COADD2D] MODULE %s", cs.progname );
   fits_write_comment( f, line, &status);
   sprintf(line,          "[COADD2D] VERSION = %4.2f", COADD2D_VERSION );
   fits_write_comment( f, line, &status);

   t = time(NULL);
   sprintf(line,          "[COADD2D] Processed at %s",  ctime(&t)  );
   line[strlen(line)-1]='\0';  /* ctime() produces a \n character which is disallowed in fits_write_comment() */
   fits_write_comment( f, line, &status);
   
   sprintf( line,         "[COADD2D] OUTPUT FILE:    Data = %s", cs.data_out_file );
   fits_write_comment( f, line, &status);
   
   sprintf( line,         "[COADD2D] OUTPUT FILE:    Sdev = %s", ( (cs.sdev_out_file_f)  ? cs.sdev_out_file  : "(NONE)" ) );
   fits_write_comment( f, line, &status);
   if (cs.wts_out_file_f) { 
      sprintf( line,      "[COADD2D] OUTPUT FILE: Weights = %s", ( (cs.wts_out_file_f)   ? cs.wts_out_file   : "(NONE)" ) );
      fits_write_comment( f, line, &status);
   }
   if (cs.amask_out_file_f) {
      sprintf( line,      "[COADD2D] OUTPUT FILE:   AMASK = %s", ( (cs.amask_out_file_f) ? cs.amask_out_file : "(NONE)" ) );
      fits_write_comment( f, line, &status);
   }
                             
   sprintf( line,         "[COADD2D] Output Data file is the result of coadding %1d FITS files", cs.num_files );
   fits_write_comment( f, line, &status);

   sprintf( line,         "[COADD2D] These FITS files had dimensions:  xdim=%1d & ydim=%1d", cs.xdim, cs.ydim );
   fits_write_comment( f, line, &status);

   sprintf( line,         "[COADD2D] DCE weighting was TURNED %s", ( (cs.use_weights) ? "ON" : "OFF" ) );
   fits_write_comment( f, line, &status);
   
/* sprintf( line,         "[COADD2D] Used central trim fractions:  f1=%1g & f2=%1g", cs.trim_f1, cs.trim_f2 );
** fits_write_comment( f, line, &status);
*/   
   if (cs.namelist_f) {
      sprintf( line,      "[COADD2D] INPUT FILE:       Namelist = %s", cs.namelist );
      fits_write_comment( f, line, &status);
   }

   if (cs.data_in_lfile_f) {
      sprintf( line,      "[COADD2D] INPUT FILE: Data file list = %s", cs.data_in_lfile );
      fits_write_comment( f, line, &status);
   }
   if (cs.sdev_in_lfile_f) {
      sprintf( line,      "[COADD2D] INPUT FILE: Sdev file list = %s", cs.sdev_in_lfile );
      fits_write_comment( f, line, &status);

/***
      sprintf( line,      "[COADD2D] Currently ALL sdev files ARE IGNORED; for later implementation", cs.sdev_in_lfile );
      fits_write_comment( f, line, &status);
***/

   }
   if (cs.bmask_in_lfile_f) {
      sprintf( line,      "[COADD2D] Used BMASK file list %s", cs.bmask_in_lfile );
      fits_write_comment( f, line, &status);
   }
   if (cs.pmask_in_lfile_f) {
      sprintf( line,      "[COADD2D] Used PMASK file list %s", cs.pmask_in_lfile );
      fits_write_comment( f, line, &status);
   }
   
   if (cs.fmask_in_file_f) {
      sprintf( line,      "[COADD2D] Used FMASK file %s", cs.fmask_in_file );
      if ( (cs.fov_for_wts>0) && cs.use_weights ) sprintf( line,
                          "[COADD2D]      with FOV NUMBER = %1d (used to calculate DCE weights)", cs.fov_for_wts );
   }
   else {
      sprintf( line,      "[COADD2D] Used NO FMASK file.");
   }
   fits_write_comment( f, line, &status);

   sprintf( line,      "[COADD2D] NOTE:  The output sdev FITS file is a quick approximation ");
   fits_write_comment( f, line, &status);   
   sprintf( line,      "[COADD2D]        that is intended to be upgraded in a later release. ");
   fits_write_comment( f, line, &status);   
   
   fits_write_comment( f, "[COADD2D]===============================================================", &status);
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);

}

                              
/**********************************************************************************
** WRITE OUT THE AMASK FILE
** BITPIX == 16 == SHORT_IMG 
*/
void write_amask( short int *amask, cmd_struct cs )
{
fitsfile *outfptr;  /* FITS file pointers */
char     overwrite_outfile[256] = "!";
int	 status = 0;
int	 naxis;
long	 naxes[2];   /* Values for the two axes */

   /* Create the new empty output file 
   **
   **    "An ERROR will be returned if the specified file already exists, unless 
   **    the filename is prefixed with an exclamation point (!). In that case 
   **    CFITSIO will overwrite (delete) any existing file with the same name."
   */
   /* overwrite_outfile[0] = '!';  */
   /* overwrite_outfile[1] = '\0'; */
   strcat(overwrite_outfile, cs.amask_out_file);
   if (0) printf("amask=%s, overwrite_outfile=\"%s\"\n",
                         cs.amask_out_file,overwrite_outfile);
   if (!fits_create_file(&outfptr, overwrite_outfile, &status)) {
   
      naxis    = 2;     /* TWO DIMENSIONS TO DESCRIBE ONE PLANE OF DATA */
      naxes[0] = cs.xdim;
      naxes[1] = cs.ydim;
      fits_create_img( outfptr, SHORT_IMG, naxis, naxes, &status );
      if (status) {
         printf("\aERROR: Attempt to create short image in FITS file %s FAILED in write_amask().\n", cs.amask_out_file); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   

      /* Write special Flatfield Module header values and comments */   
      write_fits_headers( outfptr, cs );
      
      fits_write_img( outfptr, TSHORT, 1, cs.xdim*cs.ydim, amask, &status );
      if (status) {
         printf("\aERROR: Attempt to write short image to file %s FAILED in write_flt_fits_from_dbl().\n", cs.amask_out_file); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   
   }
   
   fits_close_file(outfptr, &status);
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
   
}


/**********************************************************************************
** WRITE OUT THE Weights List FILE -- an ASCII text file
*/
void write_weights_list( float *dce_weight, files_struct *data_files, cmd_struct cs )
{
FILE  *ascii_outfile;
int   i;

   if (cs.debug) printf("Beginning writing %1d Weights to the Weights List file %s\n", 
                                           cs.num_files,                 cs.wts_out_file );
   ascii_outfile = fopen( cs.wts_out_file, "w" );
   if (ascii_outfile==NULL) {
      printf("\aERROR: Unable to open Weights List File %s for writing\n", cs.wts_out_file);
      exit(TERMINATE_ERROR);
   }
   
   for (i=0; i<cs.num_files; i++ ) {
      fprintf(ascii_outfile, "%15.4f  %s\n", dce_weight[i], data_files[i].v );
   }
   
   fclose( ascii_outfile );                                           

}


/**********************************************************************************
** WRITE OUT THE FITS FILE -- FLOAT DATA
*/
void write_flt_fits( char *fname, float *image, cmd_struct cs )
{
fitsfile *outfptr;  /* FITS file pointers */
char     overwrite_outfile[256] = "!";
int	 status = 0;
int	 naxis;
long	 naxes[2];   /* Values for the two axes */

   /* Create the new empty output file 
   **
   **    "An ERROR will be returned if the specified file already exists, unless 
   **    the filename is prefixed with an exclamation point (!). In that case 
   **    CFITSIO will overwrite (delete) any existing file with the same name."
   */
   /* overwrite_outfile[0] = '!';  */
   /* overwrite_outfile[1] = '\0'; */
   strcat(overwrite_outfile, fname);
   if (0) printf("fname=%s, overwrite_outfile=\"%s\"\n",
                         fname,overwrite_outfile);
   if (!fits_create_file(&outfptr, overwrite_outfile, &status)) {
      if (status) {
         printf("\aERROR: Attempt to create FITS file %s FAILED in write_flt_fits().\n", fname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   
      
      /* WRITE OUT THE ONE PLANE OF DATA */
      naxis    = 2;     /* TWO DIMENSIONS TO DESCRIBE ONE PLANE OF DATA */
      naxes[0] = cs.xdim;
      naxes[1] = cs.ydim;
      fits_create_img( outfptr, FLOAT_IMG, naxis, naxes, &status );
      if (status) {
         printf("\aERROR: Attempt to create one-plane image in FITS file %s FAILED in write_flt_fits().\n", fname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   

      /* Write special COADD2D Module header values and comments */   
      write_fits_headers( outfptr, cs );
      
      fits_write_img( outfptr, TFLOAT, 1, (cs.xdim)*(cs.ydim), image, &status );
      if (status) {
         printf("\aERROR: Attempt to write float image to file %s FAILED in write_flt_fits().\n", fname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   
   }
   
   fits_close_file(outfptr, &status);
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
   
   if (cs.debug) printf("Completed writing FLOAT DATA file %s\n", fname );

}


/**********************************************************************************
** WRITE OUT THE FITS FILE -- FLOAT DATA
*/
void write_flt_fits_with_hdrs( char *infname, char *outfname, float *image, cmd_struct cs )
{
fitsfile *infptr, *outfptr;  /* FITS file pointers */
char     overwrite_outfile[256] = "!";
int	 status = 0;
int	 naxis;
long	 naxes[2];   /* Values for the two axes */

   if (cs.debug) printf("Beginning to write file %s using write_flt_fits_from_dbl()\n", outfname );

   /* Create the new empty output file 
   **
   **    "An ERROR will be returned if the specified file already exists, unless 
   **    the filename is prefixed with an exclamation point (!). In that case 
   **    CFITSIO will overwrite (delete) any existing file with the same name."
   */
   /* overwrite_outfile[0] = '!';  */
   /* overwrite_outfile[1] = '\0'; */
   strcat(overwrite_outfile, outfname);
   if (0) printf("outfname=%s, overwrite_outfile=\"%s\"\n",outfname,overwrite_outfile);

   if (!fits_create_file(&outfptr, overwrite_outfile, &status)) {
      if (status) {
         printf("\aERROR: Attempt to create FITS file %s FAILED in write_flt_fits_with_hdrs().\n", outfname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   
   
      /* Copy (propagate) ALL the headers AND DATA (the entire "HDU") */
      /* from the input FF image to the output file                   */
      if (!fits_open_file( &infptr, infname, READONLY, &status )) {
         if (cs.debug) printf("Copy the HDU from %s to %s in write_flt_fits_with_hdrs()\n", infname, outfname);
         fits_copy_hdu( infptr, outfptr, MAX_NUM_C2D_HDRS, &status ); /* Leave room for MAX_NUM_FF_HEADERS */                  
         if (status) {
            printf("Problem in fits_copy_hdu() from %s to %s in write_flt_fits_with_hdrs()\n", infname, outfname);
            status = 0;
         }
         if (cs.debug) printf("HDU Copy SUCCESSFUL.\n");
         fits_close_file(infptr, &status);
      }

      /* WRITE OUT THE ONE PLANE OF DATA */
      naxis    = 2;     /* TWO DIMENSIONS TO DESCRIBE ONE PLANE OF DATA */
      naxes[0] = cs.xdim;
      naxes[1] = cs.ydim;
      fits_resize_img( outfptr, FLOAT_IMG, naxis, naxes, &status );
      if (status) {
         printf("fits_resize_img() failed.  Trying fits_create_img()...\n"); 
         status = 0;
         fits_create_img( outfptr, FLOAT_IMG, naxis, naxes, &status );
         if (status) printf("fits_create_img() failed too.\n"); 
      }
      if (status) {
         printf("\aERROR: Attempt to create one-plane image in FITS file %s FAILED in write_flt_fits_with_hdrs().\n", outfname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   

      /* Write special COADD2D Module header values and comments */   
      write_fits_headers( outfptr, cs );
      
      fits_write_img( outfptr, TFLOAT, 1, (cs.xdim)*(cs.ydim), image, &status );
      if (status) {
         printf("\aERROR: Attempt to write float image to file %s FAILED in write_flt_fits_from_dbl().\n", outfname); 
         fits_report_error(stdout, status);
         exit( TERMINATE_ERROR );
      }   
   }
   
   fits_close_file(outfptr, &status);
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error(stderr, status);
   
   if (cs.debug) printf("Completed writing FLOAT DATA file %s using write_flt_fits_with_hdrs()\n", outfname );

}


/**********************************************************************************
** Some test code to examine the data in a ONE-PLANE FITS file.

*** commented out for now (not used) ******

void test_fits( char *fits_filename ) 
{
float	*dimage;
int	x, y, xy, nan_count;
int	xdim, ydim;

   printf("===============================================================\n");
   printf("TEST_FITS() data examination of file %s: \n", fits_filename);

   x=-1;
   dimage = read_fits( fits_filename, &xdim, &ydim, &x, 1 );
   
   printf("Data sampled using x+=8 and y+=5: \n", fits_filename);
   for ( x=0; x<(xdim); x+=8 )  printf(" %06d ",x);
   printf("\n");
   
   for    ( y=0; y<(ydim); y+=5 )  {
      for ( x=0; x<(xdim); x+=8 )  {
         xy  = ( y * xdim )  + x;
         printf("%7.1g ",dimage[xy]);
      }
      printf("\n");
   }
   
   nan_count = 0;
   for    ( y=0; y<(ydim); y++ )  {
      for ( x=0; x<(xdim); x++ )  {
         xy  = ( y * xdim )  + x;
         if (iznand( (double) dimage[xy] )) nan_count++;
     }
   }
   printf("====> nan_count = %1d\n", nan_count);
   printf("===============================================================\n");
   
   free(dimage);
   
}
************/


/**********************************************************************************
** Some test code to examine the data in a ONE-PLANE FITS file.

*** commented out for now (not used) ******

void test_fits_s( char *fits_filename ) 
{
short int *dimage;
int	  x, y, xy, nan_count;
int	  xdim, ydim;

   printf("===============================================================\n");
   printf("TEST_FITS() data examination of file %s: \n", fits_filename);

   dimage = read_fits_s( fits_filename, &xdim, &ydim, 1 );
   
   printf("Data sampled using x+=8 and y+=5: \n", fits_filename);
   for ( x=0; x<(xdim); x+=8 )  printf(" %06d ",x);
   printf("\n");
   
   for    ( y=0; y<(ydim); y+=5 )  {
      for ( x=0; x<(xdim); x+=8 )  {
         xy  = ( y * xdim )  + x;
         printf("%7d ",dimage[xy]);
      }
      printf("\n");
   }
   
   nan_count = 0;
   for    ( y=0; y<(ydim); y++ )  {
      for ( x=0; x<(xdim); x++ )  {
         xy  = ( y * xdim )  + x;
         if (iznand( (double) dimage[xy] )) nan_count++;
     }
   }
   printf("====> nan_count = %1d\n", nan_count);
   printf("===============================================================\n");
   
   free(dimage);
   
}
************/


/**********************************************************************************
** A little routine to print out data for Very Small FITS files, 
** or a little bit of Big FITS files
*/
void print_fits_data( char *fitsfile )
{
float    *image;
int      xdim, ydim;
int      x,y,xy;      

   x=-1;
   image = read_fits( fitsfile, &xdim, &ydim, &x, 0 );
   printf("\nFITS FILE DATA in %s with xdim=%0d and ydim=%0d\n", 
           fitsfile, xdim, ydim);
           
   if (xdim>10) xdim=10;  /* Don't print out too much data! */
   if (ydim>10) ydim=10;     
           
   for(y=0;y<ydim;y++) {
      for(x=0;x<xdim;x++) {
         xy = ( y * xdim ) + x;
         printf( "%6.1f ", image[xy] ); 
      }
      printf("\n");
   }
   free( image );
   
}

/**********************************************************************************
** A little routine to print out data for Very Small FITS files (integer version)
*/
void print_fits_data_s( char *fitsfile )
{
short int  *image;
int        xdim, ydim;
int        x,y,xy;      

   image = read_fits_s( fitsfile, &xdim, &ydim, 0 );
   printf("\nFITS FILE DATA in %s with xdim=%0d and ydim=%0d\n", 
           fitsfile, xdim, ydim);
   for(y=0;y<ydim;y++) {
      for(x=0;x<xdim;x++) {
         xy = ( y * xdim ) + x;
         printf( "%6d ", image[xy] ); 
      }
      printf("\n");
   }
   free( image );
   
}

/**********************************************************************************
** A little routine to compare data in FITS files.
*/
void compare_fits_data( char *fitsfile_A, char *fitsfile_B )
{
float    *image_A, *image_B;
int      xdim, ydim;
int      x,y,xy;    
int      nan_pair_count;  

   x=-1;
   printf("\n");
   image_A = read_fits( fitsfile_A, &xdim, &ydim, &x, 0 );
   printf("FITS FILE DATA in %s with xdim=%0d and ydim=%0d\n", 
           fitsfile_A, xdim, ydim);
   x=-1;
   image_B = read_fits( fitsfile_B, &xdim, &ydim, &x, 0 );
   printf("FITS FILE DATA in %s with xdim=%0d and ydim=%0d\n", 
           fitsfile_B, xdim, ydim);
   printf("\n");
   
   nan_pair_count = 0;      
   for(y=0;y<ydim;y++) {
      for(x=0;x<xdim;x++) {
         xy = ( y * xdim ) + x;
         if ( image_A[xy] != image_B[xy] ) {
            if ( ( iznand((double) image_A[xy]) && iznand((double) image_B[xy]) ) ) nan_pair_count++;
            else {
               printf("x=%03d y=%03d  |  A=%7.4f  B=%7.4f  DELTA=%g\n", 
                       x, y, image_A[xy], image_B[xy], image_A[xy]-image_B[xy] ); 
            }
         }
      }
   }
   printf("There were %1d matching NaN pairs found\n",nan_pair_count);
   free( image_A );
   free( image_B );
   
}

/***********************************************************************************
** A little routine to insert an integer keyword into an existing FITS file.
** This routine also deletes any previous copies of this keyword.
*/
#define THE_POPE_IS_CATHOLIC  1
void insert_int_keyword( char *fname, char *keyname, int keyval )
{
fitsfile  *fptr;    /* FITS file pointer */
int	  status=0;
int       dummyval;

   /* printf("In insert_int_keyword()\n"); */

   fits_open_file( &fptr, fname, READWRITE, &status );
   if (status) {
      printf("\aERROR: Attempt to open file %s FAILED in insert_int_keyword().\n", fname); 
      fits_report_error(stderr, status);
      exit( TERMINATE_ERROR );
   }   
   
   /* Delete any previous copies of the keyword */
   while (THE_POPE_IS_CATHOLIC) {
      fits_read_key( fptr, TINT, keyname, &dummyval, NULL, &status);
      if (status) break;  /* if keyname isn't found, exit this loop */
      fits_delete_key( fptr, keyname, &status);
      if (status) {
         printf("\aERROR: failure while trying to delete keyword %s in FITS file %s [in insert_int_keyword()]\n", keyname, fname); 
         fits_report_error(stderr, status);
         exit( TERMINATE_ERROR );
      }   
   }
   status=0;

   /* Insert the integer keyword */
   fits_write_key( fptr, TINT, keyname, &keyval, "CW: added keyword", &status );
   if (status) {
      printf( "\aERROR: Unable to insert keyword %s=%1d into FITS file %s [in insert_int_keyword()]\n", keyname, keyval, fname );
      printf( "returned status=%d\n", status );
      exit( TERMINATE_ERROR );
   }   
   
   fits_close_file( fptr, &status );
   /* if ERROR occured, print out ERROR message */
   if (status) fits_report_error( stderr, status );
}   
