/* $Id: cropfits_defs.h,v 1.3 2007/11/12 18:12:16 laher Exp $ */


/****************************************************
 *  cropfits_defs.h
 ****************************************************

 Purpose:
 Define data structures for the module

 Overview:
 Provide a structure for images, filenames, aux.
 paths, computation, and module state

 ****************************************************/
#include "fitsio.h"

typedef struct {
  char	Filename_Namelist[MAX_FILENAME_LENGTH];
  char	Filename_Input[MAX_FILENAME_LENGTH];
  char	Filename_Output[MAX_FILENAME_LENGTH];
  char	Filename_Table[MAX_FILENAME_LENGTH];
  char	Filename_Map[MAX_FILENAME_LENGTH];
  char	Filename_Log[MAX_FILENAME_LENGTH];
} CPF_Filenames;


typedef struct {
  char	CP_Ancillary_File_Path[MAX_FILENAME_LENGTH];
} CPF_Pathnames;


typedef struct {
  int	Status;
  int   Debug;
  int	Verbose;
  int	SuperVerbose;
} CPF_State;

typedef struct {
  double* Image;
  int	  LengthX;
  int	  LengthY;
  int	  LengthZ;
  int     Plane;
  double* CroppedImage;
  int*    Map;
} CPF_ImageInfo;


typedef struct {
  int    istart;
  int    jstart;
  int    width;
  int    height;
} CPF_Computation;



/* Function prototypes. */

void cropfits_exit( CPF_ImageInfo   *CPFP_Image,
	            CPF_State       *CPFP_Stat);

void cropfits_compute(CPF_Filenames     *CPFP_Fnames,
		      CPF_Computation   *CPFP_Comp,
		      CPF_ImageInfo     *CPFP_Image,
		      CPF_State         *CPFP_Stat);

void cropfits_expand_envvar( CPF_Filenames  *CPFP_Fnames,
			     CPF_State      *CPFP_Stat);

int     expand_envvar_string(char *CP_In);

void cropfits_init( CPF_Filenames     *CPFP_Fnames,
	            CPF_Pathnames     *CPFP_Paths,
	            CPF_ImageInfo     *CPFP_Image,
	            CPF_Computation   *CPFP_Comp,
	            CPF_State	      *CPFP_Stat);

void cropfits_log( CPF_Filenames   *CPFP_Fnames,
	           CPF_Computation *CPFP_Comp,
	           CPF_ImageInfo   *CPFP_Image,
	           CPF_Pathnames   *CPFP_Paths,
	           CPF_State	   *CPFP_Stat);

void cropfits_output( CPF_Filenames    *CPFP_Fnames,
		      CPF_ImageInfo    *CPFP_Image,
		      CPF_Computation  *CPFP_Comp,
		      CPF_State	       *CPFP_Stat);

void cropfits_output_image( fitsfile        *ffp_FITS_Input, 
		            CPF_Filenames   *CPFP_Fnames,
		            CPF_ImageInfo   *CPFP_Image,
		            CPF_Computation *CPFP_Comp,
		            CPF_State       *CPFP_Stat);

void cropfits_output_map( fitsfile        *ffp_FITS_Input, 
		          CPF_Filenames   *CPFP_Fnames,
		          CPF_ImageInfo   *CPFP_Image,
		          CPF_Computation *CPFP_Comp,
		          CPF_State       *CPFP_Stat);

void cropfits_parse_args( int		  argc,
			  char		  **argv,
			  CPF_Filenames	  *CPFP_Fnames,
			  CPF_Pathnames	  *CPFP_Paths,
			  CPF_ImageInfo    *CPFP_Image,
			  CPF_Computation  *CPFP_Comp,
			  CPF_State	  *CPFP_Stat);

void cropfits_parse_namelist( int		argc,
			      char	        **argv,
			      CPF_Filenames	*CPFP_Fnames,
			      CPF_Pathnames	*CPFP_Paths,
			      CPF_ImageInfo     *CPFP_Image,
			      CPF_Computation   *CPFP_Comp,
			      CPF_State	        *CPFP_Stat);

void cropfits_read_data( CPF_Filenames	 *CPFP_Fnames,
			 CPF_ImageInfo	 *CPFP_Image,
			 CPF_Computation *CPFP_Comp,
			 CPF_State	 *CPFP_Stat);

void cropfits_read_image( CPF_Filenames   *CPFP_Fnames,
		          CPF_ImageInfo   *CPFP_Image,
		          CPF_State       *CPFP_Stat);

void decode_message( int           I_message, 
                     int           I_root,
		     char	   *CP_Message,
		     char	   *CP_Function,
		     CPF_Pathnames *CPFP_Paths,
		     CPF_State     *CPFP_Stat);
