/* $Id: cropfits_exec.c,v 1.3 2007/11/12 18:12:17 laher Exp $ */


/******************************************************** 
 *  cropfits_exec.c
 ********************************************************

 Purpose:
 "Main" function of cropfits module which calls the
 sub-modules in sequential order.

 Definitions of Variables 

 See cropfits_defs.h
 
 CPF_Fnames = Structure for filenames
 CPF_Paths = Structure for auxilliary path
 CPF_Image = Structure for image, masks, and information 
             about them
 CPF_Comp = Structure for variables used in computation
 CPF_Stat = Structure for status and output level flags
  

 *********************************************************/

#include <time.h>
#include <stdio.h>
#include "cropfits_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"
#include "cropfits.h"
#include "cropfits_defs.h"

int main(int argc, char **argv)
{		

  CPF_Filenames		CPF_Fnames;
  CPF_Pathnames		CPF_Paths;
  CPF_ImageInfo 	CPF_Image;
  CPF_Computation	CPF_Comp;
  CPF_State		CPF_Stat;
  
  clock();
  
  /* Set default values */
  cropfits_init( &CPF_Fnames, &CPF_Paths, &CPF_Image, &CPF_Comp, &CPF_Stat);
  
  /* Parse the namelist, replacing default values if specified */
  cropfits_parse_namelist( argc, argv, &CPF_Fnames, &CPF_Paths, &CPF_Image, 
			   &CPF_Comp, &CPF_Stat);
  
  /* Parse the command line, possibly replacing namelist values*/
  cropfits_parse_args( argc, argv, &CPF_Fnames, &CPF_Paths, &CPF_Image, 
		       &CPF_Comp, &CPF_Stat);
  
  /* Expand any environmental variables passed into filename defintions */
  cropfits_expand_envvar( &CPF_Fnames, &CPF_Stat);
  
  /* Get the data and keyword values from the input and output files */
  cropfits_read_data( &CPF_Fnames, &CPF_Image, &CPF_Comp, &CPF_Stat);

  /* Copute the muxbleed correction */
  cropfits_compute(&CPF_Fnames, &CPF_Comp, &CPF_Image, &CPF_Stat);
 
  /* Write results to output file */			
  cropfits_output( &CPF_Fnames, &CPF_Image, &CPF_Comp, &CPF_Stat);
  
  /* Print filename, error message and timing data */
  cropfits_log( &CPF_Fnames, &CPF_Comp, &CPF_Image, &CPF_Paths, &CPF_Stat);
  
  /* Deallocate memory */			
  cropfits_exit( &CPF_Image, &CPF_Stat);
  
  /* Send return value */
  exit(CPF_Stat.Status);

}




