/* $Id: cropfits_expand_envvar.c,v 1.3 2007/11/12 18:12:17 laher Exp $ */


/********************************************************************
 *  cropfits_expand_envvar.c
 ********************************************************************

 Purpose:
 If environmental variables were used in filename, expand 
 them to string values and rewrite entry in filenames structure.
 
 Overview:
 Use string functions to search for the separator slash character,
 search for the '$' character indicating a variable, then 
 getenv to look for the definition of the variable.
 
 Definitions of Variables

 External:
 See cropfits_defs.h and cropfits_exec.c
 
 Internal:
 CP_Copy = Copy of input string.
 CP_Path = Copy of input string, modified by strtok.
 CP_Expanded = Holds output expanded string.
 CP_Ent = Parsed segment of string.
 CPP_TokSlash = Array of string positions where "/" characters were.
 CPP_TokDollar = Array of string positions where "$" characters were.
 CP_Slash = Marks where "/" characters are.
 CP_Dollar = Marks where "$" characters are.
  
 ********************************************************************/

#include <stdio.h>
#include "cropfits_errcodes.h"
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include "cropfits.h"
#include "cropfits_defs.h"

#define STRING_BUFFER_SIZE       256

void cropfits_expand_envvar( CPF_Filenames  *CPFP_Fnames,
			     CPF_State      *CPFP_Stat)
{
  
  if (CPFP_Stat->Status) return;
  
  if (strcmp(CPFP_Fnames->Filename_Namelist, "")) {
    expand_envvar_string(CPFP_Fnames->Filename_Namelist);
    if(CPFP_Stat->Verbose)      
      printf("cropfits_expand_envvar: %s\n", 
             CPFP_Fnames->Filename_Namelist);
  }

  if (strcmp(CPFP_Fnames->Filename_Table, "")) {
    expand_envvar_string(CPFP_Fnames->Filename_Table);
    if(CPFP_Stat->Verbose)      
      printf("cropfits_expand_envvar: %s\n", 
             &(CPFP_Fnames->Filename_Table[1]));
  }
  
  if (strcmp(CPFP_Fnames->Filename_Map, "")) {
    expand_envvar_string(CPFP_Fnames->Filename_Map);
    if(CPFP_Stat->Verbose)      
      printf("cropfits_expand_envvar: %s\n", 
             &(CPFP_Fnames->Filename_Map[1]));
  }


  expand_envvar_string(CPFP_Fnames->Filename_Input);  
  expand_envvar_string(CPFP_Fnames->Filename_Output);
  expand_envvar_string(CPFP_Fnames->Filename_Log);
  
  if(CPFP_Stat->Verbose)        {
    printf("cropfits_expand_envvar: %s\n", 
           CPFP_Fnames->Filename_Input);
    printf("cropfits_expand_envvar: %s\n", 
           &(CPFP_Fnames->Filename_Output[1]));
    printf("cropfits_expand_envvar: %s\n", 
           CPFP_Fnames->Filename_Log);
  }
  
}
  



int     expand_envvar_string(char *CP_In)
  {

  char  CP_Copy[STRING_BUFFER_SIZE];
  char  CP_Path[STRING_BUFFER_SIZE];
  char  CP_Expanded[STRING_BUFFER_SIZE];
  char  CP_Ent[STRING_BUFFER_SIZE];
  char  *CPP_TokSlash[STRING_BUFFER_SIZE];
  char  *CPP_TokDollar[STRING_BUFFER_SIZE];
  char  *CP_Slash, *CP_Dollar;
  
  int   i, j, k, l, I_numchars;
  
  
                strcpy(CP_Copy, CP_In);
                strcpy(CP_Path, CP_In);
                
                /* strtok clobbers a leading "/" */
                if(strncmp(CP_Path, "/", 1))
                strcpy(CP_Expanded, "");
                else
                strcpy(CP_Expanded, "/");       
                
                /* Replace the "/" token with nulls */
                i=0;
                CPP_TokSlash[i++] = strtok(CP_Path, "/");
                while (CPP_TokSlash[i++] = strtok(NULL, "/"));
                
                for(j=0; j<i-1; j++)    {
                
                /* For each string between the "/"'s put a null at any "$" */
                k=0;
                CPP_TokDollar[k++] = strtok(CPP_TokSlash[j], "$");
                while (CPP_TokDollar[k++] = strtok(NULL, "$"));

                /* Read the resulting parsed strings */         
                for(l=0; l<k-1; l++)    {
                sscanf(CPP_TokDollar[l], "%s", CP_Ent);
                /* Expand as an environmental variable if there was a "$" */                    
                if(CPP_TokDollar[l]==CPP_TokSlash[j])
                strcat(CP_Expanded, CP_Ent);            
                else
                strcat(CP_Expanded, getenv(CP_Ent));
                }
                if(j<i-2) strcat(CP_Expanded, "/");
                }

                /* strtok clobbers a trailing "/" */
                I_numchars = strlen(CP_Copy);
                if(!strncmp(&CP_Copy[I_numchars-1], "/", 1))
                strcat(CP_Expanded, "/");
                                
                strcpy(CP_In, CP_Expanded);
                
  }
  


