/* $Id: cropfits_output.c,v 1.3 2007/11/12 18:12:17 laher Exp $ */


/********************************************************** 
 *  cropfits_output.c
 **********************************************************

 Purpose:
 Save the cropfits outputs to files.
 
 Overview:
 Creates output files per command-input control.
 
 Definitions of Variables
 External:
 See cropfits_defs.h and cropfits_exec.c
 
 Internal:
 ffp_FITS_Input = Handle to input fits image (for copying
                  the header keys into the output files)
 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "cropfits.h"
#include "cropfits_defs.h"
#include "cropfits_errcodes.h"


void cropfits_output( CPF_Filenames    *CPFP_Fnames,
		      CPF_ImageInfo    *CPFP_Image,
		      CPF_Computation  *CPFP_Comp,
		      CPF_State	       *CPFP_Stat)
{
  fitsfile 	*ffp_FITS_Input;
  int           I_fits_return_status=0;
  
  
  if(CPFP_Stat->Status) return;
  
  /*****************************************************
   * open the input file for header keyword reading 
   */
  fits_open_file(&ffp_FITS_Input,
		 CPFP_Fnames->Filename_Input,
		 READONLY,
		 &I_fits_return_status);

  /*****************************************************
   * output the cropped image
   */
  if (CPFP_Stat->Verbose)
    printf("cropfits_output: Creating output FITS image\n");
  cropfits_output_image(ffp_FITS_Input, CPFP_Fnames, CPFP_Image, 
			CPFP_Comp, CPFP_Stat);
  if(CPFP_Stat->Status)
    return;

  /*****************************************************
   * if the map image was requested, output that 
   */
  if(strcmp(CPFP_Fnames->Filename_Map,"")) {
    if (CPFP_Stat->Verbose) 
      printf("cropfits_output: Creating FITS map\n"); 
    cropfits_output_map(ffp_FITS_Input, CPFP_Fnames, CPFP_Image,
			CPFP_Comp, CPFP_Stat);
  }
    
  
  fits_close_file(ffp_FITS_Input, &I_fits_return_status);

  return;
}
