/* $Id: cropfits_output_image.c,v 1.6 2007/11/12 18:12:17 laher Exp $ */

/********************************************************** 
 *  cropfits_output_image.c
 **********************************************************

 Purpose:
 Save the cropfits outputs to files.
 
 Overview:
 Creates output files per command-input control.
 
 Definitions of Variables
 External:
 See cropfits_defs.h and cropfits_exec.c
 
 Internal:
 ffp_FITS_Input = Handle to input fits image (for copying
                  the header keys into the output files)
 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "cropfits.h"
#include "cropfits_defs.h"
#include "cropfits_errcodes.h"

void cropfits_output_image( fitsfile        *ffp_FITS_Input, 
		            CPF_Filenames   *CPFP_Fnames,
		            CPF_ImageInfo   *CPFP_Image,
		            CPF_Computation *CPFP_Comp,
		            CPF_State       *CPFP_Stat)
{
  fitsfile  *ffp_FITS_Output;
  int       I_fits_return_status=0;
  int       morekeys = 0;
  int       start, index;
  int       NumAxes, CP_Value;

  double    version;

  char      CP_Comment[FLEN_COMMENT];
  char      CP_Keyname[FLEN_KEYWORD];
  char      CP_Funct[32];

  long      OutputPixels;

  if(CPFP_Stat->Status) return;
  

  /*****************************************************
   * create the output image
   */

  fits_create_file(&ffp_FITS_Output,
		   CPFP_Fnames->Filename_Output,
		   &I_fits_return_status);

  OutputPixels = CPFP_Comp->width * CPFP_Comp->height;
  if(CPFP_Image->Plane==1) 
    OutputPixels *= CPFP_Image->LengthZ;

  if(I_fits_return_status) {
    if(CPFP_Stat->Verbose)
      printf("cropfits_output_image: FITS file i/o error %d\n", I_fits_return_status);
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }
  
  /*****************************************************
   * copy the input image header to the output image
   */

  if(CPFP_Stat->SuperVerbose)
    printf("cropfits_output_image: Copying header from input file\n");

  fits_copy_header(ffp_FITS_Input,
		ffp_FITS_Output,
		&I_fits_return_status);

  if(I_fits_return_status) {
    if(CPFP_Stat->Verbose)
      printf("cropfits_output_image: FITS file i/o error %d\n", I_fits_return_status);
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }

  /*****************************************************
   * update the output header to reflect the operations
   * of this module
   */
  fits_write_date(ffp_FITS_Output, &I_fits_return_status);		 

  if (I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status; 
    return;
  }		 
 
  sprintf(CP_Keyname, "%s", "NAXIS1");
  sprintf(CP_Comment, "%s", "");
  CP_Value = CPFP_Comp->height;
  if(CPFP_Stat->Debug)
    printf("debug - comp.height=%d  CP_Value=%d\n", CPFP_Comp->height, CP_Value);
  fits_update_key(ffp_FITS_Output,
		  TINT,
		  CP_Keyname,
		  &CP_Value,
		  CP_Comment,
		  &I_fits_return_status);
  
  
  sprintf(CP_Keyname, "%s", "NAXIS2");
  sprintf(CP_Comment, "%s", "");
  CP_Value = CPFP_Comp->width;
  if(CPFP_Stat->Debug)
    printf("debug - comp.width=%d  CP_Value=%d\n", CPFP_Comp->width, CP_Value);
  fits_update_key(ffp_FITS_Output,
		  TINT,
		  CP_Keyname,
		  &CP_Value,
		  CP_Comment,
		  &I_fits_return_status);

  if( (CPFP_Image->Plane!=1) &&
      (CPFP_Image->LengthZ > 1) ) {
    sprintf(CP_Keyname, "%s", "NAXIS3");
    sprintf(CP_Comment, "%s", "");
    CP_Value = 1;
    fits_update_key(ffp_FITS_Output,
		    TINT,
		    CP_Keyname,
		    &CP_Value,
		    CP_Comment,
		    &I_fits_return_status);
  }
  
  fits_flush_file(ffp_FITS_Output, &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }   


  sprintf(CP_Keyname, "%s", PROGRAMABB);
  strcat(CP_Keyname, "VERSN");
  version = CPFVERSN;
  sprintf(CP_Comment, "%s%s%s", "Version number of ",PROGRAM," program");
  
  fits_write_key_fixflt(ffp_FITS_Output, 
			CP_Keyname, version, 2,
			CP_Comment,
			&I_fits_return_status);
  fits_flush_file(ffp_FITS_Output, &I_fits_return_status);
  if (I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status; 
    return;
  }		 
  
  sprintf(CP_Keyname, "CPFINPUT");
  sprintf(CP_Comment, "Input signal image");
  fits_write_key(ffp_FITS_Output,
		 TSTRING,
		 CP_Keyname,
		 &(CPFP_Fnames->Filename_Input),
		 CP_Comment,
		 &I_fits_return_status);
  fits_flush_file(ffp_FITS_Output, &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  }

  sprintf(CP_Keyname, "CPFAXIS1");
  sprintf(CP_Comment, "Original length NAXIS1");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Image->LengthY),
		 CP_Comment,
		 &I_fits_return_status);

  sprintf(CP_Keyname, "CPFAXIS2");
  sprintf(CP_Comment, "Original length NAXIS2");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Image->LengthX),
		 CP_Comment,
		 &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  }
  
  sprintf(CP_Keyname, "CPFISTRT");
  sprintf(CP_Comment, "Crop box i start");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Comp->jstart),
		 CP_Comment,
		 &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  }
    
  sprintf(CP_Keyname, "CPFJSTRT");
  sprintf(CP_Comment, "Crop box j start");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Comp->istart),
		 CP_Comment,
		 &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  } 

  sprintf(CP_Keyname, "CPFWIDTH");
  sprintf(CP_Comment, "Crop box width");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Comp->height),
		 CP_Comment,
		 &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  }
  
  sprintf(CP_Keyname, "CPFHIGHT");
  sprintf(CP_Comment, "Crop box height");
  fits_write_key(ffp_FITS_Output,
		 TINT,
		 CP_Keyname,
		 &(CPFP_Comp->width),
		 CP_Comment,
		 &I_fits_return_status);
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
    return;
  }
    
  if(strcmp(CPFP_Fnames->Filename_Map,"")) {
    sprintf(CP_Keyname, "CPFMAPFL");
    sprintf(CP_Comment, "Map of original pixel orientation");
    fits_write_key(ffp_FITS_Output,
		   TSTRING,
		   CP_Keyname,
		   &(CPFP_Fnames->Filename_Map[1]),
		   CP_Comment,
		   &I_fits_return_status);
    if(I_fits_return_status) {
      CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
      return;
    }
  }

  if(strcmp(CPFP_Fnames->Filename_Table,"")) {
    sprintf(CP_Keyname, "CPFTABLE");
    sprintf(CP_Comment, "Table of cropped pixel values");
    fits_write_key(ffp_FITS_Output,
		   TSTRING,
		   CP_Keyname,
		   &(CPFP_Fnames->Filename_Table[1]),
		   CP_Comment,
		   &I_fits_return_status);
    if(I_fits_return_status) {
      CPFP_Stat->Status = OUTPUT_PROBS|FITS_MSG_MASK|I_fits_return_status;
      return;
    }
  }

  fits_flush_file(ffp_FITS_Output, &I_fits_return_status);
  
  if(I_fits_return_status) {
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }

  /*****************************************************
   * write the data into the image DU
   */
  if(CPFP_Stat->SuperVerbose)
    printf("cropfits_output_image: Writing data\n");
  fits_flush_file(ffp_FITS_Output, &I_fits_return_status);

  fits_write_img(ffp_FITS_Output,
		 TDOUBLE,
		 1, OutputPixels,
		 CPFP_Image->CroppedImage,
		 &I_fits_return_status);

  fits_flush_file(ffp_FITS_Output,
		  &I_fits_return_status);

  if(I_fits_return_status) {
    if(CPFP_Stat->Verbose)
      printf("cropfits_output_image: FITS file i/o error %d\n", I_fits_return_status);
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }
  
  if(CPFP_Stat->SuperVerbose)
    printf("cropfits_output_image: Closing output cropped image\n");
  
  fits_close_file(ffp_FITS_Output, &I_fits_return_status);
  
  if(I_fits_return_status) {
    if(CPFP_Stat->Verbose)
      printf("cropfits_output_image: FITS file i/o error %d\n", I_fits_return_status);
    CPFP_Stat->Status = OUTPUT_PROBS | FITS_MSG_MASK | I_fits_return_status;
    return;
  }

  return;
}
