/* $Id: cropfits_parse_args.c,v 1.4 2007/11/12 18:12:17 laher Exp $ */

/****************************************************** 
 *  cropfits_parse_args.c
 ******************************************************

 Purpose
 Parse the command line to get input and output
 file names constants, and processing flags.
 
 Overview
 Loop over the arguments, invoking case statement. 
 Check arguments for correctness & completeness.
 
 Definitions of Variables
 
 External:
 See cropfits_defs.h and cropfits_exec.c
 

 *****************************************************/

#include <stdio.h>
#include <string.h>
#include "cropfits_errcodes.h"
#include "cropfits.h"
#include "cropfits_defs.h"


void cropfits_parse_args( int		  argc,
			  char		  **argv,
			  CPF_Filenames	  *CPFP_Fnames,
			  CPF_Pathnames	  *CPFP_Paths,
			  CPF_ImageInfo    *CPFP_Image,
			  CPF_Computation  *CPFP_Comp,
			  CPF_State	  *CPFP_Stat)
{

  int	i=1; 
  int   I_gotallinputs;
  
  int temp_num=0;
  int temp_den=0;

  /* Run regardless of CPF_Stat.Status so that
     at least log file name will be known */

  while( i < argc ) {
    if( argv[i][0] == '-' ) {
      switch( argv[i][1] ) {
      case 'n':      /* -n <input_namelist_fname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -n <namelist_fname> missing argument\n" );
	else
	  if(CPFP_Fnames->Filename_Namelist) /* Filename can be stored */
	    sscanf(argv[i], "%s", CPFP_Fnames->Filename_Namelist);
	break;
      case 'i':      /* -i <input_fname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -i <input_image_fname> missing argument\n" );
	else {
	  if(argv[i-1][2] == '1') 
	    sscanf(argv[i], "%d", &(CPFP_Comp->jstart));
	  else if (argv[i-1][2] == '\0') 
	    sscanf(argv[i], "%s", CPFP_Fnames->Filename_Input);
	  else
	    printf("cropfits_parse_args: -i? incomplete parameter specification\n");
	}
	break;
      case 'o':      /* -o <output_image_fname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -o <output_fname> missing argument\n");
	else 
	  if(CPFP_Fnames->Filename_Output) {
	    sscanf("!", "%c", CPFP_Fnames->Filename_Output);
	    sscanf(argv[i], "%s", &(CPFP_Fnames->Filename_Output[1]));
	  }
	break;
      case 't':    /* -t <table_fname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -t <table_fname> missing argument\n");
	else
	  if(CPFP_Fnames->Filename_Table) {
	    sscanf("!", "%c", CPFP_Fnames->Filename_Table);
	    sscanf(argv[i], "%s", &(CPFP_Fnames->Filename_Table[1]));
	  }
	break;
      case 'm':    /* -m <org_map_fname> */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -m <map_fname> missing argument\n");
	else
	  if(CPFP_Fnames->Filename_Map) {
	    sscanf("!", "%c", CPFP_Fnames->Filename_Map);
	    sscanf(argv[i], "%s", &(CPFP_Fnames->Filename_Map[1]));
	  }
	break;
      case 'l':      /* -l <log_fname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -l <log_fname> missing argument\n" );
	else
	  if(CPFP_Fnames->Filename_Log)
	    sscanf(argv[i], "%s", CPFP_Fnames->Filename_Log);
	break;
      case 'j':      /* -j <j_start> */
	if( ++i >= argc)
	  printf("cropfits_parse_args: -j <j_start> missing argument\n");
	else if (argv[i-1][2] == '1')
	  sscanf(argv[i], "%d", &(CPFP_Comp->istart));
	break;
      case 'w':  /* -w <width> */
	if( ++i >= argc)
	  printf("cropfits_parse_args: -w <width> missing argument\n");
	else
	  sscanf(argv[i], "%d", &(CPFP_Comp->height));
	break;
      case 'h':  /* -h <height> */
	if( ++i >= argc)
	  printf("cropfits_parse_args: -h <height> missing argument\n");
	else
	  sscanf(argv[i], "%d", &(CPFP_Comp->width));
	break;
      case 'a':      /* -a <ancillary_file_pathname>  */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -a <ancillary_file_path> missing argument\n" );
	else
	  if(CPFP_Paths->CP_Ancillary_File_Path) /* Filename can be stored */
	    sscanf(argv[i], "%s", CPFP_Paths->CP_Ancillary_File_Path);
	break;
      case 'z':      /* -z data_plane */
	if( ++i >= argc )
	  printf("cropfits_parse_args: -z <data_plane> missing argument\n");
	else
	  sscanf(argv[i], "%d", &(CPFP_Image->Plane));
	break;
      case 'b':      /* - debug   */
	CPFP_Stat->Debug = TRUE;
	break;
      case 'v':      /* - verbose & superverbose */
	if(argv[i][2] != 'v')
	  CPFP_Stat->Verbose = TRUE;
	if(argv[i][2] == 'v')
	  CPFP_Stat->SuperVerbose = TRUE;
	break;
      default:
	printf("cropfits_parse_args: unknown argument\n");
      }
    }
    else
      {
	printf("cropfits_parse_args: Command line syntax error:\n");
	printf("   Previous argument = %s\n",argv[i - 1]);
	printf("   Current argument = %s\n",argv[i]);
	CPFP_Stat->Status = PARSE_ARGS | COULD_NOT_READ;
      }
    i++;
  }
  
  if(CPFP_Stat->Verbose) 
    printf("cropfits_parse_args: Program %s, Version %2.1f\n", 
	   PROGRAM, CPFVERSN);


  /* Check module inputs. */
  I_gotallinputs = 0;

  if(CPFP_Stat->Verbose) 
    if (strcmp(CPFP_Fnames->Filename_Namelist,"") != 0) 
      printf("cropfits_parse_args: Namelist File = %s\n", 
	     CPFP_Fnames->Filename_Namelist);
    else 
      printf("cropfits_parse_args: Information only: %s\n",
	     "No namelist file specified.");
  if(CPFP_Stat->Verbose) 
    if(strcmp(CPFP_Paths->CP_Ancillary_File_Path,"") != 0)
      printf("cropfits_parse_args: Ancilllary File Pathname = %s\n", 
	     CPFP_Paths->CP_Ancillary_File_Path);
    else 
      printf("cropfits_parse_args: Information only: %s %s\n",
	     "No ancillary file pathname specified;",
	     "defaulting to current directory.");
  if(strcmp(CPFP_Fnames->Filename_Input,"") == 0) {  
    printf("cropfits_parse_args: Missing input filename %s\n",
	   "(-i <input_fname>).");
    I_gotallinputs = 1;
  } else if (CPFP_Stat->Verbose)
    printf("cropfits_parse_args: Input File = %s\n", CPFP_Fnames->Filename_Input);
  if(strcmp(CPFP_Fnames->Filename_Output,"") == 0) {
    printf("cropfits_parse_args: Missing output filename %s\n",
	   "(-o <output_fname>).");
    I_gotallinputs = 1;
  } else if(CPFP_Stat->Verbose) 
    printf("cropfits_parse_args: Output Filename = %s\n", 
	   &(CPFP_Fnames->Filename_Output[1]));

  if(CPFP_Stat->Verbose) {
    if(strcmp(CPFP_Fnames->Filename_Map,"") != 0) 
      printf("cropfits_parse_args: Org. Pixel Map = %s\n",
	     &(CPFP_Fnames->Filename_Map[1]));
    if(strcmp(CPFP_Fnames->Filename_Table,"") !=0) 
      printf("cropfits_parse_args: Table filename = %s\n",
	     CPFP_Fnames->Filename_Table);
  } 

  if(! ((CPFP_Image->Plane==1) || 
	(CPFP_Image->Plane==2) ||
	(CPFP_Image->Plane==3))) {
    printf("cropfits_parse_args: Error -z <data_plane> must be 1 (all), 2 (first), or 3 (last)\n");
    I_gotallinputs = 2;
  } else if(CPFP_Stat->Verbose)
    printf("cropfits_parse_args: Plane to Process = %d\n", CPFP_Image->Plane);

  if((CPFP_Comp->istart < 1)||(CPFP_Comp->jstart < 1)) {
    printf("cropfits_parse_args: Error -i1 and -j1 parameters must be >= 1\n");
    I_gotallinputs = 2;
  } 

  if((CPFP_Comp->width < 1)||(CPFP_Comp->height < 1)) {
    printf("cropfits_parse_args: Error -w & -h must be >= 1 \n");
    I_gotallinputs = 2;
  } 
  
  if(CPFP_Stat->Verbose) {
    printf("cropfits_parse_args: i start = %d\n", 
	   CPFP_Comp->jstart);
    printf("cropfits_parse_args: j start = %d\n", 
	   CPFP_Comp->istart);
    printf("cropfits_parse_args: height = %d\n",
	   CPFP_Comp->width);
    printf("cropfits_parse_args: width = %d\n", 
	   CPFP_Comp->height);
    printf("cropfits_parse_args: Log File = %s\n", 
	   CPFP_Fnames->Filename_Log);
    printf("cropfits_parse_args: Debug = %d\n", 
	   CPFP_Stat->Debug);
    printf("cropfits_parse_args: Verbose = %d\n", 
	   CPFP_Stat->Verbose);
    printf("cropfits_parse_args: Superverbose = %d\n", 
	   CPFP_Stat->SuperVerbose);
  }
  

   if (CPFP_Stat->Status) return;
   
   if (I_gotallinputs == 1) {
     CPFP_Stat->Status = PARSE_ARGS | MISSING_INPUT;
     return;
   } else if (I_gotallinputs == 2) {
     CPFP_Stat->Status = PARSE_ARGS | OUT_OF_RANGE;
     return;
   }
   
   return;

}

