/* $Id: cropfits_read_data.c,v 1.3 2007/11/12 18:12:18 laher Exp $ */


/*********************************************************** 
 * cropfits_read_data.c
 ***********************************************************

 Purpose:
 Direct the read-in of data.

 Overview:
 Call functions to read in the fits image (including 
 dimensions & associated p & d masks), read the scaling
 function coefficient table, read the lookup table.

 Definitions of Variables

 External:
 See cropfits_defs.h and cropfits_exec.c

 Internal:
 I_fits_return_status = Resturn value of fits function.
 I_ndims_found = Number of data dimensions found.
 I_anynull = Whether nulls were found in the data.
 LP_naxes = Array to store lengths of axes of data.
 D_nullval = Set to zero, indicating not to replace nulls.


***********************************************/

#include <time.h>
#include <stdio.h>
#include "cropfits_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "cropfits.h"
#include "cropfits_defs.h"


void cropfits_read_data( CPF_Filenames	*CPFP_Fnames,
			 CPF_ImageInfo	*CPFP_Image,
			 CPF_Computation *CPFP_Comp,
			 CPF_State	*CPFP_Stat)
{
  int index;
  int NumPixels;

  if(CPFP_Stat->Status) return;

  /********************************************************
   * read the input image 
   */
  cropfits_read_image(CPFP_Fnames, CPFP_Image, CPFP_Stat);

  if(CPFP_Stat->Status) return;

  /********************************************************
   * check that crop box & image fit together
   * issue warning if they do not? or abort?
   * warning (but not a warning code) - in this case the 
   * height and width will be reset in cropfits_compute.
   */
  if(CPFP_Image->LengthX < (CPFP_Comp->istart-1+CPFP_Comp->width)) {
    printf("cropfits_read_data: Warning - Crop box does not fit in image in x direction\n");
    printf("cropfits_read_data: Cropping will stop at end of image (pixel = %d)\n", 
	   CPFP_Image->LengthX);
  }
  if(CPFP_Image->LengthY < (CPFP_Comp->jstart-1+CPFP_Comp->height)) {
    printf("cropfits_read_data: Warning - Crop box does not fit in image in y direction\n");
    printf("cropfits_read_data: Cropping will stop at end of image (pixel = %d)\n", 
	   CPFP_Image->LengthY);
  }

  return;
}
