/* $Id: cubesub_error_handler.c,v 1.8 2002/09/04 19:56:45 fan Exp $ */


/*********************************************** 
cubesub_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of cubesub functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_cubesub_message = Code of error that occurred.
I_cubesub_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "cubesub.h"
#include "cubesub_defs.h"


cubesub_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_cubesub_message;
int	I_cubesub_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_cubesub_message = RIGHT_MASK&(Stat->I_status);
I_cubesub_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_cubesub_message&FITS_MSG_MASK)	{
I_cubesub_message -= FITS_MSG_MASK;
}

cubesub_decode_message(I_cubesub_message, CP_Message);
cubesub_decode_fcnname(I_cubesub_root, CP_Function);

if (Stat->I_status)
{
printf("cubesub Error Message      0x%04x: %s.\n", I_cubesub_message, CP_Message);
printf("cubesub Error in Function  0x%04x: %s.\n", I_cubesub_root, CP_Function);
}


I_Time = clock();
printf("Processor Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("cubesub_error_handler: Exception Handler finished.\n");

}
