/* $Id: cubesub_exec.c,v 1.9 2004/05/13 18:21:08 fan Exp $ */


/*********************************************** 
cubesub_exec.c

Purpose

Calls the initialization functions and main loop of program. Saves
output to a file.


Overview



Definitions of Variables 
See cubesub_defs.h
 
***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include "fitsio.h"
#include "cubesub.h"
#include "cubesub_defs.h"

int main(int argc, char **argv)
{			
float	*FP_Data_In=0;
float   *FP_Vars_In=0;
short   *SP_pmask=0;
short   *SP_dmask=0;
short   *SP_bmask=0;
short   *SP_cmask=0;
float	*FP_Dark=0;
float   *FP_Dark_Vars=0;
int     i,j;

Filenames	Fnames;
Status		Stat;
Dims            Dim_In;
Constants       Const;
struct tm       *tmp_time;
time_t          L_time;

/* null filename pointers */

clock();

printf("Pipeline Module CUBESUB Version %2.1f\n", VERSN);

/* Calculate and print to standard out time
   Keep tmp_time and module time for time
   time stamping all output
*/
L_time = time(NULL);
tmp_time = localtime(&L_time);
printf("Processing time ");
printf(asctime(tmp_time));
printf("\n");

/* Set constant default values */
cubesub_init_constants( &Fnames, &Const, &Stat);

/* Parse the namelist file */
cubesub_parse_namelist( argc, argv, &Fnames, &Const, &Stat);

/* Parse the command line, replacing values if specified */
cubesub_parse_args( argc, argv, &Fnames, &Const, &Stat);

/* Get the data and keyword values from the input file */
cubesub_read_data( &Fnames, &Const, &Dim_In, &FP_Data_In, &FP_Vars_In,
&Stat, &SP_pmask, &SP_dmask, &SP_bmask, &SP_cmask, &FP_Dark, &FP_Dark_Vars);

/* Perform numerical computations on the data */
cubesub_compute_results( &Fnames, &Const, &Dim_In, FP_Data_In, FP_Vars_In,
&Stat, SP_pmask, SP_dmask, SP_bmask, SP_cmask, FP_Dark, FP_Dark_Vars);

/* Save the results as fits files. */			
cubesub_output( &Fnames, &Const, &Dim_In, &Stat, tmp_time, SP_dmask,
FP_Data_In, FP_Vars_In);

/* Print filename, error message and timing data */
cubesub_error_handler( &Stat, &Fnames);

/* Deallocate memory */			
cubesub_exit( FP_Data_In, FP_Vars_In, &Dim_In, &Stat, SP_pmask, SP_dmask,
SP_bmask, SP_cmask, FP_Dark, FP_Dark_Vars);

printf("exit code %d\n", Stat.I_status);

/* Send return value */
exit(Stat.I_status);

}
