/* $Id: cubesub_output.c,v 1.9 2003/02/18 19:21:57 fan Exp $ */


/*********************************************** 
cubesub_output.c

Purpose

Save the cubesub outputs to files.

Overview
Write the results of the computations as a FITS file.

Definitions of Variables

Input:
I_NX = Number of pixels in a row of the image
I_NY = Number of pixels in a column of the image
CP_Filename_FITS_In = Input Data File Name. 

Output:


Internal:
ffp_FITS = Handle to output FITS file.
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_String = String value to be written to header.
CP_Comment = String value to be written in comment field of header.


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "cubesub.h"
#include "cubesub_defs.h"


cubesub_output(
            Filenames	  *Fnames,
	    Constants     *Const,
            Dims          *Dim_In,
            Status	  *Stat,
            struct tm     *tmp_time,
            short         *SPP_dmask,
            float         *FPP_Data_In,
            float         *FPP_Vars_In)
{

int	I_fits_return_status=0;
int	I_Num_In;
int     string_length;
int     i;
float   F_Num_In;

char    CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_CARD];
char    CP_Key_Comment[FLEN_COMMENT], CP_Filename_In[MAX_FILENAME_LENGTH];
char    CP_String[FLEN_VALUE];
char    programname[FITS_HIS_LEN], versionstring[FITS_HIS_LEN];
char    format[10];

fitsfile 	*ffp_FITS_Out;
fitsfile        *ffp_Vars_Out;
fitsfile 	*ffp_FITS_In;
fitsfile        *ffp_Vars_In;
fitsfile 	*ffp_FITS_Dmask;

printf(asctime(tmp_time));

if(Stat->I_status) return;

/* Open dmask file. If found replace data.  */

if (strcmp(Fnames->CP_Filename_Dmask, ""))
   {
   fits_open_file( &ffp_FITS_Dmask,
                   Fnames->CP_Filename_Dmask,
                   READWRITE,
                   &I_fits_return_status);

/* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin CUBESUB module additions");
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "CUBESUB");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_FITS_Dmask, programname, versionstring, &I_fits_return_status);

   fits_write_img(ffp_FITS_Dmask,
                  TSHORT,
                  1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                  SPP_dmask,
                  &I_fits_return_status);		

   fits_flush_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
		
   fits_close_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
   }

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* write out the image data */

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_In,
                READWRITE,
                &I_fits_return_status);

fits_create_file( &ffp_FITS_Out,
                Fnames->CP_Filename_FITS_Out,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out,
                 &I_fits_return_status);

/* update BUNIT key value for operation modes other than 1 */
if (Const->Operation != 1) {
    sprintf(CP_Keyname, "%s", "BUNIT");
    sprintf(CP_Key_Comment, "%s", "");
    sprintf(CP_String, "%s", Const->Bunit) ;
    fits_update_key(ffp_FITS_Out, TSTRING, CP_Keyname, CP_String, CP_Key_Comment, &I_fits_return_status);
}

/* Add module specific keywords */

sprintf(CP_Comment, "%s", "Begin CUBESUB module additions");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "Dark cube subtraction performed");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

if (Const->Operation == 1)
    sprintf(CP_Comment, "%s", "Dark cube not scaled before operation");
else if (Const->Operation == 2)
    sprintf(CP_Comment, "%s", "Dark cube scaled by 1/GAIN before operation");
else if (Const->Operation == 3)
    sprintf(CP_Comment, "%s", "Dark cube scaled by Time/GAIN before operation");

fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

if(Const->QAheader) {
   sprintf(CP_Keyname, "%s", "CUAVGRSF");
   sprintf(CP_Comment, "%s", "");
   F_Num_In = Const->Cuavgrsf;
   fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "CUAVGRSL");
   sprintf(CP_Comment, "%s", "");
   F_Num_In = Const->Cuavgrsl;
   fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

}

sprintf(programname, "%s", "CUBESUB");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_In,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_In,
                &I_fits_return_status);

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

/* write out the uncertainty data if output filename specified */

if (strcmp(Fnames->CP_Filename_Vars_Out, "")) {

   if (Const->In_Vars)
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_Vars_In,
                      READWRITE,
                      &I_fits_return_status);
   else
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_FITS_In,
                      READWRITE,
                      &I_fits_return_status);

   fits_create_file( &ffp_Vars_Out,
                   Fnames->CP_Filename_Vars_Out,
                   &I_fits_return_status);

   if (I_fits_return_status)
   {Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_copy_header(ffp_Vars_In,
                    ffp_Vars_Out,
                    &I_fits_return_status);

   /* update BUNIT key value for operation modes other than 1 */
   if (Const->Operation != 1) {
       sprintf(CP_Keyname, "%s", "BUNIT");
       sprintf(CP_Key_Comment, "%s", "");
       sprintf(CP_String, "%s", Const->Bunit) ;
       fits_update_key(ffp_Vars_Out, TSTRING, CP_Keyname, CP_String, CP_Key_Comment, &I_fits_return_status);
   }

   /* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin CUBESUB module additions");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "This is an uncertainty image");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "Dark cube subtraction performed");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   if (Const->Operation == 1)
       sprintf(CP_Comment, "%s", "Dark cube not scaled before operation");
   else if (Const->Operation == 2)
       sprintf(CP_Comment, "%s", "Dark cube scaled by 1/GAIN before operation");
   else if (Const->Operation == 3)
       sprintf(CP_Comment, "%s", "Dark cube scaled by Time/GAIN before operation");

   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "CUBESUB");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_Vars_Out, programname, versionstring, &I_fits_return_status);

   fits_write_img( ffp_Vars_Out,
                   TFLOAT,
                   1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                   FPP_Vars_In,
                   &I_fits_return_status);

   fits_flush_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_In,
                   &I_fits_return_status);

}

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

if(Stat->I_Verbose)
printf("cubesub_output: Results written to file(s).\n");

}
