/* $Id: cubesub_parse_args.c,v 1.8 2002/09/04 19:56:47 fan Exp $ */


/*********************************************** 
cubesub_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output Probabilities File Name.
IP_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include <string.h>
#include "cubesub.h"
#include "cubesub_defs.h"


cubesub_parse_args(	
            int	        argc,
            char        **argv,
            Filenames   *Fnames,
	    Constants   *Const,
            Status	*Stat)
{
int	i=1;
float   xtest;

int     I_out_chk, I_in_chk, I_nl_chk;
int     I_dark_chk;
int     I_pair_err;

/* Run regardless of I_cubesub_status so that
   at least log file name will be known */

I_out_chk = 1;
I_in_chk = 1;
I_dark_chk = 1;
I_nl_chk = 1;
I_pair_err = 0;

while( i < argc ) {
     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               I_nl_chk = 0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            break;
          case 'd':      /* -d <dmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
            break;
          case 'i':      /* -i <input_image_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
	       else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
                  I_in_chk = 0;
               }
            }
            else {       /* input dark image file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Dark);
                  I_dark_chk = 0;
               }
            }
            break;
          case 'e':      /* -e <input_uncertainty_image_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Vars_In);
               }
            }
            else {       /* input uncertainty images of the dark file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Dark_Vars);
               }
            }
            break;
          case 'o':      /* -o <output_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out);
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
                  I_out_chk = 0;
               }
            }
            else {       /* output uncertainty image */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out);
                  sscanf(argv[i], "%s", &(Fnames->CP_Filename_Vars_Out[1]));
               }
            }
            break;
          case 's':      /* -s <scaling> */
            if ( ++i >= argc )
               I_pair_err = 1;
            else
               sscanf(argv[i], "%d", &(Const->Operation));
            break;
          case 'b':      /* -b <bmask file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Bmask);
            break;
          case 'c':      /* -c <cmask file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p')           /* for Pmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->PmaskFatal);
                  }
               }
            else if (argv[i][2] == 'd')      /* for Dmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->DmaskFatal);
                  }
               }
            else if (argv[i][2] == 'c')      /* for Cmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->CmaskFatal);
                  }
               }
            else if (argv[i][2] == 'b')      /* for Bmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->BmaskFatal);
                  }
               }
            else
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  printf("cubesub_parse_args: unknown argument\n");
               }
            break;
          case 'q':      /* QA header writer mode */
            Const->QAheader = 1;
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;
            break;
          default:
            printf("cubesub_parse_args: unknown argument\n");
       }
     }
     else {
       printf("cubesub_parse_args: command line syntax error\n");
     }
     i++;
}

if ( I_pair_err ) {
   printf("cubesub_parse_args: command line syntax error\n");
   Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   printf("Required arguments:\n\n");
   printf("-i1	input filename\n");
   printf("-i2	input dark cube file name\n");
   printf("-o1	output filename\n\n");
   printf("Optional arguments\n\n");
   printf("-n	namelist filename\n");
   printf("-e1	uncertainty image filename for input file\n");
   printf("-e2	uncertainty image filename for input dark cube file\n");
   printf("-o2	output uncertainty filename\n");
   printf("-s	operation mode: 1=no scaling, 2=scale dark by 1/GAIN, 3=scale dark by T_INT*layer/GAIN\n");
   printf("-d	dmask filename 'DCE pixel status'\n");
   printf("-p	pmask filename 'hardware status'\n");
   printf("-b	bmask filename 'BCD status'\n");
   printf("-c	cmask filename calibration status\n");
   printf("-fp	pmask fatal bit template\n");
   printf("-fd	dmask fatal bit template\n");
   printf("-fc	cmask fatal bit template\n");
   printf("-fb	bmask fatal bit template\n");
   printf("-q	QA header update mode on if specified\n");
   printf("-v	puts the code in verbose mode\n\n");
}

if ((I_in_chk || I_dark_chk || I_out_chk) && I_nl_chk) {
   printf("Error in command line arguments.\n\n");
   printf("Required arguments:\n\n");
   printf("-i1	input filename\n");
   printf("-i2	input dark cube file name\n");
   printf("-o1	output filename\n\n");
   printf("Optional arguments\n\n");
   printf("-n	namelist filename\n");
   printf("-e1	uncertainty image filename for input file\n");
   printf("-e2	uncertainty image filename for input dark cube file\n");
   printf("-o2	output uncertainty filename\n");
   printf("-s	operation mode: 1=no scaling, 2=scale dark by 1/GAIN, 3=scale dark by T_INT*layer/GAIN\n");
   printf("-d	dmask filename 'DCE pixel status'\n");
   printf("-p	pmask filename 'hardware status'\n");
   printf("-b	bmask filename 'BCD status'\n");
   printf("-c	cmask filename calibration status\n");
   printf("-fp	pmask fatal bit template\n");
   printf("-fd	dmask fatal bit template\n");
   printf("-fc	cmask fatal bit template\n");
   printf("-fb	bmask fatal bit template\n");
   printf("-q	QA header updatE mode on if specified\n");
   printf("-v	puts the code in verbose mode\n\n");
   Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
}

if(strcmp(Fnames->CP_Filename_Vars_In, "")) Const->In_Vars = 1;

if(Stat->I_Verbose) {
printf("cubesub_parse_args: Namelist File = \t%s\n",
      Fnames->CP_Filename_Namelist);
printf("cubesub_parse_args: Input File = \t%s\n", 
      Fnames->CP_Filename_FITS_In);
printf("cubesub_parse_args: Input Dark Cube File = \t%s\n",
      Fnames->CP_Filename_Dark);
printf("cubesub_parse_args: Input Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Vars_In);
printf("cubesub_parse_args: Input Dark Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Dark_Vars);
printf("cubesub_parse_args: Pmask File = \t%s\n",
      Fnames->CP_Filename_Pmask);
printf("cubesub_parse_args: Dmask File = \t%s\n",
      Fnames->CP_Filename_Dmask);
printf("cubesub_parse_args: Bmask File = \t%s\n",
      Fnames->CP_Filename_Bmask);
printf("cubesub_parse_args: Cmask File = \t%s\n",
      Fnames->CP_Filename_Cmask);
printf("cubesub_parse_args: Pmask Fatal = \t%d\n",
      Const->PmaskFatal);
printf("cubesub_parse_args: Dmask Fatal = \t%d\n",
      Const->DmaskFatal);
printf("cubesub_parse_args: Cmask Fatal = \t%d\n",
      Const->CmaskFatal);
printf("cubesub_parse_args: Bmask Fatal = \t%d\n",
      Const->BmaskFatal);
printf("cubesub_parse_args: Operation mode = \t%d\n",
      Const->Operation);
printf("cubesub_parse_args: Ouput File = \t%s\n", 
      Fnames->CP_Filename_FITS_Out);
printf("cubesub_parse_args: Ouput Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Vars_Out);
printf("cubesub_parse_args: QA Header Update Mode = \t%d\n",
      Const->QAheader);
printf("cubesub_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
}

}
