/* $Id: cubesub_compute_results.c,v 1.9 2004/05/13 18:21:07 fan Exp $ */


/*********************************************** 
cubesub_compute_results.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cubesub.h"
#include "cubesub_defs.h"
/* Added TYB - 6/1/09 */
#include "nanvalue.h"


cubesub_compute_results(	
            Filenames      *Fnames,
	    Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
            float          *FP_Vars_In,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask,
            float          *FP_Dark,
            float          *FP_Dark_Vars)
{
int	i, j, m, nf, nl;
int     itemp;
int     I_fits_return_status=0;
float   nsratf, nsratl;
float   *FP_Cubesub;

if(Stat->I_status) return;

FP_Cubesub = (float *) calloc(Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
          sizeof(float));

for (i=0; i<=Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ-1; ++i)
    {
    if (Const->Operation == 1)
       *(FP_Cubesub +i) = *(FP_Dark +i);
    else if (Const->Operation == 2)
       {
       *(FP_Cubesub +i) = (*(FP_Dark +i) / Const->Gain );
       *(FP_Dark_Vars +i) /= Const->Gain; 
       strcpy(Const->Bunit, "DN");
       }
    else if (Const->Operation == 3)
       {
       *(FP_Cubesub +i) = (*(FP_Dark +i) * Const->T_Int * ((int)((float)i/(float)(Dim_In->D_NX*Dim_In->D_NY)) + 1) / Const->Gain );
       *(FP_Dark_Vars +i) = (*(FP_Dark_Vars +i) * Const->T_Int * ((int)((float)i/(float)(Dim_In->D_NX*Dim_In->D_NY)) + 1) / Const->Gain );
       strcpy(Const->Bunit, "DN");
       }
    }

for (i=0; i<=Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ-1; ++i)
    {
    if ( (*(SP_pmask + i%(Dim_In->I_NX*Dim_In->I_NY)) & Const->PmaskFatal) == 0 &&
         (*(SP_dmask + i) & Const->DmaskFatal) == 0 && 
         (*(SP_cmask + i) & Const->CmaskFatal) == 0 &&
         (*(SP_bmask + i%(Dim_In->I_NX*Dim_In->I_NY)) & Const->BmaskFatal) == 0 )
       {
       *(FP_Data_In + i) = *(FP_Data_In + i) - (*(FP_Cubesub +i));
       *(FP_Vars_In + i) = sqrt(*(FP_Vars_In + i)*(*(FP_Vars_In + i)) +
                                *(FP_Dark_Vars + i)*(*(FP_Dark_Vars + i)));
       }
/*    else
       {
//  Replace with NANVALUE - TYB 6/1/09
       *(FP_Vars_In + i) = NANVALUE;
//       *(FP_Vars_In + i) = atof("NaN");

       } 
*/
    }

/* compute QA items if warranted */

if (Const->QAheader) {
   nf = 0;
   nl = 0;
   nsratf = 0.0;
   nsratl = 0.0;
   for(j=0; j<Dim_In->I_NX*Dim_In->I_NY; j++) {
      if ( (*(SP_pmask + j) & Const->PmaskFatal) == 0 &&
         (*(SP_dmask + j) & Const->DmaskFatal) == 0 &&
         (*(SP_cmask + j) & Const->CmaskFatal) == 0 &&
         (*(SP_bmask + j) & Const->BmaskFatal) == 0 ) {

         if( !(iznand((double) *(FP_Data_In + j))) && !(iznand((double) *(FP_Vars_In + j))) && *(FP_Data_In + j) != 0.0 ) {
            nsratf += *(FP_Vars_In + j)/(*(FP_Data_In + j));
            nf++;
         }
      }
      if ( (*(SP_pmask + j) & Const->PmaskFatal) == 0 &&
         (*(SP_dmask + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j) & Const->DmaskFatal) == 0 &&
         (*(SP_cmask + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j) & Const->CmaskFatal) == 0 &&
         (*(SP_bmask + j) & Const->BmaskFatal) == 0) {

         if( !(iznand((double) *(FP_Data_In + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j))) && !(iznand((double) *(FP_Vars_In + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j))) && *(FP_Data_In + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j) != 0.0 ) {
            nsratl += *(FP_Vars_In + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j)/(*(FP_Data_In + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + j));
            nl++;
         }
      }
   }
   if (nf != 0) Const->Cuavgrsf = nsratf/(float)nf;
   if (nl != 0) Const->Cuavgrsl = nsratl/(float)nl;
printf("values are %f %f %f %f\n", Const->Cuavgrsf, Const->Cuavgrsl, nsratf, nsratl);
}

free(FP_Cubesub);

if(Stat->I_Verbose)
printf("Cubesub_compute_results: Numerical Computations finished.\n");

}
