/* $Id: darkbase_compute.c,v 1.8 2004/05/13 18:30:11 fan Exp $ */


/*********************************************** 
darkbase_compute.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "darkbase.h"
#include "darkbase_errcodes.h"
#include "darkbase_defs.h"

darkbase_compute(	
            Filenames      *Fnames,
            Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
            float          *FP_Vars_In,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask,
            short          *SP_umask) {
 int	 i, j, l;
 int     trimfactflag1, trimfactflag2, trimfactflag3, trimfactflag4;
 int     do_chan1, do_chan2, do_chan3, do_chan4;
 long    nstart=1;
 long    nchan1, nchan2, nchan3, nchan4;
 double  f1, f2;
 double  *chan1, *chan2, *chan3, *chan4;
 double  trim_avg_chan1, trim_avg_chan2, trim_avg_chan3, trim_avg_chan4;
 double  stddev_chan1, stddev_chan2, stddev_chan3, stddev_chan4;
 double  datascale_chan1, datascale_chan2, datascale_chan3, datascale_chan4;
 double  facttotptr1, facttotptr2, facttotptr3, facttotptr4;

 if(Stat->I_status) return;

 do_chan1 = 1;
 do_chan2 = 1;
 do_chan3 = 1;
 do_chan4 = 1;

 f1 = Const->Trim_Frac;
 f2 = Const->Trim_STD_Frac;

 chan1 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan2 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan3 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan4 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));

 if (chan1 == NULL || chan2 == NULL || chan3 == NULL || chan4 == NULL) {
    printf("droopres_compute: memory allocation failed\n");
    Stat->I_status = COMPUTE|MALLOC_FAILED;
    return;
 }

 /* obtain channel-dependent pixel values at zero exposure time in defined box in the second and third layers */
 nchan1 = nchan2 = nchan3 = nchan4 = 0;
 for (i=0; i<Dim_In->I_NY; ++i) {
    for (j=0; j<Dim_In->I_NX; ++j) {
       if ( *(SP_umask + j + i*Dim_In->I_NX) == 1 &&
            ((*(SP_dmask + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
            ((*(SP_dmask + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
            ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
            ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
            ((*(SP_cmask + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0) &&
            ((*(SP_cmask + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0) ) {
          if (j%4 == 0) {            /* channel 1 */
             chan1[nchan1] = 3*((double) *(FP_Data_In + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY)) - 2*((double) *(FP_Data_In + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY));
             nchan1++;
          }
          else if (j%4 == 1) {            /* channel 2 */
             chan2[nchan2] = 3*((double) *(FP_Data_In + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY)) - 2*((double) *(FP_Data_In + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY));
             nchan2++;
          }
          else if (j%4 == 2) {            /* channel 3 */
             chan3[nchan3] = 3*((double) *(FP_Data_In + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY)) - 2*((double) *(FP_Data_In + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY));
             nchan3++;
          }
          else if (j%4 == 3) {            /* channel 4 */
             chan4[nchan4] = 3*((double) *(FP_Data_In + j + i*Dim_In->I_NX + Dim_In->I_NX*Dim_In->I_NY)) - 2*((double) *(FP_Data_In + j + i*Dim_In->I_NX + 2*Dim_In->I_NX*Dim_In->I_NY));
             nchan4++;
          }
       }
    }
 }

 if(Stat->I_Verbose) {
    printf("%d data points used for channel 1 from the first 2 layers\n", nchan1);
    printf("%d data points used for channel 2 from the first 2 layers\n", nchan2);
    printf("%d data points used for channel 3 from the first 2 layers\n", nchan3);
    printf("%d data points used for channel 4 from the first 2 layers\n\n", nchan4);
 }

/* if (nchan1 == 0 && nchan2 == 0 && nchan3 == 0 && nchan4 == 0) {
    Stat->I_status = COMPUTE|NO_DATA;
    printf("darkbase_compute: no data to estimate baseline\n");
    return;
 }
*/

 if (nchan1 == 0) {
    do_chan1 = 0;
    trim_avg_chan1 = 0.0;
    stddev_chan1 = 0.0;
    datascale_chan1 = 0.0;
    printf("darkbase_compute: no data in read channel 1 to estimate baseline\n");
 }

 if (nchan2 == 0) {
    do_chan2 = 0;
    trim_avg_chan2 = 0.0;  
    stddev_chan2 = 0.0;
    datascale_chan2 = 0.0;
    printf("darkbase_compute: no data in read channel 2 to estimate baseline\n"
);
 }

 if (nchan3 == 0) {
    do_chan3 = 0;
    trim_avg_chan3 = 0.0;  
    stddev_chan3 = 0.0;
    datascale_chan3 = 0.0;
    printf("darkbase_compute: no data in read channel 3 to estimate baseline\n"
);
 }

 if (nchan4 == 0) {
    do_chan4 = 0;
    trim_avg_chan4 = 0.0;  
    stddev_chan4 = 0.0;
    datascale_chan4 = 0.0;
    printf("darkbase_compute: no data in read channel 4 to estimate baseline\n"
);
 }
  

 /* obtain median value and error for each channel */
 if (do_chan1) {
    trimfactflag1 = 0;
    trimmedstatistics(chan1, nstart, nchan1, f1, f2, &trimfactflag1, &facttotptr1, &trim_avg_chan1, &stddev_chan1, &datascale_chan1);
 }

 if (do_chan2) {
    trimfactflag2 = 0;
    trimmedstatistics(chan2, nstart, nchan2, f1, f2, &trimfactflag2, &facttotptr2, &trim_avg_chan2, &stddev_chan2, &datascale_chan2);
 }

 if (do_chan3) {
    trimfactflag3 = 0;
    trimmedstatistics(chan3, nstart, nchan3, f1, f2, &trimfactflag3, &facttotptr3, &trim_avg_chan3, &stddev_chan3, &datascale_chan3);
 }

 if (do_chan4) {
    trimfactflag4 = 0;
    trimmedstatistics(chan4, nstart, nchan4, f1, f2, &trimfactflag4, &facttotptr4, &trim_avg_chan4, &stddev_chan4, &datascale_chan4);
 }

 if (Stat->I_Verbose) {
    printf("darkbase_compute: found for %d data points in channel 1 trimmed-average %f, trimmed standard deviation %f, and datascale %f\n", nchan1, trim_avg_chan1, stddev_chan1, datascale_chan1);
    printf("darkbase_compute: found for %d data points in channel 2 trimmed-average %f, trimmed standard deviation %f, and datascale %f\n", nchan2, trim_avg_chan2, stddev_chan2, datascale_chan2);
    printf("darkbase_compute: found for %d data points in channel 3 trimmed-average %f, trimmed standard deviation %f, and datascale %f\n", nchan3, trim_avg_chan3, stddev_chan3, datascale_chan3);
    printf("darkbase_compute: found for %d data points in channel 4 trimmed-average %f, trimmed standard deviation %f, and datascale %f\n\n", nchan4, trim_avg_chan4, stddev_chan4, datascale_chan4);
 }

 Const->Baseline_Chan1 = trim_avg_chan1;
 Const->Baseline_Chan2 = trim_avg_chan2;
 Const->Baseline_Chan3 = trim_avg_chan3;
 Const->Baseline_Chan4 = trim_avg_chan4;
 Const->Baseunc_Chan1 = stddev_chan1;
 Const->Baseunc_Chan2 = stddev_chan2;
 Const->Baseunc_Chan3 = stddev_chan3;
 Const->Baseunc_Chan4 = stddev_chan4;

 printf("darkbase_compute: found baseline %f and uncertainty %f for read-channel 1\n", Const->Baseline_Chan1, Const->Baseunc_Chan1);
 printf("darkbase_compute: found baseline %f and uncertainty %f for read-channel 2\n", Const->Baseline_Chan2, Const->Baseunc_Chan2);
 printf("darkbase_compute: found baseline %f and uncertainty %f for read-channel 3\n", Const->Baseline_Chan3, Const->Baseunc_Chan3);
 printf("darkbase_compute: found baseline %f and uncertainty %f for read-channel 4\n\n", Const->Baseline_Chan4, Const->Baseunc_Chan4);

 /* subtract channel-dependent pixel trimmed average values from all pixels */
 for (l=0; l<=Dim_In->I_NZ-1; ++l) {  /* loop thru all layers */
    for (j=0; j < Dim_In->I_NX; ++j) {
       for (i=0; i<Dim_In->I_NY; ++i) {
           if (j%4 == 0) {
              if (((*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
                  ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
                  ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
                  ((*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0)) {
                 *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= Const->Baseline_Chan1;
                 *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                     (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + Const->Baseunc_Chan1*Const->Baseunc_Chan1));
              }
              else {
                 if ( *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & 1024 == 1024 )
                    printf("Warning: bit 10 already set in dmask\n");
                 else
                    *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) += 1024;
              }
           }
           else if (j%4 == 1) {
              if (((*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
                  ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
                  ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
                  ((*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0)) {
                 *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= Const->Baseline_Chan2;
                 *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                     (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + Const->Baseunc_Chan2*Const->Baseunc_Chan2));
              }
              else {
                 if ( *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & 1024 == 1024 )
                    printf("Warning: bit 10 already set in dmask\n");
                 else
                    *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) += 1024;
              }
           }
           else if (j%4 == 2) {
              if (((*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
                  ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
                  ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
                  ((*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0)) {
                 *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= Const->Baseline_Chan3;
                 *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                     (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + Const->Baseunc_Chan3*Const->Baseunc_Chan3));
              }
              else {
                 if ( *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & 1024 == 1024 )
                    printf("Warning: bit 10 already set in dmask\n");
                 else
                    *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) += 1024;
              }
           }
           else if (j%4 == 3) {
              if (((*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
                  ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
                  ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
                  ((*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0)) {
                 *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= Const->Baseline_Chan4;
                 *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                     (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + Const->Baseunc_Chan4*Const->Baseunc_Chan4));
              }
              else {
                 if ( *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & 1024 == 1024 )
                    printf("Warning: bit 10 already set in dmask\n");
                 else
                    *(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) += 1024;
              }
           }
       }
    }
 }

 free(chan1);
 free(chan2);
 free(chan3);
 free(chan4);

 printf("darkbase_compute: Numerical Computations finished.\n");

}
