/* $Id: darkbase_error_handler.c,v 1.6 2002/09/05 00:15:17 fan Exp $ */


/*********************************************** 
darkbase_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of darkbase functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_darkbase_message = Code of error that occurred.
I_darkbase_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "darkbase.h"
#include "darkbase_errcodes.h"
#include "darkbase_defs.h"


darkbase_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_darkbase_message;
int	I_darkbase_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_darkbase_message = RIGHT_MASK&(Stat->I_status);
I_darkbase_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_darkbase_message&FITS_MSG_MASK)	{
I_darkbase_message -= FITS_MSG_MASK;
}

darkbase_decode_message(I_darkbase_message, CP_Message);
darkbase_decode_fcnname(I_darkbase_root, CP_Function);

if (Stat->I_status)
{
printf("darkbase Exception Message      0x%04x: %s.\n", I_darkbase_message, CP_Message);
printf("darkbase Exception in Function  0x%04x: %s.\n", I_darkbase_root, CP_Function);
}


I_Time = clock();
printf("Processor Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("darkbase_error_handler: Exception Handler finished.\n");

}
