/* $Id: darkbase_output.c,v 1.7 2003/02/18 18:48:21 fan Exp $ */


/*********************************************** 
darkbase_output.c

Purpose

Save the darkbase outputs to files.

Overview
Write the results of the computations as a FITS file.

Definitions of Variables

Input:
Const = structure to hold contant parameters
I_NX = Number of pixels in a row of the image
I_NY = Number of pixels in a column of the image
I_NZ = Number of image layers in a data cube
CP_Filename_FITS_In = Input Data File Name. 
FPP_Data_In = Pointer to Hold Input Data Array.
SPP_dmask = Pointer to Hold Input Dmask Array

Output:


Internal:
ffp_FITS_Out = Handle to output FITS file.
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_String = String value to be written to header.
CP_Comment = String value to be written in comment field of header.


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkbase.h"
#include "darkbase_errcodes.h"
#include "darkbase_defs.h"


darkbase_output(
            Filenames	  *Fnames,
            Dims          *Dim_In,
	    Constants     *Const,
            Status	  *Stat,
            struct tm     *tmp_time,
            short         *SPP_dmask,
            float         *FPP_Data_In,
            float         *FPP_Vars_In)
{

int	I_fits_return_status=0;
int	I_Num_In;
int     string_length;
int     i;

float   F_Num_In;

char    CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT], CP_Filename_In[MAX_FILENAME_LENGTH];
char    CP_String[22];
char    programname[32], versionstring[32];
char    format[10];

fitsfile 	*ffp_FITS_Out;
fitsfile        *ffp_Vars_Out;
fitsfile 	*ffp_FITS_In;
fitsfile        *ffp_Vars_In;
fitsfile 	*ffp_FITS_Dmask;

printf(asctime(tmp_time));

if(Stat->I_status) return;

/* Open dmask file. If found replace data.  */

if (strcmp(Fnames->CP_Filename_Dmask, ""))
   {
   fits_open_file( &ffp_FITS_Dmask,
                   Fnames->CP_Filename_Dmask,
                   READWRITE,
                   &I_fits_return_status);

/* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin DARKBASE module additions");
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "Reference dark baseline computed using Dmask");
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);
   
   
   sprintf(programname, "%s", "DARKBASE");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_FITS_Dmask, programname, versionstring, &I_fits_return_status);

   fits_write_img(ffp_FITS_Dmask,
                  TSHORT,
                  1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                  SPP_dmask,
                  &I_fits_return_status);		

   fits_flush_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
		
   fits_close_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
   }

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* write out the image data */

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_In,
                READWRITE,
                &I_fits_return_status);

fits_create_file( &ffp_FITS_Out,
                Fnames->CP_Filename_FITS_Out,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "%s", "Begin DARKBASE module additions");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "Reference dark baseline computed and subtracted");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BASECH1");
sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 1");
F_Num_In = Const->Baseline_Chan1;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BASECH2");
sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 2");
F_Num_In = Const->Baseline_Chan2;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BASECH3");
sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 3");
F_Num_In = Const->Baseline_Chan3;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BASECH4");
sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 4");
F_Num_In = Const->Baseline_Chan4;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BUNCCH1");
sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 1");
F_Num_In = Const->Baseunc_Chan1;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BUNCCH2");
sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 2");
F_Num_In = Const->Baseunc_Chan2;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BUNCCH3");
sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 3");
F_Num_In = Const->Baseunc_Chan3;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "BUNCCH4");
sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 4");
F_Num_In = Const->Baseunc_Chan4;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(programname, "%s", "DARKBASE");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_In,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_In,
                &I_fits_return_status);

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

/* write out the uncertainty data */

if (strcmp(Fnames->CP_Filename_Vars_Out, "")) {

   if (Const->In_Vars)
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_Vars_In,
                      READWRITE,
                      &I_fits_return_status);
   else
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_FITS_In,
                      READWRITE,
                      &I_fits_return_status);

   fits_create_file( &ffp_Vars_Out,
                   Fnames->CP_Filename_Vars_Out,
                   &I_fits_return_status);

   if (I_fits_return_status)
   {Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_copy_header(ffp_Vars_In,
                    ffp_Vars_Out,
                    &I_fits_return_status);

   /* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin DARKBASE module additions");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "This is an uncertainty image");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "Reference dark baseline computed and subtracted");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BASECH1");
   sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 1");
   F_Num_In = Const->Baseline_Chan1;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BASECH2");
   sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 2");
   F_Num_In = Const->Baseline_Chan2;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BASECH3");
   sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 3");
   F_Num_In = Const->Baseline_Chan3;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BASECH4");
   sprintf(CP_Comment, "%s", "reference dark baseline for read-channel 4");
   F_Num_In = Const->Baseline_Chan4;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BUNCCH1");
   sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 1");
   F_Num_In = Const->Baseunc_Chan1;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BUNCCH2");
   sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 2");
   F_Num_In = Const->Baseunc_Chan2;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BUNCCH3");
   sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 3");
   F_Num_In = Const->Baseunc_Chan3;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BUNCCH4");
   sprintf(CP_Comment, "%s", "baseline uncertainty for read-channel 4");
   F_Num_In = Const->Baseunc_Chan4;
   fits_write_key(ffp_Vars_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

   sprintf(programname, "%s", "DARKBASE");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_Vars_Out, programname, versionstring, &I_fits_return_status);

   fits_write_img( ffp_Vars_Out,
                   TFLOAT,
                   1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                   FPP_Vars_In,
                   &I_fits_return_status);

   fits_flush_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_In,
                   &I_fits_return_status);

}

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

if(Stat->I_Verbose)
printf("darkbase_output: Results written to file(s).\n");

}
