/* $Id: darkbase_parse_args.c,v 1.6 2002/09/05 00:15:17 fan Exp $ */


/*********************************************** 
darkbase_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output FITS File Name.
CP_Filename_Pmask = Input Pmask.
CP_Filename_Dmask = Input Dmask.
CP_Filename_Bmask = Input Bmask.
CP_Filename_Cmask = Input Cmask.
Const = Structure to Hold Constant Parameters.
I_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include <string.h>
#include "darkbase.h"
#include "darkbase_errcodes.h"
#include "darkbase_defs.h"


darkbase_parse_args(	
            int	        argc,
            char        **argv,
            Constants   *Const,
            Filenames   *Fnames,
            Status	*Stat) {

   int	i=1;

   int     I_out_chk, I_in_chk;
   int     I_umask_chk, I_nl_chk;
   int     I_pair_err;

   /* flags to check the completeness of the switch-value pairs */
   I_in_chk = 1;
   I_out_chk = 1;
   I_umask_chk = 1;
   I_nl_chk = 1;
   I_pair_err = 0;

   while( i < argc ) {

     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               I_nl_chk =0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'i':      /* -i <input_image_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  I_in_chk = 0;
                  sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
               }
            }
            else {       /* -i2 <input uncertainty filename> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Vars_In);
               }
            }
            break;
          case 'o':      /* -o <output_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  I_out_chk = 0;
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
               }
            }
            else {       /* -o2 <output uncertainty filename> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_Vars_Out[1]));
               }
            }
            break;
          case 't':      /* -t <trim-average percentage> */
            if (argv[i][2] == '1')           /* trim fraction */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%f", &(Const->Trim_Frac));
                  }
               }
            else                             /* trim fraction for standard deviation */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%f", &(Const->Trim_STD_Frac));
                  }
               }
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p')           /* for Pmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->PmaskFatal));
                  }
               }
            else if (argv[i][2] == 'd')      /* for Dmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->DmaskFatal));
                  }
               }
            else if (argv[i][2] == 'c')      /* for Cmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->CmaskFatal));
                  }
               }
            else if (argv[i][2] == 'b')      /* for Bmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->BmaskFatal));
                  }
               }
            else
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  printf("darkbase_parse_args: unknown argument\n");
               }
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            break;
          case 'd':      /* -d <dmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
            break;
          case 'b':      /* -b <bmask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Bmask);
            break;
          case 'c':      /* -c <cmask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
            break;
          case 'u':      /* -u <umask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               I_umask_chk = 0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Umask);
            }
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;
            break;
          default:
            printf("darkbase_parse_args: unknown argument\n");
       }
     }
     else {
       I_pair_err = 1;
     }
     i++;
   }

   if ( I_pair_err ) {
      printf("darkbase_parse_args: command line syntax error\n");
      darkbase_usage();
      Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   }


   if ( (I_in_chk || I_umask_chk || I_out_chk) && I_nl_chk ) {
      printf("Error in command line arguments; required input not complete.\n\n");
      darkbase_usage();
      Stat->I_status = PARSE_ARGS|REQ_ARGS_INCOMP;
   }

   if (strcmp(Fnames->CP_Filename_Vars_In, "")) Const->In_Vars = 1;

   if(Stat->I_Verbose)	{
      printf("darkbase_parse_args: Input Namelist File = \t%s\n", Fnames->CP_Filename_Namelist);
      printf("darkbase_parse_args: Input Image File = \t%s\n", Fnames->CP_Filename_FITS_In);
      printf("darkbase_parse_args: Input Uncertainty Image File = \t%s\n", Fnames->CP_Filename_Vars_In);
      printf("darkbase_parse_args: Ouput Image File = \t%s\n", Fnames->CP_Filename_FITS_Out);
      printf("darkbase_parse_args: Ouput Uncertainty Image File = \t%s\n", Fnames->CP_Filename_Vars_Out);
      printf("darkbase_parse_args: Trim Fraction is %f\n", Const->Trim_Frac);
      printf("darkbase_parse_args: Trim Fraction for Standard Deviation is %f\n", Const->Trim_STD_Frac);
      printf("darkbase_parse_args: Umask File = \t%s\n", Fnames->CP_Filename_Umask);
      printf("darkbase_parse_args: Pmask File = \t%s\n", Fnames->CP_Filename_Pmask);
      printf("darkbase_parse_args: Dmask File = \t%s\n", Fnames->CP_Filename_Dmask);
      printf("darkbase_parse_args: Bmask File = \t%s\n", Fnames->CP_Filename_Bmask);
      printf("darkbase_parse_args: Cmask File = \t%s\n", Fnames->CP_Filename_Cmask);
      printf("darkbase_parse_args: Pmask Fatal = \t%d\n", Const->PmaskFatal);
      printf("darkbase_parse_args: Dmask Fatal = \t%d\n", Const->DmaskFatal);
      printf("darkbase_parse_args: Cmask Fatal = \t%d\n", Const->CmaskFatal);
      printf("darkbase_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
   }
}
