/* $Id: darkdrift_compute_results.c,v 1.8 2007/08/08 20:25:28 laher Exp $ */


/*********************************************** 
darkdrift_compute_results.c

Purpose

Main computational processing loop of program.

************************************************/

#include <stdio.h>
#include <string.h>
#include "darkdrift_errcodes.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"
#include "datascale_estimate.h"
/* Includes for mathc90 sort routine. */
#include "mathc90.h"
#include <stdlib.h>
#include <math.h>
#include "fcrt.h"


void darkdrift_compute_results(DRI_Constants		*DRIP_Const,
			       DRI_Filenames	        *DRIP_Fnames,
			       DRI_FITSinfo		*DRIP_FITS,
			       double			*DP_Data_Image1,
			       double			*DP_Data_Image2,
			       double			*DP_Data_Image3,
			       double			*DP_Data_Image4,
			       double			*DP_Data_In_Unc_Image,
			       int                      *DP_Data_Image_GMask,
			       DRI_Computation		*DRIP_Comp,
			       DRI_Status		*DRIP_Stat)
{
int	i, j, k, ii, jj, kk, kkk, iindex, jindex;
int	I_Length_X, I_Length_Y, I_Num_Frames, I_Operation,I_Img_Pixels;
int	I_Tot_Pixels, I_Index, I_Tot_PixelsOut,I_Index_Inp,tmp,I_Index_Stat;
int     kstart, kend, I_Data_Plane, I_RO_channel_to_exclude, I_Mult_Corr;
int     kstart2, kend2, doff;
int     kstart3, kend3;
int     kstart4, kend4;
int     kstartg, kendg;
int     GMaskMax, nfatal;
int     *Transposed_GMask;
int     inputuncflag = 0;

double   *Input_Array, *Aux_Array;
double   *Transposed_Image, *Transposed_Image2, *Transposed_Image3;
double   *Transposed_Image4, *Transposed_In_Unc_Image;
double   *Transposed_Output_Image, *Transposed_Output_Image2; 
double   *Transposed_Output_Image3;
double	 *DP_Output_Array, *DP_Output_Array2, *DP_Output_Array3, rmse;
double background, slope, yint, slope2, yint2, slope3, yint3, term1, term2;

long l, lzero = 0, lone = 1;

/* For debug only */
FILE	*fp_data;



int     nsize;
int     n, nn, nbadpixels[NCALCDOMS],nsamps[NCALCDOMS];
int     istart[NCALCDOMS], iend[NCALCDOMS], istep[NCALCDOMS];
int     jstart[NCALCDOMS], jend[NCALCDOMS], jstep[NCALCDOMS];
double   median[NCALCDOMS],stddev[NCALCDOMS],datascale[NCALCDOMS];
double   correction[NCALCDOMS],uncertainty[NCALCDOMS];

int      nbadpixels1[NCALCDOMS],nsamps1[NCALCDOMS];
double   median1[NCALCDOMS],stddev1[NCALCDOMS],datascale1[NCALCDOMS];

int      nbadpixels2[NCALCDOMS],nsamps2[NCALCDOMS];
double   median2[NCALCDOMS],stddev2[NCALCDOMS],datascale2[NCALCDOMS];

double  meanval,stddevval,datascaleval,f1,f2;

double  temp, compensationtemp, pixelvalue;


if (DRIP_Stat->I_status) return;
 
if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,""))
   inputuncflag = 1;

/* Copy structure info to local variables. */

I_Length_X = DRIP_FITS->I_Length_X;
I_Length_Y = DRIP_FITS->I_Length_Y;
I_Num_Frames = DRIP_FITS->I_Num_Frames;
I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;
I_Img_Pixels = I_Length_X*I_Length_Y;
I_Operation = DRIP_FITS->I_Operation;
I_Data_Plane = DRIP_FITS->I_Data_Plane;
I_RO_channel_to_exclude = DRIP_FITS->I_RO_channel_to_exclude;
I_Mult_Corr = DRIP_FITS->I_Mult_Corr;
GMaskMax = DRIP_Const->GMaskMax;


/* Create local arrays. */

Input_Array = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Input_Array==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Aux_Array = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Aux_Array==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_GMask = (int *) calloc(I_Img_Pixels, sizeof(int));
if(Transposed_GMask==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Image = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Image==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Image2 = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Image2==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Image3 = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Image3==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Image4 = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Image4==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_In_Unc_Image = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_In_Unc_Image==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Output_Image = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Output_Image==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Output_Image2 = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Output_Image2==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}

Transposed_Output_Image3 = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Transposed_Output_Image3==NULL)
{DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}


/* Check image attributes. */

if (DRIP_FITS->I_Operation == 2 || DRIP_FITS->I_Operation == 4) {

   if (I_Length_X != DRIP_FITS->I_Length_X2)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_Y != DRIP_FITS->I_Length_Y2)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_X != DRIP_FITS->I_Length_X4)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_Y != DRIP_FITS->I_Length_Y4)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

}

if (DRIP_FITS->I_Operation == 3 || DRIP_FITS->I_Operation == 4) {

   if (I_Length_X != DRIP_FITS->I_Length_X3)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_Y != DRIP_FITS->I_Length_Y3)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_X != DRIP_FITS->I_Length_X4)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

   if (I_Length_Y != DRIP_FITS->I_Length_Y4)
      { DRIP_Stat->I_status = COMPUTE_RES|INCONSIST_IMAGE_ATTRIB; return; }

}

/* Set controls for number image-data planes to treat. */

if (I_Data_Plane == 3) 
{
   kstart = I_Num_Frames - 1;
   kend = I_Num_Frames;
   I_Num_Frames = 1;
   DRIP_FITS->I_Num_Frames = I_Num_Frames;
   I_Tot_PixelsOut = I_Length_X*I_Length_Y*I_Num_Frames;
}
else if (I_Data_Plane == 2)
{
   kstart = 0;
   kend = 1;
   I_Num_Frames = 1;
   DRIP_FITS->I_Num_Frames = I_Num_Frames;
}
else
{
   kstart = 0;
   kend = I_Num_Frames;
} 

kstart2 = 0;
kend2 = 1;

kstart3 = 0;
kend3 = 1;

kstart4 = 0;
kend4 = 1;

kstartg = 0;
kendg = 1;


/* 
   Transpose image if specified.  NOTE: SINCE IMAGE IS READ IN AND STORED
   IN TRANSPOSED FORMAT, UN-TRANSPOSING IS DONE HERE IF  
   DRIP_FITS->I_Transpose == 1
*/


if (DRIP_FITS->I_Transpose == 1) {

      for (k = kstart; k < kend; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_Image[I_Index] = DP_Data_Image1[I_Index_Inp];

            }
      }

   DP_Data_Image1 = Transposed_Image;

   tmp = DRIP_FITS->I_Length_X;
   DRIP_FITS->I_Length_X = DRIP_FITS->I_Length_Y;
   DRIP_FITS->I_Length_Y = tmp;

   I_Length_X = DRIP_FITS->I_Length_X;
   I_Length_Y = DRIP_FITS->I_Length_Y;

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,"")) {

      for (k = kstart; k < kend; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_In_Unc_Image[I_Index] = 
                  DP_Data_In_Unc_Image[I_Index_Inp];

            }
      }

      DP_Data_In_Unc_Image = Transposed_In_Unc_Image;

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) {

      for (k = kstart2; k < kend2; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_Image2[I_Index] = DP_Data_Image2[I_Index_Inp];

            }
      }

      DP_Data_Image2 = Transposed_Image2;

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {

      for (k = kstart3; k < kend3; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_Image3[I_Index] = DP_Data_Image3[I_Index_Inp];

            }
      }

      DP_Data_Image3 = Transposed_Image3;

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,"")) {

      for (k = kstart4; k < kend4; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_Image4[I_Index] = DP_Data_Image4[I_Index_Inp];

            }
      }

      DP_Data_Image4 = Transposed_Image4;

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask,"")) {

      for (k = kstartg; k < kendg; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_GMask[I_Index] = DP_Data_Image_GMask[I_Index_Inp];

            }
      }

      DP_Data_Image_GMask = Transposed_GMask;

   }


} else {
   if (DRIP_Stat->I_Verbose)
      printf(
         "darkdrift_compute_results: Processing transposed input image...\n");

}



/* 
   Copy input image into Aux_Array.
*/

   for (i = 0; i < I_Length_X; i++) 
   for (j = 0; j < I_Length_Y; j++)
   for (k = kstart; k < kend; k++) {

      I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

      Aux_Array[I_Index] = DP_Data_Image1[I_Index];

   }


/* 
   If gmask is specified, then replace input data with NaNs at the fatal
   locations.
*/

   if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask, "") == 0)) { 

      if (DRIP_Stat->I_Verbose)
         printf(
            "Processing gmask data...\n");

      nfatal = 0;

      for (i = 0; i < I_Length_X; i++) {
      for (j = 0; j < I_Length_Y; j++) {

         I_Index_Inp = i*I_Length_Y+j;

         for (k = kstart; k < kend; k++) {

            I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

            if (DP_Data_Image_GMask[I_Index_Inp] > GMaskMax) {

               DRIP_Stat->I_status = COMPUTE_RES|BAD_INPUT_DATA;

               /* 
                  Write the error code into the array as an NaN instead 
                  of the result. 
               */

               darkdrift_nan(&pixelvalue, DRIP_Stat);

               DP_Data_Image1[I_Index] = pixelvalue;

               nfatal++;

               DRIP_Stat->I_status = 0; 

            }

         }

      }
      }

      if (DRIP_Stat->I_Verbose)
         printf(
            "Number of gmask-rejected pixels = %d\n", nfatal/I_Num_Frames);


      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) {

         for (i = 0; i < I_Length_X; i++) {
         for (j = 0; j < I_Length_Y; j++) {

            I_Index_Inp = i*I_Length_Y+j;

            for (k = kstart2; k < kend2; k++) {

               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

               if (DP_Data_Image_GMask[I_Index_Inp] > GMaskMax) {

                  DRIP_Stat->I_status = COMPUTE_RES|BAD_INPUT_DATA;

                  /* 
                     Write the error code into the array as an NaN instead 
                     of the result. 
                  */

                  darkdrift_nan(&pixelvalue, DRIP_Stat);

                  DP_Data_Image2[I_Index] = pixelvalue;

                  DRIP_Stat->I_status = 0; 

               }

            }

         }
         }

      }

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {

         for (i = 0; i < I_Length_X; i++) {
         for (j = 0; j < I_Length_Y; j++) {

            I_Index_Inp = i*I_Length_Y+j;

            for (k = kstart3; k < kend3; k++) {

               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

               if (DP_Data_Image_GMask[I_Index_Inp] > GMaskMax) {

                  DRIP_Stat->I_status = COMPUTE_RES|BAD_INPUT_DATA;

                  /* 
                     Write the error code into the array as an NaN instead 
                     of the result. 
                  */

                  darkdrift_nan(&pixelvalue, DRIP_Stat);

                  DP_Data_Image3[I_Index] = pixelvalue;

                  DRIP_Stat->I_status = 0; 

               }

            }

         }
         }

      }

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,"")) {

         for (i = 0; i < I_Length_X; i++) {
         for (j = 0; j < I_Length_Y; j++) {

            I_Index_Inp = i*I_Length_Y+j;

            for (k = kstart4; k < kend4; k++) {

               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

               if (DP_Data_Image_GMask[I_Index_Inp] > GMaskMax) {

                  DRIP_Stat->I_status = COMPUTE_RES|BAD_INPUT_DATA;

                  /* 
                     Write the error code into the array as an NaN instead 
                     of the result. 
                  */

                  darkdrift_nan(&pixelvalue, DRIP_Stat);

                  DP_Data_Image4[I_Index] = pixelvalue;

                  DRIP_Stat->I_status = 0; 

               }

            }

         }
         }

      }

   }


/* Create the output arrays for writing to file. */

I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;

DRIP_Comp->DP_Output_Array = (double *) calloc(I_Tot_Pixels, 
   sizeof(double));
if(DRIP_Comp->DP_Output_Array==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DP_Output_Array2 = (double *) calloc(I_Tot_Pixels, 
   sizeof(double));
if(DRIP_Comp->DP_Output_Array2==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DP_Output_Array3 = (double *) calloc(I_Tot_Pixels, 
   sizeof(double));
if(DRIP_Comp->DP_Output_Array3==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}


/* Copy structure info to local variables */

DP_Output_Array = DRIP_Comp->DP_Output_Array;
DP_Output_Array2 = DRIP_Comp->DP_Output_Array2;
DP_Output_Array3 = DRIP_Comp->DP_Output_Array3;


/* Allocate arrays for readout-channel statistics. */

nsize = I_Num_Frames * NCALCDOMS;

DRIP_Comp->NumberBadPixels = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberBadPixels==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberSamples = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberSamples==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->Median = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->Median==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->StdDev = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->StdDev==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DataScale = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->DataScale==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->Correction = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->Correction==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->Uncertainty = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->Uncertainty==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->Background = 
   (double *) calloc(I_Num_Frames, sizeof(double));
if(DRIP_Comp->Background==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}


DRIP_Comp->NumberBadPixelsRef = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberBadPixelsRef==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberSamplesRef = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberSamplesRef==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->MedianRef = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->MedianRef==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->StdDevRef = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->StdDevRef==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DataScaleRef = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->DataScaleRef==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberBadPixelsBefore = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberBadPixelsBefore==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberSamplesBefore = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberSamplesBefore==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->MedianBefore = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->MedianBefore==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->StdDevBefore = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->StdDevBefore==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DataScaleBefore = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->DataScaleBefore==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberBadPixelsAfter = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberBadPixelsAfter==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->NumberSamplesAfter = 
   (unsigned long *) calloc(nsize, sizeof(unsigned long));
if(DRIP_Comp->NumberSamplesAfter==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->MedianAfter = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->MedianAfter==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->StdDevAfter = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->StdDevAfter==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

DRIP_Comp->DataScaleAfter = 
   (double *) calloc(nsize, sizeof(double));
if(DRIP_Comp->DataScaleAfter==NULL) {
   DRIP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

/* Set up readout-channel strides through image data. */


   istart[0] = 0;
   iend[0] = I_Length_X - 3;
   istep[0] = 4;

   jstart[0] = 0;
   jend[0] = I_Length_Y;
   jstep[0] = 1;

   istart[1] = 1;
   iend[1] = I_Length_X - 2;
   istep[1] = 4;

   jstart[1] = 0;
   jend[1] = I_Length_Y;
   jstep[1] = 1;

   istart[2] = 2;
   iend[2] = I_Length_X - 1;
   istep[2] = 4;

   jstart[2] = 0;
   jend[2] = I_Length_Y;
   jstep[2] = 1;

   istart[3] = 3;
   iend[3] = I_Length_X;
   istep[3] = 4;

   jstart[3] = 0;
   jend[3] = I_Length_Y;
   jstep[3] = 1;



/* 
   Compute trimmed statistics for each readout channel in image
   to be corrected. 
*/

   if (DRIP_Stat->I_Verbose)
      printf(
         "Computing readout-channel offsets from image to be corrected...\n");

      kkk = 0;

      for (k = kstart; k < kend; k++) {

         for (n = 0; n < NCALCDOMS; n++) {
            median[n] = 0.0;
            stddev[n] = 0.0;
            nbadpixels[n] = 0;
            nsamps[n] = 0;
         }

         for (n = 0; n < NCALCDOMS; n++) {

            I_Index_Stat = kkk * NCALCDOMS + n;

            l = 0;
   
            for (i = istart[n]; i < iend[n]; i+=istep[n])
            for (j = jstart[n]; j < jend[n]; j+=jstep[n]) {
               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

/* Check that input is a valid number. */

//  Replaced with this NaN check - TYB 5/1/09

	       if (!iznanorinfd(DP_Data_Image1[I_Index])) 
		 //               if (! (DP_Data_Image1[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image1[I_Index])))
               {

                  Input_Array[l] = DP_Data_Image1[I_Index];
                  l++;

	       } else {
                  nbadpixels[n]++;
               }

            }

            nsamps[n] = l;

            if (nbadpixels[n] > NBADPIXGLOTOL)
	    {
               DRIP_Stat->I_status = COMPUTE_RES|TOO_MANY_BAD_PIXELS;

               printf("%s%d%s%d%s 0x%x\n",
                  "darkdrift_compute_results: Readout channel = ",
                  n+1,"; Number of bad pixels found = ",
                  nbadpixels[n], "; Warning code:", DRIP_Stat->I_status);


/* Write the error code into the array as an NaN instead of the result. */

               darkdrift_nan(&median[n], DRIP_Stat);
               darkdrift_nan(&stddev[n], DRIP_Stat);
               darkdrift_nan(&datascale[n], DRIP_Stat);
               DRIP_Stat->I_status = 0; 
            } else {

               f1 = DRIP_Const->CentralFraction1; 
               f2 = DRIP_Const->CentralFraction2; 
               trimmedstatistics(Input_Array, lone, l, f1, f2, 
                  &meanval, &stddevval, &datascaleval, DRIP_Stat);
               median[n] = meanval;
               stddev[n] = stddevval;
               datascale[n] = datascaleval;

            }

	 }

         if (DRIP_Stat->I_SuperVerbose) 
            printf("Image plane = %d\n",kkk);

         for (n = 0; n < NCALCDOMS; n++) {
            I_Index = kkk * NCALCDOMS + n;
            DRIP_Comp->Median[I_Index] = median[n];
            DRIP_Comp->StdDev[I_Index] = stddev[n];
            DRIP_Comp->DataScale[I_Index] = datascale[n];
            DRIP_Comp->Uncertainty[I_Index] = datascale[n] / sqrt(nsamps[n]);
            DRIP_Comp->NumberSamples[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixels[I_Index] = nbadpixels[n];
            DRIP_Comp->MedianRef[I_Index] = median[n];
            DRIP_Comp->StdDevRef[I_Index] = stddev[n];
            DRIP_Comp->DataScaleRef[I_Index] = datascale[n];
            DRIP_Comp->NumberSamplesRef[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixelsRef[I_Index] = nbadpixels[n];

            if (DRIP_Stat->I_SuperVerbose) {
               printf("n = %d\n",n);
               printf("Trimmed average value = %f\n",
                  DRIP_Comp -> MedianRef[I_Index]);
               printf("Trimmed std. dev. value = %f\n",
                  DRIP_Comp -> StdDevRef[I_Index]);
               printf("Data scale = %f\n",
                  DRIP_Comp -> DataScaleRef[I_Index]);
               printf("Number of samples (before trimming) = %d\n",
                  DRIP_Comp -> NumberSamplesRef[I_Index]);
               printf("Number of bad pixels (before trimming) = %d\n",
                  DRIP_Comp -> NumberBadPixelsRef[I_Index]);
               printf("Uncertainty = %f\n",
                  DRIP_Comp -> Uncertainty[I_Index]);
            }
         }

         kkk++;

      }


if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) {

/* 
   Compute trimmed statistics for each readout channel in before 
   dark-current image. 
*/

   if (DRIP_Stat->I_Verbose)
      printf("%s%s\n",
         "Computing readout-channel offsets from before ",
         "dark-current image...");


      /* Subtract off canonical dark. */

      for (k = kstart2; k < kend2; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

//  Replaced with this NaN check - TYB 5/1/09
	       if ( (!iznanorinfd(DP_Data_Image2[I_Index])) &&
                    (!iznanorinfd(DP_Data_Image4[I_Index])) )

		 //               if ((! (DP_Data_Image2[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image2[I_Index]))) &&
		 //                   (! (DP_Data_Image4[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image4[I_Index]))))
               {

                  DP_Data_Image2[I_Index] = DP_Data_Image2[I_Index] - 
                     DP_Data_Image4[I_Index];
	       } else {
                  DRIP_Stat->I_status = COMPUTE_RES|INPUT_BAD_PIXEL;

                  /* 
                     Write the error code into the array as an NaN 
                     instead of the result. 
                  */

                  darkdrift_nan(&DP_Data_Image2[I_Index], DRIP_Stat);
                  DRIP_Stat->I_status = 0; 
               }

            }
      }


      kkk = 0;

      for (k = kstart2; k < kend2; k++) {

         for (n = 0; n < NCALCDOMS; n++) {
            median[n] = 0.0;
            stddev[n] = 0.0;
            datascale[n] = 0.0;
            nbadpixels[n] = 0;
            nsamps[n] = 0;
         }

         for (n = 0; n < NCALCDOMS; n++) {

            I_Index_Stat = kkk * NCALCDOMS + n;

            l = 0;
   
            for (i = istart[n]; i < iend[n]; i+=istep[n])
            for (j = jstart[n]; j < jend[n]; j+=jstep[n]) {
               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

/* Check that input is a valid number. */

//  Replaced with this NaN check - TYB 5/1/09

	       if (!iznanorinfd(DP_Data_Image2[I_Index])) 

		 //               if (! (DP_Data_Image2[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image2[I_Index])))
               {

                  Input_Array[l] = DP_Data_Image2[I_Index];
                  l++;

	       } else {
                  nbadpixels[n]++;
               }

            }

            nsamps[n] = l;

            if (nbadpixels[n] > NBADPIXGLOTOL)
	    {
               DRIP_Stat->I_status = COMPUTE_RES|TOO_MANY_BAD_PIXELS;

               printf("%s%d%s%d%s 0x%x\n",
                  "darkdrift_compute_results: Readout channel = ",
                  n+1,"; Number of bad pixels found = ",
                  nbadpixels[n], "; Warning code:", DRIP_Stat->I_status);


/* Write the error code into the array as an NaN instead of the result. */

               darkdrift_nan(&median[n], DRIP_Stat);
               darkdrift_nan(&stddev[n], DRIP_Stat);
               darkdrift_nan(&datascale[n], DRIP_Stat);
               DRIP_Stat->I_status = 0; 
            } else {

               f1 = DRIP_Const->CentralFraction1; 
               f2 = DRIP_Const->CentralFraction2; 
               trimmedstatistics(Input_Array, lone, l, f1, f2, 
                  &meanval, &stddevval, &datascaleval, DRIP_Stat);
               median[n] = meanval;
               stddev[n] = stddevval;
               datascale[n] = datascaleval;

            }

	 }

         if (DRIP_Stat->I_SuperVerbose) 
            printf("Image plane = %d\n",kkk);

         for (n = 0; n < NCALCDOMS; n++) {
            I_Index = kkk * NCALCDOMS + n;

            if (DRIP_FITS->TempInput != -9999. &&
                DRIP_FITS->TempBefore != -9999.) {

               /* Apply temperature compensation. */

               meanval = median[n];
               temp = DRIP_FITS->TempBefore;
               compensationtemp = DRIP_FITS->TempInput;
               median[n] = temperaturecompensate(meanval, 
                                     temp, compensationtemp, 
                                     DRIP_Stat->I_SuperVerbose);

            }

            DRIP_Comp->Median[I_Index] = median[n];
            DRIP_Comp->StdDev[I_Index] = stddev[n];
            DRIP_Comp->DataScale[I_Index] = datascale[n];
            DRIP_Comp->Uncertainty[I_Index] = datascale[n] / sqrt(nsamps[n]);
            DRIP_Comp->NumberSamples[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixels[I_Index] = nbadpixels[n];
            DRIP_Comp->MedianBefore[I_Index] = median[n];
            DRIP_Comp->StdDevBefore[I_Index] = stddev[n];
            DRIP_Comp->DataScaleBefore[I_Index] = datascale[n];
            DRIP_Comp->NumberSamplesBefore[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixelsBefore[I_Index] = nbadpixels[n];

            if (DRIP_Stat->I_SuperVerbose) {
               printf("n = %d\n",n);
               printf("Trimmed average value = %f\n",
                  DRIP_Comp -> MedianBefore[I_Index]);
               printf("Trimmed std. dev. value = %f\n",
                  DRIP_Comp -> StdDevBefore[I_Index]);
               printf("Data scale = %f\n",
                  DRIP_Comp -> DataScaleBefore[I_Index]);
               printf("Number of samples (before trimming) = %d\n",
                  DRIP_Comp -> NumberSamplesBefore[I_Index]);
               printf("Number of bad pixels (before trimming) = %d\n",
                  DRIP_Comp -> NumberBadPixelsBefore[I_Index]);
            }
         }

         kkk++;

      }


} 


if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {

/* 
   Compute trimmed statistics for each readout channel in after
   dark-current image. 
*/

   if (DRIP_Stat->I_Verbose)
      printf("%s%s\n",
         "Computing readout-channel offsets from after ",
         "dark-current image...");


      /* Subtract off canonical dark. */

      for (k = kstart3; k < kend3; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index = k*I_Length_X*I_Length_Y+j*I_Length_X+i;

//  Replaced with this NaN check - TYB 5/1/09
	       if ( (!iznanorinfd(DP_Data_Image3[I_Index])) &&
                    (!iznanorinfd(DP_Data_Image4[I_Index])) )

		 //               if ((! (DP_Data_Image3[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image3[I_Index]))) &&
		 //                   (! (DP_Data_Image4[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image4[I_Index]))))
               {
                  DP_Data_Image3[I_Index] = DP_Data_Image3[I_Index] - 
                     DP_Data_Image4[I_Index];
	       } else {
                  DRIP_Stat->I_status = COMPUTE_RES|INPUT_BAD_PIXEL;

                  /* 
                     Write the error code into the array as an NaN 
                     instead of the result. 
                  */

                  darkdrift_nan(&DP_Data_Image3[I_Index], DRIP_Stat);
                  DRIP_Stat->I_status = 0; 
               }

            }
      }


      kkk = 0;

      for (k = kstart3; k < kend3; k++) {

         for (n = 0; n < NCALCDOMS; n++) {
            median[n] = 0.0;
            stddev[n] = 0.0;
            datascale[n] = 0.0;
            nbadpixels[n] = 0;
            nsamps[n] = 0;
         }

         for (n = 0; n < NCALCDOMS; n++) {

            I_Index_Stat = kkk * NCALCDOMS + n;

            l = 0;
   
            for (i = istart[n]; i < iend[n]; i+=istep[n])
            for (j = jstart[n]; j < jend[n]; j+=jstep[n]) {
               I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;

/* Check that input is a valid number. */

//  Replaced with this NaN check - TYB 5/1/09
	       if (!iznanorinfd(DP_Data_Image3[I_Index])) 

		 //               if (! (DP_Data_Image3[I_Index] != 0 && 
		 //                   dnan(*(short *) &DP_Data_Image3[I_Index])))
               {
                  Input_Array[l] = DP_Data_Image3[I_Index];
                  l++;

	       } else {
                  nbadpixels[n]++;
               }

            }

            nsamps[n] = l;

            if (nbadpixels[n] > NBADPIXGLOTOL)
	    {
               DRIP_Stat->I_status = COMPUTE_RES|TOO_MANY_BAD_PIXELS;

               printf("%s%d%s%d%s 0x%x\n",
                  "darkdrift_compute_results: Readout channel = ",
                  n+1,"; Number of bad pixels found = ",
                  nbadpixels[n], "; Warning code:", DRIP_Stat->I_status);


/* Write the error code into the array as an NaN instead of the result. */

               darkdrift_nan(&median[n], DRIP_Stat);
               darkdrift_nan(&stddev[n], DRIP_Stat);
               darkdrift_nan(&datascale[n], DRIP_Stat);
               DRIP_Stat->I_status = 0; 
            } else {

               f1 = DRIP_Const->CentralFraction1; 
               f2 = DRIP_Const->CentralFraction2; 
               trimmedstatistics(Input_Array, lone, l, f1, f2, 
                  &meanval, &stddevval, &datascaleval, DRIP_Stat);
               median[n] = meanval;
               stddev[n] = stddevval;
               datascale[n] = datascaleval;

            }

	 }

         if (DRIP_Stat->I_SuperVerbose) 
            printf("Image plane = %d\n",kkk);

         for (n = 0; n < NCALCDOMS; n++) {
            I_Index = kkk * NCALCDOMS + n;

            if (DRIP_FITS->TempInput != -9999. &&
                DRIP_FITS->TempAfter != -9999.) {

               /* Apply temperature compensation. */

               meanval = median[n];
               temp = DRIP_FITS->TempAfter;
               compensationtemp = DRIP_FITS->TempInput;
               median[n] = temperaturecompensate(meanval, 
                                     temp, compensationtemp, 
                                     DRIP_Stat->I_SuperVerbose);

            }


            DRIP_Comp->Median[I_Index] = median[n];
            DRIP_Comp->StdDev[I_Index] = stddev[n];
            DRIP_Comp->DataScale[I_Index] = datascale[n];
            DRIP_Comp->Uncertainty[I_Index] = datascale[n] / sqrt(nsamps[n]);
            DRIP_Comp->NumberSamples[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixels[I_Index] = nbadpixels[n];
            DRIP_Comp->MedianAfter[I_Index] = median[n];
            DRIP_Comp->StdDevAfter[I_Index] = stddev[n];
            DRIP_Comp->DataScaleAfter[I_Index] = datascale[n];
            DRIP_Comp->NumberSamplesAfter[I_Index] = nsamps[n];
            DRIP_Comp->NumberBadPixelsAfter[I_Index] = nbadpixels[n];

            if (DRIP_Stat->I_SuperVerbose) {
               printf("n = %d\n",n);
               printf("Trimmed average value = %f\n",
                  DRIP_Comp -> MedianAfter[I_Index]);
               printf("Trimmed std. dev. value = %f\n",
                  DRIP_Comp -> StdDevAfter[I_Index]);
               printf("Data scale = %f\n",
                  DRIP_Comp -> DataScaleAfter[I_Index]);
               printf("Number of samples (before trimming) = %d\n",
                  DRIP_Comp -> NumberSamplesAfter[I_Index]);
               printf("Number of bad pixels (before trimming) = %d\n",
                  DRIP_Comp -> NumberBadPixelsAfter[I_Index]);
            }
         }

         kkk++;

      }

} 

if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") && 
    strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {



/* 
   Combine before and after dark-current image results via 
   linear interpolation. 
*/

   if (DRIP_Stat->I_Verbose)
      printf("Linearly interpolating results...\n");

      kkk = 0;

      for (k = kstart4; k < kend4; k++) {

         if (DRIP_Stat->I_SuperVerbose) 
            printf("Image plane = %d\n",kkk);


         for (n = 0; n < NCALCDOMS; n++) {
            I_Index = kkk * NCALCDOMS + n;


            if (DRIP_FITS->TimeInput == -9999. ||
                DRIP_FITS->TimeBefore == -9999. ||
                DRIP_FITS->TimeAfter == -9999. ||
                DRIP_FITS->TimeAfter == DRIP_FITS->TimeBefore) {

               DRIP_Comp->Median[I_Index] = 
                  0.5 * (DRIP_Comp->MedianBefore[I_Index] +
                         DRIP_Comp->MedianAfter[I_Index]);

               DRIP_Comp->StdDev[I_Index] = 
                  0.5 * (DRIP_Comp->StdDevBefore[I_Index] +
                         DRIP_Comp->StdDevAfter[I_Index]);

               DRIP_Comp->DataScale[I_Index] = 
                  0.5 * (DRIP_Comp->DataScaleBefore[I_Index] +
                         DRIP_Comp->DataScaleAfter[I_Index]);

            } else {

               slope = (DRIP_Comp->MedianAfter[I_Index] -
                        DRIP_Comp->MedianBefore[I_Index]) / 
                       (DRIP_FITS->TimeAfter - DRIP_FITS->TimeBefore);
            
               yint = DRIP_Comp->MedianAfter[I_Index] - 
                  slope * DRIP_FITS->TimeAfter;

               DRIP_Comp->Median[I_Index] = 
                  slope * DRIP_FITS->TimeInput + yint;


               slope2 = (DRIP_Comp->StdDevAfter[I_Index] -
                        DRIP_Comp->StdDevBefore[I_Index]) / 
                       (DRIP_FITS->TimeAfter - DRIP_FITS->TimeBefore);
            
               yint2 = DRIP_Comp->StdDevAfter[I_Index] - 
                  slope2 * DRIP_FITS->TimeAfter;

               DRIP_Comp->StdDev[I_Index] = 
                  slope2 * DRIP_FITS->TimeInput + yint2;


               slope3 = (DRIP_Comp->DataScaleAfter[I_Index] -
                        DRIP_Comp->DataScaleBefore[I_Index]) / 
                       (DRIP_FITS->TimeAfter - DRIP_FITS->TimeBefore);
            
               yint3 = DRIP_Comp->DataScaleAfter[I_Index] - 
                  slope3 * DRIP_FITS->TimeAfter;

               DRIP_Comp->DataScale[I_Index] = 
                  slope3 * DRIP_FITS->TimeInput + yint3;

            }

            DRIP_Comp->Uncertainty[I_Index] = 
               DRIP_Comp->DataScale[I_Index] / 
               sqrt(0.5 * (DRIP_Comp->NumberSamplesBefore[I_Index] +
                           DRIP_Comp->NumberSamplesAfter[I_Index]));

            if (DRIP_Stat->I_SuperVerbose) {
               printf("n = %d\n",n);
               printf("Interpolated trimmed average value = %f\n",
                  DRIP_Comp -> Median[I_Index]);
               printf("Interpolated trimmed std. dev. value = %f\n",
                  DRIP_Comp -> StdDev[I_Index]);
               printf("Interpolated data scale = %f\n",
                  DRIP_Comp -> DataScale[I_Index]);
               printf("Interpolated uncertainty = %f\n",
                  DRIP_Comp -> Uncertainty[I_Index]);
            }
         }

         kkk++;

      }

}


   /* 
      Subtract readout-channel offsets from input image. Also, add 
      the darkdrift uncertainty to the input uncertainty in quadrature.
   */

   if (DRIP_Stat->I_Verbose)
      printf("Subtracting readout-channel offsets from input image...\n");

      kkk = 0;

      for (k = kstart; k < kend; k++) {

         term1 = 0;
         term2 = 0;
         for (n = 0; n < NCALCDOMS; n++) {
            if (I_RO_channel_to_exclude != n + 1) {
               I_Index_Stat = kkk * NCALCDOMS + n;
               median[n] = DRIP_Comp->Median[I_Index_Stat];
               term1 += median[n];
               term2 += median[n] * median[n];
	    }
         }

 	 if (I_Mult_Corr == 0) {
            if (I_RO_channel_to_exclude == 0) {
               doff = NCALCDOMS;
            } else {
               doff = NCALCDOMS - 1;
	    }
            background = term1 / (double) doff;
	 } else {
            background = term2 / term1;
	 }

         DRIP_Comp->Background[kkk] = background;

         rmse = 0;
         for (n = 0; n < NCALCDOMS; n++) {
            I_Index_Stat = kkk * NCALCDOMS + n;
            median[n] = DRIP_Comp->Median[I_Index_Stat];
            uncertainty[n] = DRIP_Comp->Uncertainty[I_Index_Stat];

            if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") ||
                strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {
               correction[n] = median[n];
	    } else {
 	       if (I_Mult_Corr == 0) {
                  correction[n] = median[n] - background;
                  rmse += correction[n] * correction[n];
	       } else {
                  if (background == 0) {
                     printf("*** Error: %s\n", 
                     "Computed background (denominator) is zero; quitting...");
                     DRIP_Stat->I_status = COMPUTE_RES|DIVIDE_ZERO;
                     return;
                  }
                  correction[n] = median[n]/background;
                  rmse += (1 - correction[n]) * (1 - correction[n]);
               }
            }

            DRIP_Comp->Correction[I_Index_Stat] = correction[n];

            if (DRIP_Stat->I_SuperVerbose) {
               printf("n = %d\n",n);
               printf("Jail-bar correction = %f\n",
                  DRIP_Comp->Correction[I_Index_Stat]);
               printf("Jail-bar correction uncertainty = %f\n",
                  DRIP_Comp->Uncertainty[I_Index_Stat]);
            }

         }


         if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") ||
             strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,""))) {
            if (I_RO_channel_to_exclude == 0) {
               doff = NCALCDOMS;
            } else {
               doff = NCALCDOMS - 1;
	    }
            rmse = sqrt(rmse / (double) doff);
            if (DRIP_Stat->I_Verbose) 
               printf("R.S.M.E, degrees of freedom = %f, %d\n",rmse,doff);
         }



         if (DRIP_Stat->I_SuperVerbose) 
            printf("Image plane, background = %d, %f\n",kkk,background);

         for (n = 0; n < NCALCDOMS; n++) {

            I_Index_Stat = kkk * NCALCDOMS + n;
            nbadpixels[n] = DRIP_Comp->NumberBadPixels[I_Index_Stat];
            median[n] = DRIP_Comp->Median[I_Index_Stat];

            for (i = istart[n]; i < iend[n]; i+=istep[n])
            for (j = jstart[n]; j < jend[n]; j+=jstep[n]) {

               I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = kkk*I_Length_X*I_Length_Y+i*I_Length_Y+j;

//  Replaced with this NaN check - TYB 5/1/09

	       if (!iznanorinfd(Aux_Array[I_Index_Inp])) { 

		 //               if (! (Aux_Array[I_Index_Inp] != 0 && 
		 //	          dnan(*(short *) &Aux_Array[I_Index_Inp]))) {

//  Replaced with this NaN check, note switch in logic - TYB 5/1/09
	       if (iznanorinfd(median[n])) { 

		 //                  if (median[n] != 0 && 
		 //                     dnan(*(short *) &median[n])) {

                     DRIP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

                     /* Write the error code into the output arrays as 
                        a NaN instead of the result.
                     */
                     darkdrift_nan(&DP_Output_Array[I_Index], DRIP_Stat);
                     darkdrift_nan(&DP_Output_Array2[I_Index], DRIP_Stat);
                     darkdrift_nan(&DP_Output_Array3[I_Index], DRIP_Stat);

                  } else {

 		     if (I_Mult_Corr == 0) {
                        DP_Output_Array[I_Index] = 
                           Aux_Array[I_Index_Inp] - correction[n];
                     } else {
                        if (correction[n] <= 0) {
                           printf("*** Error: %s%f%s\n", 
			       "Computed multiplicative correction (=",
			       correction[n],
                               ") <= zero; quitting...");
                           DRIP_Stat->I_status = COMPUTE_RES|OUT_OF_RANGE;
                           return;
                        }
                        DP_Output_Array[I_Index] = 
                           Aux_Array[I_Index_Inp]/correction[n];
                     }

                     DP_Output_Array2[I_Index] = correction[n]; 

                     if (inputuncflag) {
                        DP_Output_Array3[I_Index] = 
                           sqrt(uncertainty[n] * uncertainty[n] + 
  	                   DP_Data_In_Unc_Image[I_Index] * 
		           DP_Data_In_Unc_Image[I_Index]);
		     } else {
                        DP_Output_Array3[I_Index] = uncertainty[n]; 
		     }

                  }

               } else {
                  DRIP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;
                  /* Write the error code into the array as an NaN 
                     instead of the result.
                  */
                  darkdrift_nan(&DP_Output_Array[I_Index], DRIP_Stat);

//  Replaced with this NaN check - TYB 5/1/09

	       if (!iznanorinfd(median[n])) { 

		 //                  if (! (median[n] != 0 && 
		 //                     dnan(*(short *) &median[n]))) {
                     DP_Output_Array2[I_Index] = correction[n]; 
                     DP_Output_Array3[I_Index] = uncertainty[n]; 
                  }
               }

/* If this pixel had NaN, will keep going and try next pixel */

               DRIP_Stat->I_status = 0;

	    }

         }

         kkk++;

      }



/* 
   Un-transpose output images if input image was transposed.
*/


if (DRIP_FITS->I_Transpose == 1) {

      kkk = 0;

      for (k = kstart; k < kend; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {

               I_Index_Inp = kkk*I_Length_X*I_Length_Y+i*I_Length_Y+j;
               I_Index = kkk*I_Length_X*I_Length_Y+j*I_Length_X+i;

               Transposed_Output_Image[I_Index] = DP_Output_Array[I_Index_Inp];
               Transposed_Output_Image2[I_Index] = 
                  DP_Output_Array2[I_Index_Inp];
               Transposed_Output_Image3[I_Index] = 
                  DP_Output_Array3[I_Index_Inp];

            }
         kkk++;
      }


      kkk = 0;

      for (k = kstart; k < kend; k++) {

         for (i = 0; i < I_Length_X; i++)
            for (j = 0; j < I_Length_Y; j++) {
               I_Index = kkk*I_Length_X*I_Length_Y+j*I_Length_X+i;
               DRIP_Comp->DP_Output_Array[I_Index] = 
                  Transposed_Output_Image[I_Index];
               DRIP_Comp->DP_Output_Array2[I_Index] = 
                  Transposed_Output_Image2[I_Index];
               DRIP_Comp->DP_Output_Array3[I_Index] = 
                  Transposed_Output_Image3[I_Index];
            }
         kkk++;
      }


   tmp = DRIP_FITS->I_Length_X;
   DRIP_FITS->I_Length_X = DRIP_FITS->I_Length_Y;
   DRIP_FITS->I_Length_Y = tmp;

   I_Length_X = DRIP_FITS->I_Length_X;
   I_Length_Y = DRIP_FITS->I_Length_Y;

}


	
if (DRIP_Stat->I_Debug)	
{
   fp_data = fopen("darkdrift_data.dump", "w");
   kkk = 0;
   for(k=0;k<I_Num_Frames;k++) {
      for(i=0;i<I_Length_X;i++)
      for(j=0;j<I_Length_Y;j++) {
         I_Index = kkk*I_Length_X*I_Length_Y+i*I_Length_Y+j;
         fprintf(fp_data, "%f %f %f %f\n", DP_Output_Array2[I_Index],
            Transposed_Output_Image2[I_Index],
            DRIP_Comp->DP_Output_Array2[I_Index],
            DRIP_Comp->DP_Output_Array3[I_Index]);
      }
      kkk++;
   }
   fclose(fp_data);
}

   if (DRIP_Stat->I_Verbose) {
      printf("darkdrift_compute_results: ");
      printf("Dark-current offset calibration and correction completed.\n");
   }



end_compute_results:

   free(Input_Array);
   free(Aux_Array);
   free(Transposed_Image);
   free(Transposed_Image2);
   free(Transposed_Image3);
   free(Transposed_Image4);
   free(Transposed_GMask);
   free(Transposed_In_Unc_Image);
   free(Transposed_Output_Image);
   free(Transposed_Output_Image2); 
   free(Transposed_Output_Image3); 
   return;

}


void trimmedstatistics(double y[], long nstart, long n, double f1, 
                  double f2, double *mean, double *stddev, 
                  double *datascale,
                  DRI_Status *DRIP_Stat) {

   long mid, nsamps1, nsamps2, istart, iend, jstart, jend, i, j;

   double fmean, fstddev, stddev2;

   double sampclippercen, fact, facttot, alpha;

   double Sn;

   sampclippercen = 0.5 * (1.0 - f2) * 100.0;

   fact = getclipcorrfact(sampclippercen);

   /* facttot is the STD. DEV. correction due to the Gaussian trimming. */

   alpha = alphafactor(f2);

   facttot = sqrt(fact * f2) * alpha;

   if (DRIP_Stat->I_SuperVerbose) 
      printf("trimmedstatistics: fact, alpha, facttot = %f, %f\n",
         fact, alpha, facttot);

   dsort(y, nstart, n);

   datascale_estimate(y, n, &Sn);

   *datascale = Sn;

   if (f1 == 0.0) {
      if (n % 2 > 0) {
         mid = n / 2;
         *mean = y[mid];
      } else {
         mid = n / 2 - 1;
         *mean = (y[mid] + y[mid+1]) * 0.5;
      }

      nsamps2 = 0;
      *stddev = 0.0;
      stddev2 = 0.0;
      fstddev = 0.5 * (1.0 - f2);
      jstart = nstart - 1 + nint(fstddev * (n - nstart + 1));
      jend = n - 1 - nint(fstddev * (n - nstart + 1));

      for (j = jstart; j <= jend; j++) {
         *stddev += (y[j] - (*mean)) * (y[j] - (*mean));
         stddev2 += y[j] - (*mean);
         nsamps2++;
      }

      if (nsamps2 <= 1) {
         *stddev = 0.0;
      } else {
         *stddev = facttot * sqrt(1. / (nsamps2 - 1) *
            (*stddev - 1. / nsamps2 * stddev2 * stddev2));
      }

   } else {

      nsamps1 = 0;
      *mean = 0.0;
      fmean = 0.5 * (1.0 - f1);
      istart = nstart - 1 + (long) (fmean * (n - nstart + 1));
      iend = n - 1 - (long) (fmean * (n - nstart + 1));

      nsamps2 = 0;
      *stddev = 0.0;
      stddev2 = 0.0;
      fstddev = 0.5 * (1.0 - f2);
      jstart = nstart - 1 + (long) (fstddev * (n - nstart + 1));
      jend = n - 1 - (long) (fstddev * (n - nstart + 1));

      for (i = istart; i <= iend; i++) {
         *mean += y[i];
         nsamps1++;
      }
      *mean /= nsamps1;

      for (j = jstart; j <= jend; j++) {
         *stddev += (y[j] - (*mean)) * (y[j] - (*mean));
         stddev2 += y[j] - (*mean);
         nsamps2++;
      }

      if (nsamps2 <= 1) {
         *stddev = 0.0;
      } else {
         *stddev = facttot * sqrt(1. / (nsamps2 - 1) *
            (*stddev - 1. / nsamps2 * stddev2 * stddev2));
      }

   }            

   if (DRIP_Stat->I_SuperVerbose) 
      printf("mean,stddev,sn = %f, %f, %f\n",*mean,*stddev,*datascale);

}


double temperaturecompensate(double inputvalue, double inputtemp,
                             double compensationtemp, int I_SuperVerbose) {

   double compensatedvalue;

   /* Stub for temperature-compensation transfer function. */

   compensatedvalue = inputvalue;

   if (I_SuperVerbose) {
      printf("Temperature compensation:\n",
         inputvalue);
      printf("   Input value, temperature  = %f, %f\n",inputvalue,inputtemp);
      printf("   Output value, temperature = %f, %f\n",compensatedvalue,
         compensationtemp);
   }

   return compensatedvalue;

}


double getclipcorrfact(double sampclippercen) {

   double sum,fact;
   double bigf,xbot,xtop;

   bigf=sampclippercen*0.01;

   if (bigf > 0.0) {

      if (bigf < 1.0) {
            getlimits(bigf,&xbot,&xtop);
            integrategauss(xbot,xtop,&sum);
            fact = 1.0/sum;
      } else {
         fact = 0.0;
      }

   } else {
      fact=1.0;
   }

      return fact;
}

void getlimits(double bigf, double *xbot, double *xtop) {

   int n, i;

   double a, b, step, halfstep, x, x2, rest, sum;

   rest = 1.0 - bigf;

   a = 0.0;
   b = 4.0;

   n = 100000;
   step = 4.0 / (double) n;
   halfstep = 0.5 * step;

   sum = 0.5;

   for (i = 1; i <= n - 1; i++) {
      x = a + ((double)(i-1)) * step;
      x2 = x + step;
      sum += (gausspdf(x) + gausspdf(x2)) * halfstep;
      if (sum >= rest) { break; }
   }

      *xbot = -x2;
      *xtop = x2;
}

void integrategauss(double xbot, double xtop, double *sum) {

      int n, i;

      double step, halfstep, x;

      n = 100000;
      step = (xtop - xbot) / (double) n;
      halfstep = 0.5 * step;

      *sum = gausspdf(xbot) + gausspdf(xtop);

      for (i = 2; i <= n - 1; i++) {
         x = xbot + (double)i * step;
         *sum += gausspdf(x) * 2.0 * x * x;
      }

      *sum *= halfstep;

}


double gausspdf(double x) {

   double gausspdf, pi;

   pi = 3.14159265358979323846;

   gausspdf = exp(-0.5 * x * x) / sqrt(2.0 * pi);

   return gausspdf;
 
}


double alphafactor(double fcentral) {

   long i, n;

   double alphafactor, f[6], a[6], slope, yint;

   f[0] = 0.0;
   f[1] = 0.2;
   f[2] = 0.4;
   f[3] = 0.6;
   f[4] = 0.8;
   f[5] = 1.0;

   a[0] = 1.253;
   a[1] = 1.184;
   a[2] = 1.126;
   a[3] = 1.080;
   a[4] = 1.036;
   a[5] = 1.000;

   n = 6;

   alphafactor = f[5];

   for (i = 1; i <  n; i++) {
      if (fcentral <= f[i]) {
         slope = (a[i] - a[i - 1])/(f[i] - f[i - 1]);
         yint = a[i] - slope * f[i];
         alphafactor = slope * fcentral + yint;
         break;
      } 
   }

   return alphafactor;
 
}



