/* $Id: darkdrift_defs.h,v 1.7 2007/08/08 20:25:28 laher Exp $ */


/***********************************************
darkdrift_defs.h

Purpose

Define structures.

***********************************************/

#include "fitsio.h"

typedef struct {

double  CentralFraction1;
double  CentralFraction2;

int     GMaskMax;

} DRI_Constants;


typedef struct {

char	CP_Filename_Namelist[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Image1[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Image2[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Image3[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Image4[MAX_FILENAME_LENGTH];
char    CP_Filename_FITS_Image_GMask[MAX_FILENAME_LENGTH];
char    CP_Filename_FITS_In_Unc_Image[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Out[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Out2[MAX_FILENAME_LENGTH];
char	CP_Filename_FITS_Out3[MAX_FILENAME_LENGTH];
char	CP_Filename_Data_Out[MAX_FILENAME_LENGTH];
char	CP_Filename_Log[MAX_FILENAME_LENGTH];

} DRI_Filenames;


typedef struct {

char	CP_Ancillary_File_Path[MAX_FILENAME_LENGTH];

} DRI_Pathnames;


typedef struct {

int	I_status;
int	I_NaNFlag;
int	I_NaNCount;
int	I_Debug;
int	I_Verbose;
int	I_SuperVerbose;

} DRI_Status;

typedef struct {

int	I_Length_X;
int	I_Length_Y;
int	I_Num_Frames;

int	I_Length_X2;
int	I_Length_Y2;
int	I_Num_Frames2;

int	I_Length_X3;
int	I_Length_Y3;
int	I_Num_Frames3;

int	I_Length_X4;
int	I_Length_Y4;
int	I_Num_Frames4;

int	I_Operation;	
int     I_Fits_Type;
int     I_Data_Plane;
int     I_Transpose;

int     I_RO_channel_to_exclude;
int     I_Mult_Corr;

double TimeInput;
double TimeBefore;
double TimeAfter;

double TempInput;
double TempBefore;
double TempAfter;

} DRI_FITSinfo;


typedef struct {

/* Stored Results */
double*	DP_Output_Array;
double*	DP_Output_Array2;
double*	DP_Output_Array3;
unsigned long* NumberBadPixels;
unsigned long* NumberSamples;
double*	Median;
double*	StdDev;
double*	DataScale;
double*	Correction;
double*	Uncertainty;
double*	Background;

unsigned long* NumberBadPixelsRef;
unsigned long* NumberSamplesRef;
double*	MedianRef;
double*	StdDevRef;
double*	DataScaleRef;

unsigned long* NumberBadPixelsBefore;
unsigned long* NumberSamplesBefore;
double*	MedianBefore;
double*	StdDevBefore;
double*	DataScaleBefore;

unsigned long* NumberBadPixelsAfter;
unsigned long* NumberSamplesAfter;
double*	MedianAfter;
double*	StdDevAfter;
double*	DataScaleAfter;

} DRI_Computation;



/* Function prototypes. */

double temperaturecompensate(double, double, double, int);
double getclipcorrfact(double);
double gausspdf(double);
double alphafactor(double);
void getlimits(double bigf, double *xbot, double *xtop);
void integrategauss(double xbot, double xtop, double *sum);

void getRootFilename( char *, char * );

void trimmedstatistics(double y[], long nstart, long n, double f1, 
		       double f2, double *mean, double *stddev, 
		       double *datascale,
		       DRI_Status *DRIP_Stat);

void darkdrift_log_writer(DRI_Constants	   *DRIP_Const,
			  DRI_Computation  *DRIP_Comp,
			  DRI_FITSinfo     *DRIP_FITS,
			  DRI_Status	   *DRIP_Stat,
			  DRI_Filenames	   *DRIP_Fnames,
			  DRI_Pathnames	   *DRIP_Paths);

void decode_message(int             I_message, 
                    int             I_root,
		    char	    *CP_Message,
		    char	    *CP_Function,
		    DRI_Pathnames   *DRIP_Paths,
		    DRI_Status	    *DRIP_Stat);

void darkdrift_compute_results(DRI_Constants		*DRIP_Const,
			       DRI_Filenames	        *DRIP_Fnames,
			       DRI_FITSinfo		*DRIP_FITS,
			       double			*DP_Data_Image1,
			       double			*DP_Data_Image2,
			       double			*DP_Data_Image3,
			       double			*DP_Data_Image4,
			       double			*DP_Data_In_Unc_Image,
			       int                      *DP_Data_Image_GMask,
			       DRI_Computation		*DRIP_Comp,
			       DRI_Status		*DRIP_Stat);

void darkdrift_init_constants(DRI_Constants	*DRIP_Const,
			      DRI_Filenames	*DRIP_Fnames,
			      DRI_Pathnames	*DRIP_Paths,
		 	      DRI_FITSinfo	*DRIP_FITS,
			      DRI_Computation	*DRIP_Comp,
			      DRI_Status	*DRIP_Stat);

void darkdrift_expand_envvar(DRI_Filenames	*DRIP_Fnames,
			     DRI_Status	*DRIP_Stat);

int expand_envvar_string(char *CP_In);

void darkdrift_nan(double*	 DP_Result,
		   DRI_Status	*DRIP_Stat);

void darkdrift_parse_namelist(int		argc,
			      char		**argv,
			      DRI_Constants	*DRIP_Const,
			      DRI_Filenames	*DRIP_Fnames,
			      DRI_Pathnames	*DRIP_Paths,
			      DRI_FITSinfo	*DRIP_FITS,
			      DRI_Status	*DRIP_Stat);

void darkdrift_parse_args(int		argc,
		  	  char		**argv,
			  DRI_Constants	*DRIP_Const,
			  DRI_Filenames	*DRIP_Fnames,
			  DRI_Pathnames	*DRIP_Paths,
			  DRI_FITSinfo	*DRIP_FITS,
			  DRI_Status	*DRIP_Stat);

void darkdrift_read_data(DRI_Filenames	 *DRIP_Fnames,
			 DRI_FITSinfo	 *DRIP_FITS,
			 double 	**DPP_Data_Image1,
			 double 	**DPP_Data_Image2,
			 double	        **DPP_Data_Image3,
			 double         **DPP_Data_Image4,
                         double	        **DPP_Data_In_Unc_Image,
			 int	        **DPP_Data_Image_GMask,
			 DRI_Status	 *DRIP_Stat);

void darkdrift_read_gmask_image(DRI_Filenames	*DRIP_Fnames,
			        DRI_FITSinfo	*DRIP_FITS,
			        int	        **DPP_Data_Image_GMask,
			        DRI_Status	*DRIP_Stat);

void darkdrift_read_image1(DRI_Filenames	*DRIP_Fnames,
			   DRI_FITSinfo 	*DRIP_FITS,
			   double       	**DPP_Data_Image1,
			   DRI_Status   	*DRIP_Stat);

void darkdrift_read_image2(DRI_Filenames	*DRIP_Fnames,
			   DRI_FITSinfo 	*DRIP_FITS,
			   double       	**DPP_Data_Image2,
		   	   DRI_Status       	*DRIP_Stat);

void darkdrift_read_image3(DRI_Filenames	*DRIP_Fnames,
			   DRI_FITSinfo         *DRIP_FITS,
			   double	        **DPP_Data_Image3,
			   DRI_Status	        *DRIP_Stat);

void darkdrift_read_image4(DRI_Filenames	*DRIP_Fnames,
			   DRI_FITSinfo 	*DRIP_FITS,
			   double          	**DPP_Data_Image4,
			   DRI_Status      	*DRIP_Stat);

void darkdrift_read_status_image(DRI_Filenames	*DRIP_Fnames,
 			         DRI_FITSinfo	*DRIP_FITS,
			         double	        **DPP_Data_Image2,
			         DRI_Status	*DRIP_Stat);

void darkdrift_read_unc_image(DRI_Filenames	*DRIP_Fnames,
			      DRI_FITSinfo	*DRIP_FITS,
		              double	       **DPP_Data_In_Unc_Image,
			      DRI_Status	*DRIP_Stat);

void darkdrift_output(DRI_Constants		*DRIP_Const,
		      DRI_Filenames		*DRIP_Fnames,
		      DRI_FITSinfo		*DRIP_FITS,
		      DRI_Computation		*DRIP_Comp,
		      DRI_Status		*DRIP_Stat);

void darkdrift_output_keywords(fitsfile	        *ffp_FITS,
			       DRI_Constants	*DRIP_Const,
			       DRI_Filenames	*DRIP_Fnames,
			       DRI_FITSinfo	*DRIP_FITS,
			       DRI_Computation	*DRIP_Comp,
			       DRI_Status	*DRIP_Stat);

void darkdrift_output_stdkeywords(fitsfile	*ffp_FITS,
				  DRI_Filenames	*DRIP_Fnames,
				  DRI_FITSinfo	*DRIP_FITS,
				  DRI_Status	*DRIP_Stat);

void darkdrift_output_filenkeywords(fitsfile    	*ffp_FITS,
				    DRI_Filenames	*DRIP_Fnames,
			            DRI_FITSinfo	*DRIP_FITS,
				    DRI_Status  	*DRIP_Stat);

void darkdrift_output_constkeywords(fitsfile    	*ffp_FITS,
				    DRI_Constants	*DRIP_Const,
	  		            DRI_Computation	*DRIP_Comp,
  			            DRI_FITSinfo	*DRIP_FITS,
				    DRI_Status   	*DRIP_Stat);

void darkdrift_exit(double		 **DPP_Data_Image1,
                    double		 **DPP_Data_Image2,
                    double		 **DPP_Data_Image3,
                    double		 **DPP_Data_Image4,
                    double		 **DPP_Data_In_Unc_Image,
                    int			 **DPP_Data_Image_GMask,
		    DRI_Computation	 *DRIP_Comp,
		    DRI_Filenames	 *DRIP_Fnames,
		    DRI_FITSinfo	 *DRIP_FITS,
		    DRI_Status		 *DRIP_Stat);
