/* $Id: darkdrift_exit.c,v 1.6 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_exit.c

Purpose

Cleanly exit the program.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_exit(double		 **DPP_Data_Image1,
                    double		 **DPP_Data_Image2,
                    double		 **DPP_Data_Image3,
                    double		 **DPP_Data_Image4,
                    double		 **DPP_Data_In_Unc_Image,
                    int			 **DPP_Data_Image_GMask,
		    DRI_Computation	 *DRIP_Comp,
		    DRI_Filenames	 *DRIP_Fnames,
		    DRI_FITSinfo	 *DRIP_FITS,
		    DRI_Status		 *DRIP_Stat)
{
   if (DRIP_Stat->I_status) return;

   if (DRIP_Stat->I_Debug) {
      printf("darkdrift_exit: DPP_Data_Image1 0x%x \n",
         *DPP_Data_Image1);
      printf("darkdrift_exit: DPP_Data_Image2 0x%x \n",
         *DPP_Data_Image2);
      printf("darkdrift_exit: DPP_Data_Image3 0x%x \n",
         *DPP_Data_Image3);
      printf("darkdrift_exit: DPP_Data_Image4 0x%x \n",
         *DPP_Data_Image4);
   }


   /* Allocated in darkdrift_read_data or its subfunctions */

   free(*DPP_Data_Image1);

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") != 0)
      free(*DPP_Data_Image2);

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"") != 0)
      free(*DPP_Data_Image3);

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,"") != 0)
      free(*DPP_Data_Image4);

   if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask, "") == 0)) 
      free(*DPP_Data_Image_GMask);

   if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image, "") == 0)) 
      free(*DPP_Data_In_Unc_Image);


   /* Allocated in darkdrift_compute_results or its subfunctions */

   free(DRIP_Comp->DP_Output_Array);

   if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_Out2, "") == 0)) 
      free(DRIP_Comp->DP_Output_Array2);

   if (! (strcmp(DRIP_Fnames->CP_Filename_FITS_Out3, "") == 0)) 
      free(DRIP_Comp->DP_Output_Array3);

   free(DRIP_Comp->NumberSamples);
   free(DRIP_Comp->Median);
   free(DRIP_Comp->NumberBadPixels);


   if(DRIP_Stat->I_Verbose)
      printf("darkdrift_exit: Memory Deallocated.\n");

}












