/* $Id: darkdrift_init_constants.c,v 1.6 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_init_constants.c

Purpose

Set constant values as defined in the header file darkdrift.h.

Overview


Definitions of Variables

See "main" function darkdrift_exec.c and darkdrift_defs.h


***********************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"

void darkdrift_init_constants(DRI_Constants	*DRIP_Const,
			      DRI_Filenames	*DRIP_Fnames,
			      DRI_Pathnames	*DRIP_Paths,
		 	      DRI_FITSinfo	*DRIP_FITS,
			      DRI_Computation	*DRIP_Comp,
			      DRI_Status	*DRIP_Stat)
{

/* Initialize all status flags. */

   DRIP_Stat->I_status = 0;
   DRIP_Stat->I_NaNFlag = 0;
   DRIP_Stat->I_Debug = 0;
   DRIP_Stat->I_Verbose = 0;
   DRIP_Stat->I_SuperVerbose = 0;
   DRIP_Stat->I_NaNCount = 0;


/* Set parameters for trimmed average and uncertainty calculations. */

   DRIP_Const->CentralFraction1 = 0.0;
   DRIP_Const->CentralFraction2 = 0.9;


/* Set default general mask threshold. */

   DRIP_Const->GMaskMax = 32767;

/* Set array pointers to NULL in case calloc is never called. */

   DRIP_Comp->DP_Output_Array = NULL;
   DRIP_Comp->DP_Output_Array2 = NULL;


/* Put in stdio defaults in case filenames are not given later. */

   sprintf(DRIP_Fnames->CP_Filename_Namelist, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Image1, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Image2, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Image3, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Image4, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Image_GMask, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Out, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Out2, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_FITS_Out3, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_Data_Out, "%s", "");
   sprintf(DRIP_Fnames->CP_Filename_Log, "%s", "stdout");


/* Put in stdio defaults in case pathnames are not given later. */

   sprintf(DRIP_Paths->CP_Ancillary_File_Path, "%s", ".");


/* Default operation is global-median-value calculation. */

   DRIP_FITS->I_Operation = 1;


/* Default is compute median for all planes in data cube of FITS file. */

   DRIP_FITS->I_Data_Plane = 1;


/* Set the default to NO image transposition. */

   DRIP_FITS->I_Transpose = 1;


/*  Zero means don't exclude any readout channel. */

    DRIP_FITS->I_RO_channel_to_exclude = 0;


/* 0=additive correction, 1=multiplicative correction */

    DRIP_FITS->I_Mult_Corr = 0;  


/* Set default times and temperatures. */

    DRIP_FITS->TimeInput = -9999.;
    DRIP_FITS->TimeBefore = -9999.;
    DRIP_FITS->TimeAfter = -9999.;

    DRIP_FITS->TempInput = -9999.;
    DRIP_FITS->TempBefore = -9999.;
    DRIP_FITS->TempAfter = -9999.;

}





