/* $Id: darkdrift_log_writer.c,v 1.6 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_log_writer.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c


Internal:
I_Time = Running time of program.
I_darkdrift_message = Code of error that occurred.
I_darkdrift_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

CP_Message = String interpretation of error code part of status word.
CP_Function = String interpretation of function part of status word.

fp_Logfile = File pointer of logfile.

***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "darkdrift_errcodes.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_log_writer(DRI_Constants	   *DRIP_Const,
			  DRI_Computation  *DRIP_Comp,
			  DRI_FITSinfo     *DRIP_FITS,
			  DRI_Status	   *DRIP_Stat,
			  DRI_Filenames	   *DRIP_Fnames,
			  DRI_Pathnames	   *DRIP_Paths)
{
int	I_Time;
int	I_darkdrift_message;
int	I_darkdrift_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];

FILE	*fp_Logfile;

time_t  currenttime;
char    *datetime;

int k, kkk=0, kstart, kend, I_Data_Plane, index, kp1, I_Num_Frames, i, ip1;

   I_Num_Frames = DRIP_FITS->I_Num_Frames;
   I_Data_Plane = DRIP_FITS->I_Data_Plane;

   if (I_Data_Plane == 3) {
      kstart = I_Num_Frames - 1;
      kend = I_Num_Frames;
   } else if (I_Data_Plane == 2) {
      kstart = 0;
      kend = 1;
   } else {
      kstart = 0;
      kend = I_Num_Frames;
   } 

if (strcmp(DRIP_Fnames->CP_Filename_Log, "stdout") == 0)	{
   if (DRIP_Stat->I_Verbose)
      fprintf(stdout, "darkdrift_log_writer: Printing Log output to stdout\n");
   fp_Logfile = stdout;
} else {
   fp_Logfile = fopen(DRIP_Fnames->CP_Filename_Log, "a");

   if(fp_Logfile==NULL)	{
      fprintf(stdout, 
         "darkdrift_log_writer: Could not open Log File, using stdout\n");
      fp_Logfile = stdout;
   }
}

   fprintf(fp_Logfile,"Program %s, Version %2.1f\n", 
      PROGRAM, DRIVERSN);

   if (strcmp(DRIP_Fnames->CP_Filename_Namelist,"")) {
      fprintf(fp_Logfile, "Namelist File  = %s\n", 
      DRIP_Fnames->CP_Filename_Namelist);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image1,"")) {
      fprintf(fp_Logfile, "Input Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_Image1);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) {
      fprintf(fp_Logfile, "Input Before Dark-Current Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_Image2);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {
      fprintf(fp_Logfile, "Input After Dark-Current Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_Image3);
   }
   if ((strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) ||
       (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,""))) {
      fprintf(fp_Logfile, "Input Canonical Dark-Current Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_Image4);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,"")) {
      fprintf(fp_Logfile, "Input Uncertainty Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_In_Unc_Image);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask,"")) {
      fprintf(fp_Logfile, "Input GMask Image = %s\n", 
      DRIP_Fnames->CP_Filename_FITS_Image_GMask);
      fprintf(fp_Logfile, "Max. Usable GMask Level = %d\n", 
      DRIP_Const->GMaskMax);
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out,"")) {
      fprintf(fp_Logfile, "Output Image 1 (corrected image) = %s\n", 
      &(DRIP_Fnames->CP_Filename_FITS_Out[1]));
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out2,"")) {
      fprintf(fp_Logfile, "Output Image 2 (corrections) = %s\n", 
      &(DRIP_Fnames->CP_Filename_FITS_Out2[1]));
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out3,"")) {
      fprintf(fp_Logfile, "Output Image 3 (uncertainties) = %s\n", 
      &(DRIP_Fnames->CP_Filename_FITS_Out3[1]));
   }
   if (strcmp(DRIP_Fnames->CP_Filename_Data_Out,"")) {
      fprintf(fp_Logfile, "Output File 3 = %s\n", 
      &(DRIP_Fnames->CP_Filename_Data_Out[1]));
   }

   fprintf(fp_Logfile, "Central fraction for trimmed average  = %f\n", 
         DRIP_Const->CentralFraction1);
   fprintf(fp_Logfile, "Central fraction for trimmed std. dev. = %f\n", 
         DRIP_Const->CentralFraction2);

   if (DRIP_FITS->TimeInput != -9999.) {
      fprintf(fp_Logfile, "Time_Input = %f\n", 
         DRIP_FITS->TimeInput);
      fprintf(fp_Logfile, "Time_Before = %f\n", 
         DRIP_FITS->TimeBefore);
      fprintf(fp_Logfile, "Time_After = %f\n", 
         DRIP_FITS->TimeAfter);
   }

   if (DRIP_FITS->TempInput != -9999.) {
      fprintf(fp_Logfile, "Temp_Input = %f\n", 
         DRIP_FITS->TempInput);
      fprintf(fp_Logfile, "Temp_Before = %f\n", 
         DRIP_FITS->TempBefore);
      fprintf(fp_Logfile, "Temp_After = %f\n", 
         DRIP_FITS->TempAfter);
   }

   fprintf(fp_Logfile, "Calculation Type = %d\n", 
      DRIP_FITS->I_Operation);
   fprintf(fp_Logfile, "Input-Image Transpose Flag = %d\n", 
      DRIP_FITS->I_Transpose);
   fprintf(fp_Logfile, "%s = %d\n",
      "Multiplicative-Correction Flag (Additive if zero)", 
      DRIP_FITS->I_Mult_Corr);
   fprintf(fp_Logfile, "Readout Channel to exclude (none if zero) = %d\n", 
      DRIP_FITS->I_RO_channel_to_exclude);
   fprintf(fp_Logfile, "Data-Plane Flag = %d\n", 
      DRIP_FITS->I_Data_Plane);
   fprintf(fp_Logfile, "Ancillary Data-File Path = %s\n", 
      DRIP_Paths->CP_Ancillary_File_Path);
   fprintf(fp_Logfile, "Verbose flag = %d\n", 
      DRIP_Stat->I_Verbose);
   fprintf(fp_Logfile, "Super-verbose flag = %d\n", 
      DRIP_Stat->I_SuperVerbose);
   fprintf(fp_Logfile, "Debug flag = %d\n", 
      DRIP_Stat->I_Debug);

I_darkdrift_message = RIGHT_MASK&(DRIP_Stat->I_status);
I_darkdrift_root = (LEFT_MASK&(DRIP_Stat->I_status))>>16;

if(I_darkdrift_message&FITS_MSG_MASK)	{
I_darkdrift_message -= FITS_MSG_MASK;
ffrprt(fp_Logfile, I_darkdrift_message);
}

if (DRIP_Stat->I_Verbose)
   printf("Decoding messages...\n");

decode_message(I_darkdrift_message, I_darkdrift_root,
               CP_Message, CP_Function, DRIP_Paths, DRIP_Stat);

I_darkdrift_message = RIGHT_MASK&(DRIP_Stat->I_status);
I_darkdrift_root = (LEFT_MASK&(DRIP_Stat->I_status))>>16;
if (I_darkdrift_message&FITS_MSG_MASK)
   I_darkdrift_message -= FITS_MSG_MASK;

if (I_darkdrift_message != 0) {

   fprintf(fp_Logfile, 
      "\nProgram darkdrift: Status Message: 0x%04x\n",
      I_darkdrift_message);
   fprintf(fp_Logfile, 
      "%s from Function 0x%04x: %s\n",
      CP_Message, I_darkdrift_root, CP_Function);

} else {

   fprintf(fp_Logfile, "%s", "Dark-current offsets ");
   if (DRIP_FITS->I_Operation==1)
      fprintf(fp_Logfile, "%s\n", "estimated from input image.");
   else if (DRIP_FITS->I_Operation==2)
      fprintf(fp_Logfile, "%s\n", 
         "estimated from local before dark-current image.");
   else if(DRIP_FITS->I_Operation==3)
      fprintf(fp_Logfile, "%s\n", 
         "interpolated from local after dark-current image.");
   else if(DRIP_FITS->I_Operation==4)
      fprintf(fp_Logfile, "%s\n", 
         "interpolated from two local dark-current images.");

   fprintf(fp_Logfile, "\nDetailed darkdrift calculational results:\n");
   for (k = kstart; k < kend; k++) {
      kp1 = k + 1;
      fprintf(fp_Logfile, "...for data plane = %d\n", kp1);
      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         fprintf(fp_Logfile, 
            "Darkdrift correction for readout-channel %d = %f\n",
     	    ip1, DRIP_Comp->Correction[index]);
      }
      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         fprintf(fp_Logfile, 
            "Darkdrift uncertainty for readout-channel %d = %f\n",
     	    ip1, DRIP_Comp->Uncertainty[index]);
      }
      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         fprintf(fp_Logfile, 
            "Trimmed average of values in readout-channel %d = %f\n",
	    ip1, DRIP_Comp->Median[index]);
      }
      fprintf(fp_Logfile, 
         "Darkdrift background term = %f\n",
	 DRIP_Comp->Background[kkk]);
      kkk++;
   }

   fprintf(fp_Logfile, 
      "\nProgram darkdrift: Status Message: 0x%04x\n",
      I_darkdrift_message);
   fprintf(fp_Logfile, 
      "Normal exit from Function 0x%04x: %s\n",
      I_darkdrift_root, CP_Function);

}



/* According to John Fowler (8/3/99), there is no need to write anything
   to stderr.

if (I_darkdrift_message != 0) {
   fprintf(stderr, 
     ")-: darkdrift Status Message      0x%04x: %s from Function 0x%04x: %s.\n",
     I_darkdrift_message, CP_Message,  I_darkdrift_root, CP_Function);
}

*/



if (DRIP_Stat->I_NaNFlag) {
   fprintf(fp_Logfile,
      "darkdrift Warning: NaN's were produced in the results.\n");
   fprintf(fp_Logfile,
      "A total of %8d   NaN's were produced in the results.\n",
      DRIP_Stat->I_NaNCount);
}



I_Time = clock();
fprintf(fp_Logfile, "Processing time: %f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

time(&currenttime);
datetime = ctime(&currenttime);
fprintf(fp_Logfile,"Current date/time: %s",datetime);
fprintf(fp_Logfile,"Program %s, version %2.1f, terminated.\n", 
   PROGRAM, DRIVERSN);


if(fp_Logfile != stdout) fclose(fp_Logfile);


if(DRIP_Stat->I_Verbose)
printf("darkdrift_log_writer: Log Writer finished.\n");


}



void decode_message(int             I_message, 
                    int             I_root,
		    char	    *CP_Message,
		    char	    *CP_Function,
		    DRI_Pathnames   *DRIP_Paths,
		    DRI_Status	    *DRIP_Stat)
{

char codefile[2*MAX_FILENAME_LENGTH],line[80],mess[80],numstr[80],*rem;
int ifun=0, imess=0;
unsigned long num, nums;

FILE *GETCODEBOOK, *fopen();

   sprintf(CP_Function, "%s", "Unspecified function.");
   sprintf(CP_Message, "%s", "Unknown error");
   strcpy(codefile, DRIP_Paths->CP_Ancillary_File_Path);
   strcat(codefile, "/darkdrift_errcodes.h");

   if(DRIP_Stat->I_Verbose) 
      printf("Message-code file = %s\n",codefile);

   if ((GETCODEBOOK = fopen(codefile, "r")) == NULL) {

      printf(
         "*** darkdrift_log_writer: Could not open darkdrift_errcodes.h\n");

      DRIP_Stat->I_status = LOG_WRITER|LOG_WRITER;
      sprintf(CP_Message,"%s","ERRCODE_FILE_NOT_FOUND");
      sprintf(CP_Function,"%s","LOG_WRITER");

      return;

   } else {

     while (!feof(GETCODEBOOK)) {
        fscanf(GETCODEBOOK, "%s",line);
	if (line[0]== '#') {
           fscanf(GETCODEBOOK, "%s",mess);
           fscanf(GETCODEBOOK, "%s",numstr);
           num = strtoul(numstr,&rem,16);
           nums = (LEFT_MASK & num) >> 16;

           if ((nums == I_root) && (ifun == 0)) {
	     /* printf("%s %X %s\n",mess,nums,rem); */
              sprintf(CP_Function, "%s", mess);
              ifun = 1;
           }

           if ((num == I_message) && (ifun == 1)) {
	     /* printf("%s %X %s\n",mess,num,rem); */
              sprintf(CP_Message, "%s", mess);
	      imess = 1;
              break;
           }
        }
     }
    
     fclose(GETCODEBOOK);

   }

   if ((ifun != 1) || (imess != 1)) {
      printf("*** darkdrift_log_writer: Error in decoding error message.");
   }

   return;

}


