/* $Id: darkdrift_output.c,v 1.8 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_output.c

Purpose

Save the darkdrift outputs to files.

***********************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_output(DRI_Constants		*DRIP_Const,
		      DRI_Filenames		*DRIP_Fnames,
		      DRI_FITSinfo		*DRIP_FITS,
		      DRI_Computation		*DRIP_Comp,
		      DRI_Status		*DRIP_Stat)
{
int	i, j, k, kk, n, index, k2, k3, k4;
int	I_Length_X, I_Length_Y, I_Num_Frames,I_Index_Inp;
int	I_Tot_Pixels;
int	I_fits_return_status=0;
int     morekeys = 0;

fitsfile 	*ffp_FITS;
fitsfile 	*ffp_FITS_In;

char *tblfile;
int nlen;

FILE *ffp_DATA, *fopen();

time_t  currenttime;
char    *datetime;

int nstat = 4;
char stat[7][15] = {"Tot_Num_Samps ",
                    "Num_Bad_Pixels",
                    "Trimmed_Mean  ",
                    "Trimmd_Std_Dev",
                    "Data_Scale    ",
                    "Correction    ",
                    "Uncertainty   "};

char unit[7][15] = {"i             ",
                    "i             ",
                    "r             ",
                    "r             ",
                    "r             ",
                    "r             ",
                    "r             "};

char calctype[5][16] = {"Input_Image    ",
                        "Before_DC_Image",
                        "After_DC_Image ",
                        "Interpolated   ",
                        "Final          "};



/* Don't return unless error, not warning. */
if ((RIGHT_MASK&DRIP_Stat->I_status) >= 64) return;



/* Copy structure info to local variables */

I_Length_X = DRIP_FITS->I_Length_X;
I_Length_Y = DRIP_FITS->I_Length_Y;
I_Num_Frames = DRIP_FITS->I_Num_Frames;
I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;


/* Output global median value to IPAC-table if available. */

   tblfile = DRIP_Fnames->CP_Filename_Data_Out;
   if (DRIP_Fnames->CP_Filename_Data_Out[0] == '!') 
      tblfile = strtok(DRIP_Fnames->CP_Filename_Data_Out,"!");

   if (DRIP_Stat->I_Verbose) 
      printf("Output IPAC-table file = %s\n",tblfile);

   if ((ffp_DATA = fopen(tblfile, "w")) == NULL) {

      DRIP_Stat->I_status = OUTPUT_MAGS|FOPEN_FAILED; return;

   } else {

      time(&currenttime);
      datetime = ctime(&currenttime);

      if (DRIP_Stat->I_Verbose) {
         printf("\\character comment = Output from %s, version %2.1f\n",
            PROGRAM, DRIVERSN);

         printf("\\character Date-Time = %s",datetime);

         printf("\\character inputFITSfile = %s\n",
            DRIP_Fnames->CP_Filename_FITS_Image1);

      }

      fprintf(ffp_DATA,"\\character comment = Output from %s, version %5.2f\n",
         PROGRAM, DRIVERSN);

      fprintf(ffp_DATA,"\\character Date-Time = %s",datetime);

      fprintf(ffp_DATA,"\\character inputFITSfile = %s\n",
         DRIP_Fnames->CP_Filename_FITS_Image1);

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,""))
         fprintf(ffp_DATA,"\\character beforelocaldarkFITSfile = %s\n",
            DRIP_Fnames->CP_Filename_FITS_Image2);

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,""))
         fprintf(ffp_DATA,"\\character afterlocaldarkFITSfile = %s\n",
            DRIP_Fnames->CP_Filename_FITS_Image3);

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,""))
         fprintf(ffp_DATA,"\\character canonicaldarkFITSfile = %s\n",
            DRIP_Fnames->CP_Filename_FITS_Image4);

      if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask,"")) {
         fprintf(ffp_DATA,"\\character GMaskFITSfile = %s\n",
            DRIP_Fnames->CP_Filename_FITS_Image_GMask);
         fprintf(ffp_DATA,"\\integer GMaskMax = %d\n",
            DRIP_Const->GMaskMax);
      }

      fprintf(ffp_DATA,"\\character outputFITSfile = %s\n",
         &(DRIP_Fnames->CP_Filename_FITS_Out[1]));

      fprintf(ffp_DATA,
      "\\character comment = multCorrFlag values: 0=additive, 1=multiplicative\n");

      fprintf(ffp_DATA,"\\character multCorrFlag = %d\n",
         DRIP_FITS->I_Mult_Corr);

      fprintf(ffp_DATA,
      "\\character comment = ROchannelToExclude values: 0=none, otherwise channels 1-4 allowed\n");

      fprintf(ffp_DATA,"\\character ROchannelToExclude = %d\n",
         DRIP_FITS->I_RO_channel_to_exclude);

      fprintf(ffp_DATA,
      "\\character comment = dataPlane values: 1=All, 2=First or 3=Last.\n");

      fprintf(ffp_DATA,"\\character dataPlane = %d\n",
         DRIP_FITS->I_Data_Plane);

      fprintf(ffp_DATA, "\\real Central_Fraction_Trimmed_Average  = %f\n", 
         DRIP_Const->CentralFraction1);
      fprintf(ffp_DATA, "\\real Central_Fraction_Trimmed_Std_Dev = %f\n", 
         DRIP_Const->CentralFraction2);

      if (DRIP_FITS->I_Operation == 2 || DRIP_FITS->I_Operation == 3 || 
          DRIP_FITS->I_Operation == 4) {
         fprintf(ffp_DATA, "\\real Time_Input = %f\n", 
            DRIP_FITS->TimeInput);
         fprintf(ffp_DATA, "\\real Temp_Input = %f\n", 
            DRIP_FITS->TempInput);
      }

      if (DRIP_FITS->I_Operation == 2 || DRIP_FITS->I_Operation == 4) {
         fprintf(ffp_DATA, "\\real Time_Before = %f\n", 
            DRIP_FITS->TimeBefore);
         fprintf(ffp_DATA, "\\real Temp_Before = %f\n", 
            DRIP_FITS->TempBefore);
      }

      if (DRIP_FITS->I_Operation == 2 || DRIP_FITS->I_Operation == 4) {
         fprintf(ffp_DATA, "\\real Time_After = %f\n", 
            DRIP_FITS->TimeAfter);
         fprintf(ffp_DATA, "\\real Temp_After = %f\n", 
            DRIP_FITS->TempAfter);
      }

      fprintf(ffp_DATA,
         "\\character comment = %s\n", 
         "\"Plane\" column below refers to outputFITSfile.\n");

      fprintf(ffp_DATA,
         "|Index|Plane|Calc_Type       |Statistic      |Readout_1  |Readout_2  |Readout_3  |Readout_4  |\n");


      fprintf(ffp_DATA,
         "|i    |i    |c               |c              |r          |r          |r          |r          |\n");

      nlen = NCALCDOMS;
      index = 1;

      k2 = 0;
      k3 = 0;
      k4 = 0;

      for (k = 0; k < I_Num_Frames; k++) {
         kk = k + 1;

	 /* Output for input image (reference). */

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[0],stat[0]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberSamplesRef[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[0],stat[1]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberBadPixelsRef[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[0],stat[2]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->MedianRef[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[0],stat[3]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->StdDevRef[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[0],stat[4]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->DataScaleRef[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;


            if (DRIP_FITS->I_Operation == 2 || DRIP_FITS->I_Operation == 4) {

  	 /* Output for before image. */

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[1],stat[0]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k2*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberSamplesBefore[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[1],stat[1]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k2*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberBadPixelsBefore[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[1],stat[2]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k2*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->MedianBefore[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[1],stat[3]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k2*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->StdDevBefore[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[1],stat[4]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k2*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->DataScaleBefore[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            }

            if (DRIP_FITS->I_Operation == 3 || DRIP_FITS->I_Operation == 4) {

  	 /* Output for after image. */

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[2],stat[0]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k3*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberSamplesAfter[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[2],stat[1]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k3*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberBadPixelsAfter[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[2],stat[2]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k3*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->MedianAfter[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[2],stat[3]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k3*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->StdDevAfter[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[2],stat[4]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k3*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->DataScaleAfter[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            }


            if (DRIP_FITS->I_Operation == 4) {

	 /* Output for interpolated images. */

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[3],stat[0]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberSamples[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[3],stat[1]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11d ",
                  DRIP_Comp->NumberBadPixels[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[3],stat[2]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->Median[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[3],stat[3]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->StdDev[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[3],stat[4]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.2f ",
                  DRIP_Comp->DataScale[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            }


	 /* Final output. */

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[4],stat[5]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.6f ",
                  DRIP_Comp->Correction[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

            fprintf(ffp_DATA," %5d %5d %15s %15s  ",
               index,kk,calctype[4],stat[6]);
            for (n = 0; n < nlen; n++) {
               I_Index_Inp = k4*NCALCDOMS+n;
               fprintf(ffp_DATA,"%11.3f ",
                  DRIP_Comp->Uncertainty[I_Index_Inp]);
            }
            fprintf(ffp_DATA,"\n");
            index++;

          k2++;
          k3++;
          k4++;

      }


   }

   fclose(ffp_DATA);



   /* OUTPUT IMAGE #1. */

   /* Open the FITS file for writing. */
   
   if (DRIP_Stat->I_Verbose)
      printf("Opening output FITS file #1...\n");

   fits_create_file(&ffp_FITS,
		 DRIP_Fnames->CP_Filename_FITS_Out,
		 &I_fits_return_status);

   if (DRIP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }


   /* 
      Copy input-file header to output file.
   */

   if (DRIP_Stat->I_Verbose)
      printf("Copying input-file FITS keywords to output file...\n");

   fits_open_file(&ffp_FITS_In,
		DRIP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

   fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

   fits_close_file(ffp_FITS_In, &I_fits_return_status);

   fits_flush_file(ffp_FITS, &I_fits_return_status);

   if (DRIP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DRIP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }	


   /* Write header keywords FITS file. */

   if (DRIP_Stat->I_Verbose)
      printf("Outputting FITS keywords...\n");

   darkdrift_output_keywords(ffp_FITS, DRIP_Const, DRIP_Fnames, 
                                DRIP_FITS, DRIP_Comp, DRIP_Stat);

   /* Don't return unless error, not warning. */
   if (((RIGHT_MASK&DRIP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


   /* Write image-data results to the output FITS file. */

   if (DRIP_Stat->I_Verbose)
      printf("Outputting FITS image #1 data...\n");

   fits_write_img(ffp_FITS,
		  TDOUBLE,
		  1, I_Tot_Pixels,
		  DRIP_Comp->DP_Output_Array,
		  &I_fits_return_status);		

   fits_flush_file(ffp_FITS, &I_fits_return_status);

   if (DRIP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }
		

   /* Close the FITS file. */

   if (DRIP_Stat->I_Verbose)
      printf("Closing output FITS file...\n");

   fits_close_file(ffp_FITS, &I_fits_return_status);

   if (DRIP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }

   if(DRIP_Stat->I_Verbose) 
      printf("darkdrift_output: Result written to file:\n%s.\n",
         &DRIP_Fnames->CP_Filename_FITS_Out[1]);



   /* OUTPUT IMAGE #2. */

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out2, "") != 0) {


      /* Open the FITS file for writing. */

      if (DRIP_Stat->I_Verbose)
         printf("Opening output FITS file #2...\n");

      fits_create_file(&ffp_FITS,
	   	    DRIP_Fnames->CP_Filename_FITS_Out2,
		    &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if(I_fits_return_status) {
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* 
         Copy input-file header to output file.
      */

      if (DRIP_Stat->I_Verbose)
         printf("Copying input-file FITS keywords to output file...\n");

      fits_open_file(&ffp_FITS_In,
		DRIP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

      fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

      fits_close_file(ffp_FITS_In, &I_fits_return_status);

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DRIP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }	


      /* Write header keywords FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Outputting FITS keywords...\n");

      darkdrift_output_keywords(ffp_FITS, DRIP_Const, DRIP_Fnames, 
                                DRIP_FITS, DRIP_Comp, DRIP_Stat);

      /* Don't return unless error, not warning. */
      if (((RIGHT_MASK&DRIP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


      /* Write image-data results to the output FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Outputting FITS image #2 data...\n");

      fits_write_img(ffp_FITS,
	      	  TDOUBLE,
		  1, I_Tot_Pixels,
		  DRIP_Comp->DP_Output_Array2 ,
		  &I_fits_return_status);		

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* Close the FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Closing output FITS file...\n");

      fits_close_file(ffp_FITS, &I_fits_return_status);

      if(I_fits_return_status)
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_output: Result written to file:\n%s.\n",
         &DRIP_Fnames->CP_Filename_FITS_Out2[1]);

   }



   /* OUTPUT IMAGE #3. */

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out3, "") != 0) {


      /* Open the FITS file for writing. */

      if (DRIP_Stat->I_Verbose)
         printf("Opening output FITS file #3...\n");

      fits_create_file(&ffp_FITS,
	   	    DRIP_Fnames->CP_Filename_FITS_Out3,
		    &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if(I_fits_return_status) {
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* 
         Copy input-file header to output file.
      */

      if (DRIP_Stat->I_Verbose)
         printf("Copying input-file FITS keywords to output file...\n");

      fits_open_file(&ffp_FITS_In,
		DRIP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

      fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

      fits_close_file(ffp_FITS_In, &I_fits_return_status);

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DRIP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }	


      /* Write header keywords FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Outputting FITS keywords...\n");

      darkdrift_output_keywords(ffp_FITS, DRIP_Const, DRIP_Fnames, 
                                DRIP_FITS, DRIP_Comp, DRIP_Stat);

      /* Don't return unless error, not warning. */
      if (((RIGHT_MASK&DRIP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


      /* Write image-data results to the output FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Outputting FITS image #3 data...\n");

      fits_write_img(ffp_FITS,
	      	  TDOUBLE,
		  1, I_Tot_Pixels,
		  DRIP_Comp->DP_Output_Array3 ,
		  &I_fits_return_status);		

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DRIP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* Close the FITS file. */

      if (DRIP_Stat->I_Verbose)
         printf("Closing output FITS file...\n");

      fits_close_file(ffp_FITS, &I_fits_return_status);

      if(I_fits_return_status)
         DRIP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_output: Result written to file:\n%s.\n",
         &DRIP_Fnames->CP_Filename_FITS_Out3[1]);

   }

}





