/* $Id: darkdrift_output_constkeywords.c,v 1.7 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_output_constkeywords.c

Purpose

Write the keyword values for computation constants
to the open fits output file.

Overview
Use cfitsio functions.

Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c


Internal:
I_fits_return_status = Resturn value of fits function.
D_Num_In = Floating point value to be written to header.
CP_Keyname = name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"

void darkdrift_output_constkeywords(fitsfile    	*ffp_FITS,
				    DRI_Constants	*DRIP_Const,
	  		            DRI_Computation	*DRIP_Comp,
  			            DRI_FITSinfo	*DRIP_FITS,
				    DRI_Status   	*DRIP_Stat)
{
   int	k, I_fits_return_status=0, I_Num_Frames, I_Num_In, i, ip1;
   double D_Num_In;
   char	CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT];
   int kkk, kstart, kend, I_Data_Plane, index, kp1;
   char planeNumStr[3]; /* Allows up to 2 digits for data-plane number */


   if (DRIP_Stat->I_status) return;

   I_Num_Frames = DRIP_FITS->I_Num_Frames;
   I_Data_Plane = DRIP_FITS->I_Data_Plane;

   if (I_Data_Plane == 3) {
      kstart = I_Num_Frames - 1;
      kend = I_Num_Frames;
   } else if (I_Data_Plane == 2) {
      kstart = 0;
      kend = 1;
   } else {
      kstart = 0;
      kend = I_Num_Frames;
   }
   if (kend > 100) kend = 100;  /* Only 2 digits allocated in keyword names. */

   kkk = 0;

   sprintf(CP_Keyname, "%s", "DRICRTYP");
   sprintf(CP_Comment, "%s", 
      "Correction type: 0=Additive, 1=Multiplicative");
   I_Num_In = DRIP_FITS->I_Mult_Corr;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   sprintf(CP_Keyname, "%s", "DRIROEXC");
   sprintf(CP_Comment, "%s", 
      "R.O. channel excluded from background (0=None)");
   I_Num_In = DRIP_FITS->I_RO_channel_to_exclude;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   sprintf(CP_Keyname, "%s", "DRIPFLAG");
   sprintf(CP_Comment, "%s", 
      "Data planes processed: 1=All, 2=First, 3=Last");
   I_Num_In = I_Data_Plane;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   for (k = kstart; k < kend; k++) {

      kp1 = k + 1;

      sprintf(CP_Comment, "%s%d", 
         "Darkdrift calculations for data plane ", kp1);
      fits_write_comment(ffp_FITS,
         CP_Comment,
         &I_fits_return_status);

      if (kp1 < 10) {
         sprintf(planeNumStr, "%s%d", "0", kp1);
      } else if (kp1 < 100) {
         sprintf(planeNumStr, "%d", kp1);
      } else {
	 printf("***Error: Data plane number > 99 not expected; terminating...\n");
         DRIP_Stat->I_status = 
            CONST_KEYWORDS|INPUTS_OUT_OF_RANGE; 
         return;
      }

      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         sprintf(CP_Keyname, "%s%s%s%d", "DDCR", planeNumStr, "_", ip1);
         sprintf(CP_Comment, "%s%d%s%d", 
		 "Darkdrift corr. for data plane=", kp1,", r.o.-chan.=",ip1);
         D_Num_In = DRIP_Comp->Correction[index];
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
      }

      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         sprintf(CP_Keyname, "%s%s%s%d", "DDUN", planeNumStr, "_", ip1);
         sprintf(CP_Comment, "%s%d%s%d", 
		 "Darkdrift unc. for data plane=", kp1,", r.o.-chan.=",ip1);
         D_Num_In = DRIP_Comp->Uncertainty[index];
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
      }

      for (i = 0; i < NCALCDOMS; i++) {
         ip1 = i + 1;
         index = kkk * NCALCDOMS + i;
         sprintf(CP_Keyname, "%s%s%s%d", "DDAV", planeNumStr, "_", ip1);
         sprintf(CP_Comment, "%s%d%s%d", 
		 "Darkdrift avg. for data plane=", kp1,", r.o.-chan.=",ip1);
         D_Num_In = DRIP_Comp->Median[index];
         fits_write_key(ffp_FITS,
   		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
      }

      sprintf(CP_Keyname, "%s%s", "DDBKGD", planeNumStr);
      sprintf(CP_Comment, "%s%d", 
         "Darkdrift background term for data plane=", kp1);
      D_Num_In = DRIP_Comp->Background[kkk];
      fits_write_key(ffp_FITS,
   		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);

      kkk++;

   }

   fits_flush_file(ffp_FITS,
		&I_fits_return_status);

   if (DRIP_Stat->I_Verbose)
      printf("darkdrift_output_constkeywords:  I_fits_return_status = %d\n",
         I_fits_return_status);

		
   if (I_fits_return_status) {
      DRIP_Stat->I_status = 
         CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }		 			




}
