/* $Id: darkdrift_output_filenkeywords.c,v 1.6 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_output_filenkeywords.c

Purpose

Write the keyword values for input and output filenames
to the open fits output file.

Overview
Use cfitsio functions.

Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c


Internal:
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"

void darkdrift_output_filenkeywords(fitsfile    	*ffp_FITS,
				    DRI_Filenames	*DRIP_Fnames,
			            DRI_FITSinfo	*DRIP_FITS,
				    DRI_Status  	*DRIP_Stat)
{
int	I_fits_return_status=0;
char	CP_Keyname[FLEN_KEYWORD];
char    CP_Keyvalue[FLEN_VALUE];
char    RootFilename[FLEN_VALUE],CompleteFilename[MAX_FILENAME_LENGTH];

if (DRIP_Stat->I_status) return;

sprintf(CP_Keyname, "%s", "DRINLFIL");

strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_Namelist);
getRootFilename(RootFilename,CompleteFilename);
strcpy(CP_Keyvalue,RootFilename);
fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		


fits_flush_file(ffp_FITS,
		&I_fits_return_status);

if (DRIP_Stat->I_Verbose)
   printf("darkdrift_output_filenkeywords:  %s%s%s%s%s = %d\n",
      "Writing ", CP_Keyname, " = ",CP_Keyvalue, 
      ", I_fits_return_status = ",I_fits_return_status);

if (I_fits_return_status)
   DRIP_Stat->I_status = FILEN_KEYWORDS|FITS_MSG_MASK|I_fits_return_status;




   sprintf(CP_Keyname, "%s", "DRIIFIL ");
   strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Image1);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,"")) {
      sprintf(CP_Keyname, "%s", "DRIIUFIL");
      strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_In_Unc_Image);
      getRootFilename(RootFilename,CompleteFilename);
      strcpy(CP_Keyvalue,RootFilename);
      fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) {
      sprintf(CP_Keyname, "%s", "DRIIFIL2");
      strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Image2);
      getRootFilename(RootFilename,CompleteFilename);
      strcpy(CP_Keyvalue,RootFilename);
      fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		
   }
   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"")) {
      sprintf(CP_Keyname, "%s", "DRIIFIL3");
      strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Image3);
      getRootFilename(RootFilename,CompleteFilename);
      strcpy(CP_Keyvalue,RootFilename);
      fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		
   }
   if ((strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"")) ||
       (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,""))) {
      sprintf(CP_Keyname, "%s", "DRIIFIL4");
      strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Image4);
      getRootFilename(RootFilename,CompleteFilename);
      strcpy(CP_Keyvalue,RootFilename);
      fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		
   }

   sprintf(CP_Keyname, "%s", "DRIOFIL1");
   strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Out);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		


   sprintf(CP_Keyname, "%s", "DRIOFIL2");
   strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_FITS_Out2);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);				
			

   sprintf(CP_Keyname, "%s", "DRITFIL ");
   strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_Data_Out);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		


sprintf(CP_Keyname, "%s", "DRILGFIL");
strcpy(CompleteFilename,DRIP_Fnames->CP_Filename_Log);
getRootFilename(RootFilename,CompleteFilename);
strcpy(CP_Keyvalue,RootFilename);
fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);	


fits_flush_file(ffp_FITS,
		&I_fits_return_status);

if (DRIP_Stat->I_Verbose)
   printf("darkdrift_output_filenkeywords:  %s%s%s%s%s = %d\n",
      "Writing ", CP_Keyname, " = ",CP_Keyvalue, 
      ", I_fits_return_status = ",I_fits_return_status);

if (I_fits_return_status)
   DRIP_Stat->I_status = FILEN_KEYWORDS|FITS_MSG_MASK|I_fits_return_status;





}


/* Strip pathname off filename. */

void getRootFilename( char *RootFilename, char *CompleteFilename )

{

   int i,n,si,sf;

   n = strlen(CompleteFilename);

   si = 0;
   sf = n;
   for (i = n - 1; i >= 0; i--) {
      if (CompleteFilename[i] == '/' || CompleteFilename[i] == '!') 
      { 
         si = i + 1; 
         break;
      }
      if (CompleteFilename[i] == ' ') { sf = i; }
   }

   for (i = 0; i < sf - si; i++) {
      RootFilename[i] = CompleteFilename[i + si];
   }

   RootFilename[sf - si] = '\0';

}

