/* $Id: darkdrift_output_keywords.c,v 1.8 2007/08/08 20:25:29 laher Exp $ */


/*********************************************** 
darkdrift_output_keywords.c

Purpose

Write the keyword values to the open fits output file.

Overview


Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c


Internal:
I_fits_return_status = Resturn value of fits function.
I_Operation = Which operation was performed.
I_DoConstant = Flags whether operation was writing a constant.
CP_Keyname = name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"
#include "writeDLhistoryline.h"


void darkdrift_output_keywords(fitsfile	        *ffp_FITS,
			       DRI_Constants	*DRIP_Const,
			       DRI_Filenames	*DRIP_Fnames,
			       DRI_FITSinfo	*DRIP_FITS,
			       DRI_Computation	*DRIP_Comp,
			       DRI_Status	*DRIP_Stat)
{
int	I_fits_return_status=0, I_Operation;
char	CP_Keyname[FLEN_KEYWORD];
char    CP_Comment[FLEN_COMMENT];
char    CP_Keyvalue[FLEN_VALUE];
double  version;
char    programname[32], versionstring[32];


if(DRIP_Stat->I_status) return;


/* Copy structure info to local variable */

I_Operation = DRIP_FITS->I_Operation;


/* Write the keywords that must be in every FITS file */

if (DRIP_Stat->I_Verbose)
   printf("Writing mandatory header keywords to output FITS file...\n");

darkdrift_output_stdkeywords( ffp_FITS, DRIP_Fnames, DRIP_FITS, DRIP_Stat);
if(DRIP_Stat->I_status) return;

if (DRIP_Stat->I_Verbose) {
   printf("Writing date to output FITS file...\n");
   printf("darkdrift_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);
}
				
fits_write_date(ffp_FITS, &I_fits_return_status);		 

fits_flush_file(ffp_FITS, &I_fits_return_status);

if (DRIP_Stat->I_Verbose)
   printf("darkdrift_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);

if (I_fits_return_status) {
   DRIP_Stat->I_status = DATE|I_fits_return_status; 
   return;
}		 


/* 
   Description of the data in this FITS file.  Input string to CP_Keyname
   should be no longer than 8 characters.  Otherwise, data in memory will 
   be stepped on.
*/

sprintf(programname, "%s", "DARKDRIFT");
sprintf(versionstring, "%.1f", DRIVERSN);
writeDLhistoryline(ffp_FITS, programname, versionstring, 
   &I_fits_return_status);

		
sprintf(CP_Keyname, "%s", "DRIPROC");
strcpy(CP_Keyvalue,PROGRAM);
   sprintf(CP_Comment, "%s", PROGRAMPURPOSE);

fits_write_key_str(ffp_FITS,
		CP_Keyname, CP_Keyvalue,
		CP_Comment,
		&I_fits_return_status);


sprintf(CP_Keyname, "%s", PROGRAMABB);
strcat(CP_Keyname, "VERSN");
sprintf(CP_Comment, "%s%s%s", "Version number of ",PROGRAM," program");
version = DRIVERSN;

fits_write_key_fixflt(ffp_FITS, 
		CP_Keyname, version, 2,
		CP_Comment,
		&I_fits_return_status);

fits_flush_file(ffp_FITS, &I_fits_return_status);

if (DRIP_Stat->I_Verbose)
   printf("darkdrift_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);

if (I_fits_return_status) {
   DRIP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
   return;
}		 

if (DRIP_Stat->I_Verbose) 
   printf("Writing constant header keywords to output FITS file...\n");

darkdrift_output_constkeywords( ffp_FITS, DRIP_Const, DRIP_Comp, 
   DRIP_FITS, DRIP_Stat);

if (DRIP_Stat->I_Verbose) 
   printf("Writing output header keywords to output FITS file...\n");

/* Files associated with this one. */				
darkdrift_output_filenkeywords( ffp_FITS, DRIP_Fnames, DRIP_FITS, 
   DRIP_Stat);

}

