/* $Id: darkdrift_parse_args.c,v 1.6 2007/08/08 20:25:30 laher Exp $ */


/*********************************************** 
darkdrift_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c


***********************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "darkdrift_errcodes.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_parse_args(int		argc,
		  	  char		**argv,
			  DRI_Constants	*DRIP_Const,
			  DRI_Filenames	*DRIP_Fnames,
			  DRI_Pathnames	*DRIP_Paths,
			  DRI_FITSinfo	*DRIP_FITS,
			  DRI_Status	*DRIP_Stat)
{
int	i=1;
int     I_gotallinputs;

char    CP_Value[STRING_BUFFER_SIZE];

/* Run regardless of I_darkdrift_status so that
   at least log file name will be known */



      while( i < argc )
        {
          if( argv[i][0] == '-' )
            {
              switch( argv[i][1] ) {
              case 'n':      /* -n <input_namelist_fname>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -n <input_namelist_fname> missing argument\n" );
		else
		if(DRIP_Fnames->CP_Filename_Namelist) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_Namelist);
                break;
              case 'i':      /* -i <input_image_fname>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -i <input_image_fname> missing argument\n" );
		else
		if(argv[i-1][2] == '\0')
		if(DRIP_Fnames->CP_Filename_FITS_Image1) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_FITS_Image1);
		if(argv[i-1][2] == 'b')
		if(DRIP_Fnames->CP_Filename_FITS_Image2) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_FITS_Image2);
		if(argv[i-1][2] == 'a')
		if(DRIP_Fnames->CP_Filename_FITS_Image3) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_FITS_Image3);
		if(argv[i-1][2] == 'c')
		if(DRIP_Fnames->CP_Filename_FITS_Image4) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_FITS_Image4);
		if(argv[i-1][2] == 'g')
		if(DRIP_Fnames->CP_Filename_FITS_Image_GMask) 
                   sscanf(argv[i], "%s", 
                      DRIP_Fnames->CP_Filename_FITS_Image_GMask);
		if(argv[i-1][2] == 'u')
		if(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image) 
                   sscanf(argv[i], "%s", 
                      DRIP_Fnames->CP_Filename_FITS_In_Unc_Image);
                break;
	      case 'o':      /* -o <output_fname>  */
		if( ++i >= argc )
                printf("darkdrift_parse_args: -o <fname> missing argument\n" );
		else  
		if(argv[i-1][2] == '1')
                if(DRIP_Fnames->CP_Filename_FITS_Out)	{
		   sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out);  /* clobber */
                   sscanf(argv[i], "%s", &(DRIP_Fnames->CP_Filename_FITS_Out[1]));
		}
		if(argv[i-1][2] == '2')
                if(DRIP_Fnames->CP_Filename_FITS_Out2)	{
		   sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out2);  /* clobber */
                   sscanf(argv[i], "%s", &(DRIP_Fnames->CP_Filename_FITS_Out2[1]));
		}
		if(argv[i-1][2] == 'u')
                if(DRIP_Fnames->CP_Filename_FITS_Out3)	{
		   sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out3);  /* clobber */
                   sscanf(argv[i], "%s", &(DRIP_Fnames->CP_Filename_FITS_Out3[1]));
		}
		if(argv[i-1][2] == '3')
                if(DRIP_Fnames->CP_Filename_Data_Out)	{
		   sscanf("!", "%c", DRIP_Fnames->CP_Filename_Data_Out);  /* clobber */
                   sscanf(argv[i], "%s", &(DRIP_Fnames->CP_Filename_Data_Out[1]));
		}
                break;
	      case 'l':      /* -l <log_fname>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -l <fname> missing argument\n" );
		else
		if(DRIP_Fnames->CP_Filename_Log)
                sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_Log);
                break;
	      case 'g':      /* -g <mult_corr>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -g <mult_corr> missing argument\n" );
		else
		sscanf(argv[i], "%d", &(DRIP_FITS->I_Mult_Corr));
                break;
	      case 'c':      /* -c <ro_channel_to_exclude>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -c <ro_channel_to_exclude> missing argument\n" );
		else
		sscanf(argv[i], "%d", &(DRIP_FITS->I_RO_channel_to_exclude));
                break;
	      case 'p':      /* -p <data_plane>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -p <data_plane> missing argument\n" );
		else
		sscanf(argv[i], "%d", &(DRIP_FITS->I_Data_Plane));
                break;
/* -t <transposeflag>  or -ti, -tib, -tia, -te, -teb, -tea */
	      case 't':   

                 if ( ++i >= argc )
                    printf("darkdrift_parse_args: -t... <value> missing argument\n" );
                 else
		 if (argv[i-1][2] == 'i') {
		    if (argv[i-1][3] == '\0') {
  		       sscanf(argv[i], "%s", CP_Value);
                       DRIP_FITS->TimeInput = atof(CP_Value);
                    }
		    if (argv[i-1][3] == 'b') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DRIP_FITS->TimeBefore = atof(CP_Value);
                    }
		    if (argv[i-1][3] == 'a') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DRIP_FITS->TimeAfter = atof(CP_Value);
                    }
                 }

		 if (argv[i-1][2] == 'e') {
		    if (argv[i-1][3] == '\0') {
  		         sscanf(argv[i], "%s", CP_Value);
                         DRIP_FITS->TempInput = atof(CP_Value);
		    }
		    if (argv[i-1][3] == 'b') {
  		       sscanf(argv[i], "%s", CP_Value);
                       DRIP_FITS->TempBefore = atof(CP_Value);
		    }
		    if (argv[i-1][3] == 'a') {
  		         sscanf(argv[i], "%s", CP_Value);
                         DRIP_FITS->TempAfter = atof(CP_Value);
		    }
                 }

		 if (argv[i-1][2] == '\0')
		    sscanf(argv[i], "%d", &(DRIP_FITS->I_Transpose));

                 break;
	      case 'm':      /* -m <max_usable_mask_level>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -m <max_usable_mask_level> missing argument\n" );
		else
		if(argv[i-1][2] == 'g') {
		   sscanf(argv[i], "%d", &(DRIP_Const->GMaskMax));
                }
                break;
              case 'a':      /* -a <input_ancillary_file_pathname>  */
                if( ++i >= argc )
                printf("darkdrift_parse_args: -a <input_ancillary_file_pathname> missing argument\n" );
		else
		if(DRIP_Paths->CP_Ancillary_File_Path) /* Filename can be stored */
                sscanf(argv[i], "%s", DRIP_Paths->CP_Ancillary_File_Path);
                break;
              case 'd':      /* -debug   */
                DRIP_Stat->I_Debug = 1 - DRIP_Stat->I_Debug;
                break;
              case 'v':      /* -debug   */
		if(argv[i][2] != 'v')
                DRIP_Stat->I_Verbose = 1 - DRIP_Stat->I_Verbose;
		if(argv[i][2] == 'v')
		DRIP_Stat->I_SuperVerbose = 1 - DRIP_Stat->I_SuperVerbose;
                break;

	      case 'f':      /* -o <output_fname>  */
		if( ++i >= argc )
                printf("darkdrift_parse_args: -o <center_fraction> missing argument\n" );
		else  
		if(argv[i-1][2] == '1') {
  		   sscanf(argv[i], "%s", CP_Value);
                   DRIP_Const->CentralFraction1 = atof(CP_Value);
                }
		if(argv[i-1][2] == '2') {
  		   sscanf(argv[i], "%s", CP_Value);
                   DRIP_Const->CentralFraction2 = atof(CP_Value);
                }
                break;
              default:
                printf("darkdrift_parse_args: unknown argument\n");
              }
            }
          else
            {
              printf("darkdrift_parse_args: Command line syntax error:\n");
              printf("   Previous argument = %s\n",argv[i - 1]);
              printf("   Current argument = %s\n",argv[i]);
            }
          i++;
        }


/* Print to stdout the program name and version number */

if(DRIP_Stat->I_Verbose) 
   printf("darkdrift_parse_args: Program %s, Version %2.1f\n", 
      PROGRAM, DRIVERSN);


/* Check code inputs. */

I_gotallinputs = 0;

   if(DRIP_Stat->I_Verbose) {
      if (strcmp(DRIP_Fnames->CP_Filename_Namelist,"") != 0) {
         printf("darkdrift_parse_args: Namelist File = %s\n", 
            DRIP_Fnames->CP_Filename_Namelist);
      } else {
         printf("darkdrift_parse_args: Information only: %s\n",
            "No namelist file specified.");
      }
   }

   if(DRIP_Stat->I_Verbose) {
      if (strcmp(DRIP_Paths->CP_Ancillary_File_Path,"") != 0) {
         printf("darkdrift_parse_args: Ancilllary File Pathname = %s\n", 
            DRIP_Paths->CP_Ancillary_File_Path);
      } else {
         printf("darkdrift_parse_args: Information only: %s %s\n",
            "No ancillary file pathname specified;",
            "defaulting to current directory.");
      }
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image1,"") != 0) {
      DRIP_FITS->I_Operation = 1;
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_Image1);
   } else {
      printf(
         "darkdrift_parse_args: Missing input FITS filename %s\n",
         "(-i <fname>).");
      I_gotallinputs = 1;
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") != 0) {
      DRIP_FITS->I_Operation = 2;
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input Dark-Current Before Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_Image2);
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"") != 0) {
      if (DRIP_FITS->I_Operation == 1) { 
         DRIP_FITS->I_Operation = 3;
      } else {
         DRIP_FITS->I_Operation = 4;
      }
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input Dark-Current After Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_Image3);
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,"") != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input Canonical Dark Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_Image4);
   } else {
      if ((strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") != 0) ||
          (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"") != 0)) {
         printf(
            "darkdrift_parse_args: Missing input FITS filename %s\n",
            "(-ic <canonical_dark_fname>).");
         I_gotallinputs = 1;
      }
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask,"") != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input GMask Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_Image_GMask);
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,"") != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Input Uncertainty Image = %s\n", 
         DRIP_Fnames->CP_Filename_FITS_In_Unc_Image);
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out,"")  != 0) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: Output File Corrected Image = %s\n", 
         &DRIP_Fnames->CP_Filename_FITS_Out[1]);
   } else {
      printf(
         "darkdrift_parse_args: Missing output FITS filename %s\n",
         "(-o1 <fname>).");
      I_gotallinputs = 1;
   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out2,"") != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Output Corrections FITS-file = %s\n", 
         &DRIP_Fnames->CP_Filename_FITS_Out2[1]);
   } 

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Out3,"") != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Output Uncertainties FITS-file = %s\n", 
         &DRIP_Fnames->CP_Filename_FITS_Out3[1]);
   } 

   if (strcmp(DRIP_Fnames->CP_Filename_Data_Out,"")  != 0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: Output Table = %s\n", 
         &DRIP_Fnames->CP_Filename_Data_Out[1]);
   } else {
      printf(
         "darkdrift_parse_args: Missing output table-data filename %s\n",
         "(-o3 <fname>).");
      I_gotallinputs = 1;
   }

   if (DRIP_Const->CentralFraction1 >= 0.0 && 
       DRIP_Const->CentralFraction1 <= 1.0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: %s = %f\n", 
            "Trimmed average central fraction",DRIP_Const->CentralFraction1);
   } else {
      printf("darkdrift_parse_args: %s%s\n", 
         "Trimmed average central fraction not in [0, 1] range ",
         "(-f1 <value>).");
      I_gotallinputs = 2;
   }

   if (DRIP_Const->CentralFraction2 >= 0.1 && 
       DRIP_Const->CentralFraction2 <= 1.0) {
      if(DRIP_Stat->I_Verbose)
         printf("darkdrift_parse_args: %s = %f\n", 
            "Trimmed std. dev. central fraction",DRIP_Const->CentralFraction2);
   } else {
      printf("darkdrift_parse_args: %s%s\n", 
         "Trimmed std. dev. central fraction not in [0.1, 1] range ",
         "(-f2 <value>).");
      I_gotallinputs = 2;
   }

   if (!((DRIP_Const->GMaskMax < 0) || (DRIP_Const->GMaskMax > 32767))) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: %s = %d\n", 
            "GMaskMax",DRIP_Const->GMaskMax);
   } else {
      printf("darkdrift_parse_args: %s\n",
         "GMaskMax out of range (-e <between 0 and 32767, inclusive>).");
      I_gotallinputs = 2;
   }

   if (!(DRIP_FITS->I_Transpose < 1 || DRIP_FITS->I_Transpose > 2)) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: %s = %d\n", 
            "Input-image transpose flag",DRIP_FITS->I_Transpose);
   } else {
      printf("darkdrift_parse_args: %s\n",
         "Input-image transpose flag out of range (-t <1 or 2>).");
      I_gotallinputs = 2;
   }

   if (!(DRIP_FITS->I_Mult_Corr < 0 || DRIP_FITS->I_Mult_Corr > 1)) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: %s = %d\n", 
            "Multiplicative-correction flag",DRIP_FITS->I_Mult_Corr);
   } else {
      printf("darkdrift_parse_args: %s\n",
         "Multiplicative-correction flag out of range (-g <0 or 1>).");
      I_gotallinputs = 2;
   }

   if (!(DRIP_FITS->I_RO_channel_to_exclude < 0 || 
         DRIP_FITS->I_RO_channel_to_exclude > 4)) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: %s = %d\n", 
            "Readout channel to exclude",DRIP_FITS->I_RO_channel_to_exclude);
   } else {
      printf("darkdrift_parse_args: %s\n",
         "Readout channel to exclude of range (-c <0-4>; 0=include all).");
      I_gotallinputs = 2;
   }

   if (!(DRIP_FITS->I_Data_Plane < 1 || DRIP_FITS->I_Data_Plane > 3)) {
      if(DRIP_Stat->I_Verbose) 
         printf("darkdrift_parse_args: %s = %d\n", 
            "Data-plane calculation flag",DRIP_FITS->I_Data_Plane);
   } else {
      printf("darkdrift_parse_args: %s\n",
         "Data-plane calculation flag out of range (-p <1, 2, or 3>).");
      I_gotallinputs = 2;
   }

   if (DRIP_FITS->TimeInput > DRIP_FITS->TimeAfter || 
       DRIP_FITS->TimeInput < DRIP_FITS->TimeBefore ||
       DRIP_FITS->TimeAfter < DRIP_FITS->TimeBefore) {
      printf("darkdrift_parse_args: %s%s\n",
         "Time values out of sequence ",
         "(require t_before < t_input < t_after).");
      I_gotallinputs = 3;
   }


   if(DRIP_Stat->I_Verbose) {

      printf("darkdrift_parse_args: Log File = %s\n", 
         DRIP_Fnames->CP_Filename_Log);

      printf("darkdrift_parse_args: I_Debug = %d\n", 
         DRIP_Stat->I_Debug);

      printf("darkdrift_parse_args: I_Verbose = %d\n", 
         DRIP_Stat->I_Verbose);

      printf("darkdrift_parse_args: I_SuperVerbose = %d\n", 
         DRIP_Stat->I_SuperVerbose);

   }


   if (DRIP_Stat->I_status) return;

   if (I_gotallinputs == 1) {
      DRIP_Stat->I_status = PARSE_ARGS|MISSING_INPUT;
      return;
   }
   if (I_gotallinputs == 2) {
      DRIP_Stat->I_status = PARSE_ARGS|INPUTS_OUT_OF_RANGE;
      return;
   }
   if (I_gotallinputs == 3) {
      DRIP_Stat->I_status = PARSE_ARGS|TIMES_OUT_OF_ORDER;
      return;
   }



}





