/* $Id: darkdrift_parse_namelist.c,v 1.6 2007/08/08 20:25:30 laher Exp $ */


/*********************************************** 
darkdrift_parse_namelist.c

Purpose
Read and parse a parameter file to get input and output file
names constants, and processing flags. 

************************************************/

#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_parse_namelist(int		argc,
			      char		**argv,
			      DRI_Constants	*DRIP_Const,
			      DRI_Filenames	*DRIP_Fnames,
			      DRI_Pathnames	*DRIP_Paths,
			      DRI_FITSinfo	*DRIP_FITS,
			      DRI_Status	*DRIP_Stat)
{


  FILE	*fp_namelist;
  char	*CP_buf;
  char	CP_NextWord[STRING_BUFFER_SIZE];
  char  CP_Equals[STRING_BUFFER_SIZE];
  char  CP_Value[STRING_BUFFER_SIZE];
  char  CP_Comment[STRING_BUFFER_SIZE];
  char	*CP_Pointer, *CP_Comma;
     
  int	i=1, I_Got_Namelist=0, I_SectionsFound=0;
  int	I_fildes_nl=0, I_str_length=0;
  int	I_past_end=0, I_found=0;
  
  struct stat  STAT_nlbuf, *STATP_nlbuf;
  
if(argc==1)	{

fprintf(stdout, "Usage: darkdrift\n");
fprintf(stdout, "       -n <input_namelist_fname> (Optional)\n");
fprintf(stdout, "       -i <input_image_fname> (Required or namelist)\n");
fprintf(stdout, "       -iu <input_uncertainty_image_fname> (Optional)\n");
fprintf(stdout, "       -ig <input_gmask_image_fname> (Optional)\n");
fprintf(stdout, 
   "       -ib <input_darkcurrentimage_before_fname> (Optional)\n");
fprintf(stdout, 
   "       -ia <input_darkcurrentimage_after_fname> (Optional)\n");
fprintf(stdout, 
   "       -ic <input_darkcurrentimage_canonical_fname> %s\n",
   "(Required if before/after darks are specified)");
fprintf(stdout, "       -t <integer value> %s",
   "(1=Don't transpose input image, 2=Do)\n");
fprintf(stdout, "       -g <mult_corr> %s\n",
   "(0=Additive, 1=Multiplicative; default is 0)");
fprintf(stdout, "       -c <ro_channel_to_exclude> %s\n",
   "(0=None; default is 0)");
fprintf(stdout, "       -p <data_plane_to_process> %s\n",
   "(1=All, 2=First, 3=Last; default is 1)");
fprintf(stdout, "       -o1 <output_corrected_fits_fname> %s\n",
   "(Required)");
fprintf(stdout, "       -o2 <output_correction_fits_fname> %s\n",
   "(Optional)");
fprintf(stdout, "       -ou <output_uncertainty_fits_fname> %s\n",
   "(Optional)");
fprintf(stdout, "       -o3 <output_table_data_fname> %s\n",
   "(Required)");
fprintf(stdout, "       -f1 <trim_average_central_fraction> %s\n",
   "(Optional)");
fprintf(stdout, "       -f2 <trim_stddev_central_fraction> %s\n",
   "(Optional)");
fprintf(stdout, 
   "       -ti <time_input> (Optional)\n");
fprintf(stdout, 
   "       -tib <time_before> (Optional)\n");
fprintf(stdout, 
   "       -tia <time_after> (Optional)\n");
fprintf(stdout, 
   "       -te <temp_after> (Optional)\n");
fprintf(stdout, 
   "       -teb <temp_before> (Optional)\n");
fprintf(stdout, 
   "       -tea <temp_after> (Optional)\n");
fprintf(stdout, 
   "       -mg <max_usable_gmask_level> %s\n",
   "(Optional)");
fprintf(stdout, "       -l <log_fname> (Default is stdout)\n");
fprintf(stdout, "       -a <ancillary_file_path> (Optional)\n");
fprintf(stdout, "       -d (prints debug statements) \n");
fprintf(stdout, "       -v (verbose output)          \n");
fprintf(stdout, "       -vv (superverbose output)\n\n");   

DRIP_Stat->I_status = NAMELIST|NO_ARGS; return;

}
				

/* Get namelist filename from command line */
      while( i < argc )	{
          if( argv[i][0] == '-' )	{
              switch( argv[i][1] ) {
              case 'n':      /* -n <input_namelist_fname>  */
                if( ++i >= argc )
                printf("darkdrift_parse_namelist: -n <input_namelist_fname> missing argument\n" );
		else
		if(DRIP_Fnames->CP_Filename_Namelist) /* Filename can be stored */
                I_Got_Namelist = sscanf(argv[i], "%s", DRIP_Fnames->CP_Filename_Namelist); 
                break;
              default:
                ;
              }
            }
          i++;
        }

if (!I_Got_Namelist) {
   printf("darkdrift_parse_namelist: Information only: %s\n",
          "Namelist not specified.");
   return;
}

   /* Open the namelist file to get its length */  

   if ((I_fildes_nl = open(DRIP_Fnames->CP_Filename_Namelist,O_RDONLY)) == -1)
   {
      printf("darkdrift_parse_namelist: Error opening file %s\n",
      DRIP_Fnames->CP_Filename_Namelist);
      DRIP_Stat->I_status = NAMELIST|FOPEN_FAILED; return;
   }

/* Find number of characters in namelist file */
   STATP_nlbuf = &STAT_nlbuf;
   fstat(I_fildes_nl, STATP_nlbuf);
   I_str_length = STATP_nlbuf->st_size;


   #if 0
   /* Open the namelist file */  
   if(strcmp(DRIP_Fnames->CP_Filename_Namelist, "stdin") == 0)	{
   fprintf(stdout, "darkdrift_parse_namelist: Taking namelist from stdin\n");
   fp_namelist = stdin;
   }
   else	{  
   if( (fp_namelist=fopen(DRIP_Fnames->CP_Filename_Namelist,"r"))==NULL) {
    printf("darkdrift_parse_namelist: Error opening file %s",
    DRIP_Fnames->CP_Filename_Namelist);
    DRIP_Stat->I_status = NAMELIST|DRIPEN_FAILED; return;
  }
  }
#endif

/* Associate a stream with the fildes generated above */
  fp_namelist=fdopen(I_fildes_nl,"r");

/* Allocate memory for namelist buffer, */
  CP_buf = (char *) calloc(I_str_length+2, sizeof(char));

/* Read the namelist data */
  /*memset(CP_buf, 0, I_str_length);*/
  if(fread(CP_buf, sizeof(char), I_str_length, fp_namelist) == 0) {
	printf("darkdrift_parse_namelist: Could not read from file %s",
	DRIP_Fnames->CP_Filename_Namelist);
    DRIP_Stat->I_status = NAMELIST|COULD_NOT_READ; return;
  }
  CP_buf[I_str_length] = '/0';
  
/* Close the stream */
  fclose(fp_namelist);

/* Look for the darkdrift section */
#if 0
	for(CP_Pointer = CP_buf;
	strncmp("&DARKDRIFTIN", CP_Pointer, strlen("&DARKDRIFTIN")) != 0;
	CP_Pointer = strchr(CP_Pointer+1, '&'))
	I_SectionsFound++;
#endif
#if 1
CP_Pointer = CP_buf;
while(CP_Pointer <= CP_buf + I_str_length && !I_past_end && !I_found)
     {
     CP_Pointer = strchr(CP_Pointer, '&');
     if (CP_Pointer + strlen("&DARKDRIFTIN") >= CP_buf + I_str_length)
        I_past_end = 1;
     else
        {
        if (strncmp("&DARKDRIFTIN", CP_Pointer, strlen("&DARKDRIFTIN")))
           CP_Pointer += strlen("&");
        else
           I_found = 1;
        }
     }

if (I_past_end)
   {
   DRIP_Stat->I_status = NAMELIST|COULD_NOT_READ; 
   return;
   }
#endif

	
	CP_Pointer += strlen("&DARKDRIFTIN");

/* Match the string values read to the variable definitions and
   write the  values into the appropriate variables. */
	for(CP_Comma = strtok(CP_Pointer, ","); 
	CP_Comma != NULL && strchr(CP_Comma, '&') == 0;
	CP_Comma = strtok(NULL, ","))	{
	  sscanf(CP_Comma, "%s%s%s", CP_NextWord, CP_Equals, CP_Value);
	  if(strcmp(CP_NextWord, "Comment")==0) 
	  strcpy(CP_Comment, CP_Value);

	  if(strcmp(CP_NextWord, "FITS_Image_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DRIP_Fnames->CP_Filename_FITS_Image1);

	  if(strcmp(CP_NextWord, "FITS_Before_Image_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DRIP_Fnames->CP_Filename_FITS_Image2);

	  if(strcmp(CP_NextWord, "FITS_After_Image_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DRIP_Fnames->CP_Filename_FITS_Image3);

	  if(strcmp(CP_NextWord, "FITS_Dark_Image_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DRIP_Fnames->CP_Filename_FITS_Image4);

	  if(strcmp(CP_NextWord, "FITS_Image_GMask_Filename")==0)
	     sscanf(&CP_Value[1], "%[^']", 
                DRIP_Fnames->CP_Filename_FITS_Image_GMask);

	  if(strcmp(CP_NextWord, "FITS_Image_In_Unc_Filename")==0)
	     sscanf(&CP_Value[1], "%[^']", 
                DRIP_Fnames->CP_Filename_FITS_In_Unc_Image);

	  if(strcmp(CP_NextWord, "FITS_Out_Filename")==0) {
  	     sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out); 
   	     sscanf(&CP_Value[1], "%[^']", 
                &(DRIP_Fnames->CP_Filename_FITS_Out[1]));
	  }

	  if(strcmp(CP_NextWord, "FITS_Out2_Filename")==0) {
	     sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out2);   
	     sscanf(&CP_Value[1], "%[^']", 
                &(DRIP_Fnames->CP_Filename_FITS_Out2[1]));
	  }

	  if(strcmp(CP_NextWord, "FITS_Image_Out_Unc_Filename")==0) {
	     sscanf("!", "%c", DRIP_Fnames->CP_Filename_FITS_Out3);   
	     sscanf(&CP_Value[1], "%[^']", 
                &(DRIP_Fnames->CP_Filename_FITS_Out3[1]));
	  }

	  if(strcmp(CP_NextWord, "Data_Out_Filename")==0) {
  	     sscanf("!", "%c", DRIP_Fnames->CP_Filename_Data_Out);   
	     sscanf(&CP_Value[1], "%[^']", 
                &(DRIP_Fnames->CP_Filename_Data_Out[1]));
	  }

	  if(strcmp(CP_NextWord, "Log_Filename")==0) 
	     sscanf(&CP_Value[1], "%[^']", DRIP_Fnames->CP_Filename_Log);
	  if(strcmp(CP_NextWord, "Ancillary_File_Path")==0) 
	     sscanf(&CP_Value[1], "%[^']", DRIP_Paths->CP_Ancillary_File_Path);
	  if(strcmp(CP_NextWord, "Transpose")==0) 
	     DRIP_FITS->I_Transpose = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "Mult_Corr")==0) 
	     DRIP_FITS->I_Mult_Corr = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "RO_Chan_To_Exclude")==0) 
	     DRIP_FITS->I_RO_channel_to_exclude = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "Data_Plane")==0) 
	     DRIP_FITS->I_Data_Plane = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "Time_Input")==0) 
	     DRIP_FITS->TimeInput = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Time_Before")==0) 
	     DRIP_FITS->TimeBefore = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Time_After")==0) 
	     DRIP_FITS->TimeAfter = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Temp_Input")==0) 
	     DRIP_FITS->TempInput = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Temp_Before")==0) 
	     DRIP_FITS->TempBefore = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Temp_After")==0) 
	     DRIP_FITS->TempAfter = atof(CP_Value);
	  if(strcmp(CP_NextWord, "CentralFraction1")==0) 
  	     DRIP_Const->CentralFraction1 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "CentralFraction2")==0) 
  	     DRIP_Const->CentralFraction2 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "GMaskMax")==0) 
	     DRIP_Const->GMaskMax = atoi(CP_Value);
	}


  free(CP_buf);
   
if(DRIP_Stat->I_Verbose)	
   printf("darkdrift_parse_namelist: Namelist File = \t%s\n",
      DRIP_Fnames->CP_Filename_Namelist);

}





