/* $Id: darkdrift_read_data.c,v 1.6 2007/08/08 20:25:30 laher Exp $ */


/*********************************************** 
darkdrift_read_data.c

Purpose

Read the image data from a FITS file and save into an array.

Overview

Open the FITS file, allocate memory, read the data into an array, 

Definitions of Variables

External:
See darkdrift_defs.h and darkdrift_exec.c

 
Internal:
I_fits_return_status = Resturn value of fits function.
I_ndims_found = Number of data dimensions found.
I_anynull = Whether nulls were found in the data.
LP_naxes = Array to store lengths of axes of data.
D_nullval = Set to zero, indicating not to replace nulls.


***********************************************/

#include <time.h>
#include <stdio.h>
#include "darkdrift_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkdrift.h"
#include "darkdrift_defs.h"


void darkdrift_read_data(DRI_Filenames	 *DRIP_Fnames,
			 DRI_FITSinfo	 *DRIP_FITS,
			 double 	**DPP_Data_Image1,
			 double 	**DPP_Data_Image2,
			 double	        **DPP_Data_Image3,
			 double         **DPP_Data_Image4,
                         double	        **DPP_Data_In_Unc_Image,
			 int	        **DPP_Data_Image_GMask,
			 DRI_Status	 *DRIP_Stat)
{
int	i, j, k;
int	I_fits_return_status=0, I_ndims_found, I_anynull;
int     I_Length_X=0, I_Length_Y=0;
long	LP_naxes[3];
double	D_nullval=0;


   if (DRIP_Stat->I_status) return;

   /* Get the data and keyword values from the input image1 file */
   darkdrift_read_image1( DRIP_Fnames, DRIP_FITS, DPP_Data_Image1, DRIP_Stat);

   I_Length_X = DRIP_FITS->I_Length_X;
   I_Length_Y = DRIP_FITS->I_Length_Y;


   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image2,"") != 0) {
      darkdrift_read_image2( DRIP_Fnames, DRIP_FITS, DPP_Data_Image2, 
         DRIP_Stat);

         if (I_Length_X != DRIP_FITS->I_Length_X2) 
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

         if (I_Length_Y != DRIP_FITS->I_Length_Y2)
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image3,"") != 0) {
      darkdrift_read_image3( DRIP_Fnames, DRIP_FITS, DPP_Data_Image3, 
         DRIP_Stat);

         if (I_Length_X != DRIP_FITS->I_Length_X3) 
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

         if (I_Length_Y != DRIP_FITS->I_Length_Y3)
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image4,"") != 0) {
      darkdrift_read_image4( DRIP_Fnames, DRIP_FITS, DPP_Data_Image4, 
         DRIP_Stat);

         if (I_Length_X != DRIP_FITS->I_Length_X4) 
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

         if (I_Length_Y != DRIP_FITS->I_Length_Y4)
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_Image_GMask,"") != 0) {
      darkdrift_read_gmask_image( DRIP_Fnames, DRIP_FITS, 
         DPP_Data_Image_GMask, DRIP_Stat);

         if (I_Length_X != DRIP_FITS->I_Length_X) 
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

         if (I_Length_Y != DRIP_FITS->I_Length_Y)
            { DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; return; }

   }

   if (strcmp(DRIP_Fnames->CP_Filename_FITS_In_Unc_Image,"") != 0) {
      darkdrift_read_unc_image( DRIP_Fnames, DRIP_FITS, 
         DPP_Data_In_Unc_Image, DRIP_Stat);

         if ((I_Length_X != DRIP_FITS->I_Length_X) ||
             (I_Length_Y != DRIP_FITS->I_Length_Y)) {
	   printf("*** darkdrift_read_date: %s\n", 
		  "Inconsistent input uncertainty-image size; quitting...");
           DRIP_Stat->I_status = READ_DATA|INCONSIST_IMAGE_ATTRIB; 
           return; 
        }

   }

}

