/* $Id: datascale_estimate.c,v 1.5 2007/08/08 20:25:31 laher Exp $ */


/*********************************************** 
datascale_estimate.c

Purpose

Implements robust estimation method of Rousse and Croux (1993)
for scale of a distribution of data.

Reference:

     Rousseeuw, P.J. and Croux, C. (1993), "Alternatives to the
        Median Absolute Deviation," Journal of the American
        Statistical Association, Vol. 88, 1273-1283.

Number of data points must be greater than or equal to 2 and
less than or equal to 100000.

************************************************/

#include <stdio.h>
#include "datascale_estimate.h"
/* Includes for mathc90 sort routine. */
#include "mathc90.h"
#include <stdlib.h>
#include <math.h>
#include "fcrt.h"


void datascale_estimate(double x[], long n, double *Sn) {

   int	i, k, tryA, tryB, diff, Amin, Amax, even, half;
   int leftA, leftB, rightA, rightB, nA, nB, length;

   long nstart;

   double y[100000],a2[100000], medA, medB, cn, pull;


   nstart = 1;
   dsort(x, nstart, n);

   for (i = n; i >= 1 ; i--) {
      y[i] = x[i-1];
   }


   a2[1] = y[n/2+1] - y[1];

   for (i = 2; i <= (n + 1) / 2; i++) {

      nA = i - 1;
      nB = n - i;
      diff = nB - nA;
      leftA = 1;
      leftB = 1;
      rightA = nB;
      rightB = nB;
      Amin = diff / 2 + 1;
      Amax = diff / 2 + nA;

loopback:

      if (leftA < rightA) {
         length = rightA - leftA + 1;
         even = 1 - length % 2;
         half = (length - 1) / 2;
         tryA = leftA + half;
         tryB = leftB + half;
         if (tryA < Amin) {
            rightB = tryB;
            leftA = tryA + even;
         } else {
	    if (tryA > Amax) {
               rightA = tryA;
               leftB = tryB + even;           
	    } else {
               medA = y[i] - y[i - tryA + Amin - 1];
               medB = y[tryB + i] - y[i];
               if (medA >= medB) {
                  rightA = tryA;
                  leftB = tryB + even;
               } else {
                  rightB = tryB;
                  leftA = tryA + even;
               }
            }
         }
         goto loopback;
      }
      if (leftA > Amax) {
         a2[i] = y[leftB + i] - y[i];
      } else {
         medA = y[i] - y[i - leftA + Amin - 1];
         medB = y[leftB + i] - y[i];

         a2[i] = medA;
         if (medB < medA) a2[i] = medB;
      }
   }

   for (i = (n + 1) / 2 + 1; i <= n - 1; i++) {
      nA = n - i;
      nB = i - 1;
      diff = nB - nA;
      leftA = 1;
      leftB = 1;
      rightA = nB;
      rightB = nB;
      Amin = diff / 2 + 1;
      Amax = diff / 2 + nA;

loopback2:

      if (leftA < rightA) {
         length = rightA - leftA + 1;
         even = 1 - length % 2;
         half = (length - 1) / 2;
         tryA = leftA + half;
         tryB = leftB + half;
         if (tryA < Amin) {
            rightB = tryB;
            leftA = tryA + even;
         } else {
	    if (tryA > Amax) {
               rightA = tryA;
               leftB = tryB + even;           
	    } else {
               medA = y[i + tryA - Amin + 1] - y[i];
               medB = y[i] - y[i - tryB];
               if (medA >= medB) {
                  rightA = tryA;
                  leftB = tryB + even;
               } else {
                  rightB = tryB;
                  leftA = tryA + even;
               }
            }
         }
         goto loopback2;
      }
      if (leftA > Amax) {
         a2[i] = y[i] - y[i - leftB ];
      } else {
         medA = y[i + leftA - Amin + 1] - y[i];
         medB = y[i] - y[i - leftB];
         a2[i] = medA;
         if (medB < medA) a2[i] = medB;
      }
   }
   a2[n] = y[n] - y[(n + 1) / 2];
   cn = 1.0;
   if (n <= 9) {
      if (n == 2) cn = 0.743;
      if (n == 3) cn = 1.851;
      if (n == 4) cn = 0.954;
      if (n == 5) cn = 1.351;
      if (n == 6) cn = 0.993;
      if (n == 7) cn = 1.198;
      if (n == 8) cn = 1.005;
      if (n == 9) cn = 1.131;
   } else {
      if (n % 2 == 1) 
         cn = ((double) n) / ((double) n - 0.9);
   }

   k = (n + 1) / 2;
   pull_datascale_estimate(a2, n, k, &pull);

   *Sn = cn * 1.1926 * pull;

}


void pull_datascale_estimate(double a[], long n, long k, double *pull) {

   /* Finds the kth-order statistic. */

   int j, l, lr, jnc;

   double b[100000], ax, buffer;

   for (j = 1; j <= n; j++) {
      b[j] = a[j];
   }

   l = 1;
   lr = n;
 branch20:
   if (l >= lr) 
      goto branch90;
   ax = b[k];
   jnc = l;
   j = lr;
 branch30:
   if (jnc > j) 
      goto branch80;
 branch40:
   if (b[jnc] >= ax) 
      goto branch50;
   jnc += 1;
   goto branch40;
 branch50:
   if (b[j] <= ax) 
      goto branch60;
   j -= 1;
   goto branch50;
 branch60:
   if (jnc > j) 
      goto branch70;
   buffer = b[jnc];
   b[jnc] = b[j];
   b[j] = buffer;
   jnc += 1;
   j -= 1;
 branch70:
   goto branch30;
 branch80:
   if (j < k)
      l = jnc;
   if (k < jnc)
      lr = j;      
   goto branch20;
 branch90:
   *pull = b[k];

}






