/* $Id: darkest_asym_trim.c,v 1.4 2002/09/04 19:58:48 fan Exp $ */


/***************************************
darkest_find_trim_avg.c

Purpose
find the trim average value for a given list;

****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"

darkest_asym_trim(double  *array,
                  double  **array_clip,
	          long    *dimension,
                  Constants *Const,
                  Status *Stat) {

int i, j, j1, k, start_index=0, end_index=0, tmp;
int num_samples;
int reject;
int start_flag, end_flag;
int index_max, index_min;
long dim;
double *local_array;
double median, absdev, sum, sum_def;
double dmax, *dis_from_median;
double clip, med_dis, absdev_dis, value;

dim = *dimension;
if (dim == 0) {
    printf("darkest_find_trim_avg:  do not have useful data for trim-average\n");
    Stat->I_status = COMPUTE|NO_USEFUL_DATA;
    return;
}

*array_clip = (double *)calloc(dim, sizeof(double));
dis_from_median = (double *)calloc(dim-1, sizeof(double));
local_array = (double *)calloc(dim, sizeof(double));
if (*array_clip == NULL || dis_from_median == NULL || local_array == NULL) {
   printf("Cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED; 
   return;
}

for(j=0; j<dim; j++)
   local_array[j] = array[j];

if (Const->Asym_Trim != 0.0) {
   reject = (int)(dim*Const->Asym_Trim);
   while (reject > 0) {

      darkest_find_median(local_array, dim, &median, &absdev, Stat);

      index_max = 0;
      index_min = 0;
      for(j=0; j<dim; j++) {
         if (j>0) {
            if ( local_array[j] > array[index_max] )
               index_max = j;
            if ( local_array[j] <= array[index_min] )
               index_min = j;
         }
      }

      start_flag=0;
      end_flag=0;
      if ( fabs(local_array[index_max] - median) > fabs(local_array[index_min] - median) ) {
         dmax = fabs(local_array[index_max] - median);
         clip = local_array[index_max];
         local_array[index_max] = median;
         end_flag = 1;
      }
      else {
         dmax = fabs(local_array[index_min] - median);
         clip = local_array[index_min];
         local_array[index_min] = median;
         start_flag = 1;
      }
      i=0;
      for (j=0; j<dim; j++) {
         if((value = fabs(local_array[j] - median)) != 0.0) {
            dis_from_median[i] = value;     
            i++;
         }
      }

      darkest_find_median(dis_from_median, i, &med_dis, &absdev_dis, Stat);

      if(dmax >= Const->Med_Fac*med_dis) {
         if(start_flag) {
            j1=0;
            for(j=0; j<dim; j++) {
               if (j!=index_min) {
                  local_array[j1] = local_array[j];
                  j1++;
               }
            }
         }
         if(end_flag) {
            j1=0;
            for(j=0; j<dim; j++) {
               if (j!=index_max) {
                  local_array[j1] = local_array[j];
                  j1++;
               }
            }
         }
         dim--;
         if(dim < 1) break;
      }
      else {  /* restore clipped values */
         if(start_flag) local_array[index_min] = clip;
         if(end_flag) local_array[index_max] = clip;
      }
        
      reject--;

   }

}

for(j=0; j<dim; j++)
   *(*array_clip+j) = local_array[j];
  
*dimension = dim;

free(dis_from_median);
free(local_array);

}
