/* $Id: darkest_compute.c,v 1.8 2004/05/13 18:38:32 fan Exp $ */


/*********************************************** 
darkest_compute.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"
/* Added - TYB 6/1/09 */
#include "nanvalue.h"

darkest_compute(	
            Filenames      *Fnames,
            Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
            float          *FP_Vars_In,
            float          **FP_Data_Out,
            float          **FP_Vars_Out,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_cmask_in,
            short          **SP_cmask_out) {
 int	 i, j, k, m, n, l, x, y, fc, fn, ln;

 /* Need to initialize this for Mac - TYB 6/1/09 */
 int     trimfactflag = 0;

 short   pmaskbit, dmaskbit, cmaskbit;
 long    nstart, ndim, adim;
 float   data_temp;
 double  *pixdata;
 double  *trimpixdata=0;
 double  f1, f2;
 double  facttotptr, mean, stddev, datascale;
 double  outdataf, outdatal;

 if(Stat->I_status) return;

 if(Const->Trim_Frac == 0.0)
   printf("darkest_compute: extreme trim on data; calculating median value\n");
 if(Const->Trim_Frac == 1.0)
   printf("darkest_compute: no trim on data; calculating average\n");

 pixdata = (double *) calloc(Const->Num_FITSfile, sizeof(double));
 if (pixdata == NULL) {
   printf("Cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED; 
   return;
 }

 *FP_Data_Out = (float *)calloc(Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ, sizeof(float));

 /*  Try this kludge to recover from the error in darkest_variance_check.c - TYB 6/1/09 */
 /*  Forces this version to give the same results as the original by padding the array. */

 *FP_Vars_Out = (float *)calloc((Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ + Dim_In->I_NX), sizeof(float));
 // *FP_Vars_Out = (float *)calloc(Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ, sizeof(float));


 if (*FP_Data_Out == NULL || *FP_Vars_Out == NULL) {
   printf("Cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED;
   return;
 }

 *SP_cmask_out = (short *) calloc(Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ, sizeof(short));
 if (*SP_cmask_out == NULL) {
   printf("Cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED;
   return;
 }

 nstart = 1;
 f1 = Const->Trim_Frac;
 f2 = Const->Trim_STD_Frac;
 for (i=0; i<Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ; ++i) {

    k=0;
    n=1;
    l=1;
    pmaskbit = *(SP_pmask + i%(Dim_In->I_NX*Dim_In->I_NY));
    cmaskbit = *(SP_cmask_in + i);

    for (fc=0; fc<Const->PmaskNFatal; fc++) {
       if ((pmaskbit & (Const->PmaskFatal[fc])) == 0) n++;
    }
    for (fc=0; fc<Const->CmaskNFatal; fc++) {
       if ((cmaskbit & (Const->CmaskFatal[fc])) == 0) l++;
    }
    for (j=0; j<Const->Num_FITSfile; ++j) {
        m=1;
        dmaskbit = *(SP_dmask + i + j*Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ);
        for (fc=0; fc<Const->DmaskNFatal; fc++) {
           if ((dmaskbit & (Const->DmaskFatal[fc])) == 0) m++;
        }
        if (m != 1 && n != 1 && l != 1) {
           data_temp = *(FP_Data_In+i+j*Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ);
           if (!(iznand( (double) data_temp))) {
              pixdata[k] = data_temp;
              k++;
           }
        }
        else if (m == 1) 
           *(*SP_cmask_out + i) = 2;
    }
    if (n == 1) *(*SP_cmask_out + i) = 4096;
    if (l == 1) *(*SP_cmask_out + i) = 4;
    
    ndim = k;
    adim = k;

    /* obtain trimmed value and uncertainty for each pixel */
    if (adim == 0) {
       *(*SP_cmask_out + i) = 8192;

       /*  Replaced with NANVALUE - TYB 6/1/09 */

       mean = NANVALUE;
       stddev = NANVALUE;
       datascale = NANVALUE;
       //       mean = (float)atof("NaN");
       //       stddev = (float)atof("NaN");
       //       datascale = (float)atof("NaN");

    }
    else if (Const->Asym_Trim == 0.0) {
       trimmedstatistics(pixdata, nstart, ndim, f1, f2, &trimfactflag, &facttotptr,
                     &mean, &stddev, &datascale);
    }
    else {
       if (adim < 4) {
          if (Stat->I_status)
             printf("darkest_compute: number of data sample insufficient; symmetrical trim is performed\n");
          trimmedstatistics(pixdata, nstart, ndim, f1, f2, &trimfactflag, &facttotptr,
                        &mean, &stddev, &datascale);
       }
       else {
          darkest_asym_trim(pixdata, &trimpixdata, &adim, Const, Stat);
          trimmedstatistics(trimpixdata, nstart, adim, f1, f2, &trimfactflag, &facttotptr,
                        &mean, &stddev, &datascale);
       }
    }

    *(*FP_Data_Out+i) = mean;
    *(*FP_Vars_Out+i) = stddev;

 }

 free(pixdata);
 free(trimpixdata);

 /* compute QA parameters if warranted */
 if (Const->QAheader) {
    fn = 0;
    ln = 0;
    outdataf = 0.0;
    outdatal = 0.0;
    for (i=0; i<Dim_In->I_NX*Dim_In->I_NY; ++i) {
       if ( *(*SP_cmask_out + i) == 0 ) {
          outdataf += *(*FP_Data_Out+i); 
          fn++;
       }
       if ( *(*SP_cmask_out + (Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY + i) == 0 ) {
          outdatal += *(*FP_Data_Out+(Dim_In->I_NZ-1)*Dim_In->I_NX*Dim_In->I_NY+i);
          ln++;
       }
    }
    if (fn != 0)
       Const->Dksavgof = outdataf/(double)fn;
    if (ln != 0)
       Const->Dksavgol = outdatal/(double)ln;
 }

 printf("darkest_compute: Numerical Computations finished.\n");

}
