/* $Id: darkest_error_handler.c,v 1.6 2002/09/04 19:58:50 fan Exp $ */


/*********************************************** 
darkest_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of darkest functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_darkest_message = Code of error that occurred.
I_darkest_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"


darkest_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_darkest_message;
int	I_darkest_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_darkest_message = RIGHT_MASK&(Stat->I_status);
I_darkest_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_darkest_message&FITS_MSG_MASK)	{
I_darkest_message -= FITS_MSG_MASK;
}

darkest_decode_message(I_darkest_message, CP_Message);
darkest_decode_fcnname(I_darkest_root, CP_Function);

if (Stat->I_status)
{
printf("darkest Exception Message      0x%04x: %s.\n", I_darkest_message, CP_Message);
printf("darkest Exception in Function  0x%04x: %s.\n", I_darkest_root, CP_Function);
}


I_Time = clock();
printf("Processor Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("darkest_error_handler: Exception Handler finished.\n");

}
