/* $Id: darkest_exec.c,v 1.8 2004/05/13 18:38:32 fan Exp $ */


/*********************************************** 
darkest_exec.c

Calls the initialization functions and main loop of program. Saves
output to a file.

Definitions of Variables in darkest_defs.h
 
***********************************************/

#include <time.h>         /* time() may need this */
#include <sys/types.h>    /* time() may need this */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "fitsio.h"
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"

int main(int argc, char **argv) {			
float	*FP_Data_In=0;
float   *FP_Vars_In=0;
float   *FP_Data_Out=0;
float   *FP_Vars_Out=0;
float   *DP_Data_In=0;
float   *DP_Vars_In=0;
short   *DD_Data_In=0;
short   *SP_pmask=0;
short   *SP_dmask=0;
short   *SP_cmask_in=0;
short   *SP_cmask_out=0;
float   mainvalue;
int     i,j;

Filenames	Fnames;
Constants       Const;
Status		Stat;
Dims            Dim_In;
struct tm       *tmp_time;
time_t          L_time;

/* null filename pointers */

clock();

printf("Pipeline Module DARKEST Version %2.1f\n", VERSN);

/* Calculate and print to standard out time
   Keep tmp_time and module time for time
   time stamping all output
*/
L_time = time(NULL);
tmp_time = localtime(&L_time);
printf("Processing time ");
printf(asctime(tmp_time));
printf("\n");

/* Set constant default values */
darkest_init_const( &Fnames, &Const, &Stat);

/* Parse the namelist file */
darkest_parse_namelist( argc, argv, &Fnames, &Const, &Stat);

/* Parse the command line, replacing values if specified */
darkest_parse_args( argc, argv, &Fnames, &Const, &Stat);

/* Get the data and keyword values from the input file */
darkest_read_data( &Fnames, &Dim_In, &Const, &FP_Data_In, &FP_Vars_In, &DP_Data_In, &DP_Vars_In, &DD_Data_In, &Stat, &SP_pmask, &SP_dmask, &SP_cmask_in);

/* Perform numerical computations on the data */
darkest_compute( &Fnames, &Const, &Dim_In, FP_Data_In, FP_Vars_In, &FP_Data_Out, &FP_Vars_Out, &Stat, SP_pmask, SP_dmask, SP_cmask_in, &SP_cmask_out);

/* Check variance for blocks of pixels, if read noise is specified */
if (Const.Read_Noise != 0.0) 
   darkest_variance_check( &Fnames, &Const, &Dim_In, &FP_Vars_Out, &SP_cmask_out, &Stat);

/* Save the results as fits files. */			
darkest_output( &Fnames, &Dim_In, &Const, &Stat, tmp_time, SP_dmask, SP_cmask_out, FP_Data_Out, FP_Vars_Out);

/* Print filename, error message and timing data */
darkest_error_handler( &Stat, &Fnames);

/* Deallocate memory */			
darkest_exit( FP_Data_In, FP_Vars_In, FP_Data_Out, FP_Vars_Out, DP_Data_In, DD_Data_In, &Dim_In, &Stat, SP_pmask, SP_dmask, SP_cmask_in, SP_cmask_out);

printf("exit code %d\n", Stat.I_status);

/* Send return value */
exit(Stat.I_status);

}
