/* $Id: darkest_find_trim_avg.c,v 1.2 2001/07/13 00:11:20 fan Exp $ */


/***************************************
darkest_find_trim_avg.c

Purpose
find the trim average value for a given list;

****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"

darkest_find_trim_avg(float  *array,
	               int    dim,
                       float  *trim_avg,
		       float  *variance,
                       Constants *Const,
                       Status *Stat) {

int i, j, k, start_index=0, end_index=0, tmp;
int num_samples;
int reject;
int start_flag, end_flag;
float median, absdev, sum, sum_def;
float dmax, *array_clip, *dis_from_median;
float clip, med_dis, absdev_dis;

if (dim == 0) {
    printf("darkest_find_trim_avg:  do not have useful data for trim-average\n");
    Stat->I_status = COMPUTE|NO_USEFUL_DATA;
    return;
}

array_clip = (float *)calloc(dim, sizeof(float));
dis_from_median = (float *)calloc(dim-1, sizeof(float));
if (array_clip == NULL || dis_from_median == NULL) {
   printf("Cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED; 
   return;
}

for(j=0; j<dim; j++)
   array_clip[j] = array[j];

if (Const->Unsym_Trim != 0.0) {
   reject = (int)(dim*Const->Unsym_Trim);
   while (reject > 0) {

      start_flag=0;
      end_flag=0;
      darkest_find_median(array_clip, dim, &median, &absdev, Stat);

      if ( abs(array_clip[dim-1] - median) > abs(array_clip[0] - median) ) {
         dmax = abs(array_clip[dim-1] - median);
         clip = array_clip[dim-1];
         array_clip[dim-1] = median;
         end_flag = 1;
      }
      else {
         dmax = abs(array_clip[0] - median);
         clip = array_clip[0];
         array_clip[0] = median;
         start_flag = 1;
      }

      i=0;
      while(abs(array_clip[i] - median) != 0.0) {
         dis_from_median[i] = abs(array_clip[i] - median);     
         i++;
      }

      darkest_find_median(dis_from_median, i, &med_dis, &absdev_dis, Stat);

      if(dmax >= Const->Med_Fac*med_dis) {
         dim--;
         if(dim < 1) break;
         if(start_flag) {
            for(j=0; j<dim; j++)
               array_clip[j] = array_clip[j+1];
         }
      }
      else {  /* restore clipped values */
         if(start_flag) array_clip[0] = clip;
         if(end_flag) array_clip[dim-1] = clip;
      }
        
      reject--;

   }

}

if (dim < 1) {
    printf("darkest_find_trim_avg:  do not have useful data for trim-average\n");
    Stat->I_status = COMPUTE|NO_USEFUL_DATA;
    return;
}

if(dim%2 != 0) {
   start_index = (int)((dim-1)/2.0) - (int)(dim*Const->Trim_Perc/2.0);
   end_index = (int)((dim-1)/2.0) + (int)(dim*Const->Trim_Perc/2.0);
}
else {
   start_index = (int)((dim-1)/2.0 - dim*Const->Trim_Perc/2.0 + 0.5);
   end_index = (int)((dim-1)/2.0 + dim*Const->Trim_Perc/2.0 - 0.5);
   if (start_index > end_index) {  /* swap */
       tmp = start_index;
       start_index = end_index;
       end_index = tmp;
   }
}

num_samples = end_index - start_index + 1;

sum=0.0;
for (k=start_index; k<=end_index; k++)
   sum += array_clip[k];
*trim_avg = sum/(float)(num_samples);

/* trim-average variance may have an additional factor -- to be revisited */
/* error for median estimate to be modified */
sum_def=0.0;
for (k=start_index; k<=end_index; k++)
   sum_def += (array_clip[k] - *trim_avg)*(array_clip[k] - *trim_avg);
if(Const->Trim_Perc != 0.0)  /* non-median */
  *variance = sqrt(sum_def/(float)(num_samples*(num_samples-1)));
else  /* median */
  *variance = 0.0;

free(array_clip);
free(dis_from_median);

}
