/* $Id: darkest_output.c,v 1.9 2005/01/24 16:44:47 fan Exp $ */


/*********************************************** 
darkest_output.c

Purpose

Save the darkest outputs to files.

Overview
Write the results of the computations as a FITS file.

Definitions of Variables

Input:
I_NX = Number of pixels in a row of the image
I_NY = Number of pixels in a column of the image
I_NZ = Number of layers in the image
Const = structure to provide constant values 
CP_Filename_FITS_In = Input Data File Name. 

Output:
FPP_Data_Out = pointer to hold output data array
FPP_Vars_Out = pointer to hold output error array

Internal:
ffp_FITS = Handle to output FITS file.
ffp_Vars = handle to output variance FITS file.
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_String = String value to be written to header.
CP_Comment = String value to be written in comment field of header.


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"


darkest_output(
            Filenames	  *Fnames,
            Dims          *Dim_In,
	    Constants     *Const,
            Status	  *Stat,
            struct tm     *tmp_time,
            short         *SPP_dmask,
	    short         *SPP_cmask_out,
            float         *FPP_Data_Out,
            float         *FPP_Vars_Out)
{

int	I_fits_return_status=0;
long	I_Num_In;
int     string_length;
int     i;
char    C_Num_In[MAX_FILENAME_LENGTH];

char    CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT];
char    CP_Filename_In[MAX_FILENAME_LENGTH];
char    CP_String[FLEN_VALUE];
char    programname[FITS_HIS_LEN], versionstring[FITS_HIS_LEN];
char    format[10];

int     cmask_naxis = 3;
long    cmask_naxes[3];
float   F_Num_In;
fitsfile 	*ffp_FITS_Out;
fitsfile        *ffp_Vars_Out;
fitsfile 	*ffp_FITS_In;
fitsfile 	*ffp_FITS_Dmask;
fitsfile        *ffp_FITS_Cmask;

printf(asctime(tmp_time));

if(Stat->I_status) return;

cmask_naxes[0] = Dim_In->I_NX;
cmask_naxes[1] = Dim_In->I_NY;
cmask_naxes[2] = Dim_In->I_NZ;

/* make cmask output */

if (strcmp(Fnames->CP_Filename_Cmask_Out, "")) {
   fits_create_file( &ffp_FITS_Cmask,
                     Fnames->CP_Filename_Cmask_Out,
                     &I_fits_return_status);

   if (I_fits_return_status)
   {Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_create_img(ffp_FITS_Cmask,
                   SHORT_IMG,
                   cmask_naxis,
                   cmask_naxes,
                   &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "SIMPLE");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = 1;
   fits_write_key(ffp_FITS_Cmask,
                  TLOGICAL,
                  CP_Keyname,
                  &I_Num_In,
                  CP_Comment,
                  &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "BITPIX  ");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = 8*sizeof(short);
   fits_update_key(ffp_FITS_Cmask,
                   TLONG,
                   CP_Keyname,
                   &I_Num_In,
                   CP_Comment,
                   &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "NAXIS");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = 3;
   fits_update_key(ffp_FITS_Cmask,
                   TLONG,
                   CP_Keyname,
                   &I_Num_In,
                   CP_Comment,
                   &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "NAXIS1");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = Dim_In->I_NX;
   fits_update_key(ffp_FITS_Cmask,
                   TLONG,
                   CP_Keyname,
                   &I_Num_In,
                   CP_Comment,
                   &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "NAXIS2");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = Dim_In->I_NY;
   fits_update_key(ffp_FITS_Cmask,
                   TLONG,
                   CP_Keyname,
                   &I_Num_In,
                   CP_Comment,
                   &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "NAXIS3");
   sprintf(CP_Comment, "%s", "");
   I_Num_In = Dim_In->I_NZ;
   fits_update_key(ffp_FITS_Cmask,
                   TLONG,
                   CP_Keyname,
                   &I_Num_In,
                   CP_Comment,
                   &I_fits_return_status);
   fits_flush_file(ffp_FITS_Cmask, &I_fits_return_status);

   sprintf(CP_Comment, "%s", "This is a cmask image");
   fits_write_comment(ffp_FITS_Cmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "Input image list file %s", Fnames->CP_Filename_FITSlist_In);
   fits_write_comment(ffp_FITS_Cmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "DARKEST");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_FITS_Cmask, programname, versionstring, &I_fits_return_status);

   fits_write_img( ffp_FITS_Cmask,
                   TSHORT,
                   1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                   SPP_cmask_out,
                   &I_fits_return_status);

   fits_flush_file(ffp_FITS_Cmask,
                   &I_fits_return_status);

   fits_close_file(ffp_FITS_Cmask,
                   &I_fits_return_status);

   if (I_fits_return_status)
   {Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

}

/* write out the image data */
if (Stat->I_Verbose)
printf("darkest_output: Output FITS header based on input file %s\n", Fnames->CP_Filename_FITS_Image[Const->Goodfile_Index]);

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_Image[Const->Goodfile_Index],      
                READWRITE,
                &I_fits_return_status);
fits_create_file( &ffp_FITS_Out,
                Fnames->CP_Filename_FITS_Out,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* copy the FITS header of the first file in the list to output files */
fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out,
                 &I_fits_return_status);

/* Add module specific keywords to data cube */
sprintf(CP_Comment, "%s", "Begin DARKEST module additions");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "Optimal reference dark current estimated");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "Input image list file %s", Fnames->CP_Filename_FITSlist_In);
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

if(Const->QAheader) {
   sprintf(CP_Keyname, "%s", "DKSAVGOF");
   sprintf(CP_Comment, "%s", "");
   F_Num_In = Const->Dksavgof;
   fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "DKSAVGOL");
   sprintf(CP_Comment, "%s", "");
   F_Num_In = Const->Dksavgol;
   fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

   sprintf(CP_Keyname, "%s", "DKNOSYBK");
   sprintf(CP_Comment, "%s", "total number of 2x2 noisy blocks");
   I_Num_In = Const->Noisy_Blocks;
   fits_write_key(ffp_FITS_Out, TLONG, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out, &I_fits_return_status);
}

sprintf(programname, "%s", "DARKEST");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_Out,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out,
                &I_fits_return_status);

/* write out the uncertainty data */
fits_create_file( &ffp_Vars_Out,
                Fnames->CP_Filename_Vars_Out,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* copy the FITS header of the first file in the list to output files */
fits_copy_header(ffp_FITS_In,
                 ffp_Vars_Out,
                 &I_fits_return_status);

/* Add module specific keywords to variance cube */
sprintf(CP_Comment, "%s", "Begin DARKEST module additions");
fits_write_comment(ffp_Vars_Out,
                   CP_Comment,
                   &I_fits_return_status);
sprintf(CP_Comment, "%s", "This is the Uncertainty Image");
fits_write_comment(ffp_Vars_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "Optimal reference dark current estimated");
fits_write_comment(ffp_Vars_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "Input image list file %s", Fnames->CP_Filename_FITSlist_In);
fits_write_comment(ffp_Vars_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DKNOSYBK");
sprintf(CP_Comment, "%s", "total number of 2x2 noisy blocks");
I_Num_In = Const->Noisy_Blocks;
fits_write_key(ffp_Vars_Out, TLONG, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_Vars_Out, &I_fits_return_status);

sprintf(programname, "%s", "DARKEST");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_Vars_Out, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_Vars_Out,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Vars_Out,
                &I_fits_return_status);

fits_flush_file(ffp_Vars_Out,
                &I_fits_return_status);

fits_close_file(ffp_Vars_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_In,
                &I_fits_return_status);

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

if(Stat->I_Verbose)
printf("darkest_output: Results written to file(s).\n");

}
