/* $Id: darkest_parse_args.c,v 1.7 2002/09/04 19:58:51 fan Exp $ */


/*********************************************** 
darkest_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITSlist_In = In List File Name Holding Data Files.
CP_Filename_FITS_Out  = Output File Name.
CP_Filename_Pmask = Input Pmask.
CP_Filename_Dmasklist = Input List File Name holding Dmask Files.
CP_Filename_Cmask = Input Cmask.
I_Verbose = Toggle whether to output verbose info.

Output:
Const = Structure to Hold Constant Parameters.

Internal:


***********************************************/

#include <stdio.h>
#include <string.h>
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"


darkest_parse_args(	
            int	        argc,
            char        **argv,
            Filenames   *Fnames,
            Constants   *Const,
            Status	*Stat) {

   int	i=1;

   int     I_out_chk, I_vars_chk, I_in_chk;
   int     I_nl_chk;
   int     I_pair_err;

   int j;
   char *p;
   char layer_num_str[100];
   char file_num_str[100];
   char fatal_bit_str[100];

   /* flags to check the completeness of the switch-value pairs */
   I_in_chk = 1;
   I_out_chk = 1;
   I_vars_chk = 1;
   I_nl_chk = 1;
   I_pair_err = 0;

   while( i < argc ) {

     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               I_nl_chk =0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'i':      /* -i <input_image_list_fnames>  */
            if (argv[i][2] == '1') {     /* image list file */
               if ( ++i >= argc )
                  I_pair_err = 1;
	       else {
                  I_in_chk = 0;
                  sscanf(argv[i], "%s", Fnames->CP_Filename_FITSlist_In);
               }
            }
            else {       /* -i2 <input uncertainty list filenames> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Varslist_In);
               }
            }
            break;
          case 'o':      /* -o <output_image_fnames>  */
            if (argv[i][2] == '1') {       /* image file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  I_out_chk = 0;
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
               }
            }
            else if (argv[i][2] == '2') {  /* uncertainty file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  I_vars_chk = 0;
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_Vars_Out[1]));
               }
            }
            else {                         /* cmask file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Cmask_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_Cmask_Out[1]));
               }
            }
            break;
          case 't':      /* -t <fraction for symmetrical trim-average>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i],"%f", &Const->Trim_Frac);
               }
            }
            else {       /* -t2 <fraction for trim-average standard deviation calculation> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i],"%f", &Const->Trim_STD_Frac);
               }
            }
            break;
          case 'u':      /* -u <unsymmetrical trim fraction> */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i],"%f", &Const->Asym_Trim);
            }
            break;
          case 'e':      /* -e <factor of median distance for unsymmetrical trim> */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i],"%f", &Const->Med_Fac);
            }
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p') {          /* for Pmask */
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else {
                  sscanf(argv[i],"%s", fatal_bit_str);
                  p = strtok(fatal_bit_str, ",");
                  Const->PmaskFatal[0] = atoi(p);
                  j=1;
                  do {
                     p=strtok('\0', ",");
                     if (p) {
                        Const->PmaskFatal[j]=atoi(p);
                        j++;
                     }
                  } while(p);
                  Const->PmaskNFatal = j;
               }
            }
            else if (argv[i][2] == 'd') {     /* for Dmask */
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else {
                  sscanf(argv[i],"%s", fatal_bit_str);
                  p = strtok(fatal_bit_str, ",");
                  Const->DmaskFatal[0] = atoi(p);
                  j=1;
                  do {
                     p=strtok('\0', ",");
                     if (p) {
                        Const->DmaskFatal[j]=atoi(p);
                        j++;
                     }
                  } while(p);
                  Const->DmaskNFatal = j;
               }
            }
            else if (argv[i][2] == 'c') {     /* for Cmask */
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else {
                  sscanf(argv[i],"%s", fatal_bit_str);
                  p = strtok(fatal_bit_str, ",");
                  Const->CmaskFatal[0] = atoi(p);
                  j=1;
                  do {
                     p=strtok('\0', ",");
                     if (p) {
                        Const->CmaskFatal[j]=atoi(p);
                        j++;
                     }
                  } while(p);
                  Const->CmaskNFatal = j;
               }
            }
            break;
          case 'r':      /* -r <read noise per pixel>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i],"%f", &Const->Read_Noise);
            }
            break;
          case 'a':      /* -a <abnormal input file and layer numbers */
            if (argv[i][2] == 'c') {     /* abnormal file number */
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else {
                  sscanf(argv[i],"%s", file_num_str);
                  p = strtok(file_num_str, ",");
                  Const->Cube_Abn[0] = atoi(p);
                  j=1;
                  do {
                     p=strtok('\0', ",");
                     if (p) {
                        Const->Cube_Abn[j]=atoi(p);
                        j++;
                     }
                  } while(p);
                  Const->Num_Cube_Abn = j;
               }
            }
            else if (argv[i][2] == 'p') {     /* abnormal plane number in cube */
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else {
                  sscanf(argv[i],"%s", layer_num_str);
                  p = strtok(layer_num_str, ",");
                  Const->Layer_Abn[0] = atoi(p);
                  j=1;
                  do {
                     p=strtok('\0', ",");
                     if (p) {
                        Const->Layer_Abn[j]=atoi(p);
                        j++;
                     }
                  } while(p);
                  Const->Num_Layer_Abn = j;
               }
            }
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            }
            break;
          case 'd':      /* -d <dmask_list_fname>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmasklist);
            }
            break;
          case 'c':      /* -c <cmask pixel status file name>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask_In);
            }
            break;
          case 'q':      /* -q  QA header writer mode */
            Const->QAheader = 1;
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;
            break;
          default:
            printf("darkest_parse_args: unknown argument\n");
       }
     }
     else {
       I_pair_err = 1;
     }
     i++;
   }

   if ( I_pair_err ) {
      printf("darkest_parse_args: command line syntax error\n");
      darkest_usage();
      Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   }

   if ( (I_in_chk || I_out_chk || I_vars_chk) && I_nl_chk ) {
      printf("Error in command line arguments; required input not complete.\n\n");
      darkest_usage();
      Stat->I_status = PARSE_ARGS|REQ_ARGS_INCOMP;
   }

   if (Const->Trim_Frac > 1.0 || Const->Trim_Frac < 0.0 || Const->Asym_Trim > 1.0 || Const->Asym_Trim < 0.0) {
      printf("Error in trim fraction specfication.\n\n");
      darkest_usage();
      Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   }

   if(Stat->I_Verbose)	{
      printf("darkest_parse_args: Input Namelist File = \t%s\n", Fnames->CP_Filename_Namelist);
      printf("darkest_parse_args: Input Image List File = \t%s\n", Fnames->CP_Filename_FITSlist_In);
      printf("darkest_parse_args: Input Uncertainty Image List File = \t%s\n", Fnames->CP_Filename_Varslist_In);
      printf("darkest_parse_args: Ouput Image File = \t%s\n", Fnames->CP_Filename_FITS_Out);
      printf("darkest_parse_args: Ouput Uncertainty Image file = \t%s\n", Fnames->CP_Filename_Vars_Out);
      printf("darkest_parse_args: Ouput Cmask Images File = \t%s\n", Fnames->CP_Filename_Cmask_Out);
      printf("darkest_parse_args: Fraction for Symetrical Trim_average = \t%f\n", Const->Trim_Frac);
      printf("darkest_parse_args: Fraction for Trim Standard Deviation Calculation = \t%f\n", Const->Trim_STD_Frac);
      printf("darkest_parse_args: Fraction for Asymmetrical Trim = \t%f\n", Const->Asym_Trim);
      printf("darkest_parse_args: Factor of Median Distance for Asymmetrical Trim = \t%f\n", Const->Med_Fac);
      printf("darkest_parse_args: Read Noise = \t%f\n", Const->Read_Noise);
      printf("darkest_parse_args: QA Header Mode = \t%d\n", Const->QAheader);
      printf("darkest_parse_args: Pmask File = \t%s\n", Fnames->CP_Filename_Pmask);
      printf("darkest_parse_args: Dmask List File = \t%s\n", Fnames->CP_Filename_Dmasklist);
      printf("darkest_parse_args: Cmask File = \t%s\n", Fnames->CP_Filename_Cmask_In);
      for (i=0; i<Const->PmaskNFatal; i++)
         printf("darkest_parse_args: Pmask Fatal = \t%d\n", Const->PmaskFatal[i]);
      for (i=0; i<Const->DmaskNFatal; i++)
         printf("darkest_parse_args: Dmask Fatal = \t%d\n", Const->DmaskFatal[i]);
      for (i=0; i<Const->CmaskNFatal; i++)
         printf("darkest_parse_args: Cmask Fatal = \t%d\n", Const->CmaskFatal[i]);
      printf("darkest_parse_args: Total Number of Fatal Pmask bits = \t%d\n", Const->PmaskNFatal);
      printf("darkest_parse_args: Total Number of Fatal Dmask bits = \t%d\n", Const->DmaskNFatal);
      printf("darkest_parse_args: Total Number of Fatal Cmask bits = \t%d\n", Const->CmaskNFatal);
      printf("darkest_parse_args: Total %d of the Input Image Files Have %d Fewer Planes\n", Const->Num_Cube_Abn, Const->Num_Layer_Abn);
      printf("darkest_parse_args: I_Verbose = \t\t\t%d\n\n", Stat->I_Verbose);
   }
}
