/* $Id: darkest_read_dmask.c,v 1.8 2003/02/18 19:41:21 fan Exp $ */


/*********************************************** 
darkest_read_dmask.c

Purpose

Read the dmask data from a FITS file and save into an array.

Overview

Open the FITS file, allocate memory, read the data into an array, 

Definitions of Variables

Input:

Output:
 
Internal:

***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"


darkest_read_dmask(	
            char        *fits_filename,
            Dims        *Dim_In,
            Constants   *Const,
            short	**DPP_Data_In,
            Status	*Stat) {
int	I_fits_return_status=0, I_ndims_found, I_anynull;
int     I_naxis, I_bitpix;
int     i, j, k;
long    LP_naxes[3];
long    LP_naxes1[3];
short   **DDP_Temp;
float	D_nullval=0;
fitsfile 	*ffp_FITS_In;
char    fits_comment[FLEN_CARD];

if(Stat->I_status) return;

/* find dimensions of data */
fits_open_file( &ffp_FITS_In,
                fits_filename,
                READWRITE,
                &I_fits_return_status);

fits_read_key( ffp_FITS_In,
               TINT,
               "BITPIX",
               &I_bitpix,
               fits_comment,
               &I_fits_return_status);
if(I_bitpix != 16)
{Stat->I_status = READ_DATA|BIT_ERROR; return;}

fits_read_key( ffp_FITS_In,
               TINT,
               "NAXIS",
               &I_naxis,
               fits_comment,
               &I_fits_return_status);

fits_read_keys_lng( ffp_FITS_In,
                    "NAXIS",
                    1, I_naxis,
                    LP_naxes,
                    &I_ndims_found,
                    &I_fits_return_status);

if (I_naxis == 2) LP_naxes[2] = 1;
/* Read the data */
/* Don't try if problem with fits file */

if (LP_naxes[2] = Dim_In->I_NZ) {

   if(I_fits_return_status)
   {Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_read_img(ffp_FITS_In,
                 TSHORT,
                 1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                 &D_nullval,
                 *DPP_Data_In,
                 &I_anynull,
                 &I_fits_return_status);

   fits_close_file(ffp_FITS_In,
     		 &I_fits_return_status);

}
else {    /* dmask with missing layers; assumed to correspond with FITS data missing layers */

   if (Const->Num_Layer_Abn == 0) {
      printf("darkest_read_dmask: abnormal missing planes detected in Dmask\n");
      printf("darkest_read_dmask: Dmask may not be reliable from this point on\n");
   }
   /* Allocate memory for temporary array */

   *DDP_Temp = (short *) calloc(LP_naxes[0]*LP_naxes[1]*LP_naxes[2], sizeof(short));
   if(*DDP_Temp==NULL) I_fits_return_status = MALLOC_FAILED;

   /* Read the data */
   /* Don't try if problem with fits file */

   if(I_fits_return_status)
   {Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_read_img(ffp_FITS_In,
                 TFLOAT,
                 1, LP_naxes[0]*LP_naxes[1]*LP_naxes[2],
                 &D_nullval,
                 *DDP_Temp,
                 &I_anynull,
                 &I_fits_return_status);
   /* set bit 14 in Dmask for missing layers */
   k=0;
   for (i=0; i<Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ; ++i) {
      for (j=0; j<Const->Num_Layer_Abn; ++j) {
         if (Const->Layer_Abn[j] != i%Dim_In->I_NX*Dim_In->I_NY+1) {
            *(*DPP_Data_In+i) = *(*DDP_Temp+k);
            k++;
         }
         else
            *(*DPP_Data_In+i) = 16384;
      }
   }

   free(*DDP_Temp);
   fits_close_file(ffp_FITS_In,
                 &I_fits_return_status);

}

if(I_fits_return_status)
{Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

}
