/* $Id: darkest_read_image.c,v 1.7 2003/02/18 19:41:22 fan Exp $ */


/*********************************************** 
darkest_read_image.c

Purpose

Read the image data from a FITS file and save into an array.

Overview

Open the FITS file, allocate memory, read the data into an array, 

Definitions of Variables

Input:

Output:
 
Internal:

***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "darkest.h"
#include "darkest_errcodes.h"
#include "darkest_defs.h"
/* Added - TYB 6/1/09 */
#include "nanvalue.h"

darkest_read_image(	
            char        *fits_filename,
            int         readflag,
            Dims        *Dim_In,
            Constants   *Const,
            float	**DPP_Data_In,
            Status	*Stat) {
char    I_bunit[FLEN_VALUE];
int	I_fits_return_status=0, I_ndims_found, I_anynull;
int     I_naxis, I_bitpix;
int     i, j, k;
long	LP_naxes[3];
long	LP_naxes1[3];
float	D_nullval=0;
float   **DPP_Temp;
fitsfile 	*ffp_FITS_In;
char    fits_comment[FLEN_CARD];

if(Stat->I_status) return;

/* find dimensions of data based on first DCE */

fits_open_file( &ffp_FITS_In,
                fits_filename,
                READWRITE,
                &I_fits_return_status);

fits_read_key( ffp_FITS_In,
               TINT,
               "BITPIX",
               &I_bitpix,
               fits_comment,
               &I_fits_return_status);
if(I_bitpix != -32)
{Stat->I_status = READ_DATA|BIT_ERROR; return;}

fits_read_key( ffp_FITS_In,
               TINT,
               "NAXIS",
               &I_naxis,
               fits_comment,
               &I_fits_return_status);

fits_read_keys_lng( ffp_FITS_In,
                    "NAXIS",
                    1, I_naxis,
                    LP_naxes,
                    &I_ndims_found,
                    &I_fits_return_status);

if (I_naxis == 2) LP_naxes[2] = 1;

fits_read_key( ffp_FITS_In,
               TSTRING,
               "BUNIT",
               I_bunit,
               fits_comment,
               &I_fits_return_status);

if (readflag == 1) {
   Dim_In->I_NX = LP_naxes[0];
   Dim_In->I_NY = LP_naxes[1];
   Dim_In->I_NZ = LP_naxes[2];
   if ( !(strcmp(I_bunit, "")) ) {
      printf("cubesub_read_data: value of BUNIT key missing in input reference data file\n");
      Stat->I_status = READ_DATA|MISSING_KEY;
      return;
   }
   strcpy(Const->Bunit, I_bunit);
   fits_close_file(ffp_FITS_In,
                   &I_fits_return_status);
   return;
}
else if (readflag == 0) {
   if (LP_naxes[0] != Dim_In->I_NX || LP_naxes[1] != Dim_In->I_NY || LP_naxes[2] != Dim_In->I_NZ) {
      printf("darkest_read_data: FITS cube dimension not consistent\n");
      Stat->I_status = READ_DATA|DIM_INCONSIS;
      return;
   }

   if (strcmp(I_bunit, Const->Bunit)) {
      printf("cubesub_read_data: value of BUNIT key missing or inconsistent in at least one input data file\n");
      printf("cubesub_read_data: using BUNIT key value in input reference data file\n");
   }

   /* Read the data */
   /* Don't try if problem with fits file */

   if(I_fits_return_status)
   {Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_read_img(ffp_FITS_In,
                 TFLOAT,
                 1, LP_naxes[0]*LP_naxes[1]*LP_naxes[2],
                 &D_nullval,
                 *DPP_Data_In,
                 &I_anynull,
                 &I_fits_return_status);

   fits_close_file(ffp_FITS_In,
	     	 &I_fits_return_status);

}
else {    /* cube with missing layers */
   /* check BUNIT key */
   if (strcmp(I_bunit, Const->Bunit)) {
      printf("cubesub_read_data: value of BUNIT key missing inconsistent in at least one input data file with missing layers\n");
      printf("cubesub_read_data: using BUNIT key value in input reference data file\n");
   }

   /* Allocate memory for temporary array */

   if(abs(Dim_In->I_NZ-LP_naxes[2]) != Const->Num_Layer_Abn) {
      printf("\ndarkest_read_image: input abnormal plane number inconsistent with data\n\n");
      Stat->I_status = READ_DATA|ABNP_INCONSIS;
      return;
   }

   *DPP_Temp = (float *) calloc(LP_naxes[0]*LP_naxes[1]*LP_naxes[2], sizeof(float));
   if(*DPP_Temp==NULL) I_fits_return_status = MALLOC_FAILED;

   /* Read the data */
   /* Don't try if problem with fits file */

   if(I_fits_return_status)
   {Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_read_img(ffp_FITS_In,
                 TFLOAT,
                 1, LP_naxes[0]*LP_naxes[1]*LP_naxes[2],
                 &D_nullval,
                 *DPP_Temp,
                 &I_anynull,
                 &I_fits_return_status);
   /* insert NaNs for missing layers */
   k=0;
   for (i=0; i<Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ; ++i) {
      j=0;
      while ((int)((float)i/(float)(Dim_In->I_NX*Dim_In->I_NY))+1 != Const->Layer_Abn[j] && j<Const->Num_Layer_Abn) j++;
      if (j==Const->Num_Layer_Abn) {
         *(*DPP_Data_In+i) = *(*DPP_Temp+k);
         k++;
      }
      else
	/*  Switch to NANVALUE - TYB 6/1/09 */

         *(*DPP_Data_In+i) = NANVALUE;
      //         *(*DPP_Data_In+i) = (float)atof("NaN");

   }

   free(*DPP_Temp);
   fits_close_file(ffp_FITS_In,
                 &I_fits_return_status);

}

if(I_fits_return_status)
{Stat->I_status = READ_DATA|FITS_MSG_MASK|I_fits_return_status; return;}

}
