/* $Id: dntoflux_compute_results.c,v 1.9 2003/12/30 17:45:01 laher Exp $ */


/*********************************************** 
dntoflux_compute_results.c

Purpose

Main computational processing loop of program.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"

#include "dntoflux.h"
#include "dntoflux_defs.h"

/* Includes for mathc90 sort routine. */
#include "mathc90.h"
#include <stdlib.h>
#include <math.h>
#include "fcrt.h"
#include "uprop.h"

/* #include "ieeefp.h" */

dntoflux_compute_results(DNT_Constants		*DNTP_Const,
		        DNT_Filenames		*DNTP_Fnames,
			DNT_FITSinfo		*DNTP_FITS,
			double			*DP_Data_Image1,
			double			*DP_Data_Image2,
			double			*DP_Data_Image3,
			double			*DP_Data_Image4,
			DNT_Computation		*DNTP_Comp,
			DNT_Status		*DNTP_Stat)
{
int	i, j, k, ii, jj, kk, kkk, iindex, jindex;
int	I_Length_X, I_Length_Y, I_Num_Frames, I_Operation;
int	I_Tot_Pixels, I_Index, I_Tot_PixelsOut;
int     nbadpixels;
int     kstart, kend, I_Data_Plane;
int     I_Index_Inp;
int     smask = 0, omask = 0;
double  outval, osigma;
double  *Input_Array;
double	*DP_Output_Array, *DP_Output_Array2, *DP_Output_Array4;
double  nanvalue, inverseExpTime;
unsigned long mid, *NumberBadPixels;
uprop_state status;


/* For debug only */
FILE	*fp_data;

if (DNTP_Stat->I_status) return;


/* Copy structure info to local variables. */

I_Length_X = DNTP_FITS->I_Length_X;
I_Length_Y = DNTP_FITS->I_Length_Y;
I_Num_Frames = DNTP_FITS->I_Num_Frames;
I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;
I_Data_Plane = DNTP_FITS->I_Data_Plane;


/* Create local array. */

Input_Array = (double *) calloc(I_Tot_Pixels, sizeof(double));
if(Input_Array==NULL)
{DNTP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; return;}


/* Set controls for number image-data planes to treat. */

if (I_Data_Plane == 3) 
{
   kstart = I_Num_Frames - 1;
   kend = I_Num_Frames;
   I_Num_Frames = 1;
   DNTP_FITS->I_Num_Frames = I_Num_Frames;
   I_Tot_PixelsOut = I_Length_X*I_Length_Y*I_Num_Frames;
}
else if (I_Data_Plane == 2)
{
   kstart = 0;
   kend = 1;
   I_Num_Frames = 1;
   DNTP_FITS->I_Num_Frames = I_Num_Frames;
}
else
{
   kstart = 0;
   kend = I_Num_Frames;
} 

/* Do the division only once, since EXPTIME is a constant. */

inverseExpTime = 1. / DNTP_FITS->ExpTime;


/* Create the output arrays for writing to file. */

I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;

DNTP_Comp->DP_Output_Array = (double *) calloc(I_Tot_Pixels, 
   sizeof(double));
if (DNTP_Comp->DP_Output_Array==NULL) {
   DNTP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}

if ((strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) ||
    (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0)) {
   DNTP_Comp->DP_Output_Array2 = (double *) calloc(I_Tot_Pixels, 
      sizeof(double));
   if(DNTP_Comp->DP_Output_Array2==NULL) {
      DNTP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
      goto end_compute_results;
   }
}

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {
   DNTP_Comp->DP_Output_Array4 = (double *) calloc(I_Tot_Pixels, 
      sizeof(double));
   if(DNTP_Comp->DP_Output_Array4==NULL) {
      DNTP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
      goto end_compute_results;
   }
}

DNTP_Comp->NumberBadPixels = 
   (unsigned long *) calloc(I_Num_Frames, sizeof(unsigned long));
if(DNTP_Comp->NumberBadPixels==NULL) {
   DNTP_Stat->I_status = COMPUTE_RES|MALLOC_FAILED; 
   goto end_compute_results;
}


/* Copy structure info to local variables */

DP_Output_Array = DNTP_Comp->DP_Output_Array;
if ((strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) ||
    (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0)) {
   DP_Output_Array2 = DNTP_Comp->DP_Output_Array2;
}
if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {
   DP_Output_Array4 = DNTP_Comp->DP_Output_Array4;
}

NumberBadPixels = DNTP_Comp->NumberBadPixels;


/* Perform DN-to-flux conversion on input image. */

if (DNTP_Stat->I_Verbose) {
   printf("dntoflux_compute_results: Performing DN to flux conversion...\n");
}
if (DNTP_Stat->I_SuperVerbose) {
   printf("dntoflux_compute_results: Operation = %d\n", 
      DNTP_Const->I_Operation);
   printf("dntoflux_compute_results: Normalize-by_EXPTIME flag = %d\n", 
      DNTP_Const->I_NormalizeByExpTimeFlag);
}


/****************************************************************************
THIS SECTION IS APPLIED ONLY IF NO INPUT UNCERTAINTY IMAGE IS GIVEN.
*****************************************************************************/

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") == 0) {


if (DNTP_Const->I_NormalizeByExpTimeFlag == 1) {


   /*  CASE 1: Do optional EXPTIME normalization. */

   if (DNTP_Stat->I_Verbose)
      printf("dntoflux_compute_results: EXPTIME = %f\n", 
         DNTP_FITS->ExpTime);

   if (DNTP_Const->I_Operation == 0) {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0;i<I_Length_X;i++)
         for (j=0;j<I_Length_Y;j++) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor * inverseExpTime; 

	   //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	   //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   } else {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0; i<I_Length_X; i++)
         for (j=0; j<I_Length_Y; j=j+4) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor1 * inverseExpTime; 

	   /* Readout channel 1 */

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor1 * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 2 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor2 * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor2 * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 3 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor3 * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor3 * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 4 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor4 * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor4 * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;

            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   }

} else {


   /*  CASE 2: Don't do optional EXPTIME normalization. */

   if (DNTP_Const->I_Operation == 0) {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0;i<I_Length_X;i++)
         for (j=0;j<I_Length_Y;j++) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   } else {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0; i<I_Length_X; i++)
         for (j=0; j<I_Length_Y; j=j+4) {

	   /* Readout channel 1 */

	   /*  This is the case IRS CUPID is using - TYB 5/1/09 */
	   /*  Comment out NaN checking. */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor1; 

	       //	    if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor1; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;

               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //	       dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 2 */

	   /*  This is the case IRS CUPID is using - TYB 5/1/09 */
	   /*  Comment out NaN checking. */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor2; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor2; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 3 */

	   /*  This is the case IRS CUPID is using - TYB 5/1/09 */
	   /*  Comment out NaN checking. */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor3; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor3; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 4 */

	   /*  This is the case IRS CUPID is using - TYB 5/1/09 */
	   /*  Comment out NaN checking. */

               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
                  DNTP_Const->D_ConversionFactor4; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //               DP_Output_Array[I_Index] = DP_Data_Image1[I_Index_Inp] * 
	       //                  DNTP_Const->D_ConversionFactor4; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;

            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   }

}


/****************************************************************************
THIS SECTION IS APPLIED ONLY IF AN INPUT UNCERTAINTY IMAGE IS GIVEN.
*****************************************************************************/

} else {


if (DNTP_Const->I_NormalizeByExpTimeFlag == 1) {


   /*  CASE 1: Do optional EXPTIME normalization. */

   if (DNTP_Stat->I_Verbose)
      printf("dntoflux_compute_results: EXPTIME = %f\n", 
         DNTP_FITS->ExpTime);

   if (DNTP_Const->I_Operation == 0) {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0;i<I_Length_X;i++)
         for (j=0;j<I_Length_Y;j++) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor,
                          DNTP_Const->D_ConversionFactorUnc, smask);

               DP_Output_Array[I_Index] = outval * inverseExpTime; 
               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	      /*
	      printf("Calling uprop_dmul: %f %f %d %f %f %d\n",
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor,
		          DNTP_Const->D_ConversionFactorUnc, smask);
	      */

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor,
	       //                          DNTP_Const->D_ConversionFactorUnc, smask);

	       /*
	       printf("Return from uprop_dmul: %f %f %d\n",
		      outval, osigma, omask);
	       */
                          
	       //               DP_Output_Array[I_Index] = outval * inverseExpTime; 
	       //               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   } else {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0; i<I_Length_X; i++)
         for (j=0; j<I_Length_Y; j=j+4) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

	   /* Readout channel 1 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor1,
                          DNTP_Const->D_ConversionFactorUnc1, smask);
                          
               DP_Output_Array[I_Index] = outval * inverseExpTime; 
               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor1,
	       //                          DNTP_Const->D_ConversionFactorUnc1, smask);
                          
	       //               DP_Output_Array[I_Index] = outval * inverseExpTime; 
	       //               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 2 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor2,
                          DNTP_Const->D_ConversionFactorUnc2, smask);
                          
               DP_Output_Array[I_Index] = outval * inverseExpTime; 
               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor2,
	       //                          DNTP_Const->D_ConversionFactorUnc2, smask);
                          
	       //               DP_Output_Array[I_Index] = outval * inverseExpTime; 
	       //               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 3 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor3,
                          DNTP_Const->D_ConversionFactorUnc3, smask);
                          
               DP_Output_Array[I_Index] = outval * inverseExpTime; 
               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor3,
	       //                          DNTP_Const->D_ConversionFactorUnc3, smask);
                          
	       //               DP_Output_Array[I_Index] = outval * inverseExpTime; 
	       //               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 4 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor4,
                          DNTP_Const->D_ConversionFactorUnc4, smask);
                          
               DP_Output_Array[I_Index] = outval * inverseExpTime; 
               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor4,
	       //                          DNTP_Const->D_ConversionFactorUnc4, smask);
                          
	       //               DP_Output_Array[I_Index] = outval * inverseExpTime; 
	       //               DP_Output_Array4[I_Index] = osigma * inverseExpTime; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;

            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   }

} else {


   /*  CASE 2: Don't do optional EXPTIME normalization. */

   if (DNTP_Const->I_Operation == 0) {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0;i<I_Length_X;i++)
         for (j=0;j<I_Length_Y;j++) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor,
                          DNTP_Const->D_ConversionFactorUnc, smask);
                          
               DP_Output_Array[I_Index] = outval; 
               DP_Output_Array4[I_Index] = osigma; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor,
	       //                          DNTP_Const->D_ConversionFactorUnc, smask);
                          
	       //               DP_Output_Array[I_Index] = outval; 
	       //               DP_Output_Array4[I_Index] = osigma; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   } else {

      I_Index = 0;
      nbadpixels = 0;

      for (k = kstart; k < kend; k++) {
         I_Index_Inp = k*I_Length_X*I_Length_Y;
         for (i=0; i<I_Length_X; i++)
         for (j=0; j<I_Length_Y; j=j+4) {

	   /*  Comment out NaN bit writing/checking - TYB 6/1/09 */

	   /* Readout channel 1 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor1,
                          DNTP_Const->D_ConversionFactorUnc1, smask);
                          
               DP_Output_Array[I_Index] = outval; 
               DP_Output_Array4[I_Index] = osigma; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor1,
	       //                          DNTP_Const->D_ConversionFactorUnc1, smask);
                          
	       //               DP_Output_Array[I_Index] = outval; 
	       //               DP_Output_Array4[I_Index] = osigma; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 2 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor2,
                          DNTP_Const->D_ConversionFactorUnc2, smask);
                          
               DP_Output_Array[I_Index] = outval; 
               DP_Output_Array4[I_Index] = osigma; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor2,
	       //                          DNTP_Const->D_ConversionFactorUnc2, smask);
                          
	       //               DP_Output_Array[I_Index] = outval; 
	       //               DP_Output_Array4[I_Index] = osigma; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 3 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor3,
                          DNTP_Const->D_ConversionFactorUnc3, smask);
                          
               DP_Output_Array[I_Index] = outval; 
               DP_Output_Array4[I_Index] = osigma; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor3,
	       //                          DNTP_Const->D_ConversionFactorUnc3, smask);
                          
	       //               DP_Output_Array[I_Index] = outval; 
	       //               DP_Output_Array4[I_Index] = osigma; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;


	   /* Readout channel 4 */

	       uprop_dmul(&outval, &osigma, &omask, 
                          DP_Data_Image1[I_Index_Inp],
                          DP_Data_Image4[I_Index_Inp], smask,
                          DNTP_Const->D_ConversionFactor4,
                          DNTP_Const->D_ConversionFactorUnc4, smask);
                          
               DP_Output_Array[I_Index] = outval; 
               DP_Output_Array4[I_Index] = osigma; 

	       //            if (! (DP_Data_Image1[I_Index_Inp] != 0 && 
	       //	        dnan(*(short *) &DP_Data_Image1[I_Index_Inp]))) {

	       //	       uprop_dmul(&outval, &osigma, &omask, 
	       //                          DP_Data_Image1[I_Index_Inp],
	       //                          DP_Data_Image4[I_Index_Inp], smask,
	       //                          DNTP_Const->D_ConversionFactor4,
	       //                          DNTP_Const->D_ConversionFactorUnc4, smask);
                          
	       //               DP_Output_Array[I_Index] = outval; 
	       //               DP_Output_Array4[I_Index] = osigma; 

	       //            } else {

	       //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;


               /* Write the error code into the array as an NaN 
                  instead of the result.
               */
	       //               dntoflux_nan(&DP_Output_Array[I_Index], DNTP_Stat);
	       //               dntoflux_nan(&DP_Output_Array4[I_Index], DNTP_Stat);

	       //            }

            I_Index_Inp++;
            I_Index++;

            /* If this pixel had NaN, will keep going and try next pixel */

            DNTP_Stat->I_status = 0;
         }

      }

   }

}


}


/* Blank image pixels as indicated by the mask(s). */


if ((strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) ||
    (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0)) {

   if (DNTP_Stat->I_Verbose)
      printf("dntoflux_compute_results: Blanking bad pixels...\n");

   kkk = 0;

   for (k = kstart; k < kend; k++) {
      for (i=0;i<I_Length_X;i++)
      for (j=0;j<I_Length_Y;j++) {

         I_Index_Inp = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
         I_Index = kkk*I_Length_X*I_Length_Y+i*I_Length_Y+j;

	 /* Comment out NaN bit writing/checking - TYB 6/1/09 */

               DP_Output_Array2[I_Index] = DP_Output_Array[I_Index]; 

	 //         if (DP_Output_Array[I_Index] != 0 && 
	 //            dnan(*(short *) &DP_Output_Array[I_Index])) {

	 //            DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

            /* Write the error code into the array as an NaN 
               instead of the result.
            */
	 //            dntoflux_nan(&DP_Output_Array2[I_Index], DNTP_Stat);

	 //         } else {

	       //            if ((strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0)
		//                &&
		//                (DP_Data_Image2[I_Index_Inp] >= DNTP_Const->I_PMaskThreshold) 
		//               ) {

	      //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;

               /* 
                  Write the error code into the array as an NaN 
                  instead of the result.
               */

	       //               dntoflux_nan(&DP_Output_Array2[I_Index], DNTP_Stat);

	       //            } else if (
		       //               (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0) 
	       //               &&
	       //               (DP_Data_Image3[I_Index_Inp] >= DNTP_Const->I_DMaskThreshold)
	       //            ) {

	      //               DNTP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

	       //               nbadpixels++;

               /* 
                  Write the error code into the array as an NaN 
                  instead of the result.
               */

	       //               dntoflux_nan(&DP_Output_Array2[I_Index], DNTP_Stat);


	       //            } else {

	      //               DP_Output_Array2[I_Index] = DP_Output_Array[I_Index]; 
	       //            }

	    //         }


/* If this pixel had NaN, will keep going and try next pixel */

         DNTP_Stat->I_status = 0;
      }

      kkk++;
   }

}



	
   if (DNTP_Stat->I_Debug) {
      fp_data = fopen("dntoflux_data.dump", "w");
      for(k=0;k<I_Num_Frames;k++)
      for(i=0;i<I_Length_X;i++)
      for(j=0;j<I_Length_Y;j++) {
         I_Index = k*I_Length_X*I_Length_Y+i*I_Length_Y+j;
         fprintf(fp_data, "%f\n", DP_Output_Array[I_Index]);
      }
      fclose(fp_data);
   }

   if (DNTP_Stat->I_Verbose) {
      printf("dntoflux_compute_results: ");
      printf("Conversion completed.\n");
   }


end_compute_results:

   free(Input_Array);
   return;

}




