/* $Id: dntoflux_exec.c,v 1.4 2003/04/21 19:51:39 laher Exp $ */


/*********************************************** 
dntoflux_exec.c

Purpose

Reads in a FITS image.  Converts the data units using a single
scale factor if the operation flag is zero, or using four
readout-channel-dependent scale factors otherwise.  Outputs
the converted image to another FITS file, with the BUNIT keyword
set as prescribed by the user.  Optionally reads PMask and/or 
DMask files and thresholds, and blanks the masked pixels in a 
copy of the data-converted image, which is then outputted to a 
second FITS file.  

***********************************************/

#include <time.h>
#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


int main(int argc, char **argv)
{		
double	*DP_Data_Image1=NULL;
double	*DP_Data_Image2=NULL;
double	*DP_Data_Image3=NULL;
double	*DP_Data_Image4=NULL;

DNT_Constants		DNT_Const;
DNT_Filenames		DNT_Fnames;
DNT_Pathnames		DNT_Paths;
DNT_FITSinfo		DNT_FITS;
DNT_Computation		DNT_Comp;
DNT_Status		DNT_Stat;

clock();


/* Set constant default values */
dntoflux_init_constants( &DNT_Const, &DNT_Fnames, &DNT_Paths, &DNT_FITS, 
   &DNT_Comp, &DNT_Stat);

/* Parse the namelist, replacing default values if specified */
dntoflux_parse_namelist( argc, argv, &DNT_Const, &DNT_Fnames, &DNT_Paths, 
   &DNT_FITS, &DNT_Stat);

/* Parse the command line, replacing values if specified */
dntoflux_parse_args( argc, argv, &DNT_Const, &DNT_Fnames, &DNT_Paths, 
   &DNT_FITS, &DNT_Stat);

/* Expand any environmental variables passed into filename defintions */
dntoflux_expand_envvar( &DNT_Fnames, &DNT_Stat);

/* Get the data and keyword values from the input and output files */
dntoflux_read_data( &DNT_Fnames, &DNT_FITS, &DNT_Const, &DP_Data_Image1, 
   &DP_Data_Image2, &DP_Data_Image3, &DP_Data_Image4, &DNT_Stat);

/* Perform binary operation on the data*/
dntoflux_compute_results( &DNT_Const, &DNT_Fnames, &DNT_FITS, DP_Data_Image1, 
   DP_Data_Image2, DP_Data_Image3, DP_Data_Image4, &DNT_Comp, &DNT_Stat);

/* Write results to output file */			
dntoflux_output( &DNT_Const, &DNT_Fnames, &DNT_FITS, &DNT_Comp, &DNT_Stat);

/* Print filename, error message and timing data */
dntoflux_log_writer( &DNT_Const, &DNT_FITS, &DNT_Stat, &DNT_Fnames, 
   &DNT_Paths);

/* Deallocate memory */			
dntoflux_exit( &DP_Data_Image1, &DP_Data_Image2, &DP_Data_Image3, 
   &DP_Data_Image4, &DNT_Comp, &DNT_FITS, &DNT_Fnames, &DNT_Stat);

/* Send return value */
exit(DNT_Stat.I_status);

}




