/* $Id: dntoflux_output.c,v 1.5 2003/04/22 15:11:32 laher Exp $ */


/*********************************************** 
dntoflux_output.c

Purpose

Save the dntoflux outputs to files.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_output(	DNT_Constants		*DNTP_Const,
		DNT_Filenames		*DNTP_Fnames,
		DNT_FITSinfo		*DNTP_FITS,
		DNT_Computation		*DNTP_Comp,
		DNT_Status		*DNTP_Stat)
{
   int	i, j, k;
   int	I_Length_X, I_Length_Y, I_Num_Frames;
   int	I_Tot_Pixels;
   int	I_fits_return_status=0;
   int     morekeys = 0;

   fitsfile 	*ffp_FITS;
   fitsfile 	*ffp_FITS_In;


   if (DNTP_Stat->I_status) return;


   /* Copy structure info to local variables */
   I_Length_X = DNTP_FITS->I_Length_X;
   I_Length_Y = DNTP_FITS->I_Length_Y;
   I_Num_Frames = DNTP_FITS->I_Num_Frames;
   I_Tot_Pixels = I_Length_X*I_Length_Y*I_Num_Frames;


   /* OUTPUT IMAGE #1. */

   /* Open the FITS file for writing. */
   
   if (DNTP_Stat->I_Verbose)
      printf("Opening output FITS file #1...\n");

   fits_create_file(&ffp_FITS,
		 DNTP_Fnames->CP_Filename_FITS_Out,
		 &I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }


   /* 
      Copy input-file header to output file.
   */

   if (DNTP_Stat->I_Verbose)
      printf("Copying input-file FITS keywords to output file...\n");

   fits_open_file(&ffp_FITS_In,
		DNTP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

   fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

   fits_close_file(ffp_FITS_In, &I_fits_return_status);

   fits_flush_file(ffp_FITS, &I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DNTP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }	


   /* Write header keywords FITS file. */

   if (DNTP_Stat->I_Verbose)
      printf("Outputting FITS keywords...\n");

   dntoflux_output_keywords(ffp_FITS, DNTP_Const, DNTP_Fnames, 
                                DNTP_FITS, DNTP_Comp, DNTP_Stat);

   /* Don't return unless error, not warning. */
   if (((RIGHT_MASK&DNTP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


   /* Write image-data results to the output FITS file. */

   if (DNTP_Stat->I_Verbose)
      printf("Outputting FITS image #1 data...\n");

   fits_write_img(ffp_FITS,
		  TDOUBLE,
		  1, I_Tot_Pixels,
		  DNTP_Comp->DP_Output_Array,
		  &I_fits_return_status);		

   fits_flush_file(ffp_FITS, &I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }
		

   /* Close the FITS file. */

   if (DNTP_Stat->I_Verbose)
      printf("Closing output FITS file...\n");

   fits_close_file(ffp_FITS, &I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf("I_fits_return_status = %d\n",I_fits_return_status);

   if (I_fits_return_status) {
      DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }

   if(DNTP_Stat->I_Verbose) 
      printf("dntoflux_output: Result written to file:\n%s.\n",
         &DNTP_Fnames->CP_Filename_FITS_Out[1]);



   /* OUTPUT IMAGE #2 (WITH BLANKS). */

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out2, "") != 0) {


      /* Open the FITS file for writing. */

      if (DNTP_Stat->I_Verbose)
         printf("Opening output FITS file #2...\n");

      fits_create_file(&ffp_FITS,
	   	    DNTP_Fnames->CP_Filename_FITS_Out2,
		    &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if(I_fits_return_status) {
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* 
         Copy input-file header to output file.
      */

      if (DNTP_Stat->I_Verbose)
         printf("Copying input-file FITS keywords to output file...\n");

      fits_open_file(&ffp_FITS_In,
	 	DNTP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

      fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

      fits_close_file(ffp_FITS_In, &I_fits_return_status);

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }	


      /* Write header keywords FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Outputting FITS keywords...\n");

      dntoflux_output_keywords(ffp_FITS, DNTP_Const, DNTP_Fnames, 
                                DNTP_FITS, DNTP_Comp, DNTP_Stat);

      /* Don't return unless error, not warning. */
      if (((RIGHT_MASK&DNTP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


      /* Write image-data results to the output FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Outputting FITS image #2 data...\n");

      fits_write_img(ffp_FITS,
	   	  TDOUBLE,
		  1, I_Tot_Pixels,
		  DNTP_Comp->DP_Output_Array2 ,
		  &I_fits_return_status);		

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* Close the FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Closing output FITS file...\n");

      fits_close_file(ffp_FITS, &I_fits_return_status);

      if (I_fits_return_status)
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

      if (DNTP_Stat->I_Verbose)
         printf("dntoflux_output: Result written to file:\n%s.\n",
         &DNTP_Fnames->CP_Filename_FITS_Out2[1]);

   }


   /* OUTPUT IMAGE #4 (UNCERTAINTY). */

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {

      /* Open the FITS file for writing. */

      if (DNTP_Stat->I_Verbose)
         printf("Opening output FITS file #4...\n");

      fits_create_file(&ffp_FITS,
	   	    DNTP_Fnames->CP_Filename_FITS_Out4,
		    &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if(I_fits_return_status) {
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* 
         Copy input-file header to output file.
      */

      if (DNTP_Stat->I_Verbose)
         printf("Copying input-file FITS keywords to output file...\n");

      fits_open_file(&ffp_FITS_In,
	 	DNTP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

      fits_copy_hdu (ffp_FITS_In, ffp_FITS, morekeys, &I_fits_return_status);

      fits_close_file(ffp_FITS_In, &I_fits_return_status);

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }	


      /* Write header keywords FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Outputting FITS keywords...\n");

      dntoflux_output_keywords(ffp_FITS, DNTP_Const, DNTP_Fnames, 
                                DNTP_FITS, DNTP_Comp, DNTP_Stat);

      /* Don't return unless error, not warning. */
      if (((RIGHT_MASK&DNTP_Stat->I_status)&FITS_MSG_MASK) == 4096) return;


      /* Write image-data results to the output FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Outputting FITS image #4 data...\n");

      fits_write_img(ffp_FITS,
	   	  TDOUBLE,
		  1, I_Tot_Pixels,
		  DNTP_Comp->DP_Output_Array4 ,
		  &I_fits_return_status);		

      fits_flush_file(ffp_FITS, &I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf("I_fits_return_status = %d\n",I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }


      /* Close the FITS file. */

      if (DNTP_Stat->I_Verbose)
         printf("Closing output FITS file...\n");

      fits_close_file(ffp_FITS, &I_fits_return_status);

      if (I_fits_return_status)
         DNTP_Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

      if (DNTP_Stat->I_Verbose)
         printf("dntoflux_output: Result written to file:\n%s.\n",
         &DNTP_Fnames->CP_Filename_FITS_Out4[1]);

   }


}





