/* $Id: dntoflux_output_constkeywords.c,v 1.6 2004/08/27 18:29:37 laher Exp $ */


/*********************************************** 
dntoflux_output_constkeywords.c

Purpose

Write the keyword values for computation constants
to the open fits output file.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_output_constkeywords(	fitsfile	*ffp_FITS,
		                DNT_Filenames	*DNTP_Fnames,
				DNT_Constants	*DNTP_Const,
	  		        DNT_Computation	*DNTP_Comp,
  			        DNT_FITSinfo	*DNTP_FITS,
				DNT_Status	*DNTP_Stat)
{
int	k,I_fits_return_status=0,I_Num_In;
double	D_Num_In;
char	CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT];
char    CP_Keyvalue[FLEN_VALUE];


   if (DNTP_Stat->I_status) return;


   sprintf(CP_Keyname, "%s", "DNTOP");
   sprintf(CP_Comment, "%s", 
      "Operation type");
   I_Num_In = DNTP_Const->I_Operation;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   fits_flush_file(ffp_FITS,
		&I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf("DNT_output_constkeywords (DNTOP):  I_fits_return_status = %d\n",
         I_fits_return_status);

		
   if (I_fits_return_status) {
      DNTP_Stat->I_status = 
         CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }		 			



   if (DNTP_Const->I_Operation == 0) {

      sprintf(CP_Keyname, "%s", "DNTCNVFA");
      sprintf(CP_Comment, "%s", 
         "Conversion factor");
      D_Num_In = DNTP_Const->D_ConversionFactor;	
      fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTCNVFA):  I_fits_return_status = %d\n",
           I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			


      if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {


         sprintf(CP_Keyname, "%s", "DNTCNVFU");
         sprintf(CP_Comment, "%s", 
            "Conversion-factor uncertainty");
         D_Num_In = DNTP_Const->D_ConversionFactorUnc;	
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
         fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

         if (DNTP_Stat->I_Verbose)
            printf("DNT_output_constkeywords (DNTCNVFU): %s = %d\n",
              "I_fits_return_status", I_fits_return_status);

         if (I_fits_return_status) {
            DNTP_Stat->I_status = 
               CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
            return;
         }		 			
      }


   } else {

      sprintf(CP_Keyname, "%s", "DNTCNVF1");
      sprintf(CP_Comment, "%s", 
         "Readout-channel-1 conversion factor");
      D_Num_In = DNTP_Const->D_ConversionFactor1;	
      fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTCNVF1):  I_fits_return_status = %d\n",
           I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			

      sprintf(CP_Keyname, "%s", "DNTCNVF2");
      sprintf(CP_Comment, "%s", 
         "Readout-channel-2 conversion factor");
      D_Num_In = DNTP_Const->D_ConversionFactor2;	
      fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTCNVF2):  I_fits_return_status = %d\n",
           I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			

      sprintf(CP_Keyname, "%s", "DNTCNVF3");
      sprintf(CP_Comment, "%s", 
         "Readout-channel-3 conversion factor");
      D_Num_In = DNTP_Const->D_ConversionFactor3;	
      fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTCNVF3):  I_fits_return_status = %d\n",
           I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			

      sprintf(CP_Keyname, "%s", "DNTCNVF4");
      sprintf(CP_Comment, "%s", 
         "Readout-channel-4 conversion factor");
      D_Num_In = DNTP_Const->D_ConversionFactor4;	
      fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTCNVF4):  I_fits_return_status = %d\n",
           I_fits_return_status);

      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			


      if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {

         sprintf(CP_Keyname, "%s", "DNTCNFU1");
         sprintf(CP_Comment, "%s", 
            "RO-chnl-1 conversion-factor uncertainty");
         D_Num_In = DNTP_Const->D_ConversionFactorUnc1;	
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
         fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

         if (DNTP_Stat->I_Verbose)
            printf("DNT_output_constkeywords (DNTCNFU1): %s = %d\n",
              "I_fits_return_status", I_fits_return_status);

         if (I_fits_return_status) {
            DNTP_Stat->I_status = 
               CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
            return;
         }		 	

         sprintf(CP_Keyname, "%s", "DNTCNFU2");
         sprintf(CP_Comment, "%s", 
            "RO-chnl-2 converson-factor uncertainty");
         D_Num_In = DNTP_Const->D_ConversionFactorUnc2;	
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
         fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

         if (DNTP_Stat->I_Verbose)
            printf("DNT_output_constkeywords (DNTCNFU2): %s = %d\n",
              "I_fits_return_status", I_fits_return_status);

         if (I_fits_return_status) {
            DNTP_Stat->I_status = 
               CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
            return;
         }		 	

         sprintf(CP_Keyname, "%s", "DNTCNFU3");
         sprintf(CP_Comment, "%s", 
            "RO-chnl-3 conversion-factor uncertainty");
         D_Num_In = DNTP_Const->D_ConversionFactorUnc3;	
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
         fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

         if (DNTP_Stat->I_Verbose)
            printf("DNT_output_constkeywords (DNTCNFU3): %s = %d\n",
              "I_fits_return_status", I_fits_return_status);

         if (I_fits_return_status) {
            DNTP_Stat->I_status = 
               CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
            return;
         }		 	

         sprintf(CP_Keyname, "%s", "DNTCNFU4");
         sprintf(CP_Comment, "%s", 
            "RO-chnl-4 conversion-factor uncertainty");
         D_Num_In = DNTP_Const->D_ConversionFactorUnc4;	
         fits_write_key(ffp_FITS,
		TDOUBLE,
		CP_Keyname,
		&D_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
         fits_flush_file(ffp_FITS,
	  	&I_fits_return_status);

         if (DNTP_Stat->I_Verbose)
            printf("DNT_output_constkeywords (DNTCNFU4): %s = %d\n",
              "I_fits_return_status", I_fits_return_status);

         if (I_fits_return_status) {
            DNTP_Stat->I_status = 
               CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
            return;
         }		 	

      }

   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) {

      sprintf(CP_Keyname, "%s", "DNTPMTHR");
      sprintf(CP_Comment, "%s", 
         "PMask threshold");
      I_Num_In = DNTP_Const->I_PMaskThreshold;	
      fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
		&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTPMTHR):  I_fits_return_status = %d\n",
           I_fits_return_status);


      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }

   }		 			


   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0) {

      sprintf(CP_Keyname, "%s", "DNTDMTHR");
      sprintf(CP_Comment, "%s", 
         "DMask threshold");
      I_Num_In = DNTP_Const->I_DMaskThreshold;
      fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);
		
      fits_flush_file(ffp_FITS,
		&I_fits_return_status);

      if (DNTP_Stat->I_Verbose)
         printf(
           "DNT_output_constkeywords (DNTDMTHR):  I_fits_return_status = %d\n",
           I_fits_return_status);


      if (I_fits_return_status) {
         DNTP_Stat->I_status = 
            CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
         return;
      }		 			

   }

   sprintf(CP_Keyname, "%s", "DNTPLANE");
   sprintf(CP_Comment, "%s", 
      "Data planes processed: 1=All, 2=First, 3=Last");
   I_Num_In = DNTP_FITS->I_Data_Plane;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   fits_flush_file(ffp_FITS,
		&I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf(
        "DNT_output_constkeywords (DNTPLANE):  I_fits_return_status = %d\n",
        I_fits_return_status);

		
   if (I_fits_return_status) {
      DNTP_Stat->I_status = 
         CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }		 			

   sprintf(CP_Keyname, "%s", "DNTNORMT");
   sprintf(CP_Comment, "%s", 
      "Normalized by EXPTIME: 0=No, 1=Yes");
   I_Num_In = DNTP_Const->I_NormalizeByExpTimeFlag;
   fits_write_key(ffp_FITS,
		TINT,
		CP_Keyname,
		&I_Num_In,
		CP_Comment,
		&I_fits_return_status);

   fits_flush_file(ffp_FITS,
		&I_fits_return_status);

   if (DNTP_Stat->I_Verbose)
      printf(
        "DNT_output_constkeywords (DNTNORMT):  I_fits_return_status = %d\n",
        I_fits_return_status);

		
   if (I_fits_return_status) {
      DNTP_Stat->I_status = 
         CONST_KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }		 			

}






