/* $Id: dntoflux_output_filenkeywords.c,v 1.5 2004/08/27 18:29:38 laher Exp $ */


/*********************************************** 
dntoflux_output_filenkeywords.c

Purpose

Write the keyword values for input and output filenames
to the open fits output file.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"

void getRootFilename( char *, char * );


dntoflux_output_filenkeywords(	fitsfile	*ffp_FITS,
				DNT_Filenames	*DNTP_Fnames,
			        DNT_FITSinfo	*DNTP_FITS,
				DNT_Status	*DNTP_Stat)
{
int	I_fits_return_status=0;
char	CP_Keyname[FLEN_KEYWORD];
char    CP_Keyvalue[FLEN_VALUE];
char    RootFilename[FLEN_VALUE],CompleteFilename[MAX_FILENAME_LENGTH];

if (DNTP_Stat->I_status) return;

sprintf(CP_Keyname, "%s", "DNTNLFIL");

strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_Namelist);
getRootFilename(RootFilename,CompleteFilename);
strcpy(CP_Keyvalue,RootFilename);
fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		

sprintf(CP_Keyname, "%s", "DNTIFIL ");
strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Image1);
getRootFilename(RootFilename,CompleteFilename);
strcpy(CP_Keyvalue,RootFilename);
fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		


if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) {

   sprintf(CP_Keyname, "%s", "DNTIFIL2");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Image2);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		

}

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0) {

   sprintf(CP_Keyname, "%s", "DNTIFIL3");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Image3);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		

}

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image4,"") != 0) {

   sprintf(CP_Keyname, "%s", "DNTIUFIL");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Image4);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		

}

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out, "") != 0) {
			
   sprintf(CP_Keyname, "%s", "DNTOFIL1");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Out);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);		
}

if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out2, "") != 0) {

   sprintf(CP_Keyname, "%s", "DNTOFIL2");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Out2);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);				
}


if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {

   sprintf(CP_Keyname, "%s", "DNTOUFIL");
   strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_FITS_Out4);
   getRootFilename(RootFilename,CompleteFilename);
   strcpy(CP_Keyvalue,RootFilename);
   fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);				

}


sprintf(CP_Keyname, "%s", "DNTLGFIL");
strcpy(CompleteFilename,DNTP_Fnames->CP_Filename_Log);
getRootFilename(RootFilename,CompleteFilename);
strcpy(CP_Keyvalue,RootFilename);
fits_update_key(ffp_FITS, TSTRING,
		CP_Keyname, CP_Keyvalue,
		"",
		&I_fits_return_status);	


fits_flush_file(ffp_FITS,
		&I_fits_return_status);

if (DNTP_Stat->I_Verbose)
   printf("DNT_output_filenkeywords:  I_fits_return_status = %d\n",
      I_fits_return_status);

if (I_fits_return_status)
   DNTP_Stat->I_status = FILEN_KEYWORDS|FITS_MSG_MASK|I_fits_return_status;

}


/* Strip pathname off filename. */

void getRootFilename( char *RootFilename, char *CompleteFilename )

{

   int i,n,si,sf;

   n = strlen(CompleteFilename);

   si = 0;
   sf = n;
   for (i = n - 1; i >= 0; i--) {
      if (CompleteFilename[i] == '/' || CompleteFilename[i] == '!') 
      { 
         si = i + 1; 
         break;
      }
      if (CompleteFilename[i] == ' ') { sf = i; }
   }

   for (i = 0; i < sf - si; i++) {
      RootFilename[i] = CompleteFilename[i + si];
   }

   RootFilename[sf - si] = '\0';

}

