/* $Id: dntoflux_output_keywords.c,v 1.5 2003/04/21 19:51:39 laher Exp $ */


/*********************************************** 
dntoflux_output_keywords.c

Purpose

Write the keyword values to the open fits output file.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_output_keywords(	fitsfile	*ffp_FITS,
			DNT_Constants	*DNTP_Const,
			DNT_Filenames	*DNTP_Fnames,
			DNT_FITSinfo	*DNTP_FITS,
			DNT_Computation	*DNTP_Comp,
			DNT_Status	*DNTP_Stat)
{
int	I_fits_return_status=0, I_Operation;
char	CP_Keyname[FLEN_KEYWORD];
char    CP_Comment[FLEN_COMMENT];
char    CP_Keyvalue[FLEN_VALUE];
double  version;
char    programname[32], versionstring[32];


if(DNTP_Stat->I_status) return;


/* Write the keywords that must be in every FITS file */

if (DNTP_Stat->I_Verbose)
   printf("Writing mandatory header keywords to output FITS file...\n");

dntoflux_output_stdkeywords( ffp_FITS, DNTP_Fnames, DNTP_FITS, DNTP_Const, 
   DNTP_Stat);

if(DNTP_Stat->I_status) return;

if (DNTP_Stat->I_Verbose) {
   printf("Writing date to output FITS file...\n");
   printf("DNT_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);
}
				
fits_write_date(ffp_FITS, &I_fits_return_status);		 

fits_flush_file(ffp_FITS, &I_fits_return_status);

if (DNTP_Stat->I_Verbose)
   printf("DNT_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);

if (I_fits_return_status) {
   DNTP_Stat->I_status = DATE|I_fits_return_status; 
   return;
}		 


/* 
   Description of the data in this FITS file.  Input string to CP_Keyname
   should be no longer than 8 characters.  Otherwise, data in memory will 
   be stepped on.
*/
					
sprintf(programname, "%s", "DNTOFLUX");
sprintf(versionstring, "%.1f", DNTVERSN);
writeDLhistoryline(ffp_FITS, programname, versionstring, 
   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DNTPROC");
sprintf(CP_Keyvalue, "%s", PROGRAM);
sprintf(CP_Comment, "%s", PROGRAMPURPOSE);


fits_write_key_str(ffp_FITS,
		CP_Keyname, CP_Keyvalue,
		CP_Comment,
		&I_fits_return_status);


sprintf(CP_Keyname, "%s", PROGRAMABB);
strcat(CP_Keyname, "VERSN");
version = DNTVERSN;
sprintf(CP_Comment, "%s%s%s", "Version number of ",PROGRAM," program");

fits_write_key_fixflt(ffp_FITS, 
		CP_Keyname, version, 2,
		CP_Comment,
		&I_fits_return_status);

fits_flush_file(ffp_FITS, &I_fits_return_status);

if (DNTP_Stat->I_Verbose)
   printf("DNT_output_keywords: I_fits_return_status = %d\n",
      I_fits_return_status);

if (I_fits_return_status) {
   DNTP_Stat->I_status = KEYWORDS|FITS_MSG_MASK|I_fits_return_status; 
   return;
}		 

if (DNTP_Stat->I_Verbose) 
   printf("Writing constant header keywords to output FITS file...\n");

dntoflux_output_constkeywords( ffp_FITS, DNTP_Fnames, DNTP_Const, DNTP_Comp, 
   DNTP_FITS, DNTP_Stat);

if (DNTP_Stat->I_Verbose) 
   printf("Writing output header keywords to output FITS file...\n");

/* Files associated with this one. */				
dntoflux_output_filenkeywords( ffp_FITS, DNTP_Fnames, DNTP_FITS, 
   DNTP_Stat);

}





