/* $Id: dntoflux_parse_args.c,v 1.7 2003/04/21 20:17:29 laher Exp $ */


/*********************************************** 
dntoflux_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement, and
store command-line inputs.

***********************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "dntoflux_errcodes.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_parse_args(	int		argc,
			char		**argv,
			DNT_Constants	*DNTP_Const,
			DNT_Filenames	*DNTP_Fnames,
			DNT_Pathnames	*DNTP_Paths,
			DNT_FITSinfo	*DNTP_FITS,
			DNT_Status	*DNTP_Stat)
{
int	i=1;
int     I_gotallinputs;
char    CP_Value[STRING_BUFFER_SIZE];

/* Run regardless of I_dntoflux_status so that
   at least log file name will be known */



      while( i < argc )
        {
          if( argv[i][0] == '-' )
            {
              switch( argv[i][1] ) {
              case 'n':      /* -n <input_namelist_fname>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -n <input_namelist_fname> missing argument\n" );
		else
		if(DNTP_Fnames->CP_Filename_Namelist) /* Filename can be stored */
                sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_Namelist);
                break;
              case 'i':      /* -i <input_image_fname>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -i <input_image_fname> missing argument\n" );
		else
		if(argv[i-1][2] == '\0')
 		   if(DNTP_Fnames->CP_Filename_FITS_Image1)
                   sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_FITS_Image1);
		if(argv[i-1][2] == 'p')
		   if(DNTP_Fnames->CP_Filename_FITS_Image2)
                   sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_FITS_Image2);
		if(argv[i-1][2] == 'd')
  		   if(DNTP_Fnames->CP_Filename_FITS_Image3)
                   sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_FITS_Image3);
		if(argv[i-1][2] == 'u')
  		   if(DNTP_Fnames->CP_Filename_FITS_Image4)
                   sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_FITS_Image4);
                break;
	      case 'o':      /* -o <output_fname>  */
		if( ++i >= argc )
                printf("dntoflux_parse_args: -o <output_fname> missing argument\n" );
		else  
		if(argv[i-1][2] == '1')
                   if(DNTP_Fnames->CP_Filename_FITS_Out)	{
		      sscanf("!", "%c", 
                         DNTP_Fnames->CP_Filename_FITS_Out);  /* clobber */
                      sscanf(argv[i], "%s", 
                         &(DNTP_Fnames->CP_Filename_FITS_Out[1]));
		   }
		if(argv[i-1][2] == '2')
                   if(DNTP_Fnames->CP_Filename_FITS_Out2)	{
		      sscanf("!", "%c", 
                         DNTP_Fnames->CP_Filename_FITS_Out2);  /* clobber */
                      sscanf(argv[i], "%s", 
                         &(DNTP_Fnames->CP_Filename_FITS_Out2[1]));
		   }
		if(argv[i-1][2] == 'u')
                   if(DNTP_Fnames->CP_Filename_FITS_Out4)	{
		      sscanf("!", "%c", 
                         DNTP_Fnames->CP_Filename_FITS_Out4);  /* clobber */
                      sscanf(argv[i], "%s", 
                         &(DNTP_Fnames->CP_Filename_FITS_Out4[1]));
		   }
                break;
	      case 'l':      /* -l <log_fname>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -l <fname> missing argument\n" );
		else
		if(DNTP_Fnames->CP_Filename_Log)
                sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_Log);
                break;
	      case 'c':      /* -c <dntoflux_conversion_factor>  */
                if( ++i >= argc )
                   printf("dntoflux_parse_args: -c <dntoflux_conversion_factor> missing argument\n" );
		else {
 		   if(argv[i-1][2] == 'o') {
  		      sscanf(argv[i], "%d", &(DNTP_Const->I_Operation));
		   }
 		   if(argv[i-1][2] == '\0') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactor = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '1') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactor1 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '2') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactor2 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '3') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactor3 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '4') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactor4 = atof(CP_Value);
		   }
                }
                break;
	      case 'u':      /* -u <dntoflux_conversion_factor_uncertainty>  */
                if( ++i >= argc )
                   printf("dntoflux_parse_args: -u <dntoflux_conversion_factor_uncertainty> missing argument\n" );
		else {
 		   if(argv[i-1][2] == '\0') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactorUnc = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '1') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactorUnc1 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '2') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactorUnc2 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '3') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactorUnc3 = atof(CP_Value);
		   }
 		   if(argv[i-1][2] == '4') {
  		      sscanf(argv[i], "%s", CP_Value);
                      DNTP_Const->D_ConversionFactorUnc4 = atof(CP_Value);
		   }
                }
                break;
	      case 'b':      /* -b <mask_threshold>  */
                if( ++i >= argc )
                   printf("dntoflux_parse_args: -b <mask_threshold> missing argument\n" );
		else {
		   if(argv[i-1][2] == 'p')
  		      sscanf(argv[i], "%d", &(DNTP_Const->I_PMaskThreshold));
		   if(argv[i-1][2] == 'd')
  		      sscanf(argv[i], "%d", &(DNTP_Const->I_DMaskThreshold));
                }
                break;
	      case 'p':      /* -p <data_plane>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -p <data_plane> missing argument\n" );
		else
		sscanf(argv[i], "%d", &(DNTP_FITS->I_Data_Plane));
                break;
	      case 't':      /* -t <normalize_by_EXPTIME_flag>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -t <normalize_by_EXPTIME_flag> missing argument\n" );
		else
		sscanf(argv[i], "%d", &(DNTP_Const->I_NormalizeByExpTimeFlag));
                break;
	      case 's':      /* -s <bunit>  */
                if( ++i >= argc )
                printf("dntoflux_parse_args: -s <bunit> missing argument\n" );
		else
		sscanf(argv[i], "%s", &(DNTP_Const->Bunit_String));
                break;
              case 'a':      /* -a <input_ancillary_file_pathname>  */
                if( ++i >= argc )
                   printf("dntoflux_parse_args: -a <input_ancillary_file_pathname> missing argument\n" );
		else
		if(DNTP_Paths->CP_Ancillary_File_Path) /* Filename can be stored */
                   sscanf(argv[i], "%s", DNTP_Paths->CP_Ancillary_File_Path);
                break;
              case 'd':      /* -debug   */
                DNTP_Stat->I_Debug = 1 - DNTP_Stat->I_Debug;
                break;
              case 'v':      /* -verbose   */
		if(argv[i][2] != 'v')
                   DNTP_Stat->I_Verbose = 1 - DNTP_Stat->I_Verbose;
		if(argv[i][2] == 'v')
  		   DNTP_Stat->I_SuperVerbose = 1 - DNTP_Stat->I_SuperVerbose;
                break;
              default:
                printf("dntoflux_parse_args: unknown argument\n");
              }
            }
          else
            {
              printf("dntoflux_parse_args: Command line syntax error:\n");
              printf("   Previous argument = %s\n",argv[i - 1]);
              printf("   Current argument = %s\n",argv[i]);
            }
          i++;
        }


/* Print to stdout the program name and version number */

if(DNTP_Stat->I_Verbose) 
   printf("dntoflux_parse_args: Program %s, Version %2.1f\n", 
      PROGRAM, DNTVERSN);


/* Check code inputs. */

I_gotallinputs = 0;

   if(DNTP_Stat->I_Verbose) {
      if (strcmp(DNTP_Fnames->CP_Filename_Namelist,"") != 0) {
         printf("dntoflux_parse_args: Namelist File = %s\n", 
            DNTP_Fnames->CP_Filename_Namelist);
      } else {
         printf("dntoflux_parse_args: Information only: %s\n",
            "No namelist file specified.");
      }
   }

   if(DNTP_Stat->I_Verbose) {
      if (strcmp(DNTP_Paths->CP_Ancillary_File_Path,"") != 0) {
         printf("dntoflux_parse_args: Ancilllary File Pathname = %s\n", 
            DNTP_Paths->CP_Ancillary_File_Path);
      } else {
         printf("dntoflux_parse_args: Information only: %s %s\n",
            "No ancillary file pathname specified;",
            "defaulting to current directory.");
      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image1,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Input Image = %s\n", 
         DNTP_Fnames->CP_Filename_FITS_Image1);
   } else {
      printf(
         "dntoflux_parse_args: Missing input FITS filename %s\n",
         "(-i <fname>).");
      I_gotallinputs = 1;
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Input PMask Image = %s\n", 
         DNTP_Fnames->CP_Filename_FITS_Image2);
      if (DNTP_Const->I_PMaskThreshold < 0 || 
          DNTP_Const->I_PMaskThreshold > 65535) {
         printf("dntoflux_parse_args: %s\n",
         "PMask threshold out of range (-bp <0-65535>).");
         I_gotallinputs = 2;
      } else {
         if(DNTP_Stat->I_Verbose)
            printf("dntoflux_parse_args: PMask Threshold = %d\n", 
            DNTP_Const->I_PMaskThreshold);
      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Input DMask Image = %s\n", 
         DNTP_Fnames->CP_Filename_FITS_Image3);
      if (DNTP_Const->I_DMaskThreshold < 0 || 
          DNTP_Const->I_DMaskThreshold > 65535) {
         printf("dntoflux_parse_args: %s\n",
         "DMask threshold out of range (-bp <0-65535>).");
         I_gotallinputs = 2;
      } else {
         if(DNTP_Stat->I_Verbose)
            printf("dntoflux_parse_args: DMask Threshold = %d\n", 
            DNTP_Const->I_DMaskThreshold);
      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Image4,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Input Uncertainty Image = %s\n", 
         DNTP_Fnames->CP_Filename_FITS_Image4);
   } else {
      if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {
         printf(
            "dntoflux_parse_args: Missing input uncertainty FITS %s\n",
            "filename (-iu <fname>).");
         I_gotallinputs = 1;

      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out,"")  != 0) {
      if(DNTP_Stat->I_Verbose) 
         printf("dntoflux_parse_args: Output File 1 = %s\n", 
         &DNTP_Fnames->CP_Filename_FITS_Out[1]);
   } else {
      printf(
         "dntoflux_parse_args: Missing output FITS filename %s\n",
         "(-o1 <fname>).");
      I_gotallinputs = 1;
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out2,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Output File 2 = %s\n", 
         &DNTP_Fnames->CP_Filename_FITS_Out2[1]);
   } else {
      if ((strcmp(DNTP_Fnames->CP_Filename_FITS_Image2,"") != 0) ||
          (strcmp(DNTP_Fnames->CP_Filename_FITS_Image3,"") != 0)) {
         printf(
            "dntoflux_parse_args: Missing output FITS filename %s\n",
            "(-o2 <fname>).");
         I_gotallinputs = 1;
      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {
      if(DNTP_Stat->I_Verbose)
         printf("dntoflux_parse_args: Output Uncertainty Image = %s\n", 
         &DNTP_Fnames->CP_Filename_FITS_Out4[1]);
   }

   if (!(DNTP_FITS->I_Data_Plane < 1 || DNTP_FITS->I_Data_Plane > 3)) {
      if(DNTP_Stat->I_Verbose) 
         printf("dntoflux_parse_args: %s = %d\n", 
            "Data-plane calculation flag",DNTP_FITS->I_Data_Plane);
   } else {
      printf("dntoflux_parse_args: %s\n",
         "Data-plane calculation flag out of range (-p <1, 2, or 3>).");
      I_gotallinputs = 2;
   }

   if (!(DNTP_Const->I_Operation < 0 || DNTP_Const->I_Operation > 1)) {
      if(DNTP_Stat->I_Verbose) 
         printf("dntoflux_parse_args: %s = %d\n", 
            "Operation flag",DNTP_Const->I_Operation);
   } else {
      printf("dntoflux_parse_args: %s\n",
         "Operation flag out of range (-co <0 or 1>).");
      I_gotallinputs = 2;
   }

   if (!(DNTP_Const->I_NormalizeByExpTimeFlag < 0 || 
      DNTP_Const->I_NormalizeByExpTimeFlag > 1)) {
      if(DNTP_Stat->I_Verbose) 
         printf("dntoflux_parse_args: %s = %d\n", 
            "NormalizeByExpTimeFlag", DNTP_Const->I_NormalizeByExpTimeFlag);
   } else {
      printf("dntoflux_parse_args: %s\n",
         "NormalizeByExpTimeFlag out of range (-t <0 or 1>).");
      I_gotallinputs = 2;
   }

   if (DNTP_Const->I_Operation == 0) {
      if (DNTP_Const->D_ConversionFactor != -999999.0) {
         if(DNTP_Stat->I_Verbose) 
            printf("dntoflux_parse_args: %s = %f\n", 
               "DN-to-flux conversion factor", DNTP_Const->D_ConversionFactor);
      } else {
         printf("dntoflux_parse_args: %s\n",
            "Missing conversion factor (-c <value>).");
         I_gotallinputs = 1;
      }
   } else {
      if (DNTP_Const->D_ConversionFactor1 != -999999.0) {
         if(DNTP_Stat->I_Verbose) 
            printf("dntoflux_parse_args: %s = %f\n", 
               "Readout-channel-1 conversion factor", 
               DNTP_Const->D_ConversionFactor1);
      } else {
         printf("dntoflux_parse_args: Missing Readout-channel-1 %s\n",
            "DN-to-flux conversion factor (-c1 <value>).");
         I_gotallinputs = 1;
      }
      if (DNTP_Const->D_ConversionFactor2 != -999999.0) {
         if(DNTP_Stat->I_Verbose) 
            printf("dntoflux_parse_args: %s = %f\n", 
               "Readout-channel-2 conversion factor", 
               DNTP_Const->D_ConversionFactor2);
      } else {
         printf("dntoflux_parse_args: Missing Readout-channel-2 %s\n",
            "DN-to-flux conversion factor (-c2 <value>).");
         I_gotallinputs = 1;
      }
      if (DNTP_Const->D_ConversionFactor3 != -999999.0) {
         if(DNTP_Stat->I_Verbose) 
            printf("dntoflux_parse_args: %s = %f\n", 
               "Readout-channel-3 conversion factor", 
               DNTP_Const->D_ConversionFactor3);
      } else {
         printf("dntoflux_parse_args: Missing Readout-channel-3 %s\n",
            "DN-to-flux conversion factor (-c3 <value>).");
         I_gotallinputs = 1;
      }
      if (DNTP_Const->D_ConversionFactor4 != -999999.0) {
         if(DNTP_Stat->I_Verbose) 
            printf("dntoflux_parse_args: %s = %f\n", 
               "Readout-channel-4 conversion factor", 
               DNTP_Const->D_ConversionFactor4);
      } else {
         printf("dntoflux_parse_args: Missing Readout-channel-4 %s\n",
            "DN-to-flux conversion factor (-c4 <value>).");
         I_gotallinputs = 1;
      }
   }

   if (strcmp(DNTP_Fnames->CP_Filename_FITS_Out4,"") != 0) {
      if (DNTP_Const->I_Operation == 0) {
         if (DNTP_Const->D_ConversionFactorUnc != -999999.0) {
            if(DNTP_Stat->I_Verbose) 
               printf("dntoflux_parse_args: %s = %f\n", 
                  "DN-to-flux conversion-factor uncertainty", 
                  DNTP_Const->D_ConversionFactorUnc);
         } else {
            printf("dntoflux_parse_args: %s\n",
               "Missing conversion-factor uncertainty (-u <value>).");
            I_gotallinputs = 1;
         }
      } else {
         if (DNTP_Const->D_ConversionFactorUnc1 != -999999.0) {
            if(DNTP_Stat->I_Verbose) 
               printf("dntoflux_parse_args: %s = %f\n", 
                  "Readout-channel-1 conversion-factor uncertainty", 
                  DNTP_Const->D_ConversionFactorUnc1);
         } else {
            printf("dntoflux_parse_args: Missing Readout-channel-1 %s\n",
               "DN-to-flux conversion-factor uncertainty (-u1 <value>).");
            I_gotallinputs = 1;
         }
         if (DNTP_Const->D_ConversionFactorUnc2 != -999999.0) {
            if(DNTP_Stat->I_Verbose) 
               printf("dntoflux_parse_args: %s = %f\n", 
                  "Readout-channel-2 conversion-factor uncertainty", 
                  DNTP_Const->D_ConversionFactorUnc2);
         } else {
            printf("dntoflux_parse_args: Missing Readout-channel-2 %s\n",
               "DN-to-flux conversion-factor uncertainty (-u2 <value>).");
            I_gotallinputs = 1;
         }
         if (DNTP_Const->D_ConversionFactorUnc3 != -999999.0) {
            if(DNTP_Stat->I_Verbose) 
               printf("dntoflux_parse_args: %s = %f\n", 
                  "Readout-channel-3 conversion-factor uncertainty", 
                  DNTP_Const->D_ConversionFactorUnc3);
         } else {
            printf("dntoflux_parse_args: Missing Readout-channel-3 %s\n",
               "DN-to-flux conversion factor-uncertainty (-u3 <value>).");
            I_gotallinputs = 1;
         }
         if (DNTP_Const->D_ConversionFactorUnc4 != -999999.0) {
            if(DNTP_Stat->I_Verbose) 
               printf("dntoflux_parse_args: %s = %f\n", 
                  "Readout-channel-4 conversion-factor uncertainty", 
                  DNTP_Const->D_ConversionFactorUnc4);
         } else {
            printf("dntoflux_parse_args: Missing Readout-channel-4 %s\n",
               "DN-to-flux conversion factor-uncertainty (-u4 <value>).");
            I_gotallinputs = 1;
         }
      }
   }

   if (strcmp(DNTP_Const->Bunit_String,"")  != 0) {
      if(DNTP_Stat->I_Verbose) 
         printf("dntoflux_parse_args: Output BUNIT = %s\n", 
         &DNTP_Const->Bunit_String);
   } else {
      printf(
         "dntoflux_parse_args: Missing output BUNIT setting %s\n",
         "(-s <units>).");
      I_gotallinputs = 1;
   }



   if(DNTP_Stat->I_Verbose) {

      printf("dntoflux_parse_args: Log File = %s\n", 
         DNTP_Fnames->CP_Filename_Log);

      printf("dntoflux_parse_args: I_Debug = %d\n", 
         DNTP_Stat->I_Debug);

      printf("dntoflux_parse_args: I_Verbose = %d\n", 
         DNTP_Stat->I_Verbose);

      printf("dntoflux_parse_args: I_SuperVerbose = %d\n", 
         DNTP_Stat->I_SuperVerbose);

   }


   if (DNTP_Stat->I_status) return;

   if (I_gotallinputs == 1) {
      DNTP_Stat->I_status = PARSE_ARGS|MISSING_INPUT;
      return;
   }
   if (I_gotallinputs == 2) {
      DNTP_Stat->I_status = PARSE_ARGS|INPUTS_OUT_OF_RANGE;
      return;
   }


}







