/* $Id: dntoflux_parse_namelist.c,v 1.5 2003/04/21 20:17:45 laher Exp $ */


/*********************************************** 
dntoflux_parse_namelist.c

Purpose

Read and parse a parameter file to get input and output file
names constants, and processing flags.

***********************************************/

#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include "dntoflux.h"
#include "dntoflux_defs.h"

#define	FILE_BUFFER_SIZE	8192

dntoflux_parse_namelist(  int		argc,
			char		**argv,
			DNT_Constants	*DNTP_Const,
			DNT_Filenames	*DNTP_Fnames,
			DNT_Pathnames	*DNTP_Paths,
			DNT_FITSinfo	*DNTP_FITS,
			DNT_Status	*DNTP_Stat)
{


  FILE	*fp_namelist;
  /*char	CP_buf[FILE_BUFFER_SIZE];*/
  char	*CP_buf;
  char	CP_NextWord[STRING_BUFFER_SIZE];
  char  CP_Equals[STRING_BUFFER_SIZE];
  char  CP_Value[STRING_BUFFER_SIZE];
  char  CP_Comment[STRING_BUFFER_SIZE];
  char	*CP_Pointer, *CP_Comma;
     
  int	i=1, I_Got_Namelist=0, I_SectionsFound=0;
  int	I_fildes_nl=0, I_str_length=0;
  int	I_past_end=0, I_found=0;
  
  struct stat  STAT_nlbuf, *STATP_nlbuf;
  
if(argc==1)	{

fprintf(stdout, "Usage: dntoflux\n");
fprintf(stdout, "       -n <input_namelist_fname> (Command-line inputs only or required)\n");
fprintf(stdout, "       -i <input_image_fname> (Namelist or required)\n");
fprintf(stdout, "       -o1 <output_image_fits_fname> %s\n",
   "(Required)");
fprintf(stdout, "       -ip <input_pmask_image_fname> (Optional)\n");
fprintf(stdout, "       -id <input_dmask_image_fname> (Optional)\n");
fprintf(stdout, "       -iu <input_uncertainty_image_fname> \n          %s\n",
   "(Required if -ou option is specified)");
fprintf(stdout, "       -o2 <output_blanked_image_fits_fname>\n          %s\n",
   "(Required if either -ip or -id options are specified)");
fprintf(stdout, "       -ou <output_uncertainty_image_fits_fname> %s\n",
   "(Optional)");
fprintf(stdout, "       -co <operation>  \n          %s\n",
   "(Optional; 0=single-factor scaling, 1=readout-channel scaling)");
fprintf(stdout, "       -c <dntoflux_conversion_factor>  \n          %s\n",
   "(Required if operation = 0)");
fprintf(stdout, "       -c1 <channel_1_conversion_factor>  \n          %s\n",
   "(Required if operation = 1)");
fprintf(stdout, "       -c2 <channel_2_conversion_factor>  \n          %s\n",
   "(Required if operation = 1)");
fprintf(stdout, "       -c3 <channel_3_conversion_factor>  \n          %s\n",
   "(Required if operation = 1)");
fprintf(stdout, "       -c4 <channel_4_conversion_factor>  \n          %s\n",
   "(Required if operation = 1)");
fprintf(stdout, 
   "       -u <dntoflux_conversion_factor_uncertainty>  \n          %s\n",
   "(Required if operation = 0 and -ou option is specified)");
fprintf(stdout, 
   "       -u1 <channel_1_conversion_factor_uncertainty>  \n          %s\n",
   "(Required if operation = 1 and -ou option is specified)");
fprintf(stdout, 
   "       -u2 <channel_2_conversion_factor_uncertainty>  \n          %s\n",
   "(Required if operation = 1 and -ou option is specified)");
fprintf(stdout, 
   "       -u3 <channel_3_conversion_factor_uncertainty>  \n          %s\n",
   "(Required if operation = 1 and -ou option is specified)");
fprintf(stdout, 
   "       -u4 <channel_4_conversion_factor_uncertainty>  \n          %s\n",
   "(Required if operation = 1 and -ou option is specified)");
fprintf(stdout, "       -s <converted_units>  \n          %s\n",
   "(Required)");
fprintf(stdout, "       -bp <pmask_blank_threshold>  \n          %s\n",
   "(Optional)");
fprintf(stdout, "       -bd <dmask_blank_threshold>  \n          %s\n",
   "(Optional)");
fprintf(stdout, "       -t <Normalize_by_EXPTIME_flag> %s\n",
   "(0=Don't, 1=Do...; default is 0)");
fprintf(stdout, "       -p <data_plane_to_process> %s\n",
   "(1=All, 2=First, 3=Last; default is 1)");
fprintf(stdout, "       -l <log_fname> (Default is stdout)\n");
fprintf(stdout, "       -a <ancillary_file_path> (Optional)\n");
fprintf(stdout, "       -d (prints debug statements) \n");
fprintf(stdout, "       -v (verbose output)          \n");
fprintf(stdout, "       -vv (superverbose output)\n\n");   

fprintf(stdout, "%s\n%s\n\n",
   "For readout-channel-dependent data-conversion, the fast read-out",
   "direction is assumed to be along rows.");   

DNTP_Stat->I_status = NAMELIST|NO_ARGS; return;

}
				

/* Get namelist filename from command line */
      while( i < argc )	{
          if( argv[i][0] == '-' )	{
              switch( argv[i][1] ) {
              case 'n':      /* -n <input_namelist_fname>  */
                if( ++i >= argc )
                printf("dntoflux_parse_namelist: -n <input_namelist_fname> missing argument\n" );
		else
		if(DNTP_Fnames->CP_Filename_Namelist) /* Filename can be stored */
                I_Got_Namelist = sscanf(argv[i], "%s", DNTP_Fnames->CP_Filename_Namelist); 
                break;
              default:
                ;
              }
            }
          i++;
        }

if (!I_Got_Namelist) {
   printf("dntoflux_parse_namelist: Information only: %s\n",
          "Namelist not specified.");
   return;
}

   /* Open the namelist file to get its length */  

   if ((I_fildes_nl = open(DNTP_Fnames->CP_Filename_Namelist,O_RDONLY)) == -1)
   {
      printf("dntoflux_parse_namelist: Error opening file %s\n",
      DNTP_Fnames->CP_Filename_Namelist);
      DNTP_Stat->I_status = NAMELIST|FOPEN_FAILED; return;
   }

/* Find number of characters in namelist file */
   STATP_nlbuf = &STAT_nlbuf;
   fstat(I_fildes_nl, STATP_nlbuf);
   I_str_length = STATP_nlbuf->st_size;


   #if 0
   /* Open the namelist file */  
   if(strcmp(DNTP_Fnames->CP_Filename_Namelist, "stdin") == 0)	{
   fprintf(stdout, "dntoflux_parse_namelist: Taking namelist from stdin\n");
   fp_namelist = stdin;
   }
   else	{  
   if( (fp_namelist=fopen(DNTP_Fnames->CP_Filename_Namelist,"r"))==NULL) {
    printf("dntoflux_parse_namelist: Error opening file %s",
    DNTP_Fnames->CP_Filename_Namelist);
    DNTP_Stat->I_status = NAMELIST|DNTPEN_FAILED; return;
  }
  }
#endif

/* Associate a stream with the fildes generated above */
  fp_namelist=fdopen(I_fildes_nl,"r");

/* Allocate memory for namelist buffer, */
  CP_buf = (char *) calloc(I_str_length+2, sizeof(char));

/* Read the namelist data */
  /*memset(CP_buf, 0, I_str_length);*/
  if(fread(CP_buf, sizeof(char), I_str_length, fp_namelist) == 0) {
	printf("dntoflux_parse_namelist: Could not read from file %s",
	DNTP_Fnames->CP_Filename_Namelist);
    DNTP_Stat->I_status = NAMELIST|COULD_NOT_READ; return;
  }
  CP_buf[I_str_length] = '/0';
  
/* Close the stream */
  fclose(fp_namelist);

/* Look for the dntoflux section */
#if 0
	for(CP_Pointer = CP_buf;
	strncmp("&DNTOFLUXIN", CP_Pointer, strlen("&DNTOFLUXIN")) != 0;
	CP_Pointer = strchr(CP_Pointer+1, '&'))
	I_SectionsFound++;
#endif
#if 1
CP_Pointer = CP_buf;
while(CP_Pointer <= CP_buf + I_str_length && !I_past_end && !I_found)
     {
     CP_Pointer = strchr(CP_Pointer, '&');
     if (CP_Pointer + strlen("&DNTOFLUXIN") >= CP_buf + I_str_length)
        I_past_end = 1;
     else
        {
        if (strncmp("&DNTOFLUXIN", CP_Pointer, strlen("&DNTOFLUXIN")))
           CP_Pointer += strlen("&");
        else
           I_found = 1;
        }
     }

if (I_past_end)
   {
   DNTP_Stat->I_status = NAMELIST|COULD_NOT_READ; 
   return;
   }
#endif

	
	CP_Pointer += strlen("&DNTOFLUXIN");

/* Match the string values read to the variable definitions and
   write the  values into the appropriate variables. */
	for(CP_Comma = strtok(CP_Pointer, ","); 
	CP_Comma != NULL && strchr(CP_Comma, '&') == 0;
	CP_Comma = strtok(NULL, ","))	{
	  sscanf(CP_Comma, "%s%s%s", CP_NextWord, CP_Equals, CP_Value);
	  if(strcmp(CP_NextWord, "Comment")==0) 
	  strcpy(CP_Comment, CP_Value);
	  if(strcmp(CP_NextWord, "FITS_Image1_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DNTP_Fnames->CP_Filename_FITS_Image1);
	  if(strcmp(CP_NextWord, "FITS_Image2_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DNTP_Fnames->CP_Filename_FITS_Image2);
	  if(strcmp(CP_NextWord, "FITS_Image3_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DNTP_Fnames->CP_Filename_FITS_Image3);
	  if(strcmp(CP_NextWord, "FITS_In_Unc_Filename")==0)
	  sscanf(&CP_Value[1], "%[^']", DNTP_Fnames->CP_Filename_FITS_Image4);
	  if(strcmp(CP_NextWord, "FITS_Out_Filename")==0) {
	     sscanf("!", "%c", 
                DNTP_Fnames->CP_Filename_FITS_Out);  /* clobber */ 
   	     sscanf(&CP_Value[1], "%[^']", 
                &(DNTP_Fnames->CP_Filename_FITS_Out[1]));
	  }
	  if(strcmp(CP_NextWord, "FITS_Out2_Filename")==0) {
	     sscanf("!", "%c", 
                DNTP_Fnames->CP_Filename_FITS_Out2);  /* clobber */ 
	     sscanf(&CP_Value[1], "%[^']", 
                &(DNTP_Fnames->CP_Filename_FITS_Out2[1]));
	  }
	  if(strcmp(CP_NextWord, "FITS_Out_Unc_Filename")==0) {
	     sscanf("!", "%c", 
                DNTP_Fnames->CP_Filename_FITS_Out4);  /* clobber */ 
	     sscanf(&CP_Value[1], "%[^']", 
                &(DNTP_Fnames->CP_Filename_FITS_Out4[1]));
	  }
	  if(strcmp(CP_NextWord, "Log_Filename")==0) 
	     sscanf(&CP_Value[1], "%[^']", DNTP_Fnames->CP_Filename_Log);
	  if(strcmp(CP_NextWord, "Ancillary_File_Path")==0) 
	     sscanf(&CP_Value[1], "%[^']", DNTP_Paths->CP_Ancillary_File_Path);
	  if(strcmp(CP_NextWord, "Data_Plane")==0) 
	     DNTP_FITS->I_Data_Plane = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "NormalizeByExpTime")==0) 
	     DNTP_Const->I_NormalizeByExpTimeFlag = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactor")==0) 
	     DNTP_Const->D_ConversionFactor = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactor1")==0) 
	     DNTP_Const->D_ConversionFactor1 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactor2")==0) 
	     DNTP_Const->D_ConversionFactor2 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactor3")==0) 
	     DNTP_Const->D_ConversionFactor3 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactor4")==0) 
	     DNTP_Const->D_ConversionFactor4 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactorUnc")==0) 
	     DNTP_Const->D_ConversionFactorUnc = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactorUnc1")==0) 
	     DNTP_Const->D_ConversionFactorUnc1 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactorUnc2")==0) 
	     DNTP_Const->D_ConversionFactorUnc2 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactorUnc3")==0) 
	     DNTP_Const->D_ConversionFactorUnc3 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "ConversionFactorUnc4")==0) 
	     DNTP_Const->D_ConversionFactorUnc4 = atof(CP_Value);
	  if(strcmp(CP_NextWord, "Operation")==0) 
	     DNTP_Const->I_Operation = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "PMaskThreshold")==0) 
	     DNTP_Const->I_PMaskThreshold = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "DMaskThreshold")==0) 
	     DNTP_Const->I_DMaskThreshold = atoi(CP_Value);
	  if(strcmp(CP_NextWord, "BunitString")==0) 
	     sprintf(DNTP_Const->Bunit_String, "%s", CP_Value);
	}
  
  free(CP_buf);
   
if(DNTP_Stat->I_Verbose)	
   printf("dntoflux_parse_namelist: Namelist File = \t%s\n",
      DNTP_Fnames->CP_Filename_Namelist);

}





