/* $Id: dntoflux_read_image1.c,v 1.4 2003/04/21 19:51:40 laher Exp $ */


/*********************************************** 
dntoflux_read_image1.c

Purpose

Read the image data from a FITS file and save into an array.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_read_image1(	DNT_Filenames	*DNTP_Fnames,
			DNT_FITSinfo	*DNTP_FITS,
			DNT_Constants	*DNTP_Const,
			double	**DPP_Data_Image1,
			DNT_Status	*DNTP_Stat)
{
int	i, j, k;
int	I_fits_return_status=0, I_ndims_found, I_anynull;
long	LP_naxes[3];
double	D_nullval=0;
fitsfile 	*ffp_FITS_In;
char	CP_Keyname[FLEN_KEYWORD];
char    CP_Comment[FLEN_COMMENT];
double  ExpTime;


if(DNTP_Stat->I_status) return;

if(DNTP_Stat->I_Verbose)	
printf("dntoflux_read_image1: Reading input data from FITS file...\n");

fits_open_file(	&ffp_FITS_In,
		DNTP_Fnames->CP_Filename_FITS_Image1,
		READONLY,
		&I_fits_return_status);

/* Read the keywords that tell the dimensions of the data */
fits_read_keys_lng(	ffp_FITS_In,
			"NAXIS",
			1, 3,
			LP_naxes,
			&I_ndims_found,
			&I_fits_return_status);

if (!(I_ndims_found > 2)) LP_naxes[2] = 1;
	

/* DNTP_FITS->I_Length_X = LP_naxes[0]; */
/* DNTP_FITS->I_Length_Y = LP_naxes[1]; */
/* I_Length_X and I_Length_Y in these two lines should be switched 
   because of the way the image is stored in memory.
*/

DNTP_FITS->I_Length_Y = LP_naxes[0];
DNTP_FITS->I_Length_X = LP_naxes[1];

DNTP_FITS->I_Num_Frames = LP_naxes[2];


/* Optionally read the EXPTIME keyword. */

if (DNTP_Const->I_NormalizeByExpTimeFlag == 1) {

   sprintf(CP_Keyname, "%s", "EXPTIME");
   sprintf(CP_Comment, "%s", "");

   fits_read_key_dbl(ffp_FITS_In,
		CP_Keyname,
		&ExpTime,
		CP_Comment,
		&I_fits_return_status);

   /* 
      If EXPTIME keyword is not present in the FITS header, then set it equal
      to 1 so as to effectively bypass the normalization.
   */

   if (I_fits_return_status) {
      printf("*** dntoflux_read_image1: EXPTIME keyword not found; %s\n", 
         "normalizing by 1 instead...");
      ExpTime = 1.0;
      I_fits_return_status = 0;
   }

   DNTP_FITS->ExpTime = ExpTime;

   if(DNTP_Stat->I_Verbose)	
      printf("dntoflux_read_image1: EXPTIME = %f %s\n", 
      DNTP_FITS->ExpTime,CP_Comment);
}


/* Allocate memory for the data */
/* Don't try if problem with fits file */
if(I_fits_return_status)
{DNTP_Stat->I_status = READ_IMAGE|FITS_MSG_MASK|I_fits_return_status; return;}

*DPP_Data_Image1 = (double *) calloc(LP_naxes[0]*LP_naxes[1]*LP_naxes[2], sizeof(double));
/* Use I_fits_return_status to store status, 
   so if there was a problem with calloc
   subsequent fits functions can detect it */
if(*DPP_Data_Image1==NULL) I_fits_return_status = MALLOC_FAILED;

/* Read the data */
fits_read_img(	ffp_FITS_In,
		TDOUBLE,
		1, LP_naxes[0]*LP_naxes[1]*LP_naxes[2],
		&D_nullval,
		*DPP_Data_Image1,
		&I_anynull,
		&I_fits_return_status);

fits_close_file(ffp_FITS_In,
		&I_fits_return_status);

/* Return the fits status as dntoflux status, with mask to say so */		
if(I_fits_return_status)
DNTP_Stat->I_status = READ_IMAGE|FITS_MSG_MASK|I_fits_return_status;


if(DNTP_Stat->I_Verbose)	{
printf("dntoflux_read_image1: Input FITS file: %s\n",
DNTP_Fnames->CP_Filename_FITS_Image1);
printf("dntoflux_read_image1: I_Length_X = %d\n", DNTP_FITS->I_Length_X);
printf("dntoflux_read_image1: I_Length_Y = %d\n", DNTP_FITS->I_Length_Y);
printf("dntoflux_read_image1: I_Num_Frames = %d\n", DNTP_FITS->I_Num_Frames);
}

	
}
