/* $Id: dntoflux_read_image3.c,v 1.4 2003/04/21 19:51:40 laher Exp $ */


/*********************************************** 
dntoflux_read_image3.c

Purpose

Read the DMask data from a FITS file and save into an array.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "dntoflux_errcodes.h"
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "dntoflux.h"
#include "dntoflux_defs.h"


dntoflux_read_image3(	DNT_Filenames	*DNTP_Fnames,
			DNT_FITSinfo	*DNTP_FITS,
			double	**DPP_Data_Image3,
			DNT_Status	*DNTP_Stat)
{
   int	i, j, k;
   int	I_fits_return_status=0, I_ndims_found, I_anynull;
   long	LP_naxes[3], I_Index;
   double	D_nullval=0;
   fitsfile 	*ffp_FITS_In;
   long	bitpix;
   double  datashift;
   int I_Length_X, I_Length_Y, I_Num_Frames;
   char	CP_Keyname[FLEN_KEYWORD];
   char CP_Comment[FLEN_COMMENT];

   if (DNTP_Stat->I_status) return;

   if (DNTP_Stat->I_Verbose)	
      printf("dntoflux_read_image3: Reading input data from FITS file...\n");

   fits_open_file(&ffp_FITS_In,
		  DNTP_Fnames->CP_Filename_FITS_Image3,
		  READONLY,
		  &I_fits_return_status);

   /* Read the keywords that tell the dimensions of the data */

   fits_read_keys_lng(	ffp_FITS_In,
			"NAXIS",
			1, 3,
			LP_naxes,
			&I_ndims_found,
			&I_fits_return_status);

   if (!(I_ndims_found > 2)) LP_naxes[2] = 1;
	

   /* Don't try if problem with fits file */

   if (I_fits_return_status) {
      DNTP_Stat->I_status = READ_DMASK_IMAGE|FITS_MSG_MASK|I_fits_return_status; 
      return;
   }

   /* DNTP_FITS->I_Length_X = LP_naxes[0]; */
   /* DNTP_FITS->I_Length_Y = LP_naxes[1]; */
   /*    I_Length_X and I_Length_Y in these two lines should be switched 
      because of the way the image is stored in memory.
   */

   I_Length_Y = LP_naxes[0];
   I_Length_X = LP_naxes[1];
   I_Num_Frames = LP_naxes[2];


   /* Require input image and mask to be the same size. */

   if ((DNTP_FITS->I_Length_Y != I_Length_Y) ||
       (DNTP_FITS->I_Length_X != I_Length_X) ||
       (DNTP_FITS->I_Num_Frames != I_Num_Frames)) {
      DNTP_Stat->I_status = READ_DMASK_IMAGE|INCONSISTENT_IMAGE_SIZES; 
      return;
   }


   /* Get bitpix for proper conversion to unsigned values. */

   sprintf(CP_Keyname, "%s", "BITPIX  ");

   fits_read_key_lng(ffp_FITS_In,
		CP_Keyname,
		&bitpix,
		CP_Comment,
		&I_fits_return_status);


   /* Allocate input image memory. */

   *DPP_Data_Image3 = (double *) calloc(LP_naxes[0]*LP_naxes[1]*LP_naxes[2], 
      sizeof(double));


   /* Use I_fits_return_status to store status, 
      so if there was a problem with calloc
      subsequent fits functions can detect it */

   if (*DPP_Data_Image3==NULL) I_fits_return_status = MALLOC_FAILED;


   /* Read the data */

   fits_read_img(ffp_FITS_In,
                 TDOUBLE,
		 1, LP_naxes[0]*LP_naxes[1]*LP_naxes[2],
		 &D_nullval,
		 *DPP_Data_Image3,
		 &I_anynull,
		 &I_fits_return_status);

   fits_close_file(ffp_FITS_In,
	  	&I_fits_return_status);



   /* Convert from signed to unsigned values. */

   if ((bitpix == 8) || (bitpix == 16) || (bitpix == 32) ) {

      datashift =  (double) (2 << (bitpix - 1));

      for (k = 0; k < I_Num_Frames; k++)
         for (i = 0; i < I_Length_X; i++)
 	    for (j = 0; j < I_Length_Y; j++) {
               I_Index = k * I_Length_X * I_Length_Y +  i * I_Length_Y + j;
               if (*(*DPP_Data_Image3 + I_Index) < 0.0) 
                  *(*DPP_Data_Image3 + I_Index) = 
                     *(*DPP_Data_Image3 + I_Index) + datashift;
      }

   }



   /* Return the fits status as dntoflux status, with mask to say so */		
   if (I_fits_return_status)
      DNTP_Stat->I_status = READ_DMASK_IMAGE|FITS_MSG_MASK|I_fits_return_status;

   if (DNTP_Stat->I_Verbose) {
      printf("dntoflux_read_image3: Input FITS file: %s\n",
         DNTP_Fnames->CP_Filename_FITS_Image3);
      printf("dntoflux_read_image3: I_Length_X = %d\n", I_Length_X);
      printf("dntoflux_read_image3: I_Length_Y = %d\n", I_Length_Y);
      printf("dntoflux_read_image3: I_Num_Frames = %d\n", 
         I_Num_Frames);
      printf("dntoflux_read_image3: bitpix = %d\n", 
         bitpix);
   }

	
}









