/* $Id: droopop_compute.c,v 1.17 2006/12/01 20:42:02 laher Exp $ */


/*********************************************** 
droopop_compute.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/* #include <ieeefp.h> */
#include <math.h>
#include "droopop.h"
#include "droopop_errcodes.h"
#include "droopop_defs.h"

droopop_compute(	
            Filenames      *Fnames,
            Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
            float          *FP_Vars_In,
            float          *FP_Refs_In,
            float          *FP_Vars_Refs_In,
            float          **Drp_Unc,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask) {
 int	 i, j, m;
 int     itemp;
 int     I_fits_return_status=0;
 int     satmask;
 float   dcouple, dcouple_err;
 double  *sumimg, *sumsqv;
 double  dterm1, dterm2, dterm3, r_error;

 if(Stat->I_status) return;

 dcouple = Const->Droop;
 dcouple_err = Const->Droop_Error;
 r_error = dcouple_err/(Dim_In->I_NX*Dim_In->I_NY - 1.0);

 printf("Start droop removal using coupling constant %lf\n", dcouple);

 sumimg = (double *) calloc(Dim_In->I_NZ, sizeof(double));
 sumsqv = (double *) calloc(Dim_In->I_NZ, sizeof(double));
 *Drp_Unc = (float *) calloc(Dim_In->I_NZ, sizeof(float));
 if (sumimg == NULL || sumsqv == NULL || *Drp_Unc == NULL) {
    printf("droopop_compute: memory allocation failed\n");
    Stat->I_status = COMPUTE|MALLOC_FAILED;
 }

 /* calculate total singals in array that contribute to droop */

 for (j=0; j < Dim_In->I_NZ; ++j) {
    for (i=0; i<Dim_In->I_NX*Dim_In->I_NY; ++i) {
       if ( (*(SP_dmask + j*Dim_In->I_NX*Dim_In->I_NY + i) & Const->DmaskFatal) == 0 &&
            (*(SP_pmask + i) & Const->PmaskFatal) == 0 &&
            (*(SP_bmask + i) & Const->BmaskFatal) == 0 &&
            (*(SP_cmask + i) & Const->CmaskFatal) == 0 ) {
          if (strcmp(Fnames->CP_Filename_Refs_In, "")) {
             if (Const->OnePlaneDroop) {
                if( !(iznand((double) *(FP_Refs_In + i))) ) {
                   *(sumimg+j) += *(FP_Refs_In + i);
                } else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at first data plane pixel %d\n", i+1);
                }
                if( !(iznand((double) *(FP_Vars_Refs_In + i))) ) {
                   *(sumsqv+j) += (*(FP_Vars_Refs_In + i))*(*(FP_Vars_Refs_In + i));
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at first uncertainty plane pixel %d\n", i+1);
                }
             } else {
                if( !(iznand((double) *(FP_Refs_In + j*Dim_In->I_NX*Dim_In->I_NY + i))) ) {
                   *(sumimg+j) += *(FP_Refs_In + j*Dim_In->I_NX*Dim_In->I_NY + i);
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at data plane %d pixel %d\n", j+1, i+1);
                }

                if( !(iznand((double) *(FP_Vars_Refs_In + j*Dim_In->I_NX*Dim_In->I_NY + i))) ) {
                   *(sumsqv+j) += (*(FP_Vars_Refs_In + j*Dim_In->I_NX*Dim_In->I_NY + i))*(*(FP_Vars_Refs_In + j*Dim_In->I_NX*Dim_In->I_NY + i));
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at uncertainty plane %d pixel %d\n", j+1, i+1);
                }
             }
          }
          else {
             if (Const->OnePlaneDroop) {
                if( !(iznand((double) *(FP_Data_In + i))) ) {
                   *(sumimg+j) += *(FP_Data_In + i);
                } else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at first data plane pixel %d\n", i+1);
                }
                if( !(iznand((double) *(FP_Vars_In + i))) ) {
                   *(sumsqv+j) += (*(FP_Vars_In + i))*(*(FP_Vars_In + i));
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at first uncertainty plane pixel %d\n", i+1);
                }
             } else {
                if( !(iznand((double) *(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i))) ) {
                   *(sumimg+j) += *(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i);
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at data plane %d pixel %d\n", j+1, i+1);
                }

                if( !(iznand((double) *(FP_Vars_In + j*Dim_In->I_NX*Dim_In->I_NY + i))) ) {
                   *(sumsqv+j) += (*(FP_Vars_In + j*Dim_In->I_NX*Dim_In->I_NY + i))*(*(FP_Vars_In + j*Dim_In->I_NX*Dim_In->I_NY + i));
                }
                else {
                   if (Stat->I_Verbose)
                      printf("droopop_compute: NaN encountered at uncertainty plane %d pixel %d\n", j+1, i+1);
                }
             }
          }
          if (( SP_dmask [ j*Dim_In->I_NX*Dim_In->I_NY + i] & ( 8192|16384))!=0 ) {    /* saturated pixel or missing data */
/*             printf("Warning: uncorrectable saturated pixel or missing data encountered at layer %d pixel %d; droop correction may be inaccurate\n", j+1, i+1);
*/
             if ( (*(SP_dmask + j*Dim_In->I_NX*Dim_In->I_NY + i) & 64) == 64 )
                printf("Warning: bit 6 already set in dmask\n");
             else
                *(SP_dmask + j*Dim_In->I_NX*Dim_In->I_NY + i) |= 64; 
          }   
       }
    }
 }

 /* perform droop effect removal */

 dterm1 = dcouple/((1.0+dcouple)*(Dim_In->I_NX*Dim_In->I_NY - 1.0));
 dterm2 = 1.0/(1.0 - dcouple/(Dim_In->I_NX*Dim_In->I_NY - 1.0));
 dterm3 = 1.0/((1.0+dcouple)*(1.0+dcouple));

 for (j=0; j < Dim_In->I_NZ; ++j) {
    for (i=0; i<Dim_In->I_NX*Dim_In->I_NY; ++i) {
       if ( (*(SP_dmask + j*Dim_In->I_NX*Dim_In->I_NY + i) & Const->DmaskFatal) == 0 &&
            (*(SP_pmask + i) & Const->PmaskFatal) == 0 &&
            (*(SP_bmask + i) & Const->BmaskFatal) == 0 &&
            (*(SP_cmask + i) & Const->CmaskFatal) == 0 ) {
          if ( !(iznand((double) *(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i))) ) {
             *(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i) = (*(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i) -  (*(sumimg + j))*dterm1)*dterm2;
          }
       }
    }
/*    *(*Drp_Unc + j) = dterm2*dterm1*dterm2*dterm1*(*(sumsqv + j)) + r_error*r_error*(*(sumimg + j))*(*(sumimg + j))*dterm3*dterm3;
*/
    *(*Drp_Unc + j) = (dterm2*dterm1*dterm2*dterm1*(*(sumsqv + j)) + r_error*r_error*(*(sumimg + j))*(*(sumimg + j))*dterm3*dterm3)/(Dim_In->I_NX*Dim_In->I_NY-1.0);
 }

 /* Compute QA parameters if warranted */

 if (Const->QAheader) {
    Const->Drsumplf = (*(sumimg))*dterm1;
    Const->Drsumpll = (*(sumimg + Dim_In->I_NZ - 1))*dterm1;
 }

 /* if requested, reset values for pixels that have been corrected for saturation */

 if (Const->Resat) {
    satmask = pow(2, Const->SatBit);
    for (j=0; j < Dim_In->I_NZ; ++j) {
       for (i=0; i<Dim_In->I_NX*Dim_In->I_NY; ++i) {
          if ( (*(SP_dmask + j*Dim_In->I_NX*Dim_In->I_NY + i) & satmask) != 0) {
             *(FP_Data_In + j*Dim_In->I_NX*Dim_In->I_NY + i) = Const->SatVal;
          }
       }
    }
 }

 free(sumimg);
 free(sumsqv);

 printf("droopop_compute: Numerical Computations finished.\n");

}
