/* $Id: droopop_parse_args.c,v 1.12 2005/12/07 00:10:57 fan Exp $ */


/*********************************************** 
droopop_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output Probabilities File Name.
CP_Filename_Pmask = Input Pmask.
CP_Filename_Dmask = Input Dmask.
CP_Filename_Bmask = Input Bmask.
CP_Filename_Cmask = Input Cmask.
I_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include <string.h>
#include "droopop.h"
#include "droopop_errcodes.h"
#include "droopop_defs.h"


droopop_parse_args(	
            int	        argc,
            char        **argv,
            Constants   *Const,
            Filenames   *Fnames,
            Status	*Stat) {

   int	i=1;

   int     I_out_chk, I_in_chk;
   int     I_droop_chk, I_nl_chk;
   int     I_pair_err, I_sat_chk;

   /* flags to check the completeness of the switch-value pairs */
   I_in_chk = 1;
   I_out_chk = 1;
   I_droop_chk = 1;
   I_nl_chk = 1;
   I_sat_chk = 0;
   I_pair_err = 0;

   while( i < argc ) {

     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               I_nl_chk =0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'i':      /* -i <input_image_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
	       else {
                  I_in_chk = 0;
                  sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
               }
            }
            else {       /* -i2 <input uncertainty image> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Vars_In);
               }
            }
            break;
          case 'o':      /* -o <output_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  I_out_chk = 0;
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
               }
            }
            else {       /* -o2 <output uncertainty image> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out); /* clobber */
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_Vars_Out[1]));
               }
            }
            break;
          case 'j':      /* -j <input_reference_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i],"%s", Fnames->CP_Filename_Refs_In);
               }
            }
            else {       /* -j2 <input reference uncertainty image> */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i],"%s", Fnames->CP_Filename_Vars_Refs_In);
               }
            }
            break;
          case 'r':      /* -r <droop coupling constant>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               I_droop_chk = 0;
               sscanf(argv[i],"%f", &(Const->Droop));
            }
            break;
          case 'e':      /* -e <droop constant uncertainty */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i],"%f", &(Const->Droop_Error));
            }
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            break;
          case 'd':      /* -d <dmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
            break;
          case 'b':      /* -b <bmask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Bmask);
            break;
          case 'c':      /* -c <cmask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p')           /* -fp for Pmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->PmaskFatal));
                  }
               }
            else if (argv[i][2] == 'd')      /* -fd for Dmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->DmaskFatal));
                  }
               }
            else if (argv[i][2] == 'c')      /* -fc for Cmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->CmaskFatal));
                  }
               }
            else if (argv[i][2] == 'b')      /* -fb for Bmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->BmaskFatal));
                  }
               }
            else
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  printf("darkbase_parse_args: unknown argument\n");
               }
            break;
          case 's':      /* -s <saturation-related variables> */
            I_sat_chk = 1;
            if (argv[i][2] == 'b')           /* -sb for saturation bit in dmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->SatBit));
                  }
               }
            else if (argv[i][2] == 'v')      /* -sv for saturation value */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%f", &(Const->SatVal));
                  }
               }
            break;
          case 'a':      /* -a  reset saturation value mode */
            Const->Resat = 1;
            break;
          case 'q':      /* -q  QA header mode */
            Const->QAheader = 1;
            break;
          case 'g':      /* -g  Single-plane droop estimation mode */
            Const->OnePlaneDroop = 1;
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;
            break;
          default:
            printf("droopop_parse_args: unknown argument\n");
       }
     }
     else {
       I_pair_err = 1;
     }
     i++;
   }

   if ( I_pair_err ) {
      printf("droopop_parse_args: command line syntax error\n");
      droopop_usage();
      Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   }


   if ( (I_in_chk || I_droop_chk || I_out_chk) && I_nl_chk ) {
      printf("droopop_parse_args: Error in command line arguments; required input not complete.\n\n");
      droopop_usage();
      Stat->I_status = PARSE_ARGS|REQ_ARGS_INCOMP;
   }
 
   if(strcmp(Fnames->CP_Filename_Vars_In, "")) Const->In_Vars = 1;
   if(Stat->I_Verbose)	{
      printf("droopop_parse_args: Input Namelist File = \t%s\n", Fnames->CP_Filename_Namelist);
      printf("droopop_parse_args: Input Image File = \t%s\n", Fnames->CP_Filename_FITS_In);
      printf("droopop_parse_args: Input Uncertainty Image File = \t%s\n", Fnames->CP_Filename_Vars_In);
      printf("droopop_parse_args: Input Reference Image File = \t%s\n", Fnames->CP_Filename_Refs_In);
      printf("droopop_parse_args: Input Reference Uncertainty Image File = \t%s\n", Fnames->CP_Filename_Vars_Refs_In);
      printf("droopop_parse_args: Ouput Image File = \t%s\n", Fnames->CP_Filename_FITS_Out);
      printf("droopop_parse_args: Ouput Uncertainty Image File = \t%s\n", Fnames->CP_Filename_Vars_Out);
      printf("droopop_parse_args: Input Droop Constant = \t%f\n", Const->Droop);
      printf("droopop_parse_args: Input Droop Constant Uncertainty = \t%f\n", Const->Droop_Error);
      printf("droopop_parse_args: Saturation bit in dmask = \t%d\n", Const->SatBit);
      printf("droopop_parse_args: Saturation value = \t%f\n", Const->SatVal);
      printf("droopop_parse_args: Pmask File = \t%s\n", Fnames->CP_Filename_Pmask);
      printf("droopop_parse_args: Dmask File = \t%s\n", Fnames->CP_Filename_Dmask);
      printf("droopop_parse_args: Bmask File = \t%s\n", Fnames->CP_Filename_Bmask);
      printf("droopop_parse_args: Cmask File = \t%s\n", Fnames->CP_Filename_Cmask);
      printf("droopop_parse_args: Pmask Fatal Template = \t%d\n", Const->PmaskFatal);
      printf("droopop_parse_args: Dmask Fatal Template = \t%d\n", Const->DmaskFatal);
      printf("droopop_parse_args: Bmask Fatal Template = \t%d\n", Const->BmaskFatal);
      printf("droopop_parse_args: Cmask Fatal Template = \t%d\n", Const->CmaskFatal);
      printf("droopop_parse_args: Reset Saturation Mode = \t%d\n", Const->Resat);
      printf("droopop_parse_args: QA Header Writer Mode = \t%d\n", Const->QAheader);
      printf("droopop_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
   }
}
