/* $Id: droopop_parse_namelist.c,v 1.12 2005/12/07 00:10:57 fan Exp $ */


/***********************************************
droopop_parse_namelist.c

Purpose
Read and parse a parameter file to get input and output file
names constants, and processing flags.

************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include "droopop.h"
#include "droopop_errcodes.h"
#include "droopop_defs.h"


droopop_parse_namelist(  int          argc,
                         char         **argv,
                         Constants    *Const,
                         Filenames    *Fnames,
                         Status       *Stat)
{


  FILE  *fp_namelist;
  char  *CP_buf;
  char  CP_NextWord[STRING_BUFFER_SIZE];
  char  CP_Equals[STRING_BUFFER_SIZE];
  char  CP_Value[STRING_BUFFER_SIZE];
  char  CP_Comment[STRING_BUFFER_SIZE];
  char  *CP_Pointer, *CP_Comma;

  int   i=1, I_Got_Namelist=0, I_SectionsFound=0;
  int   I_fildes_nl=0, I_str_length=0;
  int   I_past_end=0, I_found=0;

  struct stat  STAT_nlbuf, *STATP_nlbuf;

  if ( argc==1 )
  {
      Stat->I_status = NAMELIST|NO_ARGS;
      return;
  }


  /* Get namelist filename from command line */
  while( i < argc )
  {
    if ( argv[i][0] == '-' )
    {
       switch( argv[i][1] )
       {
          case 'n':      /* -n <input_namelist_fname>  */
            if( ++i >= argc )
              printf("droopop_parse_namelist: -n <input_namelist_fname> missing argument\n" );
            else
              I_Got_Namelist = sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            break;
          default:
            ;
       }
    }
    i++;
  }

  if (!I_Got_Namelist)
  {
     printf("droopop_parse_namelist: Information only: %s\n", "Namelist not specified.");
     return;
  }

  /* Open the namelist file to get its length */
  if ((I_fildes_nl = open(Fnames->CP_Filename_Namelist,O_RDONLY)) == -1)
  {
      printf("droopop_parse_namelist: Error opening file %s\n", Fnames->CP_Filename_Namelist);
      Stat->I_status = NAMELIST|FOPEN_FAILED;
      return;
  }

  /* Find number of characters in namelist file */
  STATP_nlbuf = &STAT_nlbuf;
  fstat(I_fildes_nl, STATP_nlbuf);
  I_str_length = STATP_nlbuf->st_size;

  /* Associate a stream with the fildes generated above */
  fp_namelist=fdopen(I_fildes_nl,"r");

  /* Allocate memory for namelist buffer, */
  CP_buf = (char *) calloc(I_str_length+2, sizeof(char));

  /* Read the namelist data */
  if(fread(CP_buf, sizeof(char), I_str_length, fp_namelist) == 0)
  {
      printf("droopop_parse_namelist: Could not read from file %s", Fnames->CP_Filename_Namelist);
      Stat->I_status = NAMELIST|COULDNOTREAD;
      return;
  }
  CP_buf[I_str_length] = '\0';
 
  /* Close the stream */
  fclose(fp_namelist);

  /* Look for the droopop section */
  CP_Pointer = CP_buf;
  while(CP_Pointer <= CP_buf + I_str_length && !I_past_end && !I_found)
  {
     CP_Pointer = strchr(CP_Pointer, '&');
     if (CP_Pointer + strlen("&DROOPOPIN") >= CP_buf + I_str_length)
        I_past_end = 1;
     else
     {
        if (strncmp("&DROOPOPIN", CP_Pointer, strlen("&DROOPOPIN")))
           CP_Pointer += strlen("&");
        else
           I_found = 1;
        }
     }

  if (I_past_end)
  {
     Stat->I_status = NAMELIST|COULDNOTREAD;
     return;
  }

  CP_Pointer += strlen("&DROOPOPIN");

  /* Match the string values read to the variable definitions and
     write the  values into the appropriate variables. */
  for(CP_Comma = strtok(CP_Pointer, ","); CP_Comma != NULL && strchr(CP_Comma, '&') == 0; CP_Comma = strtok(NULL, ","))
  {
     sscanf(CP_Comma, "%s%s%s", CP_NextWord, CP_Equals, CP_Value);
     if(strcmp(CP_NextWord, "Comment")==0)
          strcpy(CP_Comment, CP_Value);

     if(strcmp(CP_NextWord, "FITS_Pmask")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Pmask);

     if(strcmp(CP_NextWord, "FITS_Dmask")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Dmask);

     if(strcmp(CP_NextWord, "FITS_Bmask")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Bmask);

     if(strcmp(CP_NextWord, "FITS_Cmask")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Cmask);

     if(strcmp(CP_NextWord, "PMASK_FATAL")==0)      /* Pmask fatal pixel template */
          Const->PmaskFatal = atoi(CP_Value);
     if(strcmp(CP_NextWord, "DMASK_FATAL")==0)      /* Dmask fatal pixel template */
          Const->DmaskFatal = atoi(CP_Value);
     if(strcmp(CP_NextWord, "CMASK_FATAL")==0)      /* Cmask fatal pixel template */
          Const->CmaskFatal = atoi(CP_Value);
     if(strcmp(CP_NextWord, "BMASK_FATAL")==0)      /* Bmask fatal pixel template */
          Const->BmaskFatal = atoi(CP_Value);

     if(strcmp(CP_NextWord, "FITS_Image_In")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_FITS_In);

     if(strcmp(CP_NextWord, "FITS_Error_In")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Vars_In);

     if(strcmp(CP_NextWord, "FITS_Refs_In")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Refs_In);

     if(strcmp(CP_NextWord, "FITS_Error_Refs_In")==0)
          sscanf(&CP_Value[1], "%[^']", Fnames->CP_Filename_Vars_Refs_In);

     if(strcmp(CP_NextWord, "FITS_Image_Out")==0)
     {
          sscanf("!", "%c", Fnames->CP_Filename_FITS_Out);  /* clobber */
          sscanf(&CP_Value[1], "%[^']", &(Fnames->CP_Filename_FITS_Out[1]));
     }

     if(strcmp(CP_NextWord, "FITS_Error_Out")==0)
     {
          sscanf("!", "%c", Fnames->CP_Filename_Vars_Out);  /* clobber */
          sscanf(&CP_Value[1], "%[^']", &(Fnames->CP_Filename_Vars_Out[1]));
     }

     if(strcmp(CP_NextWord, "Droop_Coef")==0)
          Const->Droop = atof(CP_Value);

     if(strcmp(CP_NextWord, "Droop_Error")==0)
          Const->Droop_Error = atof(CP_Value);

     if(strcmp(CP_NextWord, "Resat_Mode")==0) 
          Const->Resat = atoi(CP_Value);

     if(strcmp(CP_NextWord, "Saturation_Bit")==0)
          Const->SatBit = atoi(CP_Value);

     if(strcmp(CP_NextWord, "Saturation_Val")==0)
          Const->SatVal = atof(CP_Value);

     if(strcmp(CP_NextWord, "QA_Header_Mode")==0)
          Const->QAheader = atoi(CP_Value);

     if(strcmp(CP_NextWord, "First_Plane_Droop")==0)
          Const->OnePlaneDroop = atoi(CP_Value);

  }

  free(CP_buf);
  
  if(Stat->I_Verbose)  
     printf("droopop_parse_namelist: Namelist File = \t%s\n", Fnames->CP_Filename_Namelist);

}
