/* $Id: droopres_compute.c,v 1.12 2004/05/13 19:11:54 fan Exp $ */


/*********************************************** 
droopres_compute.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "droopres.h"
#include "droopres_errcodes.h"
#include "droopres_defs.h"

droopres_compute(	
            Filenames      *Fnames,
            Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
            float          *FP_Vars_In,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_cmask,
            short          *SP_bmask,
            short          *SP_umask) {
 int	 i, j, l, layer_num;
 int     trimfactflag1, trimfactflag2, trimfactflag3, trimfactflag4;
 int     do_chan1, do_chan2, do_chan3, do_chan4;
 long    nchan1, nchan2, nchan3, nchan4;
 long    nstart=1;
 double  pixvalue, f1, f2;
 double  *chan1, *chan2, *chan3, *chan4;
 double  trim_mean_chan1, trim_mean_chan2, trim_mean_chan3, trim_mean_chan4;
 double  stddev_chan1, stddev_chan2, stddev_chan3, stddev_chan4;
 double  datascale_chan1, datascale_chan2, datascale_chan3, datascale_chan4;
 double  facttotptr1, facttotptr2, facttotptr3, facttotptr4;

 if(Stat->I_status) return;

 do_chan1 = 1;
 do_chan2 = 1;
 do_chan3 = 1;
 do_chan4 = 1;

 f1 = Const->Trim_Frac;
 f2 = Const->Trim_STD_Frac;

 if(Const->Trim_Frac == 0.0)
   printf("droopres_compute: extreme trim on data; calculating median value\n");
 if(Const->Trim_Frac == 1.0)
   printf("droopres_compute: no trim on data; calculating average\n");
 
 chan1 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan2 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan3 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 chan4 = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
 if (chan1 == NULL || chan2 == NULL || chan3 == NULL || chan4 == NULL) {
    printf("droopres_compute: memory allocation failed\n");
    Stat->I_status = COMPUTE|MALLOC_FAILED;
    return;
 }
 
 for (l=0; l<Dim_In->I_NZ; ++l)   /* loop thru all layers */
 {
 
 layer_num = l+1;

 /* obtain channel-dependent pixel values in defined box */
 nchan1 = nchan2 = nchan3 = nchan4 = 0;
 trim_mean_chan1 = trim_mean_chan2 = trim_mean_chan3 = trim_mean_chan4 = 0.0;
 stddev_chan1 = stddev_chan2 = stddev_chan3 = stddev_chan4 = 0.0;
 datascale_chan1 = datascale_chan2 = datascale_chan3 = datascale_chan4 = 0.0;
 for (i=0; i<Dim_In->I_NY; ++i) {
    for (j=0; j<Dim_In->I_NX; ++j) {
       if ( *(SP_umask + j + i*Dim_In->I_NX) == 1 &&
            ((*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0) &&
            ((*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0) &&
            ((*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0) &&
            ((*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0) ) {
           if (j%4 == 0) {           /* channel 1 */
              pixvalue = (double) *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY);
              if (!(iznand(pixvalue))) {
                 chan1[nchan1] = pixvalue;
                 nchan1++;
              }
           }
           else if (j%4 == 1) {      /* channel 2 */
              pixvalue = (double) *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY);
              if (!(iznand(pixvalue))) {
                 chan2[nchan2] = pixvalue;
                 nchan2++;
              }
           }
           else if (j%4 == 2) {      /* channel 3 */
              pixvalue = (double) *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY);
              if (!(iznand(pixvalue))) {
                 chan3[nchan3] = pixvalue;
                 nchan3++;
              }
           }
           else if (j%4 == 3) {      /* channel 4 */
              pixvalue = (double) *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY);
              if (!(iznand(pixvalue))) {
                 chan4[nchan4] = pixvalue;
                 nchan4++;
              }
           }
       }
    }
 }

 if(Stat->I_Verbose) {
    printf("%d data points used for channel 1 and layer %d\n", nchan1, layer_num);
    printf("%d data points used for channel 2 and layer %d\n", nchan2, layer_num);
    printf("%d data points used for channel 3 and layer %d\n", nchan3, layer_num);
    printf("%d data points used for channel 4 and layer %d\n\n", nchan4, layer_num);
 }

/* if (nchan1 == 0 && nchan2 == 0 && nchan3 == 0 && nchan4 == 0) {
    
    Stat->I_status = COMPUTE|NO_DATA;
    printf("droopres_compute: no data to estimate residual droop\n");
    return;
*/
 if (nchan1 == 0) {
    do_chan1 = 0;
    trim_mean_chan1 = 0.0;
    stddev_chan1 = 0.0;
    datascale_chan1 = 0.0;
    printf("droopres_compute: read channel 1 has no data to compute residual droop\n");
 } 

 if (nchan2 == 0) {
    do_chan2 = 0;
    trim_mean_chan2 = 0.0;
    stddev_chan2 = 0.0;
    datascale_chan2 = 0.0;
    printf("droopres_compute: read channel 2 has no data to compute residual droop\n");
 } 

 if (nchan3 == 0) {
    do_chan3 = 0;
    trim_mean_chan3 = 0.0;
    stddev_chan3 = 0.0;
    datascale_chan3 = 0.0;
    printf("droopres_compute: read channel 3 has no data to compute residual droop\n");
 } 

 if (nchan4 == 0) {
    do_chan4 = 0;
    trim_mean_chan4 = 0.0;
    stddev_chan4 = 0.0;
    datascale_chan4 = 0.0;
    printf("droopres_compute: read channel 4 has no data to compute residual droop\n");
 }


 /* obtain median value and error for each channel */
 if (do_chan1) {
    trimfactflag1 = 0;
    trimmedstatistics(chan1, nstart, nchan1, f1, f2, &trimfactflag1, &facttotptr1, &trim_mean_chan1, &stddev_chan1, &datascale_chan1);
 }

 if (do_chan2) {
    trimfactflag2 = 0;
    trimmedstatistics(chan2, nstart, nchan2, f1, f2, &trimfactflag2, &facttotptr2, &trim_mean_chan2, &stddev_chan2, &datascale_chan2);
 }

 if (do_chan3) {
    trimfactflag3 = 0;
    trimmedstatistics(chan3, nstart, nchan3, f1, f2, &trimfactflag3, &facttotptr3, &trim_mean_chan3, &stddev_chan3, &datascale_chan3);
 }

 if (do_chan4) {
    trimfactflag4 = 0;
    trimmedstatistics(chan4, nstart, nchan4, f1, f2, &trimfactflag4, &facttotptr4, &trim_mean_chan4, &stddev_chan4, &datascale_chan4);
 }

 if (Stat->I_Verbose)
 {

 printf("Found for %d data points in channel 1 and layer %d using symmetrical trim fraction %f: trim-average %lf, trimmed standard deviation %lf, and datascale %lf\n", nchan1, layer_num, Const->Trim_Frac, trim_mean_chan1, stddev_chan1, datascale_chan1);
 printf("Found for %d data points in channel 2 and layer %d using symmetrical trim fraction %f: trim-average %lf, trimmed standard deviation %lf, and datascale %lf\n", nchan2, layer_num, Const->Trim_Frac, trim_mean_chan2, stddev_chan2, datascale_chan2);
 printf("Found for %d data points in channel 3 and layer %d using symmetrical trim fraction %f: trim-average %lf, trimmed standard deviation %lf, and datascale %lf\n", nchan3, layer_num, Const->Trim_Frac, trim_mean_chan3, stddev_chan3, datascale_chan3);
 printf("Found for %d data points in channel 4 and layer %d using symmetrical trim fraction %f: trim-average %lf, trimmed standard deviation %lf, and datascale %lf\n", nchan4, layer_num, Const->Trim_Frac, trim_mean_chan4, stddev_chan4, datascale_chan4);

 }

 /* record residual droop levels for layer 1 only */

 if (l == 0) {
    Const->Drpres_chan1 = trim_mean_chan1;
    Const->Drpres_chan2 = trim_mean_chan2;
    Const->Drpres_chan3 = trim_mean_chan3;
    Const->Drpres_chan4 = trim_mean_chan4;
    Const->Drpresunc_chan1 = stddev_chan1;
    Const->Drpresunc_chan2 = stddev_chan2;
    Const->Drpresunc_chan3 = stddev_chan3;
    Const->Drpresunc_chan4 = stddev_chan4;
 }

 /* subtract channel-dependent pixel median values from all pixels */
 for (j=0; j < Dim_In->I_NX; ++j) {
    for (i=0; i<Dim_In->I_NY; ++i) {
        if (j%4 == 0) {
          if ( (*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
               (*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
               (*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0 &&
               (*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0 ) {
             *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= trim_mean_chan1;
             *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                    (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + stddev_chan1*stddev_chan1));
          }
        }
        else if (j%4 == 1) {
          if ( (*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
               (*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
               (*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0 &&
               (*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0 ) {
             *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= trim_mean_chan2;
             *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                    (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + stddev_chan2*stddev_chan2));
          }
        }
        else if (j%4 == 2) {
          if ( (*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
               (*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
               (*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0 &&
               (*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0 ) {
             *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= trim_mean_chan3;
             *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                    (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + stddev_chan3*stddev_chan3));
          }
        }
        else if (j%4 == 3) {
          if ( (*(SP_dmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
               (*(SP_pmask + j + i*Dim_In->I_NX) & Const->PmaskFatal) == 0 && 
               (*(SP_bmask + j + i*Dim_In->I_NX) & Const->BmaskFatal) == 0 &&
               (*(SP_cmask + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) & Const->CmaskFatal) == 0 ) {
             *(FP_Data_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) -= trim_mean_chan4;
             *(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY) = sqrt((double)(
                    (*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY))*(*(FP_Vars_In + j + i*Dim_In->I_NX + l*Dim_In->I_NX*Dim_In->I_NY)) + stddev_chan4*stddev_chan4));
          }
        }
    }
 }

 /* clean-up for next in loop */

 for (i=0; i<nchan1; ++i) chan1[i] = 0.0;
 for (i=0; i<nchan2; ++i) chan2[i] = 0.0;
 for (i=0; i<nchan3; ++i) chan3[i] = 0.0;
 for (i=0; i<nchan4; ++i) chan4[i] = 0.0;

 }

 free(chan1);
 free(chan2);
 free(chan3);
 free(chan4);

 printf("droopres_compute: Numerical Computations finished.\n");

}
