/* $Id: droopres_error_handler.c,v 1.8 2002/09/04 22:07:33 fan Exp $ */


/*********************************************** 
droopres_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of droopres functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_droopres_message = Code of error that occurred.
I_droopres_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "droopres.h"
#include "droopres_errcodes.h"
#include "droopres_defs.h"


droopres_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_droopres_message;
int	I_droopres_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_droopres_message = RIGHT_MASK&(Stat->I_status);
I_droopres_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_droopres_message&FITS_MSG_MASK)	{
I_droopres_message -= FITS_MSG_MASK;
}

droopres_decode_message(I_droopres_message, CP_Message);
droopres_decode_fcnname(I_droopres_root, CP_Function);

if (Stat->I_status)
{
printf("droopres Exception Message      0x%04x: %s.\n", I_droopres_message, CP_Message);
printf("droopres Exception in Function  0x%04x: %s.\n", I_droopres_root, CP_Function);
}


I_Time = clock();
printf("Processor Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("droopres_error_handler: Exception Handler finished.\n");

}
