/* $Id: droopres_exec.c,v 1.9 2004/05/13 19:11:54 fan Exp $ */


/*********************************************** 
droopres_exec.c

Calls the initialization functions and main loop of program. Saves
output to a file.

Definitions of Variables in droopres_defs.h
 
***********************************************/

#include <time.h>         /* time() may need this */
#include <sys/types.h>    /* time() may need this */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "fitsio.h"
#include "droopres.h"
#include "droopres_errcodes.h"
#include "droopres_defs.h"

int main(int argc, char **argv) {			
float	*FP_Data_In=0;
float   *FP_Vars_In=0;
short   *SP_pmask=0;
short   *SP_dmask=0;
short   *SP_cmask=0;
short   *SP_bmask=0;
short   *SP_umask=0;
float  mainvalue;
int     i,j;

Filenames	Fnames;
Constants       Const;
Status		Stat;
Dims            Dim_In;
struct tm       *tmp_time;
time_t          L_time;

/* null filename pointers */

clock();

printf("Pipeline Module DROOPRES Version %2.1f\n", VERSN);

/* Calculate and print to standard out time
   Keep tmp_time and module time for time
   time stamping all output
*/
L_time = time(NULL);
tmp_time = localtime(&L_time);
printf("Processing time ");
printf(asctime(tmp_time));
printf("\n");

/* Set constant default values */
droopres_init_const( &Fnames, &Const, &Stat);

/* Parse the namelist file */
droopres_parse_namelist( argc, argv, &Const, &Fnames, &Stat);

/* Parse the command line, replacing values if specified */
droopres_parse_args( argc, argv, &Const, &Fnames, &Stat);

/* Get the data and keyword values from the input file */
droopres_read_data( &Fnames, &Dim_In, &FP_Data_In, &FP_Vars_In,
&Stat, &SP_pmask, &SP_dmask, &SP_cmask, &SP_bmask, &SP_umask);

/* Perform numerical computations on the data */
droopres_compute( &Fnames, &Const, &Dim_In, FP_Data_In, FP_Vars_In,
&Stat, SP_pmask, SP_dmask, SP_cmask, SP_bmask, SP_umask);

/* Save the results as fits files. */			
droopres_output( &Fnames, &Dim_In, &Const, &Stat, tmp_time, SP_dmask,
FP_Data_In, FP_Vars_In);

/* Print filename, error message and timing data */
droopres_error_handler( &Stat, &Fnames);

/* Deallocate memory */			
droopres_exit( FP_Data_In, FP_Vars_In, &Dim_In, &Stat, SP_pmask, SP_dmask, SP_cmask, SP_bmask, SP_umask);

printf("exit code %d\n", Stat.I_status);

/* Send return value */
exit(Stat.I_status);

}
