/* $Id: droopres_find_trim_avg.c,v 1.5 2001/11/26 23:01:05 fan Exp $ */


/***************************************
droopres_find_trim_avg.c

Purpose
find the trim average value in a given list;
using heapsort sorting the list

****************************************/

#include <stdlib.h>
#include <math.h>
#include "droopres.h"
#include "droopres_errcodes.h"
#include "droopres_defs.h"

droopres_find_trim_avg(float  *array,
	               int    dim,
                       float  *trim_avg,
		       float  *variance,
                       Constants *Const,
                       Status *Stat) {

int i, ir, j, l;
float rra;
int k, start_index=0, end_index=0;
int tmp, num_samples;
float median, sum, sum_def;

l = 0;
ir = dim-1;

if (dim < 2) return;
l=dim>>1;
ir=dim-1;
while(1) {
    if (l > 0) {
       rra=array[--l];
    } else {
       rra=array[ir];
       array[ir]=array[0];
       if (--ir == 0) {
          array[0]=rra;
          break;
       }
    }
    i=l;
    j=l+l+1;
    while (j <= ir) {
         if (j < ir && array[j] < array[j+1]) j++;
         if (rra < array[j]) {
            array[i]=array[j];
            i=j;
            j = (j<<1) + 1;
         } else j=ir+1;
    }
    array[i]=rra;
}

if(dim%2 != 0)
   median = array[(dim-1)/2];
else
   median = (array[dim/2-1] + array[dim/2])/2.0;

if(dim%2 != 0) {
   start_index = (int)((dim-1)/2.0) - (int)(dim*Const->Trim_Perc/2.0);
   end_index = (int)((dim-1)/2.0) + (int)(dim*Const->Trim_Perc/2.0);
}
else {
   start_index = (int)((dim-1)/2.0 - dim*Const->Trim_Perc/2.0 + 0.5);
   end_index = (int)((dim-1)/2.0 + dim*Const->Trim_Perc/2.0 - 0.5);
   if (start_index > end_index) {  /* swap */
       tmp = start_index;
       start_index = end_index;
       end_index = tmp;
   }
}

num_samples = end_index - start_index + 1;

sum=0.0;
for (k=start_index; k<=end_index; k++)
   sum += array[k];
*trim_avg = sum/(float)(num_samples);

/* trim-average variance may have an additional factor -- to be revisited */
sum_def=0.0;
for (k=start_index; k<=end_index; k++)
   sum_def += (array[k] - *trim_avg)*(array[k] - *trim_avg);
*variance = sqrt(sum_def/(float)(num_samples*(num_samples-1)));

if (Stat->I_Verbose)
   printf("droopres_find_trim_avg: trim-average computation completed for channel using trim fraction %f\n\n", Const->Trim_Perc);

}
