/* $Id: droopres_output.c,v 1.9 2003/02/18 18:51:04 fan Exp $ */


/*********************************************** 
droopres_output.c

Purpose

Save the droopres outputs to files.

Overview
Write the results of the computations as a FITS file.

Definitions of Variables

Input:
I_NX = Number of pixels in a row of the image
I_NY = Number of pixels in a column of the image
CP_Filename_FITS_In = Input Data File Name. 

Output:


Internal:
ffp_FITS = Handle to output FITS file.
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_String = String value to be written to header.
CP_Comment = String value to be written in comment field of header.


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "droopres.h"
#include "droopres_errcodes.h"
#include "droopres_defs.h"


droopres_output(
            Filenames	  *Fnames,
            Dims          *Dim_In,
	    Constants     *Const,
            Status	  *Stat,
            struct tm     *tmp_time,
            short         *SPP_dmask,
            float         *FPP_Data_In,
            float         *FPP_Vars_In)
{

int	I_fits_return_status=0;
int	I_Num_In;
int     string_length;
int     i;

char    CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT], CP_Filename_In[MAX_FILENAME_LENGTH];
char    programname[32], versionstring[32];
char    CP_String[22];
char    format[10];

float   F_Num_In;

fitsfile 	*ffp_FITS_Out;
fitsfile        *ffp_Vars_Out;
fitsfile 	*ffp_FITS_In;
fitsfile        *ffp_Vars_In;
fitsfile 	*ffp_FITS_Dmask;

printf(asctime(tmp_time));

if(Stat->I_status) return;

/* Open dmask file. If found replace data.  */

if (strcmp(Fnames->CP_Filename_Dmask, ""))
   {
   fits_open_file( &ffp_FITS_Dmask,
                   Fnames->CP_Filename_Dmask,
                   READWRITE,
                   &I_fits_return_status);

/* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin DROOPRES module additions");
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "Residual droop correction performed using dmask");
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "DROOPRES");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_FITS_Dmask, programname, versionstring, &I_fits_return_status);
   
   fits_write_img(ffp_FITS_Dmask,
                  TSHORT,
                  1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                  SPP_dmask,
                  &I_fits_return_status);		

   fits_flush_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
		
   fits_close_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
   }

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* write out the image data */

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_In,
                READWRITE,
                &I_fits_return_status);

fits_create_file( &ffp_FITS_Out,
                Fnames->CP_Filename_FITS_Out,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "%s", "Begin DROOPRES module additions");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "Residual droop removed using unilluminated box");
fits_write_comment(ffp_FITS_Out,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRAU1");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpres_chan1;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRAU2");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpres_chan2;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRAU3");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpres_chan3;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRAU4");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpres_chan4;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRSU1");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpresunc_chan1;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRSU2");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpresunc_chan2;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRSU3");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpresunc_chan3;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(CP_Keyname, "%s", "DRRTRSU4");
sprintf(CP_Comment, "%s", "");
F_Num_In = Const->Drpresunc_chan4;
fits_write_key(ffp_FITS_Out, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out, &I_fits_return_status);

sprintf(programname, "%s", "DROOPRES");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_In,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out,
                &I_fits_return_status);

fits_close_file(ffp_FITS_In,
                &I_fits_return_status);

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

/* write out the uncertainty data if output filename specified */

if (strcmp(Fnames->CP_Filename_Vars_Out, "")) {

   if (Const->In_Vars) 
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_Vars_In,
                      READWRITE,
                      &I_fits_return_status);
   else
      fits_open_file( &ffp_Vars_In,
                      Fnames->CP_Filename_FITS_In,
                      READWRITE,
                      &I_fits_return_status);

   fits_create_file( &ffp_Vars_Out,
                   Fnames->CP_Filename_Vars_Out,
                   &I_fits_return_status);

   if (I_fits_return_status)
   {Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

   fits_copy_header(ffp_Vars_In,
                    ffp_Vars_Out,
                    &I_fits_return_status);

   /* Add module specific keywords */

   sprintf(CP_Comment, "%s", "Begin DROOPRES module additions");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "This is an uncertainty image");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(CP_Comment, "%s", "Residual droop removed using unilluminated box");
   fits_write_comment(ffp_Vars_Out,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "DROOPRES");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_Vars_Out, programname, versionstring, &I_fits_return_status);

   fits_write_img( ffp_Vars_Out,
                   TFLOAT,
                   1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                   FPP_Vars_In,
                   &I_fits_return_status);

   fits_flush_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_Out,
                   &I_fits_return_status);

   fits_close_file(ffp_Vars_In,
                   &I_fits_return_status);

}

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

if(Stat->I_Verbose)
printf("droopres_output: Results written to file(s).\n");

}
