#define PROGRAM "extract"
#define VERSION "5.6"

/**UPDATES
070910 5.6 S17 Add option to divied out height of wavsamp rectangle
070703 5.5 Added code to read APERTURE keyword if FOVID not found
061129 5.4 Fix overlay output for full-slit case
061027 5.3 Make the output spectrum optional (for getting overlay file in SPICE)
           Added coordinate of center in overlay output table
061003 5.2 when doing NaN replacement, do not use replaced NaNs in calculations
060830 5.1b change "scale" parameter to 1.0 when calling tbl_setcol (GNU bug)
060829 5.1a use NANVALUE instead of atof("NaN")
060709 5.1 add optional output of overlay file
060621 5.0 don't force wavsamp height to 1.0,  set NaNs to -9999 (option to drop)
051115 4.2 don't put -10000 for NaN fluxes, delete row instead (as always)
051109 4.1 for NaN replacement, change min_per_col and min_good_col to 1 (from 2)
051103 4.0 option to replace NaNs with profile fit of nearby rows (like Jim's irsclean)
           (removed updates 3.8 and 3.9)
050316 3.7 set mask bits correctly in output file (use integrate_quad_3) AR-6475
050311 3.6 new parameter logic for lambda,width: must specify both or neither
           fixed bug: was checking lambda=-1 for constant, should be 0.0
050310 3.5 removed check that disallowed fixed width extractions in hi-res
050111 3.4 fixed to use tblio instead of tableio  (IK)                     
040817 3.3 Make -full default to 0 if -w or -l are specified
040811 3.2 Added real "-ord" parameter logic
040809 3.1 Added non-operational "-ord" parameter (kludge for Ted's testing)
040422 3.0 Add "-full" parameter for full_slit extraction
040422 3.0 Add sub_slit extraction for hiRes
030616 2.0 Add functionality to extract the full low resolution slit SL, LL 
           as a command-line and namelist parameter, thus overriding the 
           PSF extraction aperture. Default is to still extract along peak 
           or expected FOV position as determined by ridge.                (JL)
030616 2.0 Included functionality to read the b*_psf_fov.tbl               (JL)
           tables to get PSF information for low-resolution modules.
030616 2.0 For low-resolution orders, changed the extraction aperture to
           Nx1 rectangle, where N is determined by the full width at half 
           maximum w of the PSF at each lambda position in the RIDGE table, 
           and in the tilt angle direction.                                (JL)
030616 2.0 Propagated all header information contained in the RIDGE output 
           table into the header of the EXTRACT output table.              (JL)
030616 2.0 Added one new column 'bit_flagging' indicating for certain bits 
           having been set among any of the pixels in the extraction 
           aperture.                                                       (JL)
020926 1.4 Modified to use integrate library instead of local copy of code (CW)
020926 1.4 Added namelist facility (CW)
020926 1.4 Named TERMINATE_* codes (CW)
020603 1.3b added the bmask data to detect the "bad" data (JL)
020603 1.3b generilized the read_image routine to handle all of the data type (JL)
020528 1.3b took the width of aperture of extraction w as the function of
            wavelength: w = w0/lambda0*lambda (JL)
020525 1.3b read in the x_ridge_r, y_ridge_r, x_ridge_l, y_ridge_l from the
            ridge table (JL)
000914 1.3a removed 0.5 pixel coordinate kludge
000828 1.2 update to new paramIo subroutines
000814 1.2 added 0.5 pixel coordinate kludge (wavsamp will be changed soon)
000809 1.2 updated from call of integratePoly into a call of Clare's integrate_quad
000524 1.2 changed #include <strings.h> to string.h (got message non-portable
**/

/* NOTE:  uses program downlink/pipeline/include/tools/nl_and_params.c  */

/* Termination codes as specified in:
**     SIRTF Downlink Software Development Guidelines  20010515
**     (file: /ssc/pipe/dlmtg/sw_dev_guidelines)
*/
#define TERMINATE_NORMAL         0
#define TERMINATE_WARNING       32
#define TERMINATE_ERROR	        64

#ifndef FALSE
#define FALSE		         0
#define TRUE              (!FALSE)
#endif

#define MAXFILENAMESIZE        256
#define NL_START        "&EXTRACT"
#define NL_END              "&END"

#include <math.h>
#include "integrate.h"
#include <string.h>
#include <stdio.h>
#include "tbl.h"
#include "nanvalue.h"

int  ou; /* output table id */

int verbose = 1; /* level of extra printout */
int total_nans_fixed = 0;

/*==== tio_del SUBROUTINE ==========================*/
void tio_del(int iu, char* col_name)
/* delete named column (no action if can't do it) */
{
  int ind_tmp;
  ind_tmp= tbl_column(iu, col_name);
  if (ind_tmp >= 0)
      tbl_delcol(ou, ind_tmp);
}


/*==== tio_getind SUBROUTINE ==========================*/
int tio_getind(int iu, char* col_name, int *ind)
/* get index of the named column */
/* return 0=OK  non-zero = trouble */
{
  *ind = tbl_column(iu, col_name);
  if (*ind < 0) {
    printf("ERROR: bad in table file: missing column: %s\n", col_name);
    return -1;
  }
  return 0;
}


/*==== min4 SUBROUTINE ==========================*/
double min4(double v1, double v2, double v3, double v4) {
 double retval;
 retval = v1;
 if (v2 < retval) retval = v2;
 if (v3 < retval) retval = v3;
 if (v3 < retval) retval = v3;
 return retval;
}


/*==== max4 SUBROUTINE ==========================*/
double max4(double v1, double v2, double v3, double v4) {
 double retval;
 retval = v1;
 if (v2 > retval) retval = v2;
 if (v3 > retval) retval = v3;
 if (v3 > retval) retval = v3;
 return retval;
}


/*==== fix_one_nan SUBROUTINE ==========================*/
int fix_one_nan(ptr_pixels, ptr_pixels_copy, nx, x_tgt, y_tgt, x_min, x_max, y_min, y_max)
/* returns 1 if NaN replaced, 0 if not */
double ptr_pixels[]; /*  address of pixel array */
double ptr_pixels_copy[]; /*  address of copy pixel array (no nsNs corrected here) */
int nx; /* size of x axis */
int x_tgt, y_tgt; /* pixel coord of pixel to be replaced */
int x_min; int x_max; int y_min; int y_max; /* pixel coords of area to do replacement */
{
#define MAXPROFSIZE 128
  int min_per_col   = 1; /* minimum # good pixels reqd for a col to be "good" */
  int min_good_cols = 1; /* minimun number of good columns considered a valid profile */
 
  int n_good_cols; /* number of "good" columns found */

  int prof_n[MAXPROFSIZE];
  double prof_mean[MAXPROFSIZE];

  double knum; /* statistical values */
  double kden;

  double running_sum; /* tmp used for computing column means */
  int running_n; /* number of good pixels in column */

  int x, y;
  int i; /* 1-D index into pixel array */
  int p; /* index into prof_ arrays (column) */

  if (verbose >= 5) {
    i = x_tgt + y_tgt*nx;
    printf("\nNaN at: x=%d y=%d i=%d val=%lf\n", x_tgt, y_tgt, i, ptr_pixels[i]);
  }

  /* find mean of each column */
  for (x=x_min; x<=x_max; x++) {
    p = x - x_min;
    running_sum = 0.0;
    running_n   = 0;
    for (y=y_min; y<=y_max; y++) {
      if (y == y_tgt) continue; /* don't average in the NaNs row */
      i = x + y*nx;
      if (finite( ptr_pixels_copy[i] )) {
        running_n += 1;
        running_sum += ptr_pixels_copy[i];
      }
    }
    prof_n[p] = running_n;
    if (running_n > 0) {
      prof_mean[p] = running_sum / running_n;
    }
    else {
      prof_mean[p] = 0.0;
    }

    if (verbose >= 4) {
      printf ("p=%d n=%d col_mean=%f tgt_val=%f", p, prof_n[p],
          prof_mean[p], ptr_pixels[x+y_tgt*nx]);
      if (x == x_tgt) printf (" x_tgt");
      printf ("\n");
    }
  }

  /* make sure column containing NaN has enough good pixels */
  p = x_tgt - x_min;
  if (prof_n[p] < min_per_col) {
    if (verbose >= 4) {
      printf ("NaN not fixed: only %d good pixels in NaNs column\n", prof_n[p]);
    }
    return 0;
  }

  /* compute knum and kden */
  knum =  0.0;
  kden =  0.0;
  n_good_cols = 0;
  for (x=x_min; x<=x_max; x++) {
    if (x == x_tgt) continue; /* don't average in the NaNs column */
    i = x + y_tgt*nx;
    if (finite(ptr_pixels_copy[i])==0) continue;
    p = x - x_min;
    if (prof_n[p] >= min_per_col) {
      n_good_cols ++;
      knum += prof_mean[p] * ptr_pixels_copy[i];
      kden += prof_mean[p] * prof_mean[p];
    }
  }

  /* make sure there are enough good columns */
  if (n_good_cols < min_good_cols) {
    if (verbose >= 4) {
      printf ("NaN not fixed: only %d good columns in NaNs row\n", prof_n[p]);
    }
    return 0;
  }

  /* replace the NaN with estimated value */
  i = x_tgt + y_tgt*nx;
  p = x_tgt - x_min;
  ptr_pixels[i] = prof_mean[p] * knum / kden;
  if (verbose >= 4) {
    printf ("OK fix: good_cols=%d val=%f\n", n_good_cols, ptr_pixels[i]);
  }

  return 1;
}


/*==== fix_all_nans SUBROUTINE ==========================*/
int fix_all_nans(ptr_pixels, ptr_pixels_copy, nx, ny,
   dbl_x_min, dbl_x_max, dbl_y_min, dbl_y_max, dbl_x_ws_min, dbl_x_ws_max, n_rows_prof)
/* returns number of NaNs replaced */
double ptr_pixels[]; /*  address of pixel array (contining NaNs) */
double ptr_pixels_copy[]; /*  address of copy pixel array (no nsNs corrected here) */
int nx, ny; /* dimensions of pixel array */
double dbl_x_min, dbl_x_max, dbl_y_min, dbl_y_max; /* coords of extraction area */
double dbl_x_ws_min, dbl_x_ws_max; /* coords of wavsamp area */
int n_rows_prof; /* number of rows to do profiles (abobe and below) */
{
  int i;
  int x, y;
  int x_min, x_max, y_min, y_max; /* area to check for NaN replacement */
  int x_ws_min, x_ws_max; /* range for profileing */
  int y_lo, y_hi; /* y coords of profile fitting area */
  int fixed;
  int n_fixed = 0;

  /* convert to pixel coordinates and be sure not out of bounds */
  x_ws_min = dbl_x_ws_min;
  if (x_ws_min < 0) x_ws_min = 0;
  x_ws_max = dbl_x_ws_max;
  if (x_ws_max > nx-1) x_ws_max = nx-1;

  x_min = dbl_x_min;
  if (x_min < x_ws_min) x_min = x_ws_min;
  x_max  = dbl_x_max + 1;
  if (x_max > x_ws_max) x_max = x_ws_max;
  y_min = dbl_y_min;
  if (y_min < 0) y_min = 0;
  y_max  = dbl_y_max + 1;
  if (y_max > ny-1) y_max = ny-1;

  for (x=x_min; x<=x_max; x++) {
    for (y=y_min; y<=y_max; y++) {
      i = x + y*nx;
      if (finite(ptr_pixels[i]) ==0) {
        y_lo = y - n_rows_prof;
        if (y_lo < 0) y_lo = 0;
        y_hi = y + n_rows_prof;
        if (y_hi > ny-1) y_hi = ny-1;
        fixed = fix_one_nan(ptr_pixels, ptr_pixels_copy, nx, x, y,
                   x_ws_min, x_ws_max, y_lo, y_hi);
        n_fixed += fixed;
        if (verbose >= 4) {
          printf ("NaN at x=%3d y=%3d set to %lf\n", x, y, ptr_pixels[i]);
        }
      }
    }
  }
  return n_fixed;
}


/*=== extract =================================================*/
int extract(argc, argv)
int argc;
char **argv;
{
  char nl_file[MAXFILENAMESIZE];
  char in_image[MAXFILENAMESIZE];
  char in_bmask[MAXFILENAMESIZE];
  char in_error[MAXFILENAMESIZE];
  char in_rtable[MAXFILENAMESIZE];
  char in_ptable[MAXFILENAMESIZE];
  int plane=1;
  int fatal_bmask;
  double w0   =  -1.0, lambda0  = -1.0;
  double w0_2 =  -1.0, lambda0_2= -1.0;
  double w0_3 =  -1.0, lambda0_3= -1.0;
  int full_slit;         /* 1=extract full_slit. 0=extract sub_slit */
  int flag_height_1;     /* 0=extract actual height 1=force height to 1.0 */
  int flag_nanDrop;      /* 0=put -9999.0 if NaN  1=delete row  */
  int order_pick;        /* see params for meanings of values */
  char out_table[MAXFILENAMESIZE];
  char out_overlay[MAXFILENAMESIZE];
  int fix_nans;
  int normalize = 0;
  FILE *fid_overlay; /* for writing overlay file */

  float extract_width, extract_height, extract_area;
  float mean_dn, mean_error;
  float half_cos, half_sin;

  /* table IO data */
  int iu, iur,iup; /* input table id  */
  int nparms,kindex,values;
  int chnlnum, ordernum=-1, ordernum2=-1, ordernum3=-1,fovid;
  int ind_fov, ind_lambda0, ind_w0;
  int ind_dn;
  int ind_error;
  int ind_order_r, ind_order_p;
  int ind_wavelength;
  int ind_angle;
  int ind_bitflag;
  int ind_x0, ind_y0;
  int ind_x1, ind_y1;
  int ind_x2, ind_y2;
  int ind_x3, ind_y3;
  int ind_x_ridge_r, ind_y_ridge_r;
  int ind_x_ridge_l, ind_y_ridge_l;
  /* tbl i/o */
  char** hist_cards;
  char  type[80], unit[80];
  char kval[TBL_KEYLEN];
  int tstat, row, out_row, nrows;
  struct tbl_filinfo  *rinfo, *pinfo, *outinfo;

  char  chnlnum_str[80], fovid_str[80];

  /* image reading data */
  double (*addr_pixels)[]; /* address of pixel array */
  double (*addr_pixels_copy)[]; /* address of copy of pixel array (for replacing NaNs) */
  int nx, ny;


  /* bmask data */
  short (*addr_bmask)[];   /* address of bmask array */
  int nx_bmask, ny_bmask;
  
  /* error data */
  double (*addr_errors)[]; /* address of error array */
  int nx_error, ny_error;

  /* for image stats in debug */
  double image_dn, image_sum, image_count, image_min, image_max;

  /* values in input table */
  int   order;                           /* order of each extraction point */
  int fov;
  double  angle;
  double lambda;
  double x_ridge_r, y_ridge_r, x_ridge_l, y_ridge_l; 
                                        /* center of extraction point */
  double x0, y0, x1, y1, x2, y2, x3, y3; /*four corners coming from wavsamp */

  float  *dn_out = 0;
  float  *error_out = 0;

  int    *bitflag_out = 0;

  /* computed values */
  int ixmin, ixmax, iymin, iymax;
  short int bitflag = 0;
  float length_top_bottom;
  float dx, dy;
  float side_dx, side_dy;
  float x_e_top, y_e_top;
  float x_e_bottom, y_e_bottom;
  float x_delta, y_delta;
  double xm1, ym1, xm2, ym2;
  double xx1,yy1, xx2,yy2, xx3,yy3, xx4, yy4;
  double x_center, y_center;
  double pixel_area, pixel_area_sq, total_dn=0., total_error=0.;
  double dtr =  0.01745329251994329;
  double dn, error;

  /* used in fixing NaNs */
  double x_min, x_max, y_min, y_max;
  double x_ws_min, x_ws_max;
  int n_replaced;

  int i,j;
  int ix, iy;
  int pix0, pix1;
  int status;

  /* for stats of neighboring pixels */
  int n_x_plus_1=0, n_x_minus_1=0, n_x_plus_2=0, n_x_minus_2=0;
  int n_y_plus_1=0, n_y_minus_1=0, n_y_plus_2=0, n_y_minus_2=0;
  float x_plus_1=0.0, x_minus_1=0.0, x_plus_2=0.0, x_minus_2=0.0;
  float y_plus_1=0.0, y_minus_1=0.0, y_plus_2=0.0, y_minus_2=0.0;

  /**PARAMS **/
  param(in_image, "-i INFILE name of input image", "infile echelle.fits");
  param(in_rtable,"-r RIDGE_TABLE name of input ridge table", "infile ridge.tbl");
  param(in_ptable,"-p PSF_TABLE name of input PSF table", "infile - psf_fov.tbl");
  param(in_bmask, "-b BMASKFILE name of bmask file", "infile - NONE");
  param(in_error, "-e UNCERFILE name of error image", "infile - NONE");
  param(&fatal_bmask, 
                  "-f BMASKFATAL value of fatal", "int - 16384");
  param(&w0,      "-w w0 initial width at ref wavelength", "double pixels -1.0");
  param(&lambda0, "-l lambda0 ref wavelength 0=constant width", "double micron -1.0");  
  param(out_table,"-o OUTFILE name of output spectrum", "outfile - NONE");
  param(out_overlay,"-over OUT_OVERLAY name of output overlay file", "outfile - NONE");
  param(&normalize,   "-norm NORMALIZE normalize fluxes (2=divide by WS non-NaN area, 1=divide by WS height, 0=leave)", "int - 0");
  param(&verbose, "-v VERBOSE level of printed messages", "int - 1");
  param(nl_file,  "-n NAMELIST_FILE name of Namelist file", "infile - namelist.nl");
  param(&full_slit, "-full FULL_SLIT_EXTRACT 0=sub_slit  1=full_slit -1=default", "int - -1");
  param(&flag_height_1, "-height1  HEIGHT1 force wavsamp height to 1 0=use actual height 1=force height to 1.0 pixel", "int - 0");
 param(&flag_nanDrop, "-nanDrop FLAG_NAN_DROP drop row for NaNs in extraction file (default is -9999.0)", "int - 0");

  param(&order_pick,  "-ord ORDER 0=as targeted  1=1 2=2+bonus, 99=1+2+bonus (lo-res only)", "int - 0");
  param(&fix_nans,  "-fix FIXUP_NANS 0=no >0 yes, number of rows to profile", "int - 2");

  /**ENDPARAMS **/
  
  status = nl_process(argc, argv, PROGRAM, VERSION, NL_START, NL_END);
  if (status != 0) return status;

  status = param_begin(argc, argv, PROGRAM, VERSION);
  if (status != 0) return status;

  param_print();   /* Print the parameter settings to stdout. */
  param_subvers(integrate_name(), integrate_version() );

  /* check parameters */
  if (strcmp(out_table, "NONE")==0) {
    fix_nans = 0;
  } else {
    if (strcmp(in_bmask, "NONE")==0) {
      printf("ERROR: bmask file (-b) is required.\n");
      return TERMINATE_ERROR;
    }
    if (strcmp(in_error, "NONE")==0) {
      printf("ERROR: Uncertainty file (-e) is required.\n");
      return TERMINATE_ERROR;
    }
  }
  
  /* read input data files */
  status = read_image(in_image, &addr_pixels, &nx, &ny, "double");
  if (status != 0) {
    printf("Error reading input image.\n");
    return TERMINATE_ERROR;
  }
  if (strcmp(out_table, "NONE")!=0) {
    /* read input image file */
    /* read bmask file */
    status = read_image(in_bmask, &addr_bmask, &nx_bmask, &ny_bmask, "short");
    if (status != 0) { 
      printf("Error reading bmask data.\n");
      return TERMINATE_ERROR;
    }
    /* read unc file */
    status = read_image(in_error, &addr_errors, &nx_error, &ny_error, "double");
    if (status != 0) { 
      printf("Error reading uncertainty data.\n");
      return TERMINATE_ERROR;
    }  
    /* Set the fatal data in the image to be NAN */
    for (i=0; i<nx*ny; ++i) {
      if (((*addr_bmask)[i] & fatal_bmask) != 0) {
         (*addr_pixels)[i] = NANVALUE;
      }
    }
  }

  if (verbose >= 3) {
    printf("in_image {%s}\n", in_image);
    printf("in_rtable {%s}\n", in_rtable);
    printf("in_ptable {%s}\n", in_ptable);
    printf("out_table {%s}\n", out_table);
    printf("nx = %i\n", nx);
    printf("ny = %i\n", ny);

    image_min =  1.0E300;
    image_max = -1.0E300;
    image_sum = 0.0;
    image_count = 0.0;
    for (i=0; i<nx*ny; i++) {
     image_dn = (*addr_pixels)[i];
     image_count += 1.0;
     image_sum += image_dn;
     if (image_dn > image_max) image_max = image_dn;
     if (image_dn < image_min) image_min = image_dn;
    }
    printf("mean =%lf\n", image_sum/image_count);
    printf("min = %lf\n", image_min);
    printf("max = %lf\n", image_max);
  }
  

  /* Read and link the ridge table and the output table */
  iur = tbl_open(in_rtable, &rinfo);
  tstat   = tbl_fileinfo(iur, &rinfo);
  if (tstat != 0) {
    printf("Cannot open table: [%s]\n",in_rtable);
    exit(64);
  }

  /* get column indicies */
  if (tio_getind(iur, "order", &ind_order_r)) return TERMINATE_ERROR;
  if (tio_getind(iur,"wavelength", &ind_wavelength)) return TERMINATE_ERROR;
  if (tio_getind(iur,"angle", &ind_angle)) return TERMINATE_ERROR;
  if (tio_getind(iur,"x_ridge_r", &ind_x_ridge_r))   return TERMINATE_ERROR;
  if (tio_getind(iur,"y_ridge_r", &ind_y_ridge_r))   return TERMINATE_ERROR;
  if (tio_getind(iur,"x_ridge_l", &ind_x_ridge_l))   return TERMINATE_ERROR;
  if (tio_getind(iur,"y_ridge_l", &ind_y_ridge_l))   return TERMINATE_ERROR;
  if (tio_getind(iur,"x0", &ind_x0)) return TERMINATE_ERROR;
  if (tio_getind(iur,"y0", &ind_y0)) return TERMINATE_ERROR;
  if (tio_getind(iur,"x1", &ind_x1)) return TERMINATE_ERROR;
  if (tio_getind(iur,"y1", &ind_y1)) return TERMINATE_ERROR;
  if (tio_getind(iur,"x2", &ind_x2)) return TERMINATE_ERROR;
  if (tio_getind(iur,"y2", &ind_y2)) return TERMINATE_ERROR;
  if (tio_getind(iur,"x3", &ind_x3)) return TERMINATE_ERROR;
  if (tio_getind(iur,"y3", &ind_y3)) return TERMINATE_ERROR;

  if (strcmp(out_table, "NONE")!=0) {
  /* Open output table and link the ridge table */
    ou = tbl_openw(out_table, &outinfo);
    tstat   = tbl_fileinfo(ou, &outinfo);
    if (tstat != 0) {
      printf("Cannot open table: [%s]\n",out_table);
      exit(64);
    }
    tstat = tbl_link(iur, ou);

    /* delete some columns */
    tio_del(iur,"x_ridge_r");
    tio_del(iur,"y_ridge_r");
    tio_del(iur,"x_ridge_l");
    tio_del(iur,"y_ridge_l");
    tio_del(iur,"x0");
    tio_del(iur,"y0");
    tio_del(iur,"x1");
    tio_del(iur,"y1");
    tio_del(iur,"x2");
    tio_del(iur,"y2");
    tio_del(iur,"x3");
    tio_del(iur,"y3");
    tio_del(iur,"angle");

    /* add some columns */
    tbl_setcol(ou,"flux",   "real ","elec/sec ", " ", 1.0,0.0, 15,5,&ind_dn);
    tbl_setcol(ou, "error", "real ","elec/sec ", " ", 1.0,0.0, 15,5,&ind_error);
    tbl_setcol(ou, "bit-flag","int","N/A ",      " ", 1.0,0.0, 15,0,&ind_bitflag);

    /* Add HISTORY cards */
    param_hcards(&hist_cards);  
    for (i=0; hist_cards[i]; i++)
      tbl_setkey(ou, "HISTORY", "char", hist_cards[i],  "");
  }
  

  /* Read CHNLNUM from RIDGE table */
  tstat = tbl_keyval(iur, "CHNLNUM", kval, &kindex);
  if (tstat != 0) {
     printf("CHNLNUM not present in ridge.tbl, quit.\n");
     exit(64);
  }
  sscanf(kval,"%d",&chnlnum);


  
  /**************************/
  /* process -ord parameter */
  if (chnlnum == 0 || chnlnum == 2) { /* lo res */
    if (order_pick == 0) { /* (use default order) */

      /* read fovid in RIDGE table */

     tstat = tbl_keyval(iur, "FOVID", kval, &kindex);
     if (tstat != 0) tstat = tbl_keyval(iur, "APERTURE", kval, &kindex);
     if (tstat != 0) {
        printf("ERROR: No FOVID or APERTURE keyword in ridge file.\n") ;
        exit(64);
     }
     sscanf(kval,"%d",&fovid);
    }

    /* set fov based on -ord parameter */
    else if (order_pick == 1) { /* (do order 1) */
      if (chnlnum == 0) fovid = 28; /* FIRST ORDER, CENTER */
      else              fovid = 40; /* FIRST ORDER, CENTER */
    }
    else if (order_pick == 2) { /* (do orders 2  & 3) */
      if (chnlnum == 0) fovid = 34; /* SECOND and BONUS ORDER, CENTER */
      else              fovid = 46; /* SECOND and BONUS ORDER, CENTER */
    }
    else if (order_pick == 99) { /* do all orders 1,2,3 */
      if (chnlnum == 0) fovid = 29; /* MODULE CENTER */
      else              fovid = 41; /* MODULE CENTER */
    }
    else  {
      printf("ERROR: -ord must be one of: 0, 1, 2, 99.\n");
      exit(64);
    }
    
  }
  else { /* hi res */
    if (order_pick != 0) {
      printf("ERROR: May not use -ord parameter with hi-res data\n");
      exit(64);
    }
  }


  /***************************/
  /* process -full parameter */
  if (full_slit == -1) { /* set if defaulted */
    if ( (chnlnum==0 || chnlnum==2) && (fovid!=29 && fovid!=41) )
       full_slit = 0; /* lo res non-center */
    else { /* hi res -or- lo res center */
      if (w0 == -1 && lambda0 == -1) full_slit = 1;
      else full_slit = 0; /* subslit if -w or -l specified */
    }
  }
  else if (full_slit != 0 && full_slit != 1) {
    printf("ERROR: Parameter -full must be one of: -1, 0, 1\n");
    exit(64);
  }

  /***************************************/
  /* process -l (lambda0) and -w parameters */
  /* IF sub-slit and (hires -OR- one of w,l is set) */
  /* i.e. no need check if full-slit or if lo-res with neither w,l set */
  if (full_slit == 0 && (chnlnum==1 || chnlnum==3 || w0!=-1 || lambda0!=-1)) {
    /* w0 check */
    if (w0 == -1) { /* defaulted */
      printf("ERROR: Parameter -w required for sub-slit extract\n");
      exit(64);
    }
    else { /* check in range */
      if (w0 <= 0.0) {
        printf("ERROR: Parameter -w must be > 0\n");
        exit(64); 
      }
    }
 
    /* lambda0 check */
    if (lambda0 == -1) { /* defaulted */
      printf("ERROR: Parameter -l required for sub-slit extract\n");
      exit(64);
    }
    else { /* check in range */
      if (lambda0 <  0.0) {
        printf("ERROR: Parameter -l must be >= 0\n");
        printf(" (Use 0. for constant width extraction)\n");
        exit(64); 
      }
    }

    /* copy for other two orders */
    w0_2 = w0;
    lambda0_2 = lambda0;
    w0_3 = w0;
    lambda0_3 = lambda0;
  }

  else { /* full slit: not allow width parameters */
    if (lambda0 != -1.0) {
      printf("ERROR: Parameter -l not allowed with full slit extractions\n");
      exit(64); 
    }
    if (w0 != -1.0) {
      printf("ERROR: Parameter -w not allowed with full slit extractions\n");
      exit(64); 
    }
  }

  /**************************************************************/
  /* read PSF table if lo-res */
  /* (always need to read for lo res to get ordernums) */
  if (chnlnum==0 || chnlnum==2) {
     iup = tbl_open(in_ptable, &pinfo);;
     tstat   = tbl_fileinfo(iup, &pinfo);
     if (tstat != 0) {
        printf("Cannot open table: [%s]\n",in_ptable);
        exit(64);
     }
 
    if (tio_getind(iup,"FOV", &ind_fov))       return TERMINATE_ERROR;
    if (tio_getind(iup,"order", &ind_order_p))   return TERMINATE_ERROR;
    if (tio_getind(iup,"lambda_ref", &ind_lambda0)) return TERMINATE_ERROR;
    if (tio_getind(iup,"W_ref", &ind_w0))      return TERMINATE_ERROR;

    row = 0;
    while(1) {
      tstat = tbl_colval(iup, row, ind_fov, TBL_INTEGER, &fov);
      if (tstat == -4) break;
      else if (tstat < 0)
      {
         printf("Table I/O Error\n");
         return TERMINATE_ERROR;
      }
	
      if (fov == fovid) {
        if (ordernum == -1) {
          tstat = tbl_colval(iup, row, ind_order_p, TBL_INTEGER, &ordernum);
          if (lambda0 == -1.0) 
             tstat = tbl_colval(iup, row, ind_lambda0, TBL_DOUBLE, &lambda0);
          if (w0      == -1.0) 
             tstat = tbl_colval(iup, row, ind_w0, TBL_DOUBLE, &w0);
        }
        else if (ordernum2 == -1) {
          tstat = tbl_colval(iup, row, ind_order_p, TBL_INTEGER, &ordernum2);
          if (lambda0_2 == -1.0) 
             tstat = tbl_colval(iup, row, ind_lambda0, TBL_DOUBLE, &lambda0_2);
          if (w0_2      == -1.0) 
             tstat = tbl_colval(iup, row, ind_w0, TBL_DOUBLE, &w0_2);
        }
        else if (ordernum3 == -1) {
          tstat = tbl_colval(iup, row, ind_order_p, TBL_INTEGER, &ordernum3);
          if (lambda0_3 == -1.0)
             tstat = tbl_colval(iup, row, ind_lambda0, TBL_DOUBLE, &lambda0_3);
          if (w0_3      == -1.0) 
             tstat = tbl_colval(iup, row, ind_w0, TBL_DOUBLE, &w0_3);
        }
      }
      row++;
    }
    tbl_close(iup);
  } /* end: if lo-res (read PSF table) */


  /************************************************/
  /* print flags/values to be used for extraction */
  printf ("EXTRACTING channel=%d", chnlnum);
  if (chnlnum==0 || chnlnum==2) { /* lo res */
    printf (" (lo-res) effective_FOV=%d\n", fovid);
    if (ordernum!=-1) {
      printf ("order=%d: ", ordernum);
      if (full_slit==1) printf ("full slit\n");
      else printf ("lambda0=%f w0=%f\n", lambda0,   w0);
    }
    if (ordernum2!=-1) {
      printf ("order=%d: ", ordernum2);
      if (full_slit==1) printf ("full slit\n");
      else printf ("lambda0=%f w0=%f\n", lambda0_2,   w0_2);
    }
    if (ordernum3!=-1) {
      printf ("order=%d: ", ordernum3);
      if (full_slit==1) printf ("full slit\n");
      else printf ("lambda0=%f w0=%f\n", lambda0_3,   w0_3);
    }
  }
  else { /* hi res */
    printf (" (hi-res)\nALL ORDERS: ");
    if (full_slit==1) printf ("full slit\n");
    else printf ("lambda0=%f w0=%f\n", lambda0,   w0);
  }


 /*************************/
 /* START PROCESSING DATA */
 /*************************/

  tstat = tbl_numcheck(iur, &nrows);

  if (strcmp(out_table, "NONE")!=0) {
    dn_out = (float *) malloc(nrows* sizeof(float));
    error_out = (float *) malloc(nrows* sizeof(float));
    bitflag_out = (int *) malloc(nrows* sizeof(int));

    if (dn_out == NULL || error_out == NULL || bitflag_out == NULL) {
      printf("Error in allocating memory for output table params\n");
      return TERMINATE_ERROR;
    }
  }


  if (strcmp(out_overlay, "NONE")!=0) {
    fid_overlay  = fopen(out_overlay, "w");
    fprintf(fid_overlay, "|order|wavelength|  x1  |  y1  | xctr | yctr |  x2  |  y2  |\n");
    fprintf(fid_overlay, "|int  |real      |real  |real  |real  |real  |real  |real  |\n");
    fprintf(fid_overlay, "|     |micron    |pixel |pixel |pixel |pixel |pixel |pixel |\n");
  }


  if (fix_nans > 0) { /* make a copy of input pixels if doing NaN replacement */
    addr_pixels_copy = (void *) malloc((unsigned)(nx * ny * sizeof(double)));
    if (addr_pixels_copy == NULL) {
       printf("Can't allocate required memory copy for NaN correcting\n");
       return TERMINATE_ERROR;
    }
    for (i=0; i<nx*ny; i++) (*addr_pixels_copy)[i] = (*addr_pixels)[i];
  }


  /**************************************/
  row = 0;
  tstat = 0;
  /* loop for each input of RIDGE table */
  while(1) {
    /* Get data from ridge table */
    tstat = tbl_colval(iur, row, ind_order_r, TBL_INTEGER, &order);
    if (tstat == -4) break;
    else if (tstat < 0) {
      printf("Table I/O Error when reading ridge table\n");
      return TERMINATE_ERROR;
    }
    if ((chnlnum == 0 || chnlnum == 2) &&
         order != ordernum && order != ordernum2 && order != ordernum3)
    {
       if (strcmp(out_table, "NONE")!=0) tstat = tbl_delrow(ou, row);
       row++;
       continue; 
    }
    tstat = tbl_colval(iur,  row, ind_wavelength, TBL_DOUBLE, &lambda);
    tstat = tbl_colval(iur,  row, ind_angle, TBL_DOUBLE, &angle);
    tstat = tbl_colval(iur,  row, ind_x0, TBL_DOUBLE, &x0);
    tstat = tbl_colval(iur,  row, ind_y0, TBL_DOUBLE, &y0);
    tstat = tbl_colval(iur,  row, ind_x1, TBL_DOUBLE, &x1);
    tstat = tbl_colval(iur,  row, ind_y1, TBL_DOUBLE, &y1);
    tstat = tbl_colval(iur,  row, ind_x2, TBL_DOUBLE, &x2);
    tstat = tbl_colval(iur,  row, ind_y2, TBL_DOUBLE, &y2);
    tstat = tbl_colval(iur,  row, ind_x3, TBL_DOUBLE, &x3);
    tstat = tbl_colval(iur,  row, ind_y3, TBL_DOUBLE, &y3);
    tstat = tbl_colval(iur,  row, ind_x_ridge_r, TBL_DOUBLE, &x_ridge_r);
    tstat = tbl_colval(iur,  row, ind_y_ridge_r, TBL_DOUBLE, &y_ridge_r);
    tstat = tbl_colval(iur,  row, ind_x_ridge_l, TBL_DOUBLE, &x_ridge_l);
    tstat = tbl_colval(iur,  row, ind_y_ridge_l, TBL_DOUBLE, &y_ridge_l);

    if (tstat < 0) {
      printf("Table I/O Error when extracting col values in ridge table\n");
      return TERMINATE_ERROR;
    }

    x_center = (x_ridge_r+x_ridge_l)/2;
    y_center = (y_ridge_r+y_ridge_l)/2;

    /* compute coordinates of extraction rectangle */
    /* if no lambda0 specified, using the fixed width of w0 */

    if (full_slit==0) { /* extract sub-slit */
      if (lambda0 == 0.) extract_width = w0; /* fixed width extraction */
      else {
        if (order == ordernum || chnlnum == 1 || chnlnum == 3)
          extract_width = w0/lambda0 * lambda;
        else if (order == ordernum2)
          extract_width = w0_2/lambda0_2 * lambda;
        else if (order == ordernum3)
          extract_width = w0_3/lambda0_3 * lambda;
      }

      /* Extract area is a Nx1 rectangle, centered at the center of cutting 
         and along the tilt angle direction with width N */
      half_sin = sin(angle*dtr) * 0.5;
      half_cos = cos(angle*dtr) * 0.5;

      xm1 = x_center + extract_width * half_cos;
      xm2 = x_center - extract_width * half_cos;
      ym1 = y_center + extract_width * half_sin;
      ym2 = y_center - extract_width * half_sin;

      if (flag_height_1 == 1)  { extract_height = 1.0; }
      else { extract_height = sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1)); }

      xx1 = xm1 - extract_height * half_sin;
      yy1 = ym1 + extract_height * half_cos;
      xx2 = xm1 + extract_height * half_sin;
      yy2 = ym1 - extract_height * half_cos;
      xx3 = xm2 + extract_height * half_sin;
      yy3 = ym2 - extract_height * half_cos;
      xx4 = xm2 - extract_height * half_sin;
      yy4 = ym2 + extract_height * half_cos;
      extract_area = extract_width * extract_height;
    }
    else { /* extract wavsamp area  ("full slit" extraction) */
      xx1 = x0;
      yy1 = y0;
      xx2 = x1;
      yy2 = y1;
      xx3 = x2;
      yy3 = y2;
      xx4 = x3;
      yy4 = y3;
      extract_width  = sqrt( (xx1-xx2)*(xx1-xx2) + (yy1-yy2)*(yy1-yy2) );
      extract_area = 0.5 * fabs(xx1*yy2 + xx2*yy3 + xx3*yy4 + xx4*yy1
                              - yy1*xx2 - yy2*xx3 - yy3*xx4 - yy4*xx1);
      extract_height = extract_area / extract_width;
    }



    /* replace NaNs */
    if (fix_nans > 0) {
      /* find min,max x,y of extraction area */
      x_min = min4(xx1, xx2, xx3, xx4);
      x_max = max4(xx1, xx2, xx3, xx4);
      y_min = min4(yy1, yy2, yy3, yy4);
      y_max = max4(yy1, yy2, yy3, yy4);
      /* find min,max x of wavsamp area */
      x_ws_min = min4(x0, x1, x2, x3);
      x_ws_max = max4(x0, x1, x2, x3);
      n_replaced = fix_all_nans(*addr_pixels, *addr_pixels_copy, nx, ny,
        x_min, x_max, y_min, y_max, x_ws_min, x_ws_max, fix_nans);
      total_nans_fixed += n_replaced;
    }


    if (verbose >= 5) {
      printf("\n=============================================\n");
      printf("ridge_right  x=%8.4f  y=%8.4f\n", x_ridge_r, y_ridge_r);
      printf("ridge_left  x=%8.4f  y=%8.4f\n", x_ridge_l, y_ridge_l);
      printf("xcenter = %8.4f, ycenter = %8.4f\n", 
              (x_ridge_r+x_ridge_l)/2, (y_ridge_r+y_ridge_l)/2);
      printf("xwidth = %8.4f, ywidth = 1.0\n", extract_width); 
      printf("x0= %8.4f  y0= %8.4f\n", x0, y0);
      printf("x1= %8.4f  y1= %8.4f\n", x1, y1);
      printf("x2= %8.4f  y2= %8.4f\n", x2, y2);
      printf("x3= %8.4f  y3= %8.4f\n", x3, y3);
      printf("\n");

      printf("\n");
      printf("1  xx1= %8.4f  yy1= %8.4f\n", xx1, yy1);
      printf("2  xx2= %8.4f  yy2= %8.4f\n", xx2, yy2);
      printf("3  xx3= %8.4f  yy3= %8.4f\n", xx3, yy3);
      printf("4  xx4= %8.4f  yy4= %8.4f\n", xx4, yy4);
    }

    /* call integration routine */
    if (strcmp(out_table, "NONE")!=0) {
      total_dn = integrate_quad_3(
          xx1, yy1, xx2, yy2, xx3, yy3, xx4, yy4,
          *addr_pixels, *addr_errors, *addr_bmask,
          fatal_bmask, /* fatal_bmask is not used */
          nx, ny, 
          &pixel_area, &pixel_area_sq, &total_error, &bitflag);
  
      /* compute dn and error */
      if (pixel_area > 0.0) {
        if (normalize == 2) {
          dn = (float) (total_dn/pixel_area);
          error = (float) (total_error/pixel_area);
        } else if (normalize == 1) {
          dn = (float) (total_dn/extract_height);
          error = (float) (total_error/extract_height);
        } else {
          dn = (double) total_dn;
          error = (double) total_error;
        }
      }
      else {
        dn    = -9999.0;
        error = -9999.0;
      }

      if (verbose >= 3) {
        printf("l=%6.3f w=%5.3f, h=%5.3f ar=%5.3f pix=%5.3f rat=%5.3f ang=%5.3f dn=%7.3f\n",
              lambda, extract_width, extract_height, extract_area, 
              pixel_area, pixel_area/extract_area, angle, dn);
      }

      /* verbose printout */
      if (verbose >= 4) {
        printf("\n=============================================\n");
        printf("total_dn=%12.4lf\n", total_dn);
        printf("pixel_area=%12.4lf\n", pixel_area);
        printf("mean_dn=%12.4f\n", mean_dn);
        printf("dn=%12.4f\n", dn);
        printf("total_error=%12.4lf\n", total_error);
        printf("bit mask=%i\n", bitflag);
      }
  
      /* save values in main arrays */
      dn_out[row] = dn;
      error_out[row] = error; 
      bitflag_out[row] = bitflag;

      /* delete sample if "dn" is NaN */
      if (flag_nanDrop!=0 && dn== -9999.0) tstat = tbl_delrow(ou, row);
    }

  if (strcmp(out_overlay, "NONE")!=0) {
    if (full_slit==0) { /* extracting sub-slit */
      fprintf(fid_overlay,
       " %5d %10.5lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf \n",
        order, lambda, xx2,  yy2, x_center, y_center, xx3, yy3);
    }
    else { /* extracting full-slit */
      fprintf(fid_overlay,
       " %5d %10.5lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf %6.2lf \n",
        order, lambda, xx2,  yy2, x_center, y_center, xx1, yy1);
    }
  }

    /* move on to next sample */
    row++;
  }


  if (strcmp(out_overlay, "NONE")!=0) fclose(fid_overlay);

  if (fix_nans != 0) printf("Total NaNs fixed = %d\n", total_nans_fixed);

  /* set values for output table file */
  if (strcmp(out_table, "NONE")!=0) {
    tstat = tbl_setcolval_f(ou, ind_dn, "real ", row, dn_out);
    tstat = tbl_setcolval_f(ou, ind_error, "real ", row, error_out);
    tstat = tbl_setcolval_i(ou, ind_bitflag, "int", row, bitflag_out);
    if (tstat < 0) {
      printf("Table I/O Error when setting col values in extract table\n");
      return TERMINATE_ERROR;
    }

    /* outupt to table file */
    tstat = tbl_write(ou);
    if (tstat < 0) {
      printf("Table I/O Error when writing out extract table\n");
      return TERMINATE_ERROR;
    }

    /* close files and return */
    free((void *)addr_pixels);
    free((void *)addr_bmask);
    free((void *)addr_errors);
    free(dn_out);
    free(error_out);
    free(bitflag_out);
    if (fix_nans > 0) free((void *)addr_pixels_copy);
    tbl_close(ou);
  }

  tbl_close(iur);

  return 0;
  
}

int read_image(filename, ptr_pixels, ret_nx, ret_ny, type)
/* subroutine to readin  image file */
char filename[]; /* name of image file */
void **ptr_pixels[]; /* pointer to address of pixel array */
int *ret_nx;
int *ret_ny;
char *type;
{
  int nx, ny;
  int npixels; /* nx*ny */
  int fid; /* file ID for FITS reading subroutines */
  int status;
  int i, j;

  fid = im_open(filename, "r");
  if (fid < 0)
  {
     printf("Error opening or reading file %s.\n", filename);
     return TERMINATE_ERROR;
  }

  im_rkey_i(fid, &nx, "NAXIS1");
  im_rkey_i(fid, &ny, "NAXIS2");
  npixels = nx * ny;
  *ret_nx = nx;
  *ret_ny = ny;

  /* allocate memory */
  if (!strcmp(type, "double"))
     *ptr_pixels = (void **) malloc((unsigned)(npixels * sizeof(double)));

  if (!strcmp(type, "short"))
     *ptr_pixels = (void **) malloc((unsigned)(npixels * sizeof(short)));
 
  if (*ptr_pixels == NULL) 
  {
     printf("Can't allocate required memory\n");
     return TERMINATE_ERROR;
  }

  /* Read in the data */
  if (!strcmp(type, "double"))
     status = im_rpix_d(fid, *ptr_pixels, npixels);
  
  if (!strcmp(type, "short"))
     status = im_rpix_s(fid, *ptr_pixels, npixels);
  
  if (status <0) return TERMINATE_ERROR;
  return 0;
}

int main(argc, argv)
int argc;
char **argv;
{
  int status;
  status = extract(argc, argv);
  if (status < 0) return TERMINATE_ERROR;
  param_exit(status, PROGRAM);
}

