/*********************************************** 
flatap_compute_results.c

Purpose


Revision History:

   Jan-14-2004   W. Lee: Modifications per AR 6107  
   June-15-2004  W. Lee: Took over module ownership and implemented changes   
                         per CR 5561.
                         Many confusions on the original logic as well as 
                         coding bugs.


Main processing loop of program.

***********************************************/

#include <stdio.h>
#include "uprop.h"
#include "flatap.h"
#include "flatap_defs.h"

int flatap_compute_results(Dims   *Dim_In,
		       float   *data,
		       float   *sigma,
		       Status  *Stat,
		       short   *pmask,
		       short   *dmask,
		       short   *cmask,
		       float   *flat,
		       float   *transcal,  /* plane 1 = DATA, 2 = UNC  */
		       short   *cmasktc)    /* 1 plane  */ 
{
  int   index, i, k;
  int   pixels, planes;
  int   pmaskMask, dmaskMask, cmaskMask;
  float   out, osigma;
  float   *stor, *ssigma;   /*  ALLOC only if FLATFILED UNC present  */ 
  int     smask = 0, omask=0;
  uprop_state   status;

  int   nandata, nanflat, pmask_fatal, dmask_fatal;
  int   cmask_fatal, cmask_transcal_fatal ;  


  if (Stat->I_status) return (-1);

  pixels = Dim_In->I_NX*Dim_In->I_NY;
  planes = Dim_In->I_NZ;
  pmaskMask = Stat->PmaskMask;   /*  default  =   128, bit  7  */
  dmaskMask = Stat->DmaskMask;   /*  default  = 16384, bit 14  */
  cmaskMask = Stat->CmaskMask;   /*  default  =   256, bit  8  */    


 /*  If Flatfield uncertainty  is present    */

  if  (Stat->uncertainty == TRUE) 
  {

     if ((stor = (float *) malloc(pixels*sizeof(float)))==NULL) 
     {
        Stat->I_status = COMPUTE | MALLOC_FAILED;
        return(-1);
     }

     if ((ssigma = (float *) malloc(pixels*sizeof(float)))==NULL) 
     {
        Stat->I_status = COMPUTE | MALLOC_FAILED;
        free(stor);
        return(-1);
     }
    

    /*  Transmission CAL:    
         
         ABSENT: 
            set 
             STOR = FLAT & FLAT_UNC         
             SSIGMA =  FLAT_UNC

         PRESENT: 
            compute  (STOR, SSIGMA) using 

                (FLAT, FLATUNC)      
             &  (TRANSCAL,  TRANSCAL_UNC) 
             
     */

     if  (transcal == NULL)   /*   TRANSCAL = Absent  */ 
     {
        for (i=0; i<pixels; i++) 
        {
	   stor[i] = flat[i];
	   ssigma[i] = flat[i+pixels];
        }
     }
     else   /*  TRANSCAL =  Present   */ 
     {
        for(i=0; i<pixels; i++) 
        {
           /*  stor[i] is  normalized flat !  */

          /*  compute 'normalized' FLAT  */

	   uprop_mul (&(stor[i]), &(ssigma[i]), &smask,
		       flat[i], flat[i+pixels], cmask[i],
		   transcal[i], transcal[i+pixels], cmasktc[i]);
        } 
     }


    /*   FLAT UNC present scheme:   
           Looping over pixels   */


    for  (k=0; k<planes; k++) 
    { 
       for (i=0; i<pixels; i++) 
       {

	  index = k*pixels + i;   /* position in cube */
	
         /*
	    printf ("index[%d] data[%f] flat[%f] dmask[%d] ", );
                     index, data[index], flat[index], dmask[index]) ; 
            printf (" pmask[%d] cmask[%d] cmasktc[%d]\n", 
	           pmask[index], cmask[index], cmasktc[index]); 
         */


          /*  Initialiaze FLAGs to 'GOOD' for each pixel  */ 

          nandata = NO ;  
          nanflat = NO ;
          pmask_fatal = NO ;
          dmask_fatal = NO ;
          cmask_fatal = NO ;
          cmask_transcal_fatal = NO ;


          /*  Collect possibly 'BAD' FLAGs   */  

          if  (iznand ((double) data[index]) != 0)  
             nandata = YES ;        

          if  (iznand ((double) flat[index]) != 0) 
             nanflat = YES ;        

          if  ((pmask[i] & pmaskMask) != 0)      
             pmask_fatal = YES ;  

          if  ((dmask[index] & dmaskMask) != 0)   
             dmask_fatal = YES ; 

          if  ((cmask[i] & cmaskMask) != 0)    
             cmask_fatal = YES ; 

          if  (transcal != NULL) 
          {
	     if ((cmasktc[i] & cmaskMask) != 0)
                cmask_transcal_fatal = YES ; 
          }

          if  (nandata == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }

          if  (pmask_fatal == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }

          if  (dmask_fatal == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }
 
          if  (cmask_fatal == YES)
          {
            if (nanflat == YES)
            {
	       dmask [index] |= FLATFIELD_NO_APPLY;
               continue ;
            }

            if  (nanflat == NO)
	       dmask[index] |= FLATFIELD_QUESTIONABLE ;
          } 

/*
          if  (cmask[i]  &&  (cmask_fatal == NO))
          {
	     dmask[index] |= FLATFIELD_QUESTIONABLE ;
          }
*/

          if (cmask_transcal_fatal == YES)
          {
             if (nanflat == YES)
             {
	        dmask[index] |= FLATFIELD_NO_APPLY;
                continue ;
             }

             if (nanflat == NO)
	        dmask[index] |= FLATFIELD_QUESTIONABLE ;
          } 

/*
          if  (cmasktc[i]  &&  cmask_transcal_fatal == NO)
          {
	     dmask[index] |= FLATFIELD_QUESTIONABLE ;
          }
*/

	   /* apply flat */

	  uprop_div(&out, &osigma, &omask,
	            data[index], sigma[index], smask,
	            stor[i], ssigma[i], smask);

         /* 
           printf ("index[%d] ", index) ;
           printf (" databefore[%f] ", data[index]) 
           printf (" dataafter[%f]\n", out);
           printf (" flat[%f]\n", stor[i]); 
         */ 

          data[index] = out;
	  sigma[index] = osigma;

      }    /*  i=0, pixels  Loop */

    }      /*   k=0, planes  Loop  */ 

    free(stor);
    free(ssigma);

  }   /*  If FLAT UNC  =  PRESENT   */  


 /* *************************************************** */
 /* *************************************************** */
 /* *************************************************** */
 /* *************************************************** */


  /*  Flat  UNC is  NOT Present  !  */ 

  else   
  {

    for (k=0; k<planes; k++) 
    {
       for (i=0; i<pixels; i++) 
       {
	  index = k*pixels + i;   /* position in cube */


        /*  Initialiaze FLAGs to 'GOOD' for each pixel  */ 

          nandata = NO ;  
          nanflat = NO ;
          pmask_fatal = NO ;
          dmask_fatal = NO ;
          cmask_fatal = NO ;
          cmask_transcal_fatal = NO ;

        /*  Collect  possibly 'BAD' flags   */

          if  (iznand ((double) data[index]) != 0)        
             nandata = YES ;    	

          if  (iznand ((double) flat[index]) != 0)        
             nanflat = YES ;    	

          if  ((pmask[i] & pmaskMask) != 0)    
             pmask_fatal = YES ; 

          if  ((dmask[index] & dmaskMask) != 0)  
             dmask_fatal = YES ; 

          if  ((cmask[i] & cmaskMask) != 0)    
             cmask_fatal = YES ; 

          if  (transcal != NULL) 
          {
	     if ((cmasktc[i] & cmaskMask) != 0)
                cmask_transcal_fatal = YES ; 
          }

          if  (nandata == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }

          if  (pmask_fatal == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }

          if  (dmask_fatal == YES)
          {
	     dmask[index] |= FLATFIELD_NO_APPLY;
             continue ;
          }
 
          if  (cmask_fatal == YES)
          {
            if (nanflat == YES)
            {
	       dmask [index] |= FLATFIELD_NO_APPLY;
               continue ;
            }

            if  (nanflat == NO)
	       dmask[index] |= FLATFIELD_QUESTIONABLE ;
          } 
/*
          if  (cmask[i]  &&  (cmask_fatal == NO))
          {
	     dmask[index] |= FLATFIELD_QUESTIONABLE ;
          }
*/

          if (cmask_transcal_fatal == YES)
          {
             if (nanflat == YES)
             { 
	       dmask[index] |= FLATFIELD_NO_APPLY;
               continue ;
             }
             if (nanflat == NO)
               dmask[index] |= FLATFIELD_QUESTIONABLE ;
          } 

/*
          if  (cmasktc[i]  &&  cmask_transcal_fatal == NO)
          {
	     dmask[index] |= FLATFIELD_QUESTIONABLE ;
          }
*/

	      /* apply flat */

          if (transcal != NULL)
            data[index] /= (flat[i]*transcal[i]);

          if (transcal == NULL)
/*            data[index] /= (flat[i] *1.0);   */
            data[index] /= flat[i] ;   
 


       }   /*  the i= 0, pixel  loop  */

    }   /*  the k=0, planes loop  */

  }   /*  UNC is NOT present  */   
  

  if  (Stat->I_Verbose)
  {
    printf ("flatap_compute_results: ");
    printf ("Numerical computations finished.\n");
  }
  
  return(0);
}
