/*********************************************** 
flatap_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of flatap functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_flatap_message = Code of error that occurred.
I_flatap_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "flatap.h"
#include "flatap_defs.h"


int flatap_error_handler (Status *Stat, Filenames *Fnames)
{
  int	I_Time;
  int	I_flatap_message;
  int	I_flatap_root;
  int	I_cfitsio_message=0;
  
  char	CP_Message[I_MESSAGE_LENGTH];
  char	CP_Function[I_FCNNAME_LENGTH];
  
  
  I_flatap_message = RIGHT_MASK&(Stat->I_status);
  I_flatap_root = (LEFT_MASK&(Stat->I_status))>>16;
  
  if(I_flatap_message&FITS_MSG_MASK)	{
    I_flatap_message -= FITS_MSG_MASK;
  }
  
  flatap_decode_message(I_flatap_message, CP_Message);
  flatap_decode_fcnname(I_flatap_root, CP_Function);
  
  if (Stat->I_status) {
    printf("flatap Error Message      0x%04x: %s.\n", I_flatap_message, CP_Message);
    printf("flatap Error in Function  0x%04x: %s.\n", I_flatap_root, CP_Function);
  }
  
  
  I_Time = clock();
  printf("Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);
  
  if(Stat->I_Verbose)
    printf("flatap_error_handler: Error Handler finished.\n");
 
  return(0);
}
