/*********************************************** 
flatap_exec.c

Purpose

Calls the initialization functions and main loop of program. Saves
output to a file.


Overview



Definitions of Variables 
See flatap_defs.h
 
***********************************************/

#include <time.h>
#include <stdio.h>
#include "flatap.h"
#include "flatap_defs.h"

int main(int argc, char **argv)
{			
  float*  FP_Data_In=NULL;
  float*  sigma=NULL;
  short*  SP_pmask=NULL;
  short*  SP_dmask=NULL;
  short*  SP_cmask=NULL;
  short*  SP_cmasktc=NULL;
  float*  FP_Flat=NULL;
  float*  FP_Transcal=NULL;
  
  Filenames   Fnames;
  Status      Stat;
  Dims        Dim_In;
  struct tm   *tmp_time;
  time_t      L_time;
  
  clock();
  
  printf("Pipeline Module FLATAP Version %3.1f\n", VERSN);
  
  /* Calculate and print to standard out time
     Keep tmp_time and module time for time
     time stamping all output
  */
  L_time = time(NULL);
  tmp_time = localtime(&L_time);
  printf("Processing time ");
  printf(asctime(tmp_time));
  printf("\n");
  
  /* Set constant default values */
  flatap_init_constants(&Fnames, &Stat, &Dim_In);
  
  /* look for a namlist file in the command line */
  /* parse the namelist if it exists */
  flatap_parse_namelist(argc, argv, &Fnames, &Stat);
  
  /* Parse the command line, replacing values if specified */
  flatap_parse_args( argc, argv, &Fnames, &Stat);
  
  /* Get the data and keyword values from the input file */
  flatap_read_data(&Fnames, &Dim_In, &FP_Data_In, &sigma, &Stat,
		   &SP_pmask, &SP_dmask, &SP_cmask, &FP_Flat, &FP_Transcal,
		   &SP_cmasktc);

  /* Perform numerical computations on the data */
  flatap_compute_results(&Dim_In, FP_Data_In, sigma, &Stat, 
			 SP_pmask, SP_dmask, SP_cmask, FP_Flat, FP_Transcal,
			 SP_cmasktc);

  /* Save the results as fits files. */			
  flatap_output( &Fnames, &Dim_In, &Stat, tmp_time, SP_dmask,
		 FP_Data_In, sigma);
  
  /*  Write processing log   */

  flatap_write_log  (&Fnames, &Stat) ;


  /* Print filename, error message and timing data */
  flatap_error_handler( &Stat, &Fnames);
  
  /* Deallocate memory */			
  flatap_exit( FP_Data_In, sigma, &Dim_In, &Stat, SP_pmask, SP_dmask,
	       SP_cmask, SP_cmasktc, FP_Flat, FP_Transcal);
  
  printf("exit code %d\n", Stat.I_status);
  
  /* Send return value */
  exit(Stat.I_status);
  
}
