/*********************************************** 
flatap_exit.c

Purpose

Cleanly exit the program.

Overview
Deallocate memory and print any diagnostics.

Definitions of Variables

Input:



Output:


Internal:


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "flatap.h"
#include "flatap_defs.h"


int flatap_exit(	
            float     *FPP_Data_In,
	    float     *sigma,
            Dims       *Dim_In,
            Status     *Stat,
            short      *SPP_pmask,
            short      *SPP_dmask,
            short      *SPP_cmask,
            short      *SPP_cmasktc,
            float      *FPP_Flat,
            float      *FPP_Transcal)
{
  /* allocated from flatap_read_data */
  if (FPP_Data_In != NULL) free(FPP_Data_In);
  if (FPP_Flat != NULL) free(FPP_Flat);
  if (FPP_Transcal != NULL) free(FPP_Transcal);
  if (SPP_pmask != NULL) free(SPP_pmask);
  if (SPP_dmask != NULL) free(SPP_dmask);
  if (SPP_cmask != NULL) free(SPP_cmask);
  if (SPP_cmasktc != NULL) free(SPP_cmasktc);
  
  /* Set up exit status for pipeline.
     For now any non zero status will be output
     as a failure.  There are no warnings.
  */
  
  if(Stat->I_status) 
    Stat->I_status = 64;
  
  if(Stat->I_Verbose)
    printf("flatap_exit: Memory Deallocated.\n");
  
  return(0);
}
