/*********************************************** 
flatap_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Revision History:

  Jan-14-2004  W. Lee:  Added Logfile per CR 6107.


Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output Probabilities File Name.
IP_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include "flatap.h"
#include "flatap_defs.h"


int flatap_parse_args(	
		  int	        argc,
		  char        **argv,
		  Filenames   *Fnames,
		  Status	*Stat)
{
  int	i=1;
  float   xtest;
  
  int     I_out_chk = 0;
  int     I_in_chk = 0;
  int     I_other_chk = 0;
  int     I_flat_chk = 0;
  
  /* Run regardless of I_flatap_status so that
     at least log file name will be known */
  
  I_out_chk = 1;
  I_in_chk = 1;
  I_flat_chk = 1;
  I_other_chk = 0; 
  
  while( i < argc ) {
    if( argv[i][0] == '-' ) {
      switch( argv[i][1] ) {
      case 'n':      /* -n <namelist_fname> */
	if(++i >= argc)
	  I_other_chk = 1;
	else {
	  I_other_chk = 0;
	  sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
	}
	break;
      case 'p':      /* -p <pmask_fname>  */
	if ( ++i >= argc )
	  I_other_chk = 1;
	else {
	  I_other_chk = 0;
	  if(argv[i-1][2] == '\0') 
	    sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
	  else if(argv[i-1][2] == 'm')
	    sscanf(argv[i], "%d", &(Stat->PmaskMask));
	  else
	    I_other_chk = 1;
	}
	break;
      case 'd':      /* -d <dmask_fname>  */
	if ( ++i >= argc )
	  I_other_chk = 1;
	else {
	  I_other_chk = 0;
	  if(argv[i-1][2] == '\0')
	    sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
	  else if(argv[i-1][2] == 'm')
	    sscanf(argv[i], "%d", &(Stat->DmaskMask));
	  else
	    I_other_chk = 1;
	}
	break;

      case 'i':      /* -i <input_image_fname>  */
	if ( ++i >= argc )
	  I_in_chk = 1;
	else {
	  sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
	  I_in_chk = 0;
	}
	break;

      case 'l':      /* -l <proclog_fname>  */
	if ( ++i >= argc )
	  I_other_chk = 1;
	else {
	  sscanf(argv[i], "%s", Fnames->CP_Filename_Log);
	  I_other_chk = 0;
	}
	break;


      case 'o':      /* -o <output_fname>  */
	if ( ++i >= argc )
	  I_out_chk = 1;
	else {
	  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out);
	  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
	  I_out_chk = 0;
	}
	break;
      case 'f':      /* -f <flat field file name>  */
	if ( ++i >= argc )
	  I_flat_chk = 1;
	else {
	  sscanf(argv[i], "%s", Fnames->CP_Filename_Flat);
	  I_flat_chk = 0;
	}
	break;
      case 't':      /* -t <transmission calibrator flat file name>  */
	if ( ++i >= argc )
	  I_other_chk = 1;
	else
	  sscanf(argv[i], "%s", Fnames->CP_Filename_Transcal);
	break;
      case 'u':      /* uncertainty images */
	if(argv[i][2] == 'o') {
	  if(++i>=argc)
	    I_other_chk = 1;
	  else {
	    sscanf("!", "%c", Fnames->outsigma);
	    sscanf(argv[i], "%s", &(Fnames->outsigma[1]));
	  }
	} else if(argv[i][2] == 'i') {
	  if(++i>=argc)
	    I_other_chk = 1;
	  else
	    sscanf(argv[i], "%s", Fnames->insigma);
	} else
	  I_other_chk = 1;
	break;
      case 'c':      /* -c <cmask flat pixel status file name>  */
	if (argv[i][2] == 't') {
	  /*Transmission calibrator pixel status file name */
	  if ( ++i >= argc )
	    I_other_chk = 1;
	  else
	    sscanf(argv[i], "%s", Fnames->CP_Filename_Cmasktc);
	} else if (argv[i][2] == '\0') {
	  /*normal flat field pixel status file name */
	  if ( ++i >= argc )
	    I_other_chk = 1;
	  else {
	    if(argv[i-1][2] == '\0')
	      sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
	    I_other_chk = 0;
	  }
	} else if (argv[i][2] == 'm') {
	  /* cmask Mask */
	  if(++i >= argc)
		I_other_chk = 1;
	  else {
	    sscanf(argv[i], "%d", &(Stat->CmaskMask));
	    I_other_chk = 0;
	  }
	} else
	  I_other_chk = 1;
	break;
      case 'v':      /* -v  verbose mode */
	Stat->I_Verbose = 1 - Stat->I_Verbose;
	break;
      default:
	printf("flatap_parse_args: unknown argument\n");
      }
    }
    else {
      printf("flatap_parse_args: command line syntax error");
    }
    i++;
  }
  

  if(strcmp(Fnames->CP_Filename_FITS_In,"")==0) {
    printf("flatap_parse_args: Missing required information - -i <input_file>\n");
    I_in_chk = 1;
  } else
    I_in_chk = 0;

  if(strcmp(Fnames->CP_Filename_FITS_Out,"")==0) {
    printf("flatap_parse_args: Missing required information - -o <output_file>\n");
    I_out_chk = 1;
  } else
    I_out_chk = 0;

  if(strcmp(Fnames->CP_Filename_Flat,"")==0) {
    printf("flatap_parse_args: Missing required information - -f <flafield_file>\n");
    I_flat_chk = 1;
  } else
    I_flat_chk = 0;


  if (I_in_chk || I_flat_chk || I_other_chk || I_out_chk) { 
    Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
  } 
  
  if(Stat->I_Verbose)	{
    printf("flatap_parse_args: Input DCE  = \t%s\n", 
	   Fnames->CP_Filename_FITS_In);
    printf("flatap_parse_args: Ouput DCE  = \t%s\n", 
	   Fnames->CP_Filename_FITS_Out);
    printf("flatap_parse_args: flatfield  = \t%s\n",
	   Fnames->CP_Filename_Flat);
    printf("flatap_parse_args: Input Uncertainty = \t%s\n",
	   Fnames->insigma);
    printf("flatap_parse_args: Output Uncertainty = %s\n",
	   Fnames->outsigma);
    printf("flatap_parse_args: Pmask File = \t%s\n", 
	   Fnames->CP_Filename_Pmask);
    printf("flatap_parse_args: Pmask Mask = \t%d\n",
	   Stat->PmaskMask);
    printf("flatap_parse_args: Dmask File = \t%s\n", 
	   Fnames->CP_Filename_Dmask);
    printf("flatap_parse_args: Dmask Mask = \t%d\n",
	   Stat->DmaskMask);
    printf("flatap_parse_args: Cmask File = \t%s\n", 
	   Fnames->CP_Filename_Cmask);
    printf("flatap_parse_args: Cmask Mask = \t%d\n",
	   Stat->CmaskMask);
    printf("flatap_parse_args: Trans. Calibrator File = \t%s\n", 
	   Fnames->CP_Filename_Transcal);
    printf("flatap_parse_args: Trans. Calibrator Cmask File = \t%s\n", 
	   Fnames->CP_Filename_Cmasktc);
    printf("flatap_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
  }

  return(0);
}
