/*********************************************** 


   Name:  flatap_write_log.c

   Purpose:

     Decode error messages and report.


   Revision History:

     Jan-14-2004  W. Lee:  Initial delivery per CR 6107.


   Overview:

     Append Error Messages to log file.

   Definitions of Variables:

     External:


     Internal:

       I_Time = Running time of program.
       fp_Logfile = File pointer of logfile.


***********************************************/

#include <time.h>
#include <stdio.h>
#include "flatap.h"
#include "flatap_defs.h"


FILE    *fp_Logfile=0;
static int done_header=0;

void  open_logfile(Filenames	*Fnames) {

/*
  if(fp_Logfile != 0)
    return;
*/
  if  ( strcmp (Fnames->CP_Filename_Log, 
		"stdout") == 0 )	
    {
      fprintf (stdout, "flatap_log_writer:");
      fprintf (stdout, " Printing Log output to stdout\n");
      fp_Logfile = stdout;
    }

  else	
    {
      fp_Logfile 
	= fopen (Fnames->CP_Filename_Log, "a");

      if  (fp_Logfile == NULL)	
	{
          fprintf (stdout, "flatap_log_writer: Could ");
          fprintf (stdout, "not open Log File, using stdout\n");
          fp_Logfile = stdout;
	}
    }
  if(!done_header) {
    fprintf (fp_Logfile, "Flatap Version %3.1f\n", VERSN);
    done_header=0;
    }
}

void write_log_line(char* line,  Filenames     *Fnames) 
{
  if(line==NULL)
    return;
  open_logfile(Fnames);
  fprintf  (fp_Logfile,"flatap log:%s\n",line);
}


flatap_write_log   ( Filenames      	*Fnames,
	             Status	        *Stat )
{
  int	     I_Time  ;


  /* open the log file unless its already open */

  open_logfile  (Fnames);

  fprintf (fp_Logfile, "Namelist File  = %s\n", 
	   Fnames->CP_Filename_Namelist);

  fprintf (fp_Logfile, "Input DCE = %s\n", 
	   Fnames->CP_Filename_FITS_In);

  fprintf (fp_Logfile, "Output DCE = %s\n", 
	   Fnames->CP_Filename_FITS_Out);

  fprintf (fp_Logfile, "Flatfield = %s\n", 
	   Fnames->CP_Filename_Flat);

  fprintf (fp_Logfile, "Input Uncertainty = %s\n", 
	   Fnames->insigma);

  fprintf (fp_Logfile, "Output Uncertainty = %s\n", 
	   Fnames->outsigma);

  fprintf (fp_Logfile, "Pmask File = %s\n", 
	   Fnames->CP_Filename_Pmask);

  fprintf (fp_Logfile, "Pmask Mask = %d\n", 
	   Stat->PmaskMask);

  fprintf (fp_Logfile, "Dmask File = %s\n", 
	   Fnames->CP_Filename_Dmask);

  fprintf (fp_Logfile, "Dmask Mask = %d\n", 
	   Stat->DmaskMask);

  fprintf (fp_Logfile, "Cmask File = %s\n", 
	   Fnames->CP_Filename_Cmask);

  fprintf (fp_Logfile, "Cmask Mask = %d\n", 
	   Stat->CmaskMask);

  fprintf (fp_Logfile, "Trans CAL File = %s\n", 
	   Fnames->CP_Filename_Transcal);

  fprintf (fp_Logfile, "Trans CAL CMask File = %s\n", 
	   Fnames->CP_Filename_Cmasktc);

  fprintf (fp_Logfile, "Processing Log File = %s\n", 
	   Fnames->CP_Filename_Log);


  I_Time = clock();
  fprintf (fp_Logfile, "Time = %7.2f seconds\n*********************\n\n", 
	   (float)I_Time/CLOCKS_PER_SEC);

  if  (fp_Logfile != stdout) 
      fclose (fp_Logfile);
  fp_Logfile=0;

  if  (Stat->I_Verbose)
    printf  ("flatap_log_writer: Log Writer finished.\n");


}
