c=======================================================================
c
c     $Id: hdrupd8.F,v 1.4 2007/08/21 20:20:07 jwf Exp $
c
c   HdrUpd8: FITS Header Update Utility - update a FITS file by adding
c    header lines read from a specified file and deleting header lines
c    with keywords read from a specified file
c  vsn 1.0  -  original version
c  vsn 1.1  A01121: fixed bug in delete function
c  vsn 1.2  A01129: added "&" codes and special NAXIS handling; added
c                   COMMENT history line and "-n" option
c  vsn 1.3  A20918: changed COMMENT to standardized HISTORY line for vsn ID
c  vsn 1.4  A40312: added linux compatibility
c  vsn 1.5  A50126: installed more robust detection of 'END'
c  vsn 1.6  A70821: installed '&E' code for insertion before END
c
      Integer*4     IArgC, LNBlnk, FileID1, NPix, N,
     +              NPlanes, MAlloc, NRows, NCols, Access,
     +              IStat, ImRKeyC, ImWPixR, ImClose,
     +              ImRPixR, ImWKeyT, ImRKeyT, ImRKeyI,
     +              ImRKeyL, ImRKeyD, ImRKeyR, JDate(3), JTime(3)
      Character*150 InFNam,AddFNam,DelFNam
      Character*80  HdrLine, C
      Character*11  Vsn
      Character*8   Instrume, CDate, CTime, Keyword
      Character*3   TmpStr
      Character*1   AddCode
      Logical       DoAdd, DoDel, L, AddCmt
      Real*4        RCube(*), R
      Real*8        D
      Pointer       (pR, RCube)
      Data          DoAdd/.False./,DoDel/.False./,AddCmt/.True./,
     +              Vsn/'1.6  A70821'/, AddCode/'A'/
c
c=======================================================================
c
      print *,'hdrupd8 vsn ',Vsn
      If ((IArgC() .lt. 3) .or. (IArgC() .gt. 6))then
        print *
        print *,
     +   'Usage:  hdrupd8 infnam <-a|add addfnam> <-d|del delfnam> <-n>'
        print *
        print *,'Where infnam  = name of input FITS file'
        print *,'                (".fits" is optional; if present,'
        print *,'                 it must come only at the end)'
        print *,'   -a addfnam ==> add header lines in the file'
        print *,'                  named "addfnam" to the header'
        print *,'   -d delfnam ==> delete header lines with keywords'
        print *,'                  listed in the file named "delfnam"'
        print *,'   -n         ==> don''t add processing COMMENT line'
        print *
        print *,'The command-line parameters "add addfnam" and'
        print *,'"del delfnam" are both optional and may be in'
        print *,'either order. This program handles only REAL*4'
        print *,'files with either NAXIS = 2 or NAXIS = 3.'
        print *,'The "-n" option, if used, must be last.'
        stop
      end if
c
      call GetArg(1,InFNam)
      if (Index(InFNam,'.fits') .eq. 0)
     +  InFNam = InFNam(1:LNBlnk(InFNam))//'.fits'
      if (Access(InFNam(1:LNBlnk(InFNam)),' ') .ne. 0) then
        print *,'File not found: ',InFNam(1:LNBlnk(InFNam))
        call exit(64)
      end if
      call GetArg(2,TmpStr)
      if ((TmpStr .eq. 'add') .or. (TmpStr .eq. '-a')) then
        call GetArg(3,AddFNam)
        if (Access(AddFNam,' ') .ne. 0) then
          print *,'Add file not found: ',AddFNam
          call exit(64)
        end if
        DoAdd = .True.
      else if ((TmpStr .eq. 'del') .or. (TmpStr .eq. '-d')) then
        call GetArg(3,DelFNam)
        if (Access(DelFNam,' ') .ne. 0) then
          print *,'Del file not found: ',DelFNam
          call exit(64)
        end if
        DoDel = .True.
      else
        print *,'Illegal command-line parameter: ',TmpStr
        stop
      end if
      if (IArgC() .eq. 4) then
        call GetArg(4,TmpStr)
        if (TmpStr .eq. '-n') AddCmt = .False.
      end if
      if (IArgC() .ge. 5) then
        call GetArg(4,TmpStr)
        if ((TmpStr .eq. 'add') .or. (TmpStr .eq. '-a')) then
          call GetArg(5,AddFNam)
          if (Access(AddFNam,' ') .ne. 0) then
            print *,'Add file not found: ',AddFNam
            call exit(64)
          end if
          DoAdd = .True.
        else if ((TmpStr .eq. 'del') .or. (TmpStr .eq. '-d')) then
          call GetArg(5,DelFNam)
          if (Access(DelFNam,' ') .ne. 0) then
            print *,'Del file not found: ',DelFNam
            call exit(64)
          end if
          DoDel = .True.
        else
          print *,'Illegal command-line parameter: ',TmpStr
          stop
        end if
      end if
      if (IArgC() .eq. 6) then
        call GetArg(6,TmpStr)
        if (TmpStr .eq. '-n') AddCmt = .False.
      end if
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      CDate = '00/00/00'
#ifdef __INTEL_COMPILER
      Call IDate(JDate(1),JDate(2),JDate(3))    ! Linux call
#else
      Call IDate(JDate)                         ! Solaris call
#endif
      JDate(3) = Mod(JDate(3), 100)
      Write(CDate(1:2), '(I2)') JDate(2)
      Write(CDate(4:5), '(I2)') JDate(1)
      Write(CDate(7:8), '(I2)') JDate(3)
c
      CTime = '00:00:00'
      Call ITime(JTime)
      Write(CTime(1:2), '(I2)') JTime(1)
      Write(CTime(4:5), '(I2)') JTime(2)
      Write(CTime(7:8), '(I2)') JTime(3)
c
      If (CDate(4:4) .eq. ' ') CDate(4:4) = '0'
      If (CDate(7:7) .eq. ' ') CDate(7:7) = '0'
      If (CTime(4:4) .eq. ' ') CTime(4:4) = '0'
      If (CTime(7:7) .eq. ' ') CTime(7:7) = '0'
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      Call GetNAX(InFNam,NCols,NRows,NPlanes,1,FileID1)
      NPix = NCols*NRows*NPlanes
      pR = MAlloc(4*NPix)              ! Allocate mem for input
      If (pR .eq. 0) then
        print *,'Unable to allocate memory for ',
     +           InFNam(1:LNBlnk(InFNam))
        stop
      End If
      IStat = ImRPixR(FileID1,RCube,NPix)
      If (IStat .ne. NPix) then
        print *,'ERROR: could not read input file ',
     +          InFNam(1:LNBlnk(InFNam))
        print *,'       ImRPixR returned error code ',IStat
        stop
      end if
c
      If (DoDel) then
        Open (20,File = DelFNam)
10      Read (20, 20, End = 100, Err = 3000) KeyWord
20      Format(A8)
        IStat = ImRKeyT(FileID1,HdrLine,'1')
25      If (HdrLine(1:8) .eq. KeyWord(1:8)) then
          IStat = ImWKeyT(FileID1,'','D')
          Go to 10
        Else
          If (HdrLine(1:LNblnk(HdrLine)) .eq. 'END') then
            print *,'WARNING: keyword to be deleted not found: ',KeyWord
            Go to 10
          Else
            IStat = ImRKeyT(FileID1,HdrLine,'+')
            If (IStat .ge. 0) Go to 25
            print *,'WARNING: keyword to be deleted not found: ',KeyWord
          End If
        End If
        Go to 10
      End If
c
100   If (DoAdd) Then
        IStat = ImRKeyT(FileID1,HdrLine,'1')
        IStat = ImRKeyC(FileID1,Instrume,'INSTRUME')
        If (IStat .lt. 0) Then
          If (NPlanes .eq. 1) IStat = ImRKeyI(FileID1,N,'NAXIS2')
          If (NPlanes .gt. 1) IStat = ImRKeyI(FileID1,N,'NAXIS3')
        End If
        Open (21,File = AddFNam)
110     Read (21, 120, End = 200, Err = 3001) HdrLine
120     Format(A80)
        If (HdrLine(1:8) .eq. 'NAXIS ')
     +      IStat = ImRKeyI(FileID1,N,'BITPIX')
        If (HdrLine(1:1) .eq. '&') then
          If (HdrLine(2:2) .eq. 'R')
     +        IStat = ImRKeyR(FileID1,R,HdrLine(5:12))
          If (HdrLine(2:2) .eq. 'I')
     +        IStat = ImRKeyI(FileID1,N,HdrLine(5:12))
          If (HdrLine(2:2) .eq. 'D')
     +        IStat = ImRKeyD(FileID1,D,HdrLine(5:12))
          If (HdrLine(2:2) .eq. 'L')
     +        IStat = ImRKeyL(FileID1,L,HdrLine(5:12))
          If (HdrLine(2:2) .eq. 'C')
     +        IStat = ImRKeyC(FileID1,C,HdrLine(5:12))
          If (HdrLine(2:2) .eq. 'E') then
              AddCode = 'E'
          Else
              AddCode = 'A'
          End If
          If (HdrLine(3:3) .eq. '-')
     +        IStat = ImRKeyT(FileID1,C,'-')
          Read (21, 120, End = 200, Err = 3001) HdrLine
        End If
        IStat = ImWKeyT(FileID1,HdrLine,AddCode)
        Go to 110
      End If
c
200   If (AddCmt) then
        IStat = ImWKeyT(FileID1,'HISTORY hdrupd8           v.  '
     +        //Vsn//',  updated  '//CDate//' at '//CTime,'E')
        If (DoDel) then
          N = LNBlnk(DelFNam) - 56
          If (N .lt. 1) then
           N = 1
          Else
            DelFNam(N:N+2) = '...'
          End If
          IStat = ImWKeyT(FileID1,
     +     'COMMENT      del file: '//DelFNam(N:LNBlnk(DelFNam)),'E')
        End If
        If (DoAdd) then
          N = LNBlnk(AddFNam) - 56
          If (N .lt. 1) then
           N = 1
          Else
            AddFNam(N:N+2) = '...'
          End If
          IStat = ImWKeyT(FileID1,
     +     'COMMENT      add file: '//AddFNam(N:LNBlnk(AddFNam)),'E')
        End If
      End If
c
      IStat = ImWPixR(FileID1,RCube,NPix)
      If (IStat .ne. NPix) then
        print *,'Unable to generate output file '
        print *,'Error code: ',IStat
        stop
      End If
      IStat = ImClose(FileID1)
c
      Stop
c
3000  print *,'READ ERROR in del file'
      stop
3001  print *,'READ ERROR in add file'
      stop
c
      end
c
c=======================================================================
c
      Subroutine GetNAX(FilNam,NAXIS1,NAXIS2,NAXIS3,IDOp,FileID)
c-----------------------------------------------------------------------
c
c    Gets the dimensions of the file named FilNam; if IDOp = 0, the
c    file is closed, otherwise it's left open with handle FileID
c
c-----------------------------------------------------------------------
c
      Character*150 FilNam
      Integer*4     NAXIS, NAXIS1, NAXIS2, NAXIS3, IStat, ImOpen,
     +              ImRKeyI, FileID, IDOp, LNBlnk, ImClose
c
c-----------------------------------------------------------------------
c
      FileID = ImOpen (FilNam, "U")
      If (FileID .lt. 1) Then
        print *,'Error opening ',FilNam(1:LNBlnk(FilNam))
        print *,'ImOpen returned code: ',FileID
        Stop
      End If
c
      IStat = ImRKeyI(FileID,NAXIS,'NAXIS')
      If (IStat .lt. 1) then
        print *,'Error reading NAXIS in ',FilNam(1:LNBlnk(FilNam))
        print *,'ImRKeyI returned code: ',IStat
        Stop
      End If
c
      IStat = ImRKeyI(FileID,NAXIS1,'NAXIS1')
      If (IStat .lt. 1) then
        print *,'Error reading NAXIS1 in ',FilNam(1:LNBlnk(FilNam))
        print *,'ImRKeyI returned code: ',IStat
        Stop
      End If
c
      IStat = ImRKeyI(FileID,NAXIS2,'NAXIS2')
      If (IStat .lt. 1) then
        print *,'Error reading NAXIS2 in ',FilNam(1:LNBlnk(FilNam))
        print *,'ImRKeyI returned code: ',IStat
        Stop
      End If
c
      If (NAXIS .gt. 2) then
        IStat = ImRKeyI(FileID,NAXIS3,'NAXIS3')
        If (IStat .lt. 1) then
          print *,'Error reading NAXIS3 in ',FilNam(1:LNBlnk(FilNam))
          print *,'ImRKeyI returned code: ',IStat
          Stop
        End If
      Else
        NAXIS3 = 1
      End If
c
      If (IDOp .eq. 0) IStat = ImClose(FileID)
c
      return
c
      end
