c=======================================================================
c
c   General-purpose SIRTF ramp-slope/image estimation program; reads
c    data cube with NPlanes exposures; outputs estimated-image and
c    uncertainty data cube.
c
c     $Id: imagest.f,v 1.59 2006/11/03 00:39:07 jwf Exp $
c
c   1.0  990326: initial vsn
c   1.1  A00321: S3 vsn; removed saturation image, include dmask, bmask,
c                and pmask usage
c   1.2  A00905: removed radhit file, use dmask information for radhits;
c                remove ?MaskMax usage, go to general bit test/set
c   1.3  A10322: added IRS saturation-expansion code, operational mode 2
c   1.4  A10503: changed mode 2 to retain nonlinear response
c   1.41 A10510: allow lincal model to have either 2 or 3 planes;
c                handle Alpha = 0 gracefully
c   1.50 A10615: changed uncertainty image to separate output file; added
c                optional uncertainty input (-u) and output (-s); if -u
c                is specified, weighting will be done.
c                NOTE: "Sigma" is the output uncertainty image for mode 1;
c                      "SigImg" is a debug output file with dispersion
c                      information [later made Sigma also for mode 2]
c   1.51 A10817: cosmetic changes suggested by S. Fajardo-Acosta, and
c                fixed two mode-2 dmask bugs he found (not accessed by
c                plane and not updated in disk)
c   1.52 A11012: changed DMBRad default from 5 to 9; now it sets the same
c                bit it uses to detect radhit, hence null operation but
c                consistent with new dmask definitions effective 10/12/01
c   1.53 A11019: preserve samples that are not tagged as saturated; check
c                each pixel's ramp for saturation somewhere before doing
c                anything, bail out if no saturation at any plane*; use a
c                command-overridable threshold for detecting saturation in
c                the same way as cvti2r4, but default to short-circuit; use
c                that and dmask test exclusively for detecting saturation
c                (remove second-differences method, which is for SOFT
c                saturation, whereas we're concerned only with HARD)
c               * (see vsn 1.67 A31024 below)
c   1.54 A11024: added well-depth limit to desaturation
c   1.54 A11025: bug fix - copy unaltered pixels to output array
c   1.54 A11121: bug fix - zero out "Debug" items for mode 2 w/ no sat.
c   1.55 A20307: added BUNIT & CALBUNIT processing
c   1.56 A20328: added uncertainty I/O to mode 2; modified the use of input
c                uncertainty (see SDS); note that Sigma is a single plane
c                in mode 1 and a data cube in mode 2.
c   1.57 A20524: added improved uncertainty estimation for mode 2; fixed
c                NSkip processing to be compatible with zero DN occurring
c                at highest skipped plane.
c   1.58 A20530: changed desaturation uncertainty so that it adds linearly
c                to previous-layer uncertainty iff previous layer was also
c                desaturated, RSS's otherwise.
c   1.59 A20614: added option to use GAIN1-4 or GAIN
c   1.60 A20626: added Dbit/Bbit/MinPln options
c   1.61 A20629: added NSlopLT0 processing
c   1.62 A20720: fixed BUNIT FITS header bug in Sigma output file; fixed
c                bug in mode-2 processing that referenced Unc without
c                testing whether it had been malloc'd
c   1.62 A20729: fixed bug in error message about zero uncertainty
c   1.62 A20730: fixed bug in use of DMaskVal with BitSet; added "-r"
c   1.63 A20815: added continuation of ramp usage above a radhit (CR3210);
c                changed hard-coded bit #9 to DMBRad in radhit test; also
c                fixed chi-square calculation by excluding samples
c                rejected in second pass of slope averaging
c   1.64 A20826: changed handling of first PixDN to be able to use it
c                even when its PixPln is not NSkip+1
c   1.64 A20831: changed scaling in fitlin to var = (err/dpln)^2 instead of
c                var = err^2/dpln; added chisq info image to Debug option
c   1.64 A20907: fixed bug in computation of photon noise in DN; should be
c                DN/GAIN, not GAIN*DN, since variance in e- is GAIN*DN, but
c                this has to be converted back to DN by dividing by GAIN^2
c   1.65 A20918: added QA keywords to output header (CR 3312); changed
c                COMMENT to standardized HISTORY line for vsn ID
c   1.66 A20926: added -t processing; don't update dmask in mode 1; fixed
c                tutorial spec list
c   1.66 A21010: intercept one-sample slope averages so NaNs don't get
c                into the QA summations
c   1.67 A31024: added defaulted option to process ALL ramps in mode 2 even
c                when there's no saturation, in order to do radhit tagging,
c                which Sergio prefers herein to Ted's module; but output the
c                input ramp if no saturation was actually present (DetRad);
c                also fixed bug in -z option
c   1.68 A31114: switched NaN testing to IzNaN function for linux
c   1.69 A40114: default DetRad = F in both modes 1;
c                don't set output = input if output already Nan'd
c   1.70 A40120: scan cmd line to determine mode before processing other
c                flags; if mode 1 and "-n", read &ImagIn NAMELIST; if mode 2
c                and "-n", read &SatCor NAMELIST unless EOF, in which case
c                try for &ImagIn
c   1.71 A40317: completed linux compatibility by changing makefile to link
c                new aloha (from aloha.F) from include library
c   1.72 A40420: added Abs function to assignment of 100% uncertainty and
c                clip at SigMin
c   1.73 A40504: added "calc0" arg for "-t" option; like "calc" but don't
c                subtract 1 from
c   1.74 A40723: changed definition of NSkip: planes "0" through NSkip may
c                not be used for ANYTHING, not even the bottom of a slope
c                sample; allow NSkip = -1 in case someone wants to use
c                plane "0"; also changed output uncertainty for case of
c                only one slope sample (hence no averaging possible): use
c                100% uncertainty only if no sigma-weighting is being done,
c                else keep the uncertainty of the sole sample for output.
c   1.75 A40730: fixed DegFrdm for unweighted averaging; fixed ChiSq code
c                to sigma-edit with first-pass sigma (not second-pass)
c   1.76 A40804: fixed nused for case when no samples are available
c   1.77 A41129: changed bmask bit 12 to be on ONLY if no slope sample
c                is possible but one plane IS OK (bit 13 on when NO plane OK)
c   1.78 A50512: fixed bug in chisq, degfrdm (hence nused); chisq loop in
c                fitlin incorrectly emulated rejection in fitting code by
c                using the second-pass slope in the test; also changed
c                fitting-loop rejection to use unweighted dispersion
c   1.79 A60531: fixed bug that overflowed Img array when setting NaNs due
c                to fatal pmask value for mode 1
c   1.80 A61012: changed rejection of uncertainty = 0 from fatal error to
c                skipped plane; added test for NaN with same handling
c
      Integer*4     IArgC, LNBlnk, FileID1, FileID2, NPixpL, NUse, NChk,
     +              NPlanes, MAlloc, NRows, NCols, Access, IJPix, NPix,
     +              I, N, ImOpen, IStat, ImWkeyT, ImWPixR, ImClose,
     +              ImRPixR, ImWKeyI, NArg, I1, I2, Mode, NPxOut,
     +              ImWPixS, NArgs, DMBRad, DMBSat, NSkip, Kode,
     +              NCM, NRM, NPM, FileID3, ImWPixC, ImRPixS,
     +              NCNL, NRNL, NPNL, NWD, ImRKeyC, ImWKeyC, J, ITmp,
     +              ImRKeyI, Nfl8, NPlns, ImRKeyR, NN, IA2D, Bbit(15),
     +              Dbit(15), MinPln(15), NBits, NSlopLT0, IRenorm,
     +              NRadSkip, LastHit, NSkipt, N1stUse, ImNPixAv,
     +              ImWKeyR, ImWKeyL, NTType, NFrTmin, NFrTmax, NFrCmin,
     +              NFrCmax, FrmCnt(10), NErrZero, NErrNaN, MaxWZero,
     +              MaxWNaN
      Integer*2     BMask(*), DMask(*), PMask(*), MaskD, MaskP, MaskS
      Character*150 InFNam, OutFNam, FilNam, BMaskNam, DMaskNam,
     +              PMaskNam, NLNam, NLMFNam, SigmaNam, UncNam, TmpNam
      Character*50  BUNIT, BUNIT2, CALBUNIT
      Character*25  NumChr, Instrum, RNTChr
      Character*11  Vsn
      Character*8   CDate, CTime
      Character*2   Flag
      Real*4        Img(*),Cube(*),PixDN(256),SigImg(*),SlpImg(*),
     +              TopLayr,Slp(2),Sig(2),SigTL,RBlank,Z,PixPln(256),
     +              ZRad,PxDiff,Coef,ImgTmp,RTmp,WellDepth,VarD,VarAlf,
     +              NLModel(*),e2kn,e2knm1,DeltImg,Unc(*),Sigma(*),
     +              PixErr(256),SatLim,Qmin,Gain(4),Za,Zd,FN,AD2,SigD,
     +              LastUncErr,LastCorErr,Renorm,ChiSqImg(*),ChiSq,
     +              DegFrdm,Q,ImAvgNUs,ImAvgSlo,ImRSgSlo,FullTime,
     +              FrmTim(10),SigMin
      Logical       Debug,NeedHelp,GotIF,GotOF,UseBM,UseDM,UsePM,IzDN,
     +              BitSet,GotSat,FilNaN,GotLF,Weighting,DoSigma,Infl8,
     +              DeSat,BaseOnly(256),RNTime,DetRad,AnySat,IzNaN,Sub1,
     +              UsePln0
      Byte          NUsed(*)
      Pointer       (pC, Cube), (pNL, NLModel), (pU, Unc),
     +              (pI, Img), (pSi, SigImg), (pSl, SlpImg),
     +              (pN, NUsed), (pBM, BMask), (pDM, DMask),
     +              (pPM, PMask), (pS, Sigma), (pCh, ChiSqImg)
c
      NameList / ImagIn / Bbit, Dbit, Debug, DetRad, DMBRad,
     +                    DMBSat, FrmCnt, FrmTim, MaskD,
     +                    MaskP, MaskS, MaxWNaN, MaxWZero,
     +                    MinPln, NFrCmin, NFrCmax,
     +                    NFrTmin, NFrTmax, NRadSkip, NSkip,
     +                    Qmin, SatLim, SigMin, WellDepth, Z, ZRad
c
      NameList / SatCor / Bbit, Dbit, Debug, DetRad, DMBRad,
     +                    DMBSat, FrmCnt, FrmTim, MaskD,
     +                    MaskP, MaskS, MaxWNaN, MaxWZero,
     +                    MinPln, NFrCmin, NFrCmax,
     +                    NFrTmin, NFrTmax, NRadSkip, NSkip,
     +                    Qmin, SatLim, SigMin, WellDepth, Z, ZRad
c
      Common / VDT / CDate, CTime, Vsn
c
      Data Vsn/'1.80 A61012'/, Debug/.False./, Z/1.0/, MaskS/12296/,
     +     NeedHelp/.False./, GotIF/.False./, GotOF/.False./,
     +     UseBM/.False./, UseDM/.False./, Kode/0/, Mode/1/,
     +     UsePM/.False./, MaskD/30728/, MaskP/32640/, NSkip/0/,
     +     DMBRad/9/, DMBSat/4/, ZRad/6.0/, SatLim/9.9e25/, NWD/0/,
     +     GotLF/.False./, Weighting/.False./, DoSigma/.False./,
     +     WellDepth/9.9e25/, Qmin/0.001/, Nfl8/0/, MinPln/15*999999/,
     +     Bbit/15*32000/, Dbit/15*32000/, NSlopLT0/0/, IRenorm/2/,
     +     Renorm/1.0/, NRadSkip/2/, ImNPixAv/0/, ImAvgNUs/0.0/,
     +     ImAvgSlo/0.0/, ImRSgSlo/0.0/, FullTime/1.0/, RNTime/.False./,
     +     NTType/0/, NFrTmin/2/, NFrTmax/7/, FrmTim/1.0, 1.0486,
     +     2.0316, 4.0632, 8.0609, 16.0563, 32.0471, 1.0, 1.0, 1.0/,
     +     NFrCmin/1/, NFrCmax/5/, FrmCnt/2, 4, 8, 16, 32, 64, 128,
     +     256, 512, 1024/, DetRad/.False./, SigMin/1.0/, Sub1/.True./,
     +     NErrZero/0/, NErrNaN/0/, MaxWZero/100/, MaxWNaN/100/
c
c=======================================================================
c
      NArgs = IArgC()
1     If ((NArgs .lt. 4) .or. (NArgs .gt. 32) .or.
     +    (mod(NArgs,2) .ne. 0) .or. NeedHelp) then
        print *,'SIRTF Ramp-Slope/Image-Estimation Program vsn ',Vsn
        print *
        print *,'usage:  imagest <flags> <specifications>'
        print *
        print *,'where <flags> may be: ',
     +          '-b -d -h -i -l -m -n -o -p -r -s -t -u -w -x -z'
        print *,'any one of which, if specified, must be followed ',
     +          'by a blank'
        print *,'and a corresponding specification, as follows.'
        print *
        print *,'    -b bmaskfile  (name of output bmask ',
     +                             'file for mode 1; optional)'
        print *,'    -d dmaskfile  (name of dmask ',
     +                             'file; optional; updated in mode 2)'
        print *,'    -h hardsat    (hard saturation limit; optional)'
        print *,'    -i inputfile  (name of input image ',
     +                             'file; required)'
        print *,'    -l nonlinmod  (name of nonlinearity model ',
     +                             'file; required for mode 2)'
        print *,'    -m mode       (mode = 1 or 2; optional',
     +                             '; default = 1)'
        print *,'    -n namelist   (name of namelist input ',
     +                             'file; optional)'
        print *,'    -o outputfile (name of output file; required)'
        print *,'    -p pmasknam   (name of input pmask file; optional)'
        print *,'    -r renorm#    (mode-1 output image ',
     +                             'renormalization; optional;'
        print *,'                   1: mean slope per plane;'
        print *,'                   2: mean slope * (NPlanes-NSkip) ',
     +                                '[default];'
        print *,'                   3: mean slope * NPlanes)'
        print *,'    -s sigfilnam  (name of output uncertainty file; ',
     +                             'optional)'
        print *,'    -t timenorm   (mode-1 output image normalization ',
     +                             'time; optional)'
        print *,'    -u uncrtnam   (name of input uncertainty ',
     +                             'file; optional)'
        print *,
     +   '    -w welldepth  (maximum desaturation value; optional)'
        print *,'    -x nskip      (number of lower planes to ',
     +                             'skip; optional)'
        print *,'    -z nradskip   (number of planes after a radhit ',
     +                             'to skip; optional)'
        call exit(64)
      end if
c
      write (6, 6000)
      call Aloha(1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c                                determine mode before reading NAMELIST
      NArg = 0
2     NArg = NArg + 1
      call GetArg(NArg,Flag)
      NArg = NArg + 1
      If (Flag .eq. '-m') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=3) N
        If ((N .ge. 1) .and. (N .le. 2)) Mode = N
      End If
3     If (NArg .lt. NArgs) Go to 2
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c                                                  process command line
      NArg = 0
4     NArg = NArg + 1
      call GetArg(NArg,Flag)
      NArg = NArg + 1
c
      If (Flag .eq. '-b') then
        call GetArg(NArg,BMaskNam)
        if (Index(BMaskNam,'.fits') .eq. 0)
     +    BMaskNam = BMaskNam(1:LNBlnk(BMaskNam))//'.fits'
        UseBM = .True.
c
      Else If (Flag .eq. '-d') then
        call GetArg(NArg,DMaskNam)
        if (Index(DMaskNam,'.fits') .eq. 0)
     +    DMaskNam = DMaskNam(1:LNBlnk(DMaskNam))//'.fits'
        if (Access(DMaskNam(1:LNBlnk(DMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',DMaskNam(1:LNBlnk(DMaskNam))
          print *,'Execution aborted'
          call exit(64)
        end if
        UseDM = .True.
c
      Else If (Flag .eq. '-i') then
        call GetArg(NArg,InFNam)
        GotIF = .True.
        if (Index(InFNam,'.fits') .eq. 0)
     +  InFNam = InFNam(1:LNBlnk(InFNam))//'.fits'
        if (Access(InFNam(1:LNBlnk(InFNam)),' ') .ne. 0) then
          print *,'File not found: ',InFNam(1:LNBlnk(InFNam))
          print *,'Execution aborted'
          call exit(64)
        end if
c
      Else If (Flag .eq. '-l') then
        call GetArg(NArg,NLMFNam)
        GotLF = .True.
        if (Index(NLMFNam,'.fits') .eq. 0)
     +    NLMFNam = NLMFNam(1:LNBlnk(NLMFNam))//'.fits'
        if (Access(NLMFNam(1:LNBlnk(NLMFNam)),' ') .ne. 0) then
          print *,'File not found: ',NLMFNam(1:LNBlnk(NLMFNam))
          print *,'Execution aborted'
          call exit(64)
        end if
c
      Else If (Flag .eq. '-n') then
        call GetArg(NArg,NLNam)
        if (Access(NLNam(1:LNBlnk(NLNam)),' ') .ne. 0) then
          print *,'File not found: ',NLNam(1:LNBlnk(NLNam))
          print *,'Execution aborted'
          call exit(64)
        end if
        Open (10, File = NLNam)
        If (mode .eq. 2) then
          Read (10, SatCor, End = 5, Err= 3002)
          Write (6, SatCor)
          Go to 20
5         Rewind (10)
          Read (10, ImagIn, End = 3003, Err= 3002)
          Write (6, ImagIn)
        Else
          Read (10, ImagIn, End = 3003, Err= 3002)
          Write (6, ImagIn)
        End If
        Close (10)
c
      Else If (Flag .eq. '-o') then
        call GetArg(NArg,OutFNam)
        GotOF = .True.
        if (Index(OutFNam,'.fits') .eq. 0)
     +    OutFNam = OutFNam(1:LNBlnk(OutFNam))//'.fits'
c
      Else If (Flag .eq. '-p') then
        call GetArg(NArg,PMaskNam)
        if (Index(PMaskNam,'.fits') .eq. 0)
     +    PMaskNam = PMaskNam(1:LNBlnk(PMaskNam))//'.fits'
        if (Access(PMaskNam(1:LNBlnk(PMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',PMaskNam(1:LNBlnk(PMaskNam))
          print *,'Execution aborted'
          call exit(64)
        end if
        UsePM = .True.
c
      Else If (Flag .eq. '-m') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=6) N
        If ((N .lt. 1) .or. (N .gt. 2)) go to 6
        Mode = N
        print *,'Using Operational Mode ', N
        Go to 7
6       print *,'WARNING: illegal Mode specification: ',NumChr
        NeedHelp = .True.
        Print *
        Go to 1
7       Continue
c
      Else If (Flag .eq. '-x') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=8) N
        if (N .lt. -1) go to 8
        NSkip = N
        print *,'Using NSkip = ',NSkip
        Go to 9
8       print *,'WARNING: ignored illegal NSkip specification: ',NumChr
        NSkip = 0
        Kode = 32
9       Continue
c
      Else If (Flag .eq. '-h') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=10) RTmp
        SatLim = RTmp
        print *,'Using hard saturation limit ', RTmp
        Go to 11
10      print *,
     + 'WARNING: ignored illegal hard-saturation limit specification: ',
     +  NumChr
        Kode = 32
11      Continue
c
      Else If (Flag .eq. '-w') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=12) RTmp
        WellDepth = RTmp
        print *,'Using well depth ', RTmp
        Go to 13
12      print *,
     + 'WARNING: ignored illegal well-depth specification: ',NumChr
        Kode = 32
13      Continue
c
      Else If (Flag .eq. '-s') then
        call GetArg(NArg,SigmaNam)
        if (Index(SigmaNam,'.fits') .eq. 0)
     +    SigmaNam = SigmaNam(1:LNBlnk(SigmaNam))//'.fits'
        DoSigma = .True.
c
      Else If (Flag .eq. '-u') then
        call GetArg(NArg,UncNam)
        if (Index(UncNam,'.fits') .eq. 0)
     +    UncNam = UncNam(1:LNBlnk(UncNam))//'.fits'
        if (Access(UncNam(1:LNBlnk(UncNam)),' ') .ne. 0) then
          print *,'File not found: ',UncNam(1:LNBlnk(UncNam))
          print *,'Execution aborted'
          call exit(64)
        end if
        Weighting = .True.
c
      Else If (Flag .eq. '-r') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=14) N
        If ((N .lt. 1) .or. (N .gt. 3)) go to 14
        IRenorm = N
        print *,'Using Image Renormalization Option ', N
        Go to 15
14      print *,'WARNING: illegal renormalization specification: ',
     +    NumChr
        NeedHelp = .True.
        Print *
        Go to 1
15      Continue
c
      Else If (Flag .eq. '-t') then
        call GetArg(NArg,RNTChr)
        If (Index(RNTChr,'calc') .eq. 1) then
          NTType = 1
          If (Index(RNTChr,'calc0') .eq. 1) Sub1 = .False.
          Go to 17
        Else If (Index(RNTChr,'tran') .eq. 1) then
          NTType = 2
          Go to 17
        Else
          Read (RNTChr,*,Err=16) FullTime
          If (FullTime .le. 0.0) Go to 16
          print *,'Using Image Renormalization Time ', RNTChr
          Go to 17
16        print *,'WARNING: illegal renormalization specification: ',
     +      RNTChr
          NeedHelp = .True.
          Print *
          Go to 1
        End If
17      RNTime = .True.
c
      Else If (Flag .eq. '-z') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=18) N
        NRadSkip = N
        print *,'Using NRadSkip = ',NRadSkip
        Go to 19
18      print *,'WARNING: ignored illegal NRadSkip specification: ',
     +           NumChr
        Kode = 32
19      Continue
c
      Else
        print *,'Unrecognized flag: ',Flag
        NeedHelp = .True.
        Print *
        Go to 1
      End If
c
20    If (NArg .lt. NArgs) Go to 4
      If (.not.(GotIF .and. GotOF)) then
        If (.not.GotIF) print *,'No input image file specified'
        If (.not.GotOF) print *,'No output file name specified'
        Print *
        NeedHelp = .True.
        Go to 1
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (InFNam .eq. OutFNam) then
        print *,'The input and output files may not have the same name'
        print *,'Execution aborted'
        call exit(64)
      End If
c
      If ((SatLim .lt. 9.9e25) .and. (WellDepth .lt. 9.9e25)) then
        If (WellDepth .lt. SatLim) then
          print *,'Imagest WARNING: Well depth specified less than ',
     +            'the hard saturation limit.'
          print *,'                 Well depth value rejected.'
          WellDepth = 9.9e25
        End If
      End If
c
      If (Mode .ne. 1) then
        If (UseBM) then
          UseBM = .False.                ! No bmask for mode 2
          Kode = 32
          print *,'WARNING: bmask specified for mode 2; ignored'
        End If
        If (.not.GotLF) then
          print *,'ERROR: mode 2 selected but nonlinearity model',
     +            ' not supplied'
          print *,'Execution aborted'
          call exit(64)
        end if
      End If
c
      Call GetNAX(InFNam,NCols,NRows,NPlanes,1,FileID1)
      pC = MAlloc(4*NRows*NCols*NPlanes)  ! Allocate mem for input
      If (pC .eq. 0) then
        print *,'Unable to allocate memory for ',
     +           InFNam(1:LNBlnk(InFNam))
        print *,'Execution aborted'
        call exit(64)
      End If
c
      If (RNTime) then
c
        If (NTType .eq. 1) then          ! calculate FullTime
c
          IStat = ImRKeyI(FileID1,N,'AIRSEXPM')
          If (IStat .lt. 1) Go to 22     ! not an error condition
          If (N .ne. 1)     Go to 22     ! also not an error
c
          IStat = ImRKeyI(FileID1,N,'HCFRAM_T')
          If (IStat .lt. 1) then         ! this IS an error!
            print *,'WARNING: HCFRAM_T not in input header'
            Go to 21
          End If
          If ((N .lt. NFrTmin) .or. (N .gt. NFrTmax)) then
            print *,'WARNING: HCFRAM_T = ',N,'; out of limits: ',
     +              NFrTmin,' to ',NFrTmax
            Go to 21
          End If
          FullTime = FrmTim(N)
c
          IStat = ImRKeyI(FileID1,N,'AIRSFRCT')
          If (IStat .lt. 1) then
            print *,'WARNING: AIRSFRCT not in input header'
            Go to 21
          End If
          If ((N .lt. NFrCmin) .or. (N .gt. NFrCmax)) then
            print *,'WARNING: AIRSFRCT = ',N,'; out of limits: ',
     +              NFrCmin,' to ',NFrCmax
            Go to 21
          End If
          If (Sub1) then
            FullTime = FullTime*Float(FrmCnt(N)-1)
          else
            FullTime = FullTime*Float(FrmCnt(N))
          end if
c
          IStat = ImRKeyI(FileID1,N,'HCSPIN_C')
          If (IStat .lt. 1) then
            print *,'WARNING: HCSPIN_C not in input header'
            Go to 21
          End If
          FullTime = FullTime*Float(N+1)
          print *,'Using calculated renormalization time: ',FullTime
c
        Else If (NTType .eq. 2) then     ! get FullTime from header
          IStat = ImRKeyR(FileID1,FullTime,'RAMPTIME')
          If (IStat .lt. 1) then
            print *,'WARNING: RAMPTIME not in input header'
            Go to 21
          End If
          print *,'Using tranhead renormalization time: ',FullTime
        End If
      End If
      Go to 23
c
21    Kode = 32
      Print *,'         Could not compute NORMTIME; CVT2RATE set F'
22    NTType = 3                         ! failed to get FullTime
      FullTime = 1.0                     ! effectively no rescaling
c
23    IStat = ImRKeyC(FileID1,BUNIT,'BUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading BUNIT in input data cube'
        print *,'      File name: ',InFNam(1:LNBlnk(InFNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
      IzDN = Index(BUNIT,'DN') .ne. 0
c
      If (Weighting) then
        IStat = ImRKeyR(FileID1,Gain(1),'GAIN1')
        If (IStat .lt. 1) Go to 24
        IStat = ImRKeyR(FileID1,Gain(2),'GAIN2')
        If (IStat .lt. 1) Go to 24
        IStat = ImRKeyR(FileID1,Gain(3),'GAIN3')
        If (IStat .lt. 1) Go to 24
        IStat = ImRKeyR(FileID1,Gain(4),'GAIN4')
        If (IStat .ge. 1) Go to 26
24      IStat = ImRKeyR(FileID1,Gain(1),'GAIN')
        If (IStat .lt. 1) then
          print *,'ERROR reading GAIN or GAIN1-4 in input data cube'
          print *,'      File name: ',InFNam(1:LNBlnk(InFNam))
          print *,'      ImRKeyC returned error code: ',IStat
          call exit(64)
        Else
          Gain(2) = Gain(1)
          Gain(3) = Gain(1)
          Gain(4) = Gain(1)
        End If
      End If
c
26    IStat = ImRPixR(FileID1,Cube,NRows*NCols*NPlanes)
      If (IStat .ne. NRows*NCols*NPlanes) then
        print *,'ERROR: could not read input data cube'
        print *,'       ImRPixR returned error code ',IStat
        print *,'Execution aborted'
        call exit(64)
      end if
c
      If (Mode .eq. 1) then            ! Allocate mem for output
        pI = MAlloc(4*NRows*NCols)         ! mode 1: 1 image plane
        pS = Malloc(4*NRows*NCols)         ! mode 1: 1 sigma plane
        If (pS .eq. 0) then
          print *,'Unable to allocate memory for uncertainties'
          print *,'Execution aborted'
          call exit(64)
        End If
      Else
        pI = MAlloc(4*NPlanes*NRows*NCols)   ! mode 2: full data cube
      End If
      If (pI .eq. 0) then
        print *,'Unable to allocate memory for ',
     +           OutFNam(1:LNBlnk(InFNam))
        print *,'Execution aborted'
        call exit(64)
      End If
c
      pU = MAlloc(4*NRows*NCols*NPlanes)  ! Allocate mem for uncertainty
      If (pU .eq. 0) then
        print *,'Unable to allocate memory for input uncertainties'
        print *,'Execution aborted'
        call exit(64)
      End If
      Do 28 I = 1, NRows*NCols*NPlanes
        Unc(I) = 0.0
28    Continue
      If (.not.Weighting) Go to 30
c
      Call GetNAX(UncNam,NCM,NRM,NPM,1,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. NPlanes)) then
        print *,'ERROR: wrong dimensions in input uncertainty file'
        print *,'       File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'       Expected ',NCols,' by ',NRows,' by ',NPlanes
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        print *,'Execution aborted'
        call exit(64)
      End If
      IStat = ImRKeyC(FileID3,BUNIT2,'BUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading BUNIT in uncertainty image'
        print *,'      File name: ',UncNam(1:LNBlnk(UncNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
      If (BUNIT2 .ne. BUNIT) then
        print *,'ERROR: BUNIT mismatch between image and uncertainty'
        print *,'       Image BUNIT: ',BUNIT(1:LNBlnk(BUNIT))
        print *,'        Unc. BUNIT: ',BUNIT2(1:LNBlnk(BUNIT2))
        print *,'BUNIT values must match'
        call exit(64)
      end if
      IStat = ImRPixR(FileID3,Unc,NRows*NCols*NPlanes)
      If (IStat .ne. NRows*NCols*NPlanes) then
        print *,'ERROR: could not read input uncertainties'
        print *,'       ImRPixR returned error code ',IStat
        print *,'Execution aborted'
        call exit(64)
      end if
      IStat = ImClose(FileID3)
c
30    If (GotLF) then
c                                      ! Get no. planes in nonlin model
        Call GetNAX(NLMFNam,NCNL,NRNL,NPNL,1,FileID2)
c
        If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols)) Then
          print *,'ERROR: Mismatch in number of rows and columns!'
          print *,'       Input Data Cube:    ',NRows,NCols
          print *,'       Nonlinearity Model: ',NRNL,NCNL
          print *,'Execution aborted'
          call exit(64)
        End If
        If ((NPNL .lt. 2) .or. (NPNL .gt. 3)) then
          print *,'ERROR: No. planes in nonlinearity model = ',NPNL
          print *,
     +     '       Required no. of planes = 2 or 3 for quadratic model'
          print *,'Execution aborted'
          call exit(64)
        End If
c
        IStat = ImRKeyC(FileID2,CALBUNIT,'CALBUNIT')
        If (IStat .lt. 1) then
          print *,'ERROR reading CALBUNIT in nonlinearity model'
          print *,'      File name: ',NLMFNam(1:LNBlnk(NLMFNam))
          print *,'      ImRKeyC returned error code: ',IStat
          call exit(64)
        End If
        If (BUNIT .ne. CALBUNIT) then
          print *,'ERROR: Mismatch between BUNIT and CALBUNIT'
          print *,'      BUNIT:    ',BUNIT(1:LNBlnk(BUNIT))
          print *,'      CALBUNIT: ',CALBUNIT(1:LNBlnk(CALBUNIT))
          print *,'BUNIT must match CALBUNIT'
          call exit(64)
        end if
c
        pNL = MAlloc(4*NRows*NCols*NPNL)
        if (pNL .eq. 0) then
          print *,
     +     'Unable to allocate memory for nonlinearity correction model'
          print *,'Execution aborted'
          call exit(64)
        end if
c
        IStat = ImRPixR(FileID2,NLModel,NRows*NCols*NPNL)
        If (IStat .ne. NRows*NCols*NPNL) then
          print *,'ERROR: could not read nonlinearity model'
          print *,'       ImRPixR returned error code ',IStat
          print *,'Execution aborted'
          call exit(64)
        end if
        IStat = ImClose(FileID2)
c
      End If
c
      If (Mode .eq. 1) then
        pBM = MAlloc(2*NRows*NCols)     ! Allocate mem for BMask
        If (pBM .eq. 0) then
          print *,'ERROR: unable to allocate memory for bmask image'
          print *,'Execution aborted'
          call exit(64)
        End If
        Do 32 I = 1, NCols*NRows
          BMask(I) = 0
32      Continue
      End If
c
      pPM = MAlloc(2*NRows*NCols)     ! Allocate mem for PMask
      If (pPM .eq. 0) then
        print *,'ERROR: unable to allocate memory for pmask image'
        print *,'Execution aborted'
        call exit(64)
      End If
c
      NPix = NCols*NRows*NPlanes
      pDM = MAlloc(2*NPix)            ! Allocate mem for DMask
      If (pDM .eq. 0) then
        print *,'ERROR: unable to allocate memory for dmask image'
        print *,'Execution aborted'
        call exit(64)
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (.not.UsePM) then
        Do 35 I = 1, NCols*NRows
          PMask(I) = 0
35      Continue
        Go to 40
      End If
c
      Call GetNAX(PMaskNam,NCM,NRM,NPM,1,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. 1)) then
        print *,'ERROR: wrong dimensions in pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by 1'
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        print *,'Execution aborted'
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID3,PMask,NCols*NRows)
      If (IStat .ne. NCols*NRows) then
        print *,'ERROR: could not read input pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        print *,'Execution aborted'
        call exit(64)
      end if
      IStat = ImClose(FileID3)
c
40    If (.not.UseDM) then
        Do 50 I = 1, NPix
          DMask(I) = 0
50      Continue
        Go to 60
      End If
c
      Call GetNAX(DMaskNam,NCM,NRM,NPM,1,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. NPlanes)) then
        print *,'ERROR: wrong dimensions in dmask'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by ',NPlanes
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        print *,'Execution aborted'
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID3,DMask,NPix)
      If (IStat .ne. NPix) then
        print *,'ERROR: could not read input dmask'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        print *,'Execution aborted'
        call exit(64)
        IStat = ImClose(FileID3)
      end if
c
60    If (Debug) then
        pCh = MAlloc(12*NRows*NCols)      ! Allocate mem for ChiSq info
        If (pCh .eq. 0) then
          print *,'Unable to allocate memory for chi-square info image'
          print *,'Execution aborted'
          call exit(64)
        End If
        pSi = MAlloc(8*NRows*NCols)       ! Allocate mem for sigmas
        If (pSi .eq. 0) then
          print *,'Unable to allocate memory for sigmas image'
          print *,'Execution aborted'
          call exit(64)
        End If
        pSl = MAlloc(8*NRows*NCols)       ! Allocate mem for slopes
        If (pSl .eq. 0) then
          print *,'Unable to allocate memory for slope images'
          print *,'Execution aborted'
          call exit(64)
        End If
        pN = MAlloc(NRows*NCols)          ! Allocate mem for NUsed
        If (pN .eq. 0) then
          print *,'Unable to allocate memory for number-used image'
          print *,'Execution aborted'
          call exit(64)
        End If
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c                                         Process Image
      NPixPL = NCols*NRows
      Call IMBlank(RBlank)
      if (NSkip .lt. 0) then              ! they want to use plane "0"
        NSkip   = 0
        UsePln0 = .True.
      else
        UsePln0 = .False.
      end if
      NPlns = NPlanes - NSkip             ! Actual number of integrations
      If (Mode .eq. 1) then
        If (IRenorm .eq. 1) Renorm = 1.0/Float(NPlns)
        If (IRenorm .eq. 3) Renorm = Float(NPlanes)/Float(NPlns)
        If (RNTime) Renorm = Renorm/FullTime
      End If
c
c     Do 500  J = 1, NRows                ! Loop over pixels
c       Do 400  I = 1, NCols
        Do 500  IJPix = 1, NPixPL
c
c         IJPix = I + (J-1)*NCols         ! Map 2-D to 1-D
          NUse = 0                        ! Init no. of usable planes
          NChk = NPlanes                  ! Init no. of planes to check
          FilNaN = .False.                ! Init Fill-NaN flag
c
          If (And(PMask(IJPix),MaskP) .ne. 0) then   ! check PMask
            If (Mode .eq. 1) then
              Img(IJPix) = RBlank
              Call SetBit(BMask(IJPix),14)   ! masked in PMask; no good
            Else
              Do 62 N = 1, NPlanes
                Img(IJPix+(N-1)*NPixPL) = RBlank
62            Continue
            End If
            TopLayr = RBlank
            SigTL   = RBlank
            Slp(1)  = RBlank
            Slp(2)  = RBlank
            Sig(1)  = RBlank
            Sig(2)  = RBlank
            FilNaN  = .True.
            Go to 350
          End If
c                                         ! Check for Radhits & Saturation
          GotSat = .False.
          AnySat = .False.
          Do 70 N = NSkip+1, NPlanes      ! Look at each dmask plane
            If ((Mode .eq. 1) .or. GotSat) go to 65
            If ((And(DMask(IJPix+(N-1)*NPixPL),MaskS) .ne. 0) .or.
     +          (Cube(IJPix+(N-1)*NPixPL) .ge. SatLim)) then
              GotSat = .True.
              AnySat = .True.
            End If
65          If (BitSet(DMask(IJPix+(N-1)*NPixPL),DMBRad)) then  ! if radhit,
              If (N+NRadSkip .ge. NPlanes) then       ! if no planes left,
                If (NChk .eq. NPlanes) NChk = N - 1   !  mark place to quit
              End If
            End If
70        Continue
c                                                        ! If mode 2, skip cases
          If ((Mode .eq. 2) .and. .not.AnySat) then      !  with no saturation
            Do 80 N = 1, NPlanes
              If (.not.IzNaN(Img(IJPix+(N-1)*NPixPL)))
     +        Img(IJPix+(N-1)*NPixPL) = Cube(IJPix+(N-1)*NPixPL)
80          Continue
            If (Debug) then
              SlpImg(IJPix)            = RBlank
              SigImg(IJPix)            = RBlank
              SlpImg(IJPix+NPixPL)     = RBlank
              SigImg(IJPix+NPixPL)     = RBlank
              NUsed(IJPix)             = 0
              ChiSqImg(IJPix)          = RBlank
              ChiSqImg(IJPix+NPixPL)   = RBlank
              ChiSqImg(IJPix+2*NPixPL) = RBlank
            end if
            If (.not.DetRad) Go to 500
          End If
c
          If ((NChk - NSkip) .lt. 1) Go to 110
          LastHit = -999
          NSkipt  =  0
          N1stUse =  0
c
          Do 100 N = NSkip+1, NChk
c
            If (BitSet(DMask(IJPix+(N-1)*NPixPL),DMBRad)) then  ! if radhit,
              LastHit = N                                       ! mark it
              Go to 100                                         ! and skip
            End If
            If ((N-LastHit) .le. NRadSkip) Go to 100
c
            If ((And(DMask(IJPix+(N-1)*NPixPL),MaskD) .eq. 0)   ! if status OK,
     +          .and. (Cube(IJPix+(N-1)*NPixPL) .lt. SatLim)) then
              NUse = NUse + 1                    !  put in buffer
              PixDN(NUse)  = Cube(IJPix+(N-1)*NPixPL)
              PixPln(NUse) = N
              BaseOnly(NUse) = ((N-LastHit) .eq. (NRadSkip+1))
     +                    .or. ((N .eq. NSkip+1) .and. (.not.UsePln0))
              If (BaseOnly(NUse)) then
                NSkipt = NSkipt + 1
              Else
                If (N1stUse .eq. 0) N1stUse = NUse
              End If
c
              If (Weighting) then
                PixErr(NUse) = Unc(IJPix+(N-1)*NPixPL)
                If ((PixErr(NUse) .eq. 0.0) .or. IzNaN(PixErr(NUse)))
     +          then
                  J = (IJPix-1)/NCols + 1
                  I = IJPix - (J-1)*NCols
                  Kode = 32
                  if (IzNaN(PixErr(NUse))) then
                    if (NErrNaN .lt. MaxWNaN) then
                      print *,
     +                'ERROR: uncertainty for unmasked pixel is NaN'
                      print *,'Pixel (I,J), Plane = ',I,' ',J,' ',N
                      print *,'Plane skipped'
                    end if
                    NErrNaN = NErrNaN + 1
                    if (NErrNaN .eq. MaxWNaN) print *,
     +               '(last warning of this type; see summary)'
                  else
                    if (NErrZero .lt. MaxWZero) then
                      print *,
     +                'ERROR: uncertainty for unmasked pixel is zero'
                      print *,'Pixel (I,J), Plane = ',I,' ',J,' ',N
                      print *,'Plane skipped'
                    end if
                    NErrZero = NErrZero + 1
                    if (NErrZero .eq. MaxWZero) print *,
     +               '(last warning of this type; see summary)'
                  end if
c                 print *,'Execution aborted'
c                 call exit(64)
                  NUse = NUse - 1
                  go to 100
                End If
              end if
c
              If (NUse .eq. 256) then
                print *,'Imagest: no. of usable planes >= max = 256'
                print *,'         no. of planes = ',NPlanes
                print *,'         arrays(256) could be expanded.'
                go to 110
              end if
            end if
c
100       Continue
c
110       If ((NUse-NSkipt) .gt. 1) Go to 250
          If ((NUse-NSkipt) .eq. 1) then
            If (NInt(PixPln(N1stUse)) .eq. NSkip+1) then
              PxDiff = PixDN(N1stUse)
            Else If (N1stUse .gt. 1) then
              PxDiff = (PixDN(N1stUse)  - PixDN(N1stUse-1))
     +               / (PixPln(N1stUse) - PixPln(N1stUse-1))
            Else
              NUse = NSkipt
            End If
          End If
          If (Mode .eq. 1) then
            If ((NUse-NSkipt) .eq. 0) then
              TopLayr = RBlank                     ! no usable delta
              SigTL   = RBlank
              If (NUse .eq. 0) then
                Call SetBit(BMask(IJPix),13)
              Else
                Call SetBit(BMask(IJPix),12)
              End If
              Slp(1) = TopLayr
              Sig(1) = SigTL
              Slp(2) = TopLayr
              Sig(2) = SigTL
            else
              TopLayr = Float(NPlns)*PxDiff
              SigTL   = Abs(TopLayr)               ! only 1 usable delta
              If (SigTL .lt. SigMin) SigTL = SigMin
              Slp(1) = PxDiff
              Sig(1) = PxDiff
              Slp(2) = PxDiff
              Sig(2) = PxDiff
            end if
          Else
            If ((NUse-NSkipt) .eq. 0) then
              FilNaN = .True.
              TopLayr = RBlank
              SigTL   = RBlank
              Slp(1) = TopLayr
              Sig(1) = SigTL
              Slp(2) = TopLayr
              Sig(2) = SigTL
            else
              TopLayr = PxDiff
              SigTL   = Abs(TopLayr)
              If (SigTL .lt. SigMin) SigTL = SigMin
              Slp(1) = PxDiff
              Sig(1) = PxDiff
              Slp(2) = PxDiff
              Sig(2) = PxDiff
            end if
          End If
c
          ChiSq   = RBlank
          DegFrdm = RBlank
          Q       = RBlank
          Go to 350
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
250       IA2D = Mod(IJPix-1,4) + 1         ! A/D channel; assumes no.
c                                           !  of columns = 4*integer
          Call FitLin(PixDN,PixPln,PixErr,TopLayr,Slp,Sig,
     +                SigTL,NPlns,NUse,Z,Weighting,Qmin,
     +                Infl8,Gain(IA2D),IzDN,BaseOnly,NSkip,
     +                ChiSq,DegFrdm,Q,SigMin)
          If (Infl8) Nfl8 = Nfl8 + 1
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
350       If (Mode .eq. 1) then             !  Mode 1
            Img(IJPix)   = Renorm*TopLayr
            Sigma(IJPix) = Renorm*SigTL
c           If (Img(IJPix) .eq. Img(IJPix)) then  ! skip NaNs
            If (.not.IzNaN(Img(IJPix)))     then  ! skip NaNs
              If (Slp(2) .lt. 0.0) NSlopLT0 = NSlopLT0 + 1
              If (.not.BitSet(BMask(IJPix),12))   ! skip NaN chi-squares
     +            ImAvgNUs = ImAvgNUs + DegFrdm   ! 0 --> 1 later
              ImAvgSlo = ImAvgSlo + Slp(2)
              If (Slp(2) .ne. 0.0)      ! if slope = 0, S/N = 0 by def.
     +           ImRSgSlo = ImRSgSlo + Sig(2)/Slp(2)
              ImNPixAv  = ImNPixAv + 1
            End If
          Else                              !  Mode 2
            If (NSkip .gt. 0) then
              Do 355 N = 1, NSkip           ! Transfer the dud planes
                If (.not.IzNaN(Img(IJPix+(N-1)*NPixPL)))
     +          Img(IJPix+(N-1)*NPixPL) = Cube(IJPix+(N-1)*NPixPL)
355           Continue
            End If
            SigD   = SigTL/Float(NPlns)
            VarD   = SigD**2
            LastUncErr = Unc(IJPix+NSkip*NPixPL)
            LastCorErr = 0.0
            VarAlf = NLModel(IJPix+2*NPixPL)**2
            If (FilNaN .and. AnySat) then
              Img(IJPix+NSkip*NPixPL) = RBlank
              If (DoSigma) Unc(IJPix+NSkip*NPixPL) = RBlank
            Else
              GotSat = (And(DMask(IJPix+NSkip*NPixPL),MaskS) .ne. 0)
     +            .or. (Cube(IJPix+NSkip*NPixPL) .ge. SatLim)
              If (GotSat) then   ! Note: saturation at 1st sample is bad news
                If (.not.IzNaN(Img(IJPix+NSkip*NPixPL))) then
                  Img(IJPix+NSkip*NPixPL) = Slp(2)    ! 2nd-pass slope average
                  DeSat = .True.
                End If
              Else
                If (.not.IzNaN(Img(IJPix+NSkip*NPixPL)))
     +          Img(IJPix+NSkip*NPixPL) = Cube(IJPix+NSkip*NPixPL)
                DeSat = .False.
              End If
              Coef = Alog(1.0 + 2.0*NLModel(IJPix)*Slp(2))
              e2kn = exp(Coef)
              PxDiff = Cube(IJPix+NSkip*NPixPL)
     +               - Img(IJPix+NSkip*NPixPL)
              If (PxDiff .gt. 0.0) then
                If (.not.IzNaN(Img(IJPix+NSkip*NPixPL)))
     +          Img(IJPix+NSkip*NPixPL) = Cube(IJPix+NSkip*NPixPL)
                DeSat = .False.
                If (PxDiff .gt. ZRad*Sig(2))
     +          Call SetBit(DMask(IJPix+NSkip*NPixPL),DMBRad)
              End If
              If (GotSat) Call SetBit(DMask(IJPix+NSkip*NPixPL),DMBSat)
              If (DoSigma .and. DeSat .and. AnySat) then
                Unc(IJPix+NSkip*NPixPL) = sqrt(VarD)
              End If
              If (Img(IJPix+NSkip*NPixPL) .gt. WellDepth) then
                Img(IJPix+NSkip*NPixPL) = WellDepth
                NWD = NWD + 1
              End If
            End If
c
            If (NSkip+2 .gt. NPlanes) Go to 380
c
            Do 360 N = NSkip+2, NPlanes
              If (FilNaN .and. AnySat) then
                Img(IJPix+(N-1)*NPixPL) = RBlank
                If (DoSigma) Unc(IJPix+(N-1)*NPixPL) = RBlank
              Else
                GotSat = (And(DMask(IJPix+(N-1)*NPixPL),MaskS) .ne. 0)
     +              .or. (Cube(IJPix+(N-1)*NPixPL) .ge. SatLim)
                If (NLModel(IJPix) .ne. 0.0) then
                  e2knm1 = e2kn
                  e2kn   = exp(Coef*Float(N-NSkip))
                  DeltImg = (e2kn - e2knm1)/(2.0*NLModel(IJPix))
                Else
                  DeltImg = Slp(2)
                End If
                ImgTmp = Img(IJPix+(N-2)*NPixPL) + DeltImg
                If (GotSat) then
                  If (.not.IzNaN(Img(IJPix+(N-1)*NPixPL))) then
                    Img(IJPix+(N-1)*NPixPL) = ImgTmp
                    DeSat = .True.
                  End If
                Else
                  If (.not.IzNaN(Img(IJPix+(N-1)*NPixPL)))
     +            Img(IJPix+(N-1)*NPixPL) = Cube(IJPix+(N-1)*NPixPL)
                  DeSat = .False.
                End If
                PxDiff = Cube(IJPix+(N-1)*NPixPL) - ImgTmp
                If (PxDiff .gt. 0.0) then
                  If (.not.IzNaN(Img(IJPix+(N-1)*NPixPL)))
     +            Img(IJPix+(N-1)*NPixPL) = Cube(IJPix+(N-1)*NPixPL)
                  DeSat = .False.
                  If (PxDiff .gt. ZRad*Sig(2))
     +            Call SetBit(DMask(IJPix+(N-1)*NPixPL),DMBRad)
                End If
                If (GotSat)
     +            Call SetBit(DMask(IJPix+(N-1)*NPixPL),DMBSat)
                If (DoSigma .and. DeSat) then
                  NN = N-NSkip  ! minimum value of this is 2
                  FN = NN
                  AD2 = 2.0*NLModel(IJPix)*Slp(2)
                  Za = (AD2*FN*(AD2 + 1.0)**(NN-1)
     +               -  AD2*(FN - 1.0)*(AD2 + 1.0)**(NN-2)
     +               -  (AD2 + 1.0)**NN + (AD2 + 1.0)**(NN-1))
     +               /  (2.0*NLModel(IJPix)**2)
                  Zd =  FN*(AD2 + 1.0)**(NN-1)
     +               - (FN - 1.0)*(AD2 + 1.0)**(NN-2)
                  If (BitSet(DMask(IJPix+(N-2)*NPixPL),DMBSat)) then
                    LastCorErr = LastCorErr   ! accumulate desat unc.
     +                         + sqrt(VarAlf*Za**2 + VarD*Zd**2)
                    Unc(IJPix+(N-1)*NPixPL)
     +                            = sqrt(LastUncErr**2 + LastCorErr**2)
                  Else
                    LastUncErr = Unc(IJPix+(N-2)*NPixPL)
                    LastCorErr = sqrt(VarAlf*Za**2 + VarD*Zd**2)
                    If (AnySat) Unc(IJPix+(N-1)*NPixPL)
     +                        = sqrt(LastUncErr**2 + LastCorErr**2)
                  End If
                Else
                  If (DoSigma) then
                    LastUncErr = Unc(IJPix+(N-2)*NPixPL)
                    LastCorErr = 0.0
                  End If
                End If
                If (Img(IJPix+(N-1)*NPixPL) .gt. WellDepth) then
                  Img(IJPix+(N-1)*NPixPL) = WellDepth
                  If (Img(IJPix+(N-2)*NPixPL) .lt. WellDepth)
     +             NWD = NWD + 1
                End If
              End If
360         Continue
          End If               ! End of mode 2 processing
c
380       If (Debug) then
            SlpImg(IJPix)            = Slp(1)
            SigImg(IJPix)            = Sig(1)
            SlpImg(IJPix+NPixPL)     = Slp(2)
            SigImg(IJPix+NPixPL)     = Sig(2)
            If (IzNaN(Slp(2))) then
              NUsed(IJPix)           = 0
            Else
              NUsed(IJPix)           = NInt(DegFrdm) + 1
            End If
            ChiSqImg(IJPix)          = ChiSq
            ChiSqImg(IJPix+NPixPL)   = DegFrdm
            ChiSqImg(IJPix+2*NPixPL) = Q
          end if
c
c400    Continue
        If (DetRad .and. .not.AnySat) then
          Do 480 N = 1, NPlanes
            if (.not.IzNan(Img(IJPix+(N-1)*NPixPL)))
     +      Img(IJPix+(N-1)*NPixPL) = Cube(IJPix+(N-1)*NPixPL)
480       Continue
        End If
500   Continue
c
      FileID2 = ImOpen(OutFNam, "W")
      If (FileID2 .lt. 1) Then
        print *,'Unable to open output file ',OutFNam(1:LNBlnk(OutFNam))
        print *,'Execution aborted'
        call exit(64)
      End If
c
      Call CpyHdr(FileID1,FileID2)
      If (Mode .eq. 1) then
        IStat = ImWKeyI(FileID2,1,'NAXIS3')
        IStat = ImWKeyT(FileID2,'','D')
        IStat = ImWKeyI(FileID2,2,'NAXIS')
      End If
      IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +        //Vsn//', generated '//CDate//' at '//CTime,'E')
      I1 = LNBlnk(InFNam) - 63
      If (I1 .lt. 1) then
        I1 = 1
      Else
        InFnam(I1:I1+2) = '...'
      End If
      IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +        //InFNam(I1:LNBlnk(InFNam)),'E')
c
      If (Mode .eq. 1) then
        IStat = ImWKeyI(FileID2,NSlopLT0,'NSLOPLT0')
        IStat = ImWKeyT(FileID2,'no. of slopes < 0','/')
        If (ImNPixAv .gt. 0) then                    ! add the degree of
          ImAvgNUs = ImAvgNUs/Float(ImNPixAv) + 1.0  !  freedom back in
          ImAvgSlo = ImAvgSlo/Float(ImNPixAv)
          ImRSgSlo = ImRSgSlo/Float(ImNPixAv)
        End If
        IStat = ImWKeyR(FileID2,ImAvgNUs,'IMAVGNUS')
        IStat = ImWKeyT(FileID2,'mean no. of samples/slope','/')
        IStat = ImWKeyR(FileID2,ImAvgSlo,'IMAVGSLO')
        IStat = ImWKeyT(FileID2,'mean ramp slope','/')
        IStat = ImWKeyR(FileID2,ImRSgSlo,'IMRSGSLO')
        IStat = ImWKeyT(FileID2,'mean ratio of sigma/slope','/')
        IStat = ImWKeyI(FileID2,ImNPixAv,'IMNPIXAV')
        IStat = ImWKeyT(FileID2,'no. of pixels averaged','/')
      End If
c
      If (Weighting) then
        IStat = ImWKeyT(FileID2,
     + 'COMMENT  weighted averages were used in the slope estimation',
     +  'A')
        I = LNBlnk(UncNam) - 63
        If (I .lt. 1) then
          I = 1
        Else
          UncNam(I:I+2) = '...'
        End If
        UncNam = UncNam(I:LNBlnk(UncNam))
        IStat = ImWKeyT(FileID2,'HISTORY  unc.: '
     +          //UncNam(1:LNBlnk(UncNam)),'A')
      End If
c
      If (Mode .eq. 1) then
        NPxOut = NCols*NRows
        If (IRenorm .eq. 1) then
          IStat = ImWKeyT(FileID2,
     +     'COMMENT  This image contains mean slopes per plane',
     +     'A')
        Else If ((IRenorm .eq. 2) .and. (NSkip .gt. 0)) then
          IStat = ImWKeyT(FileID2,
     +     'COMMENT  This image contains mean slopes'
     +          // ' scaled by NPlanes-NSkip','A')
        Else !  (IRenorm .eq. 3) .or. (NSkip .eq. 0)
          IStat = ImWKeyT(FileID2,
     +     'COMMENT  This image contains mean slopes'
     +          // ' scaled by NPlanes','A')
        End If
        If (RNTime) then
          If (NTType .lt. 3) then
            IStat = ImWKeyT(FileID2,
     +       'COMMENT  Then the slopes were divided by NORMTIME '
     +            // '(see below)','A')
            IStat = ImWKeyR(FileID2,FullTime,'NORMTIME')
            IStat = ImWKeyT(FileID2,
     +             'Time used to create rate from image','/')
            IStat = ImWKeyL(FileID2,.True.,'CVT2RATE')
            BUNIT = BUNIT(1:LNBlnk(BUNIT))//'/sec'
            IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
            IStat = ImWKeyT(FileID2,'Units of image data','/')
          Else
            IStat = ImWKeyL(FileID2,.False.,'CVT2RATE')
          End If
        End If
      Else                  ! Mode = 2
        NPxOut = NPlanes*NCols*NRows
      End If
      IStat = ImWPixR(FileID2,Img,NPxOut)
      If (IStat .ne. NPxOut) then
        print *,'Unable to generate output file ',OutFNam
        print *,'Error code: ',IStat
        print *,'Execution aborted'
        call exit(64)
      End If
      IStat = ImClose(FileID2)
c
      If (DoSigma) Then
        FileID2 = ImOpen (SigmaNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output sigma file ',SigmaNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        If (Mode .eq. 1) then
          IStat = ImWKeyI(FileID2,2,'NAXIS')
          IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
          IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        Else
          IStat = ImWKeyI(FileID2,3,'NAXIS')
          IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
          IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
          IStat = ImWKeyI(FileID2,NPlanes,'NAXIS3')
        End If
        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
        If (RNTime) then
          If (NTType .lt. 3) then
            IStat = ImWKeyR(FileID2,FullTime,'NORMTIME')
            IStat = ImWKeyT(FileID2,
     +         'Time used to create rate from image','/')
            IStat = ImWKeyL(FileID2,.True.,'CVT2RATE')
          Else
            IStat = ImWKeyL(FileID2,.False.,'CVT2RATE')
          End If
        End If
        IStat = ImRKeyI(FileID1,ITmp,'CHNLNUM')
        Call ChkStat(IStat,'CHNLNUM')
        IStat = ImWKeyI(FileID2,ITmp,'CHNLNUM')
        IStat = ImRKeyI(FileID1,ITmp,'AORKEY')
        Call ChkStat(IStat,'AORKEY')
        IStat = ImWKeyI(FileID2,ITmp,'AORKEY')
        IStat = ImRKeyI(FileID1,ITmp,'EXPID')
        Call ChkStat(IStat,'EXPID')
        IStat = ImWKeyI(FileID2,ITmp,'EXPID')
        IStat = ImRKeyI(FileID1,ITmp,'DCENUM')
        Call ChkStat(IStat,'DCENUM')
        IStat = ImWKeyI(FileID2,ITmp,'DCENUM')
        IStat = ImRKeyC(FileID1,Instrum,'INSTRUME')
        Call ChkStat(IStat,'INSTRUME')
        IStat = ImWKeyC(FileID2,Instrum,'INSTRUME')
        IStat = ImRKeyI(FileID1,ITmp,'FILE_VER')
        Call ChkStat(IStat,'FILE_VER')
        IStat = ImWKeyI(FileID2,ITmp,'FILE_VER')
        If (Mode .eq. 1) then
          IStat = ImWKeyT(FileID2,'COMMENT  This is the sigma file for'
     +            // ' the following BCD image:','E')
        Else
          IStat = ImWKeyT(FileID2,'COMMENT  This is the sigma file for'
     +            // ' the following DCE image:','E')
        End If
        TmpNam = OutFNam
        I = LNBlnk(TmpNam) - 70
        If (I .lt. 1) then
          I = 1
        Else
          Tmpnam(I:I+2) = '...'
        End If
        TmpNam = TmpNam(I:LNBlnk(TmpNam))
        IStat = ImWKeyT(FileID2,'COMMENT  '
     +          //TmpNam(1:LNBlnk(TmpNam)),'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        If (Weighting) then
          IStat = ImWKeyT(FileID2,
     +   'COMMENT  weighted averages were used in the slope estimation',
     +   'A')
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +            //UncNam(1:LNBlnk(UncNam)),'A')
        End If
        If (Mode .eq. 1) then
          IStat = ImWPixR(FileID2,Sigma,NCols*NRows)
        Else
          IStat = ImWPixR(FileID2,Unc,NCols*NRows*NPlanes)
        End If
        If (((Mode .eq. 1) .and. (IStat .ne. NCols*NRows)) .or.
     +      ((Mode .eq. 2) .and. (IStat .ne. NCols*NRows*NPlanes))) then
          print *,'Unable to generate sigma image ',SigmaNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      End If
c
      If (Debug) then
        I2 = Index(OutFNam,'.fits') - 1
        FilNam = OutFNam(1:I2) // '.slopes.fits'
        FileID2 = ImOpen (FilNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output file ',FilNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,3,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyI(FileID2,2,'NAXIS3')
        IStat = ImWKeyT(FileID2,'COMMENT  These are the slopes '
     +          // 'computed on the two-pass ramp-slope estimation'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        IStat = ImWPixR(FileID2,SlpImg,2*NCols*NRows)
        If (IStat .ne. 2*NCols*NRows) then
          print *,'Unable to generate slope image file ',FilNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
c
        FilNam = OutFNam(1:I2) // '.sigmas.fits'
        FileID2 = ImOpen (FilNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output file ',FilNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,3,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyI(FileID2,2,'NAXIS3')
        IStat = ImWKeyT(FileID2,'COMMENT  These are the sigmas '
     +          // 'computed on the two-pass ramp-slope estimation'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        IStat = ImWPixR(FileID2,SigImg,2*NCols*NRows)
        If (IStat .ne. 2*NCols*NRows) then
          print *,'Unable to generate slope image file ',FilNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
c
        FilNam = OutFNam(1:I2) // '.nused.fits'
        FileID2 = ImOpen (FilNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output file ',FilNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyI(FileID2,8,'BITPIX')
        IStat = ImWKeyI(FileID2,2,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyT(FileID2,'COMMENT  These are the number of '
     +          // 'slope samples for each pixel'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        IStat = ImWPixC(FileID2,NUsed,NCols*NRows)
        If (IStat .ne. NCols*NRows) then
          print *,'Unable to generate top-usable plane image ',FilNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
c
        FilNam = OutFNam(1:I2) // '.chisq.fits'
        FileID2 = ImOpen (FilNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output file ',FilNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyI(FileID2,-32,'BITPIX')
        IStat = ImWKeyI(FileID2,3,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyI(FileID2,3,'NAXIS3')
        IStat = ImWKeyT(FileID2,'COMMENT  Plane 1: chi-squares of '
     +          // 'slope averages for each pixel'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'COMMENT  Plane 2: number of degre'
     +          // 'es of freedom for each pixel'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'COMMENT  Plane 3: fraction of dis'
     +          // 'tribution higher than observed value'
     +          ,'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        IStat = ImWPixR(FileID2,ChiSqImg,3*NCols*NRows)
        If (IStat .ne. 3*NCols*NRows) then
          print *,'Unable to generate chisq info image ',FilNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      end if                   ! End of Debug output images
c
      if (UseBM) then
        FileID2 = ImOpen (BMaskNam, "W")
        If (FileID2 .lt. 1) Then
          print *,'Unable to open output bmask file ',BMaskNam
          print *,'Execution aborted'
          call exit(64)
        End If
c
        Do 620 I = 1, 15       ! Check for Dbit/Bbit transfers
          If ((MinPln(I) .le. NPlns) .and.
     +        (Bbit(I) .ge. 0) .and. (Bbit(I) .lt. 15) .and.
     +        (Dbit(I) .ge. 0) .and. (Dbit(I) .lt. 15)) then
            Do 610 IJPix = 1, NPixPL
              NBits = 0
              Do 600 N = NSkip+1, NPlanes     ! Look at each dmask plane
                If (BitSet(DMask(IJPix+(N-1)*NPixPL),DBit(I)))
     +            NBits = NBits + 1                  ! count bits set
600           Continue
              If (NBits .ge. MinPln(I))
     +            Call SetBit(BMask(IJPix),Bbit(I))
610         Continue
          End If
620     Continue
c
        IStat = ImWKeyI(FileID2,16,'BITPIX')
        IStat = ImWKeyI(FileID2,2,'NAXIS')
        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
        IStat = ImWKeyT(FileID2,'COMMENT  This is the bmask file for'
     +          // ' the following BCD image:','E')
        IStat = ImWKeyT(FileID2,'COMMENT  '
     +          //TmpNam(1:LNBlnk(TmpNam)),'E')
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//', generated '//CDate//' at '//CTime,'E')
        IStat = ImWKeyT(FileID2,'HISTORY  input: '
     +          //InFNam(I1:LNBlnk(InFNam)),'E')
        If (Weighting) then
          IStat = ImWKeyT(FileID2,
     +   'COMMENT  weighted averages were used in the slope estimation',
     +   'A')
          IStat = ImWKeyT(FileID2,'HISTORY   unc.: '
     +            //UncNam(1:LNBlnk(UncNam)),'A')
        End If
        IStat = ImWPixS(FileID2,BMask,NCols*NRows)
        If (IStat .ne. NCols*NRows) then
          print *,'Unable to generate bmask image ',BMaskNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      end if
c
      if (UseDM .and. (Mode .eq. 2)) then
        FileID2 = ImOpen (DMaskNam, "U")
        If (FileID2 .lt. 1) Then
          print *,'Unable to update dmask file ',DMaskNam
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImWKeyT(FileID2,'HISTORY imagest           v.  '
     +          //Vsn//',  updated  '//CDate//' at '//CTime,'E')
        IStat = ImWPixS(FileID2,DMask,NCols*NRows*NPlanes)
        If (IStat .ne. NCols*NRows*NPlanes) then
          print *,'Unable to write updated dmask image ',DMaskNam
          print *,'Error code: ',IStat
          print *,'Execution aborted'
          call exit(64)
        End If
        IStat = ImClose(FileID2)
      end if
c
      If (NWD .gt. 0) then
        print *,'No. of desaturated pixels clipped at well depth: ',NWD
      End If
      If (Nfl8 .gt. 0) then
        print *,'No. of pixels needing goodness-of-fit compensation: ',
     +           Nfl8
      End If
      if (NErrZero .gt. 0) print *,
     + 'No. of zero uncertainties for unmasked pixels: ', NErrZero
      if (NErrNaN .gt. 0) print *,
     + 'No. of NaN uncertainties for unmasked pixels: ', NErrNaN
c
      write (6, 6000)
      call Aloha(2)
c
      Call Exit(Kode)
      Stop
c
3002  print *,'ERROR reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      print *,'Execution aborted'
      call exit(64)
3003  print *,'EOF reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      print *,'Execution aborted'
      call exit(64)
      stop
c
6000  Format('imagest'$)
c
      end
c
c=======================================================================
c
      Subroutine SetBit(I2Var,NBit)
C-----------------------------------------------------------------------
C     Set Bit no. "NBit" in "I2Var"
C     LSB = bit no. 0, MSB = bit no. 14, Sign Bit = bit no. 15
C-----------------------------------------------------------------------
      Integer*4 NBit,NWarns
      Integer*2 Power(16),I2Var
      Data NWarns/0/
C
C  Try these two changes - TYB 4/8/09
C
      Parameter (hexbit15 = Z'8000')
C
      Data Power/1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,
     + 16384,hexbit15/
C
C      Data Power/1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,
C     + 16384,Z8000/
C-----------------------------------------------------------------------
C
      If ((NBit .lt. 0) .or. (NBit .gt. 15)) Then
        NWarns = NWarns + 1
        If (NWarns .le. 10) Write(6, 6000) NBit
        If (NWarns .eq. 10) Write(6, 6001)
        Return
      End If
C
C  Try this change - TYB 4/9/08
C
       I2Var = ior(I2Var, Power(NBit+1))
C
C      I2Var = I2Var .or. Power(NBit+1)
C
      Return
C-----------------------------------------------------------------------
6000  Format(' SetBit: Warning! bit number outside range (0 - 15), ',
     + I21)
6001  Format('         This is the last warning that will be printed')
C-----------------------------------------------------------------------
      End
c
c=======================================================================
c
      Logical Function BitSet(I2Var,NBit)
C-----------------------------------------------------------------------
C     Returns .TRUE. iff Bit no. "NBit" in "I2Var" is set (turned on)
C     LSB = bit no. 0, MSB = bit no. 14, Sign Bit = bit no. 15
C-----------------------------------------------------------------------
      Integer*4 NBit,NWarns
      Integer*2 I2Var,Power(16)
      Data NWarns/0/
C
C  Try these two changes - TYB 4/8/09
C
      Parameter (hexbit15 = Z'8000')
C
      Data Power/1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,
     + 16384,hexbit15/
C
C      Data Power/1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,
C     + 16384,Z8000/
C-----------------------------------------------------------------------
C
      If ((NBit .lt. 0) .or. (NBit .gt. 15)) Then
        NWarns = NWarns + 1
        If (NWarns .le. 10) Write(6, 6000) NBit
        If (NWarns .eq. 10) Write(6, 6001)
        BitSet = .False.
        Return
      End If
C
C
C  Try this change - TYB 4/8/09
C
       BitSet = iand(I2Var,Power(NBit+1)) .ne. 0
C
C      BitSet = (I2Var .and. Power(NBit+1)) .ne. 0
C
      Return
C-----------------------------------------------------------------------
6000  Format(' BitSet: Warning! bit number outside range (0 - 15), ',
     + I21)
6001  Format('         This is the last warning that will be printed')
C-----------------------------------------------------------------------
      End
c
c=======================================================================
c
      Subroutine ChkStat(I,S)
      Integer*4   I
      Character*8 S
c
      If (I .lt. 1) then
        print *,'ERROR reading ',S,' in input DCE image'
        print *,'Error code: ',I
        print *,'Execution aborted'
        call exit(64)
      End If
c
      Return
      End
c
c=======================================================================
c
      Function IzNaN(R4)
c
      Logical   IzNaN
      Real*4    R4, R4a
      Integer*4 I4, MaskExp, MaskMant
      Equivalence (I4, R4a)
c
      Data MaskExp/2139095040/, MaskMant/8388607/
c
c-----------------------------------------------------------------------
c
c   01111111100000000000000000000000 = 2139095040 = 7F80000 hex
c   00000000011111111111111111111111 =    8388607 =  7FFFFF hex
c
c-----------------------------------------------------------------------
c
      R4a = R4
      IzNaN = ((And(I4,MaskExp)  .eq. MaskExp) .and.
     +         (And(I4,MaskMant) .ne. 0))
      Return
      End
