c=======================================================================
c
c   General-purpose SIRTF linearization program; reads data cube with
c    NPlanes exposures; reads nonlinearity-correction data cube and
c    applies to main input data cube, outputs linearized data cube.
c
c     $Id: lineariz.f,v 1.15 2004/03/18 21:16:51 jwf Exp $
c
c   1.0  990219: initial vsn
c   1.0  991204: S3 vsn; removed saturation image, include cmask, dmask,
c                and pmask usage, including setting dmask bits 10, 12,
c                and 13 (linearity correction applied, could not be
c                applied, and saturation, resp.)
c   1.1  A00627: installed new Zhong Wang nonlinearity model
c   1.2  A10201: installed additional quadratic nonlinearity model
c   1.3  A10418: added SetBit 12 in dmask for masked pixels
c   1.31 A10504: made compatible with new lincal model with ModTyp = 2;
c                doesn't use uncertainty but can use new model cube
c   1.32 A11012: removed setting of dmask bit 10; changed mask tests
c                from ">=" to "AND <> 0"
c   1.33 A11121: added cubic model (ModTyp = 3) and uncertainty for
c                both quadratic and cubic; added "-u" (uncertainty
c                input) and "-s" (sigma output)
c   1.34 A20220: added BUNIT & CALBUNIT processing
c   1.35 A20508: changed cmask to bits 8, 9, 10, 12 (not 11)
c   1.36 A20916: added LinQA processing
c   1.37 A20918: changed COMMENT to standardized HISTORY line for vsn ID
c   1.38 A31114: switched LinQA NaN testing to IzNaN function for linux
c   1.39 A40318: completed linux compatibility by changing makefile to link
c                new aloha (from aloha.F) from include library
c
      Integer*4     IArgC, LNBlnk, NPix, NPNL, FileID1, FileID2,
     +              NPlanes, MAlloc, NRows, NCols, NRNL, NCNL, Access,
     +              I, J, N, ImOpen, IStat, ImWkeyT, ImWPixR, ImClose,
     +              ImRPixR, JJ, ImWPixS, NSatd, FileID3, ModTyp,
     +              ImRPixS, NArgs, NArg, NCM, NRM, NPM, ImWKeyI, L,
     +              ImRKeyC, ITmp, ImRKeyI, ImWKeyC, QAopt, Indx(*),
     +              ImRKeyR, NPlQA, NSkip, xh1, xh2, xl1, xl2, yh1,
     +              yh2, yl1, yl2, LZNPCHIF, LZNPCHIM, FileID4, ImWKeyR
      Integer*2     CMask(*), DMask(*), PMask(*), MaskC,
     +              MaskD, MaskP
      Character*150 InFNam, OutFNam, NLMFNam, CMaskNam, DMaskNam,
     +              PMaskNam, NLNam, UncNam, SigmaNam
      Character*50  BUNIT, BUNIT2, CALBUNIT
      Character*25  Instrum
      Character*11  Vsn
      Character*8   CDate, CTime, NumChr
      Character*2   Flag
      Logical       Satd,NeedHelp,GotIF,GotOF,GotLF,UseCM,UseDM,UsePM,
     +              DoUnc,DoSigma,IzDN,Corrl8d
      Real*8        Cube(*), NLModel(*), DNLin, AvgSP, SigDNlin,
     +              Sigma(*), QAimg(*), Covar(*), Weight(*) ,T(*),
     +              Gain(4), X(*), Y(*), W(*), Z(*), ChiSqrMax,
     +              ChiSqrMin,
     +              LZILAVGF, LZILAVGL, LZILSTDF, LZILSTDL, LZILCHIS,
     +              LZILCOER, LZILRMSS, LZILSLOP, LZILZINT, LZILFREE,
     +              LZUNAVGF, LZUNAVGL, LZUNSTDF, LZUNSTDL, LZUNCHIS,
     +              LZUNCOER, LZUNRMSS, LZUNSLOP, LZUNZINT, LZUNFREE
      Pointer       (pC, Cube), (pNL, NLModel), (pS, Sigma),
     +              (pCM, CMask), (pDM, DMask), (pPM, PMask),
     +              (pQ, QAimg), (pI, Indx), (pCv, Covar),
     +              (pWt, Weight), (pT, T), (pX, X), (pY, Y),
     +              (pW, W), (pZ, Z)
c
c  Added for CFITSIO - TYB 5/1/09
c
      Integer       status, unit, unit1, unit2, unit4, last_space
      Integer       readwrite, blocksize
      Character*151 myoutname
c
      Namelist / LineIN / ChiSqrMax, ChiSqrMin, Corrl8d,
     +                    MaskC, MaskD, MaskP,
     +                    xh1, xh2, xl1, xl2, yh1, yh2, yl1, yl2
c
      Data Vsn/'1.39 A40318'/, NSatd/0/, AvgSP/0.0/, GotIF/.False./,
     +     GotOF/.False./, ModTyp/1/, DoUnc/.False./, DoSigma/.False./,
     +     GotLF/.False./, UseCM/.False./, UseDM/.False./,
     +     UsePM/.False./, NeedHelp/.False./, MaskC/5888/,
     +     MaskD/16385/, MaskP/32640/, QAopt/0/, NSkip/0/,
     +     Corrl8d/.True./, ChiSqrMax/3.0/, ChiSqrMin/0.3/,
     +     xh1/60/, xh2/120/, xl1/30/, xl2/90/,
     +     yh1/60/, yh2/120/, yl1/30/, yl2/90/
C
      Common / VDT / CDate, CTime, Vsn
c
      Common / QAcom / ChiSqrMax, ChiSqrMin, LZNPCHIF, LZNPCHIM,
     +                 xh1, xh2, xl1, xl2, yh1, yh2, yl1, yl2,
     +                 LZILAVGF, LZILAVGL, LZILSTDF, LZILSTDL, LZILCHIS,
     +                 LZILCOER, LZILRMSS, LZILSLOP, LZILZINT, LZILFREE,
     +                 LZUNAVGF, LZUNAVGL, LZUNSTDF, LZUNSTDL, LZUNCHIS,
     +                 LZUNCOER, LZUNRMSS, LZUNSLOP, LZUNZINT, LZUNFREE
c
c=======================================================================
c
      NArgs = IArgC()
1     If ((NArgs .lt. 6) .or. (NArgs .gt. 22) .or.
     +    (mod(NArgs,2) .ne. 0) .or. NeedHelp) then
        print *,'SIRTF Linearization Program vsn ',Vsn
        print *
        print *,'usage:  lineariz <flags> <specifications>'
        print *
        print *,'where <flags> may be: -c -i -l -m -n -o -p -q -s -u -x'
        print *,'any one of which, if specified, must be followed ',
     +          'by a blank'
        print *,'and a corresponding specification, as follows.'
        print *
        print *,'    -c cmaskfile  (name of cmask ',
     +                             'file; optional)'
        print *,'    -i inputfile  (name of input image ',
     +                             'file; required)'
        print *,'    -l nonlinmod  (name of nonlinearity model ',
     +                             'file; required)'
        print *,'    -m dmaskfile  (name of dmask ',
     +                             'file; optional)'
        print *,'    -n namelist   (name of namelist input ',
     +                             'file; optional)'
        print *,'    -o outputfile (name of output ',
     +                             'file; required)'
        print *,'    -p pmasknam   (name of pmask file; optional)'
        print *,'    -q QAopt#     (QA option number; optional;'
        print *,'                   0 = no QA output [default]'
        print *,'                   1 = QA output to header only'
        print *,'                   2 = QA header & FITS output'
        print *,'                   3 = 1 + linear coefficients)'
        print *,'                   4 = 2 + linear coefficients)'
        print *,'    -s sigmanam   (name of output sigma ',
     +                             'file; optional)'
        print *,'    -u uncrtnam   (name of input uncertainty ',
     +                             'file; optional)'
        print *,'    -x nskip      (number of lower planes to ',
     +                             'skip in QA; optional)'
        call exit(64)
      end if
c
      write (6, 6000)
      call Aloha(1)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
2     NArg = 0
4     NArg = NArg + 1
      call GetArg(NArg,Flag)
      NArg = NArg + 1
c
      If (Flag .eq. '-i') then
        call GetArg(NArg,InFNam)
        GotIF = .True.
        if (Index(InFNam,'.fits') .eq. 0)
     +  InFNam = InFNam(1:LNBlnk(InFNam))//'.fits'
        if (Access(InFNam(1:LNBlnk(InFNam)),' ') .ne. 0) then
          print *,'File not found: ',InFNam(1:LNBlnk(InFNam))
          call exit(64)
        end if
c
      Else If (Flag .eq. '-l') then
        call GetArg(NArg,NLMFNam)
        GotLF = .True.
        if (Index(NLMFNam,'.fits') .eq. 0)
     +    NLMFNam = NLMFNam(1:LNBlnk(NLMFNam))//'.fits'
        if (Access(NLMFNam(1:LNBlnk(NLMFNam)),' ') .ne. 0) then
          print *,'File not found: ',NLMFNam(1:LNBlnk(NLMFNam))
          call exit(64)
        end if
c
      Else If (Flag .eq. '-o') then
        call GetArg(NArg,OutFNam)
        GotOF = .True.
        if (Index(OutFNam,'.fits') .eq. 0)
     +    OutFNam = OutFNam(1:LNBlnk(OutFNam))//'.fits'
        If (InFNam .eq. OutFNam) then
          print *,'The input and output files may not have ',
     +            'the same name'
          call exit(64)
        End If
c
      else If (Flag .eq. '-c') then
        call GetArg(NArg,CMaskNam)
        if (Index(CMaskNam,'.fits') .eq. 0)
     +    CMaskNam = CMaskNam(1:LNBlnk(CMaskNam))//'.fits'
        if (Access(CMaskNam(1:LNBlnk(CMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',CMaskNam(1:LNBlnk(CMaskNam))
          call exit(64)
        end if
        UseCM = .True.
c
      else If (Flag .eq. '-m') then
        call GetArg(NArg,DMaskNam)
        if (Index(DMaskNam,'.fits') .eq. 0)
     +    DMaskNam = DMaskNam(1:LNBlnk(DMaskNam))//'.fits'
        if (Access(DMaskNam(1:LNBlnk(DMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',DMaskNam(1:LNBlnk(DMaskNam))
          call exit(64)
        end if
        UseDM = .True.
c
      else If (Flag .eq. '-p') then
        call GetArg(NArg,PMaskNam)
        if (Index(PMaskNam,'.fits') .eq. 0)
     +    PMaskNam = PMaskNam(1:LNBlnk(PMaskNam))//'.fits'
        if (Access(PMaskNam(1:LNBlnk(PMaskNam)),' ') .ne. 0) then
          print *,'File not found: ',PMaskNam(1:LNBlnk(PMaskNam))
          call exit(64)
        end if
        UsePM = .True.
c
      else If (Flag .eq. '-n') then
        call GetArg(NArg,NLNam)
        if (Access(NLNam(1:LNBlnk(NLNam)),' ') .ne. 0) then
          print *,'File not found: ',NLNam(1:LNBlnk(NLNam))
          call exit(64)
        end if
        Open (10, File = NLNam)
        Read (10, LineIN, End = 3003, Err= 3002)
        Write (6, LineIN)
        Close (10)
c
      else If (Flag .eq. '-s') then
        call GetArg(NArg,SigmaNam)
        if (Index(SigmaNam,'.fits') .eq. 0)
     +    SigmaNam = SigmaNam(1:LNBlnk(SigmaNam))//'.fits'
        DoSigma = .True.
c
      else If (Flag .eq. '-u') then
        call GetArg(NArg,UncNam)
        if (Index(UncNam,'.fits') .eq. 0)
     +    UncNam = UncNam(1:LNBlnk(UncNam))//'.fits'
        if (Access(UncNam(1:LNBlnk(UncNam)),' ') .ne. 0) then
          print *,'File not found: ',UncNam(1:LNBlnk(UncNam))
          call exit(64)
        end if
        DoUnc = .True.
c
      Else If (Flag .eq. '-q') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=5) N
        If ((N .lt. -4) .or. (N .gt. 4)) go to 5
        If (N .lt. 0) Corrl8d = .False.
        QAopt = IAbs(N)
        print *,'Using QA option no. ', N
        If (Corrl8d)
     +    print *,'Correlated errors will be taken into account'
        If (N .lt. 0)
     +    print *,'Errors will be assumed uncorrelated'
        Go to 6
5       print *,'ERROR: illegal QA option specification: ',NumChr
        NeedHelp = .True.
        Print *
        Go to 1
6       Continue
c
      Else If (Flag .eq. '-x') then
        call GetArg(NArg,NumChr)
        Read (NumChr,*,Err=7) NSkip
        print *,'Using NSkip = ',NSkip
        Go to 8
7       print *,'WARNING: illegal NSkip specification: ',NumChr
        NeedHelp = .True.
        Print *
        Go to 1
8       Continue
c
      Else
        print *,'Unrecognized flag: ',Flag
        NeedHelp = .True.
        Print *
        Go to 1
      End If
c
      If (NArg .lt. NArgs) Go to 4
      If (.not.(GotIF .and. GotOF .and. GotLF)) then
        If (.not.GotIF) print *,'No input image file specified'
        If (.not.GotOF) print *,'No output file name specified'
        If (.not.GotLF) print *,'No linearity model file name specified'
        Print *
        NeedHelp = .True.
        Go to 1
      End If
      If (DoUnc .and. .not.DoSigma) then
        print *,
     +    'Uncertainty input specified, but no sigma output specified'
        print *,'Uncertainty input image will be ignored'
        DoUnc = .False.
      End If
      If ((QAopt .gt. 0) .and. .not.DoSigma) then
        print *,
     +    'QA output specified, but no sigma output specified'
        print *,'QA output will not be performed'
        QAopt = 0
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      Call GetNAX(InFNam,NCols,NRows,NPlanes,1,FileID1)
      pC = MAlloc(8*NRows*NCols*NPlanes)  ! Allocate mem for input
      If (pC .eq. 0) then
        print *,'Unable to allocate memory for ',
     +           InFNam(1:LNBlnk(InFNam))
        call exit(64)
      End If
c
      IStat = ImRPixD(FileID1,Cube,NRows*NCols*NPlanes)
      If (IStat .ne. NRows*NCols*NPlanes) then
        print *,'ERROR: could not read input data cube'
        print *,'       ImRPixD returned error code ',IStat
        call exit(64)
      end if
c
      IStat = ImRKeyC(FileID1,BUNIT,'BUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading BUNIT in input image'
        print *,'      File name: ',InFNam(1:LNBlnk(InFNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
c
      NPix = NCols*NRows*NPlanes
      If (DoUnc .or. DoSigma) then
        pS = MAlloc(8*NPix)          ! Allocate mem for Sigma
      Else
        pS = MAlloc(1)               ! Legitimize array address
      End If
      If (pS .eq. 0) then
        print *,'ERROR: unable to allocate memory for sigma image'
        call exit(64)
      End If
c
      If (DoUnc) then
        FileID3 = ImOpen(UncNam, "R")
        If (FileID3 .lt. 1) Then
          print *,'Unable to open uncertainty input file ',
     +       UncNam(1:LNBlnk(UncNam))
          call exit(64)
        End If
        IStat = ImRPixD(FileID3,Sigma,NRows*NCols*NPlanes)
        If (IStat .ne. NRows*NCols*NPlanes) then
          print *,'ERROR: could not read input uncertainty image'
          print *,'       ImRPixD returned error code ',IStat
          call exit(64)
        end if
        IStat = ImRKeyC(FileID3,BUNIT2,'BUNIT')
        If (IStat .lt. 1) then
          print *,'ERROR reading BUNIT in uncertainty image'
          print *,'      File name: ',UncNam(1:LNBlnk(UncNam))
          print *,'      ImRKeyC returned error code: ',IStat
          call exit(64)
        End If
        If (BUNIT2 .ne. BUNIT) then
          print *,'ERROR: BUNIT mismatch between image and uncertainty'
          print *,'       Image BUNIT: ',BUNIT(1:LNBlnk(BUNIT))
          print *,'        Unc. BUNIT: ',BUNIT2(1:LNBlnk(BUNIT2))
          print *,'BUNIT values must match'
          call exit(64)
        end if
        IStat = ImClose(FileID3)
      End If
c                                      ! Get no. planes in nonlin model
      Call GetNAX(NLMFNam,NCNL,NRNL,NPNL,1,FileID2)
c
      If ((NRNL .ne. NRows) .or. (NCNL .ne. NCols)) Then
        print *,'ERROR: Mismatch in number of rows and columns!'
        print *,'       Input Data Cube:    ',NRows,NCols
        print *,'       Nonlinearity Model: ',NRNL,NCNL
        call exit(64)
      End If
      If (NPNL .lt. 4) ModTyp = 2      ! Quadratic model has 3 planes
      If (NPNL .gt. 4) ModTyp = 3      ! Cubic model has 10 planes
c
      IStat = ImRKeyC(FileID2,CALBUNIT,'CALBUNIT')
      If (IStat .lt. 1) then
        print *,'ERROR reading CALBUNIT in nonlinearity model'
        print *,'      File name: ',NLMFNam(1:LNBlnk(NLMFNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      End If
      If (BUNIT .ne. CALBUNIT) then
        print *,'ERROR: Mismatch between BUNIT and CALBUNIT'
        print *,'      BUNIT:    ',BUNIT(1:LNBlnk(BUNIT))
        print *,'      CALBUNIT: ',CALBUNIT(1:LNBlnk(CALBUNIT))
        print *,'BUNIT must match CALBUNIT'
        call exit(64)
      end if
c
      pNL = MAlloc(8*NRows*NCols*NPNL)
      if (pNL .eq. 0) then
        print *,
     +   'Unable to allocate memory for nonlinearity correction model'
        call exit(64)
      end if
c
      pCM = MAlloc(2*NRows*NCols)     ! Allocate mem for CMask
      If (pCM .eq. 0) then
        print *,'ERROR: unable to allocate memory for cmask image'
        call exit(64)
      End If
c
      pPM = MAlloc(2*NRows*NCols)     ! Allocate mem for PMask
      If (pPM .eq. 0) then
        print *,'ERROR: unable to allocate memory for pmask image'
        call exit(64)
      End If
c
      pDM = MAlloc(2*NPix)            ! Allocate mem for DMask
      If (pDM .eq. 0) then
        print *,'ERROR: unable to allocate memory for dmask image'
        call exit(64)
      End If
c
      If (QAopt .gt. 0) then
        If (QAopt .gt. 2) then          ! Allocate mem for QAimg
          pQ = MAlloc(48*NRows*NCols)
          NPlQA = 6                     !  6 planes
        Else
          pQ = MAlloc(24*NRows*NCols)
          NPlQA = 3                     !  3 planes
        End If
        If (pQ .eq. 0) then
          print *,'ERROR: unable to allocate memory for QA image'
          call exit(64)
        End If
        pI = MAlloc(4*NPlanes)          ! Allocate mem for Indx
        If (pI .eq. 0) then
          print *,'ERROR: unable to allocate memory for Indx'
          call exit(64)
        End If
        pCv = MAlloc(8*NPlanes*NPlanes) ! Allocate mem for Covar
        If (pCv .eq. 0) then
          print *,'ERROR: unable to allocate memory for Covar'
          call exit(64)
        End If
        pWt = MAlloc(8*NPlanes*NPlanes) ! Allocate mem for Weight
        If (pWt .eq. 0) then
          print *,'ERROR: unable to allocate memory for Weight'
          call exit(64)
        End If
        pT = MAlloc(8*NPlanes*NPlanes)  ! Allocate mem for T
        If (pT .eq. 0) then
          print *,'ERROR: unable to allocate memory for T (tmp array)'
          call exit(64)
        End If
        pX = MAlloc(8*NPlanes)          ! Allocate mem for X array
        If (pX .eq. 0) then
          print *,'ERROR: unable to allocate memory for X array'
          call exit(64)
        End If
        pY = MAlloc(8*NPlanes)          ! Allocate mem for Y array
        If (pY .eq. 0) then
          print *,'ERROR: unable to allocate memory for Y array'
          call exit(64)
        End If
        pW = MAlloc(8*NPlanes)          ! Allocate mem for W array
        If (pW .eq. 0) then
          print *,'ERROR: unable to allocate memory for W array'
          call exit(64)
        End If
        pZ = MAlloc(8*NPlanes)          ! Allocate mem for Z array
        If (pZ .eq. 0) then
          print *,'ERROR: unable to allocate memory for Z array'
          call exit(64)
        End If
      End If
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      If (.not.UsePM) then
        Do 10 I = 1, NCols*NRows
          PMask(I) = 0
10      Continue
        Go to 20
      End If
c
      Call GetNAX(PMaskNam,NCM,NRM,NPM,1,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. 1)) then
        print *,'ERROR: wrong dimensions in pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by 1'
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID3,PMask,NCols*NRows)
      If (IStat .ne. NCols*NRows) then
        print *,'ERROR: could not read input pmask'
        print *,'       File name: ',PMaskNam(1:LNBlnk(PMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        call exit(64)
      end if
      IStat = ImClose(FileID3)
c
20    If (.not.UseCM) then
        Do 30 I = 1, NCols*NRows
          CMask(I) = 0
30      Continue
        Go to 40
      End If
c
      Call GetNAX(CMaskNam,NCM,NRM,NPM,1,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. 1)) then
        print *,'ERROR: wrong dimensions in cmask'
        print *,'       File name: ',CMaskNam(1:LNBlnk(CMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by 1'
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        call exit(64)
      End If
c
      IStat = ImRPixS(FileID3,CMask,NCols*NRows)
      If (IStat .ne. NCols*NRows) then
        print *,'ERROR: could not read input cmask'
        print *,'       File name: ',CMaskNam(1:LNBlnk(CMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        call exit(64)
      end if
      IStat = ImClose(FileID3)
c
40    If (.not.UseDM) then
        Do 50 I = 1, NPix
          DMask(I) = 0
50      Continue
        Go to 60
      End If
c
      Call GetNAX(DMaskNam,NCM,NRM,NPM,0,FileID3)
      If ((NRM .ne. NRows) .or. (NCM .ne. NCols) .or.
     +    (NPM .ne. NPlanes)) then
        print *,'ERROR: wrong dimensions in dmask'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       Expected ',NCols,' by ',NRows,' by ',NPlanes
        print *,'       Got ',NCM,' by ',NRM,' by ',NPM
        call exit(64)
      End If
c
      FileID3 = ImOpen(DMaskNam, "U")  ! reopen for update
      IStat = ImRPixS(FileID3,DMask,NPix)
      If (IStat .ne. NPix) then
        print *,'ERROR: could not read input dmask'
        print *,'       File name: ',DMaskNam(1:LNBlnk(DMaskNam))
        print *,'       ImRPixS returned error code ',IStat
        call exit(64)
      end if
c                                  ! Initialize nonlinearity subroutine
60    Call NonLin0(NLModel,NCols,NRows,NPNL,FileID2)
c
c = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
c
      Do 300 N = 1, NPlanes
        Do 200 J = 1, NRows
          JJ = (J-1)*NCols
          Do 100 I = 1, NCols
            NPix = I + JJ + (N-1)*NCols*NRows
            Call NonLin(Cube(NPix),DNLin,I,J,NLModel,NCols,NRows,
     +                  NPlanes,Satd,N,ModTyp,SigDNlin,
     +                  CMask,DMask,PMask,MaskC,MaskD,MaskP)
            Cube(NPix) = DNLin
            If (DoSigma) then
              If (DoUnc) then
                Sigma(NPix) = dsqrt(Sigma(NPix)**2 + SigDNlin**2)
              Else
                Sigma(NPix) = SigDNlin
              End If
            End If
            If (Satd) then
              NSatd = NSatd + 1
              AvgSP = AvgSP + dfloat(N)
            end if
100       Continue
200     Continue
300   Continue
c
c = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
c
      print *,'No. of saturated pixels: ',NSatd
      if (NSatd .gt. 0) then
        NSatd = NInt(AvgSP/dfloat(NSatd))
        print *,'Average plane of saturation: ',NSatd
      end if
c
c  Substitute CFITSIO equivalents for output data - TYB 5/1/09
c
      call ftgiou(unit4,status)
      last_space = index(OutFNam, ' ')
      myoutname = '!'//OutFNam(1:last_space)
      call ftinit(unit4, myoutname, 1, status)
      If (status .ne. 0) Then
        print *,'Unable to open output data file ',
     +      OutFNam(1:LNBlnk(OutFNam))
        call exit(64)
      End If
c
c  Get header from input data and copy it
c
      call ftgiou(unit1,status)
      last_space = index(InFNam, ' ')
      myoutname = InFNam(1:last_space)
      readwrite = 0
      blocksize = 1
      call ftopen(unit1, myoutname, readwrite, blocksize, status)
      If (status .ne. 0) Then
        print *,'Unable to open input data file ',
     +      InFNam(1:LNBlnk(InFNam))
        call exit(64)
      End If
      call ftcphd(unit1, unit4, status)
      call ftclos(unit1, status)
      call ftfiou(unit1, status)
c
      call ftprec(unit4,'HISTORY lineariz          v.  '
     +        //Vsn//', generated '//CDate//' at '//CTime, status)
      call ftpkyj(unit4,'MODTYP',ModTyp,'',status)
      L = LNBlnk(InFNam) - 63
      If (L .lt. 1) then
        L = 1
      Else
        InFnam(L:L+2) = '...'
      End If
      call ftprec(unit4, 'HISTORY  input: '
     +        //InFNam(L:LNBlnk(InFNam)),status)
      I = LNBlnk(NLMFNam) - 63
      If (I .lt. 1) then
        I = 1
      Else
        NLMFNam(I:I+2) = '...'
      End If
      call ftprec(unit4, 'HISTORY  model: '
     +        //NLMFNam(I:LNBlnk(NLMFNam)),status)
c
c      FileID4 = ImOpen(OutFNam, "W")
c      If (FileID4 .lt. 1) Then
c        print *,'Unable to open output file ',OutFNam(1:LNBlnk(OutFNam))
c        call exit(64)
c      End If
cc
c      Call CpyHdr(FileID1,FileID4)
c      IStat = ImWKeyT(FileID4,'HISTORY lineariz          v.  '
c     +        //Vsn//', generated '//CDate//' at '//CTime,'A')
c      IStat = ImWKeyI(FileID4,ModTyp,'MODTYP')
c      L = LNBlnk(InFNam) - 63
c      If (L .lt. 1) then
c        L = 1
c      Else
c        InFnam(L:L+2) = '...'
c      End If
c      IStat = ImWKeyT(FileID4,'HISTORY  input: '
c     +        //InFNam(L:LNBlnk(InFNam)),'A')
c      I = LNBlnk(NLMFNam) - 63
c      If (I .lt. 1) then
c        I = 1
c      Else
c        NLMFNam(I:I+2) = '...'
c      End If
c      IStat = ImWKeyT(FileID4,'HISTORY  model: '
c     +        //NLMFNam(I:LNBlnk(NLMFNam)),'A')
c
      NPix = NCols*NRows*NPlanes
      If (DoSigma) then
c
c  Substitute CFITSIO equivalents for Sigma - TYB 5/1/09
c
        call ftgiou(unit,status)
        last_space = index(SigmaNam, ' ')
        myoutname = '!'//SigmaNam(1:last_space)
        call ftinit(unit, myoutname, blocksize, status)
        If (status .ne. 0) Then
          print *,'Unable to open output sigmafile ',
     +      SigmaNam(1:LNBlnk(SigmaNam))
          call exit(64)
        End If
        call ftpkyl(unit,'SIMPLE',T,'',status)
        call ftpkyj(unit,'BITPIX',-32,'',status)
        If (NPlanes .eq. 1) then
          call ftpkyj(unit,'NAXIS',2,'',status)
          call ftpkyj(unit,'NAXIS1',NCols,'',status)
          call ftpkyj(unit,'NAXIS2',NRows,'',status)
        Else
          call ftpkyj(unit,'NAXIS',3,'',status)
          call ftpkyj(unit,'NAXIS1',NCols,'',status)
          call ftpkyj(unit,'NAXIS2',NRows,'',status)
          call ftpkyj(unit,'NAXIS3',NPlanes,'',status)
        End If
        call ftpkyl(unit,'EXTEND',T,
     +        'TAPE MAY HAVE STANDARD FITS EXTENSIONS',status)
        call ftpkys(unit,'BUNIT',BUNIT,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'CHNLNUM')
        Call ChkStat(IStat,'CHNLNUM')
        call ftpkyj(unit,'CHNLNUM',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'AORKEY')
        Call ChkStat(IStat,'AORKEY')
        call ftpkyj(unit,'AORKEY',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'EXPID')
        Call ChkStat(IStat,'EXPID')
        call ftpkyj(unit,'EXPID',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'DCENUM')
        Call ChkStat(IStat,'DCENUM')
        call ftpkyj(unit,'DCENUM',ITmp,'',status)
        IStat = ImRKeyC(FileID1,Instrum,'INSTRUME')
        Call ChkStat(IStat,'INSTRUME')
        call ftpkys(unit,'INSTRUME',Instrum,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'FILE_VER')
        Call ChkStat(IStat,'FILE_VER')
        call ftpkyj(unit,'FILE_VER',ITmp,'',status)
        call ftprec(unit,'HISTORY lineariz          v.  '
     +        //Vsn//', generated '//CDate//' at '//CTime,status)
        call ftpkyj(unit,'MODTYP',ModTyp,'',status)
        call ftprec(unit,'HISTORY  input: '
     +          //InFNam(L:LNBlnk(InFNam)),status)
        call ftprec(unit,'COMMENT  This is a sigma image',status)
c
        call ftppre(unit, 1, 1, NPix, Sigma, status)
        call ftclos(unit, status)
        call ftfiou(unit, status)
c
c        FileID2 = ImOpen(SigmaNam, "W")
c        If (FileID2 .lt. 1) Then
c          print *,'Unable to open output sigmafile ',
c     +      SigmaNam(1:LNBlnk(SigmaNam))
c          call exit(64)
c        End If
c        IStat = ImWKeyI(FileID2,-32,'BITPIX')
c        If (NPlanes .eq. 1) then
c          IStat = ImWKeyI(FileID2,2,'NAXIS')
c          IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
c          IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
c        Else
c          IStat = ImWKeyI(FileID2,3,'NAXIS')
c          IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
c          IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
c          IStat = ImWKeyI(FileID2,NPlanes,'NAXIS3')
c        End If
c        IStat = ImWKeyC(FileID2,BUNIT,'BUNIT')
c        IStat = ImRKeyI(FileID1,ITmp,'CHNLNUM')
c        Call ChkStat(IStat,'CHNLNUM')
c        IStat = ImWKeyI(FileID2,ITmp,'CHNLNUM')
c        IStat = ImRKeyI(FileID1,ITmp,'AORKEY')
c        Call ChkStat(IStat,'AORKEY')
c        IStat = ImWKeyI(FileID2,ITmp,'AORKEY')
c        IStat = ImRKeyI(FileID1,ITmp,'EXPID')
c        Call ChkStat(IStat,'EXPID')
c        IStat = ImWKeyI(FileID2,ITmp,'EXPID')
c        IStat = ImRKeyI(FileID1,ITmp,'DCENUM')
c        Call ChkStat(IStat,'DCENUM')
c        IStat = ImWKeyI(FileID2,ITmp,'DCENUM')
c        IStat = ImRKeyC(FileID1,Instrum,'INSTRUME')
c        Call ChkStat(IStat,'INSTRUME')
c        IStat = ImWKeyC(FileID2,Instrum,'INSTRUME')
c        IStat = ImRKeyI(FileID1,ITmp,'FILE_VER')
c        Call ChkStat(IStat,'FILE_VER')
c        IStat = ImWKeyI(FileID2,ITmp,'FILE_VER')
c        IStat = ImWKeyT(FileID2,'HISTORY lineariz          v.  '
c     +        //Vsn//', generated '//CDate//' at '//CTime,'A')
c        IStat = ImWKeyI(FileID2,ModTyp,'MODTYP')
c        IStat = ImWKeyT(FileID2,'HISTORY  input: '
c     +          //InFNam(L:LNBlnk(InFNam)),'A')
c        IStat = ImWKeyT(FileID2,'COMMENT  This is a sigma image','A')
c
c          IStat = ImWPixR(FileID2,Sigma,NPix)
c
c        If (IStat .ne. NPix) then
c          print *,'Unable to generate output sigma file ',
c     +             SigmaNam(1:LNBlnk(SigmaNam))
c          print *,'Error code: ',IStat
c          call exit(64)
c        End If
c        IStat = ImClose(FileID2)
c
      End If
c
      If (.not.UseDM) Go to 350
      IStat = ImWKeyT(FileID3,'HISTORY lineariz          v.  '
     +        //Vsn//',  updated  '//CDate//' at '//CTime,'A')
c
c  This appears to be a typo, should be FileID3 ? - TYB 6/1/09 
c      IStat = ImWKeyI(FileID2,ModTyp,'MODTYP')
c
      IStat = ImWPixS(FileID3,DMask,NPix)
      If (IStat .ne. NPix) then
        print *,'Unable to update dmask file ',
     +           DMaskNam(1:LNBlnk(DMaskNam))
        print *,'Error code: ',IStat
        call exit(64)
      End If
      IStat = ImClose(FileID3)
c
350   If (QAopt .eq. 0) Go to 1000
      IzDN = Index(BUNIT,'DN') .gt. 0
      IStat = ImRKeyR(FileID1,Gain(1),'GAIN1')
      If (IStat .lt. 1) Go to 400
      IStat = ImRKeyR(FileID1,Gain(2),'GAIN2')
      If (IStat .lt. 1) Go to 400
      IStat = ImRKeyR(FileID1,Gain(3),'GAIN3')
      If (IStat .lt. 1) Go to 400
      IStat = ImRKeyR(FileID1,Gain(4),'GAIN4')
      If (IStat .ge. 1) Go to 500
400   IStat = ImRKeyR(FileID1,Gain(1),'GAIN')
      If (IStat .lt. 1) then
        print *,'ERROR reading GAIN or GAIN1-4 in input data cube'
        print *,'      File name: ',InFNam(1:LNBlnk(InFNam))
        print *,'      ImRKeyC returned error code: ',IStat
        call exit(64)
      Else
        Gain(2) = Gain(1)
        Gain(3) = Gain(1)
        Gain(4) = Gain(1)
      End If
c
500   Call LinQA(Cube,QAimg,NCols,NRows,NPlanes,NSkip,Corrl8d,
     +           DMask,PMask,MaskD,MaskP,IzDN,X,Y,W,Z,
     +           Gain,Sigma,Covar,Weight,T,Indx,NPlQA)
c
c  Finish writing data output keywords
c
      call ftpkyj(unit4,'LZNPCHIF',LZNPCHIF,
     +      'no. chi-squares > ChiSqrMax',status)
      call ftpkyj(unit4,'LZNPCHIM',LZNPCHIM,
     +      'no. chi-squares < ChiSqrMin',status)
      call ftpkye(unit4,'LZILAVGF',LZILAVGF,-10,
     +      'mean signal plane 1 box 1',status)
      call ftpkye(unit4,'LZILAVGL',LZILAVGL,-10,
     +      'mean signal last plane box 1',status)
      call ftpkye(unit4,'LZILSTDF',LZILSTDF,-10,
     +      'std dev about LZILAVGF',status)
      call ftpkye(unit4,'LZILSTDL',LZILSTDL,-10,
     +      'std dev about LZILAVGL',status)
      call ftpkye(unit4,'LZILCHIS',LZILCHIS,-10,
     +      'mean reduced chi-square box 1',status)
      call ftpkye(unit4,'LZILCOER',LZILCOER,-10,
     +      'mean linear corr. coefft. box 1',status)
      call ftpkye(unit4,'LZILRMSS',LZILRMSS,-10,
     +      'mean linear RMS dispersion box 1',status)
      If (NPlQA .gt. 3) then
        call ftpkye(unit4,'LZILSLOP',LZILSLOP,-10,
     +      'mean linear ramp slope box 1',status)
        call ftpkye(unit4,'LZILZINT',LZILZINT,-10,
     +      'mean linear ramp intercept box 1',status)
        call ftpkye(unit4,'LZILFREE',LZILFREE,-10,
     +      'mean no. deg. freedom box 1',status)
      End If
      call ftpkye(unit4,'LZUNAVGF',LZUNAVGF,-10,
     +      'mean signal plane 1 box 2',status)
      call ftpkye(unit4,'LZUNAVGL',LZUNAVGL,-10,
     +      'mean signal last plane box 2',status)
      call ftpkye(unit4,'LZUNSTDF',LZUNSTDF,-10,
     +      'std dev about LZUNAVGF',status)
      call ftpkye(unit4,'LZUNSTDL',LZUNSTDL,-10,
     +      'std dev about LZUNAVGL',status)
      call ftpkye(unit4,'LZUNCHIS',LZUNCHIS,-10,
     +      'mean reduced chi-square box 2',status)
      call ftpkye(unit4,'LZUNCOER',LZUNCOER,-10,
     +      'mean linear corr. coefft. box 2',status)
      call ftpkye(unit4,'LZUNRMSS',LZUNRMSS,-10,
     +      'mean linear RMS dispersion box 2',status)
      If (NPlQA .gt. 3) then
        call ftpkye(unit4,'LZUNSLOP',LZUNSLOP,-10,
     +      'mean linear ramp slope box 2',status)
        call ftpkye(unit4,'LZUNZINT',LZUNZINT,-10,
     +      'mean linear ramp intercept box 2',status)
        call ftpkye(unit4,'LZUNFREE',LZUNFREE,-10,
     +      'mean no. deg. freedom box 2',status)
      End If
c
c      IStat = ImWKeyI(FileID4,LZNPCHIF,'LZNPCHIF')
c      IStat = ImWKeyT(FileID4,'no. chi-squares > ChiSqrMax','/')
c      IStat = ImWKeyI(FileID4,LZNPCHIM,'LZNPCHIM')
c      IStat = ImWKeyT(FileID4,'no. chi-squares < ChiSqrMin','/')
c      IStat = ImWKeyR(FileID4,LZILAVGF,'LZILAVGF')
c      IStat = ImWKeyT(FileID4,'mean signal plane 1 box 1','/')
c      IStat = ImWKeyR(FileID4,LZILAVGL,'LZILAVGL')
c      IStat = ImWKeyT(FileID4,'mean signal last plane box 1','/')
c      IStat = ImWKeyR(FileID4,LZILSTDF,'LZILSTDF')
c      IStat = ImWKeyT(FileID4,'std dev about LZILAVGF','/')
c      IStat = ImWKeyR(FileID4,LZILSTDL,'LZILSTDL')
c      IStat = ImWKeyT(FileID4,'std dev about LZILAVGL','/')
c      IStat = ImWKeyR(FileID4,LZILCHIS,'LZILCHIS')
c      IStat = ImWKeyT(FileID4,'mean reduced chi-square box 1','/')
c      IStat = ImWKeyR(FileID4,LZILCOER,'LZILCOER')
c      IStat = ImWKeyT(FileID4,'mean linear corr. coefft. box 1','/')
c      IStat = ImWKeyR(FileID4,LZILRMSS,'LZILRMSS')
c      IStat = ImWKeyT(FileID4,'mean linear RMS dispersion box 1',
c     +               '/')
c      If (NPlQA .gt. 3) then
c        IStat = ImWKeyR(FileID4,LZILSLOP,'LZILSLOP')
c        IStat = ImWKeyT(FileID4,'mean linear ramp slope box 1','/')
c        IStat = ImWKeyR(FileID4,LZILZINT,'LZILZINT')
c        IStat = ImWKeyT(FileID4,'mean linear ramp intercept box 1',
c     +                 '/')
c        IStat = ImWKeyR(FileID4,LZILFREE,'LZILFREE')
c        IStat = ImWKeyT(FileID4,'mean no. deg. freedom box 1','/')
c      End If
c      IStat = ImWKeyR(FileID4,LZUNAVGF,'LZUNAVGF')
c      IStat = ImWKeyT(FileID4,'mean signal plane 1 box 2','/')
c      IStat = ImWKeyR(FileID4,LZUNAVGL,'LZUNAVGL')
c      IStat = ImWKeyT(FileID4,'mean signal last plane box 2','/')
c      IStat = ImWKeyR(FileID4,LZUNSTDF,'LZUNSTDF')
c      IStat = ImWKeyT(FileID4,'std dev about LZUNAVGF','/')
c      IStat = ImWKeyR(FileID4,LZUNSTDL,'LZUNSTDL')
c     IStat = ImWKeyT(FileID4,'std dev about LZUNAVGL','/')
c      IStat = ImWKeyR(FileID4,LZUNCHIS,'LZUNCHIS')
c      IStat = ImWKeyT(FileID4,'mean reduced chi-square box 2','/')
c      IStat = ImWKeyR(FileID4,LZUNCOER,'LZUNCOER')
c      IStat = ImWKeyT(FileID4,'mean linear corr. coefft. box 2',
c     +               '/')
c      IStat = ImWKeyR(FileID4,LZUNRMSS,'LZUNRMSS')
c      IStat = ImWKeyT(FileID4,'mean linear RMS dispersion box 2',
c     +               '/')
c      If (NPlQA .gt. 3) then
c        IStat = ImWKeyR(FileID4,LZUNSLOP,'LZUNSLOP')
c        IStat = ImWKeyT(FileID4,'mean linear ramp slope box 2',
c     +                 '/')
c        IStat = ImWKeyR(FileID4,LZUNZINT,'LZUNZINT')
c        IStat = ImWKeyT(FileID4,
c     +                 'mean linear ramp intercept box 2','/')
c        IStat = ImWKeyR(FileID4,LZUNFREE,'LZUNFREE')
c        IStat = ImWKeyT(FileID4,'mean no. deg. freedom box 2','/')
c      End If
c
c  Write qa output
c
      If ((QAopt .eq. 2) .or. (QAopt .eq. 4)) then
        call ftgiou(unit2,status)
        myoutname = OutFNam(1:Index(OutFnam,'.fits'))//'qa.fits'
        last_space = index(myoutname, ' ')
        myoutname = '!'//myoutname(1:last_space)
        call ftinit(unit2, myoutname, blocksize, status)
        If (status .ne. 0) Then
          print *,'Unable to open output QA file ',
     +      OutFNam(1:Index(OutFNam,'.fits'))//'qa.fits'
          call exit(64)
        End If
        call ftpkyl(unit2,'SIMPLE',T,'',status)
        call ftpkyj(unit2,'BITPIX',-32,'',status)
        call ftpkyj(unit2,'NAXIS',3,'',status)
        call ftpkyj(unit2,'NAXIS1',NCols,'',status)
        call ftpkyj(unit2,'NAXIS2',NRows,'',status)
        call ftpkyj(unit2,'NAXIS3',NPlQA,'',status)
        call ftpkyl(unit2,'EXTEND',T,
     +        'TAPE MAY HAVE STANDARD FITS EXTENSIONS',status)
        IStat = ImRKeyI(FileID1,ITmp,'CHNLNUM')
        Call ChkStat(IStat,'CHNLNUM')
        call ftpkyj(unit2,'CHNLNUM',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'AORKEY')
        Call ChkStat(IStat,'AORKEY')
        call ftpkyj(unit2,'AORKEY',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'EXPID')
        Call ChkStat(IStat,'EXPID')
        call ftpkyj(unit2,'EXPID',ITmp,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'DCENUM')
        Call ChkStat(IStat,'DCENUM')
        call ftpkyj(unit2,'DCENUM',ITmp,'',status)
        IStat = ImRKeyC(FileID1,Instrum,'INSTRUME')
        Call ChkStat(IStat,'INSTRUME')
        call ftpkys(unit2,'INSTRUME',Instrum,'',status)
        IStat = ImRKeyI(FileID1,ITmp,'FILE_VER')
        Call ChkStat(IStat,'FILE_VER')
        call ftpkyj(unit2,'FILE_VER',ITmp,'',status)
        call ftprec(unit2,'HISTORY lineariz          v.  '
     +        //Vsn//', generated '//CDate//' at '//CTime,status)
        call ftpkyj(unit2,'MODTYP',ModTyp,'',status)
        call ftprec(unit2,'HISTORY  input: '
     +          //InFNam(L:LNBlnk(InFNam)),status)
        call ftprec(unit2,'COMMENT  This is a QA image',status)
        call ftprec(unit2,'COMMENT  Plane 1: reduced chi-'
     +          // 'squares of linear fits to linearized data',status)
        call ftprec(unit2,'COMMENT  Plane 2: linear correlat'
     +          // 'tion coefficients of fits',status)
        call ftprec(unit2,'COMMENT  Plane 3: RMS dispersion '
     +          // 'about linear fits in units of '//BUNIT,status)
        If (NPlQA .gt. 3) then
          call ftprec(unit2,'COMMENT  Plane 4: linear-fit slop'
     +          // 'es',status)
          call ftprec(unit2,'COMMENT  Plane 5: linear-fit inte'
     +          // 'rcepts',status)
          call ftprec(unit2,'COMMENT  Plane 6: no. of degrees '
     +          // 'of freedom in chi-square',status)
        End If
c
        call ftppre(unit2, 1, 1, NPlQA*NCols*NRows, QAimg, status)
        call ftclos(unit2, status)
        call ftfiou(unit2, status)
      End If
c
c      If ((QAopt .eq. 2) .or. (QAopt .eq. 4)) then
c
c        FileID2 = ImOpen(OutFNam(1:Index(OutFNam,'.fits'))
c     +                   //'qa.fits','W')
c        If (FileID2 .lt. 1) Then
c          print *,'Unable to open output QA file ',
c     +      OutFNam(1:Index(OutFNam,'.fits'))//'qa.fits'
c          call exit(64)
c        End If
c        IStat = ImWKeyI(FileID2,-32,'BITPIX')
c        IStat = ImWKeyI(FileID2,3,'NAXIS')
c        IStat = ImWKeyI(FileID2,NCols,'NAXIS1')
c        IStat = ImWKeyI(FileID2,NRows,'NAXIS2')
c        IStat = ImWKeyI(FileID2,NPlQA,'NAXIS3')
c        IStat = ImRKeyI(FileID1,ITmp,'CHNLNUM')
c        Call ChkStat(IStat,'CHNLNUM')
c        IStat = ImWKeyI(FileID2,ITmp,'CHNLNUM')
c        IStat = ImRKeyI(FileID1,ITmp,'AORKEY')
c        Call ChkStat(IStat,'AORKEY')
c        IStat = ImWKeyI(FileID2,ITmp,'AORKEY')
c        IStat = ImRKeyI(FileID1,ITmp,'EXPID')
c        Call ChkStat(IStat,'EXPID')
c        IStat = ImWKeyI(FileID2,ITmp,'EXPID')
c        IStat = ImRKeyI(FileID1,ITmp,'DCENUM')
c        Call ChkStat(IStat,'DCENUM')
c        IStat = ImWKeyI(FileID2,ITmp,'DCENUM')
c        IStat = ImRKeyC(FileID1,Instrum,'INSTRUME')
c        Call ChkStat(IStat,'INSTRUME')
c        IStat = ImWKeyC(FileID2,Instrum,'INSTRUME')
c        IStat = ImRKeyI(FileID1,ITmp,'FILE_VER')
c        Call ChkStat(IStat,'FILE_VER')
c        IStat = ImWKeyI(FileID2,ITmp,'FILE_VER')
c        IStat = ImWKeyT(FileID2,'HISTORY lineariz          v.  '
c     +        //Vsn//', generated '//CDate//' at '//CTime,'E')
c        IStat = ImWKeyI(FileID2,ModTyp,'MODTYP')
c        IStat = ImWKeyT(FileID2,'HISTORY  input: '
c     +          //InFNam(L:LNBlnk(InFNam)),'A')
c        IStat = ImWKeyT(FileID2,'COMMENT  This is a QA image','A')
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 1: reduced chi-'
c     +          // 'squares of linear fits to linearized data','E')
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 2: linear correlat'
c     +          // 'tion coefficients of fits','E')
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 3: RMS dispersion '
c     +          // 'about linear fits in units of '//BUNIT,'E')
c        If (NPlQA .gt. 3) then
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 4: linear-fit slop'
c     +          // 'es','E')
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 5: linear-fit inte'
c     +          // 'rcepts','E')
c        IStat = ImWKeyT(FileID2,'COMMENT  Plane 6: no. of degrees '
c     +          // 'of freedom in chi-square','E')
c        End If
c        IStat = ImWPixR(FileID2,QAimg,NPlQA*NCols*NRows)
c        If (IStat .ne. NPlQA*NCols*NRows) then
c          print *,'Unable to generate output QA file ',
c     +             OutFNam(1:Index(OutFNam,'.fits'))//'qa.fits'
c          print *,'Error code: ',IStat
c          call exit(64)
c        End If
c        IStat = ImClose(FileID2)
c      End If
c
1000  call ftppre(unit4, 1, 1, NPix, Cube, status)
      call ftclos(unit4, status)
      call ftfiou(unit4, status)
c
c1000  IStat = ImWPixR(FileID4,Cube,NPix)
c      If (IStat .ne. NPix) then
c        print *,'Unable to generate output file ',
c     +           OutFNam(1:LNBlnk(OutFNam))
c        print *,'Error code: ',IStat
c        call exit(64)
c      End If
c      IStat = ImClose(FileID4)
c
      write (6, 6000)
      call Aloha(2)
      call exit(0)
c
      Stop
c
3002  print *,'ERROR reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      call exit(64)
3003  print *,'EOF reading NAMELIST input file: ',
     +         NLNam(1:LNBlnk(NLNam))
      call exit(64)
      stop
c
6000  Format('lineariz'$)
c
      End
c
c=======================================================================
c
      Subroutine ChkStat(I,S)
      Integer*4   I
      Character*8 S
c
      If (I .lt. 1) then
        print *,'ERROR reading ',S,' in input DCE image'
        print *,'Error code: ',I
        print *,'Execution aborted'
        call exit(64)
      End If
c
      Return
      End
