      Subroutine LinQA(Cube,QAimg,NCols,NRows,NPlanes,NSkip,Corrl8d,
     +                 DMask,PMask,MaskD,MaskP,IzDN,X,Y,W,Z,
     +                 Gain,Sigma,Covar,Weight,T,Indx,NPlQA)
c======================================================================c
c      This routine performs post-linearization QA processing          c
c      in which each ramp is fit to a straight line via chi-           c
c      square minimization, then a three-plane data cube is            c
c      generated with images of the chi-squares from the fit,          c
c      the linear correlation coefficients, and the fit dispersion.    c
c      In addition, averaging within two windows is performed; the     c
c      two windows are an illuminated region and an unilluminated      c
c      region; the means and standard deviations of the three          c
c      quantities in those windows are computed and written to the     c
c      log file.                                                       c
c======================================================================c
c
      Integer*4 NCols, NRows, NPlanes, Indx(NPlanes), NUse, N, M, IA2D,
     +          I, J, NNOK1, NNOK2, NNOK3, NNOK4, NNOK5, NPlQA, NSkip,
     +          LZNPCHIF, LZNPCHIM, xh1, xh2, xl1, xl2, yh1,
     +          yh2, yl1, yl2
      Integer*2 DMask(NCols,NRows,NPlanes), PMask(NCols,NRows),
     +          MaskD, MaskP
      Logical   IzDN, OK, Corrl8d
      Real*8    Cube(NCols,NRows,NPlanes), QAimg(NCols,NRows,NPlQA),
     +          Weight(NPlanes,NPlanes), Covar(NPlanes,NPlanes),
     +          R4Blank, T(NPlanes,NPlanes), X(NPlanes), Y(NPlanes),
     +          Sigma(NCols,NRows,NPlanes), Gain(4), W(NPlanes),
     +          Z(NPlanes), A, B, Tmp, XBar, YBar, ChiSqrMax,
     +          ChiSqrMin,
     +          LZILAVGF, LZILAVGL, LZILSTDF, LZILSTDL, LZILCHIS,
     +          LZILCOER, LZILRMSS, LZILSLOP, LZILZINT, LZILFREE,
     +          LZUNAVGF, LZUNAVGL, LZUNSTDF, LZUNSTDL, LZUNCHIS,
     +          LZUNCOER, LZUNRMSS, LZUNSLOP, LZUNZINT, LZUNFREE
      Real*8    K1, K2, K3, K4, K5, ChiSq, RhoLin, SumSq
c
      Data      NNOK1/0/, NNOK2/0/, NNOK3/0/, NNOK4/0/, NNOK5/0/
c
      Common / QAcom / ChiSqrMax, ChiSqrMin, LZNPCHIF, LZNPCHIM,
     +                 xh1, xh2, xl1, xl2, yh1, yh2, yl1, yl2,
     +                 LZILAVGF, LZILAVGL, LZILSTDF, LZILSTDL, LZILCHIS,
     +                 LZILCOER, LZILRMSS, LZILSLOP, LZILZINT, LZILFREE,
     +                 LZUNAVGF, LZUNAVGL, LZUNSTDF, LZUNSTDL, LZUNCHIS,
     +                 LZUNCOER, LZUNRMSS, LZUNSLOP, LZUNZINT, LZUNFREE
c
c=======================================================================
c
      Call ImBlankD(R4Blank)            ! Get a NaN to use later
      LZNPCHIF = 0
      LZNPCHIM = 0
c
      Do 200 J = 1, NRows
        Do 100 I = 1, NCols            ! Process pixel (I,J)
c
          If (And(PMask(I,J),MaskP) .ne. 0) then
            NNOK1 = NNOK1 + 1
            Go to 99
          End If
          IA2D = Mod(I-1,4) + 1        ! A/D channel; assumes no.
c                                      !  of columns = 4*integer
          NUse = 0
          Do 10 N = NSkip+1, NPlanes   ! Load X, Y, & Indx arrays
            If (And(DMask(I,J,N),MaskD) .ne. 0) Go to 10
            NUse = NUse + 1
            Indx(NUse) = N
            X(NUse)    = dfloat(N-NSkip)
            Y(NUse)    = Cube(I,J,N)
10        Continue
          If (NUse .lt. 3) then        ! Need at least 3 points
            NNOK2 = NNOK2 + 1          !  for linearity tests
            Go to 99
          End If
c
          Do 30 N = 1, NUse
            Do 20 M = N, NUse          ! Load Covar array
              If (M .eq. N) then
                If (Sigma(I,J,N) .eq. Sigma(I,J,N)) then
                  Covar(N,N) = Sigma(I,J,N)**2
                  If (.not. Corrl8d) Go to 30
                Else
                  NNOK5 = NNOK5 + 1    ! Got a NaN in the Sigma array
                  if (NNOK5 .eq. 1) print *,
     +               'NaN in Sigma array for pixel ',I,' ',J
                  Go to 99
                End If
              Else
                If (IzDN) then
                  Covar(M,N) = Cube(I,J,Indx(N))/Gain(IA2D)
                Else
                  Covar(M,N) = Cube(I,J,Indx(N))
                End If
                If (Covar(M,N) .gt. 0.99*Covar(N,N))
     +              Covar(M,N) = 0.99*Covar(N,N)
                If (Covar(M,N) .lt. 0.0) Covar(M,N) = 0.0
                Covar(N,M) = Covar(M,N)
              End If
20          Continue
30        Continue
c
          If (Corrl8d) then            ! Correlated errors

            Call MatInv(Covar,Weight,NUse,NPlanes,T,Indx,OK)
            If (.not.OK) then
              NNOK3 = NNOK3 + 1
              If (NNOK3 .eq. 1) then
                print *,'LinQA: singular covariance matrix for pixel ',
     +                   I,' ',J
                do 34 N = 1, NUse
                  do 32 M = 1, NUse-1
                    write(6,'(F10.1$)') Covar(M,N)
32                continue
                  write(6,'(F10.1)') Covar(NUse,N)
34              continue
              End If
              Go to 99
            End If
c
            Do 50 N = 1, NUse
              W(N) = 0.0
              Z(N) = 0.0
              Do 40 M = 1, NUse
                W(N) = W(N) + Weight(N,M)
                Z(N) = Z(N) + Weight(N,M)*X(M)
40            Continue
50          Continue
c
          Else                         ! Uncorrelated errors

          Do 55 N = 1, NUse
              W(N) = 1.0/Covar(N,N)
55          Continue
          End If
c
          K1 = 0.0d0
          K2 = 0.0d0
          K3 = 0.0d0
          K4 = 0.0d0
          K5 = 0.0d0
          XBar = 0.0
          YBar = 0.0
          Do 60 N = 1, NUse
            If (Corrl8d) then
              K1 = K1 - Y(N)*Z(N)
              K2 = K2 + X(N)*Z(N)
            Else
              K1 = K1 - W(N)*X(N)*Y(N)
              K2 = K2 + W(N)*X(N)*X(N)
            End If
            K3 = K3 + W(N)*X(N)
            K4 = K4 - W(N)*Y(N)
            K5 = K5 + W(N)
            XBar = XBar + X(N)
            YBar = YBar + Y(N)
60        Continue
c
          Tmp = K3*K3 - K2*K5
          If (Tmp .eq. 0.0) then
            NNOK4 = NNOK4 + 1
            go to 99
          End If
          A = (K1*K5 - K3*K4)/Tmp
          B = -(K1 + A*K2)/K3
          XBar = XBar/dfloat(NUse)
          YBar = YBar/dfloat(NUse)
c
          ChiSq = 0.0d0
          SumSq = 0.0d0
          K1    = 0.0d0
          K2    = 0.0d0
          K3    = 0.0d0
          Do 80 N = 1, NUse
            SumSq = SumSq + Dble(Y(N) - A*X(N) - B)**2
            K1 = K1 + Dble(Y(N) - YBar)*Dble(X(N) - XBar)
            K2 = K2 + Dble(Y(N) - YBar)**2
            K3 = K3 + Dble(X(N) - XBar)**2
            If (Corrl8d) then
              Do 70 M = 1, NUse
                ChiSq = ChiSq + Dble(Weight(M,N))*Dble(Y(M) - A*X(M)-B)
     +                                           *Dble(Y(N) - A*X(N)-B)
70            Continue
            Else
              ChiSq = ChiSq + W(N)*Dble(Y(N) - A*X(N)-B)**2
            End If
80        Continue
c
          If (K2*K3 .gt. 0.0d0) then
            RhoLin = K1/dsqrt(K2*K3)
          Else
            RhoLin = R4Blank
          End If
          QAimg(I,J,1) = ChiSq/dfloat(NUse-2)
          QAimg(I,J,2) = RhoLin
          QAimg(I,J,3) = dsqrt(SumSq/dfloat(NUse))
          If (NPlQA .gt. 3) then
            QAimg(I,J,4) = A
            QAimg(I,J,5) = B
            QAimg(I,J,6) = dfloat(NUse-2)
          End If
          If (QAimg(I,J,1) .gt. ChiSqrMax) LZNPCHIF = LZNPCHIF + 1
          If (QAimg(I,J,1) .lt. ChiSqrMin) LZNPCHIM = LZNPCHIM + 1
          Go to 100
c
99        QAimg(I,J,1) = R4Blank
          QAimg(I,J,2) = R4Blank
          QAimg(I,J,3) = R4Blank
          If (NPlQA .gt. 3) then
            QAimg(I,J,4) = R4Blank
            QAimg(I,J,5) = R4Blank
            QAimg(I,J,6) = R4Blank
          End If
100     Continue
200   Continue
c
      If (NNOK1+NNOK2+NNOK3+NNOK4+NNOK5 .gt. 0)
     +  print *,'LinQA: no. pixels not processed for reason shown'
      If (NNOK1 .ne. 0) print *,'       pmask:                  ',NNOK1
      If (NNOK2 .ne. 0) print *,'       not enough points:      ',NNOK2
      If (NNOK3 .ne. 0) print *,'       singular cov. matrix:   ',NNOK3
      If (NNOK4 .ne. 0) print *,'       singular linear system: ',NNOK4
      If (NNOK5 .ne. 0) print *,'       NaNs in Sigma array:    ',NNOK5
c
      LZILAVGF = 0.0
      LZILAVGL = 0.0
      LZILSTDF = 0.0
      LZILSTDL = 0.0
      LZILCHIS = 0.0
      LZILCOER = 0.0
      LZILRMSS = 0.0
      LZILSLOP = 0.0
      LZILZINT = 0.0
      LZILFREE = 0.0
      N = 0
c
      Do 320 J = yl1, yh1
        Do 310 I = xl1, xh1
          If (Cube(I,J,NSkip+1) .ne. Cube(I,J,NSkip+1)) go to 310
          If (Cube(I,J,NPlanes) .ne. Cube(I,J,NPlanes)) go to 310
          Do 305 M = 1, NPlQA
            If (QAimg(I,J,M) .ne. QAimg(I,J,M)) go to 310
305       Continue
          N = N + 1
          LZILAVGF = LZILAVGF + Cube(I,J,NSkip+1)
          LZILAVGL = LZILAVGL + Cube(I,J,NPlanes)
          LZILSTDF = LZILSTDF + Cube(I,J,NSkip+1)**2
          LZILSTDL = LZILSTDL + Cube(I,J,NPlanes)**2
          LZILCHIS = LZILCHIS + QAimg(I,J,1)
          LZILCOER = LZILCOER + QAimg(I,J,2)
          LZILRMSS = LZILRMSS + QAimg(I,J,3)
          If (NPlQA .gt. 3) then
            LZILSLOP = LZILSLOP + QAimg(I,J,4)
            LZILZINT = LZILZINT + QAimg(I,J,5)
            LZILFREE = LZILFREE + QAimg(I,J,6)
          End If
310     Continue
320   Continue
c
      If (N. lt. 3) then
        LZILAVGF = R4Blank
        LZILAVGL = R4Blank
        LZILSTDF = R4Blank
        LZILSTDL = R4Blank
        LZILCHIS = R4Blank
        LZILCOER = R4Blank
        LZILRMSS = R4Blank
        If (NPlQA .gt. 3) then
          LZILSLOP = R4Blank
          LZILZINT = R4Blank
          LZILFREE = R4Blank
        End If
      Else
        LZILAVGF = LZILAVGF/dfloat(N)
        LZILAVGL = LZILAVGL/dfloat(N)
        LZILSTDF = dsqrt(dabs(LZILSTDF/dfloat(N) - LZILAVGF**2))
        LZILSTDL = dsqrt(dabs(LZILSTDL/dfloat(N) - LZILAVGL**2))
        LZILCHIS = LZILCHIS/dfloat(N)
        LZILCOER = LZILCOER/dfloat(N)
        LZILRMSS = LZILRMSS/dfloat(N)
        If (NPlQA .gt. 3) then
          LZILSLOP = LZILSLOP/dfloat(N)
          LZILZINT = LZILZINT/dfloat(N)
          LZILFREE = LZILFREE/dfloat(N)
        End If
      End If
c
      LZUNAVGF = 0.0
      LZUNAVGL = 0.0
      LZUNSTDF = 0.0
      LZUNSTDL = 0.0
      LZUNCHIS = 0.0
      LZUNCOER = 0.0
      LZUNRMSS = 0.0
      LZUNSLOP = 0.0
      LZUNZINT = 0.0
      LZUNFREE = 0.0
      N = 0
c
      Do 340 J = yl2, yh2
        Do 330 I = xl2, xh2
          If (Cube(I,J,NSkip+1) .ne. Cube(I,J,NSkip+1)) go to 330
          If (Cube(I,J,NPlanes) .ne. Cube(I,J,NPlanes)) go to 330
          Do 325 M = 1, NPlQA
            If (QAimg(I,J,M) .ne. QAimg(I,J,M)) go to 330
325       Continue
          N = N + 1
          LZUNAVGF = LZUNAVGF + Cube(I,J,NSkip+1)
          LZUNAVGL = LZUNAVGL + Cube(I,J,NPlanes)
          LZUNSTDF = LZUNSTDF + Cube(I,J,NSkip+1)**2
          LZUNSTDL = LZUNSTDL + Cube(I,J,NPlanes)**2
          LZUNCHIS = LZUNCHIS + QAimg(I,J,1)
          LZUNCOER = LZUNCOER + QAimg(I,J,2)
          LZUNRMSS = LZUNRMSS + QAimg(I,J,3)
          If (NPlQA .gt. 3) then
            LZUNSLOP = LZUNSLOP + QAimg(I,J,4)
            LZUNZINT = LZUNZINT + QAimg(I,J,5)
            LZUNFREE = LZUNFREE + QAimg(I,J,6)
          End If
330     Continue
340   Continue
c
      If (N. lt. 3) then
        LZUNAVGF = R4Blank
        LZUNAVGL = R4Blank
        LZUNSTDF = R4Blank
        LZUNSTDL = R4Blank
        LZUNCHIS = R4Blank
        LZUNCOER = R4Blank
        LZUNRMSS = R4Blank
        If (NPlQA .gt. 3) then
          LZUNSLOP = R4Blank
          LZUNZINT = R4Blank
          LZUNFREE = R4Blank
        End If
      Else
        LZUNAVGF = LZUNAVGF/dfloat(N)
        LZUNAVGL = LZUNAVGL/dfloat(N)
        LZUNSTDF = dsqrt(dabs(LZUNSTDF/dfloat(N) - LZUNAVGF**2))
        LZUNSTDL = dsqrt(dabs(LZUNSTDL/dfloat(N) - LZUNAVGL**2))
        LZUNCHIS = LZUNCHIS/dfloat(N)
        LZUNCOER = LZUNCOER/dfloat(N)
        LZUNRMSS = LZUNRMSS/dfloat(N)
        If (NPlQA .gt. 3) then
          LZUNSLOP = LZUNSLOP/dfloat(N)
          LZUNZINT = LZUNZINT/dfloat(N)
          LZUNFREE = LZUNFREE/dfloat(N)
        End If
      End If
c
      Return
      End
