/*********************************************** 
charge_addition.c

Purpose

Estimate the charge added by the posited radiation hit.

Overview
Subtract the average charge jump as indicated by the fit to the data
before the posited rad hit from the charge jump at the posited rad hit
location.

Definitions of Variables

External:
See radhit.h

Internal:
D_Charge_StepHere = Charge step at posited radhit location.
D_Charge_StepNominal = Charge step before posited radhit.

***********************************************/

#include <iostream>
#include <string.h>

#include "more_includes.h"

extern "C" void dlutl_vecmult(int, int, double*, double*, double*, int*);
			
int M_Computation::charge_addition(	M_Initialize	*GP_Init,
					M_Status	*GP_Stat)
{
int	k, m;
int	I_Mult;
double	D_Sum;
double	D_Charge_StepNominal;
float	*FP_Mat_GProj;

if(GP_Stat->I_status) return(GP_Stat->I_status);

FP_Mat_GProj =
&(GP_Init->FP_Mat_G[I_NumSamples*I_NumSamples*I_Max_LLikely_Position]);
#if 0
dlutl_vecmult(	I_NumSamples,
		I_NumSamples,
		FP_Mat_GProj,
		DP_Data_Vec_Charge,
		DP_Fit_Vec_Charge,
		&(GP_Stat->I_status));

#else
for(k=0;k<I_NumSamples;k++)	{
D_Sum = 0; I_Mult = k*I_NumSamples;
for(m=0;m<I_NumSamples;m++)
D_Sum += FP_Mat_GProj[I_Mult+m]*DP_Data_Vec_Charge[m];
DP_Fit_Vec_Charge[k] = D_Sum;
}
#endif

//for(k=0;k<I_NumSamples;k++)
//printf("charge_addition: DP_Fit_Vec_Charge[%d] = %f\tDP_Data_Vec_Charge[%d] = %f\n", k, DP_Fit_Vec_Charge[k],  k, DP_Data_Vec_Charge[k]);

		
D_Charge_StepHere    = DP_Fit_Vec_Charge[I_Max_LLikely_Position    ] -
		       DP_Fit_Vec_Charge[I_Max_LLikely_Position - 1];

if(I_Max_LLikely_Position-2>=0 && I_Max_LLikely_Position+1<I_NumSamples) {			    
D_Charge_StepNominal = ((DP_Fit_Vec_Charge[I_Max_LLikely_Position - 1] -
		         DP_Fit_Vec_Charge[I_Max_LLikely_Position - 2])* \
			I_Max_LLikely_Position +
		        (DP_Fit_Vec_Charge[I_Max_LLikely_Position + 1] -
		         DP_Fit_Vec_Charge[I_Max_LLikely_Position    ])* \
			(I_NumSamples-I_Max_LLikely_Position))/I_NumSamples;			  

D_Meas_Flux = D_Charge_StepNominal;
}

D_Meas_Charge_Add = D_Charge_StepHere - D_Charge_StepNominal;

//if(GP_Stat->I_Verbose)
//printf("charge_addition: D_Charge_StepHere = %f - %f\n",DP_Fit_Vec_Charge[I_Max_LLikely_Position], DP_Fit_Vec_Charge[I_Max_LLikely_Position - 1]);

//if(GP_Stat->I_Verbose)
//printf("charge_addition: D_Charge_StepHere = %f at %d\n", D_Charge_StepHere, I_Max_LLikely_Position);

//if(GP_Stat->I_Verbose)
//printf("charge_addition: D_Meas_Charge_Add = %f at %d\n", D_Meas_Charge_Add, I_Max_LLikely_Position);

  return(GP_Stat->I_status);
}




