/*********************************************** 
endpoint_near.c

Purpose

Check the near endpoint for a radhit.

Overview
Compute the endpoint charge added and compare to the
charge added at the  maximum likelyhood position.
Save values of added charge, maximum likelyhood
index, and flux if the endpoint charge difference is greater.

Definitions of Variables

External:
See radhit.h

Internal:
D_Meas_FluxHere = Estimate of flux using additional good sample.
D_Meas_FluxAfter = Flux of the longer segment.
D_Meas_Charge_AddHere = Charge jump at the first two samples.

***********************************************/

#include <iostream>
#include <math.h>
#include "more_includes.h"
#include "nanvalue.h"


int M_Computation::endpoint_near(	M_Status	*GP_Stat)
{
if(GP_Stat->I_status) return(GP_Stat->I_status);

double D_Meas_Charge_StepHere;

/* Find the charge step at the near endpoint (sample no. 2) */	
D_Meas_Charge_StepHere	=
DP_Fit_Vec_Charge[I_Max_LLikely_Position - 1] -
DP_Fit_Vec_Charge[I_Max_LLikely_Position - 2];

/* If the charge added by the posited radhit here is greater than
   the previous value, save the new flux, charge, and index. */
if(fabs(D_Meas_Charge_StepHere) > fabs(D_Charge_StepHere))	{

/* The new value of Charge_Step */
D_Charge_StepHere = D_Meas_Charge_StepHere;

/* The sample before this one is good; use it for the slope */
recompute_slope_near(GP_Stat);

/* Find the charge portion presumed to come from radiation hit */
D_BestChargeAdd = D_Meas_Charge_StepHere - D_BestFlux;

/* Save the index */
I_BestMaxLLPos = I_Max_LLikely_Position - 1;
}

  return(GP_Stat->I_status);
}


int M_Computation::recompute_slope_near(	M_Status	*GP_Stat)
{
if(GP_Stat->I_status) return(GP_Stat->I_status);

int		k, I_NewLength;
double	D_Mean_X, D_InnerProd_XX, D_Correlation_XX;
double	D_Mean_Y, D_InnerProd_XY, D_Correlation_XY;

I_NewLength = I_NumSamples-I_Max_LLikely_Position+1;

/* Compute the slope and standard deviation */
D_Mean_X = 0;
for(k=I_Max_LLikely_Position-1;k<I_NumSamples;k++) D_Mean_X += (k+1);
D_Mean_X = D_Mean_X/I_NewLength;

D_Mean_Y = 0;
for(k=I_Max_LLikely_Position-1;k<I_NumSamples;k++) {
	D_Mean_Y += DP_Data_Vec_Charge[k];
}
D_Mean_Y = D_Mean_Y/I_NewLength;

D_InnerProd_XX = 0;
for(k=I_Max_LLikely_Position-1;k<I_NumSamples;k++) D_InnerProd_XX += (k+1)*(k+1);
D_Correlation_XX = D_InnerProd_XX - I_NewLength*D_Mean_X*D_Mean_X;

D_InnerProd_XY = 0;
for(k=I_Max_LLikely_Position-1;k<I_NumSamples;k++) D_InnerProd_XY += (k+1)*DP_Data_Vec_Charge[k];
D_Correlation_XY = D_InnerProd_XY - I_NewLength*D_Mean_X*D_Mean_Y;

/* Check for exception */
if(D_Correlation_XX<=0) {
	GP_Stat->I_status = COMPUTE_RES|POS_EXPECTED;

        /* Changed TYB 6/1/09 */
        D_BestFlux = *(double *)&mynand;

	//	D_BestFlux = NANVALUE;

    return(GP_Stat->I_status);
}

D_BestFlux = D_Correlation_XY/D_Correlation_XX;

	return(GP_Stat->I_status);
}
