/*********************************************** 
ll_comp.c

Purpose

Compute Log-Likelyhood given intermediate inputs.

Overview
The squared error of the fit of the data to the basis vectors
is found and raised to a power, then divided by the determinant
of the product of the transpose of the basis matrix and itself.
Then the natural log is taken.

Definitions of Variables

External:
See radhit.h

Internal:
D_Fit_Error = Inner product of data and error between linear fit and data.
D_Pow = Power in likelyhood formula.
D_Fit_Error_Pow = D_Fit_Error raised to power.
D_Likelyhood = Likelyhood of changepoint occurrence.

***********************************************/

#include <stdio.h>
#include <math.h>
#include <float.h>
#include "more_includes.h"

/* Added TYB 5/1/09 */
#include "nanvalue.h"

int M_Computation::ll_comp(	int	I_Posited_Radhit_Position_LL,
				double	*DP_Data_Fit_Inner_Prod,
				double	*DP_Data_Data_Inner_Prod,
				double	*DP_LLhood,
				M_Initialize	*GP_Init,
				M_Status	*GP_Stat)
{
int	I_NumColumns;
double	D_Fit_Error;
double	D_Pow;
double	D_Fit_Error_Pow, D_Log_Fit_Error_Pow;
double	D_Likelyhood, D_Log_Likelyhood;
double	D_Frac;
int	I_Exponent;

if(GP_Stat->I_status) return(GP_Stat->I_status);

/* Copy structure info to local variable */
I_NumColumns = GP_Init->I_NumColumns;

/* Cast to float in difference to eliminate rounding errors */
/* which could cause small negative fit error when it should be zero  */
D_Fit_Error = (float) *DP_Data_Data_Inner_Prod - (float) *DP_Data_Fit_Inner_Prod;
//D_Fit_Error =  *DP_Data_Data_Inner_Prod - *DP_Data_Fit_Inner_Prod;

//printf("%d\t%f\t%f\n", I_Posited_Radhit_Position_LL, *DP_Data_Data_Inner_Prod, *DP_Data_Fit_Inner_Prod);
//D_Fit_Error *= 0.00000001;
//printf("%e\t", D_Fit_Error);


if(D_Fit_Error < 0)
/* Account for possible small negative error */
if(fabs(D_Fit_Error/(*DP_Data_Data_Inner_Prod)) > 1e-6)
{GP_Stat->I_status = LL_COMP|ROOT_NEG; return(GP_Stat->I_status);}
else {
D_Fit_Error = 0;
}

/* Fit is perfect, likelyhood is infinite */
if(D_Fit_Error == 0)	{
*DP_LLhood = (double) 3.40282346638528860e38;
return(GP_Stat->I_status);
}

D_Pow = -((double) I_Num_Frames - (double) I_NumColumns - 1)/2;
//D_Fit_Error_Pow = (D_Fit_Error==0) ? 0 : pow(D_Fit_Error, D_Pow);

D_Log_Fit_Error_Pow = D_Pow*log(D_Fit_Error);

D_Log_Likelyhood = D_Log_Fit_Error_Pow - \
                   log(GP_Init->DP_Det_G[I_Posited_Radhit_Position_LL]);

//printf("%e\t", D_Log_Fit_Error_Pow);
//printf("%e\t", GP_Init->DP_Det_G[I_Posited_Radhit_Position_LL]);
//printf("%e\t", log(GP_Init->DP_Det_G[I_Posited_Radhit_Position_LL]));

#if 0
if(GP_Init->DP_Det_G[I_Posited_Radhit_Position_LL] == 0)
{GP_Stat->I_status = LL_COMP|DIVIDE_ZERO; return(GP_Stat->I_status);}
D_Likelyhood = D_Fit_Error_Pow/GP_Init->DP_Det_G[I_Posited_Radhit_Position_LL];

printf("%e\t", D_Likelyhood);
if(D_Likelyhood < 0)
{GP_Stat->I_status = LL_COMP|LOG_NEG; return(GP_Stat->I_status);}
if(D_Likelyhood == 0)
*DP_LLhood = (double) 3.40282346638528860e38;
else
*DP_LLhood = log(D_Likelyhood);
#endif
*DP_LLhood = D_Log_Likelyhood;

/* Check that D_Log_Likelyhood is a valid number */
if(iznand(*DP_LLhood))
GP_Stat->I_status = LL_COMP|NOT_A_NUMBER;


//printf("DP_LLikelyhood[%d] = %f\n", I_Posited_Radhit_Position_LL,
//*DP_LLhood);

  return(GP_Stat->I_status);
}




