/*********************************************** 
log_likelyhood.c

Purpose

Compute the log-likelyhood of the data
for each possible radiation hit location.

Overview
Use the pseudoinverse of the basis matrix to project the data vector
onto the linear regression subspace. Find the length of the error 
vector. Apply with determinant to formula form compuation of likelyhood
and take natural log. Do for changepoints from M=3 to M=I_NumSamples-1.

Definitions of Variables

External:
See radhit.h

Internal:
m, I_Posited_Radhit_Position_LL = Posited Radiation Hit Position.
FP_Mat_GPseudo = Pseudomatrix associated with I_Posited_Radhit_Position_LL.
DP_Fit_Vec_Charge_LL = Fit of data Charge vector.
D_Data_Fit_Inner_Prod = Inner Product of Data and Fit Vectors.
D_Data_Data_Inner_Prod = Inner Product of Data Vector and itself.

***********************************************/

#include <iostream>

#include "more_includes.h"

extern "C" void dlutl_vecmult(int, int, double*, double*, double*, int*);
extern "C" void dlutl_innerprod(int, double*, double*, double*, int*);

 
int M_Computation::log_likelyhood(	int I_Posited_Radhit_Position_LL,
					M_Initialize	*GP_Init,
					M_Status	*GP_Stat)
{
int	k, m;
float	*FP_Mat_GProj;
double	D_Data_Fit_Inner_Prod;
double	D_Data_Data_Inner_Prod;
double	D_Sum;
int	I_Mult;

//printf("%d\t0x%x\n", 1, GP_Stat->I_status);

if(GP_Stat->I_status) return(GP_Stat->I_status);

//printf("%d\t0x%x\n", m, GP_Stat->I_status);
 
/* Multiply the charge vector by the projector matrix */
FP_Mat_GProj =
&(GP_Init->FP_Mat_G[I_NumSamples*I_NumSamples*I_Posited_Radhit_Position_LL]);
#if 0
dlutl_vecmult(	I_NumSamples,
		I_NumSamples,
		FP_Mat_GProj,
		DP_Data_Vec_Charge,
		DP_Fit_Vec_Charge,
		&(GP_Stat->I_status));
#else		
for(k=0;k<I_NumSamples;k++)	{
D_Sum = 0; I_Mult = k*I_NumSamples;
for(m=0;m<I_NumSamples;m++)
D_Sum += FP_Mat_GProj[I_Mult+m]*DP_Data_Vec_Charge[m];
DP_Fit_Vec_Charge[k] = D_Sum;
}
#endif
		
//for(k=0;k<I_NumSamples;k++)
//DP_Fit_Vec_Charge[k] = DP_Data_Vec_Charge[k] - DP_Fit_Vec_Charge[k];

/* Compute the inner product of DP_Fit_Vec_Charge_LL and the charge vector. */
dlutl_innerprod(	I_NumSamples,
			DP_Data_Vec_Charge,
			DP_Fit_Vec_Charge,
			&D_Data_Fit_Inner_Prod,
			&(GP_Stat->I_status));
 

/* Compute the inner product of DP_Fit_Vec_Charge_LL and the charge vector. */
//#if 0
dlutl_innerprod(	I_NumSamples,
			DP_Data_Vec_Charge,
			DP_Data_Vec_Charge,
			&D_Data_Data_Inner_Prod,
			&(GP_Stat->I_status));
//#endif

//printf("%d\t%f\t%f\n", m,D_Data_Fit_Inner_Prod, D_Data_Data_Inner_Prod);

/* Apply as inputs to Log-Likelyhood formula */
ll_comp(	I_Posited_Radhit_Position_LL,
		&D_Data_Fit_Inner_Prod,
		&D_Data_Data_Inner_Prod,
		&DP_LLikelyhood[I_Posited_Radhit_Position_LL],
		GP_Init,
		GP_Stat);
		
  return(GP_Stat->I_status);
}
