/*********************************************** 
max_llikelyhood.c

Purpose

Find the sample number which has the highest
log-likelyhood of having a radiation hit.
Create the correspoinding best-fit vector.

Overview
Run through the values, saving the index of the
current highest value. Project the data using
the  pseudoinverse correspoinding to that index.

Definitions of Variables

External:
See radhit.h

Internal:
FP_Mat_GPseudo = Pseudomatrix associated with IP_Max_LLikely_Position.

***********************************************/

#include <iostream>
#include <string.h>

#include "more_includes.h"

extern "C" void dlutl_vecmult(int, int, double*, double*, double*, int*);

int M_Computation::max_llikelyhood(	M_Initialize	*GP_Init,
					M_Status	*GP_Stat)
{
int k, mm;
float	*FP_Mat_GPseudo;

if(GP_Stat->I_status) return(GP_Stat->I_status);

/* Find the index of the maximum likelyhood */
//maxll_findmax( GP_Init, GP_Stat);			

//printf("maxll_findmax done.\n");

/* Multlply the charge vector by the pseudoinverse
   for I_Max_LLikely_Position */
FP_Mat_GPseudo =
&(GP_Init->FP_Mat_G[I_NumSamples*I_NumSamples*I_Max_LLikely_Position]);
#if 1
dlutl_vecmult(	I_NumSamples,
		I_NumSamples,
		FP_Mat_GPseudo,
		DP_Data_Vec_Charge,
		DP_Fit_Vec_Charge,
		&(GP_Stat->I_status));
#else
memset((void *) DP_Fit_Vec_Charge, 0, I_NumSamples*sizeof(double));

if(0)
for(k=0;k<I_NumSamples;k++)	
for(mm=0;mm<I_NumSamples;mm++)
printf("%d\t%d\t%f\t%f\n", k, mm, FP_Mat_GPseudo[k*I_NumSamples+mm], DP_Data_Vec_Charge[mm]);
//DP_Fit_Vec_Charge[k] += FP_Mat_GPseudo[k*I_NumSamples+mm]*DP_Data_Vec_Charge[mm];
#endif

//printf("vecmult done.\n");

  return(GP_Stat->I_status);
}





