/*********************************************** 
mkbasis.c

Purpose

Create a matrix whose columns are the basis vectors.

Overview
The gemeral form of the regression and the changepoint location is encoded here.
For a linear regression, the two basis vectors are a column of ones and a column
consisting of 1, 2, 3,... When a changepoint is included, there are four basis
vectors, one pair with all zeros in the column entries including and after the
changepoint, and one pair with all zeros in the column entries up to the changepoint.
This creates a two line-segment fit to the data.

Definitions of Variables

External:
See radhit.h

Internal:
I_GMatrix_Offset = Optionally leave first part of output matrix untouched.


***********************************************/

#include <iostream>

#include "more_includes.h"



int M_Initialize::mkbasis(	int	I_Posited_Radhit_Position_Init,
				int	I_Num_Frames,
				int	I_NumColumnsIn,
				double	*DP_GMatrix,
				M_Status	*GP_Stat)
{
int	k;
int	I_GMatrix_Offset;

if(GP_Stat->I_status) return(GP_Stat->I_status);

I_GMatrix_Offset = 0;

for(k=0;k<I_Posited_Radhit_Position_Init;k++)	{
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 0] = 1;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 1] = 0;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 2] = k+1;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 3] = 0;
}

for(k=I_Posited_Radhit_Position_Init;k<I_Num_Frames;k++)	{
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 0] = 0;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 1] = 1;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 2] = 0;
DP_GMatrix[I_GMatrix_Offset+I_NumColumnsIn*k + 3] = k+1;
}

  return(GP_Stat->I_status);
}






