// radhit.h
// Classes to be defined for each application are written here.


class M_Constants {
public:
	short	S_NotData;
	int	I_NumColumns;
	int	I_NumSamplesSkip; /* n samples to skip at start of each ramp
                                     (added by Bob 07/01/11) */
	int	I_MinPlanesForSkip; /* Minimmu planes required for any skip
                                     (added by Bob 07/04/03) */
	int	I_NumSamplesMax;
	int	I_Decimation;
	int	I_MaxNumHits;
	int	I_ANDInteger;
	int	I_DeclBadSampsNum;
	int     I_MaximumStoredMatrixLen;
	double	D_Readnoise;
	double	D_NomRHMag;
	double	D_RPPriorProb;
	double	D_Thresh;
	double	D_Gain;
	double	D_DeclBadSampsThres;

	
public:
int get_values(M_CmdParse*, M_Namelist*, void*, M_Status*);
M_Constants();
~M_Constants();
};




class M_Initialize {
public:
/* Number of columns in each basis matrix */
int	I_NumColumns;
/* Basis matrices and their determinants */
float*	FP_Mat_G;
double*	DP_Det_G;


public:
int initialize(int, M_Constants*, M_Status*);
int allocate(int, M_Status*);
int mkbasis(int, int, int, double*, M_Status*);
M_Initialize();
~M_Initialize();
};


class M_Computation{
public:
/* Flag */
int	I_Found_Radhit;

/* Dimensions */
int	I_Length_X;
int	I_Length_Y;
int	I_Num_Frames;
int	I_WindowLen;

/* Intermediate Results */
short	*SP_MaskVector;
short	*SP_SegmentVector;

double	*DP_Det_G;
float	*FP_Mat_G;

int	*IP_Declared_Position;
double	*DP_Measured_Flux;
double	*DP_Measured_Charge_Add;
double	*DP_Probability_Radhit;

double	*DP_Data_Vec_Charge;
double	*DP_LLikelyhood;
double	*DP_Fit_Vec_Charge;
double	D_Readnoise;

/* Computed Results */
int	I_NumSamples;
int	I_SegmentStart;
int	I_SegmentStop;
int	I_SegmentLen;
int	I_Max_LLikely_Position;
double	D_Charge_StepHere;
double	D_Meas_Flux;
double	D_Meas_Charge_Add;
double	D_Prob_Radhit;
int	I_HitCount;
int	I_PixelHitCount;
int	I_HitPixels;

int	I_BestMaxLLPos;
double	D_BestFlux;
double	D_BestChargeAdd;

/* Stored Results */
double*	data_in;
double*	dmask_in;
double*	pmask_in;
double*	readnoise_in;
double*	data1;
double*	data2;
double*	data3;
double*	data4;
double*	data5;

public:
int compute_results(M_FITSinfo*, M_Constants*, M_Initialize*, M_Status*);
int allocate(M_Status*);
int find_max_ll_samp(int, int, M_Constants*, M_Initialize*, M_Status*);
int log_likelyhood(int, M_Initialize*, M_Status*);
int ll_comp(int, double*, double*, double*, M_Initialize*, M_Status*);
int max_llikelyhood(M_Initialize*, M_Status*);
int maxll_findmax(M_Initialize*, M_Status*);
int charge_addition(M_Initialize*, M_Status*);
int saturating_radhit(int, int, int, M_Constants*, M_Initialize*, M_Status*);
int endpoints(M_Status*);
int endpoint_far(M_Status*);
int endpoint_near(M_Status*);
int recompute_slope_far(M_Status*);
int recompute_slope_near(M_Status*);
int prob_radhit(M_Constants*, M_Initialize*, M_Status*);
M_Computation();
~M_Computation();
};

