/*********************************************** 
computation.cpp

Purpose
Main computational processing loop of program.

Overview
Loop over pixels, call processing functions for each pixel.

Definitions of Variables

External:
See radhit.h

Internal:


***********************************************/

#include <iostream>
#include <string.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include "more_includes.h"
#include "nanvalue.h"

int write_nan(double* DP_Result, M_Status *GP_Stat);
int write_nan(float* FP_Result, M_Status *GP_Stat);
int bits_printf(short x);
short bits_get(short x, int i);
int compute_segments(int, int, short*, short, int, short*);
int radhit_detector();

M_Computation::M_Computation()
{
	/* Constructor */
	I_Found_Radhit = 0;
	I_Length_X = 0;
	I_Length_Y = 0;
	I_Num_Frames = 0;
	I_NumSamples = 0;
	I_SegmentLen = 0;
	I_HitCount = 0;
	I_PixelHitCount = 0;
	I_HitPixels = 0;
	D_Meas_Flux = 0;
	D_Meas_Charge_Add = 0;
	D_Prob_Radhit = 0;
	data_in = NULL;
	dmask_in = NULL;
	pmask_in = NULL;
	readnoise_in = NULL;
	data1 = NULL;
	data2 = NULL;
	data3 = NULL;
	data4 = NULL;
	data5 = NULL;
}


M_Computation::~M_Computation()
{
if(SP_MaskVector!=NULL) free(SP_MaskVector);
if(SP_SegmentVector!=NULL) free(SP_SegmentVector);
if(IP_Declared_Position!=NULL) free(IP_Declared_Position);
if(DP_Measured_Flux!=NULL) free(DP_Measured_Flux);
if(DP_Measured_Charge_Add!=NULL) free(DP_Measured_Charge_Add);
if(DP_Probability_Radhit!=NULL) free(DP_Probability_Radhit);
if(DP_Data_Vec_Charge!=NULL) free(DP_Data_Vec_Charge);
if(DP_LLikelyhood!=NULL) free(DP_LLikelyhood);
if(DP_Fit_Vec_Charge!=NULL) free(DP_Fit_Vec_Charge);
if(data1!=NULL) free(data1);
if(data2!=NULL) free(data2);
if(data3!=NULL) free(data3);
if(data4!=NULL) free(data4);
if(data5!=NULL) free(data5);
}


int M_Computation::compute_results(	M_FITSinfo	*GP_Infiles,
					M_Constants	*GP_ConstIn,
					M_Initialize	*GP_Pseudo,
					M_Status	*GP_Stat)
{

int	i, j, k, I_IndexOut, I_IndexIn, I_ShowX=13, I_ShowY=1;
int	I_Segment, I_SetSegment, I_DoNotProcess;
int	I_NumPixels=0, I_Decimation=1, I_ANDInteger;

M_Constants* GP_Const = (M_Constants*)  GP_ConstIn;

/* Declare local names for input FITS data */
M_FITSinfo	*GP_RAMP;
M_FITSinfo	*GP_DMASK;
M_FITSinfo	*GP_PMASK;
M_FITSinfo	*GP_RN;

GP_RAMP = &GP_Infiles[0];
GP_DMASK = &GP_Infiles[1];
GP_PMASK = &GP_Infiles[2];
GP_RN = &GP_Infiles[3];

if(GP_Stat->I_status) return(GP_Stat->I_status);

/* Copy structure info to class or local variables */
I_Length_X = GP_RAMP->I_Length_X;
I_Length_Y = GP_RAMP->I_Length_Y;
I_Num_Frames = GP_RAMP->I_Num_Frames;
I_NumPixels = I_Length_X*I_Length_Y;
I_ANDInteger = GP_Const->I_ANDInteger;

if (I_Num_Frames < GP_Const->I_MinPlanesForSkip) GP_Const->I_NumSamplesSkip =0;

if (GP_Const->I_NumSamplesSkip != 0)
 printf ("Note: Ignoring the first %d sample(s) of each ramp\n",
  GP_Const->I_NumSamplesSkip);

if(GP_Stat->I_Debug)
printf("compute_results: I_Num_Frames = %d.\n", I_Num_Frames);

/* Allocate space for comp arrays */
allocate( GP_Stat);
if(GP_Stat->I_status) return(GP_Stat->I_status);

/* If there was an input dmask, use it, otherwise, use zeros from calloc */
if(GP_DMASK->data != NULL)
memcpy( data5, GP_DMASK->data, I_NumPixels*I_Num_Frames*sizeof(double));

/* Use local names for array pointers */
data_in = GP_RAMP->data;
dmask_in = data5;
pmask_in = GP_PMASK->data;
readnoise_in = GP_RN->data;

/* For each pixel */
for(i=0;i<I_Length_X;i++)	
for(j=0;j<I_Length_Y;j++)	{

/* Check if pixel mask indicates that */
/* no processing to be done on this pixel */
if(pmask_in!=NULL)
if(((short)pmask_in[j*I_Length_X+i])&PMASK) continue;

/* Ensure that if last pixel had NaN,
   will keep going and try next pixel */
GP_Stat->I_status = 0;

/* Convert and copy the dmask values for the current pixel */
for(k=0;k<I_Num_Frames;k++)	{
I_IndexIn = k*I_NumPixels+j*I_Length_X+i;

/* First check that inputs are numbers */
if(iznand(dmask_in[I_IndexIn]))
GP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

if(GP_Stat->I_Debug && i==I_ShowX && j==I_ShowY)	{
printf("compute_results: \t");
bits_printf((short) I_ANDInteger);
printf("\t");
bits_printf((short) dmask_in[I_IndexIn]);
}

/* Copy the dmask. I_ANDInteger can be used to set bits to zero */
SP_MaskVector[k] = (short) I_ANDInteger & (short) dmask_in[I_IndexIn];

if(GP_Stat->I_Debug && i==I_ShowX && j==I_ShowY)	{	
printf("\t");
bits_printf(SP_MaskVector[k]);
printf("\n");
}

}

/* Copy the readnoise */
if(readnoise_in!=NULL)
D_Readnoise = readnoise_in[j*I_Length_X+i];
else
D_Readnoise = GP_Const->D_Readnoise;

/* S_NotData */
//if(GP_Stat->I_Debug && i==I_ShowX && j==I_ShowY)	{ return 0; }


/* Run radhit detector at most GP_Const->I_MaxNumHits times */
I_PixelHitCount = 0;
I_SegmentStart = 0;


I_Segment = 1;
while(I_PixelHitCount < GP_Const->I_MaxNumHits ) {

/* Compute the segments table */
I_SetSegment = 0;
for(k=I_SegmentStart;k<I_Num_Frames;k++)	{

/* skip if specified by NumSamplesSkip (set segment # to 0) */
/*  (added 07/04/03 - Bob) */
if (k < GP_Const->I_NumSamplesSkip) {
  SP_SegmentVector[k] = 0;
  continue;
}

if(	SP_MaskVector[k]&GP_Const->S_NotData ||
	SP_MaskVector[k]&ged_SEGMENT ||
	SP_MaskVector[k]&ged_RADHIT)
if(I_SetSegment != 0) I_Segment++;
	  
if(SP_MaskVector[k]&GP_Const->S_NotData)
I_SetSegment = 0;
else
I_SetSegment = I_Segment;

SP_SegmentVector[k] = I_SetSegment;
}

/* Will go on to next pixel if this remains zero */
I_Found_Radhit = 0;


/* Go through segments until one is found that needs to be run */
for(I_Segment=1;;I_Segment++)	{

/* Find beginning of current segment */
for(k=0;k<I_Num_Frames;k++) if(SP_SegmentVector[k] == I_Segment) break;
I_SegmentStart = k;

/* Find beginning of next segment */
for(;k<I_Num_Frames;k++) if(SP_SegmentVector[k] != I_Segment) break;
I_SegmentStop = k;

/* Could not find both segment endpoints */
if(I_SegmentStart>I_Num_Frames - 1 || I_SegmentStop>I_Num_Frames) break;

//if(GP_Stat->I_Verbose)
//printf(" %d %d %d %d\t", i, j, I_SegmentStart, I_SegmentStop);
//if(GP_Stat->I_Verbose)
//bits_printf(SP_MaskVector[I_SegmentStart]);
//if(GP_Stat->I_Verbose)
//printf("\n");


if(!(SP_MaskVector[I_SegmentStart]&ged_DIDRHDET))	{

I_SegmentLen = I_SegmentStop - I_SegmentStart;

/* If segment length is not long enough, include adjoining saturated samples */
/*
if(I_SegmentLen < 4)	{
for(k=I_SegmentStop;k<I_Num_Frames;k++)
if(!(SP_MaskVector[k]&ged_SATURATEHI)) break;
I_SegmentStop = k;
I_SegmentLen = I_SegmentStop - I_SegmentStart;
// Mark that radhit detection was done on this segment 
SP_MaskVector[I_SegmentStart] |= ged_DIDRHDET;
// Don't need to recheck the beginning of this segment for radhits 
}
*/

/* Skip this segment if there still aren't enough samples. */
if(I_SegmentLen < 4) continue;

/* Compute how much decimation can be done. */
I_Decimation = 1;
while(ceil(I_SegmentLen/((double)I_Decimation)) > GP_Const-> I_NumSamplesMax) I_Decimation++;

/* Copy the charge ramp, with decimation, into an array, optionally
   converting from dn (A/D output) to electrons. Also, check for NaNs */
I_NumSamples = 0;
for(k=0;k<I_SegmentLen;k+=I_Decimation)	{
I_IndexIn = (I_SegmentStart+k)*I_NumPixels+j*I_Length_X+i;
if(iznand(data_in[I_IndexIn])) GP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;
DP_Data_Vec_Charge[I_NumSamples] = GP_Const->D_Gain*(data_in[I_IndexIn]);
I_NumSamples++;
}

/* Loop over all possible changepoints for which the method can work */	
find_max_ll_samp( 2, I_NumSamples-1, GP_ConstIn, GP_Pseudo, GP_Stat);

/* Use the whole segment to resolve amibiguity of decimation */
if(I_Decimation > 1)	{
for(k=0;k<I_SegmentLen;k++)	{
I_IndexIn = (I_SegmentStart+k)*I_NumPixels+j*I_Length_X+i;
DP_Data_Vec_Charge[k] = GP_Const->D_Gain*(data_in[I_IndexIn]);
}
I_NumSamples = I_SegmentLen;
find_max_ll_samp( (I_Max_LLikely_Position-1)*I_Decimation+1,
		  I_Max_LLikely_Position*I_Decimation+1,
		  GP_ConstIn, GP_Pseudo, GP_Stat);
}


/* Find the probability that a radiation hit occurred. */
prob_radhit( GP_Const, GP_Pseudo, GP_Stat);

/* Check that result is a valid number */
if(	iznand(D_Prob_Radhit)		||
	iznand(D_Meas_Charge_Add)	||
	iznand(D_Meas_Flux)	)
GP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

I_IndexOut = (I_SegmentStart+I_Max_LLikely_Position)*I_NumPixels+j*I_Length_X+i;
/* Write any error code into the array as an NaN instead of the result */
if(GP_Stat->I_status)	{
if(GP_Stat->I_SuperVerbose)
printf("compute_results: NaN at %d %d\n", i, j);

/* Changed - TYB 6/1/09 */

data1[I_IndexOut] = *(double *)&mynand;
data4[I_IndexOut] = *(double *)&mynand;

//data1[I_IndexOut] = NANVALUE;
//data4[I_IndexOut] = NANVALUE;

if(GP_Stat->I_Debug)
printf("compute_results: write_nan done: 0x%x.\n", GP_Stat->I_status);
continue;
}

/* Write the results to the corresponding arrays.*/
data1[I_IndexOut] = D_Prob_Radhit;
data4[I_IndexOut] = D_Meas_Flux;

/* Check result against given probability threshhold */
/* No radhit, check if there is a saturating radhit at the end of this segment */
if(D_Prob_Radhit <= GP_Const->D_Thresh )	{	
saturating_radhit(i, j, I_IndexOut, GP_Const, GP_Pseudo, GP_Stat);
}

if(GP_Stat->I_status==0)
if(D_Prob_Radhit > GP_Const->D_Thresh )	{
I_Found_Radhit++;
I_HitCount++;
I_PixelHitCount++;

/* Save accumulated radhit charge for this pixel */
data3[j*I_Length_X+i] += D_Meas_Charge_Add;

/* Mark the radhit in the dmask */
SP_MaskVector[I_SegmentStart+I_Max_LLikely_Position] |= ged_RADHIT;

/* Mark samples following the radhit as bad in the dmask */
if (D_Meas_Charge_Add > GP_Const->D_DeclBadSampsThres )
for(k=I_Max_LLikely_Position;k<I_SegmentLen;k++)	{
if (k-I_Max_LLikely_Position+1 > GP_Const->I_DeclBadSampsNum ) break;
SP_MaskVector[I_SegmentStart+k] |= ged_RHBADSAMP;
}

if(GP_Stat->I_Verbose)	{
printf("%02d %02d ", i, j);
printf("0x%x ", GP_Stat->I_status);
printf("%02d %02d ", I_SegmentStart, I_SegmentStop);
printf("%f ", data1[I_IndexOut]);
printf("%f ", data2[j*I_Length_X+i]);
printf("%f\t", data3[j*I_Length_X+i]);
printf("%f ", data4[I_IndexOut]);
bits_printf(SP_MaskVector[I_SegmentStart+I_Max_LLikely_Position]);
printf("\n");
}

break; /* Radhit found, must recompute segments vector: continue while loop */
}
else /* Mark that radhit detection was done on this segment */
SP_MaskVector[I_SegmentStart] |= ged_DIDRHDET;
/* If a radhit was found, should check the beginning of this segment next */


}   /* Conditional on whether to run radhit */

}   /* End of I_Segment for loop   */

/* No radhits were found, so break out of while loop */
/* after checking for a saturating radhit */
//if(GP_Stat->I_Verbose)	{
//printf("compute_results: I_Found_Radhit = %d\n", I_Found_Radhit);
//}

if(I_Found_Radhit==0)  break; 

}   /* End of while loop */

/* Copy the dmask values back into the input array */
for(k=0;k<I_Num_Frames;k++)	{
I_IndexIn = k*I_NumPixels+j*I_Length_X+i;
dmask_in[I_IndexIn] = SP_MaskVector[k];
}

/* Save the number of detected radhits for this pixel */
data2[j*I_Length_X+i] = I_PixelHitCount;

/* Add to count of number of pixels that had radhits */
 if(GP_Stat->I_Verbose) {
if ( I_PixelHitCount > 0 ) {
   if((I_HitPixels++)==0) {
       printf("first detected radhit i=%d,j=%d, I_SegmentStart=%d, I_SegmentStop=%d\n",i,j, I_SegmentStart, I_SegmentStop);
       }
   }
 }

}     /* End of j, i, loop */

/* If last pixel had NaN, will not skip rest of program */
GP_Stat->I_status = 0;

if(GP_Stat->I_Verbose)	{
printf("compute_results: done. Num NaNs = %d\tNum hits = %d\n", GP_Stat->I_NaNCount, I_HitCount);
}

return(GP_Stat->I_status);

}    /* End of compute_results */







short bits_get(short x, int i) {
    return ( x >> i) & 1;
}


int bits_printf(short x) {
    int i;
    for ( i = sizeof(short)*8-1; 0 <= i; i-- ) {
        printf("%d", bits_get(x,i));
    }
    
    return 0;
}


/* Compute the segments table */
int compute_segments(	int I_SegmentStart,
			int I_Num_Frames,
			short *SP_MaskVector,
			short S_NotData,
			int I_Segment,
			short *SP_SegmentVector
		    )
{

int	k, I_SetSegment;

I_SetSegment = 0;
for(k=I_SegmentStart;k<I_Num_Frames;k++)	{
if(SP_MaskVector[k]&S_NotData || SP_MaskVector[k]&ged_RADHIT)
if(I_SetSegment != 0) I_Segment++;
	  
if(SP_MaskVector[k]&S_NotData)
I_SetSegment = 0;
else
I_SetSegment = I_Segment;

SP_SegmentVector[k] = I_SetSegment;
}

return 0;
}


//int radhit_detector()	{	

