/*********************************************** 
constants.cpp

Purpose
Initialize the variables of the M_Constants class.

Overview
Set counts to values indicated by constant definitions.

Definitions of Variables

External:
See radhit.h


Internal:


***********************************************/

#include <iostream>

#include "more_includes.h"

M_Constants::M_Constants()
{
	/* Constructor */
	/* Conditions for which data cannot be used */
	S_NotData  = ged_SAMPSMISSING;
	S_NotData |= ged_SATURATEHI;
	S_NotData |= ged_BADSAMP;
	S_NotData |= ged_RHBADSAMP;

	/* Other variables */
	I_NumColumns = I_NUM_COLUMNS;
	I_NumSamplesMax = I_NUM_SAMPLES_MAX;
	I_NumSamplesSkip = I_NUM_SAMPLES_SKIP;
	I_MinPlanesForSkip = I_MIN_PLANES_FOR_SKIP;
	I_Decimation = 2;
	I_MaxNumHits = MAXHITS;
	I_ANDInteger = AND_INT;
	I_DeclBadSampsNum = RH9BADSAMPNUM;
	I_MaximumStoredMatrixLen = MAXSTOREDLEN;
	D_Readnoise = READNOISE;
	D_NomRHMag = NOMRHMAG;
	D_RPPriorProb = PRIORRH;
	D_Thresh = THRESH;
	D_Gain = GAIN;
	D_DeclBadSampsThres = ged_RHBADSAMPTHRES;
	
}


M_Constants::~M_Constants()
{

}


/*********************************************** 
get_values()

Purpose

Set value of selected variables.

Overview
Look up flags, namelist names, or keywords and write
associated value using get_flag_value, get_namelist_value,
or get_keyword_value. Values will be overwritten, so
highest priority lookup should be last.

Definitions of Variables

External:
See radhit_defs.h and radhit_exec.c

Internal:
I_fits_return_status = Return value of fits function.
CP_Keyname = Name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/

int M_Constants::get_values(M_CmdParse 	*GP_Cmd,
			M_Namelist	*GP_Name,
			void 		*GP_InfilesIn,
			M_Status	*GP_Stat)
{
	M_FITSinfo* GP_Infiles = (M_FITSinfo*) GP_InfilesIn;

	GP_Infiles[0].get_keyword_value("READNOISE", D_Readnoise);
	GP_Name->get_namelist_value("Readnoise", D_Readnoise);
	GP_Cmd->get_flag_value("-r", D_Readnoise);
	
	GP_Infiles[0].get_keyword_value("NOMRHMAG", D_NomRHMag);
	GP_Name->get_namelist_value("NominalRHMag", D_NomRHMag);
	GP_Cmd->get_flag_value("-m", D_NomRHMag);
	
	GP_Infiles[0].get_keyword_value("PRIOR", D_RPPriorProb);
	GP_Name->get_namelist_value("RHPriorProb", D_RPPriorProb);
	GP_Cmd->get_flag_value("-p", D_RPPriorProb);
	
	GP_Infiles[0].get_keyword_value("PTHRESH", D_Thresh);
	GP_Name->get_namelist_value("DeclThresh", D_Thresh);
	GP_Cmd->get_flag_value("-t", D_Thresh);
	
        /* (I_MinPlanesForSkip added by Bob 07/04/03) */
	GP_Name->get_namelist_value("MinPlanesForSkip", I_MinPlanesForSkip);
	GP_Cmd->get_flag_value("-xp", I_MinPlanesForSkip);
	
        /* (I_NumSamplesSkip added by Bob 07/01/11) */
	GP_Name->get_namelist_value("NumSamplesSkip", I_NumSamplesSkip);
	GP_Cmd->get_flag_value("-x", I_NumSamplesSkip);

	GP_Infiles[0].get_keyword_value("NSAMPMAX", I_NumSamplesMax);
	GP_Name->get_namelist_value("NumSamplesMax", I_NumSamplesMax);
	GP_Cmd->get_flag_value("-w", I_NumSamplesMax);
	/* Set maximum window length */
	if(I_NumSamplesMax > GP_Infiles[0].I_Num_Frames)	{
	I_NumSamplesMax = GP_Infiles[0].I_Num_Frames;
	if(GP_Stat->I_Debug)
	printf("get_values: I_NumSamplesMax too large, setting to %d\n", I_NumSamplesMax);
	}
	
	GP_Infiles[0].get_keyword_value("MAXHITS", I_MaxNumHits);
	GP_Name->get_namelist_value("MaxNumHits", I_MaxNumHits);
	GP_Cmd->get_flag_value("-h", I_MaxNumHits);
	
	GP_Infiles[0].get_keyword_value("AND_INT", I_ANDInteger);
	GP_Name->get_namelist_value("ANDInteger", I_ANDInteger);
	GP_Cmd->get_flag_value("-f", I_ANDInteger);
	
	GP_Infiles[0].get_keyword_value("MXSTRMLN", I_MaximumStoredMatrixLen);
	GP_Name->get_namelist_value("MaximumStoredMatrixLen", I_MaximumStoredMatrixLen);
	GP_Cmd->get_flag_value("-k", I_MaximumStoredMatrixLen);

	GP_Infiles[0].get_keyword_value("GAIN", D_Gain);
	GP_Name->get_namelist_value("Gain", D_Gain);
	GP_Cmd->get_flag_value("-g", D_Gain);
	
	GP_Infiles[0].get_keyword_value("RHBADSN", I_DeclBadSampsNum);
	GP_Name->get_namelist_value("NumDeclareBadAfterRH", I_DeclBadSampsNum);
	GP_Cmd->get_flag_value("-b", I_DeclBadSampsNum);
	
	GP_Infiles[0].get_keyword_value("RHBADST", D_DeclBadSampsThres);
	GP_Name->get_namelist_value("ThreshDeclareBadAfterRH", D_DeclBadSampsThres);
	GP_Cmd->get_flag_value("-S", D_DeclBadSampsThres);
	
  return(GP_Stat->I_status);
}
