/*******************************
radhit_defs.h

Purpose
Define default values for constants.

Overview
Structure members will be initialized with these constants.

*******************************/

const	char	MODULENAME[]="RADHIT";
const	char	VERSION[]="6.3";

/**** UPDATES **********************************************************
 6.3 2007/04/03 Added -xmin option (min planes required for sample skipping (-x)
 6.2 2007/01/11 Added -x option to skip planes at atart of each ramp
***********************************************************************/

/* For assigning a double NaN value - TYB 6/1/09 */
const unsigned long long mynand = 0x7fffffffffffffffLL;

const	int	NO_MORE_RADHITS=31;

#define	USAGE \
{ \
"usage: radhit  -n  <input_namelist_fname>      \\\n", \
"                -i1 <input charge ramp name>       \\\n", \
"                -i2 <dce mask file name> (optional)   \\\n", \
"                -i3 <pixel mask file name> (optional)   \\\n", \
"                -i4 <readnoise file name> (optional)   \\\n", \
"                -o1 <output_fname1>  (probs)   \\\n", \
"                -o2 <output_fname2>  (sample number) \\\n", \
"                -o3 <output_fname3>  (magnitude) \\\n", \
"                -o4 <output_fname4>  (slope)   \\\n", \
"                -o5 <output_fname5>  (dmask)   \\\n", \
"                -l  <log_fname>                \\\n", \
"                -r  <read noise>               \\\n", \
"                -m  <nominal radhit magnitude> \\\n", \
"                -p  <prior radhit probability> \\\n", \
"                -t  <declaration probability threshhold> \\\n", \
"                -w  <maximum segment length after decimation> \\\n", \
"                -h  <maximum number of hits> \\\n", \
"                -g  <gain>                     \\\n", \
"                -f  <AND integer>              \\\n", \
"                -S  <note caps in command line-radhit magnitude threshhold for declaring bad samples>\\\n", \
"                -n  <Number of samples to declare bad>\\\n", \
"                -x  <Number of samples to skip at start of ramp>\\\n", \
"                -xp <Number of planes required for doing skipping (-x)>\\\n", \
"                -k  <max segment length to store pseudoinverse for>\\\n", \
"                -d  (prints debug statements)   \\\n", \
"                -v  (verbose output)           \\\n", \
"                -vv (superverbose output)\n" \
}


const	int	DEBUGINIT=0;
const	int	VERBOSEINIT=0;
const	int	SVERBOSEINIT=0;

const	char	LOGFILENAME1[]="logfile.log";

const	double	READNOISE=120;
const	double	NOMRHMAG=3;
const	double	PRIORRH=0.01;
const	double	THRESH=0.99;
const	int	MAXHITS=1;
const	double	GAIN=1;
const	double	AND_INT=0xffffffff;
const	int	RH9BADSAMPNUM=0;
const	double	ged_RHBADSAMPTHRES=999999999999.0;
const	int	I_NUM_COLUMNS=4;
const	int	I_MIN_PLANES_FOR_SKIP=5;
const	int	I_NUM_SAMPLES_SKIP=0;
const	int	I_NUM_SAMPLES_MAX=500;
const	short	PMASK=geb_BADPIX;

const	char	NLNAME[]="NONE";
const	char	SECTION[]="RADHIT";

const	int	NUMINFILES=4;

#define	FITSinfoName(InFileNum)	G_FITSIN##InFileNum
#define	KWSinName(InFileNum)	KEYWORDSIN##InFileNum

#define	FITSINNAMES	{"infile1.fits", "NONE", "NONE", "NONE"}

#define	KEYWORDSIN {	{}, \
			{}, \
			{}, \
			{}}

#define	NLNAMESIN	{"FITS_In_Charge_Ramp", "FITS_In_dmask", "FITS_In_pmask", "FITS_In_Readnoise"}

const	int	NUMOUTFILES=5;

#define	FITSOUTNAMES {"NONE", "NONE", "NONE", "NONE", "outfile5.fits"}

#define	NLNAMESOUT	{"FITS_Out_Radhit_Probs", "FITS_Out_Radhit_Samples", "FITS_Out_Radhit_Mags", "FITS_Out_Slope", "FITS_Out_dmask"}

const	char	INTROKEYW[]="FINDS PROBABILITY OF THE OCCURRENCE OF A RADHIT.";

const	int	MAXNUMFRAMES=500;
const	int	MAXSTOREDLEN=84;
