/*********************************************** 
radhit_exec.cpp

Purpose

This module takes as input two FITS files. It
performs the specified binary operation pixel-
by-pixel. Denominator or subtracted file is
the second input file.


Overview
Calls the initialization functions and main loop of program. Calls
output function.


Definitions of Variables 
See radhit.h

***********************************************/
#include <iostream>
#include <string.h>
#include <time.h>

#include "more_includes.h"

const	char*	CPP_Outfilenames[MAX_FILENAME_LENGTH] = FITSOUTNAMES;
const	char*	CPP_NamelistNamesOut[MAX_FILENAME_LENGTH] = NLNAMESOUT;
const	char*	CPP_Infilenames[MAX_FILENAME_LENGTH] = FITSINNAMES;
const	char*	CPPP_KeywordsIn[][FLEN_KEYWORD] = KEYWORDSIN;
const	char*	CPP_NamelistNamesIn[MAX_FILENAME_LENGTH] = NLNAMESIN;

const	char*	CPP_Usage[MAX_FILENAME_LENGTH] = USAGE;

int main(int argc, char **argv)
{	
	int	i, I_NumInfiles;	  
	clock();
	
//	if(G_Stat.I_Verbose)
	printf("%s\tVersion %s\n", MODULENAME, VERSION);
	
	M_CmdParse	G_Cmd(argc, argv);
	G_Cmd.parse_args();
	
	M_Status	G_Stat(&G_Cmd);

	M_Namelist  G_Name(NLNAME, SECTION);
	G_Name.get_values(&G_Cmd, &G_Stat);
	G_Name.parse_namelist(&G_Stat);
	
	M_Logfile	G_Log(MODULENAME, VERSION, LOGFILENAME1, &G_Cmd, &G_Name);

	M_FITSinfo	GP_Infiles[NUMINFILES];
	M_ReadWrite	G_RW(NUMINFILES, NUMOUTFILES);
	G_RW.read_inputs(&G_Cmd, &G_Name, GP_Infiles, &G_Stat);	

	M_Constants	G_Const;
	G_Const.get_values(&G_Cmd, &G_Name, (void*) (GP_Infiles), &G_Stat);

	M_Initialize	GP_Pseudo[MAXNUMFRAMES];
////	for(i=4;i<=MAXNUMFRAMES;i++) GP_Pseudo[i-1].allocate(i, &G_Stat);
////		printf("%s\tVersion %s\n", MODULENAME, VERSION);
//////	for(i=4;i<=40;i++){ GP_Pseudo[i-1].initialize(i, &G_Const, &G_Stat);
//////		printf("%2d\t%s\tVersion %s\n", i, MODULENAME, VERSION); }
//	GP_Pseudo[MAXNUMFRAMES-1].allocate(MAXNUMFRAMES, &G_Stat);
//	for(i=4;i<=MAXNUMFRAMES;i++) GP_Pseudo[i-1].DP_Det_G = GP_Pseudo[MAXNUMFRAMES-1].DP_Det_G ;
//	for(i=4;i<=MAXNUMFRAMES;i++) GP_Pseudo[i-1].FP_Mat_G = GP_Pseudo[MAXNUMFRAMES-1].FP_Mat_G ;
		
	/* Will call initialize in compute_results as needed */	
	M_Computation	G_Comp;	
	G_Comp.compute_results(GP_Infiles, &G_Const, GP_Pseudo, &G_Stat);	

	M_FITSinfo	GP_Outfiles[NUMOUTFILES];
	G_RW.write_outputs(&G_Cmd, &G_Log, &G_Name, GP_Infiles, (void*) &G_Const, (void*) &GP_Pseudo[0], (void*) &G_Comp, GP_Outfiles, &G_Stat);

	G_Log.log_writer(&G_Cmd, &G_Name, &G_Stat);

	/* Return specified value if any error occurred. */
	if(G_Stat.I_status!=0) G_Stat.I_status = MODULE_FAILED;
	/* Indicate in return value if NaNs were produced in computation */
	else if(G_Stat.I_NaNFlag) G_Stat.I_status = NOT_A_NUMBER;

	
  return(G_Stat.I_status);
  
}

