/*********************************************** 
radhit_fitsinfo.cpp

Purpose

Set the output fitsinfo parameters specific to the implementation.

Overview
Define the desired output keywords and data.

Definitions of Variables

External:
See radhit.h

Internal:


***********************************************/

//#pragma warning(disable:4786)

#include <iostream>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#include "more_includes.h"


/*********************************************** 
load_extra_keywords()

Purpose

Write any additional keyword values beyond the standard
keywords to an array for later output.

Overview
Make a call to load_keyword for each output keyword. 

Definitions of Variables

External:
See radhit_defs.h and radhit_exec.c

Internal:
I_fits_return_status = Return value of fits function.
CP_Keyname = Name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/

int M_FITSinfo::load_extra_keywords(int I_index,
			M_Logfile	*GP_Log,
			M_Namelist	*GP_Name,
			M_FITSinfo	*GP_Infiles,
			void		*GP_ConstIn,
			void		*GP_InitIn,
			void		*GP_CompIn,
			M_FITSinfo	*GP_Outfiles,
			M_Status	*GP_Stat)
{
M_Constants* GP_Const = (M_Constants*)  GP_ConstIn;
M_Initialize* GP_Init = (M_Initialize*)  GP_InitIn;
M_Computation* GP_Comp = (M_Computation*) GP_CompIn;

if(GP_Stat->I_status) return(GP_Stat->I_status);

//printf("load_extra_keywords: I_index %d\n", I_index);

load_keyword(INTROKEYW, GP_Stat);
load_keyword("LOGFILE", "LOG FILENAME =", GP_Stat);
load_keyword(GP_Log->CP_Filename, GP_Stat);
load_keyword("NAMELIST", "NAMELIST NAME =", GP_Stat);
load_keyword(GP_Name->CP_Filename, GP_Stat);
load_keyword("INFILE1", "FITSINNAME1 =", GP_Stat);
load_keyword(GP_Infiles[0].CP_Filename, GP_Stat);
load_keyword("OUTFILE1", "FITSOUTNAME1 =", GP_Stat);
load_keyword(GP_Outfiles[0].CP_Filename, GP_Stat);
load_keyword("OUTFILE1", "FITSOUTNAME2 =", GP_Stat);
load_keyword(GP_Outfiles[1].CP_Filename, GP_Stat);
load_keyword("OUTFILE1", "FITSOUTNAME3 =", GP_Stat);
load_keyword(GP_Outfiles[2].CP_Filename, GP_Stat);
load_keyword("OUTFILE1", "FITSOUTNAME4 =", GP_Stat);
load_keyword(GP_Outfiles[3].CP_Filename, GP_Stat);
load_keyword("RHNPIXEL", GP_Comp->I_HitPixels, "NUMBER OF PIXELS WITH RADHITS", GP_Stat);
load_keyword("RHNHITS", GP_Comp->I_HitCount, "TOTAL NUMBER OF RADHITS", GP_Stat);
 printf("Radhit summary: RHNPIXEL=%d, RHNHITS=%d\n", GP_Comp->I_HitPixels,  GP_Comp->I_HitPixels);

//printf("load_extra_keywords: I_index %d\n", I_index);

if(I_index==0)	 
load_keyword("PROBABILITY OF RADIATION HIT",GP_Stat);
if(I_index==1)	 
load_keyword("SAMPLE NUMBER WITH MAXIMUM LIKELYHOOD OF HAVING A RADHIT",GP_Stat);
if(I_index==2)	 
load_keyword("ESITMATED MAGNITUDE OF RADIATION HIT",GP_Stat);
if(I_index==3)	 
load_keyword("ESTIMATED SLOPE PRIOR TO RADIATION HIT",GP_Stat);


return(GP_Stat->I_status);
}



/*********************************************** 
set_output_data()

Purpose

Assign what data shall be written to what output files.

Overview
Set the data pointer of the M_FITSinfo class to the 
desired data pointer of the M_Computation class.

Definitions of Variables

External:
See radhit_defs.h and radhit_exec.c

Internal:
I_fits_return_status = Return value of fits function.
CP_Keyname = Name of Key to be written.
CP_Comment = String value to be written in comment field of header.

***********************************************/
int M_FITSinfo::set_output_data(int 		I_index,
				M_FITSinfo	*GP_Infiles,
				void		*GP_ConstIn,
				void		*GP_InitIn,
				void		*GP_CompIn,
				M_FITSinfo	*GP_Outfiles,
				M_Status	*GP_Stat)
{
M_Constants* GP_Const = (M_Constants*)  GP_ConstIn;
M_Initialize* GP_Init = (M_Initialize*)  GP_InitIn;
M_Computation* GP_Comp = (M_Computation*) GP_CompIn;

if(GP_Stat->I_status) return(GP_Stat->I_status);



I_Length_X = GP_Comp->I_Length_X;
I_Length_Y = GP_Comp->I_Length_Y;
I_Num_Frames = GP_Comp->I_Num_Frames;


#if 0
if(I_index==0)
data = GP_Comp->data1;
if(I_index==1)
data = GP_Comp->data2;
if(I_index==2)
data = GP_Comp->data3;
if(I_index==3)
data = GP_Comp->data4;
if(I_index==4)	{

int	I_ArraySize;

//printf("set_output_data: I_Bitpix %d\tdata 0x%x\n", SHORT_IMG,  GP_Infiles[1].data);
printf("set_output_data: CP_Filename %s\n", CP_Filename);
I_Bitpix = SHORT_IMG;
I_ArraySize = I_Num_Frames*I_Length_X*I_Length_Y;
data = (double *) calloc(I_ArraySize, sizeof(double));
memcpy( data, GP_Infiles[1].data, I_ArraySize*sizeof(double));
//data = GP_Infiles[1].data;
printf("set_output_data: I_Bitpix %d\tdata 0x%x\n", I_Bitpix, data);
}


#else


/* Copy the header from the first input file */
//sprintf(CP_HdrFile, "%s", GP_Infiles[0].CP_Filename);

if(I_index==0)	{
I_Num_Frames = GP_Comp->I_Num_Frames;
data = GP_Comp->data1;
}
if(I_index==1)	{
I_Num_Frames = 1;
I_Bitpix = SHORT_IMG;
data = GP_Comp->data2;
}
if(I_index==2)	{
I_Num_Frames = 1;
data = GP_Comp->data3;
}
if(I_index==3)	{
I_Num_Frames = GP_Comp->I_Num_Frames;
data = GP_Comp->data4;
}
if(I_index==4)	{
sprintf(CP_HdrFile, "%s", GP_Infiles[1].CP_Filename);
I_Num_Frames = GP_Comp->I_Num_Frames;
I_Bitpix = SHORT_IMG;
data = GP_Comp->data5;
}
#endif


  return(GP_Stat->I_status);
}




