/*********************************************** 
saturating_radhit.cpp

Purpose

Detect radhit which saturates all samples after it.

Overview
Include saturated samples in the computation.

Definitions of Variables

External:
See radhit.h

Internal:
D_Charge_StepHere = Charge step at posited radhit location.
D_Charge_StepNominal = Charge step before posited radhit.

***********************************************/

#include <iostream>
#include <string.h>
#include <math.h>

#include "more_includes.h"
#include "nanvalue.h"


int bits_printf(short x);
short bits_get(short x, int i);
			
int M_Computation::saturating_radhit(	int		i,
					int		j,
					int		I_IndexOut,
					M_Constants	*GP_Const,
					M_Initialize	*GP_Pseudo,
					M_Status	*GP_Stat)
{
//int	k, I_IndexIn, I_IndexOut;
int	k, I_IndexIn;
double	D_Charge_StepNominalSave;

//if(GP_Stat->I_Verbose)
//printf("saturating_radhit\n");


/* This is the beginning of current segment */
k=I_SegmentStart;

/* Mark that radhit detection was done on this segment */
SP_MaskVector[I_SegmentStart] |= ged_DIDRHDET;

/* Possible radhit will be checked for at the end of the previous segment */
I_Max_LLikely_Position = I_SegmentStop - I_SegmentStart;

/* Find where adjoining saturated samples end */
for(k=I_SegmentStop;k<I_Num_Frames;k++)
if(!(SP_MaskVector[k]&ged_SATURATEHI)) break;
I_SegmentStop = k;

//if(GP_Stat->I_Verbose)
//printf(" %d %d %d %d %d\t", i, j, I_SegmentStart, I_Max_LLikely_Position, I_SegmentStop);

/* Return if there were none */
if(I_SegmentStop==I_SegmentStart+I_Max_LLikely_Position) return(GP_Stat->I_status);

/* Could not find both segment endpoints */
if(I_SegmentStop>I_Num_Frames) return(GP_Stat->I_status);

/* Compute index in output data cubes: first sample of saturation */
I_IndexOut = (I_SegmentStart+I_Max_LLikely_Position)*I_Length_X*I_Length_Y+j*I_Length_X+i;

//if(GP_Stat->I_Verbose)
//printf(" %d %d %d %d\t", i, j, I_SegmentStart, I_SegmentStop);
//if(GP_Stat->I_Verbose)
//bits_printf(SP_MaskVector[I_SegmentStart]);
//if(GP_Stat->I_Verbose)
//printf("\n");

/* Skip this segment if there aren't enough samples. */
I_SegmentLen = I_SegmentStop - I_SegmentStart;
if(I_SegmentLen < 4) return(GP_Stat->I_status);


/* Copy the charge ramp */
for(k=0;k<I_SegmentLen;k++)	{
I_IndexIn = (I_SegmentStart+k)*I_Length_X*I_Length_Y+j*I_Length_X+i;
DP_Data_Vec_Charge[k] = GP_Const->D_Gain*(data_in[I_IndexIn]);
}
I_NumSamples = I_SegmentLen;


/* Construct the pseudomatrix and determinants if not done already */
if(GP_Pseudo[I_NumSamples-1].FP_Mat_G == NULL)
GP_Pseudo[I_NumSamples-1].initialize(I_NumSamples, GP_Const, GP_Stat);

if(GP_Stat->I_Debug)
printf("saturating_radhit: initialize done.\tI_NumSamples = %d\n", I_NumSamples);

/* Estimate the charge added by the posited radiation hit. */
/* Use the previous D_Meas_Flux; may not be able to compute it now */
D_Charge_StepNominalSave = D_Meas_Flux;
charge_addition( &GP_Pseudo[I_NumSamples-1], GP_Stat);
//if(GP_Stat->I_Verbose)
//printf("saturating_radhit: D_Meas_Flux = %f at %d\n", D_Meas_Flux, I_Max_LLikely_Position);

D_Meas_Flux = D_Charge_StepNominalSave;
//if(GP_Stat->I_Verbose)
//printf("saturating_radhit: D_Meas_Flux = %f at %d\n", D_Meas_Flux, I_Max_LLikely_Position);

D_Meas_Charge_Add = D_Charge_StepHere - D_Charge_StepNominalSave;

if(D_Meas_Charge_Add < 0) D_Meas_Charge_Add = 0;
//if(GP_Stat->I_Verbose)
//printf("saturating_radhit: D_Meas_Charge_Add = %f at %d\n", D_Meas_Charge_Add, I_Max_LLikely_Position);


/* Find the probability that a radiation hit occurred. */
prob_radhit( GP_Const, GP_Pseudo, GP_Stat);

if(GP_Stat->I_Verbose)
if(D_Prob_Radhit > GP_Const->D_Thresh )	{
printf("%02d %02d ", i, j);
printf("0x%x ", GP_Stat->I_status);
printf("%02d %02d ", I_SegmentStart, I_SegmentStop);
printf("saturating_radhit: D_Prob_Radhit = %f\n", D_Prob_Radhit);
printf("saturating_radhit: D_Meas_Flux = %f at %d\n", D_Meas_Flux, I_Max_LLikely_Position);
printf("saturating_radhit: D_Meas_Charge_Add = %f at %d\n", D_Meas_Charge_Add, I_Max_LLikely_Position);
}

/* Check that result is a valid number */
if(	iznand(D_Prob_Radhit)		||
	iznand(D_Meas_Charge_Add)	||
	iznand(D_Meas_Flux)	)
GP_Stat->I_status = COMPUTE_RES|NOT_A_NUMBER;

/* Write any error code into the array as an NaN instead of the result */
if(GP_Stat->I_status)	{
if(GP_Stat->I_SuperVerbose)
printf("saturating_radhit: NaN at %d %d\n", i, j);

/* Changed - TYB 6/1/09 */

data1[I_IndexOut] = *(double *)&mynand;
data4[I_IndexOut] = *(double *)&mynand;

//data1[I_IndexOut] = NANVALUE;
////data2[I_IndexOut] = I_SegmentStart+I_Max_LLikely_Position;
////data3[I_IndexOut] = NANVALUE;
//data4[I_IndexOut] = NANVALUE;

//if(GP_Stat->I_Debug)
//printf("saturating_radhit: write_nan done: 0x%x.\n", GP_Stat->I_status);
return(GP_Stat->I_status);
}

/* Write the results to the corresponding arrays.*/
data1[I_IndexOut] = D_Prob_Radhit;
data4[I_IndexOut] = D_Meas_Flux;


if(0 && GP_Stat->I_Verbose)	{
printf("%02d %02d ", i, j);
printf("0x%x ", GP_Stat->I_status);
printf("%02d %02d ", I_SegmentStart, I_SegmentStop);
//printf("%f ", data1[I_IndexOut]);
bits_printf(SP_MaskVector[I_SegmentStart+I_Max_LLikely_Position]);
printf("\n");
}

/* Mark that radhit detection was done on this segment */
SP_MaskVector[I_SegmentStart] |= ged_DIDRHDET;
/* Don't need to recheck the beginning of this segment for radhits */

/* Free the stored matrices for longer lengths
   otherwise could run out of memory */
if (I_NumSamples > GP_Const->I_MaximumStoredMatrixLen)  {
free(GP_Pseudo[I_NumSamples-1].FP_Mat_G);
GP_Pseudo[I_NumSamples-1].FP_Mat_G=NULL;
free(GP_Pseudo[I_NumSamples-1].DP_Det_G);
GP_Pseudo[I_NumSamples-1].DP_Det_G=NULL;
}


return 0;

}
