#define PROGRAM  "ridge"
#define VERSION  "1.15"

/**
 This module computes the extraction coordinates along a ridge position.

NOTE:  uses program downlink/pipeline/include/tools/nl_and_params.c
**/


/**UPDATES
061117 1.15 Commented out (buggy) baseline_percentile logic (Bob S15 patch)
060929 1.14 Increased MAX_CUTS
051121 1.13 Updated VERSION (CVS style was causing errors in nl_and_params)
050809 S13.0 Fixed bug: not getting FOVID from header for lo-res (Bob)
050111 1.9 fixed to use tblio instead of tableio (IK)
030206 1.7 FIX#1:  CR3961: Allow -c to be specified with -p (gives -c plus headers)  
           FIX#2:  Change ridge to read FOVID (and NOT read order) to get DFRAC. (CW)
030205 1.6 CR3958: Changed definition of percentage to 100-previous_percentage (CW)
030130 1.5 Updated as requested in "Ridge Req's", Brauher et al., 10/14/03 (CW)
020926 1.4 Added namelist facility (CW)
020926 1.5 Named TERMINATE_* codes (CW)
010201 1.2a deleted extra call to tblwrit
000825 1.2 added profile reader
000824 1.2 made table reader independent of column order
000823 1.2 added parameter printout and HISTORY cards
000822 1.2 updated to new paramIo
000307 1.2 changed columns added to output table to be of length 10 (was 9)
000307 1.2 changed parameters to single percentage value (was separate value per band)
000303 1.2 changed input parameters to per percentage (0-100) vs. fraction (0.0 - 1.0)
000301 1.2 changed output format of added columns from f10.5 to f10.4
**/

/* Termination codes as specified in:
**     SIRTF Downlink Software Development Guidelines  20010515
**     (file: /ssc/pipe/dlmtg/sw_dev_guidelines)
*/
#define TERMINATE_NORMAL        0
#define TERMINATE_WARNING      32
#define TERMINATE_ERROR	       64

#define MAXLINELEN            300
#define MAXFILENAMESIZE       256
#define NL_START         "&RIDGE"
#define NL_END             "&END"

#define SL 0 /* Short Low  */
#define SH 1 /* Short High */
#define LL 2 /* Long  Low  */
#define LH 3 /* Long  High */

#define MAX_REGIONS             5   /* Maximum regions allowed (numbered 0 to MAX-1) */
#define MAX_CUTS             2000   /* Maximum number of cuts allowed */

#define FOV_FULL_SLIT_SL       29   /* Don't find peaks for this FOV */
#define FOV_FULL_SLIT_LL       41   /* Don't find peaks for this FOV */

#define DEFAULT_DFRAC         0.5   /* A default fraction */

#include <stdio.h> 
#include <stdlib.h>
#include <math.h>
#include <tbl.h> 

/* Table read/write */
int iu;
int ou;
int tstat, row, nrows;
struct tbl_filinfo *ifinfo, ofinfo;


/*===========================================================================
** Find the MEDIAN of an array in O(n) time on *average*
** 
**   FROM http://www.ics.uci.edu/~eppstein/161/960125.html 
**   PSEUDOCODE TO SELECT THE kTH SMALLEST ELEMENT OF ARRAY L:
** 
**            quickselect(L,k)
**            {
**               pick x in L
**               partition L into L1<x, L2=x, L3>x
**               if (k <= length(L1))
**                   return quickselect(L1,k)
**               else if (k > length(L1)+length(L2))
**                   return quickselect(L3,k-length(L1)-length(L2))
**               else return x
**            }
**
**
**   RANDOM FUNCTION drand48() and srand48() NOTES:
**
**      Function drand48() returns non-negative double-precision
**      floating-point values uniformly distributed over the
**      interval [0.0, 1.0).
**
**      Function srand48() is an initialization entry  point,  which
**      should be invoked before drand48() is called.
**      (Although it is not recommended practice, constant default
**      initializer values will be supplied automatically  if drand48()
**      is called without a prior call to an initialization entry point.)
**===========================================================================
*/
float quickselect( float *L, int len, int k )
{
int i,len1,len2,len3;
float x;
float *L1, *L2, *L3;  /* Pointers to arrays */

   /********************************************************************
   ** Create a new arrays to use for storage of medians 
   ** (this *could* be optimized to be more storage-efficient, but I
   ** didn't bother)
   */
   /* if (cs.debug) printf("Allocate memory for local median values storage\n"); */
   L1 = (float *) malloc( (unsigned) ( len * sizeof(float) ) );
   L2 = (float *) malloc( (unsigned) ( len * sizeof(float) ) );
   L3 = (float *) malloc( (unsigned) ( len * sizeof(float) ) );
   if ( (L1 == NULL) || (L2 == NULL) || (L3 == NULL) ) {
      printf("ERROR: %s: Can't allocate required memory for medians in quickselect()\n","ridge.c");
      exit( TERMINATE_ERROR );
   }
   
   /* if (cs.debug) printf("*** Find median of list of length %1d and median is element %1d\n",len,k); */
   /******************************************************************
   ** PSEUDOCODE: pick x in L 
   */
   i = (int)( len * (float)drand48() );  /* A random integer in [0..len-1] */
   x = L[i];
   /* if (cs.debug) printf("Chose L[%1d] = x = %g\n", i, x); */

   /******************************************************************
   ** PSEUDOCODE: partition L into L1<x, L2=x, L3>x 
   */
   len1=0; len2=0; len3=0;
   for( i=0; i<len; i++ ) {
      if      ( L[i] < x )    L1[len1++] = L[i];
      else if ( L[i] > x )    L3[len3++] = L[i];
      else   /* L[i] == x */  L2[len2++] = L[i];
   }
   
   /******************************************************************
   ** PSEUDOCODE: 
   ** if (k <= length(L1))
   **    return quickselect(L1,k)
   ** else if (k > length(L1)+length(L2))
   **    return quickselect(L3,k-length(L1)-length(L2))
   ** else return x

   if (cs.debug) {
      if      (k <= len1      ) printf("RECURSE: L1, <len1=%02d> len2=%02d  len3=%02d  k=%02d  x=%g\n",len1,len2,len3,k,x);
      else if (k >  len1+len2 ) printf("RECURSE: L3,  len1=%02d  len2=%02d <len3=%02d> k=%02d  x=%g\n",len1,len2,len3,k-len1-len2,x);
      else                      printf("RETURN:  x=%g\n",x);
   }
   */
   
   if      (k <= len1      ) { free(L2); free(L3);
                               return quickselect(L1, len1, k );
                             }
   else if (k >  len1+len2 ) { free(L1); free(L2);
                               return quickselect(L3, len3, k-len1-len2 );
                             }
   else                      { free(L1); free(L2); free(L3);
                               return x;
                             }

}


/*===========================================================================
** Find the MEDIAN of pixel values within an array.
** MEDIAN is found only for those pixels for which the mask==1.
**
** requires routine quickselect()
*/
float find_median( float *array, int start, int num_data  )
{
int	i;
float	*data_for_median;
float	median;

   /***********************************************************************
   /* Initialize the random number generator used in quickselect()
   srand(999);
   
   /***********************************************************************
   ** Create & populate a new linear (1-D) array to store values where 
   ** there were mask values marking "inside pixels".
   */
   data_for_median = (float *) malloc( (unsigned) ( num_data * sizeof(float) ) );
   if (data_for_median == NULL) {
      printf("ERROR: %s: Can't allocate required memory for spectral values in quickselect()\n","ridge.c");
      exit( TERMINATE_ERROR );
   }
   for( i=0; i<num_data; i++ ) {
      data_for_median[i]=array[start+i];
   }

   /***********************************************************************
   ** Find the median of these data_for_median
   */
   median = quickselect( data_for_median, num_data, (num_data+1)/2 );

   /***********************************************************************
   ** Clean up & return median
   */
   free(data_for_median);
   return median;

}



/*============== tio_del SUBROUTINE ========================================================*/
void tio_del(char* col_name)
/* delete named column (no action if can't do it) */
{
  int ind_tmp;
  ind_tmp = tbl_column(iu, col_name);
  if (ind_tmp >= 0) tbl_delcol(ou, ind_tmp);
}


/*============== tio_getind SUBROUTINE ========================================================*/
int tio_getind(char* col_name, int *ind)
/* get index of the named column */
/* return 0=OK  non-zero = trouble */
{
  *ind = tbl_column(iu, col_name);
  if (*ind < 0) {
    printf("ERROR: bad in table file: missing column: %s\n", col_name);
    return -1;
  }
  return 0;
}


/*=============== ridge =================================================================*/
ridge(argc, argv)
int argc;
char *argv[];
{
  int i, j;

  /* data for wavsamp and ridge tables */
  int    nparms;
  int    ind_x_ridge_right, ind_y_ridge_right;
  int    ind_x_ridge_left, ind_y_ridge_left;
  int    ind_order, ind_wavelength, ind_angle;
  int    ind_x0, ind_y0;
  int    ind_x1, ind_y1;
  int    ind_x2, ind_y2;
  int    ind_x3, ind_y3;
  double  x0, y0, x1, y1, x2, y2, x3, y3;
  float  *x_ridge_right = 0;
  float  *y_ridge_right = 0; 
  float  *x_ridge_left = 0;
  float  *y_ridge_left = 0;

  /* data for reading channelID from WAVSAMP file */
  int    ch_id, ch_index;
  char   ch_type[MAXLINELEN], ch_unit[MAXLINELEN], ch_val[MAXLINELEN];
  char   fov_string[MAXLINELEN];
  char   ch_string[MAXLINELEN];
  int    do_max_of_maxes; /* set non-zero to find max of maxes (vs. search) */
  
  /* data for reading FOV table */
  int    ind_fov, ind_dfrac;
  int    fov, order;
  double  dfrac;
  int    fov_id, fov_index;
  char   fov_type[MAXLINELEN], fov_unit[MAXLINELEN], fov_val[MAXLINELEN];
  int    the_order, order_index;
  char   order_type[MAXLINELEN], order_unit[MAXLINELEN], order_val[MAXLINELEN];
  
  /* For copying keywords from profile tbl to ridge tbl */
  FILE   *profile_file;
  FILE   *ridge_file;
  FILE   *temp_ridge_file;
  char   r_keyword_line[MAXLINELEN];
  char   p_keyword_line[MAXLINELEN];
  char   temp_ridge[] = "TEMPORARY_RIDGEFILE";
  char   cmd[MAXLINELEN];

  /* Misc vars and flags */
  int status;
  char** hist_cards;
  char   maxpct_string[120];
  char   outstring[80];
  int    have_profile_file;    /* Set if profile file was specified with -p */
  int    have_fov_file;        /* Set if fov     file was specified with -f */
  int    found_channelID;
  int    found_fov_id;
  /* int found_order; */       /* No longer needed */
  int    found_dfrac;
  int    find_peaks;           /* Should peaks be calculated? */
  int    found_max_signal;     /* Max signal has been calculated by finding peaks */
  int    found_source;         /* i.e. "Source is detected" i.e. m_sigma criterion satisfied */
  float  source_threshold;     /* How many sigmas the detected source was above the threshold */
  float  source_mean;          /* Mean used for source_threshold */
  float  source_sigma;         /* Sigma used for source_threshold */
  int    median_filter_used;   /* 0 if not used, set to higher to indicate number of points if used */
  
  /* HI-RES: Data for reading profile table */
  int ind_dn, ind_pct, ind_region;
  double   dn, pct;
  int    region;
  int    region_used[MAX_REGIONS];
  float  max_dn[MAX_REGIONS], max_dn_at_pct[MAX_REGIONS];
  float  max_yf[MAX_REGIONS], max_yf_at_pct[MAX_REGIONS];
  float  max_yh[MAX_REGIONS];
  float  overall_max_dn, overall_max_dn_at_pct;
  int    overall_max_in_region;
  
  /* LO-RES: Profile data variables;  x=>pct y=>dn */
  int    num_regions;
  int    num_cuts;
  float  x[MAX_REGIONS][MAX_CUTS];  /* original datum */
  float  y[MAX_REGIONS][MAX_CUTS];  /* original datum */
  int    n[MAX_REGIONS];      /* Number of data in region   */
  float  x_sum[MAX_REGIONS];  /* Sum of data in region      */
  float  y_sum[MAX_REGIONS];  /* Sum of data in region      */
  float  x2sum[MAX_REGIONS];  /* Sum of squares in region   */
  float  xysum[MAX_REGIONS];  /* Sum of x*y terms in region */
  float  m,b,denom,ysum,y2sum,expr;
  float  sigma[MAX_REGIONS];
  float  mean[MAX_REGIONS];
  int    nn,npts,nleft,nright;
  float  yh[MAX_REGIONS][MAX_CUTS];  /* "horizontal" data after linear fit removed */
  float  yf[MAX_REGIONS][MAX_CUTS];  /* median filtered data */
  double shift_to_baseline;

  /* parameters */
  char   in_wavsamp[MAXFILENAMESIZE];
  char   in_namelist[MAXFILENAMESIZE];
  char   in_profile[MAXFILENAMESIZE];
  char   in_fovfile[MAXFILENAMESIZE];
  float  percentage;
  float  percent_frac; /* percentage/100 */
  float  g_pct_rg, g_pct_lo, g_pct_hi;   /* Range limits in percent (not fractional) units */
  float  m_sigma;
  int    median_pts;  /* Number of points to be used in a median filter */
  char   out_ridge[MAXFILENAMESIZE];
  /* int    baseline_percentile; */
  int    verbose;

  void param_print( void );
  
  
  /************************************************************************************************************
  ** PROCESS THE PARAMS 
  */
  /** PARAMS **/
  param(in_wavsamp,   "-i WAVSAMP_FILE name of Wavsamp file",       "infile wavsamp.tbl");
  param(in_profile,   "-p PROFILE_FILE name of profile table file", "infile profile.tbl NONE no profile table");
  param(in_fovfile,   "-f PSF_FOV_FILE name of FOV table file",     "infile fov.tbl NONE no FOV table"); /*S7*/
  param(&percentage,  "-c CUT_POSITION percentage position of cut (0=left 100=right)", 
                                                      "float:>=-50.0:<=150.0 percent -1.0 use max in PROFILE_FILE");
  param(&g_pct_rg,    "-g PCT_RANGE plus-or-minus pct in cross-dispersion dir. to be analyzed, centered on expected position (LO-RES only)", 
                                                                    "float:>0.0:<=100.0 percent 100.0");
  param(&g_pct_lo,    "-glo PCT_LO explicitly specify the low-end  cutoff of g (LO-RES only)", 
                                                                    "float:>0.0:<=100.0 percent -1.0 use g value");
  param(&g_pct_hi,    "-ghi PCT_HI explicitly specify the high-end cutoff of g (LO-RES only) (use -glo & -ghi instead of -g if desired)", 
                                                                    "float:>0.0:<=100.0 percent -1.0 use g value");
  param(&m_sigma,     "-s NUM_SIGMA Number of sigma to require peak to be above mean (LO-RES only)",
                                                                    "float:>0.0:<=10.0 - 5.0");                                                                    
  param(&median_pts,  "-m MEDIAN_PTS Number points to use for median filtering (LO-RES only), must be ODD",  
                                                                    "int:>=1:<=11 - 5" );                                                                    
  param(out_ridge,    "-o OUTFILE name of output ridge file",       "outfile ridge.tbl");
  /* param(&baseline_percentile, "-v BASEL_PCTILE adjust baseline so this percentile is at 0 (50=median)",       "int - -1"); */
  param(&verbose,     "-v VERBOSE level of printed messages",       "int - 1");
  param(in_namelist,  "-n NAMELIST_FILE name of Namelist file",     "infile - namelist.nl");
  /**ENDPARAMS **/

  status = nl_process(argc, argv, PROGRAM, VERSION, NL_START, NL_END );
  if (status != 0) return status;

  status = param_begin(argc, argv, PROGRAM, VERSION);
  if (status != 0) return status;

  param_print();   /* Print the parameter settings to stdout. */
  
  have_profile_file  = strcmp(in_profile, "NONE");
  have_fov_file      = strcmp(in_fovfile, "NONE");

  found_max_signal   = 0;
  found_source       = 0;
  median_filter_used = 0;
  maxpct_string[0]   = '\0';

  /************************************************************************************************************
  ** check that exactly one of -p and -c is specified; set whether finding peaks is the goal (vs. simple cuts)
  **
  **      Have -p only  ==> find peaks AND propagate headers from profile file
  **      Have -c only  ==> don't find peaks, take simple cut percentage
  **      Have -p & -c  ==> don't find peaks, take simple cut percentage AND propagate headers from profile file
  */
  i=0;
  if (have_profile_file) i++;
  if (percentage != -1.0) i++;
  if (i==0) {
    printf("ERROR: Must supply either -p or -c parameter\n");
    return TERMINATE_ERROR;
  }
  if (i==2) {
    if (verbose>0) {
       printf("NOTE: Both -p and -c were specified, so \"-c %g\" will override profile data,\n",percentage);
       printf("      (no peak will be found).  In addition, all the file headers will be\n");
       printf("      propagated from the profile file \"%s\".\n",in_profile);
    }
  }
  find_peaks = (percentage == -1.0);  /* Want to find peaks if no -c cut percentage was specified */
  if (verbose>1) printf("--> Find Peaks: %s\n", (find_peaks?"YES":"NO") );
  if (verbose>2) printf("----> Percentage=%g  find_peaks=%s\n", percentage, (find_peaks?"YES":"NO") );

  /************************************************************************************************************
  ** Set range parameters g_pct_lo and g_pct_hi; check that at most one of -g and (-glo and -ghi) is specified 
  */
  i=0;
  if (g_pct_lo==-1) i++;
  if (g_pct_hi==-1) i++;
  if (i==0) {              /* BOTH -glo and -ghi specified */
     if (g_pct_lo>g_pct_hi) {
        printf("ERROR: -glo > -ghi  not allowed.\n");
        return TERMINATE_ERROR;        
     }
     if (g_pct_rg!=-1) {
        printf("WARNING: -glo and -ghi HAVE OVERRIDEN the -g specificaton.  -g value IGNORED.\n");
        printf("USING:   PCT_LO = %g  and  PCT_HI = %g\n", g_pct_lo, g_pct_hi );
        g_pct_rg =  -1;     /* Have both g_pct_lo and g_pct_hi specified from command line */
     }
  }
  if (i==1) {
     printf("WARNING: -glo and -ghi must be specified as a pair.  Range restriction ignored.\n");
     g_pct_lo =   0;
     g_pct_hi = 100;
     g_pct_rg =  -1;       
  } 
  /* if (i==2) params are already set properly:  g_pct_lo = g_pct_hi = -1  and  g_pct_rg is from command line */
  if (verbose>2) printf("----> g_pct_rg=%g     g_pct_lo=%g  g_pct_hi=%g\n", g_pct_rg,g_pct_lo,g_pct_hi);

  /************************************************************************************************************
  ** Read the channelID from the wavsamp file 
  */
  if (verbose>1) printf("--> Reading channelID from the WAVSAMP file =\"%s\"\n", in_wavsamp);
  iu = tbl_open(in_wavsamp, &ifinfo);
  tstat=1;
  tstat   =  tbl_fileinfo(iu, &ifinfo);
  if (tstat != 0) {
    printf("Cannot open table: [%s]\n",in_wavsamp); 
    exit(64);
   }
   nparms = ifinfo->ncols;
  if (verbose>2) printf("----> nparms=%1d\n", nparms);
  found_channelID = tbl_keyval(iu, "channelID", ch_string, &ch_index);
  if (!found_channelID) {
    ch_id = atoi(ch_string);
    if (verbose>1) printf("--> found_channelID=%s   ch_string=\"%s\"    ch_id=%1d\n", 
                    (found_channelID?"NO":"YES"), ch_string,          ch_id          );
  }
  tbl_close(iu); /* close table file */
  if (verbose>1) printf("--> Reading channelID is complete.\n");
  do_max_of_maxes = ( (ch_id==SH) || (ch_id==LH) );
  if (verbose>1) printf("--> Processing %s data.\n", (do_max_of_maxes?"MAX-OF-MAXES":"SEARCH-ALGORITHM") );


  /************************************************************************************************************
  ** Read profile table to find fov_id=FOVID 
  **
  **   Used to also find the_order=ORDER, but apparently it's not needed since 
  **   the DFRAC is the same for all order numbers in a given FOVID.
  */
  found_fov_id = 0;
  /* found_order  = 0; */
  if (have_profile_file) {  
     if (verbose>1) printf("--> Reading params from PROFILE_FILE=\"%s\"\n", in_profile);
     iu = tbl_open(in_profile, &ifinfo);
     tstat   = tbl_fileinfo(iu, &ifinfo);
     if (tstat != 0) {
       printf("Cannot open table: [%s]\n",in_profile);
       exit(64);
     }
     nparms = ifinfo->ncols;
     if (verbose>2) printf("----> nparms=%1d\n", nparms);

     /* look up FOVID in header */
     tstat = tbl_keyval(iu, "FOVID",  fov_string, &fov_index);
     if (tstat == 0) {
       found_fov_id = 1;
       sscanf (fov_string, "%d", &fov_id); 
       printf("--> FOVID = %d\n", fov_id);
     }
     else printf("NOTE:  FOVID was not found in %s\n",in_profile);

     tbl_close(iu); /* close table file */
     if (verbose>1) printf("--> Reading params from PROFILE_FILE=\"%s\" is complete.\n", in_profile);

     /* Don't find peaks for these "Full Slit" cases -- just use DFRAC (if available) */
     if ( (fov_id==FOV_FULL_SLIT_SL) || (fov_id==FOV_FULL_SLIT_LL) ) {
        find_peaks = 0;  
        if (verbose>0) printf("NOTE:  FOVID is set to %1d, no peak finding will be done.\n",fov_id);
        if (verbose>1) printf("--> Find Peaks: NO  (changed)\n");
     }
  }
  

  /************************************************************************************************************
  ** IF NECESSARY, Read FOV lookup table (used for SL and LL) -- search for {FOVID and ORDER} to find DFRAC 
  **
  **     (references to ORDER=the_order removed)
  */
  found_dfrac = 0;

  if ( (have_fov_file) && (found_fov_id) ) {  
     if (verbose>1) printf("--> Reading DFRAC from FOV_FILE=\"%s\"\n", in_fovfile);
     iu = tbl_open(in_fovfile, &ifinfo);
     tstat   = tbl_fileinfo(iu, &ifinfo);
     if (tstat != 0) {
       printf("Cannot open table: [%s]\n",in_fovfile);
       exit(64);
     }
     nparms = ifinfo->ncols;
     if (verbose>2) printf("----> nparms=%1d\n", nparms);

     if (tio_getind("FOV",   &ind_fov    )) return TERMINATE_ERROR;
     if (tio_getind("DFRAC", &ind_dfrac  )) return TERMINATE_ERROR;

     row = 0;
     while (!found_dfrac)
     {
        tstat = tbl_colval(iu, row, ind_fov, TBL_INTEGER, &fov);
        tstat = tbl_colval(iu, row, ind_dfrac, TBL_DOUBLE, &dfrac);
        if (tstat == -4) break;
        else if (tstat  != 0) {
          printf("Table I/O Error in FOV table file %s\n", in_fovfile);
          exit(TERMINATE_ERROR);
        }
        found_dfrac = (fov==fov_id);  /* Found the right data line; so dfrac is set correctly */
        row++;
     }

     tbl_close(iu); /* close table file */
     if (verbose>1) {
        if (found_dfrac) printf("--> FOVID=%1d FOUND in %s  ==>  DFRAC=%g\n", 
                                     fov_id,   in_fovfile,       dfrac           );
        else             printf("--> NO entry for FOVID=%1d was found in %s\n",
                                                  fov_id,            in_fovfile  );
     }

  }
  else if (verbose>1) printf("--> NO FOV file to read.\n");


  /************************************************************************************************************
  ** Read and process profile table
  */
  overall_max_dn_at_pct = -1;  /* NO OVERALL MAX FOUND YET */
  if (find_peaks && have_profile_file) {  
     if (verbose>1) printf("--> Reading and processing data from PROFILE_FILE=\"%s\"\n", in_profile);
     iu = tbl_open(in_profile, &ifinfo); 
     tstat   = tbl_fileinfo(iu, &ifinfo);
     if (tstat != 0) {
       printf("Cannot open table: [%s]\n",in_profile);
       exit(64);
     }

     if (tio_getind("pct",    &ind_pct    )) return TERMINATE_ERROR;
     if (tio_getind("dn",     &ind_dn     )) return TERMINATE_ERROR;
     if (tio_getind("region", &ind_region )) return TERMINATE_ERROR;
     
     /* SET THE PCT_RANGE PARAMETERS */
     if (g_pct_lo==-1) {
        if (verbose>2) printf("----> Using g_pct_rg = %6.2f%%\n", g_pct_rg );
        if (verbose>1) {
           if (!found_dfrac) printf("--> Set DFRAC = ");
           else              printf("--> Using DFRAC = ");
        }
        if (!found_dfrac) dfrac=DEFAULT_DFRAC;  /* a default value */
        if (verbose>1) printf("%g\n",dfrac);
        g_pct_lo = 100*dfrac - g_pct_rg;
        g_pct_hi = 100*dfrac + g_pct_rg;
        if (g_pct_lo <   0) g_pct_lo =   0;
        if (g_pct_hi > 100) g_pct_hi = 100;        
     }
     if (verbose>1) {
        printf("--> Using g_pct_lo = %6.2f%%\n", g_pct_lo );
        printf("--> Using g_pct_hi = %6.2f%%\n", g_pct_hi );
     }


     /*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     ** FIND PEAKS FOR THE SIMPLE HI-RES CASE
     */
     if (do_max_of_maxes) {
        if (verbose>1) printf("--> FIND PEAKS FOR THE HI-RES CASE\n");

        for (region=0; region<MAX_REGIONS; region++) {  /* Ensure values initialized to 0. */
           region_used[region]   =  0;
           max_dn_at_pct[region] = -1;
        }

        num_regions = 0;
        row = 0;
        while (1)         /* READ IN THE DATA FROM THE PROFILE FILE */
        {
           tstat = tbl_colval(iu, row, ind_dn, TBL_DOUBLE, &dn);
           tstat = tbl_colval(iu, row, ind_pct, TBL_DOUBLE, &pct);
           tstat = tbl_colval(iu, row, ind_region, TBL_INTEGER, &region);
           if (verbose>1) 
             printf("row%d stat %d dn %f pct %f region %d num_regions %d\n",row, tstat, dn,pct,region, num_regions);
           if (tstat == -4 ) break;
           else if (tstat !=  0) {
              printf("Table I/O Error in profile table %s\n",in_profile);
              exit(TERMINATE_ERROR);
           }

           /* Check that region is in range before using it as an array index! */
           if ( (region<0) || (region>MAX_REGIONS) ) {
              printf("Error in profile table %s at line pct=%g dn=%g\n",in_profile, pct, dn);
              printf("Region %1d is out of range (0..%1d)\n",region, MAX_REGIONS);
              exit(TERMINATE_ERROR);
           }
           
           if (pct>g_pct_lo && pct<g_pct_hi) {  /* ignore values not within ( g_pct_lo < pct < g_pct_hi ) */

              if (!region_used[region]) { 
                 num_regions++;
                 max_dn[region]        = dn;
                 max_dn_at_pct[region] = pct;
                 region_used[region]   = 1;
              }
              else if (dn > max_dn[region]) {
                 max_dn[region]        = dn;
                 max_dn_at_pct[region] = pct;
              }
           }
           row++;
        }
        
        if (verbose>1) printf("--> Number of regions found is %1d\n",num_regions);
        if (num_regions==0) {
           printf("No regions in file %s fit %g < pct < %g criteron.\n",in_profile, g_pct_lo, g_pct_hi);
           
           if (found_dfrac) { 
              percentage = 100*dfrac;
              if (verbose>0) printf("No candidate maximum found; Cut percentage set to DFRAC=%g\n",percentage);
           }
           else{
              percentage = 100*DEFAULT_DFRAC;  /* a default value */
              if (verbose>0) printf("No candidate maximum found, no DFRAC; Cut percentage set to %g\n",percentage);
           }
           
        }
        else {  /* (num_regions>0) */
        
           /* initialize overall_max_dn and overall_max_dn_at_pct */
           region=0;         /* Find first used region (ensured that at least one region_used==1 since num_regions>0) */            
           while(!region_used[region]) region++;
           overall_max_dn        = max_dn[region];
           overall_max_dn_at_pct = max_dn_at_pct[region];
           overall_max_in_region = region;

           /* find overall max; report maxes from individual regions (including first) if verbose */   
           if (num_regions>1) {       
              for ( /*1st region already done*/; region<MAX_REGIONS; region++) { /* FIND ULTIMATE MAX */
                 if (region_used[region]) {
                    if (verbose>0) printf("In region #%1d, Max DN=%g at PCT=%5.2f\n", 
                                                   region, max_dn[region], max_dn_at_pct[region] );
                    if ( max_dn[region] > overall_max_dn ) {
                       overall_max_dn        = max_dn[region];
                       overall_max_dn_at_pct = max_dn_at_pct[region];
                       overall_max_in_region = region;
                    }

                 }
              }
           }

           sprintf(maxpct_string, "OVERALL Max DN=%g at PCT=%5.2f, found in region %d", 
                                   overall_max_dn,      overall_max_dn_at_pct,  overall_max_in_region );
           if (verbose>0) printf("%s\n", maxpct_string);
           percentage = overall_max_dn_at_pct;              
           found_max_signal = 1;
        }
                     

     }
     /*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     ** FIND PEAKS FOR THE TOUGHER LO-RES CASE
     */
     else /*(lo_res)*/ {
        if (verbose>1) printf("--> FIND PEAKS FOR THE LO-RES CASE\n");

        num_regions = 0;
        for (region=0; region<MAX_REGIONS; region++) {  /* Ensure values initialized to 0. */
           n[region]        = 0;
           for (i=0; i<MAX_CUTS; i++) {
              x_sum[region] = 0;
              y_sum[region] = 0;
              x2sum[region] = 0;
              xysum[region] = 0;
           }
        }
        
        num_regions = 0;
        row = 0;
        while (1)          /* READ IN THE DATA FROM THE PROFILE FILE */
        {
           tstat = tbl_colval(iu, row, ind_dn, TBL_DOUBLE, &dn);
           tstat = tbl_colval(iu, row, ind_pct, TBL_DOUBLE, &pct);
           tstat = tbl_colval(iu, row, ind_region, TBL_INTEGER, &region);
               
           if (tstat == -4 ) break;
           else if (tstat !=  0) {
              printf("Table I/O Error in reading profile table %s\n",in_profile);
              exit(TERMINATE_ERROR);
           }

           /* Check that region is in range before using it as an array index! */
           if ( (region<0) || (region>MAX_REGIONS) ) {
              printf("Error in profile table %s at line pct=%g dn=%g\n",in_profile, pct, dn);
              printf("Region %1d is out of range (0..%1d)\n",region, MAX_REGIONS);
              exit(TERMINATE_ERROR);
           }
           
           if (pct>g_pct_lo && pct<g_pct_hi)  /* ignore values not within ( g_pct_lo < pct < g_pct_hi ) */
           {

              if (n[region]==0) num_regions++;
              i = n[region];
              x[region][i]  =  pct;
              y[region][i]  =  dn;
              x_sum[region] += pct;
              y_sum[region] += dn;
              x2sum[region] += pct * pct;
              xysum[region] += pct * dn;
              n[region]++;
           }
           row++;
        }
        
        if (verbose>1) printf("--> Number of regions found is %1d\n",num_regions);
        if (num_regions==0) {
           printf("No regions in file %s fit %g < pct < %g criteron.\n",in_profile, g_pct_lo, g_pct_hi);
        }
        else {  /* (num_regions>0) */

           /* DO FIT AND STATISTICS ON EACH REGION */
           for (region=0; region<MAX_REGIONS; region++) { 

              nn = n[region]; /* for simplicity */
              if (nn>0) {

                 /* FOR LINEAR FIT y=mx+b, calculate m and b                                              */
                 /* Using formulae from STATISTICAL TREATMENT OF EXPERIMENTAL DATA by Hugh D. Young, 1962 */
                 /* from equation (15.11), p. 145                                                         */

                 denom =     (nn*x2sum[region])            - (x_sum[region]*x_sum[region]);

                 m     =  (  (nn*xysum[region])            - (x_sum[region]*y_sum[region])  ) / denom;

                 b     =  (  (y_sum[region]*x2sum[region]) - (xysum[region]*x_sum[region])  ) / denom;

                 if (verbose>2) printf("----> Y=mX+b  =>  m=%g  b=%g\n",m,b);

                 /* SUBTRACT OFF THAT LINEAR FIT */
                 for (i=0; i<nn; i++) {
                    yh[region][i] = y[region][i] - ( m * x[region][i] ) - b;
                 }


                 /* shift to align baseline */
    /*********** buggy, commented out for now  ....
                 if (baseline_percentile >= 0) {
                   shift_to_baseline = quickselect(yh[region], nn, nn*(baseline_percentile/100.0));
                   printf ("Shifting to %d percentile baseline by %lf\n", baseline_percentile, shift_to_baseline);
                   for (i=0; i<nn; i++) {
                     yh[region][i] -= shift_to_baseline;
                   }
                 }
     ****************/


                 /* MEDIAN FILTERING */
                 if (verbose>1) printf("--> BEGINNING MEDIAN FILTERING USING %1d POINTS....\n",median_pts);
                 if (median_pts%2!=1) {
                    printf("WARNING: median_pts is REQUIRED to be ODD.  Using %1d points instead.\n",++median_pts);
                 }
                 if (median_pts>1) {
                    median_filter_used = median_pts;
                    yf[region][0]    = yh[region][0];     /* End points get only one point -- no median needed. */
                    yf[region][nn-1] = yh[region][nn-1];                 
                    for (i=1; i<nn-1; i++) {
                       nleft  = (2*i) + 1;       /* Size of median available to left of point */
                       nright = (2*(nn-i-1)) + 1;  /* Size of median available to left of point */
                       /* CALCULATE npts = MIN( nleft, nright, median_pts ) */
                       npts = ( (nleft<median_pts) ? nleft  : median_pts );
                       npts = ( (nright<npts)      ? nright : npts       );
                       yf[region][i] = find_median( yh[region] , i-(npts-1)/2 , npts );  
                    }

                    if (verbose>2) {  /* if REALLY verbose! */
                       printf( "---->   ========== Original -> Line subtract -> Median filter results =========== \n");
                       for (i=0; i<nn; i++) printf( "---->   i=%02d  x=%6.2f :  %9.1f  ->  %9.1f  ->  %9.1f \n",
                                                             i,x[region][i],y[region][i],yh[region][i],yf[region][i]);
                       printf( "---->   ========================================================================= \n");
                    }

                 }
                 else {
                    if (verbose>0) printf("NOTE: No Median Filtering requested.\n");
                    for (i=0; i<nn; i++) {
                       yf[region][i] = yh[region][i];
                    }
                 }

                 /* CALCULATE NEW mean AND sigma FOR yf                                                   */
                 /* Using formulae from STATISTICAL TREATMENT OF EXPERIMENTAL DATA by Hugh D. Young, 1962 */
                 /* from equation (3.13), p. 17                                                           */
                 ysum  = 0;
                 y2sum = 0;
                 for (i=0; i<nn; i++) {
                    ysum  += yf[region][i];
                    y2sum += yf[region][i] * yf[region][i];
                 }
                 expr = (y2sum/nn) - ((ysum*ysum)/(nn*nn));
                 if (expr<0) {
                    printf("ERROR in calculation of sigma -- imaginary result for region %1d\n",region);
                    exit(TERMINATE_ERROR);
                 }
                 sigma[region] = sqrt( expr );
                 mean[region]  = ysum/nn;
                 if (verbose>1) printf("--> CALCULATED: For region #%1d (after subtracting line), mean=%g sigma=%g\n",
                                                            region,                         mean[region],sigma[region] );

                 /* Find the MAX peak location in the region (using yh) */
                 max_yh[region]        = yh[region][0];   /* initialize it */
                 max_yf[region]        = yf[region][0];   /* initialize it */
                 max_yf_at_pct[region] =  x[region][0];
                 max_dn[region]        =  y[region][0];
                 for (i=1; i<nn; i++) {
                    if (yh[region][i] > max_yh[region]) {
                       max_yh[region]        = yh[region][i];
                       max_yf[region]        = yf[region][i];
                       max_yf_at_pct[region] =  x[region][i];
                       max_dn[region]        =  y[region][i];
                    }
                 }
                 if (verbose>0) {
                    printf("In region #%1d, Max DN=%g (unfiltered) at PCT=%5.2f\n", 
                                      region,      max_dn[region], max_yf_at_pct[region] );
                 }

                 /* DETERMINE if MAX is a candidate for OVERALL_MAX */
                 if (verbose>0) {
                    printf("*** - region=%d Sig=%g Mean=%g nSig+Mean=%g\n", 
                      region, sigma[region],
                      mean[region], m_sigma*sigma[region]+mean[region]);
                    }
                 if ( max_yh[region] >= (m_sigma*sigma[region]+mean[region]) ) {
                    if (verbose>0) {
                       printf("*** - This maximum meets the criterion of (max > mean + m*sigma+mean)");
                       printf(", where m=%g.\n",m_sigma);
                    }
                 }
                 else {
                    n[region]=0;  /* MARK THIS REGION AS UNINTERESTING */
                    num_regions--;
                    if (verbose>0) {
                       printf("xxx - This maximum does not meet the criterion of (max>mean + m*sigma+mean)");
                       printf(", where m=%g.\n",m_sigma);
                    }
                 }

              } /* end if(nn>0) */

           } /* end for(region) */
        }
        
        if (num_regions>0) {  /* i.e. if we have any candate maxima remaining */

           /* initialize overall_max_dn and overall_max_dn_at_pct */
           region=0;         /* Find first used region (ensured that at least one region_used==1 since num_regions>0) */            
           while(n[region]==0) region++;
           overall_max_dn        = max_dn[region];
           overall_max_dn_at_pct = max_yf_at_pct[region];
           overall_max_in_region = region;

           /* find overall max; report maxes from individual regions (including first) if verbose */   
           if (num_regions>1) {       
              for ( /*1st region already done*/; region<MAX_REGIONS; region++) { /* FIND ULTIMATE MAX */
                 if (n[region]>0) {
                    if (verbose>0) printf("In region #%1d, Max DN=%g at PCT=%5.2f\n", 
                                                   region, max_dn[region], max_yf_at_pct[region] );
                    if ( max_dn[region] > overall_max_dn ) {
                       overall_max_dn        = max_dn[region];
                       overall_max_dn_at_pct = max_yf_at_pct[region];
                       overall_max_in_region = region;
                    }

                 }
              }
           }
           sprintf(maxpct_string, "OVERALL BEST Max DN=%g at PCT=%5.2f, found in region %d", 
                                   overall_max_dn,      overall_max_dn_at_pct,  overall_max_in_region );
           if (verbose>0) printf("%s\n", maxpct_string);
           percentage = overall_max_dn_at_pct;  
           found_max_signal = 1;
           found_source     = 1;
           source_mean      = mean[overall_max_in_region];
           source_sigma     = sigma[overall_max_in_region];
           /* number of sigma over threshold: */
           if (source_sigma>0) source_threshold = (mean[region] - max_yh[region])/source_sigma; 
           else                source_threshold = 0;            

        } /* end if (num_regions!=0) */
        
        else {  /* No candidate maxima remaining */
           if (found_dfrac) { 
              percentage = 100*dfrac;
              if (verbose>0) printf("No candidate maximum found; Cut percentage set to DFRAC=%g\n",percentage);
           }
           else{
              percentage = 100*DEFAULT_DFRAC;  /* a default value */
              if (verbose>0) printf("No candidate maximum found, no DFRAC; Cut percentage set to %g\n",percentage);
           
           }
           row++;
        }



     }
     /*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

     tbl_close(iu); /* close table file */
     if (verbose>1) printf("--> Reading and processing data from PROFILE_FILE=\"%s\" is complete.\n", in_profile);     
     
  } /* end if (find_peaks) */
  
  else {  /* (!find_peaks) */
     if (percentage==-1) {
        if (found_dfrac) {
           percentage = 100*dfrac;
           if (verbose>0) printf("Cut percentage set to DFRAC=%g\n",percentage);
        }
        else {
           percentage = 100*DEFAULT_DFRAC;  /* a default value */
           if (verbose>0) printf("Cut percentage set to %g  (DEFAULT VALUE)\n",percentage);
        }
     }
  }
  
  

  /************************************************************************************************************
  ** Process the wavesamp table data to produce the ridge table file 
  */
  /* Open wavsamp and ridge in tableio and link them */
  iu = tbl_open(in_wavsamp, &ifinfo); 
  tstat   = tbl_fileinfo(iu, &ifinfo);
  if (tstat != 0) {
    printf("Cannot open table: [%s]\n",in_wavsamp);
    exit(64);
  }

  if (verbose>1) printf("--> Reading data from WAVSAMP_FILE=\"%s\"\n", in_wavsamp);
   ou = tbl_openw(temp_ridge, &ofinfo);
   tstat   = tbl_fileinfo(ou, &ofinfo);
  if (tstat != 0) {
    printf("Cannot open table: [%s]\n",temp_ridge);
    exit(64);
  }

  if (verbose>1) printf("--> Writing data to   OUTFILE=     \"%s\"\n", out_ridge);
  tstat = tbl_link(iu, ou);
  
  /* Delete some columns that Ridge table doesn't need */
  tio_del("x_center");
  tio_del("y_center");
  tio_del("x4");  /* Should no longer be in WAVSAMP file anyway */
  tio_del("y4");  /* Should no longer be in WAVSAMP file anyway */
  tio_del("x5");  /* Should no longer be in WAVSAMP file anyway */
  tio_del("y5");  /* Should no longer be in WAVSAMP file anyway */
  
  /* Get column indices of "keeper" columns" */
  if (tio_getind( "order",      &ind_order      )) return TERMINATE_ERROR;
  if (tio_getind( "wavelength", &ind_wavelength )) return TERMINATE_ERROR;
  if (tio_getind( "angle",      &ind_angle      )) return TERMINATE_ERROR;
  if (tio_getind( "x0",         &ind_x0         )) return TERMINATE_ERROR;
  if (tio_getind( "y0",         &ind_y0         )) return TERMINATE_ERROR;
  if (tio_getind( "x1",         &ind_x1         )) return TERMINATE_ERROR;
  if (tio_getind( "y1",         &ind_y1         )) return TERMINATE_ERROR;
  if (tio_getind( "x2",         &ind_x2         )) return TERMINATE_ERROR;
  if (tio_getind( "y2",         &ind_y2         )) return TERMINATE_ERROR;
  if (tio_getind( "x3",         &ind_x3         )) return TERMINATE_ERROR;
  if (tio_getind( "y3",         &ind_y3         )) return TERMINATE_ERROR;
 
  /* Add new columns */
  tbl_setcol(ou, "x_ridge_r", "real", " ", " ", 0., 0., 10,4, &ind_x_ridge_right );
  tbl_setcol(ou, "y_ridge_r", "real", " ", " ", 0., 0., 10,4, &ind_y_ridge_right );
  tbl_setcol(ou, "x_ridge_l", "real", " ", " ", 0., 0., 10,4, &ind_x_ridge_left  );
  tbl_setcol(ou, "y_ridge_l", "real", " ", " ", 0., 0., 10,4, &ind_y_ridge_left  );

  /* Add HISTORY cards */  
  tbl_setkey(ou, "HISTORY", "char", "========== Parameters supplied to RIDGE program below ==========",  ""); 
  param_hcards(&hist_cards);
  for (i=0; hist_cards[i]; i++)
  {
    tbl_setkey(ou, "HISTORY", "char", hist_cards[i], "");
  }
  tbl_setkey(ou, "HISTORY", "char", "========== RIDGE processing results summary below ==========",  "");
  if (find_peaks) tbl_setkey(ou, "HISTORY", "char", "FIND PEAKS: YES", "");
  else            tbl_setkey(ou, "HISTORY", "char", "FIND PEAKS: NO",  "");
  if (maxpct_string[0]!='\0') tbl_setkey(ou, "HISTORY", "char", maxpct_string, "");
  if (median_filter_used>0) sprintf(outstring,"MED_FILT_SIZE = %1d", median_filter_used);
  else                      sprintf(outstring,"MEDIAN FILTER not used.");
  tbl_setkey(ou, "HISTORY", "char", outstring, "");
  sprintf(outstring,"PERCENTAGE = %5.2f  (cut was performed here)",percentage);
  tbl_setkey(ou, "HISTORY", "char", outstring,  "");
  if (found_max_signal) {
     sprintf(outstring,"MAX_SIGNAL = %g  (maximum signal value in DN)",overall_max_dn);
     tbl_setkey(ou, "HISTORY", "char", outstring,  "");
     sprintf(outstring,"CUT_POS = %g  (pct at which the max signal was found)",percentage);
     tbl_setkey(ou, "HISTORY", "char", outstring,  "");
     sprintf(outstring,"MAX_SIGNAL_IN_REGION = %1d  (region where max signal was found)",overall_max_in_region);
     tbl_setkey(ou, "HISTORY", "char", outstring,  "");
  }
  sprintf(outstring,"DET_BOOLEAN = %g  (was a source detected)", (found_source?"T":"F") );
  if (found_source) {
     sprintf(outstring,"DET_THRESH = %g  (number of sigma over threshold)",source_threshold);
     tbl_setkey(ou, "HISTORY", "char", outstring,  "");
     sprintf(outstring,"DET_SIGMA = %g  (sigma value used to find threshold)",source_sigma);
     tbl_setkey(ou, "HISTORY", "char", outstring,  "");
  }

  /************************************************************************************************************ 
  ** Write data rows 
  */  
  percent_frac = 1 - (percentage / 100.0);  /* Changed from previous definition for Version 1.6 (CR3958) */
  if (verbose>0) printf("Calculating ridge table, using PERCENTAGE = %5.2f%%\n", percentage );

 tstat = tbl_numcheck(iu, &nrows);
 x_ridge_right = (float *) malloc(nrows* sizeof(float));
 y_ridge_right = (float *) malloc(nrows* sizeof(float));
 x_ridge_left  = (float *) malloc(nrows* sizeof(float));
 y_ridge_left  = (float *) malloc(nrows* sizeof(float));
 if (x_ridge_right == NULL || y_ridge_right == NULL ||
     x_ridge_left == NULL || y_ridge_left == NULL)
 {
   printf("Error in allocating memory for output table params\n");
   return TERMINATE_ERROR;
 } 
 
 row = 0;
  while (1)
  {
    /* Read data from input table */
    tstat = tbl_colval(iu, row, ind_x0, TBL_DOUBLE, &x0);;
    tstat = tbl_colval(iu, row, ind_y0, TBL_DOUBLE, &y0);;
    tstat = tbl_colval(iu, row, ind_x1, TBL_DOUBLE, &x1);;
    tstat = tbl_colval(iu, row, ind_y1, TBL_DOUBLE, &y1);;
    tstat = tbl_colval(iu, row, ind_x2, TBL_DOUBLE, &x2);;
    tstat = tbl_colval(iu, row, ind_y2, TBL_DOUBLE, &y2);;
    tstat = tbl_colval(iu, row, ind_x3, TBL_DOUBLE, &x3);;
    tstat = tbl_colval(iu, row, ind_y3, TBL_DOUBLE, &y3);;

    if (tstat == -4) break;   /*EOF*/
    else if (tstat !=  0)
    {
      printf("Table I/O Error reading wavesamp table\n");
      exit(TERMINATE_ERROR);
    }

    x_ridge_right[row] = percent_frac*(x1-x0) + x0;  /* Note percent_frac is defined as 1-previous_percent_frac */
    y_ridge_right[row] = percent_frac*(y1-y0) + y0;  /* for Version 1.6 (CR3958)                                */
    x_ridge_left[row]  = percent_frac*(x2-x3) + x3;
    y_ridge_left[row]  = percent_frac*(y2-y3) + y3;
    row++;
  }

/* Write to output table */
  tstat = tbl_setcolval_f(ou, ind_x_ridge_right, "real", nrows, x_ridge_right);
  tstat = tbl_setcolval_f(ou, ind_y_ridge_right, "real", nrows, y_ridge_right);
  tstat = tbl_setcolval_f(ou, ind_x_ridge_left,  "real", nrows, x_ridge_left);
  tstat = tbl_setcolval_f(ou, ind_y_ridge_left, "real", nrows,  y_ridge_left);
  if (tstat !=  0)
  {
      printf("Table I/O Error setting col values in ridge table\n");
      exit(TERMINATE_ERROR);
  }
  tstat = tbl_write(ou);
  if (tstat !=  0)
  {
      printf("Table I/O Error writing out ridge table\n");
      exit(TERMINATE_ERROR);
  }

  /* Close files and exit */
  tbl_close(iu);
  tbl_close(ou);

  
  /************************************************************************************************************
  ** Append header lines from profile table to output file; cleanup stuff
  */
  if (have_profile_file) {  /* If PROFILE_FILE has been specified */
    temp_ridge_file = fopen( temp_ridge, "r" );
    ridge_file      = fopen( out_ridge, "w" );
    while (1) {
       fgets( r_keyword_line, MAXLINELEN, temp_ridge_file );
       if (r_keyword_line[0]=='\\') fprintf( ridge_file, "%s", r_keyword_line );
       else break;
    }
    profile_file = fopen( in_profile, "r" );
    fprintf( ridge_file, "\\COMMENT ========== Keywords from PROFILE table file below ==========\n" );
    while (1) {
       fgets( p_keyword_line, MAXLINELEN, profile_file );
       if (p_keyword_line[0]=='\\') fprintf( ridge_file, "%s", p_keyword_line );
       else break;
    }
    fprintf( ridge_file, "%s", r_keyword_line );  /* Print out the first non-keyword ridge line already read in */
    while ( fgets( r_keyword_line, MAXLINELEN, temp_ridge_file ) ) {
       fprintf( ridge_file, "%s", r_keyword_line );
    }
    
    fclose(ridge_file);
    fclose(profile_file);
    
    /* Delete the temporary ridge file */
    sprintf(cmd, "rm -f %s", temp_ridge);
    system(cmd);
  }
  else {
    /* The "temporary" ridge file IS the output ridge file in this case */
    sprintf(cmd, "mv %s %s", temp_ridge, out_ridge );
    system(cmd);
  }  
    
  free (x_ridge_right);
  free (y_ridge_right);
  free (x_ridge_left);
  free (y_ridge_left);
  return 0;
}


/*============================================= MAIN ====================================================*/
main(argc, argv)
int argc;
char *argv[];
{
  int status;
  status = ridge(argc, argv);
  if (status < 0) return TERMINATE_ERROR;
  param_exit(status);
}
