

/*********************************************** 
rowdroop_compute.c

Purpose

Main processing loop of program.

compute and remove the row droop.

***********************************************/

#include <stdio.h>
/* #include <ieeefp.h> */
#include <math.h>   
#include "rowdroop.h"
#include "rowdroop_errcodes.h"
#include "rowdroop_defs.h"

void rowdroop_compute(	
            Filenames      *Fnames,
            Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In_AD,
            float	   *FP_Data_Unc_In_AD,
            float	   *FP_Data_In,
            float	   *FP_Data_Unc_In,
            float	   *FP_Data_Rd_Unc,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask) {
 int	 i, j, k, ki, l, m, n, pixels;
 int     itemp;
 int     I_fits_return_status=0;
 int BmaskFatal, CmaskFatal, DmaskFatal, PmaskFatal; 
 int nx, ny, nz;
 int d1, d2;
 int  findx;

 short *Tmp_Row_Dmsk=0;

 FILE* fid = NULL;

 float   dcouple[4], sumimg, sumusq, sumu;
 float   sumnew, sumold, var, val, unc_corr2;
 float    corr[4], unc_corr[4];
 float    fac0, fac1, uvar;

 if(Stat->I_status) return;

 BmaskFatal = Const->BmaskFatal;
 CmaskFatal = Const->CmaskFatal;
 DmaskFatal = Const->DmaskFatal;
 PmaskFatal = Const->PmaskFatal;

 nx = Dim_In->I_NX;
 ny = Dim_In->I_NY;
 nz = Dim_In->I_NZ;
 pixels = nx * ny;

 Tmp_Row_Dmsk = (short *) calloc(nx, sizeof(short));
 if (Tmp_Row_Dmsk == NULL) {
   printf("Compute_Results: Error in allocating local block\n");
   Stat->I_status = MALLOC_FAILED;
   return;
 }

if(Stat->I_Verbose)
{
    fid = fopen("rowdroop.tbl", "w"); 
    fprintf(fid,"|  Row  |  sum_in    |  sum_corr   |\n");
    fprintf(fid,"| int   |  real      |    real     |\n");
}

if(Stat->I_Verbose)
   printf("Start row droop removal using coupling constants for 4 channels:\n \t%f\t%f\t%f\t%f:\n", 
     Const->rowdroop[0], Const->rowdroop[1], Const->rowdroop[2], Const->rowdroop[3]);
     

 for (k=0; k<nz; ++k) 
 {
   ki = k*(nx*ny);
   for (j=0; j<ny; ++j) 
   {
     sumimg = 0.0;
     sumusq = 0.0;
     sumnew = 0.0;
     sumold = 0.0;
     for (i=0; i<nx; ++i) 
     {
        m = i + j*ny + ki;
        n = i + j*ny; 
        Tmp_Row_Dmsk[i] = 0;
        if ( (*(SP_bmask + n) & BmaskFatal) == 0 && (*(SP_cmask + m) & CmaskFatal) == 0
           && (*(SP_dmask + m) & DmaskFatal) == 0 && (*(SP_pmask + n) & PmaskFatal) == 0 ) 
        {

	  /* Changed TYB 6/1/09 */

	if( (iznand( (double) *(FP_Data_In_AD + m)))  || (( *(SP_dmask +  m) & 16384) > 0) ||
	  //          if( (isnan(*(FP_Data_In_AD + m)))  || (( *(SP_dmask +  m) & 16384) > 0) ||

             (( *(SP_dmask + m) & 8192)  > 0) ) 
              Tmp_Row_Dmsk[i] = 1;

          if( !(iznand((double) *(FP_Data_In_AD + m)))  && ( *(SP_dmask +  m) <  16384) )
	    //          if( !(isnan(*(FP_Data_In_AD + m)))  && ( *(SP_dmask +  m) <  16384) )

          {
              sumimg += *(FP_Data_In_AD +  m);
              if ( !(iznand((double) *(FP_Data_Unc_In_AD +  m))) )
		//              if ( !(isnan(*(FP_Data_Unc_In_AD +  m))) )

                sumusq += (*(FP_Data_Unc_In_AD +  m))  * (*(FP_Data_Unc_In_AD +  m)); 
          }
        }
      }  /* sum whole row */

      if(Stat->I_Verbose )
         printf("k %d j %d i %d sumi sumu %f %f\n",k,j,i,sumimg,sumusq);

      for (i=0; i<4; i++)     /* compute corr factors for this row */
      {
       corr[i] = Const->rowdroop[i]*sumimg;
       unc_corr[i] =  pow(Const->rowdroop_unc[i] * sumimg, 2.) +
                      Const->rowdroop[i] * Const->rowdroop[i] * sumusq;
       if(Stat->I_Verbose)
          printf("j %d corr %f unc corr %f\n",j, corr[i], unc_corr[i]);
       if (Const->Rd_Unc_Fits)
       {
          findx = j + k*ny + i *(ny*nz);
         *(FP_Data_Rd_Unc + findx) = sqrt(unc_corr[i]);           
       } 
      }

      for (i=0; i<nx; i=i+4)       /* apply the correction to this row */
     {
        for (l=0; l<=3; ++l) 
        {
          m = l + i + j*ny + ki;      
          n = l + i + j*ny; 
         
          if ( (*(SP_bmask + n) & BmaskFatal) == 0 && (*(SP_cmask + m) & CmaskFatal) == 0
             && (*(SP_dmask + m) & DmaskFatal) == 0 && (*(SP_pmask + n) & PmaskFatal) == 0 ) 
          {
            if (Tmp_Row_Dmsk[i+l] == 1)
               *(SP_dmask +  m) |= ROWDROOP_NO_APPLY;
            else {
                sumold += *(FP_Data_In +  m);
                var = *(FP_Data_In + m);
                uvar = *(FP_Data_Unc_In + m);
                if(Stat->I_Verbose) 
                  printf("l %d il %d corr %f old %f",l,i+l,corr[l],var);
                *(FP_Data_In + m) -= corr[l];           
               *(FP_Data_Unc_In + m) = sqrt( pow(*(FP_Data_Unc_In + m), 2.0) +
                                             unc_corr[l] );
                sumnew += *(FP_Data_In +  m);   
                var = var - *(FP_Data_In + m);

                if(Stat->I_Verbose) 
                   printf(" new  %f dif %f unc %f %f\n", 
                     *(FP_Data_In + m),  var, *(FP_Data_Unc_In + m), uvar); 
            }
          }
          if(Stat->I_Verbose)
            fprintf(fid, "%6d   %12.2f %12.2f\n", j, sumold, sumnew);
        }
      }
   }
 }
 

if(Stat->I_Verbose)
   fclose(fid);
 printf("rowdroop_compute: Numerical Computations finished.\n");
 free(Tmp_Row_Dmsk);

}
