

/*********************************************** 
rowdroop_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of rowdroop functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_rowdroop_message = Code of error that occurred.
I_rowdroop_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "rowdroop.h"
#include "rowdroop_errcodes.h"
#include "rowdroop_defs.h"


rowdroop_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_rowdroop_message;
int	I_rowdroop_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_rowdroop_message = RIGHT_MASK&(Stat->I_status);
I_rowdroop_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_rowdroop_message&FITS_MSG_MASK)	{
I_rowdroop_message -= FITS_MSG_MASK;
}

rowdroop_decode_message(I_rowdroop_message, CP_Message);
rowdroop_decode_fcnname(I_rowdroop_root, CP_Function);

if (Stat->I_status)
{
printf("rowdroop Error Message      0x%04x: %s.\n", I_rowdroop_message, CP_Message);
printf("rowdroop Error in Function  0x%04x: %s.\n", I_rowdroop_root, CP_Function);
}


I_Time = clock();
printf("Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("rowdroop_error_handler: Exception Handler finished.\n");

}
