

/****************************************************** 
rowdroop_exec.c
This module removes an excess of photocharge generated
along the fast clock direction of IRS detectors arrays.

This routine is the main program.
 
Calls the initialization functions and main loop of 
program. Saves output to a file.

Definitions of Variables in rowdroop_defs.h

Version 1.4 - CR #4247
Version 1.5 - CR #5883
 
********************************************************/

#include <time.h>         /* time() may need this */
#include <sys/types.h>    /* time() may need this */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
/* #include <ieeefp.h> */
#include <sys/stat.h>
#include <fcntl.h>
#include "fitsio.h"
#include "rowdroop.h"
#include "rowdroop_errcodes.h"
#include "rowdroop_defs.h"

int main(int argc, char **argv) {			

float	*FP_Data_In_AD=0;
float	*FP_Data_Unc_In_AD=0;
float	*FP_Data_In=0;
float	*FP_Data_Unc_In=0;
float	*FP_Data_Rd_Unc=0;

short   *SP_pmask=0;
short   *SP_dmask=0;
short   *SP_cmask=0;
short   *SP_bmask=0;

Filenames	Fnames;
Constants       Const;
Status		Stat;
Dims            Dim_In;
struct tm       *tmp_time;
time_t          L_time;

/* null filename pointers */

clock();

printf("Pipeline Module ROWDROOP Version %2.1f\n", VERSN);

/* Calculate and print to standard out time
   Keep tmp_time and module time for time
   time stamping all output
*/
L_time = time(NULL);
tmp_time = localtime(&L_time);
printf("Processing time ");
printf(asctime(tmp_time));
printf("\n");

/* Set constant default values */
rowdroop_init_const( &Fnames, &Const, &Stat);
if (Stat.I_Verbose)
   printf("Initialization complete\n");

/* Parse the namelist file */
rowdroop_parse_namelist( argc, argv, &Const, &Fnames, &Stat);

/* Parse the command line, replacing values if specified */
rowdroop_parse_args( argc, argv, &Const, &Fnames, &Stat);

/* Get the data and keyword values from the input file */
rowdroop_read_data( &Fnames, &Const, &Dim_In, 
                    &FP_Data_In_AD, &FP_Data_Unc_In_AD, 
                    &FP_Data_In, &FP_Data_Unc_In, &FP_Data_Rd_Unc,
                    &Stat, &SP_pmask, &SP_dmask, &SP_bmask, &SP_cmask);

/* Perform numerical computations on the data */
rowdroop_compute( &Fnames, &Const, &Dim_In, 
                  FP_Data_In_AD, FP_Data_Unc_In_AD, 
                  FP_Data_In, FP_Data_Unc_In, FP_Data_Rd_Unc,
                  &Stat, SP_pmask, SP_dmask, SP_bmask, SP_cmask);

/* Save the results as fits files. */			
rowdroop_output( &Fnames, &Const, &Dim_In, &Stat, tmp_time,
                  SP_dmask, FP_Data_In, FP_Data_Unc_In,
                  FP_Data_Rd_Unc);

/* Print filename, error message and timing data */
rowdroop_error_handler( &Stat, &Fnames);

/* Deallocate memory */			
rowdroop_exit( FP_Data_In_AD, FP_Data_Unc_In_AD, 
               FP_Data_In, FP_Data_Unc_In,
               &Dim_In, &Stat,
               SP_pmask, SP_dmask, SP_bmask, SP_cmask);

printf("exit code %d\n", Stat.I_status);

/* Send return value */
exit(Stat.I_status);

}
