

/*********************************************** 
rowdroop_exit.c

Purpose

Cleanly exit the program.

Overview
Deallocate memory and print any diagnostics.



***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "rowdroop.h"
#include "rowdroop_errcodes.h"
#include "rowdroop_defs.h"


rowdroop_exit(	
            float      *FPP_Data_In_AD,
            float      *FPP_Data_Unc_In_AD,
            float      *FPP_Data_In,
            float      *FPP_Data_Unc_In,
            Dims       *Dim_In,
            Status     *Stat,
            short      *SPP_pmask,
            short      *SPP_dmask,
            short      *SPP_bmask,
            short      *SPP_cmask) {

/* Allocated in rowdroop_read_data */

if (FPP_Data_In_AD != NULL) free(FPP_Data_In_AD);
if (FPP_Data_Unc_In_AD != NULL) free(FPP_Data_Unc_In_AD);
if (FPP_Data_In != NULL) free(FPP_Data_In);
if (FPP_Data_Unc_In != NULL) free(FPP_Data_Unc_In);
if (SPP_pmask != NULL) free(SPP_pmask);
if (SPP_dmask != NULL) free(SPP_dmask);
if (SPP_bmask != NULL) free(SPP_bmask);
if (SPP_cmask != NULL) free(SPP_cmask);

/* Set up exit status for Opus pipeline.
   For now any non zero status will be output
   as a failure.  There are no warnings.
*/

if(Stat->I_status)
  {
  Stat->I_status = 64;
  }

if(Stat->I_Verbose)
printf("rowdroop_exit: Memory Deallocated.\n");
}
