

/*************************************************************** 
rowdroop_parse_args.c

Purpose
Parse the command line to get input and output
file names,  constants to do the computations,
and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In_AD = In Data File Name 
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output Probabilities File Name.
CP_Filename_Pmask = Input Pmask.
CP_Filename_Dmask = Input Dmask.
CP_Filename_Bmask = Input Bmask.
CP_Filename_Cmask = Input Cmask.
I_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include "rowdroop.h"
#include "rowdroop_errcodes.h"
#include "rowdroop_defs.h"


rowdroop_parse_args(	
            int	        argc,
            char        **argv,
            Constants   *Const,
            Filenames   *Fnames,
            Status	*Stat) {

   int	i=1;

   int     I_out_chk, I_in_chk;
   int     I_droop_chk, I_nl_chk;
   int     droop_chk[4] = {1, 1, 1, 1};
   int     I_pair_err;

   /* flags to check the completeness of the switch-value pairs */
   I_in_chk = 1;
   I_out_chk = 1;
   I_droop_chk = 0;
   I_nl_chk = 1;
   I_pair_err = 0;

   while( i < argc ) {

     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               I_nl_chk =0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'm':      /* -i <input_image_fname_4_corr>  */
            if (argv[i][2] == 'd')       /* Input image File for correction*/
            {
              if ( ++i >= argc )
                 I_pair_err = 1;
	      else {
                 I_in_chk = 0;
                 sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In_AD);
              }
            }
            else if (argv[i][2] == 'u')           /* Unc for AD image */
            {
              if ( ++i >= argc )
                 I_pair_err = 1;
	      else {
                 I_in_chk = 0;
                 sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_Unc_In_AD);
              }
            }
            break;
          case 'i':      /* -i <input_image_fname>  */
            if (argv[i][2] == 'd')           /* Input image File */
            {
                if ( ++i >= argc )
                 I_pair_err = 1;
	      else {
                 I_in_chk = 0;
                 sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
              }
             }
             else if (argv[i][2] == 'u')           /* Unc for input image */
             {
                if ( ++i >= argc )
                 I_pair_err = 1;
	      else {
                 sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_Unc_In);
              }
             }
            break;
          case 'o':      /* -o <output_fname>  */
            if (argv[i][2] == 'd')           /* Input image File */
            {
              if ( ++i >= argc )
                 I_pair_err = 1;
              else {
                 I_out_chk = 0;
                 sscanf("!", "%c", Fnames->CP_Filename_FITS_Out); /* clobber */
                 sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out[1]));
              }
            }
            else if (argv[i][2] == 'u')           /* output Unc image file */
             {
              if ( ++i >= argc )
                 I_pair_err = 1;
              else {
                 sscanf("!", "%c", Fnames->CP_Filename_FITS_Unc_Out); /* clobber */
                 sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Unc_Out[1]));
              }
            }
            break;
          case 'c':      /* -c <Cmask Fits file>  */
            if ( ++i >= argc ) {
              I_pair_err = 1;
            }
            else {
              sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
            }
            break;
          case 'r':      /* -r <rowdroop coupling constant>  */
             if (argv[i][2] == 'u')     /* Output rowdroop unc fits */
             {
               if ( ++i >= argc ) {
                 I_pair_err = 1;
               }
               else {
                 sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_Rd_Unc_Out);
               }
              Const->Rd_Unc_Fits = 1;
             }
             else if (argv[i][2] == '1')           /* for chan 1 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  droop_chk[0] = 0;
                  sscanf(argv[i],"%f", &(Const->rowdroop[0]));
               }
             }
             else if (argv[i][2] == '2')      /* for chan 2 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  droop_chk[1] = 0;
                  sscanf(argv[i],"%f", &(Const->rowdroop[1]));
               }
             }
             else if (argv[i][2] == '3')      /* for chan 3 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  droop_chk[2] = 0;
                  sscanf(argv[i],"%f", &(Const->rowdroop[2]));
               }
             }
             else if (argv[i][2] == '4')      /* for chan 4 */
             {
               if ( ++i >= argc )  {
                  I_pair_err = 1;
               }
               else
               {
                  droop_chk[3] = 0;
                  sscanf(argv[i],"%f", &(Const->rowdroop[3]));
               }
            }
            break;
          case 'u':      /* -r <rowdroop coupling constant errors>  */
             if (argv[i][2] == '1')           /* for chan 1 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  sscanf(argv[i],"%f", &(Const->rowdroop_unc[0]));
               }
             }
             else if (argv[i][2] == '2')      /* for chan 2 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  sscanf(argv[i],"%f", &(Const->rowdroop_unc[1]));
               }
             }
             else if (argv[i][2] == '3')      /* for chan 3 */
             {
               if ( ++i >= argc ) {
                  I_pair_err = 1;
               }
               else
               {
                  sscanf(argv[i],"%f", &(Const->rowdroop_unc[2]));
               }
             }
             else if (argv[i][2] == '4')      /* for chan 4 */
             {
               if ( ++i >= argc )  {
                  I_pair_err = 1;
               }
               else
               {
                  sscanf(argv[i],"%f", &(Const->rowdroop_unc[3]));
               }
            }
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            break;
          case 'd':      /* -d <dmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
            break;
          case 'b':      /* -b <bmask pixel status file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Bmask);
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p')           /* for Pmask */
            {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->PmaskFatal));
                  }
            }
            else if (argv[i][2] == 'd')      /* for Dmask */
            {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->DmaskFatal));
                  }
            }
            else if (argv[i][2] == 'b')      /* for Bmask */
            {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->BmaskFatal));
                  }
            }
            else if (argv[i][2] == 'c')      /* for Cmask */
            {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &(Const->CmaskFatal));
                  }
            }
            else
            {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  printf("rowdroop_parse_args: unknown argument\n");
            }
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;  /* should -v definitely gives Stat->I_Verbose=1???*/
            break;
          default:
            printf("rowdroop_parse_args: unknown argument\n");
       }
     }
     else {
       I_pair_err = 1;
     }
     i++;
   }

   if ( I_pair_err ) {
      printf("rowdroop_parse_args: command line syntax error\n");
      rowdroop_usage();
      Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   }

   for (i=0; i<4; i++)
   {
     if (droop_chk[i] == 1) 
     {
       I_droop_chk = 1;
       break;
     }

   }
   if ( (I_in_chk || I_droop_chk || I_out_chk) && I_nl_chk ) {
      printf("Error in command line arguments; required input not complete.\n\n");
      rowdroop_usage();
      Stat->I_status = PARSE_ARGS|REQ_ARGS_INCOMP;
   }

   if(Stat->I_Verbose)	{
      printf("rowdroop_parse_args: Input Namelist File = \t%s\n", Fnames->CP_Filename_Namelist);
      printf("rowdroop_parse_args: Input Image File = \t%s\n", Fnames->CP_Filename_FITS_In);
      printf("rowdroop_parse_args: Input Image Uncertainty File = \t%s\n", Fnames->CP_Filename_FITS_Unc_In);
      printf("rowdroop_parse_args: Input Image File used to calculate correction = \t%s\n", Fnames->CP_Filename_FITS_In_AD);
      printf("rowdroop_parse_args: Input Image Uncertainty File for above= \t%s\n", Fnames->CP_Filename_FITS_Unc_In_AD);
      printf("rowdroop_parse_args: Ouput Image File = \t%s\n", Fnames->CP_Filename_FITS_Out);
      printf("rowdroop_parse_args: Ouput Uncertainty Image File = \t%s\n", Fnames->CP_Filename_FITS_Unc_Out);
      printf("rowdroop_parse_args: Input Rowdroop constants = \t%f\t%f\t%f\t%f\n", 
              Const->rowdroop[0], Const->rowdroop[1],Const->rowdroop[2],Const->rowdroop[3]);
      printf("rowdroop_parse_args: Input Uncertainty in Rowdroop constants = \t%f\t%f\t%f\t%f\n", 
              Const->rowdroop_unc[0], Const->rowdroop_unc[1],Const->rowdroop_unc[2],Const->rowdroop_unc[3]);
      printf("rowdroop_parse_args: Pmask File = \t%s\n", Fnames->CP_Filename_Pmask);
      printf("rowdroop_parse_args: Dmask File = \t%s\n", Fnames->CP_Filename_Dmask);
      printf("rowdroop_parse_args: Bmask File = \t%s\n", Fnames->CP_Filename_Bmask);
      printf("rowdroop_parse_args: Cmask File = \t%s\n", Fnames->CP_Filename_Cmask);
      printf("rowdroop_parse_args: Pmask Fatal Template = \t%d\n", Const->PmaskFatal);
      printf("rowdroop_parse_args: Dmask Fatal Template = \t%d\n", Const->DmaskFatal);
      printf("rowdroop_parse_args: Cmask Fatal Template = \t%d\n", Const->CmaskFatal);
      printf("rowdroop_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
    }
}
