/* $Id: slremov_average.c,v 1.6 2009/02/02 17:51:00 bob Exp $ */


/*********************************************** 
slremov_average.c

Purpose

provide mean and standard deviation for input data.

***********************************************/

#include <stdio.h>
#include <math.h>
#include "slremov.h"
#include "slremov_defs.h"

double compute_median(double*, int);


slremov_average(	
            double          *data,
            int             dim,
            double          *mean,
            double          *rms,
            double          *median,
            Status 	    *Stat)
{
int	i;
int     I_fits_return_status=0;
double  sum, sumrms;

if(Stat->I_status) return;

if(dim <= 0) {
   printf("slremov_average: no useful data\n");
   Stat->I_status = AVERAGE|NO_USEFUL_DATA;
   return;
}

sum = 0.0;
sumrms = 0.0;
for (i=0; i<dim; ++i) sum += *(data + i);

*mean = sum/(double)dim;

for (i=0; i<dim; ++i) sumrms += (*(data + i) - *mean)*(*(data + i) - *mean);

*rms = sqrt(sumrms/(double)dim);

*median = compute_median(data, dim);

/*
if(Stat->I_Verbose)
printf("slremov_compute_results: Averaging Computations finished.\n");
*/

}

double compute_median(
  double *data,
  int    dim
) {
  double tmp_data[51];
  double sorted_data[51];
  int last_tmp_i;
  double highest;
  int i_highest;
  int i, i1, i2;
  double median;

  /* first sort values */
  for (i=0; i<dim; i++) tmp_data[i] = data[i];
  for (last_tmp_i=dim-1; last_tmp_i>=0; last_tmp_i--) {
    i_highest = 0;
    highest = tmp_data[i_highest];
    for (i=1; i<=last_tmp_i; i++) 
    if (tmp_data[i] > highest) {
      i_highest = i;
      highest = tmp_data[i_highest];
    }
    sorted_data[last_tmp_i] = highest;
    tmp_data[i_highest] = tmp_data[last_tmp_i];
  }


  /* compute median */
  i1 = dim/2;  i2 = (dim-1)/2;
  median = (sorted_data[i1] + sorted_data[i2]) / 2.0;

/* DEBUG
printf ("\n");
for (i=0; i<dim; i++) printf ("DEBUG: %d = %f\n", i, data[i]);
for (i=0; i<dim; i++) printf ("DEBUG: %d = %f\n", i, sorted_data[i]);
printf ("median = %f\n", median);
*/

  return median;
}
