/* $Id: slremov_compute.c,v 1.16 2009/02/02 17:51:00 bob Exp $ */


/*********************************************** 
slremov_compute.c

Purpose

Main processing loop of program.

***********************************************/

#include <stdio.h>
/* #include <ieeefp.h> */
#include <math.h>
#include "slremov.h"
#include "slremov_defs.h"

void slremov_compute(	
            Filenames      *Fnames,
	    Constants      *Const,
            Dims           *Dim_In,
            float	   *FP_Data_In,
	    float          *FP_Vars_In,
            float          **FP_Data_Out,
            float          **FP_Vars_Out,
            Status	   *Stat,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask,
            short          *SP_lmask)
{
int	i, j, x, y, z, nrow, error_message=0;
int     itemp, left, right, rownum_avg;
int     maskflag, fit_flag;
int     l_index, r_index, zero_to_one, one_to_zero;
int     I_fits_return_status=0;
int     nomod=0;
int     poorfit=0;
float   data_temp;
double  b0, b1;
double  eb0, eb1;
double  expterm;
double  data_err, b0_err, b1_err;
double  *FP_SL, *FP_I, *FP_ERR, *FP_LEFT, *FP_RIGHT;
double  *adj, avg_col=0, rms_col=0, median_col;
double  avg_in_row=0, rms_in_row=0, median_in_row;
double  avg_left, rms_left=0, median_left, avg_right, rms_right, median_right;
double  reduced_chi2;

if(Stat->I_status) return;

rownum_avg = 5;  /* number of adjacent rows for average */

/* allocate memory for output data */

*FP_Data_Out = (float *) calloc( Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ, sizeof(float));
*FP_Vars_Out = (float *) calloc( Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ, sizeof(float));

/* Use I_fits_return_status to store status, 
   so if there was a problem with calloc
   subsequent fits functions can detect it */
if(FP_Data_Out==NULL || FP_Vars_Out==NULL) {
   Stat->I_status = READ_DATA|MALLOC_FAILED;
   return;
}

for (i=0; i<Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ; ++i) {
   *(*FP_Data_Out + i) = *(FP_Data_In + i);
   *(*FP_Vars_Out + i) = *(FP_Vars_In + i);
}

b0=0.0;
b1=0.0;
eb0=0.0;
eb1=0.0;
FP_SL = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_I = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_ERR = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_LEFT = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_RIGHT = (double *) calloc(Dim_In->I_NX, sizeof(double));
adj = (double *) calloc(rownum_avg, sizeof(double));
if (FP_SL == NULL || FP_I == NULL || FP_ERR == NULL || FP_LEFT == NULL || FP_RIGHT == NULL || adj == NULL) {
   printf("slremov_compute: cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED;;
   return;
}

for (z=0; z<Dim_In->I_NZ; ++z) {

/* examine input Lmask; here an H-shape Lmask around
   x=Const->Hx_Center is assumed to well-define the
   "left" and "right" regions to examine applicability
   of stray-light removal */
r_index = 999;
l_index = -1;
for (y=0; y<Dim_In->I_NY; ++y) {
   zero_to_one=0;
   one_to_zero=0;
   maskflag=0;
   for (x=0; x<Dim_In->I_NX; ++x) {
      if ( *(SP_lmask + x + y*Dim_In->I_NX) == 1 ) {
         if (maskflag == 0) zero_to_one++;
         if (zero_to_one == 2 && maskflag == 0) {
            if (x < r_index) r_index = x;
         }
         maskflag = 1;
      }
      else if ( *(SP_lmask + x + y*Dim_In->I_NX) == 0) {
         if (maskflag == 1) one_to_zero++;
         if (one_to_zero == 1 && maskflag == 1 && x < Const->Hx_Center) {
            if (x > l_index) l_index = x-1;
         }
         maskflag = 0;
      }
   }
}
if (r_index <= 0 || r_index >= Dim_In->I_NX || l_index <= 0 || l_index >= Dim_In->I_NX) {
   printf("slremov_compute: invalid input Lmask\n");
   Stat->I_status = COMPUTE|INVALID_LMASK;
   return;
}
else if (Stat->I_Verbose) {
   printf("slremov_compute: found Lmask left region has maximum pixel coordinate %d\n", l_index+1);
   printf("slremov_compute: found Lmask right region has minimum pixel coordinate %d\n", r_index+1);
}

for (y=0; y<Dim_In->I_NY; ++y) {
   itemp=0;
   left=0;
   right=0;
   fit_flag=0;
   reduced_chi2 = 0.0;
   for (x=0; x<Dim_In->I_NX; ++x) {
      nrow=0;
      if ( *(SP_lmask + x + y*Dim_In->I_NX) == 1 ) {
         if ( (*(SP_pmask + x + y*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
              (*(SP_dmask + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
              (*(SP_cmask + x + y*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
              (*(SP_bmask + x + y*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
            data_temp = *(FP_Data_In + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            if (!(iznand((double) data_temp))) {
               *(adj+nrow) = data_temp;
               nrow++;
            }
         }
         if (y > 1 && y < Dim_In->I_NY-2) {
            if ( *(SP_lmask + x + (y-1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) { 
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y-2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y+1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y+2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
         }
         else if (y == 0) {
            if ( *(SP_lmask + x + (y+1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y+2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
         }
         else if (y == 1) {
            if ( *(SP_lmask + x + (y-1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y+1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y+2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
         }
         else if (y == Dim_In->I_NY-1) {
            if ( *(SP_lmask + x + (y-1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y-2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
         }
         else if (y == Dim_In->I_NY-2) {
            if ( *(SP_lmask + x + (y+1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y+1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y+1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y+1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y+1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y-1)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-1)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-1)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-1)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-1)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
            if ( *(SP_lmask + x + (y-2)*Dim_In->I_NX) == 1 &&
                 (*(SP_pmask + x + (y-2)*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
                 (*(SP_dmask + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
                 (*(SP_cmask + x + (y-2)*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
                 (*(SP_bmask + x + (y-2)*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
               data_temp = *(FP_Data_In + x + (y-2)*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
               if (!(iznand((double) data_temp))) {
                  *(adj+nrow) = data_temp;
                  nrow++;
               }
            }
         }
         if (nrow != 0) {
            /* calculate average pixel value of adjacent rows for pixel */
            slremov_average(adj, nrow, &avg_col, &rms_col, &median_col, Stat);
            *(FP_SL + itemp) = median_col;
            *(FP_I + itemp) = (double)x;
            /* here uncertainty value for that pixel may need revisit */
            if (!(iznand((double) *(FP_Vars_In + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY)))) {
               *(FP_ERR + itemp) = *(FP_Vars_In + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            } else {
               *(FP_ERR + itemp) = 1.0;
            }
            if (x <= l_index) {
               *(FP_LEFT + left) = median_col;
               left++;
            }
            if (x >= r_index) {
               *(FP_RIGHT + right) = median_col;
               right++;
            }
            itemp++;
         }
      }
   }

   if (itemp > 1) {

      if (left != 0) {
         slremov_average(FP_LEFT, left, &avg_left, &rms_left, &median_left, Stat);
      }
      else {
         printf("slremov_compute: no useful data at left Lmask region\n");
         fit_flag = 1;
      }
   
      if (right != 0) {
         slremov_average(FP_RIGHT, right, &avg_right, &rms_right, &median_right, Stat);
      }
      else {
         printf("slremov_compute: no useful data at right Lmask region\n");
         fit_flag = 1;
      }

      if (!fit_flag && (fabs(avg_right-avg_left) < Const->SNR*rms_left)) {
         if (fabs(avg_right-avg_left) < Const->SNR*rms_left) 
            printf("slremov_compute: mean signal difference in left and right masked region too small; using mean masked value for stray light level\n");
         slremov_average(FP_SL, itemp, &avg_in_row, &rms_in_row, &median_in_row, Stat);
         Const->B0[y] = avg_in_row;
         Const->B1[y] = 0.0;
         Const->Stddev_B0[y] = rms_in_row;
         Const->Stddev_B1[y] = 0.0;
         nomod++;
      }
      else {     /* perform minimization of data vs. model */
         slremov_minimize(FP_SL, FP_I, FP_ERR, y, itemp, &b0, &b1, &eb0, &eb1, Const, Stat);
         error_message = RIGHT_MASK&(Stat->I_status);
         if (b1 < 0.0) {
            printf("slremov_compute: negative scaling of exponential model encountered; using mean masked value for stray light level\n");
            slremov_average(FP_SL, itemp, &avg_in_row, &rms_in_row, &median_in_row, Stat);
            Const->B0[y] = avg_in_row;
            Const->B1[y] = 0.0;
            Const->Stddev_B0[y] = rms_in_row;
            Const->Stddev_B1[y] = 0.0;
            nomod++;
         }
         else {
            Const->B0[y] = b0;
            Const->B1[y] = b1;
            Const->Stddev_B0[y] = eb0;
            Const->Stddev_B1[y] = eb1;
            for (j=0; j<itemp; ++j)
               reduced_chi2 += ((FP_SL[j] - (float)(Const->B0[y]+Const->B1[y]*exp(((double)FP_I[j]-Const->Decay_Alpha[y])/Const->Decay_Eta[y])))/FP_ERR[j])*((FP_SL[j] - (float)(Const->B0[y]+Const->B1[y]*exp(((double)FP_I[j]-Const->Decay_Alpha[y])/Const->Decay_Eta[y])))/FP_ERR[j]);
            if (itemp > 2) {
               reduced_chi2 /= (double)(itemp-2);
               if (Stat->I_Verbose)
                  printf("slremov_compute: minimization with model returns reduced Chi2 value of %lf for row %d, layer %d\n", reduced_chi2, y+1, z+1); 
               if (reduced_chi2 > 5.0) poorfit++;
            }
            else {
               printf("slremov_compute: only 1 or 2 data used for minimization; Chi2 value meaningless for row %d, layer %d\n", y+1, z+1);
            }
/*
            if (error_message == ZERO_UNC || error_message == MIN_ERROR) {
               for (x=0; x<Const->Column_Cut; ++x) {
                  if (*(SP_bmask + x + y*Dim_In->I_NX) & 512 == 512)
                     printf("slremov_compute: bmask bit 9 already set\n");
                  else
                     *(SP_bmask + x + y*Dim_In->I_NX) += 512;
               }
            }
*/
         }
      }
      if (Stat->I_Verbose)
         printf("slremov_compute: found constants B0[%d] = %lf with standard deviation %lf, B1[%d] = %lf with standard deviation %lf\n\n", y, Const->B0[y], Const->Stddev_B0[y], y, Const->B1[y], Const->Stddev_B1[y]);
   }
   else {
      printf("slremov_compute: few (0 or 1) useful data found at row %d, plane %d to model stray light\n", y, z);
/*      Stat->I_status = COMPUTE|NO_USEFUL_DATA; */
      for (x=0; x<Const->Column_Cut; ++x) {
         if (*(SP_bmask + x + y*Dim_In->I_NX) & 512 == 512)
            printf("slremov_compute: bmask bit 9 already set\n");
         else
            *(SP_bmask + x + y*Dim_In->I_NX) += 512;
      }
   }
}

}


for (z=0; z<Dim_In->I_NZ; ++z) {

for (y=0; y<Dim_In->I_NY; ++y) {
   if (Const->Noped) {
      Const->B0[y] = 0.0;
      Const->Stddev_B0[y] = 0.0;
   }
   for (x=0; x<Const->Column_Cut; ++x) {
      if ( (*(SP_pmask + x + y*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
           (*(SP_dmask + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
           (*(SP_cmask + x + y*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
           (*(SP_bmask + x + y*Dim_In->I_NX) & Const->BmaskFatal) == 0 ) {
         *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) -= (float)
            (Const->B0[y]+Const->B1[y]*exp(((double)x-Const->Decay_Alpha[y])/Const->Decay_Eta[y]));
         expterm = exp((x-Const->Decay_Alpha[y])/Const->Decay_Eta[y]);
         data_err = *(FP_Vars_In + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
         b0_err = Const->Stddev_B0[y];
         b1_err = expterm*Const->Stddev_B1[y];
         *(*FP_Vars_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) = sqrt(data_err*data_err+b0_err*b0_err+b1_err*b1_err);
      }
   }
}

}

/* calculate Q/A variables */

slremov_qa(FP_Data_Out, FP_Vars_Out, SP_pmask, SP_dmask, SP_bmask, SP_cmask, SP_lmask, Const, Dim_In, Stat);
Const->Slruncor = nomod;
Const->Slchinsq = poorfit;

free(FP_SL);
free(FP_I);
free(FP_ERR);
free(FP_LEFT);
free(FP_RIGHT);
free(adj);

if(Stat->I_Verbose)
printf("slremov_compute: Numerical Computations finished.\n");

}
