/* $Id: slremov_error_handler.c,v 1.7 2003/12/02 22:22:23 fan Exp $ */


/*********************************************** 
slremov_error_handler.c

Purpose

Decode error messages and report.

Overview
Append Error Messages to log file.

Definitions of Variables

Input:
IP_status = Return status of slremov functions.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out = Output Probabilities File Name.


Output:


Internal:
I_slremov_message = Code of error that occurred.
I_slremov_root = Code of function in which error occurred.
I_cfitsio_message = Code of cfitsio error returned.

***********************************************/

#include <time.h>
#include <stdio.h>
#include "slremov.h"
#include "slremov_defs.h"


slremov_error_handler(
            Status        *Stat,
            Filenames     *Fnames)
{
int	I_Time;
int	I_slremov_message;
int	I_slremov_root;
int	I_cfitsio_message=0;

char	CP_Message[I_MESSAGE_LENGTH];
char	CP_Function[I_FCNNAME_LENGTH];


I_slremov_message = RIGHT_MASK&(Stat->I_status);
I_slremov_root = (LEFT_MASK&(Stat->I_status))>>16;

if(I_slremov_message&FITS_MSG_MASK)	{
I_slremov_message -= FITS_MSG_MASK;
}

slremov_decode_message(I_slremov_message, CP_Message);
slremov_decode_fcnname(I_slremov_root, CP_Function);

if (Stat->I_status)
{
printf("slremov Error Message      0x%04x: %s.\n", I_slremov_message, CP_Message);
printf("slremov Error in Function  0x%04x: %s.\n", I_slremov_root, CP_Function);
}


I_Time = clock();
printf("Processor Time = %5.3f seconds\n", (float)I_Time/CLOCKS_PER_SEC);

if(Stat->I_Verbose)
printf("slremov_error_handler: Exception Handler finished.\n");

}
