/* $Id: slremov_minimize.c,v 1.8 2004/05/13 19:26:39 fan Exp $ */


/*********************************************** 
slremov_minimize.c

Purpose

Minimize function to estimate parameters.

***********************************************/

#include <stdio.h>
#include <math.h>
#include "slremov.h"
#include "slremov_defs.h"



slremov_minimize(	
            double          *FP_SL,
            double          *FP_I,
            double          *FP_ERR,
            int             row,
            int             dim,
            double          *b0,
            double          *b1,
	    double          *eb0,
	    double          *eb1,
            Constants       *Const,
            Status 	    *Stat)
{
int	i;
int     I_fits_return_status=0;
double  a_sigma, b_sigma, c_sigma, d_sigma, e_sigma;
double  c_term;

if(Stat->I_status) return;

a_sigma = 0.0;
b_sigma = 0.0;
c_sigma = 0.0;
d_sigma = 0.0;
e_sigma = 0.0;
for (i=0; i<dim; ++i) {
   if ( *(FP_ERR + i) == 0.0) {
      Stat->I_status = COMPUTE|ZERO_UNC;
      *(FP_ERR + i) = 1.0;
   }
   a_sigma += 1.0/((*(FP_ERR + i))*(*(FP_ERR + i)));
   b_sigma += (*(FP_SL + i))/((*(FP_ERR + i))*(*(FP_ERR + i)));
   c_sigma += exp((*(FP_I + i)-Const->Decay_Alpha[row])/Const->Decay_Eta[row])/((*(FP_ERR + i))*(*(FP_ERR + i)));
   d_sigma += (*(FP_SL + i))*exp((*(FP_I + i)-Const->Decay_Alpha[row])/Const->Decay_Eta[row])/((*(FP_ERR + i))*(*(FP_ERR + i)));
   e_sigma += exp((*(FP_I + i)-Const->Decay_Alpha[row])/Const->Decay_Eta[row])*exp((*(FP_I + i)-Const->Decay_Alpha[row])/Const->Decay_Eta[row])/((*(FP_ERR + i))*(*(FP_ERR + i)));
}

c_term = a_sigma*e_sigma - c_sigma*c_sigma;

if(c_term == 0.0) {
   printf("slremov_minimize: error minimizing merit function\n");
   Stat->I_status = COMPUTE|MIN_ERROR;
   c_term = 1.0;
}

*b0 = (b_sigma*e_sigma - d_sigma*c_sigma)/c_term;
*b1 = (d_sigma*a_sigma - b_sigma*c_sigma)/c_term;
*eb0 = sqrt(e_sigma/c_term);
*eb1 = sqrt(a_sigma/c_term);

if(Stat->I_Verbose)
printf("slremov_compute_results: Minimization Computations finished.\n");

}
