/* $Id: slremov_output.c,v 1.9 2003/12/02 22:22:23 fan Exp $ */


/*********************************************** 
slremov_output.c

Purpose

Save the slremov outputs to files.

Overview
Write the results of the computations as a FITS file.

Definitions of Variables

Input:
I_NX = Number of pixels in a row of the image
I_NY = Number of pixels in a column of the image
CP_Filename_FITS_In = Input Data File Name. 

Output:


Internal:
ffp_FITS = Handle to output FITS file.
I_fits_return_status = Resturn value of fits function.
CP_Keyname = name of Key to be written.
CP_String = String value to be written to header.
CP_Comment = String value to be written in comment field of header.


***********************************************/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fitsio.h"
#include "slremov.h"
#include "slremov_defs.h"


slremov_output(
            Filenames	  *Fnames,
	    Constants     *Const,
            Dims          *Dim_In,
            Status	  *Stat,
            struct tm     *tmp_time,
            short         *SPP_dmask,
            float         *FPP_Data_In,
            float         *FPP_Vars_In,
            float         *FPP_Data_Out,
            float         *FPP_Vars_Out)
{

int	I_fits_return_status=0;
int	I_Num_In;
int     string_length;
int     i;

float   F_Num_In;

char    CP_Keyname[FLEN_KEYWORD], CP_Comment[FLEN_COMMENT], CP_Filename_In[MAX_FILENAME_LENGTH];
char    programname[32], versionstring[32];
char    CP_String[22], CP_Layer[2];
char    format[10];
char    pn[10];

fitsfile 	*ffp_FITS_Out1;
fitsfile        *ffp_FITS_Out2;
fitsfile 	*ffp_FITS_In;
fitsfile        *ffp_Vars_Out1;
fitsfile        *ffp_Vars_Out2;
fitsfile        *ffp_Vars_In;
fitsfile 	*ffp_FITS_Dmask;

printf(asctime(tmp_time));

if(Stat->I_status) return;

/* Open dmask file. If found replace data.  */

if (strcmp(Fnames->CP_Filename_Dmask, ""))
   {
   fits_open_file( &ffp_FITS_Dmask,
                   Fnames->CP_Filename_Dmask,
                   READWRITE,
                   &I_fits_return_status);

/* Add module specific keywords */

   sprintf(CP_Comment, "Begin SLREMOV module, version %5.2f additions", VERSN);
   fits_write_comment(ffp_FITS_Dmask,
                      CP_Comment,
                      &I_fits_return_status);

   sprintf(programname, "%s", "SLREMOV");
   sprintf(versionstring, "%f", VERSN);
   writeDLhistoryline(ffp_FITS_Dmask, programname, versionstring, &I_fits_return_status);

   fits_write_img(ffp_FITS_Dmask,
                  TSHORT,
                  1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                  SPP_dmask,
                  &I_fits_return_status);		

   fits_flush_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
		
   fits_close_file(ffp_FITS_Dmask,
                   &I_fits_return_status);
   }

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

/* write out the corrected image data */

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_In,
                READWRITE,
                &I_fits_return_status);

fits_create_file( &ffp_FITS_Out1,
                Fnames->CP_Filename_FITS_Out1,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out1,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "Begin SLREMOV module, version %5.2f additions", VERSN);
fits_write_comment(ffp_FITS_Out1,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "SL_CORR");
sprintf(CP_Comment, "%s", "Stray-light correction flag");
I_Num_In = 1;
fits_write_key(ffp_FITS_Out1, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

sprintf(CP_Keyname, "%s", "SLRUNCOR");
sprintf(CP_Comment, "%s", "");
I_Num_In = Const->Slruncor;
fits_write_key(ffp_FITS_Out1, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

sprintf(CP_Keyname, "%s", "SLCHINSQ");
sprintf(CP_Comment, "%s", "");
I_Num_In = Const->Slchinsq;
fits_write_key(ffp_FITS_Out1, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

for (i=0; i<Dim_In->I_NZ; i++) {

   if (i>999) printf("slremov_output: warning - plane-number indicator in FITS key may be chopped off\n");
   sprintf(pn, "%d", i+1);
   strncpy(CP_Layer, pn, 2);

   sprintf(CP_Keyname, "%s", "LMED_");
   strncat(CP_Keyname, CP_Layer, 2);
   sprintf(CP_Comment, "%s", "Median corrected value in scaling region");
   F_Num_In = (float)(Const->ScaleMedian[i]);
   fits_write_key(ffp_FITS_Out1, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
   if (I_fits_return_status) {
      Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
      return;
   }

   sprintf(CP_Keyname, "%s", "SLXS_");
   strncat(CP_Keyname, CP_Layer, 2);
   sprintf(CP_Comment, "%s", "Linear slope in row fitting");
   F_Num_In = (float)(Const->Slxslp[i]);
   fits_write_key(ffp_FITS_Out1, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
   if (I_fits_return_status) {
      Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
      return;
   }

   sprintf(CP_Keyname, "%s", "SLXI_");
   strncat(CP_Keyname, CP_Layer, 2);
   sprintf(CP_Comment, "%s", "Linear intercept in row fitting");
   F_Num_In = (float)(Const->Slxicpt[i]);
   fits_write_key(ffp_FITS_Out1, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
   if (I_fits_return_status) {
      Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
      return;
   }

   sprintf(CP_Keyname, "%s", "SLYS_");
   strncat(CP_Keyname, CP_Layer, 2);
   sprintf(CP_Comment, "%s", "Linear slope in column fitting");
   F_Num_In = (float)(Const->Slyslp[i]);
   fits_write_key(ffp_FITS_Out1, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &I_fits_return_status);
   fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
   if (I_fits_return_status) {
      Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
      return;
   }

   sprintf(CP_Keyname, "%s", "SLYI_");
   strncat(CP_Keyname, CP_Layer, 2);
   sprintf(CP_Comment, "%s", "Linear intercept in column fitting");
   F_Num_In = (float)(Const->Slyicpt[i]);
   fits_write_key(ffp_FITS_Out1, TFLOAT, CP_Keyname, &F_Num_In, CP_Comment, &
I_fits_return_status);
   fits_flush_file(ffp_FITS_Out1, &I_fits_return_status);
   if (I_fits_return_status) {
      Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
      return;
   }

}

sprintf(programname, "%s", "SLREMOV");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out1, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out1,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_Out,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out1,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out1,
                &I_fits_return_status);

fits_close_file(ffp_FITS_In,
                &I_fits_return_status);


/* write out the uncorrected image data */

fits_open_file( &ffp_FITS_In,
                Fnames->CP_Filename_FITS_In,
                READWRITE,
                &I_fits_return_status);

fits_create_file( &ffp_FITS_Out2,
                Fnames->CP_Filename_FITS_Out2,
                &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_FITS_In,
                 ffp_FITS_Out2,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "Begin SLREMOV module, version %5.2f additions", VERSN);
fits_write_comment(ffp_FITS_Out2,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "SL_CORR");
sprintf(CP_Comment, "%s", "Stray-light correction flag");
I_Num_In = 0;
fits_write_key(ffp_FITS_Out2, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_FITS_Out2, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

sprintf(programname, "%s", "SLREMOV");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_FITS_Out2, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_FITS_Out2,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Data_In,
                &I_fits_return_status);

fits_flush_file(ffp_FITS_Out2,
                &I_fits_return_status);

fits_close_file(ffp_FITS_Out2,
                &I_fits_return_status);
fits_close_file(ffp_FITS_In,
                &I_fits_return_status);


/* output uncertainty file for corrected data */

if(strcmp(Fnames->CP_Filename_Vars_Out1, "")) {

if (Const->In_Vars) 
   fits_open_file( &ffp_Vars_In,
                   Fnames->CP_Filename_Vars_In,
                   READWRITE,
                   &I_fits_return_status);
else
   fits_open_file( &ffp_Vars_In,
                   Fnames->CP_Filename_FITS_In,
                   READWRITE,
                   &I_fits_return_status);

fits_create_file( &ffp_Vars_Out1,
                  Fnames->CP_Filename_Vars_Out1,
                  &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_Vars_In,
                 ffp_Vars_Out1,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "Begin SLREMOV module, version %5.2f additions", VERSN);
fits_write_comment(ffp_Vars_Out1,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "This is an uncertainty image");
fits_write_comment(ffp_Vars_Out1,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "SL_CORR");
sprintf(CP_Comment, "%s", "Stray-light correction flag");
I_Num_In = 1;
fits_write_key(ffp_Vars_Out1, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_Vars_Out1, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

sprintf(programname, "%s", "SLREMOV");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_Vars_Out1, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_Vars_Out1,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Vars_Out,
                &I_fits_return_status);

fits_flush_file(ffp_Vars_Out1,
                &I_fits_return_status);

fits_close_file(ffp_Vars_Out1,
                &I_fits_return_status);

fits_close_file(ffp_Vars_In,
                &I_fits_return_status);

}

/* output uncertainty file for uncorrected data */

if(strcmp(Fnames->CP_Filename_Vars_Out2, "")) {

if (Const->In_Vars)
   fits_open_file( &ffp_Vars_In,
                   Fnames->CP_Filename_Vars_In,
                   READWRITE,
                   &I_fits_return_status);
else
   fits_open_file( &ffp_Vars_In,
                   Fnames->CP_Filename_FITS_In,
                   READWRITE,
                   &I_fits_return_status);

fits_create_file( &ffp_Vars_Out2,
                  Fnames->CP_Filename_Vars_Out2,
                  &I_fits_return_status);

if (I_fits_return_status)
{Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status; return;}

fits_copy_header(ffp_Vars_In,
                 ffp_Vars_Out2,
                 &I_fits_return_status);

/* Add module specific keywords */

sprintf(CP_Comment, "Begin SLREMOV module, version %5.2f additions", VERSN);
fits_write_comment(ffp_Vars_Out2,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Comment, "%s", "This is an uncertainty image");
fits_write_comment(ffp_Vars_Out2,
                   CP_Comment,
                   &I_fits_return_status);

sprintf(CP_Keyname, "%s", "SL_CORR");
sprintf(CP_Comment, "%s", "Stray-light correction flag");
I_Num_In = 0;
fits_write_key(ffp_Vars_Out2, TINT, CP_Keyname, &I_Num_In, CP_Comment, &I_fits_return_status);
fits_flush_file(ffp_Vars_Out2, &I_fits_return_status);
if (I_fits_return_status) {
   Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;
   return;
}

sprintf(programname, "%s", "SLREMOV");
sprintf(versionstring, "%f", VERSN);
writeDLhistoryline(ffp_Vars_Out2, programname, versionstring, &I_fits_return_status);

fits_write_img( ffp_Vars_Out2,
                TFLOAT,
                1, Dim_In->I_NX*Dim_In->I_NY*Dim_In->I_NZ,
                FPP_Vars_In,
                &I_fits_return_status);

fits_flush_file(ffp_Vars_Out2,
                &I_fits_return_status);

fits_close_file(ffp_Vars_Out2,
                &I_fits_return_status);

fits_close_file(ffp_Vars_In,
                &I_fits_return_status);
}

if(I_fits_return_status)
Stat->I_status = OUTPUT_MAGS|FITS_MSG_MASK|I_fits_return_status;

if(Stat->I_Verbose)
printf("slremov_output: Results written to file(s).\n");

}
