/* $Id: slremov_parse_args.c,v 1.8 2005/02/01 18:01:52 fan Exp $ */


/*********************************************** 
slremov_parse_args.c

Purpose
Parse the command line to get input and output
file names constants, and processing flags.

Overview
Loop over the arguments, invoking case statement.

Definitions of Variables

Input:
CP_Filename_Namelist = Namelist (parameter file) File Name.
CP_Filename_FITS_In = In Data File Name.
CP_Filename_FITS_Out  = Output Probabilities File Name.
IP_Verbose = Toggle whether to output verbose info.

Output:


Internal:


***********************************************/

#include <stdio.h>
#include <string.h>
#include "slremov.h"
#include "slremov_defs.h"


slremov_parse_args(	
            int	        argc,
            char        **argv,
            Filenames   *Fnames,
	    Constants   *Const,
            Status	*Stat)
{
int	i=1;
float   xtest;

int     I_out_chk, I_in_chk, I_nl_chk;
int     I_const_chk, I_lmask_chk;
int     I_pair_err;

/* Run regardless of I_slremov_status so that
   at least log file name will be known */

I_out_chk = 1;
I_in_chk = 1;
I_nl_chk = 1;
I_const_chk = 1;
I_lmask_chk = 1;
I_pair_err = 0;

while( i < argc ) {
     if( argv[i][0] == '-' ) {
       switch( argv[i][1] ) {
          case 'n':      /* -n <input_namelist_fname>  */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               I_nl_chk = 0;
               sscanf(argv[i], "%s", Fnames->CP_Filename_Namelist);
            }
            break;
          case 'i':      /* -i <input_image_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
	       else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_FITS_In);
                  I_in_chk = 0;
               }
            }
            else { /* uncertainty image file */
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf(argv[i], "%s", Fnames->CP_Filename_Vars_In);
               }
            }
            break;
          case 'o':      /* -o <output_data_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out1);
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_FITS_Out1[1]));
                  I_out_chk = 0;
               }
            }
            else {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_FITS_Out2);
                  sscanf(argv[i], "%s", &(Fnames->CP_Filename_FITS_Out2[1]));
               }
            }
            break;
          case 'e':      /* -e <output_error_fname>  */
            if (argv[i][2] == '1') {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out1);
                  sscanf(argv[i],"%s", &(Fnames->CP_Filename_Vars_Out1[1]));
                  I_out_chk = 0;
               }
            }
            else {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else {
                  sscanf("!", "%c", Fnames->CP_Filename_Vars_Out2);
                  sscanf(argv[i], "%s", &(Fnames->CP_Filename_Vars_Out2[1]));
               }
            }
            break;
          case 'a':      /* -a <filename containing decay constants row-by-row> */
            if ( ++i >= argc )
               I_pair_err = 1;
            else {
               sscanf(argv[i], "%s", Fnames->CP_Filename_Const_In);
               I_const_chk = 0;
            }
            break;
          case 's':      /* -s critical signal-to-noise ratio */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%lf", &(Const->SNR));
            break;
          case 'x':      /* -x center x coordinate of H-shaped Lmask */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%d", &(Const->Hx_Center));
            break;
          case 'p':      /* -p <pmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Pmask);
            break;
          case 'd':      /* -d <dmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Dmask);
            break;
          case 'b':      /* -b <bmask file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Bmask);
            break;
          case 'c':      /* -c <cmask file name>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else
               sscanf(argv[i], "%s", Fnames->CP_Filename_Cmask);
            break;
          case 'l':      /* -l <lmask_fname>  */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               sscanf(argv[i], "%s", Fnames->CP_Filename_Lmask);
               I_lmask_chk = 0;
            }
            break;
          case 'u':      /* pixel of peak-up column boundary */
            if ( ++i >= argc ) {
               I_pair_err = 1;
            }
            else {
               sscanf(argv[i], "%d", &(Const->Column_Cut));
            }
            break;
          case 'f':      /* -f <fatal pixel bit template> */
            if (argv[i][2] == 'p')           /* for Pmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->PmaskFatal);
                  }
               }
            else if (argv[i][2] == 'd')      /* for Dmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->DmaskFatal);
                  }
               }
            else if (argv[i][2] == 'c')      /* for Cmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->CmaskFatal);
                  }
               }
            else if (argv[i][2] == 'b')      /* for Bmask */
               {
               if ( ++i >= argc )
                  I_pair_err = 1;
               else
                  {
                  sscanf(argv[i],"%d", &Const->BmaskFatal);
                  }
               }
            break;
          case 't':      /* -t  no-pedestal-subtraction mode */
            Const->Noped = 1;
            break;
          case 'v':      /* -v  verbose mode */
            Stat->I_Verbose = 1 - Stat->I_Verbose;
            break;
          default:
            printf("slremov_parse_args: unknown argument\n");
       }
     }
     else {
       printf("slremov_parse_args: command line syntax error\n");
     }
     i++;
}

if ( I_pair_err ) {
   printf("slremov_parse_args: command line syntax error\n");
   Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
   printf("Required arguments:\n\n");
   printf("-i1	input filename\n");
   printf("-o1	output filename for stray-light corrected image\n");
   printf("-o2	output filename for stray-light uncorrected image\n");
   printf("-l	input Lmask filename\n");
   printf("-a	input filename containing decay constants\n\n");
   printf("Optional arguments\n\n");
   printf("-n	namelist filename\n");
   printf("-i2	input uncertainty file name\n");
   printf("-e1	output stray-light-corrected uncertainty file name\n");
   printf("-e2	output stray-light-uncorrected uncertainty file name\n");
   printf("-u	peak-up image column boundary with spectra orders\n");
   printf("-s	critical signal-to-noise ratio to qualify a model fit\n");
   printf("-x	center in fast-read direction in pixel coordinate for H-shaped Lmask\n");
   printf("-d	dmask filename 'DCE pixel status'\n");
   printf("-p	pmask filename 'hardware status'\n");
   printf("-b	bmask filename 'BCD status'\n");
   printf("-c	cmask filename calibration status\n");
   printf("-fp	pmask fatal bit template\n");
   printf("-fd	dmask fatal bit template\n");
   printf("-fc	cmask fatal bit template\n");
   printf("-fb	bmask fatal bit template\n");
   printf("-t	no-pedestal-subtraction mode\n");
   printf("-v	puts the code in verbose mode\n\n");
}

if ((I_in_chk || I_const_chk || I_lmask_chk || I_out_chk) && I_nl_chk) {
   printf("Error in command line arguments.\n\n");
   printf("Required arguments:\n\n");
   printf("-i1	input filename\n");
   printf("-o1	output filename for stray-light corrected image\n");
   printf("-o2	output filename for stray-light uncorrected image\n");
   printf("-l	input Lmask filename\n");
   printf("-a	input filename containing decay constants\n\n");
   printf("Optional arguments\n\n");
   printf("-n	namelist filename\n");
   printf("-i2	input uncertainty file name\n");
   printf("-e1	output stray-light-corrected uncertainty file name\n");
   printf("-e2	output stray-light-uncorrected uncertainty file name\n");
   printf("-u	peak-up image column boundary with spectra orders\n");
   printf("-s	critical signal-to-noise ratio to qualify a model fit\n");
   printf("-x	center in fast-read direction in pixel coordinate for H-shaped Lmask\n");
   printf("-d	dmask filename 'DCE pixel status'\n");
   printf("-p	pmask filename 'hardware status'\n");
   printf("-b	bmask filename 'BCD status'\n");
   printf("-c	cmask filename calibration status\n");
   printf("-fp	pmask fatal bit template\n");
   printf("-fd	dmask fatal bit template\n");
   printf("-fc	cmask fatal bit template\n");
   printf("-fb	bmask fatal bit template\n");
   printf("-t	no-pedestal-subtraction mode\n");
   printf("-v	puts the code in verbose mode\n\n");
   Stat->I_status = PARSE_ARGS|CL_ARGS_INCRT;
}

if(strcmp(Fnames->CP_Filename_Vars_In, "")) Const->In_Vars = 1;

if(Stat->I_Verbose) {
printf("slremov_parse_args: Namelist File = \t%s\n",
      Fnames->CP_Filename_Namelist);
printf("slremov_parse_args: Input File = \t%s\n", 
      Fnames->CP_Filename_FITS_In);
printf("slremov_parse_args: Input Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Vars_In);
printf("slremov_parse_args: Input Constant File = \t%s\n",
      Fnames->CP_Filename_Const_In);
printf("slremov_parse_args: Ouput Stray-light Corrected File = \t%s\n",
      Fnames->CP_Filename_FITS_Out1);
printf("slremov_parse_args: Ouput Stray-light Uncorrected File = \t%s\n",
      Fnames->CP_Filename_FITS_Out2);
printf("slremov_parse_args: Ouput Stray-light Corrected Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Vars_Out1);
printf("slremov_parse_args: Ouput Stray-light Uncorrected Uncertainty File = \t%s\n",
      Fnames->CP_Filename_Vars_Out2);
printf("slremov_parse_args: Peak-up Images Column Boundary = \t%d\n",
      Const->Column_Cut);
printf("slremov_parse_args: Stray-light Mask file = \t%s\n",
      Fnames->CP_Filename_Lmask);
printf("slremov_parse_args: Critical Signal-to-Noise Ratio = \t%lf\n",
      Const->SNR);
printf("slremov_parse_args: H-shaped Lmask center in fast-read direction of pixel coordinate = \t%d\n", Const->Hx_Center);
printf("slremov_parse_args: Pmask File = \t%s\n",
      Fnames->CP_Filename_Pmask);
printf("slremov_parse_args: Dmask File = \t%s\n",
      Fnames->CP_Filename_Dmask);
printf("slremov_parse_args: Bmask File = \t%s\n",
      Fnames->CP_Filename_Bmask);
printf("slremov_parse_args: Cmask File = \t%s\n",
      Fnames->CP_Filename_Cmask);
printf("slremov_parse_args: Pmask Fatal = \t%d\n",
      Const->PmaskFatal);
printf("slremov_parse_args: Dmask Fatal = \t%d\n",
      Const->DmaskFatal);
printf("slremov_parse_args: Cmask Fatal = \t%d\n",
      Const->CmaskFatal);
printf("slremov_parse_args: I_Verbose = \t\t\t%d\n", Stat->I_Verbose);
}

}
