/* $Id: slremov_qa.c,v 1.7 2004/05/13 19:26:39 fan Exp $ */


/*********************************************** 
slremov_qa.c

Purpose

subroutine to calculate Q/A parameters.

***********************************************/

#include <stdio.h>
#include <math.h>
#include "slremov.h"
#include "slremov_defs.h"



slremov_qa(	
            float          **FP_Data_Out,
            float          **FP_Vars_Out,
            short          *SP_pmask,
            short          *SP_dmask,
            short          *SP_bmask,
            short          *SP_cmask,
            short          *SP_lmask,
	    Constants      *Const,
            Dims           *Dim_In,
            Status	   *Stat)
{
int	i, x, y, z;
int     cnt_columns, cnt_rows, cnt_rowfit, cnt_colfit, cnt_all;
int     I_fits_return_status=0;
int     trimfactflag;
int     nstart=1;
double  f1, f2;
double  facttotptr, stddev, datascale;
double  slope, intercept, xsq, mdlmask;
double  *FP_Col, *FP_Col_Sig, *FP_Row, *FP_Row_Sig, *FP_All, *Row, *Col;
double  avgcol, avgcol_sig, avgrow, avgrow_sig;

if(Stat->I_status) return;

FP_Col = (double *) calloc(Dim_In->I_NY, sizeof(double));
FP_Col_Sig = (double *) calloc(Dim_In->I_NY, sizeof(double));
Row = (double *) calloc(Dim_In->I_NY, sizeof(double));
FP_Row = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_Row_Sig = (double *) calloc(Dim_In->I_NX, sizeof(double));
Col = (double *) calloc(Dim_In->I_NX, sizeof(double));
FP_All = (double *) calloc(Dim_In->I_NX*Dim_In->I_NY, sizeof(double));
if(FP_Col == NULL || FP_Col_Sig == NULL || Row == NULL || FP_Row == NULL || FP_Row_Sig == NULL || Col == NULL || FP_All == NULL) {
   printf("slremov_qa: cannot allocate memory\n");
   Stat->I_status = COMPUTE|MALLOC_FAILED;;
   return;
}

avgrow = 0.0;
avgrow_sig = 0.0;
avgcol = 0.0;
avgcol_sig = 0.0;

for (z=0; z<Dim_In->I_NZ; z++) {

cnt_all=0;
cnt_rowfit=0;
for (x=0; x<Dim_In->I_NX; x++) {
   cnt_columns=0;
   for (y=0; y<Dim_In->I_NY; y++) {
      if ( *(SP_lmask + x + y*Dim_In->I_NX) == 1 &&
           (*(SP_pmask + x + y*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
           (*(SP_dmask + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
           (*(SP_cmask + x + y*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
           (*(SP_bmask + x + y*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
         if( !(iznand((double) *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY))) && !(iznand((double) *(*FP_Vars_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY))) ) {
            avgrow += *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            avgrow_sig += *(*FP_Vars_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            *(FP_All + cnt_all) = *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            cnt_columns++;
            cnt_all++;
         }
      }
   }
   if (cnt_columns > 0) {
      *(FP_Row + cnt_rowfit) = avgrow/(double)cnt_columns;
      *(FP_Row_Sig + cnt_rowfit) = avgrow_sig/(double)cnt_columns;
      *(Col + cnt_rowfit) = (double)x;
      cnt_rowfit++;
   }
}

if (cnt_rowfit > 1) {
   slremov_fitline(FP_Row, Col, FP_Row_Sig, cnt_rowfit, &slope, &intercept, &xsq, Const, Stat);
   if (!(iznand(slope)) && !(iznand(intercept))) {
      Const->Slxslp[z] = slope;
      Const->Slxicpt[z] = intercept;
   }
}

cnt_colfit=0;
for (y=0; y<Dim_In->I_NY; y++) {
   cnt_rows=0;
   for (x=0; x<Dim_In->I_NX; x++) {
      if ( *(SP_lmask + x + y*Dim_In->I_NX) == 1 &&
           (*(SP_pmask + x + y*Dim_In->I_NX) & Const->PmaskFatal) == 0 &&
           (*(SP_dmask + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY) & Const->DmaskFatal) == 0 &&
           (*(SP_cmask + x + y*Dim_In->I_NX) & Const->CmaskFatal) == 0 &&
           (*(SP_bmask + x + y*Dim_In->I_NX) & Const->BmaskFatal) == 0) {
         if( !(iznand((double) *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY))) && !(iznand((double) *(*FP_Vars_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY))) ) {
            avgcol += *(*FP_Data_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            avgcol_sig += *(*FP_Vars_Out + x + y*Dim_In->I_NX + z*Dim_In->I_NX*Dim_In->I_NY);
            cnt_rows++;
         }
      }
   }
   if (cnt_rows > 0) {
      *(FP_Col + cnt_colfit) = avgcol/(double)cnt_rows;
      *(FP_Col_Sig + cnt_colfit) = avgcol_sig/(double)cnt_rows;
      *(Row + cnt_colfit) = (double)y;
      cnt_colfit++;
   }
}

if (cnt_colfit > 1) {
   slremov_fitline(FP_Col, Row, FP_Col_Sig, cnt_colfit, &slope, &intercept, &xsq, Const, Stat);
   if (!(iznand(slope)) && !(iznand(intercept))) {
      Const->Slyslp[z] = slope;
      Const->Slyicpt[z] = intercept;
   }
}

/* calculate the median value in masked scaling area */
trimfactflag = 0;
f1 = 0.0;
f2 = 1.0;
trimmedstatistics(FP_All, nstart, cnt_all, f1, f2, &trimfactflag, &facttotptr, &mdlmask, &stddev, &datascale);
Const->ScaleMedian[z] = mdlmask;
if (Stat->I_Verbose)
   printf("\nslremov_qa: median value in masked scaling area is %lf for plane number %d\n", mdlmask, z+1);

}

free(FP_Row);
free(FP_Row_Sig);
free(Col);
free(FP_Col);
free(FP_Col_Sig);
free(Row);
free(FP_All);

if(Stat->I_Verbose)
printf("slremov_qa: Q/A Data Generated.\n");

}
