#define PROGRAM "tbl2fits"
#define VERSION "1.0"

/* This module applies fluxcal factors and/or tuning coefficients to a spectrum.  */

/**UPDATES
071218 1.0 New program
**/

/* TODO =====
 - illegal chars in col headers
 - warn no '|'  in col 1
 - special if '|'  in last col
 - too many cols
======  end TODO =====*/

/*****  NOTES *****************

REQUIREMENTS
 - convert extract.tbl files (and irs_tune.tbl) BCD, COADD, BKSUB
 - readable in IRAF orcet. ask Mark Lacy if any reqd kwds ORCET.; give to him to test

CALLING SEQUENCE
 tbl2fits -i INFILE -o OUTFILE

LIBRARIES
 cfitsio to write fits file
 nl_and_params for parameters

KEYWORDS HANDLING
   \int remove, reformat (fill out with spaces as needed)
   \COMMENT or \comment
   \HISTORY or \history
   \char -> COMMENT (unless followed by HISTORY or COMMENT) (or if blank)
   [other] -> COMMENT
      \wavsamp version for S13.0, 2005 Sep 13, D. Shupe
      \processing time 13:37:15 PST 09/13/2005
      \wavsamp software version 5.1 (Wed Apr 20 08:45:04 PDT 2005)
      \character channelID = 0

COLUMN HEADER HANDLING
  col names:  special hars allowed??
  data type:  real, int, ??what all?
  units:      keep

***** end NOTES ***********************/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "param_interface.h"
#include "fitsio.h"

#define MAXFILENAMESIZE   256
#define MAXCOLUMNS       1000
#define MAXCOLUMNSIZE     120
#define MAXLINESIZE      1000
#define MAXKEYWORDLINES  1000

/******* GLOBAL DATA *************************************/

/* command line parameters */
char in_filename[MAXFILENAMESIZE];
char out_filename[MAXFILENAMESIZE];
int verbose;

/************************/
int find_pipes (in_line, out_pipes)
/* returns number of pipes found (0 if none or if trouble) */
char *in_line;   /* (IN) line from column header (with pipes)  */
int out_pipes[];  /* (OUT) indexs of pipe symbols */
{
   int n_pipes   = 0; /* number of pipes */
   int i;
   for (i=0; in_line[i]; i++) {
     if (in_line[i] == '|')  { out_pipes[n_pipes] = i; n_pipes++; }
   }
   return n_pipes;
}


/************************/
char* get_column_value (upd_line, in_col_n, in_pipes)
/* returns pointer to string of column n (in upd_line) */
/* blanks trimmed from ends */
/* upd_line is modified */
char *upd_line;  /* (UPD) line string (gets modified) */
int in_col_n;    /* (IN) which column wanted (1-N) */
int in_pipes[];  /* (IN) indexes of pipes */
{
  char *c0; char *c1;
  upd_line[in_pipes[in_col_n]] = '\0';
  for (c0 = upd_line + in_pipes[in_col_n-1]+1; *c0==' '; c0++);
  if (*c0 == '\0') return c0;
  for (c1=upd_line+in_pipes[in_col_n]-1; *c1==' '; c1--);
  *(c1+1) = '\0';
  return c0;
}

/************************/
int strcpy_ascii (out_ascii_str, in_str)
/* just like strcpt but doesn't copy non-printable characters */
char *out_ascii_str;  /* (OUT) output string */
char *in_str;  /* (IN) input string */
{
  char *c_in; char *c_out;
  c_out = out_ascii_str;
  for (c_in=in_str; *c_in; c_in++) {
    if (isprint(*c_in) ) {*c_out = *c_in; c_out++; }
  }
  *c_out = '\0';
  return 0;
}

/************************/
int normalize_string (upd_string)
char *upd_string;  /* (UPD) line string (gets modified) */
{
  char *c;
  for (c=upd_string; *c; c++) {
    *c = toupper(*c);
    if (*c == '-') *c = '_';
  }
}


/************************/
void fatal(message) 
char *message;
{
  printf("\n=============================================================\n");
  printf("ERROR: %s\n", message);
  printf("=============================================================\n"); exit(64);
}


/******* main MAIN **********************/
int main(argc, argv)
int argc;
char **argv;
{

  /* data */
  FILE *in_stream;
  char in_line[MAXLINESIZE+1]; 
  char in_hdr1[MAXLINESIZE+1]; char in_hdr2[MAXLINESIZE+1]; char in_hdr3[MAXLINESIZE+1];
  char *tmp_c;
  char tmp_str[MAXCOLUMNSIZE+1]; char tmp_str2[MAXCOLUMNSIZE+1];
  int tmp_int; int tmp_real;

  int status;
  int i; int row_n;
  int n_pipes;

  int out_tfields;
  char *out_ttypes[MAXCOLUMNS];   /* column names */
  char *out_tforms[MAXCOLUMNS];  /* column datatypes */
  char *out_tunits[MAXCOLUMNS]; 
  char out_extname[] = "TABLE";
  fitsfile *out_fitsfile_ptr; /* ptr to structure for cfitsio */
  char out_format_strings[MAXCOLUMNS][8];
  char in_keyword_lines[MAXKEYWORDLINES][MAXLINESIZE+4];
  int n_keyword_lines=0;

  /* Added this - TYB 6/1/09 */
  char myoutfile[MAXFILENAMESIZE] = "";

  int col_pipes[MAXCOLUMNS];

  /* get PARAMS */
  param(in_filename,  "-i INFILE name of input ASCII table", "infile");
  param(out_filename, "-o OUTFILE name of output FITS table", "outfile");
  param(&verbose,  "-v VERBOSE level of printed messages",       "int - 1");
  status = param_begin(argc, argv, PROGRAM, VERSION);
  if (status != 0) return 64;
  if (verbose >=1) param_print();   /* Print the parameter settings */

  /* open input file */
  in_stream = fopen(in_filename, "r");

  /* read header keyword lines */
  while(fgets(in_line, MAXLINESIZE, in_stream)) {
    if (in_line[0] != '\\') break;
    strcpy_ascii(in_keyword_lines[n_keyword_lines], in_line);
    n_keyword_lines++;
    if (n_keyword_lines >= MAXKEYWORDLINES) fatal ("Too many KEYWORD lines");
  }

  /* process column header 1 (column names) */
  strcpy (in_hdr1, in_line);
  if (verbose >=3) printf("\n0123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789\n");
  if (verbose >=3) printf("\n%s", in_hdr1);
  if (in_hdr1[0] != '|') fatal("Column headers must immediately follow keywords");
  n_pipes = find_pipes (in_hdr1, col_pipes);
  for (i=1; i<n_pipes; i++) {
    tmp_c = get_column_value(in_hdr1, i, col_pipes);
    normalize_string(tmp_c);
    out_ttypes[i-1] = tmp_c;
  }

  /* process column header 2 (data types) */
  fgets(in_hdr2, MAXLINESIZE, in_stream);
  if (verbose >=3) printf("\n%s", in_hdr2);
  for (i=1; i<n_pipes; i++) {
    tmp_c = get_column_value(in_hdr2, i, col_pipes);
    normalize_string(tmp_c);
    if      (strcmp(tmp_c, "INT")==0)  strcpy(out_format_strings[i-1], "1J");
    else if (strcmp(tmp_c, "REAL")==0) strcpy(out_format_strings[i-1], "1E");
/*==============
if      (strcmp(tmp_c, "INT")==0)  sprintf(out_format_strings[i-1], "I%d", col_pipes[i]-col_pipes[i-1]-1);
else if (strcmp(tmp_c, "REAL")==0) sprintf(out_format_strings[i-1], "F%d", col_pipes[i]-col_pipes[i-1]-1);
================*/
    else fatal("Unknown data type");
    out_tforms[i-1] = out_format_strings[i-1];
  }

  /* process column header 3 (units) */
  fgets(in_hdr3, MAXLINESIZE, in_stream);
  if (verbose >=3) printf("\n%s", in_hdr3);
  for (i=1; i<n_pipes; i++) {
    tmp_c = get_column_value(in_hdr3, i, col_pipes);
    out_tunits[i-1] = tmp_c;
  }

  /* open output file */

  /* Change this to clobber any previous version - TYB 6/1/09 */
  strcat(myoutfile, "!");
  strcat(myoutfile, out_filename);
  status = fits_create_file(&out_fitsfile_ptr, myoutfile, &status);
  //  status = fits_create_file(&out_fitsfile_ptr, out_filename, &status);

  if (verbose >=2) printf ("FITS open status = %d\n", status);

  /* open output table extension */
  out_tfields = n_pipes-1;
  status = fits_create_tbl(out_fitsfile_ptr, BINARY_TBL, 0, out_tfields, out_ttypes, out_tforms, out_tunits, out_extname, &status);
  if (verbose >=2) printf ("FITS create tbl status = %d\n", status);

  /* process keyword keywords */
  for(i=0; i<n_keyword_lines; i++) {
    strcpy(in_line, in_keyword_lines[i]);
    if (verbose >= 5) printf("%s\n", in_line);
    if (strncmp(in_line+1, "int", 3)==0) {
       sscanf(in_line+5, "%s = %d", tmp_str, &tmp_int);
       fits_write_key(out_fitsfile_ptr, TINT, tmp_str, &tmp_int, "", &status);
    } else if (strncmp(in_line+1, "char COMMENT ", 13)==0) {
       in_line[14+72] = '\0'; /* trim to 80 char FITS line */
       fits_write_comment(out_fitsfile_ptr, in_line+14, &status);
    } else if (strncmp(in_line+1, "COMMENT ", 8)==0) {
       in_line[10+72] = '\0'; /* trim to 80 char FITS line */
       fits_write_comment(out_fitsfile_ptr, in_line+10, &status);
    } else   if (strncmp(in_line+1, "char HISTORY ", 13)==0) {
       in_line[13+72] = '\0'; /* trim to 80 char FITS line */
       fits_write_history(out_fitsfile_ptr, in_line+14, &status);
    } else   if (strncmp(in_line+1, "HISTORY ", 8)==0) {
       in_line[9+72] = '\0'; /* trim to 80 char FITS line */
       fits_write_history(out_fitsfile_ptr, in_line+9, &status);
    } else if (strcmp(in_line+1, "char")==0) {
       fits_write_history(out_fitsfile_ptr, " ", &status); /* empty COMMENT */
    } else if (strncmp(in_line+1, "char ", 5)==0) {
       in_line[6+72] = '\0'; /* trim to 80 char FITS line */
       fits_write_history(out_fitsfile_ptr, in_line+6, &status);
    } else fits_write_comment(out_fitsfile_ptr, in_line+1, &status);
  }


/******************
tmp_int = 888;
fits_write_col(out_fitsfile_ptr, TINT, 1, 1, 1, 1, &tmp_int, &status);
tmp_c = "444";
fits_write_col(out_fitsfile_ptr, TSTRING, 1, 2, 1, 1, &tmp_c, &status);
******************/

  /* process data rows */
  row_n = 0;
  while( fgets(in_line, MAXLINESIZE, in_stream) ) {
    row_n++;
    for (i=1; i<n_pipes; i++) {
      tmp_c = get_column_value(in_line, i, col_pipes);
      if (out_format_strings[i-1][1] == 'J') {
        sscanf(tmp_c, "%d", &tmp_int);
        fits_write_col(out_fitsfile_ptr, TINT, i, row_n, 1, 1, &tmp_int, &status);
      } else if (out_format_strings[i-1][1] == 'E') {
        sscanf(tmp_c, "%f", &tmp_real);
        fits_write_col(out_fitsfile_ptr, TFLOAT, i, row_n, 1, 1, &tmp_real, &status);
      } else fatal ("Programming error: unknown column tpe wher writint data values");
    /* for ASCII tables: ===========
    if      (strcmp(tmp_c, "INT")==0)  strcpy(out_format_strings[i-1], "1J");
    else if (strcmp(tmp_c, "REAL")==0) strcpy(out_format_strings[i-1], "1E");
    fits_write_col(out_fitsfile_ptr, TSTRING, i, row_n, 1, 1, &tmp_c, &status);
    ==============================*/
    }
  }

  /* close files and terminate */
  fits_close_file(out_fitsfile_ptr, &status);
  fits_report_error(stderr, status);
  return 0;
}
