# BCD pipeline wrapper

#=======================================================================
# initialize
use strict;
require "$ENV{'CUPID'}/wrappers/wrapper_subs.pm"; # standard subroutines
my $band = initialize();
my $dir_include      = "$ENV{'CUPID'}/include"; # ancillary directory

#=======================================================================
# Get FITS keyword values from input file
my $kwd_RAMPTIME = get_kwd('RAMPTIME');
my $kwd_GRPTIME  = get_kwd('GRPTIME');
my $kwd_GAIN1    = get_kwd('GAIN1', '4.6');
my $kwd_GAIN2    = get_kwd('GAIN2', '4.6');
my $kwd_GAIN3    = get_kwd('GAIN3', '4.6');
my $kwd_GAIN4    = get_kwd('GAIN4', '4.6');
my $kwd_APERTURE = get_kwd('APERTURE');
my $kwd_APERNAME = get_kwd('APERNAME');
my $kwd_TARGETNA = get_kwd('TARGETNA');
my $kwd_EXPTYPE  = get_kwd('EXPTYPE');

#=======================================================================
# create files to update FITS header keywords
write_kwd_files(); # (code is at end of file)

#=======================================================================
#STEP CVTI2R4 convert pixels from integer to floating point
# (must remove dmask if exists because cvti2r4 will pick it up if it's there)
run_module("CVTI2R4:RM remove_file
IN  -   dmask.fits # Input DMASK datafile
");
run_module("CVTI2R4 cvti2r4
V   -t no             # two's complement conversion: yes no
V   -a 32768          # offset added to each pixel
IN  -i <INFILE1>      # Input raw FLUX datafile
V   -b -1             # value for missing pixel
OUT -d dmask.fits     # Output DMASK datafile
OUT -o cvti2r4.fits <FLUX> # Output FLUX datafile
V   -s 65533          # saturation value in DN 32765 + 32768
");

#=======================================================================
#STEP ADDKWDS update FITS header keywords
# first delete keywords
run_module("ADDKWDS:DEL hdrupd8
UPD -  <FLUX>             # Input/Output FLUX datafile
V   -d .keywords_del.tmp  # delete keyword cards
V   -n -                  # don't add COMMENT card
");
# then add keywords
run_module("ADDKWDS hdrupd8
UPD -  <FLUX>            # Input/Output FLUX datafile
V   -a .keywords_add.tmp # append the tmp file keyword cards to add
V   -n -                 # don't add COMMENT card
");

#=======================================================================
#STEP SNEST_IRS compute preliminary uncertainty file
# first remove baseline from flux file for snest_irs to use
# before that, need to copy dmask for basecal because it will update it
run_module("SNEST_IRS:DMASK copy_file
IN  -i dmask.fits      # Input DMASK datafile
OUT -o dmask_copy.fits # Output DMASK datafile
");
run_module("SNEST_IRS:BASECAL basecal
CDF -n  basecal.nl     # namelist file
IN  -i <FLUX>          # Input FLUX datafile
OUT -o basecal.fits    # Output FLUX datafile
CAL -z umask.fits      # UMASK CAL file
UPD -d dmask_copy.fits # Input/Output DMASK datafile
");
run_module("SNEST_IRS snest_irs
CDF -n  snest_irs.nl         # namelist file
IN  -i  basecal.fits         # Input FLUX datafile
OUT -o  snest_irs.fits <UNC> # Output UNC datafile
");

#=======================================================================
#STEP DNTOFLUX convert FLUX and UNC files from DN to electrons
run_module("DNTOFLUX:UNC dntoflux
IN  -i  <UNC> # Input UNC datafile
OUT -o1 dntoflux_unc.fits <UNC> # Output UNC datafile
V   -a  $dir_include            # location of ancillary directory
V   -s  e-                      # units to put in FITS keyword
V   -co 1                       # operation: 1=readout-channel-dependent
V   -c1 $kwd_GAIN1              # gain for readout channel 1
V   -c2 $kwd_GAIN2              # gain for readout channel 2
V   -c3 $kwd_GAIN3              # gain for readout channel 3
V   -c4 $kwd_GAIN4              # gain for readout channel 4
");
run_module("DNTOFLUX dntoflux
IN  -i  <FLUX>                  # Input FLUX datafile
OUT -o1 dntoflux.fits <FLUX>    # Output FLUX datafile
V   -a  $dir_include            # location of ancillary directory
V   -s  e-                      # units to put in FITS keyword
V   -co 1                       # operation: 1=readout-channel-dependent scaling
V   -c1 $kwd_GAIN1              # gain for readout channel 1
V   -c2 $kwd_GAIN2              # gain for readout channel 2
V   -c3 $kwd_GAIN3              # gain for readout channel 3
V   -c4 $kwd_GAIN4              # gain for readout channel 4
");

#=======================================================================
#STEP SATCOR perform saturation correction
# first tag radhits
run_module("SATCOR:RADHIT radhit
CDF -n  radhit.nl            # namelist file
IN  -i1 <FLUX>               # Input FLUX datafile
IN  -i2 dmask.fits           # Input DMASK datafile
OUT -o5 dmask.fits           # Output DMASK datafile
CAL -13 pmask.fits           # PMASK CAL file
");

run_module("SATCOR imagest
CDF -n satcor.nl             # namelist file
IN  -i <FLUX>                # Input FLUX datafile
IN  -u <UNC>                 # Input UNC datafile
CAL -l lincal_satcor.fits    # lincal_satcor CAL file
CAL -p pmask.fits            # PMASK CAL file
UPD -d dmask.fits            # Input/Output DMASK datafile
OUT -o satcor.fits <FLUX>    # Output FLUX datafile
OUT -s satcor_unc.fits <UNC> # Output UNC datafile
V   -m 2                     # mode (1=find slopes  2=saturation correction)
");

#=======================================================================
#STEP DROOPOP correct for global droop
run_module("DROOPOP droopop
CDF -n  droopop.nl # namelist file
IN  -i1 <FLUX> <FLUX:ALT>      # Input FLUX datafile
IN  -i2 <UNC>  <UNC:ALT>       # Input UNC datafile
CAL -p  pmask.fits             # PMASK CAL file
UPD -d  dmask.fits             # Input/Output DMASK datafile
OUT -o1 droopop.fits <FLUX>    # Output FLUX datafile
OUT -o2 droopop_unc.fits <UNC> # Output UNC datafile
V   -q  -                      # write QA header
");

#=======================================================================
#STEP ROWDROOP correct for droop per row
run_module("ROWDROOP rowdroop
CDF -n  rowdroop.nl             # namelist file
IN  -id <FLUX>                  # Input FLUX datafile (droop removed)
IN  -iu <UNC>                   # Input UNC datafile (droop removed)
IN  -md <FLUX:ALT>              # Input FLUX datafile (droop NOT removed)
IN  -mu <UNC:ALT>               # Input UNC datafile (droop NOT removed)
CAL  -p pmask.fits              # PMASK CAL file
UPD -d  dmask.fits              # Input/Output DMASK datafile
OUT -od rowdroop.fits <FLUX>    # Output FLUX datafile
OUT -ou rowdroop_unc.fits <UNC> # Output UNC datafile
");

#=======================================================================
#STEP SNEST_IRS2 compute final uncertainty file
# first remove baseline from flux file for snest_irs to use
# before that, need to copy dmask for basecal because it will update it
run_module("SNEST_IRS2 snest_irs2
CDF -n  snest_irs2.nl         # namelist file
IN  -i  <FLUX>                # Input FLUX datafile
UPD -d  dmask.fits            # Input/Output DMASK datafile
OUT -o  snest_irs2.fits <UNC> # Output UNC datafile
");

#=======================================================================
#STEP DARKBASE zero the dark baseline
# first copy previous unc file
run_module("DARKBASE:UNC copy_file
IN  -i <UNC>                # Input UNC datafile
OUT -o drku.fits <UNC>      # Output UNC datafile
");
run_module("DARKBASE darkbase
CDF -n  darkbase.nl         # namelist file
IN  -i1 <FLUX>              # Input FLUX datafile
IN  -i2 <UNC>               # Input UNC datafile
CAL -p  pmask.fits          # PMASK CAL file
CAL -u  umask.fits          # UMASK CAL file
UPD -d  dmask.fits          # Input/Output DMASK datafile
OUT -o1 drk.fits <FLUX>     # Output FLUX datafile
");

#=======================================================================
#STEP CUBESUB subtract the dark
run_module("CUBESUB cubesub
CDF -n  cubesub.nl             # namelist file
IN  -i1 <FLUX>                 # Input FLUX datafile
IN  -e1 <UNC>                  # Input UNC datafile
IN  -d  dmask.fits             # Input DMASK datafile
CAL -i2 dark.fits              # DARK CAL file
CAL -e2 dark_unc.fits          # DARK_UNC CAL file
CAL -c  dark_cmask.fits        # DARK_CMASK CAL file
CAL -p  pmask.fits             # PMASK CAL file
OUT -o1 cubesub.fits <FLUX>    # Output FLUX datafile
OUT -o2 cubesub_unc.fits <UNC> # Output UNC datafile
V   -s  1                      # mode: 1=no scaling
V   -q  -                      # QA header update
");

#=======================================================================
#STEP LINEARIZ correct for non-linearity
run_module("LINEARIZ lineariz
CDF -n lineariz.nl        # namelist file
IN  -i <FLUX>             # Input FLUX datafile
IN  -u <UNC>              # Input UNC datafile
CAL -l lincal.fits        # LINCAL CAL file
CAL -c lincal_cmask.fits  # LINCAL_CMASK CAL file
CAL -p pmask.fits         # PMASK CAL file
UPD -m dmask.fits         # Input/Output DMASK datafile
OUT -o lnz.fits <FLUX>    # Output FLUX datafile
OUT -s lnzu.fits <UNC>    # Output UNC datafile
V   -q 4                  # QA output option
");

#=======================================================================
#STEP RADHIT flag rahhits (in DMASK file)
run_module("RADHIT radhit
CDF -n  radhit.nl         # namelist file
IN  -i1 <FLUX>            # Input FLUX datafile
CAL -i3 pmask.fits        # PMASK CAL file
IN  -i2 dmask.fits        # Input DMASK datafile
OUT -o1 radhit_prob.fits  # probabliity of radhit at pixel
OUT -o2 radhit_samp.fits  # sample most likely a radhit
OUT -o3 radhit_mag.fits   # estimated magnitude of radhit
OUT -o4 radhit_slope.fits # est. slope prior to radhit
OUT -o5 dmask.fits        # Output DMASK datafile
OUT -l  radhit.info       # ascii-misc info about the radhit run
V   -f  65019             # fatal DMASK bits
");

#=======================================================================
#STEP DARKDRIFT remove "jail bars"
run_module("DARKDRIFT darkdrift
CDF -n  darkdrift.nl             # namelist file
IN  -i  <FLUX>                   # Input FLUX datafile
IN  -iu <UNC>                    # Input UNC datafile
OUT -o1 darkdrift.fits <FLUX>    # Output FLUX datafile
OUT -ou darkdrift_unc.fits <UNC> # Output UNC datafile
OUT -o3 darkdrift2.tbl           # ascii-table-darkdrift
V   -a  $dir_include
V   -p  1                        # data plane
");

#=======================================================================
#STEP SLOPE_FINDER compute slope estimates of ramps
my $integrationtime = $kwd_RAMPTIME + $kwd_GRPTIME;
run_module("SLOPE_FINDER slope_finder
CDF -n      slope_finder.nl             # namelist file
IN  -i      <FLUX>                      # Input FLUX datafile
IN  -iu     <UNC>                       # Input UNC datafile
IN  -id     dmask.fits                  # Input DMASK datafile
CAL -ip     pmask.fits                  # PMASK CAL file
OUT -o_flux slope_finder.fits <FLUX>    # Output FLUX datafile
OUT -o_unc  slope_finder_unc.fits <UNC> # Output UNC datafile
OUT -ob     bmask.fits                  # Output BMASK datafile
V   -t      $integrationtime            # integration time (ramp + group)
");

#=======================================================================
#STEP DROOPRES remove residual droop
run_module("DROOPRES:UNC copy_file
IN  -i <UNC>                # Input UNC datafile
OUT -o drunc.fits <UNC>     # Output UNC datafile
");
run_module("DROOPRES droopres
CDF -n  droopres.nl         # namelist file
IN  -i1 <FLUX>              # Input FLUX datafile
IN  -i2 <UNC>               # Input UNC datafile
OUT -o1 droop.fits <FLUX>   # Output FLUX datafile
OUT -o2 x_droopres_unc.fits # Output UNC datafile
IN  -d  dmask.fits          # Input DMASK datafile
CAL -p  pmask.fits          # PMASK CAL file
IN  -b  bmask.fits          # Input BMASK datafile
CAL -u  umask.fits          # UMASK CAL file
");

#=======================================================================
#STEP STRAYCROSS(ch0) remove stray light
if ($band == 0) { # For CHAN=0 (Short Low) run slremov to remove stray light
  run_module("STRAYCROSS:UNC copy_file
  IN  -i <UNC>             # Input UNC datafile
  OUT -o rscu.fits <UNC>   # Output UNC datafile
  ");
  run_module("STRAYCROSS slremov
  CDF -n  slremov.nl       # namelist file
  IN  -i1 <FLUX>           # Input FLUX datafile
  CAL -l  lmask.fits       # fits-2d-lmask stray light mask
  CAL -a  slt_coeffs.tbl   # ascii-table-stray stray light coefficients
  CAL -p  pmask.fits       # fits-2d-pmask
  UPD -b  bmask.fits       # Input/Output BMASK datafile
  OUT -o1 rsc.fits <FLUX>  # Output FLUX datafile
  # OUT -o2 unstraycross.fits # Output FLUX datafile
  ");
}

#=======================================================================
#STEP STRAYCROSS(ch1&ch3) remove crosstalk
if ($band == 1 || $band == 3) { # For CHAN=1 or 3 (high res) run crosstalk to remove crosstalk between orders
  run_module("STRAYCROSS crosstalk
  CDF -n  crosstalk.nl          # namelist file
  IN  -i1 <FLUX>                # Input FLUX datafile
  IN  -i2 <UNC>                 # Input UNC datafile
  CAL -i3 omask.fits            # fits-2d-omask order mask
  UPD -b  bmask.fits            # Input/Output BMASK datafile
  # OUT -o1 unstraycross.fits   # Output FLUX datafile
  OUT -o2 rsc.fits <FLUX>       # Output FLUX datafile
  OUT -o4 rscu.fits <UNC>       # Output UNC datafile
  OUT -o3 straycross_resid.fits # Output RESIDUAL datafile
  V   -t  2.0                   #  threshold - determinator of modes
  ");
}


#=======================================================================
#STEP STRAYCROSS(ch2) just copy files
if ($band == 2) { # For CHAN=2 (long low) just copy the file
  run_module("STRAYCROSS:UNC copy_file
  IN  -i <UNC>             # Input UNC datafile
  OUT -o rscu.fits <UNC>   # Output UNC datafile
  ");
  run_module("STRAYCROSS copy_file
  IN  -i <FLUX>            # Input FLUX datafile
  OUT -o rsc.fits <FLUX>   # Output FLUX datafile
  ");
}


#=======================================================================
#STEP FLATAP apply FLAT
# (the flatap module may be picky about order of parameters)
run_module("FLATAP flatap
IN  -i  <FLUX>               # Input FLUX datafile
CAL -f  flatfield.fits       # FLATFIELD CAL file
OUT -o  flatap.fits <FLUX>   # Output FLUX datafile
V   -dm 28672                # BMASK fatal bits
V   -pm 18176                # PMASK fatal bits
V   -cm   444                # CMASK fatal bits
CAL -p  pmask.fits           # PMASK CAL file
CAL -c  flatfield_cmask.fits # FLATFIELD_CMASK CAL file
UPD -d  bmask.fits           # Input/Output BMASK datafile
IN  -ui <UNC>                # Input UNC datafile
OUT -uo func.fits <UNC>      # Output UNC datafile
");

#=======================================================================
#STEP FPGEN run 'final product generator' and add velocity keywords
# delete bcd file if it exists; FPGEN will crash if bcd file already exists
# add velocity header cards
run_module("FPGEN:VELOCITY velocity
IN  -i <FLUX>          # Input FLUX datafile
CDF -t velocity.tbl    # ASCII TABLE fluxcon CDF file
OUT -o <FLUX>          # Output FLUX datafile
");
run_module("FPGEN:RM remove_file
IN  -   bcd.fits       # Input FLUX datafile
");
# Run FPGEN final product generator (standardize the FITS header keywords)
run_module("FPGEN fpgen
IN  -i <FLUX>          # Input FLUX datafile
OUT -o bcd.fits <FLUX> # Output FLUX datafile
CDF -t fpg.tpl         # reformatting 'template' file
V   -h 1               # copy HISTORY lines too
V   -b 1               # remove blocks of blanks
V   -q 1               # 'quiet' mode
RESET_STATUS 32 0 # force status 32 (warning) change to 0 (OK) 
");

#=======================================================================
# print END message, update STATUS file,  and exit the program
terminate();



#########################################################################
#########################################################################
sub write_kwd_files { # this code moved to avoid clutter near start of script

#-----------------------------------------------------------------------
# setenv SOS_REL (for use by FPGEN to make "CREATOR" kwd)
$ENV{'SOS_REL'} = $ENV{'CUPID_VERSION'};

#-----------------------------------------------------------------------
# create .keywords_del file (to delete unwanted FITS keywords)
open(HDRUPDATE, ">.keywords_del.tmp");
printf HDRUPDATE "CREATOR\n"; # will be replaced, below
printf HDRUPDATE "CAL_SET\n"; # will be replaced, below
printf HDRUPDATE "DPID\n"; # not needed
close(HDRUPDATE);

#-----------------------------------------------------------------------
# create .keywords_add file (to add new/updated FITS keywords)
open(HDRUPDATE, ">.keywords_add.tmp"); # open file to write keywords to
# generate CREATOR keyword
printf HDRUPDATE "CREATOR = '%-8s'           / version of CUPID used to create this file\n", $ENV{'CUPID_VERSION'};;
# generate CAL_SET keyword
my $calset_file = fetch_cal('cal_set.txt');
open(CALSET_FILE, "<$calset_file") || message(64, "Couldn't open ${calset_file}");
my @calset_words = split(' ', <CALSET_FILE>);
my $calset_id = $calset_words[0];
close(CALSET_FILE);
printf HDRUPDATE "CAL_SET = '%-8s'           / ID of set of CAL files used (by CUPID)\n", $calset_id;
# generate FOVID, FOVNAME, OBJECT keywords (for SPICE)
printf HDRUPDATE "FOVID   = %d / Aperture number\n", $kwd_APERTURE;
printf HDRUPDATE "FOVNAME = '%s' / Aperture name\n", $kwd_APERNAME;
printf HDRUPDATE "OBJECT  = '%s' / Target name\n", $kwd_TARGETNA;
# generate FLXCON keywords
my $fluxcon_file = fetch_cal('fluxcon.tbl');
my $inline;
open(FLUXCON, "<$fluxcon_file") || message(64, "Couldn't open ${fluxcon_file}");
while (defined ($inline = <FLUXCON>)) {
  chomp $inline;
  next unless ($inline gt " ");
  if ($inline !~ /^(\\|\|)/) {
    my ($junk, $order, $wave, $passband, $s_irs, $s_irs_sigma, $s_ref,
        $s_ref_sigma, $fluxconv, $fluxconv_sigma, $a5, $a5_sigma,
        $a4, $a4_sigma, $a3, $a3_sigma, $a2, $a2_sigma, $a1, $a1_sigma, $a0, $a0_sigma)
        = split /\s+/, $inline;
     printf HDRUPDATE "FLXCON%02d= %20.6f / [e/Jy] Flux conversion Order %d\n", $order, $fluxconv, $order;
     printf HDRUPDATE "FLXERR%02d= %20.6f / [e/Jy] Flux conversion uncertainty Order %d\n",
                       $order, $fluxconv_sigma, $order;
   }
}
close(HDRUPDATE);

}
