# "X" (spectral extraction) pipeline wrapper 
#    (pipe_types: BCDX COADDX BKSUBX)

#=======================================================================
# initialize
use strict;
require "$ENV{'CUPID'}/wrappers/wrapper_subs.pm"; # standard subroutines
my $band = initialize();
my $lo_res = 0; $lo_res = 1 if (($band==0) || ($band==2));
my ($df_profile, $df_ridge, $df_extract, $df_tune, $df_fits);

#=======================================================================
# Set names of data files for this specific pipe_type
my $pipe_type = $ARGV[0];
if ($pipe_type eq 'BKSUBX') {
  $df_profile  = 'profile2.tbl';
  $df_ridge    = 'ridge2.tbl';
  $df_extract  = 'extract2.tbl';
  $df_tune     = 'bksub.tbl';
  $df_fits     = 'bkspc.fits';
} elsif ($pipe_type eq 'COADDX') {
  $df_profile  = 'profile.tbl';
  $df_ridge    = 'ridge.tbl';
  $df_extract  = 'extract.tbl';
  $df_tune     = 'tune.tbl';
  $df_fits     = 'tune.fits';
} elsif ($pipe_type eq 'BCDX') {
  $df_profile  = 'profile.tbl';
  $df_ridge    = 'ridge.tbl';
  $df_extract  = 'extract.tbl';
  $df_tune     = 'spect.tbl';
  $df_fits     = 'spect.fits';
}

#=======================================================================
#STEP PROFILE get a cross scan profile of flux in all orders
my $fov_param = ''; $fov_param = '-t psf_fov.tbl' if ($lo_res);
run_module("PROFILE profile
IN  -i <INFILE1>   # Input FLUX datafile
IN  -b <INFILE3>   # Input BMASK or AMASK datafile
CAL -w wavsamp.tbl # wavsamp CAL file
CAL $fov_param     # psf_fov CAL file
CDF -n profile.nl  # namelist file
OUT -o $df_profile # Output PROFILE ascii table
OUT -q profile.qa  # Output statistics
");


#=======================================================================
#STEP RIDGE locate the source (max of profile)
$fov_param = ''; $fov_param = '-f psf_fov.tbl' if ($lo_res);
run_module("RIDGE ridge
IN  -p $df_profile # Input PROFILE ascii table
CAL -i wavsamp.tbl # wavsmp CAL file
CAL $fov_param     # psf_fov CAL file
CDF -n ridge.nl    # namelist file
OUT -o $df_ridge   # Output RIDGE ascii table
");


#=======================================================================
#STEP EXTRACT extract spectrum
$fov_param = ''; $fov_param = '-p psf_fov.tbl' if ($lo_res);
run_module("EXTRACT extract
IN  -i <INFILE1>   # Input FLUX datafile
IN  -e <INFILE2>   # Input UNC datafile
IN  -b <INFILE3>   # Input BMASK or AMASK datafile
IN  -r $df_ridge   # Input RIDGE ascii table
CAL $fov_param     # psf_fov CAL file
CDF -n extract.nl  # namelist file
OUT -o $df_extract # Output EXTRACTION ascii table
");


#=======================================================================
#STEP IRS_TUNE apply flux conversion and minor tuning corrections
run_module("IRS_TUNE irs_tune
IN  -i $df_extract # Input EXTRACTION ascii table
CAL -t fluxcon.tbl # fluxcon CAL file
CDF -n irs_tune.nl # namelist file
OUT -o $df_tune    # Output TUNE ascii table
");


#=======================================================================
#STEP TBL2FITS create FIS binary table
run_module("TBL2FITS tbl2fits
IN  -i $df_tune # Input TUNE ascii table
OUT -o $df_fits # Output TUNE FITS binary table
");


#=======================================================================
# print END message, update STATUS file,  and exit the program
terminate();
